<?php

/** @file cachingrecursiveiterator.inc
 * @ingroup Examples
 * @brief class CachingRecursiveIterator
 * @author  Marcus Boerger
 * @date    2003 - 2005
 *
 * SPL - Standard PHP Library
 */

/** @ingroup Examples
 * @brief   Compatibility to PHP 5.0
 * @author  Marcus Boerger
 * @version 1.2
 * @deprecated
 *
 * Class RecursiveCachingIterator was named CachingRecursiveIterator until
 * PHP 5.0.6.
 *
 * @see RecursiveCachingIterator
 */

class CachingRecursiveIterator extends RecursiveCachingIterator
{
}

?>