typedef unsigned int size_t;
typedef long wchar_t;
struct __anonstruct___wait_terminated_1 {
   unsigned int __w_termsig : 7 ;
   unsigned int __w_coredump : 1 ;
   unsigned int __w_retcode : 8 ;
   unsigned int  : 16 ;
};
struct __anonstruct___wait_stopped_2 {
   unsigned int __w_stopval : 8 ;
   unsigned int __w_stopsig : 8 ;
   unsigned int  : 16 ;
};
union wait {
   int w_status ;
   struct __anonstruct___wait_terminated_1 __wait_terminated ;
   struct __anonstruct___wait_stopped_2 __wait_stopped ;
};
union __anonunion___WAIT_STATUS_3 {
   union wait *__uptr ;
   int *__iptr ;
};
typedef union __anonunion___WAIT_STATUS_3  __attribute__((__transparent_union__)) __WAIT_STATUS;
struct __anonstruct_div_t_4 {
   int quot ;
   int rem ;
};
typedef struct __anonstruct_div_t_4 div_t;
struct __anonstruct_ldiv_t_5 {
   long quot ;
   long rem ;
};
typedef struct __anonstruct_ldiv_t_5 ldiv_t;
struct __anonstruct_lldiv_t_6 {
   long long quot ;
   long long rem ;
};
typedef struct __anonstruct_lldiv_t_6 lldiv_t;
struct __locale_data;
struct __locale_struct {
   struct __locale_data *__locales[13] ;
   unsigned short const   *__ctype_b ;
   int const   *__ctype_tolower ;
   int const   *__ctype_toupper ;
   char const   *__names[13] ;
};
typedef struct __locale_struct *__locale_t;
typedef __locale_t locale_t;
typedef unsigned char __u_char;
typedef unsigned short __u_short;
typedef unsigned int __u_int;
typedef unsigned long __u_long;
typedef signed char __int8_t;
typedef unsigned char __uint8_t;
typedef short __int16_t;
typedef unsigned short __uint16_t;
typedef int __int32_t;
typedef unsigned int __uint32_t;
typedef long long __int64_t;
typedef unsigned long long __uint64_t;
typedef long long __quad_t;
typedef unsigned long long __u_quad_t;
typedef __u_quad_t __dev_t;
typedef unsigned int __uid_t;
typedef unsigned int __gid_t;
typedef unsigned long __ino_t;
typedef __u_quad_t __ino64_t;
typedef unsigned int __mode_t;
typedef unsigned int __nlink_t;
typedef long __off_t;
typedef __quad_t __off64_t;
typedef int __pid_t;
struct __anonstruct___fsid_t_7 {
   int __val[2] ;
};
typedef struct __anonstruct___fsid_t_7 __fsid_t;
typedef long __clock_t;
typedef unsigned long __rlim_t;
typedef __u_quad_t __rlim64_t;
typedef unsigned int __id_t;
typedef long __time_t;
typedef unsigned int __useconds_t;
typedef long __suseconds_t;
typedef int __daddr_t;
typedef long __swblk_t;
typedef int __key_t;
typedef int __clockid_t;
typedef void *__timer_t;
typedef long __blksize_t;
typedef long __blkcnt_t;
typedef __quad_t __blkcnt64_t;
typedef unsigned long __fsblkcnt_t;
typedef __u_quad_t __fsblkcnt64_t;
typedef unsigned long __fsfilcnt_t;
typedef __u_quad_t __fsfilcnt64_t;
typedef int __ssize_t;
typedef __off64_t __loff_t;
typedef __quad_t *__qaddr_t;
typedef char *__caddr_t;
typedef int __intptr_t;
typedef unsigned int __socklen_t;
typedef __u_char u_char;
typedef __u_short u_short;
typedef __u_int u_int;
typedef __u_long u_long;
typedef __quad_t quad_t;
typedef __u_quad_t u_quad_t;
typedef __fsid_t fsid_t;
typedef __loff_t loff_t;
typedef __ino_t ino_t;
typedef __ino64_t ino64_t;
typedef __dev_t dev_t;
typedef __gid_t gid_t;
typedef __mode_t mode_t;
typedef __nlink_t nlink_t;
typedef __uid_t uid_t;
typedef __off_t off_t;
typedef __off64_t off64_t;
typedef __pid_t pid_t;
typedef __id_t id_t;
typedef __ssize_t ssize_t;
typedef __daddr_t daddr_t;
typedef __caddr_t caddr_t;
typedef __key_t key_t;
typedef __clock_t clock_t;
typedef __time_t time_t;
typedef __clockid_t clockid_t;
typedef __timer_t timer_t;
typedef __useconds_t useconds_t;
typedef __suseconds_t suseconds_t;
typedef unsigned long ulong;
typedef unsigned short ushort;
typedef unsigned int uint;
typedef signed char int8_t;
typedef short int16_t;
typedef int int32_t;
typedef long long int64_t;
typedef unsigned char u_int8_t;
typedef unsigned short u_int16_t;
typedef unsigned int u_int32_t;
typedef unsigned long long u_int64_t;
typedef int register_t;
typedef int __sig_atomic_t;
struct __anonstruct___sigset_t_8 {
   unsigned long __val[1024U / (8U * sizeof(unsigned long ))] ;
};
typedef struct __anonstruct___sigset_t_8 __sigset_t;
typedef __sigset_t sigset_t;
struct timespec {
   __time_t tv_sec ;
   long tv_nsec ;
};
struct timeval {
   __time_t tv_sec ;
   __suseconds_t tv_usec ;
};
typedef long __fd_mask;
struct __anonstruct_fd_set_9 {
   __fd_mask fds_bits[1024 / (8 * (int )sizeof(__fd_mask ))] ;
};
typedef struct __anonstruct_fd_set_9 fd_set;
typedef __fd_mask fd_mask;
typedef __blksize_t blksize_t;
typedef __blkcnt_t blkcnt_t;
typedef __fsblkcnt_t fsblkcnt_t;
typedef __fsfilcnt_t fsfilcnt_t;
typedef __blkcnt64_t blkcnt64_t;
typedef __fsblkcnt64_t fsblkcnt64_t;
typedef __fsfilcnt64_t fsfilcnt64_t;
typedef unsigned long pthread_t;
union __anonunion_pthread_attr_t_10 {
   char __size[36] ;
   long __align ;
};
typedef union __anonunion_pthread_attr_t_10 pthread_attr_t;
struct __pthread_internal_slist {
   struct __pthread_internal_slist *__next ;
};
typedef struct __pthread_internal_slist __pthread_slist_t;
union __anonunion____missing_field_name_12 {
   int __spins ;
   __pthread_slist_t __list ;
};
struct __pthread_mutex_s {
   int __lock ;
   unsigned int __count ;
   int __owner ;
   int __kind ;
   unsigned int __nusers ;
   union __anonunion____missing_field_name_12 __annonCompField1 ;
};
union __anonunion_pthread_mutex_t_11 {
   struct __pthread_mutex_s __data ;
   char __size[24] ;
   long __align ;
};
typedef union __anonunion_pthread_mutex_t_11 pthread_mutex_t;
union __anonunion_pthread_mutexattr_t_13 {
   char __size[4] ;
   long __align ;
};
typedef union __anonunion_pthread_mutexattr_t_13 pthread_mutexattr_t;
struct __anonstruct___data_15 {
   int __lock ;
   unsigned int __futex ;
   unsigned long long __total_seq ;
   unsigned long long __wakeup_seq ;
   unsigned long long __woken_seq ;
   void *__mutex ;
   unsigned int __nwaiters ;
   unsigned int __broadcast_seq ;
};
union __anonunion_pthread_cond_t_14 {
   struct __anonstruct___data_15 __data ;
   char __size[48] ;
   long long __align ;
};
typedef union __anonunion_pthread_cond_t_14 pthread_cond_t;
union __anonunion_pthread_condattr_t_16 {
   char __size[4] ;
   long __align ;
};
typedef union __anonunion_pthread_condattr_t_16 pthread_condattr_t;
typedef unsigned int pthread_key_t;
typedef int pthread_once_t;
struct __anonstruct___data_18 {
   int __lock ;
   unsigned int __nr_readers ;
   unsigned int __readers_wakeup ;
   unsigned int __writer_wakeup ;
   unsigned int __nr_readers_queued ;
   unsigned int __nr_writers_queued ;
   unsigned char __flags ;
   unsigned char __shared ;
   unsigned char __pad1 ;
   unsigned char __pad2 ;
   int __writer ;
};
union __anonunion_pthread_rwlock_t_17 {
   struct __anonstruct___data_18 __data ;
   char __size[32] ;
   long __align ;
};
typedef union __anonunion_pthread_rwlock_t_17 pthread_rwlock_t;
union __anonunion_pthread_rwlockattr_t_19 {
   char __size[8] ;
   long __align ;
};
typedef union __anonunion_pthread_rwlockattr_t_19 pthread_rwlockattr_t;
typedef int volatile   pthread_spinlock_t;
union __anonunion_pthread_barrier_t_20 {
   char __size[20] ;
   long __align ;
};
typedef union __anonunion_pthread_barrier_t_20 pthread_barrier_t;
union __anonunion_pthread_barrierattr_t_21 {
   char __size[4] ;
   int __align ;
};
typedef union __anonunion_pthread_barrierattr_t_21 pthread_barrierattr_t;
struct random_data {
   int32_t *fptr ;
   int32_t *rptr ;
   int32_t *state ;
   int rand_type ;
   int rand_deg ;
   int rand_sep ;
   int32_t *end_ptr ;
};
struct drand48_data {
   unsigned short __x[3] ;
   unsigned short __old_x[3] ;
   unsigned short __c ;
   unsigned short __init ;
   unsigned long long __a ;
};
typedef int (*__compar_fn_t)(void const   * , void const   * );
typedef int (*comparison_fn_t)(void const   * , void const   * );
typedef int (*__compar_d_fn_t)(void const   * , void const   * , void * );
typedef long double float_t;
typedef long double double_t;
enum __anonenum_22 {
    FP_NAN = 0,
    FP_INFINITE = 1,
    FP_ZERO = 2,
    FP_SUBNORMAL = 3,
    FP_NORMAL = 4
} ;
enum __anonenum__LIB_VERSION_TYPE_23 {
    _IEEE_ = -1,
    _SVID_ = 0,
    _XOPEN_ = 1,
    _POSIX_ = 2,
    _ISOC_ = 3
} ;
typedef enum __anonenum__LIB_VERSION_TYPE_23 _LIB_VERSION_TYPE;
struct exception {
   int type ;
   char *name ;
   double arg1 ;
   double arg2 ;
   double retval ;
};
union __anonunion___u_24 {
   float __f ;
   int __i ;
};
union __anonunion___u_25 {
   double __d ;
   int __i[2] ;
};
union __anonunion___u_26 {
   long double __l ;
   int __i[3] ;
};
union __anonunion___n_27 {
   long double __xld ;
   unsigned int __xi[3] ;
};
union __anonunion_28 {
   double __d ;
   int __i[2] ;
};
typedef unsigned char uint8_t;
typedef unsigned short uint16_t;
typedef unsigned int uint32_t;
typedef unsigned long long uint64_t;
typedef signed char int_least8_t;
typedef short int_least16_t;
typedef int int_least32_t;
typedef long long int_least64_t;
typedef unsigned char uint_least8_t;
typedef unsigned short uint_least16_t;
typedef unsigned int uint_least32_t;
typedef unsigned long long uint_least64_t;
typedef signed char int_fast8_t;
typedef int int_fast16_t;
typedef int int_fast32_t;
typedef long long int_fast64_t;
typedef unsigned char uint_fast8_t;
typedef unsigned int uint_fast16_t;
typedef unsigned int uint_fast32_t;
typedef unsigned long long uint_fast64_t;
typedef int intptr_t;
typedef unsigned int uintptr_t;
typedef long long intmax_t;
typedef unsigned long long uintmax_t;
typedef long __gwchar_t;
struct __anonstruct_imaxdiv_t_29 {
   long long quot ;
   long long rem ;
};
typedef struct __anonstruct_imaxdiv_t_29 imaxdiv_t;
struct _IO_FILE;
struct _IO_FILE;
typedef struct _IO_FILE FILE;
typedef struct _IO_FILE __FILE;
union __anonunion___value_31 {
   unsigned int __wch ;
   char __wchb[4] ;
};
struct __anonstruct___mbstate_t_30 {
   int __count ;
   union __anonunion___value_31 __value ;
};
typedef struct __anonstruct___mbstate_t_30 __mbstate_t;
struct __anonstruct__G_fpos_t_32 {
   __off_t __pos ;
   __mbstate_t __state ;
};
typedef struct __anonstruct__G_fpos_t_32 _G_fpos_t;
struct __anonstruct__G_fpos64_t_33 {
   __off64_t __pos ;
   __mbstate_t __state ;
};
typedef struct __anonstruct__G_fpos64_t_33 _G_fpos64_t;
typedef short _G_int16_t;
typedef int _G_int32_t;
typedef unsigned short _G_uint16_t;
typedef unsigned int _G_uint32_t;
typedef __builtin_va_list __gnuc_va_list;
struct _IO_jump_t;
struct _IO_jump_t;
struct _IO_FILE;
typedef void _IO_lock_t;
struct _IO_marker {
   struct _IO_marker *_next ;
   struct _IO_FILE *_sbuf ;
   int _pos ;
};
enum __codecvt_result {
    __codecvt_ok = 0,
    __codecvt_partial = 1,
    __codecvt_error = 2,
    __codecvt_noconv = 3
} ;
struct _IO_FILE {
   int _flags ;
   char *_IO_read_ptr ;
   char *_IO_read_end ;
   char *_IO_read_base ;
   char *_IO_write_base ;
   char *_IO_write_ptr ;
   char *_IO_write_end ;
   char *_IO_buf_base ;
   char *_IO_buf_end ;
   char *_IO_save_base ;
   char *_IO_backup_base ;
   char *_IO_save_end ;
   struct _IO_marker *_markers ;
   struct _IO_FILE *_chain ;
   int _fileno ;
   int _flags2 ;
   __off_t _old_offset ;
   unsigned short _cur_column ;
   signed char _vtable_offset ;
   char _shortbuf[1] ;
   _IO_lock_t *_lock ;
   __off64_t _offset ;
   void *__pad1 ;
   void *__pad2 ;
   void *__pad3 ;
   void *__pad4 ;
   size_t __pad5 ;
   int _mode ;
   char _unused2[(15U * sizeof(int ) - 4U * sizeof(void *)) - sizeof(size_t )] ;
};
typedef struct _IO_FILE _IO_FILE;
struct _IO_FILE_plus;
struct _IO_FILE_plus;
typedef __ssize_t __io_read_fn(void *__cookie , char *__buf , size_t __nbytes );
typedef __ssize_t __io_write_fn(void *__cookie , char const   *__buf ,
                                size_t __n );
typedef int __io_seek_fn(void *__cookie , __off64_t *__pos , int __w );
typedef int __io_close_fn(void *__cookie );
typedef __io_read_fn cookie_read_function_t;
typedef __io_write_fn cookie_write_function_t;
typedef __io_seek_fn cookie_seek_function_t;
typedef __io_close_fn cookie_close_function_t;
struct __anonstruct__IO_cookie_io_functions_t_34 {
   __io_read_fn *read ;
   __io_write_fn *write ;
   __io_seek_fn *seek ;
   __io_close_fn *close ;
};
typedef struct __anonstruct__IO_cookie_io_functions_t_34 _IO_cookie_io_functions_t;
typedef _IO_cookie_io_functions_t cookie_io_functions_t;
struct _IO_cookie_file;
struct _IO_cookie_file;
typedef __gnuc_va_list va_list;
typedef _G_fpos_t fpos_t;
typedef _G_fpos64_t fpos64_t;
struct obstack;
struct obstack;
typedef unsigned long long timelib_ull;
typedef long long timelib_sll;
struct ttinfo {
   int32_t offset ;
   int isdst ;
   unsigned int abbr_idx ;
   unsigned int isstdcnt ;
   unsigned int isgmtcnt ;
};
typedef struct ttinfo ttinfo;
struct tlinfo {
   int32_t trans ;
   int32_t offset ;
};
typedef struct tlinfo tlinfo;
struct tlocinfo {
   char country_code[3] ;
   double latitude ;
   double longitude ;
   char *comments ;
};
typedef struct tlocinfo tlocinfo;
struct timelib_tzinfo {
   char *name ;
   uint32_t ttisgmtcnt ;
   uint32_t ttisstdcnt ;
   uint32_t leapcnt ;
   uint32_t timecnt ;
   uint32_t typecnt ;
   uint32_t charcnt ;
   int32_t *trans ;
   unsigned char *trans_idx ;
   ttinfo *type ;
   char *timezone_abbr ;
   tlinfo *leap_times ;
   unsigned char bc ;
   tlocinfo location ;
};
typedef struct timelib_tzinfo timelib_tzinfo;
struct timelib_special {
   unsigned int type ;
   timelib_sll amount ;
};
typedef struct timelib_special timelib_special;
struct timelib_rel_time {
   timelib_sll y ;
   timelib_sll m ;
   timelib_sll d ;
   timelib_sll h ;
   timelib_sll i ;
   timelib_sll s ;
   int weekday ;
   int weekday_behavior ;
   int first_last_day_of ;
   int invert ;
   timelib_sll days ;
   timelib_special special ;
   unsigned int have_weekday_relative ;
   unsigned int have_special_relative ;
};
typedef struct timelib_rel_time timelib_rel_time;
struct timelib_time_offset {
   int32_t offset ;
   unsigned int leap_secs ;
   unsigned int is_dst ;
   char *abbr ;
   timelib_sll transistion_time ;
};
typedef struct timelib_time_offset timelib_time_offset;
struct timelib_time {
   timelib_sll y ;
   timelib_sll m ;
   timelib_sll d ;
   timelib_sll h ;
   timelib_sll i ;
   timelib_sll s ;
   double f ;
   int z ;
   char *tz_abbr ;
   timelib_tzinfo *tz_info ;
   int dst ;
   timelib_rel_time relative ;
   timelib_sll sse ;
   unsigned int have_time ;
   unsigned int have_date ;
   unsigned int have_zone ;
   unsigned int have_relative ;
   unsigned int have_weeknr_day ;
   unsigned int sse_uptodate ;
   unsigned int tim_uptodate ;
   unsigned int is_localtime ;
   unsigned int zone_type ;
};
typedef struct timelib_time timelib_time;
struct timelib_error_message {
   int position ;
   char character ;
   char *message ;
};
typedef struct timelib_error_message timelib_error_message;
struct timelib_error_container {
   int warning_count ;
   struct timelib_error_message *warning_messages ;
   int error_count ;
   struct timelib_error_message *error_messages ;
};
typedef struct timelib_error_container timelib_error_container;
struct _timelib_tz_lookup_table {
   char *name ;
   int type ;
   float gmtoffset ;
   char *full_tz_name ;
};
typedef struct _timelib_tz_lookup_table timelib_tz_lookup_table;
struct _timelib_tzdb_index_entry {
   char *id ;
   unsigned int pos ;
};
typedef struct _timelib_tzdb_index_entry timelib_tzdb_index_entry;
struct _timelib_tzdb {
   char *version ;
   int index_size ;
   timelib_tzdb_index_entry const   *index ;
   unsigned char const   *data ;
};
typedef struct _timelib_tzdb timelib_tzdb;
enum __anonenum_35 {
    _ISupper = 256,
    _ISlower = 512,
    _ISalpha = 1024,
    _ISdigit = 2048,
    _ISxdigit = 4096,
    _ISspace = 8192,
    _ISprint = 16384,
    _ISgraph = 32768,
    _ISblank = 1,
    _IScntrl = 2,
    _ISpunct = 4,
    _ISalnum = 8
} ;
typedef unsigned char uchar;
struct timelib_elems {
   unsigned int c ;
   char **v ;
};
typedef struct timelib_elems timelib_elems;
struct Scanner {
   int fd ;
   uchar *lim ;
   uchar *str ;
   uchar *ptr ;
   uchar *cur ;
   uchar *tok ;
   uchar *pos ;
   unsigned int line ;
   unsigned int len ;
   struct timelib_error_container *errors ;
   struct timelib_time *time ;
   timelib_tzdb const   *tzdb ;
};
typedef struct Scanner Scanner;
struct _timelib_lookup_table {
   char const   *name ;
   int type ;
   int value ;
};
typedef struct _timelib_lookup_table timelib_lookup_table;
struct _timelib_relunit {
   char const   *name ;
   int unit ;
   int multiplier ;
};
typedef struct _timelib_relunit timelib_relunit;
extern  __attribute__((__nothrow__)) size_t __ctype_get_mb_cur_max(void) ;
__inline extern  __attribute__((__nothrow__)) double ( __attribute__((__nonnull__(1))) atof)(char const   *__nptr )  __attribute__((__pure__)) ;
__inline extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1))) atoi)(char const   *__nptr )  __attribute__((__pure__)) ;
__inline extern  __attribute__((__nothrow__)) long ( __attribute__((__nonnull__(1))) atol)(char const   *__nptr )  __attribute__((__pure__)) ;
__inline extern  __attribute__((__nothrow__)) long long ( __attribute__((__nonnull__(1))) atoll)(char const   *__nptr )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) double ( __attribute__((__nonnull__(1))) strtod)(char const   * __restrict  __nptr ,
                                                                                      char ** __restrict  __endptr ) ;
extern  __attribute__((__nothrow__)) float ( __attribute__((__nonnull__(1))) strtof)(char const   * __restrict  __nptr ,
                                                                                     char ** __restrict  __endptr ) ;
extern  __attribute__((__nothrow__)) long double ( __attribute__((__nonnull__(1))) strtold)(char const   * __restrict  __nptr ,
                                                                                            char ** __restrict  __endptr ) ;
extern  __attribute__((__nothrow__)) long ( __attribute__((__nonnull__(1))) strtol)(char const   * __restrict  __nptr ,
                                                                                    char ** __restrict  __endptr ,
                                                                                    int __base ) ;
extern  __attribute__((__nothrow__)) unsigned long ( __attribute__((__nonnull__(1))) strtoul)(char const   * __restrict  __nptr ,
                                                                                              char ** __restrict  __endptr ,
                                                                                              int __base ) ;
extern  __attribute__((__nothrow__)) long long ( __attribute__((__nonnull__(1))) strtoq)(char const   * __restrict  __nptr ,
                                                                                         char ** __restrict  __endptr ,
                                                                                         int __base ) ;
extern  __attribute__((__nothrow__)) unsigned long long ( __attribute__((__nonnull__(1))) strtouq)(char const   * __restrict  __nptr ,
                                                                                                   char ** __restrict  __endptr ,
                                                                                                   int __base ) ;
extern  __attribute__((__nothrow__)) long long ( __attribute__((__nonnull__(1))) strtoll)(char const   * __restrict  __nptr ,
                                                                                          char ** __restrict  __endptr ,
                                                                                          int __base ) ;
extern  __attribute__((__nothrow__)) unsigned long long ( __attribute__((__nonnull__(1))) strtoull)(char const   * __restrict  __nptr ,
                                                                                                    char ** __restrict  __endptr ,
                                                                                                    int __base ) ;
extern  __attribute__((__nothrow__)) long ( __attribute__((__nonnull__(1,4))) strtol_l)(char const   * __restrict  __nptr ,
                                                                                        char ** __restrict  __endptr ,
                                                                                        int __base ,
                                                                                        __locale_t __loc ) ;
extern  __attribute__((__nothrow__)) unsigned long ( __attribute__((__nonnull__(1,4))) strtoul_l)(char const   * __restrict  __nptr ,
                                                                                                  char ** __restrict  __endptr ,
                                                                                                  int __base ,
                                                                                                  __locale_t __loc ) ;
extern  __attribute__((__nothrow__)) long long ( __attribute__((__nonnull__(1,4))) strtoll_l)(char const   * __restrict  __nptr ,
                                                                                              char ** __restrict  __endptr ,
                                                                                              int __base ,
                                                                                              __locale_t __loc ) ;
extern  __attribute__((__nothrow__)) unsigned long long ( __attribute__((__nonnull__(1,4))) strtoull_l)(char const   * __restrict  __nptr ,
                                                                                                        char ** __restrict  __endptr ,
                                                                                                        int __base ,
                                                                                                        __locale_t __loc ) ;
extern  __attribute__((__nothrow__)) double ( __attribute__((__nonnull__(1,3))) strtod_l)(char const   * __restrict  __nptr ,
                                                                                          char ** __restrict  __endptr ,
                                                                                          __locale_t __loc ) ;
extern  __attribute__((__nothrow__)) float ( __attribute__((__nonnull__(1,3))) strtof_l)(char const   * __restrict  __nptr ,
                                                                                         char ** __restrict  __endptr ,
                                                                                         __locale_t __loc ) ;
extern  __attribute__((__nothrow__)) long double ( __attribute__((__nonnull__(1,3))) strtold_l)(char const   * __restrict  __nptr ,
                                                                                                char ** __restrict  __endptr ,
                                                                                                __locale_t __loc ) ;
__inline extern  __attribute__((__nothrow__)) double ( __attribute__((__nonnull__(1))) atof)(char const   *__nptr )  __attribute__((__pure__)) ;
__inline extern double ( __attribute__((__nonnull__(1))) atof)(char const   *__nptr ) 
{ 
  double tmp ;

  {
  tmp = strtod((char const   */* __restrict  */)__nptr,
               (char **/* __restrict  */)((char **)((void *)0)));
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1))) atoi)(char const   *__nptr )  __attribute__((__pure__)) ;
__inline extern int ( __attribute__((__nonnull__(1))) atoi)(char const   *__nptr ) 
{ 
  long tmp ;

  {
  tmp = strtol((char const   */* __restrict  */)__nptr,
               (char **/* __restrict  */)((char **)((void *)0)), 10);
  return ((int )tmp);
}
}
__inline extern  __attribute__((__nothrow__)) long ( __attribute__((__nonnull__(1))) atol)(char const   *__nptr )  __attribute__((__pure__)) ;
__inline extern long ( __attribute__((__nonnull__(1))) atol)(char const   *__nptr ) 
{ 
  long tmp ;

  {
  tmp = strtol((char const   */* __restrict  */)__nptr,
               (char **/* __restrict  */)((char **)((void *)0)), 10);
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) long long ( __attribute__((__nonnull__(1))) atoll)(char const   *__nptr )  __attribute__((__pure__)) ;
__inline extern long long ( __attribute__((__nonnull__(1))) atoll)(char const   *__nptr ) 
{ 
  long long tmp ;

  {
  tmp = strtoll((char const   */* __restrict  */)__nptr,
                (char **/* __restrict  */)((char **)((void *)0)), 10);
  return (tmp);
}
}
extern  __attribute__((__nothrow__)) char *l64a(long __n ) ;
extern  __attribute__((__nothrow__)) long ( __attribute__((__nonnull__(1))) a64l)(char const   *__s )  __attribute__((__pure__)) ;
extern int select(int __nfds , fd_set * __restrict  __readfds ,
                  fd_set * __restrict  __writefds ,
                  fd_set * __restrict  __exceptfds ,
                  struct timeval * __restrict  __timeout ) ;
extern int pselect(int __nfds , fd_set * __restrict  __readfds ,
                   fd_set * __restrict  __writefds ,
                   fd_set * __restrict  __exceptfds ,
                   struct timespec  const  * __restrict  __timeout ,
                   __sigset_t const   * __restrict  __sigmask ) ;
__inline extern  __attribute__((__nothrow__)) unsigned int gnu_dev_major(unsigned long long __dev ) ;
__inline extern  __attribute__((__nothrow__)) unsigned int gnu_dev_minor(unsigned long long __dev ) ;
__inline extern  __attribute__((__nothrow__)) unsigned long long gnu_dev_makedev(unsigned int __major ,
                                                                                 unsigned int __minor ) ;
__inline extern  __attribute__((__nothrow__)) unsigned int gnu_dev_major(unsigned long long __dev ) ;
__inline extern unsigned int gnu_dev_major(unsigned long long __dev ) 
{ 


  {
  return ((unsigned int )(((__dev >> 8) & 4095ULL) | (unsigned long long )((unsigned int )(__dev >> 32) & 4294963200U)));
}
}
__inline extern  __attribute__((__nothrow__)) unsigned int gnu_dev_minor(unsigned long long __dev ) ;
__inline extern unsigned int gnu_dev_minor(unsigned long long __dev ) 
{ 


  {
  return ((unsigned int )((__dev & 255ULL) | (unsigned long long )((unsigned int )(__dev >> 12) & 4294967040U)));
}
}
__inline extern  __attribute__((__nothrow__)) unsigned long long gnu_dev_makedev(unsigned int __major ,
                                                                                 unsigned int __minor ) ;
__inline extern unsigned long long gnu_dev_makedev(unsigned int __major ,
                                                   unsigned int __minor ) 
{ 


  {
  return (((unsigned long long )((__minor & 255U) | ((__major & 4095U) << 8)) | ((unsigned long long )(__minor & 4294967040U) << 12)) | ((unsigned long long )(__major & 4294963200U) << 32));
}
}
extern  __attribute__((__nothrow__)) long random(void) ;
extern  __attribute__((__nothrow__)) void srandom(unsigned int __seed ) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(2))) initstate)(unsigned int __seed ,
                                                                                        char *__statebuf ,
                                                                                        size_t __statelen ) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(1))) setstate)(char *__statebuf ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1,2))) random_r)(struct random_data * __restrict  __buf ,
                                                                                       int32_t * __restrict  __result ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(2))) srandom_r)(unsigned int __seed ,
                                                                                      struct random_data *__buf ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(2,4))) initstate_r)(unsigned int __seed ,
                                                                                          char * __restrict  __statebuf ,
                                                                                          size_t __statelen ,
                                                                                          struct random_data * __restrict  __buf ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1,2))) setstate_r)(char * __restrict  __statebuf ,
                                                                                         struct random_data * __restrict  __buf ) ;
extern  __attribute__((__nothrow__)) int rand(void) ;
extern  __attribute__((__nothrow__)) void srand(unsigned int __seed ) ;
extern  __attribute__((__nothrow__)) int rand_r(unsigned int *__seed ) ;
extern  __attribute__((__nothrow__)) double drand48(void) ;
extern  __attribute__((__nothrow__)) double ( __attribute__((__nonnull__(1))) erand48)(unsigned short *__xsubi ) ;
extern  __attribute__((__nothrow__)) long lrand48(void) ;
extern  __attribute__((__nothrow__)) long ( __attribute__((__nonnull__(1))) nrand48)(unsigned short *__xsubi ) ;
extern  __attribute__((__nothrow__)) long mrand48(void) ;
extern  __attribute__((__nothrow__)) long ( __attribute__((__nonnull__(1))) jrand48)(unsigned short *__xsubi ) ;
extern  __attribute__((__nothrow__)) void srand48(long __seedval ) ;
extern  __attribute__((__nothrow__)) unsigned short *( __attribute__((__nonnull__(1))) seed48)(unsigned short *__seed16v ) ;
extern  __attribute__((__nothrow__)) void ( __attribute__((__nonnull__(1))) lcong48)(unsigned short *__param ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1,2))) drand48_r)(struct drand48_data * __restrict  __buffer ,
                                                                                        double * __restrict  __result ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1,2))) erand48_r)(unsigned short *__xsubi ,
                                                                                        struct drand48_data * __restrict  __buffer ,
                                                                                        double * __restrict  __result ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1,2))) lrand48_r)(struct drand48_data * __restrict  __buffer ,
                                                                                        long * __restrict  __result ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1,2))) nrand48_r)(unsigned short *__xsubi ,
                                                                                        struct drand48_data * __restrict  __buffer ,
                                                                                        long * __restrict  __result ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1,2))) mrand48_r)(struct drand48_data * __restrict  __buffer ,
                                                                                        long * __restrict  __result ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1,2))) jrand48_r)(unsigned short *__xsubi ,
                                                                                        struct drand48_data * __restrict  __buffer ,
                                                                                        long * __restrict  __result ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(2))) srand48_r)(long __seedval ,
                                                                                      struct drand48_data *__buffer ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1,2))) seed48_r)(unsigned short *__seed16v ,
                                                                                       struct drand48_data *__buffer ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1,2))) lcong48_r)(unsigned short *__param ,
                                                                                        struct drand48_data *__buffer ) ;
extern  __attribute__((__nothrow__)) void *malloc(size_t __size )  __attribute__((__malloc__)) ;
extern  __attribute__((__nothrow__)) void *calloc(size_t __nmemb ,
                                                  size_t __size )  __attribute__((__malloc__)) ;
extern  __attribute__((__nothrow__)) void *( __attribute__((__warn_unused_result__)) realloc)(void *__ptr ,
                                                                                              size_t __size ) ;
extern  __attribute__((__nothrow__)) void free(void *__ptr ) ;
extern  __attribute__((__nothrow__)) void cfree(void *__ptr ) ;
extern  __attribute__((__nothrow__)) void *alloca(size_t __size ) ;
extern  __attribute__((__nothrow__)) void *valloc(size_t __size )  __attribute__((__malloc__)) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1))) posix_memalign)(void **__memptr ,
                                                                                           size_t __alignment ,
                                                                                           size_t __size ) ;
extern  __attribute__((__nothrow__, __noreturn__)) void abort(void) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1))) atexit)(void (*__func)(void) ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1))) at_quick_exit)(void (*__func)(void) ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1))) on_exit)(void (*__func)(int __status ,
                                                                                                   void *__arg ) ,
                                                                                    void *__arg ) ;
extern  __attribute__((__nothrow__, __noreturn__)) void exit(int __status ) ;
extern  __attribute__((__nothrow__,
__noreturn__)) void quick_exit(int __status ) ;
extern  __attribute__((__nothrow__, __noreturn__)) void _Exit(int __status ) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(1))) getenv)(char const   *__name ) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(1))) __secure_getenv)(char const   *__name ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1))) putenv)(char *__string ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(2))) setenv)(char const   *__name ,
                                                                                   char const   *__value ,
                                                                                   int __replace ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1))) unsetenv)(char const   *__name ) ;
extern  __attribute__((__nothrow__)) int clearenv(void) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(1))) mktemp)(char *__template ) ;
extern int ( __attribute__((__nonnull__(1))) mkstemp)(char *__template ) ;
extern int ( __attribute__((__nonnull__(1))) mkstemp64)(char *__template ) ;
extern int ( __attribute__((__nonnull__(1))) mkstemps)(char *__template ,
                                                       int __suffixlen ) ;
extern int ( __attribute__((__nonnull__(1))) mkstemps64)(char *__template ,
                                                         int __suffixlen ) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(1))) mkdtemp)(char *__template ) ;
extern int ( __attribute__((__nonnull__(1))) mkostemp)(char *__template ,
                                                       int __flags ) ;
extern int ( __attribute__((__nonnull__(1))) mkostemp64)(char *__template ,
                                                         int __flags ) ;
extern int ( __attribute__((__nonnull__(1))) mkostemps)(char *__template ,
                                                        int __suffixlen ,
                                                        int __flags ) ;
extern int ( __attribute__((__nonnull__(1))) mkostemps64)(char *__template ,
                                                          int __suffixlen ,
                                                          int __flags ) ;
extern int system(char const   *__command ) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(1))) canonicalize_file_name)(char const   *__name ) ;
extern  __attribute__((__nothrow__)) char *realpath(char const   * __restrict  __name ,
                                                    char * __restrict  __resolved ) ;
extern void *( __attribute__((__nonnull__(1,2,5))) bsearch)(void const   *__key ,
                                                            void const   *__base ,
                                                            size_t __nmemb ,
                                                            size_t __size ,
                                                            int (*__compar)(void const   * ,
                                                                            void const   * ) ) ;
extern void ( __attribute__((__nonnull__(1,4))) qsort)(void *__base ,
                                                       size_t __nmemb ,
                                                       size_t __size ,
                                                       int (*__compar)(void const   * ,
                                                                       void const   * ) ) ;
extern void ( __attribute__((__nonnull__(1,4))) qsort_r)(void *__base ,
                                                         size_t __nmemb ,
                                                         size_t __size ,
                                                         int (*__compar)(void const   * ,
                                                                         void const   * ,
                                                                         void * ) ,
                                                         void *__arg ) ;
extern  __attribute__((__nothrow__)) int abs(int __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long labs(long __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long long llabs(long long __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) div_t div(int __numer , int __denom )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) ldiv_t ldiv(long __numer , long __denom )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) lldiv_t lldiv(long long __numer ,
                                                   long long __denom )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(3,4))) ecvt)(double __value ,
                                                                                     int __ndigit ,
                                                                                     int * __restrict  __decpt ,
                                                                                     int * __restrict  __sign ) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(3,4))) fcvt)(double __value ,
                                                                                     int __ndigit ,
                                                                                     int * __restrict  __decpt ,
                                                                                     int * __restrict  __sign ) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(3))) gcvt)(double __value ,
                                                                                   int __ndigit ,
                                                                                   char *__buf ) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(3,4))) qecvt)(long double __value ,
                                                                                      int __ndigit ,
                                                                                      int * __restrict  __decpt ,
                                                                                      int * __restrict  __sign ) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(3,4))) qfcvt)(long double __value ,
                                                                                      int __ndigit ,
                                                                                      int * __restrict  __decpt ,
                                                                                      int * __restrict  __sign ) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(3))) qgcvt)(long double __value ,
                                                                                    int __ndigit ,
                                                                                    char *__buf ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(3,4,5))) ecvt_r)(double __value ,
                                                                                       int __ndigit ,
                                                                                       int * __restrict  __decpt ,
                                                                                       int * __restrict  __sign ,
                                                                                       char * __restrict  __buf ,
                                                                                       size_t __len ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(3,4,5))) fcvt_r)(double __value ,
                                                                                       int __ndigit ,
                                                                                       int * __restrict  __decpt ,
                                                                                       int * __restrict  __sign ,
                                                                                       char * __restrict  __buf ,
                                                                                       size_t __len ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(3,4,5))) qecvt_r)(long double __value ,
                                                                                        int __ndigit ,
                                                                                        int * __restrict  __decpt ,
                                                                                        int * __restrict  __sign ,
                                                                                        char * __restrict  __buf ,
                                                                                        size_t __len ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(3,4,5))) qfcvt_r)(long double __value ,
                                                                                        int __ndigit ,
                                                                                        int * __restrict  __decpt ,
                                                                                        int * __restrict  __sign ,
                                                                                        char * __restrict  __buf ,
                                                                                        size_t __len ) ;
extern  __attribute__((__nothrow__)) int mblen(char const   *__s , size_t __n ) ;
extern  __attribute__((__nothrow__)) int mbtowc(wchar_t * __restrict  __pwc ,
                                                char const   * __restrict  __s ,
                                                size_t __n ) ;
extern  __attribute__((__nothrow__)) int wctomb(char *__s , wchar_t __wchar ) ;
extern  __attribute__((__nothrow__)) size_t mbstowcs(wchar_t * __restrict  __pwcs ,
                                                     char const   * __restrict  __s ,
                                                     size_t __n ) ;
extern  __attribute__((__nothrow__)) size_t wcstombs(char * __restrict  __s ,
                                                     wchar_t const   * __restrict  __pwcs ,
                                                     size_t __n ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1))) rpmatch)(char const   *__response ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1,2,3))) getsubopt)(char ** __restrict  __optionp ,
                                                                                          char * const  * __restrict  __tokens ,
                                                                                          char ** __restrict  __valuep ) ;
extern  __attribute__((__nothrow__)) void ( __attribute__((__nonnull__(1))) setkey)(char const   *__key ) ;
extern int posix_openpt(int __oflag ) ;
extern  __attribute__((__nothrow__)) int grantpt(int __fd ) ;
extern  __attribute__((__nothrow__)) int unlockpt(int __fd ) ;
extern  __attribute__((__nothrow__)) char *ptsname(int __fd ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(2))) ptsname_r)(int __fd ,
                                                                                      char *__buf ,
                                                                                      size_t __buflen ) ;
extern int getpt(void) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1))) getloadavg)(double *__loadavg ,
                                                                                       int __nelem ) ;
extern  __attribute__((__nothrow__)) void *( __attribute__((__nonnull__(1,2))) memcpy)(void * __restrict  __dest ,
                                                                                       void const   * __restrict  __src ,
                                                                                       size_t __n ) ;
extern  __attribute__((__nothrow__)) void *( __attribute__((__nonnull__(1,2))) memmove)(void *__dest ,
                                                                                        void const   *__src ,
                                                                                        size_t __n ) ;
extern  __attribute__((__nothrow__)) void *( __attribute__((__nonnull__(1,2))) memccpy)(void * __restrict  __dest ,
                                                                                        void const   * __restrict  __src ,
                                                                                        int __c ,
                                                                                        size_t __n ) ;
extern  __attribute__((__nothrow__)) void *( __attribute__((__nonnull__(1))) memset)(void *__s ,
                                                                                     int __c ,
                                                                                     size_t __n ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1,2))) memcmp)(void const   *__s1 ,
                                                                                     void const   *__s2 ,
                                                                                     size_t __n )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) void *( __attribute__((__nonnull__(1))) memchr)(void const   *__s ,
                                                                                     int __c ,
                                                                                     size_t __n )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) void *( __attribute__((__nonnull__(1))) rawmemchr)(void const   *__s ,
                                                                                        int __c )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) void *( __attribute__((__nonnull__(1))) memrchr)(void const   *__s ,
                                                                                      int __c ,
                                                                                      size_t __n )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(1,2))) strcpy)(char * __restrict  __dest ,
                                                                                       char const   * __restrict  __src ) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(1,2))) strncpy)(char * __restrict  __dest ,
                                                                                        char const   * __restrict  __src ,
                                                                                        size_t __n ) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(1,2))) strcat)(char * __restrict  __dest ,
                                                                                       char const   * __restrict  __src ) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(1,2))) strncat)(char * __restrict  __dest ,
                                                                                        char const   * __restrict  __src ,
                                                                                        size_t __n ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1,2))) strcmp)(char const   *__s1 ,
                                                                                     char const   *__s2 )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1,2))) strncmp)(char const   *__s1 ,
                                                                                      char const   *__s2 ,
                                                                                      size_t __n )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1,2))) strcoll)(char const   *__s1 ,
                                                                                      char const   *__s2 )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) size_t ( __attribute__((__nonnull__(2))) strxfrm)(char * __restrict  __dest ,
                                                                                       char const   * __restrict  __src ,
                                                                                       size_t __n ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1,2,3))) strcoll_l)(char const   *__s1 ,
                                                                                          char const   *__s2 ,
                                                                                          __locale_t __l )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) size_t ( __attribute__((__nonnull__(2,4))) strxfrm_l)(char *__dest ,
                                                                                           char const   *__src ,
                                                                                           size_t __n ,
                                                                                           __locale_t __l ) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(1))) strdup)(char const   *__s )  __attribute__((__malloc__)) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(1))) strndup)(char const   *__string ,
                                                                                      size_t __n )  __attribute__((__malloc__)) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(1))) strchr)(char const   *__s ,
                                                                                     int __c )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(1))) strrchr)(char const   *__s ,
                                                                                      int __c )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(1))) strchrnul)(char const   *__s ,
                                                                                        int __c )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) size_t ( __attribute__((__nonnull__(1,2))) strcspn)(char const   *__s ,
                                                                                         char const   *__reject )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) size_t ( __attribute__((__nonnull__(1,2))) strspn)(char const   *__s ,
                                                                                        char const   *__accept )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(1,2))) strpbrk)(char const   *__s ,
                                                                                        char const   *__accept )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(1,2))) strstr)(char const   *__haystack ,
                                                                                       char const   *__needle )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(2))) strtok)(char * __restrict  __s ,
                                                                                     char const   * __restrict  __delim ) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(2,3))) __strtok_r)(char * __restrict  __s ,
                                                                                           char const   * __restrict  __delim ,
                                                                                           char ** __restrict  __save_ptr ) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(2,3))) strtok_r)(char * __restrict  __s ,
                                                                                         char const   * __restrict  __delim ,
                                                                                         char ** __restrict  __save_ptr ) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(1,2))) strcasestr)(char const   *__haystack ,
                                                                                           char const   *__needle )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) void *( __attribute__((__nonnull__(1,3))) memmem)(void const   *__haystack ,
                                                                                       size_t __haystacklen ,
                                                                                       void const   *__needle ,
                                                                                       size_t __needlelen )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) void *( __attribute__((__nonnull__(1,2))) __mempcpy)(void * __restrict  __dest ,
                                                                                          void const   * __restrict  __src ,
                                                                                          size_t __n ) ;
extern  __attribute__((__nothrow__)) void *( __attribute__((__nonnull__(1,2))) mempcpy)(void * __restrict  __dest ,
                                                                                        void const   * __restrict  __src ,
                                                                                        size_t __n ) ;
extern  __attribute__((__nothrow__)) size_t ( __attribute__((__nonnull__(1))) strlen)(char const   *__s )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) size_t ( __attribute__((__nonnull__(1))) strnlen)(char const   *__string ,
                                                                                       size_t __maxlen )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) char *strerror(int __errnum ) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(2))) strerror_r)(int __errnum ,
                                                                                         char *__buf ,
                                                                                         size_t __buflen ) ;
extern  __attribute__((__nothrow__)) char *strerror_l(int __errnum ,
                                                      __locale_t __l ) ;
extern  __attribute__((__nothrow__)) void ( __attribute__((__nonnull__(1))) __bzero)(void *__s ,
                                                                                     size_t __n ) ;
extern  __attribute__((__nothrow__)) void ( __attribute__((__nonnull__(1,2))) bcopy)(void const   *__src ,
                                                                                     void *__dest ,
                                                                                     size_t __n ) ;
extern  __attribute__((__nothrow__)) void ( __attribute__((__nonnull__(1))) bzero)(void *__s ,
                                                                                   size_t __n ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1,2))) bcmp)(void const   *__s1 ,
                                                                                   void const   *__s2 ,
                                                                                   size_t __n )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(1))) index)(char const   *__s ,
                                                                                    int __c )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(1))) rindex)(char const   *__s ,
                                                                                     int __c )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) int ffs(int __i )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int ffsl(long __l )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int ffsll(long long __ll )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1,2))) strcasecmp)(char const   *__s1 ,
                                                                                         char const   *__s2 )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1,2))) strncasecmp)(char const   *__s1 ,
                                                                                          char const   *__s2 ,
                                                                                          size_t __n )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1,2,3))) strcasecmp_l)(char const   *__s1 ,
                                                                                             char const   *__s2 ,
                                                                                             __locale_t __loc )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1,2,4))) strncasecmp_l)(char const   *__s1 ,
                                                                                              char const   *__s2 ,
                                                                                              size_t __n ,
                                                                                              __locale_t __loc )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(1,2))) strsep)(char ** __restrict  __stringp ,
                                                                                       char const   * __restrict  __delim ) ;
extern  __attribute__((__nothrow__)) char *strsignal(int __sig ) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(1,2))) __stpcpy)(char * __restrict  __dest ,
                                                                                         char const   * __restrict  __src ) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(1,2))) stpcpy)(char * __restrict  __dest ,
                                                                                       char const   * __restrict  __src ) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(1,2))) __stpncpy)(char * __restrict  __dest ,
                                                                                          char const   * __restrict  __src ,
                                                                                          size_t __n ) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(1,2))) stpncpy)(char * __restrict  __dest ,
                                                                                        char const   * __restrict  __src ,
                                                                                        size_t __n ) ;
extern  __attribute__((__nothrow__)) int ( __attribute__((__nonnull__(1,2))) strverscmp)(char const   *__s1 ,
                                                                                         char const   *__s2 )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(1))) strfry)(char *__string ) ;
extern  __attribute__((__nothrow__)) void *( __attribute__((__nonnull__(1))) memfrob)(void *__s ,
                                                                                      size_t __n ) ;
extern  __attribute__((__nothrow__)) char *( __attribute__((__nonnull__(1))) basename)(char const   *__filename ) ;
extern void *__rawmemchr(void const   *__s , int __c ) ;
__inline extern size_t __strcspn_c1(char const   *__s , int __reject ) ;
__inline extern size_t __strcspn_c1(char const   *__s , int __reject ) 
{ 
  register size_t __result ;

  {
  __result = (size_t )0;
  while (1) {
    if ((int const   )*(__s + __result) != 0) {
      if ((int const   )*(__s + __result) != (int const   )__reject) {

      } else {
        break;
      }
    } else {
      break;
    }
    __result ++;
  }
  return (__result);
}
}
__inline extern size_t __strcspn_c2(char const   *__s , int __reject1 ,
                                    int __reject2 ) ;
__inline extern size_t __strcspn_c2(char const   *__s , int __reject1 ,
                                    int __reject2 ) 
{ 
  register size_t __result ;

  {
  __result = (size_t )0;
  while (1) {
    if ((int const   )*(__s + __result) != 0) {
      if ((int const   )*(__s + __result) != (int const   )__reject1) {
        if ((int const   )*(__s + __result) != (int const   )__reject2) {

        } else {
          break;
        }
      } else {
        break;
      }
    } else {
      break;
    }
    __result ++;
  }
  return (__result);
}
}
__inline extern size_t __strcspn_c3(char const   *__s , int __reject1 ,
                                    int __reject2 , int __reject3 ) ;
__inline extern size_t __strcspn_c3(char const   *__s , int __reject1 ,
                                    int __reject2 , int __reject3 ) 
{ 
  register size_t __result ;

  {
  __result = (size_t )0;
  while (1) {
    if ((int const   )*(__s + __result) != 0) {
      if ((int const   )*(__s + __result) != (int const   )__reject1) {
        if ((int const   )*(__s + __result) != (int const   )__reject2) {
          if ((int const   )*(__s + __result) != (int const   )__reject3) {

          } else {
            break;
          }
        } else {
          break;
        }
      } else {
        break;
      }
    } else {
      break;
    }
    __result ++;
  }
  return (__result);
}
}
__inline extern size_t __strspn_c1(char const   *__s , int __accept ) ;
__inline extern size_t __strspn_c1(char const   *__s , int __accept ) 
{ 
  register size_t __result ;

  {
  __result = (size_t )0;
  while ((int const   )*(__s + __result) == (int const   )__accept) {
    __result ++;
  }
  return (__result);
}
}
__inline extern size_t __strspn_c2(char const   *__s , int __accept1 ,
                                   int __accept2 ) ;
__inline extern size_t __strspn_c2(char const   *__s , int __accept1 ,
                                   int __accept2 ) 
{ 
  register size_t __result ;

  {
  __result = (size_t )0;
  while (1) {
    if ((int const   )*(__s + __result) == (int const   )__accept1) {

    } else
    if ((int const   )*(__s + __result) == (int const   )__accept2) {

    } else {
      break;
    }
    __result ++;
  }
  return (__result);
}
}
__inline extern size_t __strspn_c3(char const   *__s , int __accept1 ,
                                   int __accept2 , int __accept3 ) ;
__inline extern size_t __strspn_c3(char const   *__s , int __accept1 ,
                                   int __accept2 , int __accept3 ) 
{ 
  register size_t __result ;

  {
  __result = (size_t )0;
  while (1) {
    if ((int const   )*(__s + __result) == (int const   )__accept1) {

    } else
    if ((int const   )*(__s + __result) == (int const   )__accept2) {

    } else
    if ((int const   )*(__s + __result) == (int const   )__accept3) {

    } else {
      break;
    }
    __result ++;
  }
  return (__result);
}
}
__inline extern char *__strpbrk_c2(char const   *__s , int __accept1 ,
                                   int __accept2 ) ;
__inline extern char *__strpbrk_c2(char const   *__s , int __accept1 ,
                                   int __accept2 ) 
{ 
  char *tmp ;

  {
  while (1) {
    if ((int const   )*__s != 0) {
      if ((int const   )*__s != (int const   )__accept1) {
        if ((int const   )*__s != (int const   )__accept2) {

        } else {
          break;
        }
      } else {
        break;
      }
    } else {
      break;
    }
    __s ++;
  }
  if ((int const   )*__s == 0) {
    tmp = (char *)((void *)0);
  } else {
    tmp = (char *)((size_t )__s);
  }
  return (tmp);
}
}
__inline extern char *__strpbrk_c3(char const   *__s , int __accept1 ,
                                   int __accept2 , int __accept3 ) ;
__inline extern char *__strpbrk_c3(char const   *__s , int __accept1 ,
                                   int __accept2 , int __accept3 ) 
{ 
  char *tmp ;

  {
  while (1) {
    if ((int const   )*__s != 0) {
      if ((int const   )*__s != (int const   )__accept1) {
        if ((int const   )*__s != (int const   )__accept2) {
          if ((int const   )*__s != (int const   )__accept3) {

          } else {
            break;
          }
        } else {
          break;
        }
      } else {
        break;
      }
    } else {
      break;
    }
    __s ++;
  }
  if ((int const   )*__s == 0) {
    tmp = (char *)((void *)0);
  } else {
    tmp = (char *)((size_t )__s);
  }
  return (tmp);
}
}
__inline extern char *__strtok_r_1c(char *__s , char __sep , char **__nextp ) ;
__inline extern char *__strtok_r_1c(char *__s , char __sep , char **__nextp ) 
{ 
  char *__result ;
  char *tmp ;
  char *tmp___0 ;

  {
  if ((unsigned int )__s == (unsigned int )((void *)0)) {
    __s = *__nextp;
  } else {

  }
  while ((int )*__s == (int )__sep) {
    __s ++;
  }
  __result = (char *)((void *)0);
  if ((int )*__s != 0) {
    tmp = __s;
    __s ++;
    __result = tmp;
    while ((int )*__s != 0) {
      tmp___0 = __s;
      __s ++;
      if ((int )*tmp___0 == (int )__sep) {
        *(__s + -1) = (char )'\000';
        break;
      } else {

      }
    }
  } else {

  }
  *__nextp = __s;
  return (__result);
}
}
extern char *__strsep_g(char **__stringp , char const   *__delim ) ;
__inline extern char *__strsep_1c(char **__s , char __reject ) ;
__inline extern char *__strsep_1c(char **__s , char __reject ) 
{ 
  register char *__retval ;
  char *tmp ;
  char *tmp___0 ;
  void *tmp___1 ;
  char *tmp___2 ;

  {
  __retval = *__s;
  if ((unsigned int )__retval != (unsigned int )((void *)0)) {
    tmp___2 = __builtin_strchr(__retval, (int )__reject);
    tmp___0 = tmp___2;
    *__s = tmp___0;
    if ((unsigned int )tmp___0 != (unsigned int )((void *)0)) {
      tmp = *__s;
      (*__s) ++;
      *tmp = (char )'\000';
    } else {

    }
  } else {

  }
  return (__retval);
}
}
__inline extern char *__strsep_2c(char **__s , char __reject1 , char __reject2 ) ;
__inline extern char *__strsep_2c(char **__s , char __reject1 , char __reject2 ) 
{ 
  register char *__retval ;
  register char *__cp ;
  char *tmp ;

  {
  __retval = *__s;
  if ((unsigned int )__retval != (unsigned int )((void *)0)) {
    __cp = __retval;
    while (1) {
      if ((int )*__cp == 0) {
        __cp = (char *)((void *)0);
        break;
      } else {

      }
      if ((int )*__cp == (int )__reject1) {
        tmp = __cp;
        __cp ++;
        *tmp = (char )'\000';
        break;
      } else
      if ((int )*__cp == (int )__reject2) {
        tmp = __cp;
        __cp ++;
        *tmp = (char )'\000';
        break;
      } else {

      }
      __cp ++;
    }
    *__s = __cp;
  } else {

  }
  return (__retval);
}
}
__inline extern char *__strsep_3c(char **__s , char __reject1 , char __reject2 ,
                                  char __reject3 ) ;
__inline extern char *__strsep_3c(char **__s , char __reject1 , char __reject2 ,
                                  char __reject3 ) 
{ 
  register char *__retval ;
  register char *__cp ;
  char *tmp ;

  {
  __retval = *__s;
  if ((unsigned int )__retval != (unsigned int )((void *)0)) {
    __cp = __retval;
    while (1) {
      if ((int )*__cp == 0) {
        __cp = (char *)((void *)0);
        break;
      } else {

      }
      if ((int )*__cp == (int )__reject1) {
        tmp = __cp;
        __cp ++;
        *tmp = (char )'\000';
        break;
      } else
      if ((int )*__cp == (int )__reject2) {
        tmp = __cp;
        __cp ++;
        *tmp = (char )'\000';
        break;
      } else
      if ((int )*__cp == (int )__reject3) {
        tmp = __cp;
        __cp ++;
        *tmp = (char )'\000';
        break;
      } else {

      }
      __cp ++;
    }
    *__s = __cp;
  } else {

  }
  return (__retval);
}
}
extern  __attribute__((__nothrow__)) char *__strdup(char const   *__string )  __attribute__((__malloc__)) ;
extern  __attribute__((__nothrow__)) char *__strndup(char const   *__string ,
                                                     size_t __n )  __attribute__((__malloc__)) ;
extern  __attribute__((__nothrow__)) double acos(double __x ) ;
extern  __attribute__((__nothrow__)) double __acos(double __x ) ;
extern  __attribute__((__nothrow__)) double asin(double __x ) ;
extern  __attribute__((__nothrow__)) double __asin(double __x ) ;
extern  __attribute__((__nothrow__)) double atan(double __x ) ;
extern  __attribute__((__nothrow__)) double __atan(double __x ) ;
extern  __attribute__((__nothrow__)) double atan2(double __y , double __x ) ;
extern  __attribute__((__nothrow__)) double __atan2(double __y , double __x ) ;
extern  __attribute__((__nothrow__)) double cos(double __x ) ;
extern  __attribute__((__nothrow__)) double __cos(double __x ) ;
extern  __attribute__((__nothrow__)) double sin(double __x ) ;
extern  __attribute__((__nothrow__)) double __sin(double __x ) ;
extern  __attribute__((__nothrow__)) double tan(double __x ) ;
extern  __attribute__((__nothrow__)) double __tan(double __x ) ;
extern  __attribute__((__nothrow__)) double cosh(double __x ) ;
extern  __attribute__((__nothrow__)) double __cosh(double __x ) ;
extern  __attribute__((__nothrow__)) double sinh(double __x ) ;
extern  __attribute__((__nothrow__)) double __sinh(double __x ) ;
extern  __attribute__((__nothrow__)) double tanh(double __x ) ;
extern  __attribute__((__nothrow__)) double __tanh(double __x ) ;
extern  __attribute__((__nothrow__)) void sincos(double __x , double *__sinx ,
                                                 double *__cosx ) ;
extern  __attribute__((__nothrow__)) void __sincos(double __x , double *__sinx ,
                                                   double *__cosx ) ;
extern  __attribute__((__nothrow__)) double acosh(double __x ) ;
extern  __attribute__((__nothrow__)) double __acosh(double __x ) ;
extern  __attribute__((__nothrow__)) double asinh(double __x ) ;
extern  __attribute__((__nothrow__)) double __asinh(double __x ) ;
extern  __attribute__((__nothrow__)) double atanh(double __x ) ;
extern  __attribute__((__nothrow__)) double __atanh(double __x ) ;
extern  __attribute__((__nothrow__)) double exp(double __x ) ;
extern  __attribute__((__nothrow__)) double __exp(double __x ) ;
extern  __attribute__((__nothrow__)) double frexp(double __x , int *__exponent ) ;
extern  __attribute__((__nothrow__)) double __frexp(double __x ,
                                                    int *__exponent ) ;
extern  __attribute__((__nothrow__)) double ldexp(double __x , int __exponent ) ;
extern  __attribute__((__nothrow__)) double __ldexp(double __x , int __exponent ) ;
extern  __attribute__((__nothrow__)) double log(double __x ) ;
extern  __attribute__((__nothrow__)) double __log(double __x ) ;
extern  __attribute__((__nothrow__)) double log10(double __x ) ;
extern  __attribute__((__nothrow__)) double __log10(double __x ) ;
extern  __attribute__((__nothrow__)) double modf(double __x , double *__iptr ) ;
extern  __attribute__((__nothrow__)) double __modf(double __x , double *__iptr ) ;
extern  __attribute__((__nothrow__)) double exp10(double __x ) ;
extern  __attribute__((__nothrow__)) double __exp10(double __x ) ;
extern  __attribute__((__nothrow__)) double pow10(double __x ) ;
extern  __attribute__((__nothrow__)) double __pow10(double __x ) ;
extern  __attribute__((__nothrow__)) double expm1(double __x ) ;
extern  __attribute__((__nothrow__)) double __expm1(double __x ) ;
extern  __attribute__((__nothrow__)) double log1p(double __x ) ;
extern  __attribute__((__nothrow__)) double __log1p(double __x ) ;
extern  __attribute__((__nothrow__)) double logb(double __x ) ;
extern  __attribute__((__nothrow__)) double __logb(double __x ) ;
extern  __attribute__((__nothrow__)) double exp2(double __x ) ;
extern  __attribute__((__nothrow__)) double __exp2(double __x ) ;
extern  __attribute__((__nothrow__)) double log2(double __x ) ;
extern  __attribute__((__nothrow__)) double __log2(double __x ) ;
extern  __attribute__((__nothrow__)) double pow(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double __pow(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double sqrt(double __x ) ;
extern  __attribute__((__nothrow__)) double __sqrt(double __x ) ;
extern  __attribute__((__nothrow__)) double hypot(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double __hypot(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double cbrt(double __x ) ;
extern  __attribute__((__nothrow__)) double __cbrt(double __x ) ;
__inline extern  __attribute__((__nothrow__)) double ceil(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __ceil(double __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) double fabs(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __fabs(double __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) double floor(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __floor(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double fmod(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double __fmod(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) int __isinf(double __value )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) int __finite(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int isinf(double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int finite(double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double drem(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double __drem(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double significand(double __x ) ;
extern  __attribute__((__nothrow__)) double __significand(double __x ) ;
extern  __attribute__((__nothrow__)) double copysign(double __x , double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __copysign(double __x , double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double nan(char const   *__tagb )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __nan(char const   *__tagb )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int __isnan(double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int isnan(double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double j0(double  ) ;
extern  __attribute__((__nothrow__)) double __j0(double  ) ;
extern  __attribute__((__nothrow__)) double j1(double  ) ;
extern  __attribute__((__nothrow__)) double __j1(double  ) ;
extern  __attribute__((__nothrow__)) double jn(int  , double  ) ;
extern  __attribute__((__nothrow__)) double __jn(int  , double  ) ;
extern  __attribute__((__nothrow__)) double y0(double  ) ;
extern  __attribute__((__nothrow__)) double __y0(double  ) ;
extern  __attribute__((__nothrow__)) double y1(double  ) ;
extern  __attribute__((__nothrow__)) double __y1(double  ) ;
extern  __attribute__((__nothrow__)) double yn(int  , double  ) ;
extern  __attribute__((__nothrow__)) double __yn(int  , double  ) ;
extern  __attribute__((__nothrow__)) double erf(double  ) ;
extern  __attribute__((__nothrow__)) double __erf(double  ) ;
extern  __attribute__((__nothrow__)) double erfc(double  ) ;
extern  __attribute__((__nothrow__)) double __erfc(double  ) ;
extern  __attribute__((__nothrow__)) double lgamma(double  ) ;
extern  __attribute__((__nothrow__)) double __lgamma(double  ) ;
extern  __attribute__((__nothrow__)) double tgamma(double  ) ;
extern  __attribute__((__nothrow__)) double __tgamma(double  ) ;
extern  __attribute__((__nothrow__)) double gamma(double  ) ;
extern  __attribute__((__nothrow__)) double __gamma(double  ) ;
extern  __attribute__((__nothrow__)) double lgamma_r(double  , int *__signgamp ) ;
extern  __attribute__((__nothrow__)) double __lgamma_r(double  ,
                                                       int *__signgamp ) ;
extern  __attribute__((__nothrow__)) double rint(double __x ) ;
extern  __attribute__((__nothrow__)) double __rint(double __x ) ;
extern  __attribute__((__nothrow__)) double nextafter(double __x , double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __nextafter(double __x , double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double nexttoward(double __x ,
                                                       long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __nexttoward(double __x ,
                                                         long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double remainder(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double __remainder(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double scalbn(double __x , int __n ) ;
extern  __attribute__((__nothrow__)) double __scalbn(double __x , int __n ) ;
extern  __attribute__((__nothrow__)) int ilogb(double __x ) ;
extern  __attribute__((__nothrow__)) int __ilogb(double __x ) ;
extern  __attribute__((__nothrow__)) double scalbln(double __x , long __n ) ;
extern  __attribute__((__nothrow__)) double __scalbln(double __x , long __n ) ;
extern  __attribute__((__nothrow__)) double nearbyint(double __x ) ;
extern  __attribute__((__nothrow__)) double __nearbyint(double __x ) ;
extern  __attribute__((__nothrow__)) double round(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __round(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double trunc(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __trunc(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double remquo(double __x , double __y ,
                                                   int *__quo ) ;
extern  __attribute__((__nothrow__)) double __remquo(double __x , double __y ,
                                                     int *__quo ) ;
__inline extern  __attribute__((__nothrow__)) long lrint(double __x ) ;
extern  __attribute__((__nothrow__)) long __lrint(double __x ) ;
__inline extern  __attribute__((__nothrow__)) long long llrint(double __x ) ;
extern  __attribute__((__nothrow__)) long long __llrint(double __x ) ;
extern  __attribute__((__nothrow__)) long lround(double __x ) ;
extern  __attribute__((__nothrow__)) long __lround(double __x ) ;
extern  __attribute__((__nothrow__)) long long llround(double __x ) ;
extern  __attribute__((__nothrow__)) long long __llround(double __x ) ;
extern  __attribute__((__nothrow__)) double fdim(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double __fdim(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double fmax(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double __fmax(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double fmin(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double __fmin(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) int __fpclassify(double __value )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) int __signbit(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double fma(double __x , double __y ,
                                                double __z ) ;
extern  __attribute__((__nothrow__)) double __fma(double __x , double __y ,
                                                  double __z ) ;
extern  __attribute__((__nothrow__)) double scalb(double __x , double __n ) ;
extern  __attribute__((__nothrow__)) double __scalb(double __x , double __n ) ;
extern  __attribute__((__nothrow__)) float acosf(float __x ) ;
extern  __attribute__((__nothrow__)) float __acosf(float __x ) ;
extern  __attribute__((__nothrow__)) float asinf(float __x ) ;
extern  __attribute__((__nothrow__)) float __asinf(float __x ) ;
extern  __attribute__((__nothrow__)) float atanf(float __x ) ;
extern  __attribute__((__nothrow__)) float __atanf(float __x ) ;
extern  __attribute__((__nothrow__)) float atan2f(float __y , float __x ) ;
extern  __attribute__((__nothrow__)) float __atan2f(float __y , float __x ) ;
extern  __attribute__((__nothrow__)) float cosf(float __x ) ;
extern  __attribute__((__nothrow__)) float __cosf(float __x ) ;
extern  __attribute__((__nothrow__)) float sinf(float __x ) ;
extern  __attribute__((__nothrow__)) float __sinf(float __x ) ;
extern  __attribute__((__nothrow__)) float tanf(float __x ) ;
extern  __attribute__((__nothrow__)) float __tanf(float __x ) ;
extern  __attribute__((__nothrow__)) float coshf(float __x ) ;
extern  __attribute__((__nothrow__)) float __coshf(float __x ) ;
extern  __attribute__((__nothrow__)) float sinhf(float __x ) ;
extern  __attribute__((__nothrow__)) float __sinhf(float __x ) ;
extern  __attribute__((__nothrow__)) float tanhf(float __x ) ;
extern  __attribute__((__nothrow__)) float __tanhf(float __x ) ;
extern  __attribute__((__nothrow__)) void sincosf(float __x , float *__sinx ,
                                                  float *__cosx ) ;
extern  __attribute__((__nothrow__)) void __sincosf(float __x , float *__sinx ,
                                                    float *__cosx ) ;
extern  __attribute__((__nothrow__)) float acoshf(float __x ) ;
extern  __attribute__((__nothrow__)) float __acoshf(float __x ) ;
extern  __attribute__((__nothrow__)) float asinhf(float __x ) ;
extern  __attribute__((__nothrow__)) float __asinhf(float __x ) ;
extern  __attribute__((__nothrow__)) float atanhf(float __x ) ;
extern  __attribute__((__nothrow__)) float __atanhf(float __x ) ;
extern  __attribute__((__nothrow__)) float expf(float __x ) ;
extern  __attribute__((__nothrow__)) float __expf(float __x ) ;
extern  __attribute__((__nothrow__)) float frexpf(float __x , int *__exponent ) ;
extern  __attribute__((__nothrow__)) float __frexpf(float __x , int *__exponent ) ;
extern  __attribute__((__nothrow__)) float ldexpf(float __x , int __exponent ) ;
extern  __attribute__((__nothrow__)) float __ldexpf(float __x , int __exponent ) ;
extern  __attribute__((__nothrow__)) float logf(float __x ) ;
extern  __attribute__((__nothrow__)) float __logf(float __x ) ;
extern  __attribute__((__nothrow__)) float log10f(float __x ) ;
extern  __attribute__((__nothrow__)) float __log10f(float __x ) ;
extern  __attribute__((__nothrow__)) float modff(float __x , float *__iptr ) ;
extern  __attribute__((__nothrow__)) float __modff(float __x , float *__iptr ) ;
extern  __attribute__((__nothrow__)) float exp10f(float __x ) ;
extern  __attribute__((__nothrow__)) float __exp10f(float __x ) ;
extern  __attribute__((__nothrow__)) float pow10f(float __x ) ;
extern  __attribute__((__nothrow__)) float __pow10f(float __x ) ;
extern  __attribute__((__nothrow__)) float expm1f(float __x ) ;
extern  __attribute__((__nothrow__)) float __expm1f(float __x ) ;
extern  __attribute__((__nothrow__)) float log1pf(float __x ) ;
extern  __attribute__((__nothrow__)) float __log1pf(float __x ) ;
extern  __attribute__((__nothrow__)) float logbf(float __x ) ;
extern  __attribute__((__nothrow__)) float __logbf(float __x ) ;
extern  __attribute__((__nothrow__)) float exp2f(float __x ) ;
extern  __attribute__((__nothrow__)) float __exp2f(float __x ) ;
extern  __attribute__((__nothrow__)) float log2f(float __x ) ;
extern  __attribute__((__nothrow__)) float __log2f(float __x ) ;
extern  __attribute__((__nothrow__)) float powf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float __powf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float sqrtf(float __x ) ;
extern  __attribute__((__nothrow__)) float __sqrtf(float __x ) ;
extern  __attribute__((__nothrow__)) float hypotf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float __hypotf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float cbrtf(float __x ) ;
extern  __attribute__((__nothrow__)) float __cbrtf(float __x ) ;
__inline extern  __attribute__((__nothrow__)) float ceilf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __ceilf(float __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) float fabsf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __fabsf(float __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) float floorf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __floorf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float fmodf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float __fmodf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) int __isinff(float __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int __finitef(float __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int isinff(float __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int finitef(float __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float dremf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float __dremf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float significandf(float __x ) ;
extern  __attribute__((__nothrow__)) float __significandf(float __x ) ;
extern  __attribute__((__nothrow__)) float copysignf(float __x , float __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __copysignf(float __x , float __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float nanf(char const   *__tagb )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __nanf(char const   *__tagb )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int __isnanf(float __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int isnanf(float __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float j0f(float  ) ;
extern  __attribute__((__nothrow__)) float __j0f(float  ) ;
extern  __attribute__((__nothrow__)) float j1f(float  ) ;
extern  __attribute__((__nothrow__)) float __j1f(float  ) ;
extern  __attribute__((__nothrow__)) float jnf(int  , float  ) ;
extern  __attribute__((__nothrow__)) float __jnf(int  , float  ) ;
extern  __attribute__((__nothrow__)) float y0f(float  ) ;
extern  __attribute__((__nothrow__)) float __y0f(float  ) ;
extern  __attribute__((__nothrow__)) float y1f(float  ) ;
extern  __attribute__((__nothrow__)) float __y1f(float  ) ;
extern  __attribute__((__nothrow__)) float ynf(int  , float  ) ;
extern  __attribute__((__nothrow__)) float __ynf(int  , float  ) ;
extern  __attribute__((__nothrow__)) float erff(float  ) ;
extern  __attribute__((__nothrow__)) float __erff(float  ) ;
extern  __attribute__((__nothrow__)) float erfcf(float  ) ;
extern  __attribute__((__nothrow__)) float __erfcf(float  ) ;
extern  __attribute__((__nothrow__)) float lgammaf(float  ) ;
extern  __attribute__((__nothrow__)) float __lgammaf(float  ) ;
extern  __attribute__((__nothrow__)) float tgammaf(float  ) ;
extern  __attribute__((__nothrow__)) float __tgammaf(float  ) ;
extern  __attribute__((__nothrow__)) float gammaf(float  ) ;
extern  __attribute__((__nothrow__)) float __gammaf(float  ) ;
extern  __attribute__((__nothrow__)) float lgammaf_r(float  , int *__signgamp ) ;
extern  __attribute__((__nothrow__)) float __lgammaf_r(float  , int *__signgamp ) ;
extern  __attribute__((__nothrow__)) float rintf(float __x ) ;
extern  __attribute__((__nothrow__)) float __rintf(float __x ) ;
extern  __attribute__((__nothrow__)) float nextafterf(float __x , float __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __nextafterf(float __x , float __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float nexttowardf(float __x ,
                                                       long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __nexttowardf(float __x ,
                                                         long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float remainderf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float __remainderf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float scalbnf(float __x , int __n ) ;
extern  __attribute__((__nothrow__)) float __scalbnf(float __x , int __n ) ;
extern  __attribute__((__nothrow__)) int ilogbf(float __x ) ;
extern  __attribute__((__nothrow__)) int __ilogbf(float __x ) ;
extern  __attribute__((__nothrow__)) float scalblnf(float __x , long __n ) ;
extern  __attribute__((__nothrow__)) float __scalblnf(float __x , long __n ) ;
extern  __attribute__((__nothrow__)) float nearbyintf(float __x ) ;
extern  __attribute__((__nothrow__)) float __nearbyintf(float __x ) ;
extern  __attribute__((__nothrow__)) float roundf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __roundf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float truncf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __truncf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float remquof(float __x , float __y ,
                                                   int *__quo ) ;
extern  __attribute__((__nothrow__)) float __remquof(float __x , float __y ,
                                                     int *__quo ) ;
__inline extern  __attribute__((__nothrow__)) long lrintf(float __x ) ;
extern  __attribute__((__nothrow__)) long __lrintf(float __x ) ;
__inline extern  __attribute__((__nothrow__)) long long llrintf(float __x ) ;
extern  __attribute__((__nothrow__)) long long __llrintf(float __x ) ;
extern  __attribute__((__nothrow__)) long lroundf(float __x ) ;
extern  __attribute__((__nothrow__)) long __lroundf(float __x ) ;
extern  __attribute__((__nothrow__)) long long llroundf(float __x ) ;
extern  __attribute__((__nothrow__)) long long __llroundf(float __x ) ;
extern  __attribute__((__nothrow__)) float fdimf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float __fdimf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float fmaxf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float __fmaxf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float fminf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float __fminf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) int __fpclassifyf(float __value )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) int __signbitf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float fmaf(float __x , float __y ,
                                                float __z ) ;
extern  __attribute__((__nothrow__)) float __fmaf(float __x , float __y ,
                                                  float __z ) ;
extern  __attribute__((__nothrow__)) float scalbf(float __x , float __n ) ;
extern  __attribute__((__nothrow__)) float __scalbf(float __x , float __n ) ;
extern  __attribute__((__nothrow__)) long double acosl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __acosl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double asinl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __asinl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double atanl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __atanl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double atan2l(long double __y ,
                                                        long double __x ) ;
__inline extern  __attribute__((__nothrow__)) long double __atan2l(long double __y ,
                                                                   long double __x ) ;
extern  __attribute__((__nothrow__)) long double cosl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __cosl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double sinl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __sinl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double tanl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __tanl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double coshl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __coshl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double sinhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __sinhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double tanhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __tanhl(long double __x ) ;
extern  __attribute__((__nothrow__)) void sincosl(long double __x ,
                                                  long double *__sinx ,
                                                  long double *__cosx ) ;
extern  __attribute__((__nothrow__)) void __sincosl(long double __x ,
                                                    long double *__sinx ,
                                                    long double *__cosx ) ;
extern  __attribute__((__nothrow__)) long double acoshl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __acoshl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double asinhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __asinhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double atanhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __atanhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double expl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __expl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double frexpl(long double __x ,
                                                        int *__exponent ) ;
extern  __attribute__((__nothrow__)) long double __frexpl(long double __x ,
                                                          int *__exponent ) ;
extern  __attribute__((__nothrow__)) long double ldexpl(long double __x ,
                                                        int __exponent ) ;
extern  __attribute__((__nothrow__)) long double __ldexpl(long double __x ,
                                                          int __exponent ) ;
extern  __attribute__((__nothrow__)) long double logl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __logl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double log10l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __log10l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double modfl(long double __x ,
                                                       long double *__iptr ) ;
extern  __attribute__((__nothrow__)) long double __modfl(long double __x ,
                                                         long double *__iptr ) ;
extern  __attribute__((__nothrow__)) long double exp10l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __exp10l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double pow10l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __pow10l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double expm1l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __expm1l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double log1pl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __log1pl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double logbl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __logbl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double exp2l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __exp2l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double log2l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __log2l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double powl(long double __x ,
                                                      long double __y ) ;
extern  __attribute__((__nothrow__)) long double __powl(long double __x ,
                                                        long double __y ) ;
extern  __attribute__((__nothrow__)) long double sqrtl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __sqrtl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double hypotl(long double __x ,
                                                        long double __y ) ;
extern  __attribute__((__nothrow__)) long double __hypotl(long double __x ,
                                                          long double __y ) ;
extern  __attribute__((__nothrow__)) long double cbrtl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __cbrtl(long double __x ) ;
__inline extern  __attribute__((__nothrow__)) long double ceill(long double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __ceill(long double __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) long double fabsl(long double __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) long double __fabsl(long double __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) long double floorl(long double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __floorl(long double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double fmodl(long double __x ,
                                                       long double __y ) ;
extern  __attribute__((__nothrow__)) long double __fmodl(long double __x ,
                                                         long double __y ) ;
extern  __attribute__((__nothrow__)) int __isinfl(long double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int __finitel(long double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int isinfl(long double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int finitel(long double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double dreml(long double __x ,
                                                       long double __y ) ;
extern  __attribute__((__nothrow__)) long double __dreml(long double __x ,
                                                         long double __y ) ;
extern  __attribute__((__nothrow__)) long double significandl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __significandl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double copysignl(long double __x ,
                                                           long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __copysignl(long double __x ,
                                                             long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double nanl(char const   *__tagb )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __nanl(char const   *__tagb )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int __isnanl(long double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int isnanl(long double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double j0l(long double  ) ;
extern  __attribute__((__nothrow__)) long double __j0l(long double  ) ;
extern  __attribute__((__nothrow__)) long double j1l(long double  ) ;
extern  __attribute__((__nothrow__)) long double __j1l(long double  ) ;
extern  __attribute__((__nothrow__)) long double jnl(int  , long double  ) ;
extern  __attribute__((__nothrow__)) long double __jnl(int  , long double  ) ;
extern  __attribute__((__nothrow__)) long double y0l(long double  ) ;
extern  __attribute__((__nothrow__)) long double __y0l(long double  ) ;
extern  __attribute__((__nothrow__)) long double y1l(long double  ) ;
extern  __attribute__((__nothrow__)) long double __y1l(long double  ) ;
extern  __attribute__((__nothrow__)) long double ynl(int  , long double  ) ;
extern  __attribute__((__nothrow__)) long double __ynl(int  , long double  ) ;
extern  __attribute__((__nothrow__)) long double erfl(long double  ) ;
extern  __attribute__((__nothrow__)) long double __erfl(long double  ) ;
extern  __attribute__((__nothrow__)) long double erfcl(long double  ) ;
extern  __attribute__((__nothrow__)) long double __erfcl(long double  ) ;
extern  __attribute__((__nothrow__)) long double lgammal(long double  ) ;
extern  __attribute__((__nothrow__)) long double __lgammal(long double  ) ;
extern  __attribute__((__nothrow__)) long double tgammal(long double  ) ;
extern  __attribute__((__nothrow__)) long double __tgammal(long double  ) ;
extern  __attribute__((__nothrow__)) long double gammal(long double  ) ;
extern  __attribute__((__nothrow__)) long double __gammal(long double  ) ;
extern  __attribute__((__nothrow__)) long double lgammal_r(long double  ,
                                                           int *__signgamp ) ;
extern  __attribute__((__nothrow__)) long double __lgammal_r(long double  ,
                                                             int *__signgamp ) ;
extern  __attribute__((__nothrow__)) long double rintl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __rintl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double nextafterl(long double __x ,
                                                            long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __nextafterl(long double __x ,
                                                              long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double nexttowardl(long double __x ,
                                                             long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __nexttowardl(long double __x ,
                                                               long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double remainderl(long double __x ,
                                                            long double __y ) ;
extern  __attribute__((__nothrow__)) long double __remainderl(long double __x ,
                                                              long double __y ) ;
extern  __attribute__((__nothrow__)) long double scalbnl(long double __x ,
                                                         int __n ) ;
extern  __attribute__((__nothrow__)) long double __scalbnl(long double __x ,
                                                           int __n ) ;
extern  __attribute__((__nothrow__)) int ilogbl(long double __x ) ;
extern  __attribute__((__nothrow__)) int __ilogbl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double scalblnl(long double __x ,
                                                          long __n ) ;
extern  __attribute__((__nothrow__)) long double __scalblnl(long double __x ,
                                                            long __n ) ;
extern  __attribute__((__nothrow__)) long double nearbyintl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __nearbyintl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double roundl(long double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __roundl(long double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double truncl(long double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __truncl(long double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double remquol(long double __x ,
                                                         long double __y ,
                                                         int *__quo ) ;
extern  __attribute__((__nothrow__)) long double __remquol(long double __x ,
                                                           long double __y ,
                                                           int *__quo ) ;
__inline extern  __attribute__((__nothrow__)) long lrintl(long double __x ) ;
extern  __attribute__((__nothrow__)) long __lrintl(long double __x ) ;
__inline extern  __attribute__((__nothrow__)) long long llrintl(long double __x ) ;
extern  __attribute__((__nothrow__)) long long __llrintl(long double __x ) ;
extern  __attribute__((__nothrow__)) long lroundl(long double __x ) ;
extern  __attribute__((__nothrow__)) long __lroundl(long double __x ) ;
extern  __attribute__((__nothrow__)) long long llroundl(long double __x ) ;
extern  __attribute__((__nothrow__)) long long __llroundl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double fdiml(long double __x ,
                                                       long double __y ) ;
extern  __attribute__((__nothrow__)) long double __fdiml(long double __x ,
                                                         long double __y ) ;
extern  __attribute__((__nothrow__)) long double fmaxl(long double __x ,
                                                       long double __y ) ;
extern  __attribute__((__nothrow__)) long double __fmaxl(long double __x ,
                                                         long double __y ) ;
extern  __attribute__((__nothrow__)) long double fminl(long double __x ,
                                                       long double __y ) ;
extern  __attribute__((__nothrow__)) long double __fminl(long double __x ,
                                                         long double __y ) ;
extern  __attribute__((__nothrow__)) int __fpclassifyl(long double __value )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) int __signbitl(long double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double fmal(long double __x ,
                                                      long double __y ,
                                                      long double __z ) ;
extern  __attribute__((__nothrow__)) long double __fmal(long double __x ,
                                                        long double __y ,
                                                        long double __z ) ;
extern  __attribute__((__nothrow__)) long double scalbl(long double __x ,
                                                        long double __n ) ;
extern  __attribute__((__nothrow__)) long double __scalbl(long double __x ,
                                                          long double __n ) ;
extern int signgam ;
extern _LIB_VERSION_TYPE _LIB_VERSION ;
extern int matherr(struct exception *__exc ) ;
__inline extern  __attribute__((__nothrow__)) int __signbitf(float __x )  __attribute__((__const__)) ;
__inline extern int __signbitf(float __x ) 
{ 
  union __anonunion___u_24 __u ;

  {
  __u.__f = __x;
  return (__u.__i < 0);
}
}
__inline extern  __attribute__((__nothrow__)) int __signbit(double __x )  __attribute__((__const__)) ;
__inline extern int __signbit(double __x ) 
{ 
  union __anonunion___u_25 __u ;

  {
  __u.__d = __x;
  return (__u.__i[1] < 0);
}
}
__inline extern  __attribute__((__nothrow__)) int __signbitl(long double __x )  __attribute__((__const__)) ;
__inline extern int __signbitl(long double __x ) 
{ 
  union __anonunion___u_26 __u ;

  {
  __u.__l = __x;
  return ((__u.__i[2] & 32768) != 0);
}
}
__inline extern  __attribute__((__nothrow__)) double __sgn(double __x ) ;
__inline extern  __attribute__((__nothrow__)) double __sgn(double __x ) ;
__inline extern double __sgn(double __x ) 
{ 
  double tmp ;
  double tmp___0 ;

  {
  if (__x == 0.0) {
    tmp___0 = 0.0;
  } else {
    if (__x > 0.0) {
      tmp = 1.0;
    } else {
      tmp = - 1.0;
    }
    tmp___0 = tmp;
  }
  return (tmp___0);
}
}
__inline extern  __attribute__((__nothrow__)) float __sgnf(float __x ) ;
__inline extern  __attribute__((__nothrow__)) float __sgnf(float __x ) ;
__inline extern float __sgnf(float __x ) 
{ 
  double tmp ;
  double tmp___0 ;

  {
  if ((double )__x == 0.0) {
    tmp___0 = 0.0;
  } else {
    if ((double )__x > 0.0) {
      tmp = 1.0;
    } else {
      tmp = - 1.0;
    }
    tmp___0 = tmp;
  }
  return ((float )tmp___0);
}
}
__inline extern  __attribute__((__nothrow__)) long double __sgnl(long double __x ) ;
__inline extern  __attribute__((__nothrow__)) long double __sgnl(long double __x ) ;
__inline extern long double __sgnl(long double __x ) 
{ 
  double tmp ;
  double tmp___0 ;

  {
  if (__x == (long double )0.0) {
    tmp___0 = 0.0;
  } else {
    if (__x > (long double )0.0) {
      tmp = 1.0;
    } else {
      tmp = - 1.0;
    }
    tmp___0 = tmp;
  }
  return ((long double )tmp___0);
}
}
__inline extern  __attribute__((__nothrow__)) long double __atan2l(long double __y ,
                                                                   long double __x ) ;
__inline extern long double __atan2l(long double __y , long double __x ) 
{ 
  long double tmp ;

  {
  tmp = __builtin_atan2l(__y, __x);
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) double fabs(double __x )  __attribute__((__const__)) ;
__inline extern double fabs(double __x ) 
{ 
  double tmp ;

  {
  tmp = __builtin_fabs(__x);
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) float fabsf(float __x )  __attribute__((__const__)) ;
__inline extern float fabsf(float __x ) 
{ 
  float tmp ;

  {
  tmp = __builtin_fabsf(__x);
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) long double fabsl(long double __x )  __attribute__((__const__)) ;
__inline extern long double fabsl(long double __x ) 
{ 
  long double tmp ;

  {
  tmp = __builtin_fabsl(__x);
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) long double __fabsl(long double __x )  __attribute__((__const__)) ;
__inline extern long double __fabsl(long double __x ) 
{ 
  long double tmp ;

  {
  tmp = __builtin_fabsl(__x);
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) long double __sgn1l(long double __x ) ;
__inline extern  __attribute__((__nothrow__)) long double __sgn1l(long double __x ) ;
__inline extern long double __sgn1l(long double __x ) 
{ 
  union __anonunion___n_27 __n ;

  {
  __n.__xld = __x;
  __n.__xi[2] = (__n.__xi[2] & 32768U) | 16383U;
  __n.__xi[1] = 2147483648U;
  __n.__xi[0] = 0U;
  return (__n.__xld);
}
}
__inline extern  __attribute__((__nothrow__)) double floor(double __x )  __attribute__((__const__)) ;
__inline extern double floor(double __x ) 
{ 
  register long double __value ;
  register int __ignore ;
  unsigned short __cw ;
  unsigned short __cwtmp ;

  {
  __asm__  volatile   ("fnstcw %3\n\t"
                       "movzwl %3, %1\n\t"
                       "andl $0xf3ff, %1\n\t"
                       "orl $0x0400, %1\n\t"
                       "movw %w1, %2\n\t"
                       "fldcw %2\n\t"
                       "frndint\n\t"
                       "fldcw %3": "=t" (__value), "=&q" (__ignore),
                       "=m" (__cwtmp), "=m" (__cw): "0" (__x));
  return ((double )__value);
}
}
__inline extern  __attribute__((__nothrow__)) float floorf(float __x )  __attribute__((__const__)) ;
__inline extern float floorf(float __x ) 
{ 
  register long double __value ;
  register int __ignore ;
  unsigned short __cw ;
  unsigned short __cwtmp ;

  {
  __asm__  volatile   ("fnstcw %3\n\t"
                       "movzwl %3, %1\n\t"
                       "andl $0xf3ff, %1\n\t"
                       "orl $0x0400, %1\n\t"
                       "movw %w1, %2\n\t"
                       "fldcw %2\n\t"
                       "frndint\n\t"
                       "fldcw %3": "=t" (__value), "=&q" (__ignore),
                       "=m" (__cwtmp), "=m" (__cw): "0" (__x));
  return ((float )__value);
}
}
__inline extern  __attribute__((__nothrow__)) long double floorl(long double __x )  __attribute__((__const__)) ;
__inline extern long double floorl(long double __x ) 
{ 
  register long double __value ;
  register int __ignore ;
  unsigned short __cw ;
  unsigned short __cwtmp ;

  {
  __asm__  volatile   ("fnstcw %3\n\t"
                       "movzwl %3, %1\n\t"
                       "andl $0xf3ff, %1\n\t"
                       "orl $0x0400, %1\n\t"
                       "movw %w1, %2\n\t"
                       "fldcw %2\n\t"
                       "frndint\n\t"
                       "fldcw %3": "=t" (__value), "=&q" (__ignore),
                       "=m" (__cwtmp), "=m" (__cw): "0" (__x));
  return (__value);
}
}
__inline extern  __attribute__((__nothrow__)) double ceil(double __x )  __attribute__((__const__)) ;
__inline extern double ceil(double __x ) 
{ 
  register long double __value ;
  register int __ignore ;
  unsigned short __cw ;
  unsigned short __cwtmp ;

  {
  __asm__  volatile   ("fnstcw %3\n\t"
                       "movzwl %3, %1\n\t"
                       "andl $0xf3ff, %1\n\t"
                       "orl $0x0800, %1\n\t"
                       "movw %w1, %2\n\t"
                       "fldcw %2\n\t"
                       "frndint\n\t"
                       "fldcw %3": "=t" (__value), "=&q" (__ignore),
                       "=m" (__cwtmp), "=m" (__cw): "0" (__x));
  return ((double )__value);
}
}
__inline extern  __attribute__((__nothrow__)) float ceilf(float __x )  __attribute__((__const__)) ;
__inline extern float ceilf(float __x ) 
{ 
  register long double __value ;
  register int __ignore ;
  unsigned short __cw ;
  unsigned short __cwtmp ;

  {
  __asm__  volatile   ("fnstcw %3\n\t"
                       "movzwl %3, %1\n\t"
                       "andl $0xf3ff, %1\n\t"
                       "orl $0x0800, %1\n\t"
                       "movw %w1, %2\n\t"
                       "fldcw %2\n\t"
                       "frndint\n\t"
                       "fldcw %3": "=t" (__value), "=&q" (__ignore),
                       "=m" (__cwtmp), "=m" (__cw): "0" (__x));
  return ((float )__value);
}
}
__inline extern  __attribute__((__nothrow__)) long double ceill(long double __x )  __attribute__((__const__)) ;
__inline extern long double ceill(long double __x ) 
{ 
  register long double __value ;
  register int __ignore ;
  unsigned short __cw ;
  unsigned short __cwtmp ;

  {
  __asm__  volatile   ("fnstcw %3\n\t"
                       "movzwl %3, %1\n\t"
                       "andl $0xf3ff, %1\n\t"
                       "orl $0x0800, %1\n\t"
                       "movw %w1, %2\n\t"
                       "fldcw %2\n\t"
                       "frndint\n\t"
                       "fldcw %3": "=t" (__value), "=&q" (__ignore),
                       "=m" (__cwtmp), "=m" (__cw): "0" (__x));
  return (__value);
}
}
__inline extern  __attribute__((__nothrow__)) long lrintf(float __x ) ;
__inline extern long lrintf(float __x ) 
{ 
  long __lrintres ;

  {
  __asm__  volatile   ("fistpl %0": "=m" (__lrintres): "t" (__x): "st");
  return (__lrintres);
}
}
__inline extern  __attribute__((__nothrow__)) long lrint(double __x ) ;
__inline extern long lrint(double __x ) 
{ 
  long __lrintres ;

  {
  __asm__  volatile   ("fistpl %0": "=m" (__lrintres): "t" (__x): "st");
  return (__lrintres);
}
}
__inline extern  __attribute__((__nothrow__)) long lrintl(long double __x ) ;
__inline extern long lrintl(long double __x ) 
{ 
  long __lrintres ;

  {
  __asm__  volatile   ("fistpl %0": "=m" (__lrintres): "t" (__x): "st");
  return (__lrintres);
}
}
__inline extern  __attribute__((__nothrow__)) long long llrintf(float __x ) ;
__inline extern long long llrintf(float __x ) 
{ 
  long long __llrintres ;

  {
  __asm__  volatile   ("fistpll %0": "=m" (__llrintres): "t" (__x): "st");
  return (__llrintres);
}
}
__inline extern  __attribute__((__nothrow__)) long long llrint(double __x ) ;
__inline extern long long llrint(double __x ) 
{ 
  long long __llrintres ;

  {
  __asm__  volatile   ("fistpll %0": "=m" (__llrintres): "t" (__x): "st");
  return (__llrintres);
}
}
__inline extern  __attribute__((__nothrow__)) long long llrintl(long double __x ) ;
__inline extern long long llrintl(long double __x ) 
{ 
  long long __llrintres ;

  {
  __asm__  volatile   ("fistpll %0": "=m" (__llrintres): "t" (__x): "st");
  return (__llrintres);
}
}
__inline extern  __attribute__((__nothrow__)) int __finite(double __x )  __attribute__((__const__)) ;
__inline extern int __finite(double __x ) 
{ 
  union __anonunion_28 __constr_expr_0 ;

  {
  __constr_expr_0.__d = __x;
  return ((int )((((unsigned int )__constr_expr_0.__i[1] | 2148532223U) + 1U) >> 31));
}
}
extern  __attribute__((__nothrow__)) intmax_t imaxabs(intmax_t __n )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) imaxdiv_t imaxdiv(intmax_t __numer ,
                                                       intmax_t __denom )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) intmax_t strtoimax(char const   * __restrict  nptr ,
                                                                 char ** __restrict  endptr ,
                                                                 int base ) ;
__inline extern  __attribute__((__nothrow__)) uintmax_t strtoumax(char const   * __restrict  nptr ,
                                                                  char ** __restrict  endptr ,
                                                                  int base ) ;
__inline extern  __attribute__((__nothrow__)) intmax_t wcstoimax(__gwchar_t const   * __restrict  nptr ,
                                                                 __gwchar_t ** __restrict  endptr ,
                                                                 int base ) ;
__inline extern  __attribute__((__nothrow__)) uintmax_t wcstoumax(__gwchar_t const   * __restrict  nptr ,
                                                                  __gwchar_t ** __restrict  endptr ,
                                                                  int base ) ;
extern  __attribute__((__nothrow__)) long long ( __attribute__((__nonnull__(1))) __strtoll_internal)(char const   * __restrict  __nptr ,
                                                                                                     char ** __restrict  __endptr ,
                                                                                                     int __base ,
                                                                                                     int __group ) ;
__inline extern  __attribute__((__nothrow__)) intmax_t strtoimax(char const   * __restrict  nptr ,
                                                                 char ** __restrict  endptr ,
                                                                 int base ) ;
__inline extern intmax_t strtoimax(char const   * __restrict  nptr ,
                                   char ** __restrict  endptr , int base ) 
{ 
  long long tmp ;

  {
  tmp = __strtoll_internal(nptr, endptr, base, 0);
  return (tmp);
}
}
extern  __attribute__((__nothrow__)) unsigned long long ( __attribute__((__nonnull__(1))) __strtoull_internal)(char const   * __restrict  __nptr ,
                                                                                                               char ** __restrict  __endptr ,
                                                                                                               int __base ,
                                                                                                               int __group ) ;
__inline extern  __attribute__((__nothrow__)) uintmax_t strtoumax(char const   * __restrict  nptr ,
                                                                  char ** __restrict  endptr ,
                                                                  int base ) ;
__inline extern uintmax_t strtoumax(char const   * __restrict  nptr ,
                                    char ** __restrict  endptr , int base ) 
{ 
  unsigned long long tmp ;

  {
  tmp = __strtoull_internal(nptr, endptr, base, 0);
  return (tmp);
}
}
extern  __attribute__((__nothrow__)) long long ( __attribute__((__nonnull__(1))) __wcstoll_internal)(__gwchar_t const   * __restrict  __nptr ,
                                                                                                     __gwchar_t ** __restrict  __endptr ,
                                                                                                     int __base ,
                                                                                                     int __group ) ;
__inline extern  __attribute__((__nothrow__)) intmax_t wcstoimax(__gwchar_t const   * __restrict  nptr ,
                                                                 __gwchar_t ** __restrict  endptr ,
                                                                 int base ) ;
__inline extern intmax_t wcstoimax(__gwchar_t const   * __restrict  nptr ,
                                   __gwchar_t ** __restrict  endptr , int base ) 
{ 
  long long tmp ;

  {
  tmp = __wcstoll_internal(nptr, endptr, base, 0);
  return (tmp);
}
}
extern  __attribute__((__nothrow__)) unsigned long long ( __attribute__((__nonnull__(1))) __wcstoull_internal)(__gwchar_t const   * __restrict  __nptr ,
                                                                                                               __gwchar_t ** __restrict  __endptr ,
                                                                                                               int __base ,
                                                                                                               int __group ) ;
__inline extern  __attribute__((__nothrow__)) uintmax_t wcstoumax(__gwchar_t const   * __restrict  nptr ,
                                                                  __gwchar_t ** __restrict  endptr ,
                                                                  int base ) ;
__inline extern uintmax_t wcstoumax(__gwchar_t const   * __restrict  nptr ,
                                    __gwchar_t ** __restrict  endptr , int base ) 
{ 
  unsigned long long tmp ;

  {
  tmp = __wcstoull_internal(nptr, endptr, base, 0);
  return (tmp);
}
}
extern struct _IO_FILE_plus _IO_2_1_stdin_ ;
extern struct _IO_FILE_plus _IO_2_1_stdout_ ;
extern struct _IO_FILE_plus _IO_2_1_stderr_ ;
extern void _IO_cookie_init(struct _IO_cookie_file *__cfile , int __read_write ,
                            void *__cookie , _IO_cookie_io_functions_t __fns ) ;
extern int __underflow(_IO_FILE * ) ;
extern int __uflow(_IO_FILE * ) ;
extern int __overflow(_IO_FILE * , int  ) ;
extern int _IO_getc(_IO_FILE *__fp ) ;
extern int _IO_putc(int __c , _IO_FILE *__fp ) ;
extern  __attribute__((__nothrow__)) int _IO_feof(_IO_FILE *__fp ) ;
extern  __attribute__((__nothrow__)) int _IO_ferror(_IO_FILE *__fp ) ;
extern int _IO_peekc_locked(_IO_FILE *__fp ) ;
extern  __attribute__((__nothrow__)) void _IO_flockfile(_IO_FILE * ) ;
extern  __attribute__((__nothrow__)) void _IO_funlockfile(_IO_FILE * ) ;
extern  __attribute__((__nothrow__)) int _IO_ftrylockfile(_IO_FILE * ) ;
extern int _IO_vfscanf(_IO_FILE * __restrict   , char const   * __restrict   ,
                       __gnuc_va_list  , int * __restrict   ) ;
extern int _IO_vfprintf(_IO_FILE * __restrict   , char const   * __restrict   ,
                        __gnuc_va_list  ) ;
extern __ssize_t _IO_padn(_IO_FILE * , int  , __ssize_t  ) ;
extern size_t _IO_sgetn(_IO_FILE * , void * , size_t  ) ;
extern __off64_t _IO_seekoff(_IO_FILE * , __off64_t  , int  , int  ) ;
extern __off64_t _IO_seekpos(_IO_FILE * , __off64_t  , int  ) ;
extern  __attribute__((__nothrow__)) void _IO_free_backup_area(_IO_FILE * ) ;
extern struct _IO_FILE *stdin ;
extern struct _IO_FILE *stdout ;
extern struct _IO_FILE *stderr ;
extern  __attribute__((__nothrow__)) int remove(char const   *__filename ) ;
extern  __attribute__((__nothrow__)) int rename(char const   *__old ,
                                                char const   *__new ) ;
extern  __attribute__((__nothrow__)) int renameat(int __oldfd ,
                                                  char const   *__old ,
                                                  int __newfd ,
                                                  char const   *__new ) ;
extern FILE *tmpfile(void) ;
extern FILE *tmpfile64(void) ;
extern  __attribute__((__nothrow__)) char *tmpnam(char *__s ) ;
extern  __attribute__((__nothrow__)) char *tmpnam_r(char *__s ) ;
extern  __attribute__((__nothrow__)) char *tempnam(char const   *__dir ,
                                                   char const   *__pfx )  __attribute__((__malloc__)) ;
extern int fclose(FILE *__stream ) ;
extern int fflush(FILE *__stream ) ;
extern int fflush_unlocked(FILE *__stream ) ;
extern int fcloseall(void) ;
extern FILE *fopen(char const   * __restrict  __filename ,
                   char const   * __restrict  __modes ) ;
extern FILE *freopen(char const   * __restrict  __filename ,
                     char const   * __restrict  __modes ,
                     FILE * __restrict  __stream ) ;
extern FILE *fopen64(char const   * __restrict  __filename ,
                     char const   * __restrict  __modes ) ;
extern FILE *freopen64(char const   * __restrict  __filename ,
                       char const   * __restrict  __modes ,
                       FILE * __restrict  __stream ) ;
extern  __attribute__((__nothrow__)) FILE *fdopen(int __fd ,
                                                  char const   *__modes ) ;
extern  __attribute__((__nothrow__)) FILE *fopencookie(void * __restrict  __magic_cookie ,
                                                       char const   * __restrict  __modes ,
                                                       _IO_cookie_io_functions_t __io_funcs ) ;
extern  __attribute__((__nothrow__)) FILE *fmemopen(void *__s , size_t __len ,
                                                    char const   *__modes ) ;
extern  __attribute__((__nothrow__)) FILE *open_memstream(char **__bufloc ,
                                                          size_t *__sizeloc ) ;
extern  __attribute__((__nothrow__)) void setbuf(FILE * __restrict  __stream ,
                                                 char * __restrict  __buf ) ;
extern  __attribute__((__nothrow__)) int setvbuf(FILE * __restrict  __stream ,
                                                 char * __restrict  __buf ,
                                                 int __modes , size_t __n ) ;
extern  __attribute__((__nothrow__)) void setbuffer(FILE * __restrict  __stream ,
                                                    char * __restrict  __buf ,
                                                    size_t __size ) ;
extern  __attribute__((__nothrow__)) void setlinebuf(FILE *__stream ) ;
extern int fprintf(FILE * __restrict  __stream ,
                   char const   * __restrict  __format  , ...) ;
extern int printf(char const   * __restrict  __format  , ...) ;
extern  __attribute__((__nothrow__)) int sprintf(char * __restrict  __s ,
                                                 char const   * __restrict  __format 
                                                 , ...) ;
extern int vfprintf(FILE * __restrict  __s ,
                    char const   * __restrict  __format , __gnuc_va_list __arg ) ;
__inline extern int vprintf(char const   * __restrict  __fmt ,
                            __gnuc_va_list __arg ) ;
extern  __attribute__((__nothrow__)) int vsprintf(char * __restrict  __s ,
                                                  char const   * __restrict  __format ,
                                                  __gnuc_va_list __arg ) ;
extern  __attribute__((__nothrow__)) int ( /* format attribute */  snprintf)(char * __restrict  __s ,
                                                                             size_t __maxlen ,
                                                                             char const   * __restrict  __format 
                                                                             , ...) ;
extern  __attribute__((__nothrow__)) int ( /* format attribute */  vsnprintf)(char * __restrict  __s ,
                                                                              size_t __maxlen ,
                                                                              char const   * __restrict  __format ,
                                                                              __gnuc_va_list __arg ) ;
extern  __attribute__((__nothrow__)) int ( /* format attribute */  vasprintf)(char ** __restrict  __ptr ,
                                                                              char const   * __restrict  __f ,
                                                                              __gnuc_va_list __arg ) ;
extern  __attribute__((__nothrow__)) int ( /* format attribute */  __asprintf)(char ** __restrict  __ptr ,
                                                                               char const   * __restrict  __fmt 
                                                                               , ...) ;
extern  __attribute__((__nothrow__)) int ( /* format attribute */  asprintf)(char ** __restrict  __ptr ,
                                                                             char const   * __restrict  __fmt 
                                                                             , ...) ;
extern int ( /* format attribute */  vdprintf)(int __fd ,
                                               char const   * __restrict  __fmt ,
                                               __gnuc_va_list __arg ) ;
extern int ( /* format attribute */  dprintf)(int __fd ,
                                              char const   * __restrict  __fmt 
                                              , ...) ;
extern int fscanf(FILE * __restrict  __stream ,
                  char const   * __restrict  __format  , ...) ;
extern int scanf(char const   * __restrict  __format  , ...) ;
extern  __attribute__((__nothrow__)) int sscanf(char const   * __restrict  __s ,
                                                char const   * __restrict  __format 
                                                , ...) ;
extern int ( /* format attribute */  vfscanf)(FILE * __restrict  __s ,
                                              char const   * __restrict  __format ,
                                              __gnuc_va_list __arg ) ;
extern int ( /* format attribute */  vscanf)(char const   * __restrict  __format ,
                                             __gnuc_va_list __arg ) ;
extern  __attribute__((__nothrow__)) int ( /* format attribute */  vsscanf)(char const   * __restrict  __s ,
                                                                            char const   * __restrict  __format ,
                                                                            __gnuc_va_list __arg ) ;
extern int fgetc(FILE *__stream ) ;
extern int getc(FILE *__stream ) ;
__inline extern int getchar(void) ;
__inline extern int getc_unlocked(FILE *__fp ) ;
__inline extern int getchar_unlocked(void) ;
__inline extern int fgetc_unlocked(FILE *__fp ) ;
extern int fputc(int __c , FILE *__stream ) ;
extern int putc(int __c , FILE *__stream ) ;
__inline extern int putchar(int __c ) ;
__inline extern int fputc_unlocked(int __c , FILE *__stream ) ;
__inline extern int putc_unlocked(int __c , FILE *__stream ) ;
__inline extern int putchar_unlocked(int __c ) ;
extern int getw(FILE *__stream ) ;
extern int putw(int __w , FILE *__stream ) ;
extern char *fgets(char * __restrict  __s , int __n ,
                   FILE * __restrict  __stream ) ;
extern char *gets(char *__s ) ;
extern char *fgets_unlocked(char * __restrict  __s , int __n ,
                            FILE * __restrict  __stream ) ;
extern __ssize_t __getdelim(char ** __restrict  __lineptr ,
                            size_t * __restrict  __n , int __delimiter ,
                            FILE * __restrict  __stream ) ;
extern __ssize_t getdelim(char ** __restrict  __lineptr ,
                          size_t * __restrict  __n , int __delimiter ,
                          FILE * __restrict  __stream ) ;
__inline extern __ssize_t getline(char ** __restrict  __lineptr ,
                                  size_t * __restrict  __n ,
                                  FILE * __restrict  __stream ) ;
extern int fputs(char const   * __restrict  __s , FILE * __restrict  __stream ) ;
extern int puts(char const   *__s ) ;
extern int ungetc(int __c , FILE *__stream ) ;
extern size_t fread(void * __restrict  __ptr , size_t __size , size_t __n ,
                    FILE * __restrict  __stream ) ;
extern size_t fwrite(void const   * __restrict  __ptr , size_t __size ,
                     size_t __n , FILE * __restrict  __s ) ;
extern int fputs_unlocked(char const   * __restrict  __s ,
                          FILE * __restrict  __stream ) ;
extern size_t fread_unlocked(void * __restrict  __ptr , size_t __size ,
                             size_t __n , FILE * __restrict  __stream ) ;
extern size_t fwrite_unlocked(void const   * __restrict  __ptr , size_t __size ,
                              size_t __n , FILE * __restrict  __stream ) ;
extern int fseek(FILE *__stream , long __off , int __whence ) ;
extern long ftell(FILE *__stream ) ;
extern void rewind(FILE *__stream ) ;
extern int fseeko(FILE *__stream , __off_t __off , int __whence ) ;
extern __off_t ftello(FILE *__stream ) ;
extern int fgetpos(FILE * __restrict  __stream , fpos_t * __restrict  __pos ) ;
extern int fsetpos(FILE *__stream , fpos_t const   *__pos ) ;
extern int fseeko64(FILE *__stream , __off64_t __off , int __whence ) ;
extern __off64_t ftello64(FILE *__stream ) ;
extern int fgetpos64(FILE * __restrict  __stream , fpos64_t * __restrict  __pos ) ;
extern int fsetpos64(FILE *__stream , fpos64_t const   *__pos ) ;
extern  __attribute__((__nothrow__)) void clearerr(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) int feof(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) int ferror(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) void clearerr_unlocked(FILE *__stream ) ;
__inline extern  __attribute__((__nothrow__)) int feof_unlocked(FILE *__stream ) ;
__inline extern  __attribute__((__nothrow__)) int ferror_unlocked(FILE *__stream ) ;
extern void perror(char const   *__s ) ;
extern int sys_nerr ;
extern char const   * const  sys_errlist[] ;
extern int _sys_nerr ;
extern char const   * const  _sys_errlist[] ;
extern  __attribute__((__nothrow__)) int fileno(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) int fileno_unlocked(FILE *__stream ) ;
extern FILE *popen(char const   *__command , char const   *__modes ) ;
extern int pclose(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) char *ctermid(char *__s ) ;
extern char *cuserid(char *__s ) ;
extern  __attribute__((__nothrow__)) int ( /* format attribute */  obstack_printf)(struct obstack * __restrict  __obstack ,
                                                                                   char const   * __restrict  __format 
                                                                                   , ...) ;
extern  __attribute__((__nothrow__)) int ( /* format attribute */  obstack_vprintf)(struct obstack * __restrict  __obstack ,
                                                                                    char const   * __restrict  __format ,
                                                                                    __gnuc_va_list __args ) ;
extern  __attribute__((__nothrow__)) void flockfile(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) int ftrylockfile(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) void funlockfile(FILE *__stream ) ;
__inline extern int vprintf(char const   * __restrict  __fmt ,
                            __gnuc_va_list __arg ) 
{ 
  int tmp ;

  {
  tmp = vfprintf((FILE */* __restrict  */)stdout, __fmt, __arg);
  return (tmp);
}
}
__inline extern int getchar(void) 
{ 
  int tmp ;

  {
  tmp = _IO_getc(stdin);
  return (tmp);
}
}
__inline extern int fgetc_unlocked(FILE *__fp ) 
{ 
  long tmp ;
  int tmp___0 ;
  char *tmp___1 ;
  int tmp___2 ;
  long tmp___3 ;

  {
  tmp___3 = __builtin_expect((long )((unsigned int )__fp->_IO_read_ptr >= (unsigned int )__fp->_IO_read_end),
                             0L);
  if (tmp___3) {
    tmp___0 = __uflow(__fp);
    tmp___2 = tmp___0;
  } else {
    tmp___1 = __fp->_IO_read_ptr;
    (__fp->_IO_read_ptr) ++;
    tmp___2 = (int )*((unsigned char *)tmp___1);
  }
  return (tmp___2);
}
}
__inline extern int getc_unlocked(FILE *__fp ) 
{ 
  long tmp ;
  int tmp___0 ;
  char *tmp___1 ;
  int tmp___2 ;
  long tmp___3 ;

  {
  tmp___3 = __builtin_expect((long )((unsigned int )__fp->_IO_read_ptr >= (unsigned int )__fp->_IO_read_end),
                             0L);
  if (tmp___3) {
    tmp___0 = __uflow(__fp);
    tmp___2 = tmp___0;
  } else {
    tmp___1 = __fp->_IO_read_ptr;
    (__fp->_IO_read_ptr) ++;
    tmp___2 = (int )*((unsigned char *)tmp___1);
  }
  return (tmp___2);
}
}
__inline extern int getchar_unlocked(void) 
{ 
  long tmp ;
  int tmp___0 ;
  char *tmp___1 ;
  int tmp___2 ;
  long tmp___3 ;

  {
  tmp___3 = __builtin_expect((long )((unsigned int )stdin->_IO_read_ptr >= (unsigned int )stdin->_IO_read_end),
                             0L);
  if (tmp___3) {
    tmp___0 = __uflow(stdin);
    tmp___2 = tmp___0;
  } else {
    tmp___1 = stdin->_IO_read_ptr;
    (stdin->_IO_read_ptr) ++;
    tmp___2 = (int )*((unsigned char *)tmp___1);
  }
  return (tmp___2);
}
}
__inline extern int putchar(int __c ) 
{ 
  int tmp ;

  {
  tmp = _IO_putc(__c, stdout);
  return (tmp);
}
}
__inline extern int fputc_unlocked(int __c , FILE *__stream ) 
{ 
  long tmp ;
  int tmp___0 ;
  char *tmp___1 ;
  char tmp___2 ;
  int tmp___3 ;
  long tmp___4 ;

  {
  tmp___4 = __builtin_expect((long )((unsigned int )__stream->_IO_write_ptr >= (unsigned int )__stream->_IO_write_end),
                             0L);
  if (tmp___4) {
    tmp___0 = __overflow(__stream, (int )((unsigned char )__c));
    tmp___3 = tmp___0;
  } else {
    tmp___1 = __stream->_IO_write_ptr;
    (__stream->_IO_write_ptr) ++;
    tmp___2 = (char )__c;
    *tmp___1 = tmp___2;
    tmp___3 = (int )((unsigned char )tmp___2);
  }
  return (tmp___3);
}
}
__inline extern int putc_unlocked(int __c , FILE *__stream ) 
{ 
  long tmp ;
  int tmp___0 ;
  char *tmp___1 ;
  char tmp___2 ;
  int tmp___3 ;
  long tmp___4 ;

  {
  tmp___4 = __builtin_expect((long )((unsigned int )__stream->_IO_write_ptr >= (unsigned int )__stream->_IO_write_end),
                             0L);
  if (tmp___4) {
    tmp___0 = __overflow(__stream, (int )((unsigned char )__c));
    tmp___3 = tmp___0;
  } else {
    tmp___1 = __stream->_IO_write_ptr;
    (__stream->_IO_write_ptr) ++;
    tmp___2 = (char )__c;
    *tmp___1 = tmp___2;
    tmp___3 = (int )((unsigned char )tmp___2);
  }
  return (tmp___3);
}
}
__inline extern int putchar_unlocked(int __c ) 
{ 
  long tmp ;
  int tmp___0 ;
  char *tmp___1 ;
  char tmp___2 ;
  int tmp___3 ;
  long tmp___4 ;

  {
  tmp___4 = __builtin_expect((long )((unsigned int )stdout->_IO_write_ptr >= (unsigned int )stdout->_IO_write_end),
                             0L);
  if (tmp___4) {
    tmp___0 = __overflow(stdout, (int )((unsigned char )__c));
    tmp___3 = tmp___0;
  } else {
    tmp___1 = stdout->_IO_write_ptr;
    (stdout->_IO_write_ptr) ++;
    tmp___2 = (char )__c;
    *tmp___1 = tmp___2;
    tmp___3 = (int )((unsigned char )tmp___2);
  }
  return (tmp___3);
}
}
__inline extern __ssize_t getline(char ** __restrict  __lineptr ,
                                  size_t * __restrict  __n ,
                                  FILE * __restrict  __stream ) 
{ 
  __ssize_t tmp ;

  {
  tmp = __getdelim(__lineptr, __n, '\n', __stream);
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int feof_unlocked(FILE *__stream ) ;
__inline extern int feof_unlocked(FILE *__stream ) 
{ 


  {
  return ((__stream->_flags & 16) != 0);
}
}
__inline extern  __attribute__((__nothrow__)) int ferror_unlocked(FILE *__stream ) ;
__inline extern int ferror_unlocked(FILE *__stream ) 
{ 


  {
  return ((__stream->_flags & 32) != 0);
}
}
extern timelib_sll timelib_day_of_week(timelib_sll y , timelib_sll m ,
                                       timelib_sll d ) ;
extern timelib_sll timelib_iso_day_of_week(timelib_sll y , timelib_sll m ,
                                           timelib_sll d ) ;
extern timelib_sll timelib_day_of_year(timelib_sll y , timelib_sll m ,
                                       timelib_sll d ) ;
extern timelib_sll timelib_daynr_from_weeknr(timelib_sll y , timelib_sll w ,
                                             timelib_sll d ) ;
extern timelib_sll timelib_days_in_month(timelib_sll y , timelib_sll m ) ;
extern void timelib_isoweek_from_date(timelib_sll y , timelib_sll m ,
                                      timelib_sll d , timelib_sll *iw ,
                                      timelib_sll *iy ) ;
extern int timelib_valid_time(timelib_sll h , timelib_sll i , timelib_sll s ) ;
extern int timelib_valid_date(timelib_sll y , timelib_sll m , timelib_sll d ) ;
timelib_time *timelib_strtotime(char *s , int len ,
                                struct timelib_error_container **errors ,
                                timelib_tzdb const   *tzdb ) ;
timelib_time *timelib_parse_from_format(char *format , char *string , int len ,
                                        timelib_error_container **errors ,
                                        timelib_tzdb const   *tzdb ) ;
void timelib_fill_holes(timelib_time *parsed , timelib_time *now , int options ) ;
char *timelib_timezone_id_from_abbr(char const   *abbr , long gmtoffset ,
                                    int isdst ) ;
timelib_tz_lookup_table const   *timelib_timezone_abbreviations_list(void) ;
extern void timelib_strtointerval(char *s , int len , timelib_time **begin ,
                                  timelib_time **end ,
                                  timelib_rel_time **period , int *recurrences ,
                                  struct timelib_error_container **errors ) ;
extern void timelib_update_ts(timelib_time *time , timelib_tzinfo *tzi ) ;
extern void timelib_do_rel_normalize(timelib_time *base , timelib_rel_time *rt ) ;
extern int timelib_apply_localtime(timelib_time *t , unsigned int localtime ) ;
extern void timelib_unixtime2gmt(timelib_time *tm , timelib_sll ts ) ;
extern void timelib_unixtime2local(timelib_time *tm , timelib_sll ts ) ;
extern void timelib_update_from_sse(timelib_time *tm ) ;
extern void timelib_set_timezone(timelib_time *t , timelib_tzinfo *tz ) ;
extern int timelib_timezone_id_is_valid(char *timezone ,
                                        timelib_tzdb const   *tzdb ) ;
extern timelib_tzinfo *timelib_parse_tzfile(char *timezone ,
                                            timelib_tzdb const   *tzdb ) ;
extern int timelib_timestamp_is_in_dst(timelib_sll ts , timelib_tzinfo *tz ) ;
extern timelib_time_offset *timelib_get_time_zone_info(timelib_sll ts ,
                                                       timelib_tzinfo *tz ) ;
extern timelib_sll timelib_get_current_offset(timelib_time *t ) ;
extern void timelib_dump_tzinfo(timelib_tzinfo *tz ) ;
extern timelib_tzdb const   *timelib_builtin_db(void) ;
extern timelib_tzdb_index_entry const   *timelib_timezone_builtin_identifiers_list(int *count ) ;
extern timelib_tzinfo *timelib_tzinfo_ctor(char *name ) ;
extern void timelib_time_tz_abbr_update(timelib_time *tm , char *tz_abbr ) ;
extern void timelib_time_tz_name_update(timelib_time *tm , char *tz_name ) ;
extern void timelib_tzinfo_dtor(timelib_tzinfo *tz ) ;
extern timelib_tzinfo *timelib_tzinfo_clone(timelib_tzinfo *tz ) ;
extern timelib_rel_time *timelib_rel_time_ctor(void) ;
extern void timelib_rel_time_dtor(timelib_rel_time *t ) ;
extern timelib_rel_time *timelib_rel_time_clone(timelib_rel_time *tz ) ;
extern timelib_time *timelib_time_ctor(void) ;
extern void timelib_time_set_option(timelib_time *tm , int option ,
                                    void *option_value ) ;
extern void timelib_time_dtor(timelib_time *t ) ;
extern timelib_time *timelib_time_clone(timelib_time *orig ) ;
extern timelib_time_offset *timelib_time_offset_ctor(void) ;
extern void timelib_time_offset_dtor(timelib_time_offset *t ) ;
extern void timelib_error_container_dtor(timelib_error_container *errors ) ;
extern long timelib_date_to_int(timelib_time *d , int *error ) ;
extern void timelib_dump_date(timelib_time *d , int options ) ;
extern void timelib_dump_rel_time(timelib_rel_time *d ) ;
extern void timelib_decimal_hour_to_hms(double h , int *hour , int *min ,
                                        int *sec ) ;
extern double timelib_ts_to_juliandate(timelib_sll ts ) ;
extern int timelib_astro_rise_set_altitude(timelib_time *time , double lon ,
                                           double lat , double altit ,
                                           int upper_limb , double *h_rise ,
                                           double *h_set ,
                                           timelib_sll *ts_rise ,
                                           timelib_sll *ts_set ,
                                           timelib_sll *ts_transit ) ;
extern timelib_rel_time *timelib_diff(timelib_time *one , timelib_time *two ) ;
extern  __attribute__((__nothrow__)) unsigned short const   **__ctype_b_loc(void)  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) __int32_t const   **__ctype_tolower_loc(void)  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) __int32_t const   **__ctype_toupper_loc(void)  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int isalnum(int  ) ;
extern  __attribute__((__nothrow__)) int isalpha(int  ) ;
extern  __attribute__((__nothrow__)) int iscntrl(int  ) ;
extern  __attribute__((__nothrow__)) int isdigit(int  ) ;
extern  __attribute__((__nothrow__)) int islower(int  ) ;
extern  __attribute__((__nothrow__)) int isgraph(int  ) ;
extern  __attribute__((__nothrow__)) int isprint(int  ) ;
extern  __attribute__((__nothrow__)) int ispunct(int  ) ;
extern  __attribute__((__nothrow__)) int isspace(int  ) ;
extern  __attribute__((__nothrow__)) int isupper(int  ) ;
extern  __attribute__((__nothrow__)) int isxdigit(int  ) ;
__inline extern  __attribute__((__nothrow__)) int tolower(int __c ) ;
__inline extern  __attribute__((__nothrow__)) int toupper(int __c ) ;
extern  __attribute__((__nothrow__)) int isblank(int  ) ;
extern  __attribute__((__nothrow__)) int isctype(int __c , int __mask ) ;
extern  __attribute__((__nothrow__)) int isascii(int __c ) ;
extern  __attribute__((__nothrow__)) int toascii(int __c ) ;
extern  __attribute__((__nothrow__)) int _toupper(int  ) ;
extern  __attribute__((__nothrow__)) int _tolower(int  ) ;
__inline extern  __attribute__((__nothrow__)) int tolower(int __c ) ;
__inline extern int tolower(int __c ) 
{ 
  __int32_t const   **tmp ;
  __int32_t tmp___0 ;

  {
  if (__c >= -128) {
    if (__c < 256) {
      tmp = __ctype_tolower_loc();
      tmp___0 = *(*tmp + __c);
    } else {
      tmp___0 = (__int32_t const   )__c;
    }
  } else {
    tmp___0 = (__int32_t const   )__c;
  }
  return ((int )tmp___0);
}
}
__inline extern  __attribute__((__nothrow__)) int toupper(int __c ) ;
__inline extern int toupper(int __c ) 
{ 
  __int32_t const   **tmp ;
  __int32_t tmp___0 ;

  {
  if (__c >= -128) {
    if (__c < 256) {
      tmp = __ctype_toupper_loc();
      tmp___0 = *(*tmp + __c);
    } else {
      tmp___0 = (__int32_t const   )__c;
    }
  } else {
    tmp___0 = (__int32_t const   )__c;
  }
  return ((int )tmp___0);
}
}
extern  __attribute__((__nothrow__)) int isalnum_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isalpha_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int iscntrl_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isdigit_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int islower_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isgraph_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isprint_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int ispunct_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isspace_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isupper_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isxdigit_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isblank_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int __tolower_l(int __c , __locale_t __l ) ;
extern  __attribute__((__nothrow__)) int tolower_l(int __c , __locale_t __l ) ;
extern  __attribute__((__nothrow__)) int __toupper_l(int __c , __locale_t __l ) ;
extern  __attribute__((__nothrow__)) int toupper_l(int __c , __locale_t __l ) ;
extern  __attribute__((__nothrow__,
__noreturn__)) void __assert_fail(char const   *__assertion ,
                                  char const   *__file , unsigned int __line ,
                                  char const   *__function ) ;
extern  __attribute__((__nothrow__,
__noreturn__)) void __assert_perror_fail(int __errnum , char const   *__file ,
                                         unsigned int __line ,
                                         char const   *__function ) ;
extern  __attribute__((__nothrow__,
__noreturn__)) void __assert(char const   *__assertion , char const   *__file ,
                             int __line ) ;
static timelib_tz_lookup_table const   timelib_timezone_lookup[1981]  = 
  {      {(char *)"acst", 1, (float )-14400, (char *)"America/Porto_Acre"}, 
        {(char *)"acst", 1, (float )-14400, (char *)"America/Eirunepe"}, 
        {(char *)"acst", 1, (float )-14400, (char *)"America/Rio_Branco"}, 
        {(char *)"acst", 1, (float )-14400, (char *)"Brazil/Acre"}, 
        {(char *)"act", 0, (float )-18000, (char *)"America/Porto_Acre"}, 
        {(char *)"act", 0, (float )-18000, (char *)"America/Eirunepe"}, 
        {(char *)"act", 0, (float )-18000, (char *)"America/Rio_Branco"}, 
        {(char *)"act", 0, (float )-18000, (char *)"Brazil/Acre"}, 
        {(char *)"addt", 1, (float )-7200, (char *)"America/Goose_Bay"}, 
        {(char *)"addt", 1, (float )-7200, (char *)"America/Pangnirtung"}, 
        {(char *)"admt", 0, (float )9320, (char *)"Africa/Addis_Ababa"}, 
        {(char *)"admt", 0, (float )9320, (char *)"Africa/Asmara"}, 
        {(char *)"admt", 0, (float )9320, (char *)"Africa/Asmera"}, 
        {(char *)"adt", 1, (float )-10800, (char *)"America/Halifax"}, 
        {(char *)"adt", 1, (float )-10800, (char *)"America/Barbados"}, 
        {(char *)"adt", 1, (float )-10800, (char *)"America/Blanc-Sablon"}, 
        {(char *)"adt", 1, (float )-10800, (char *)"America/Glace_Bay"}, 
        {(char *)"adt", 1, (float )-10800, (char *)"America/Goose_Bay"}, 
        {(char *)"adt", 1, (float )-10800, (char *)"America/Martinique"}, 
        {(char *)"adt", 1, (float )-10800, (char *)"America/Moncton"}, 
        {(char *)"adt", 1, (float )-10800, (char *)"America/Pangnirtung"}, 
        {(char *)"adt", 1, (float )-10800, (char *)"America/Thule"}, 
        {(char *)"adt", 1, (float )-10800, (char *)"Atlantic/Bermuda"}, 
        {(char *)"adt", 1, (float )-10800, (char *)"Canada/Atlantic"}, 
        {(char *)"adt", 1, (float )14400, (char *)"Asia/Baghdad"}, 
        {(char *)"aft", 0, (float )14400, (char *)"Asia/Kabul"}, 
        {(char *)"aft", 0, (float )16200, (char *)"Asia/Kabul"}, 
        {(char *)"ahdt", 1, (float )-32400, (char *)"America/Anchorage"}, 
        {(char *)"ahst", 0, (float )-36000, (char *)"America/Anchorage"}, 
        {(char *)"ahst", 0, (float )-36000, (char *)"America/Adak"}, 
        {(char *)"ahst", 0, (float )-36000, (char *)"America/Atka"}, 
        {(char *)"akdt", 1, (float )-28800, (char *)"America/Anchorage"}, 
        {(char *)"akdt", 1, (float )-28800, (char *)"America/Juneau"}, 
        {(char *)"akdt", 1, (float )-28800, (char *)"America/Nome"}, 
        {(char *)"akdt", 1, (float )-28800, (char *)"America/Yakutat"}, 
        {(char *)"akst", 0, (float )-32400, (char *)"America/Anchorage"}, 
        {(char *)"akst", 0, (float )-32400, (char *)"America/Juneau"}, 
        {(char *)"akst", 0, (float )-32400, (char *)"America/Nome"}, 
        {(char *)"akst", 0, (float )-32400, (char *)"America/Yakutat"}, 
        {(char *)"aktst", 1, (float )21600, (char *)"Asia/Aqtobe"}, 
        {(char *)"aktt", 0, (float )14400, (char *)"Asia/Aqtobe"}, 
        {(char *)"aktt", 0, (float )18000, (char *)"Asia/Aqtobe"}, 
        {(char *)"aktt", 0, (float )21600, (char *)"Asia/Aqtobe"}, 
        {(char *)"almst", 1, (float )25200, (char *)"Asia/Almaty"}, 
        {(char *)"almt", 0, (float )18000, (char *)"Asia/Almaty"}, 
        {(char *)"almt", 0, (float )21600, (char *)"Asia/Almaty"}, 
        {(char *)"amst", 1, (float )14400, (char *)"Asia/Yerevan"}, 
        {(char *)"amst", 1, (float )18000, (char *)"Asia/Yerevan"}, 
        {(char *)"amst", 1, (float )-10800, (char *)"America/Boa_Vista"}, 
        {(char *)"amst", 1, (float )-10800, (char *)"America/Campo_Grande"}, 
        {(char *)"amst", 1, (float )-10800, (char *)"America/Cuiaba"}, 
        {(char *)"amst", 1, (float )-10800, (char *)"America/Manaus"}, 
        {(char *)"amst", 1, (float )-10800, (char *)"America/Porto_Velho"}, 
        {(char *)"amst", 1, (float )-10800, (char *)"America/Santarem"}, 
        {(char *)"amst", 1, (float )-10800, (char *)"Brazil/West"}, 
        {(char *)"amt", 0, (float )10800, (char *)"Asia/Yerevan"}, 
        {(char *)"amt", 0, (float )14400, (char *)"Asia/Yerevan"}, 
        {(char *)"amt", 0, (float )-13840, (char *)"America/Asuncion"}, 
        {(char *)"amt", 0, (float )-14400, (char *)"America/Boa_Vista"}, 
        {(char *)"amt", 0, (float )-14400, (char *)"America/Campo_Grande"}, 
        {(char *)"amt", 0, (float )-14400, (char *)"America/Cuiaba"}, 
        {(char *)"amt", 0, (float )-14400, (char *)"America/Eirunepe"}, 
        {(char *)"amt", 0, (float )-14400, (char *)"America/Manaus"}, 
        {(char *)"amt", 0, (float )-14400, (char *)"America/Porto_Acre"}, 
        {(char *)"amt", 0, (float )-14400, (char *)"America/Porto_Velho"}, 
        {(char *)"amt", 0, (float )-14400, (char *)"America/Rio_Branco"}, 
        {(char *)"amt", 0, (float )-14400, (char *)"America/Santarem"}, 
        {(char *)"amt", 0, (float )-14400, (char *)"Brazil/Acre"}, 
        {(char *)"amt", 0, (float )-14400, (char *)"Brazil/West"}, 
        {(char *)"amt", 0, (float )1172, (char *)"Europe/Amsterdam"}, 
        {(char *)"amt", 0, (float )5692, (char *)"Europe/Athens"}, 
        {(char *)"anast", 1, (float )43200, (char *)"Asia/Anadyr"}, 
        {(char *)"anast", 1, (float )46800, (char *)"Asia/Anadyr"}, 
        {(char *)"anast", 1, (float )50400, (char *)"Asia/Anadyr"}, 
        {(char *)"anat", 0, (float )39600, (char *)"Asia/Anadyr"}, 
        {(char *)"anat", 0, (float )43200, (char *)"Asia/Anadyr"}, 
        {(char *)"anat", 0, (float )46800, (char *)"Asia/Anadyr"}, 
        {(char *)"ant", 0, (float )-16200, (char *)"America/Curacao"}, 
        {(char *)"ant", 0, (float )-16200, (char *)"America/Aruba"}, 
        {(char *)"aot", 0, (float )3124, (char *)"Africa/Luanda"}, 
        {(char *)"apt", 1, (float )-10800, (char *)"America/Halifax"}, 
        {(char *)"apt", 1, (float )-10800, (char *)"America/Blanc-Sablon"}, 
        {(char *)"apt", 1, (float )-10800, (char *)"America/Glace_Bay"}, 
        {(char *)"apt", 1, (float )-10800, (char *)"America/Moncton"}, 
        {(char *)"apt", 1, (float )-10800, (char *)"America/Pangnirtung"}, 
        {(char *)"apt", 1, (float )-10800, (char *)"America/Puerto_Rico"}, 
        {(char *)"apt", 1, (float )-10800, (char *)"Canada/Atlantic"}, 
        {(char *)"aqtst", 1, (float )18000, (char *)"Asia/Aqtau"}, 
        {(char *)"aqtst", 1, (float )21600, (char *)"Asia/Aqtau"}, 
        {(char *)"aqtst", 1, (float )21600, (char *)"Asia/Aqtobe"}, 
        {(char *)"aqtt", 0, (float )14400, (char *)"Asia/Aqtau"}, 
        {(char *)"aqtt", 0, (float )18000, (char *)"Asia/Aqtau"}, 
        {(char *)"aqtt", 0, (float )18000, (char *)"Asia/Aqtobe"}, 
        {(char *)"arst", 1, (float )-10800, (char *)"America/Buenos_Aires"}, 
        {(char *)"arst", 1, (float )-7200, (char *)"America/Buenos_Aires"}, 
        {(char *)"arst", 1, (float )-10800,
      (char *)"America/Argentina/Buenos_Aires"}, 
        {(char *)"arst", 1, (float )-10800, (char *)"America/Argentina/Catamarca"}, 
        {(char *)"arst", 1, (float )-10800,
      (char *)"America/Argentina/ComodRivadavia"}, 
        {(char *)"arst", 1, (float )-10800, (char *)"America/Argentina/Cordoba"}, 
        {(char *)"arst", 1, (float )-10800, (char *)"America/Argentina/Jujuy"}, 
        {(char *)"arst", 1, (float )-10800, (char *)"America/Argentina/La_Rioja"}, 
        {(char *)"arst", 1, (float )-10800, (char *)"America/Argentina/Mendoza"}, 
        {(char *)"arst", 1, (float )-10800,
      (char *)"America/Argentina/Rio_Gallegos"}, 
        {(char *)"arst", 1, (float )-10800, (char *)"America/Argentina/Salta"}, 
        {(char *)"arst", 1, (float )-10800, (char *)"America/Argentina/San_Juan"}, 
        {(char *)"arst", 1, (float )-10800, (char *)"America/Argentina/San_Luis"}, 
        {(char *)"arst", 1, (float )-10800, (char *)"America/Argentina/Tucuman"}, 
        {(char *)"arst", 1, (float )-10800, (char *)"America/Argentina/Ushuaia"}, 
        {(char *)"arst", 1, (float )-10800, (char *)"America/Catamarca"}, 
        {(char *)"arst", 1, (float )-10800, (char *)"America/Cordoba"}, 
        {(char *)"arst", 1, (float )-10800, (char *)"America/Jujuy"}, 
        {(char *)"arst", 1, (float )-10800, (char *)"America/Mendoza"}, 
        {(char *)"arst", 1, (float )-10800, (char *)"America/Rosario"}, 
        {(char *)"arst", 1, (float )-10800, (char *)"Antarctica/Palmer"}, 
        {(char *)"arst", 1, (float )-7200, (char *)"America/Argentina/Buenos_Aires"}, 
        {(char *)"arst",
      1, (float )-7200, (char *)"America/Argentina/Catamarca"}, 
        {(char *)"arst", 1, (float )-7200,
      (char *)"America/Argentina/ComodRivadavia"}, 
        {(char *)"arst", 1, (float )-7200, (char *)"America/Argentina/Cordoba"}, 
        {(char *)"arst", 1, (float )-7200, (char *)"America/Argentina/Jujuy"}, 
        {(char *)"arst", 1, (float )-7200, (char *)"America/Argentina/La_Rioja"}, 
        {(char *)"arst", 1, (float )-7200, (char *)"America/Argentina/Mendoza"}, 
        {(char *)"arst", 1, (float )-7200, (char *)"America/Argentina/Rio_Gallegos"}, 
        {(char *)"arst",
      1, (float )-7200, (char *)"America/Argentina/Salta"}, 
        {(char *)"arst", 1, (float )-7200, (char *)"America/Argentina/San_Juan"}, 
        {(char *)"arst", 1, (float )-7200, (char *)"America/Argentina/San_Luis"}, 
        {(char *)"arst", 1, (float )-7200, (char *)"America/Argentina/Tucuman"}, 
        {(char *)"arst", 1, (float )-7200, (char *)"America/Argentina/Ushuaia"}, 
        {(char *)"arst", 1, (float )-7200, (char *)"America/Catamarca"}, 
        {(char *)"arst", 1, (float )-7200, (char *)"America/Cordoba"}, 
        {(char *)"arst", 1, (float )-7200, (char *)"America/Jujuy"}, 
        {(char *)"arst", 1, (float )-7200, (char *)"America/Mendoza"}, 
        {(char *)"arst", 1, (float )-7200, (char *)"America/Rosario"}, 
        {(char *)"arst", 1, (float )-7200, (char *)"Antarctica/Palmer"}, 
        {(char *)"art", 0, (float )-10800, (char *)"America/Buenos_Aires"}, 
        {(char *)"art", 0, (float )-14400, (char *)"America/Buenos_Aires"}, 
        {(char *)"art", 0, (float )-10800, (char *)"America/Argentina/Buenos_Aires"}, 
        {(char *)"art",
      0, (float )-10800, (char *)"America/Argentina/Catamarca"}, 
        {(char *)"art", 0, (float )-10800,
      (char *)"America/Argentina/ComodRivadavia"}, 
        {(char *)"art", 0, (float )-10800, (char *)"America/Argentina/Cordoba"}, 
        {(char *)"art", 0, (float )-10800, (char *)"America/Argentina/Jujuy"}, 
        {(char *)"art", 0, (float )-10800, (char *)"America/Argentina/La_Rioja"}, 
        {(char *)"art", 0, (float )-10800, (char *)"America/Argentina/Mendoza"}, 
        {(char *)"art", 0, (float )-10800, (char *)"America/Argentina/Rio_Gallegos"}, 
        {(char *)"art",
      0, (float )-10800, (char *)"America/Argentina/Salta"}, 
        {(char *)"art", 0, (float )-10800, (char *)"America/Argentina/San_Juan"}, 
        {(char *)"art", 0, (float )-10800, (char *)"America/Argentina/San_Luis"}, 
        {(char *)"art", 0, (float )-10800, (char *)"America/Argentina/Tucuman"}, 
        {(char *)"art", 0, (float )-10800, (char *)"America/Argentina/Ushuaia"}, 
        {(char *)"art", 0, (float )-10800, (char *)"America/Catamarca"}, 
        {(char *)"art", 0, (float )-10800, (char *)"America/Cordoba"}, 
        {(char *)"art", 0, (float )-10800, (char *)"America/Jujuy"}, 
        {(char *)"art", 0, (float )-10800, (char *)"America/Mendoza"}, 
        {(char *)"art", 0, (float )-10800, (char *)"America/Rosario"}, 
        {(char *)"art", 0, (float )-10800, (char *)"Antarctica/Palmer"}, 
        {(char *)"art", 0, (float )-14400, (char *)"America/Argentina/Buenos_Aires"}, 
        {(char *)"art",
      0, (float )-14400, (char *)"America/Argentina/Catamarca"}, 
        {(char *)"art", 0, (float )-14400,
      (char *)"America/Argentina/ComodRivadavia"}, 
        {(char *)"art", 0, (float )-14400, (char *)"America/Argentina/Cordoba"}, 
        {(char *)"art", 0, (float )-14400, (char *)"America/Argentina/Jujuy"}, 
        {(char *)"art", 0, (float )-14400, (char *)"America/Argentina/La_Rioja"}, 
        {(char *)"art", 0, (float )-14400, (char *)"America/Argentina/Mendoza"}, 
        {(char *)"art", 0, (float )-14400, (char *)"America/Argentina/Rio_Gallegos"}, 
        {(char *)"art",
      0, (float )-14400, (char *)"America/Argentina/Salta"}, 
        {(char *)"art", 0, (float )-14400, (char *)"America/Argentina/San_Juan"}, 
        {(char *)"art", 0, (float )-14400, (char *)"America/Argentina/San_Luis"}, 
        {(char *)"art", 0, (float )-14400, (char *)"America/Argentina/Tucuman"}, 
        {(char *)"art", 0, (float )-14400, (char *)"America/Argentina/Ushuaia"}, 
        {(char *)"art", 0, (float )-14400, (char *)"America/Catamarca"}, 
        {(char *)"art", 0, (float )-14400, (char *)"America/Cordoba"}, 
        {(char *)"art", 0, (float )-14400, (char *)"America/Jujuy"}, 
        {(char *)"art", 0, (float )-14400, (char *)"America/Mendoza"}, 
        {(char *)"art", 0, (float )-14400, (char *)"America/Rosario"}, 
        {(char *)"art", 0, (float )-14400, (char *)"Antarctica/Palmer"}, 
        {(char *)"ashst", 1, (float )18000, (char *)"Asia/Ashkhabad"}, 
        {(char *)"ashst", 1, (float )21600, (char *)"Asia/Ashkhabad"}, 
        {(char *)"ashst", 1, (float )18000, (char *)"Asia/Ashgabat"}, 
        {(char *)"ashst", 1, (float )21600, (char *)"Asia/Ashgabat"}, 
        {(char *)"asht", 0, (float )14400, (char *)"Asia/Ashkhabad"}, 
        {(char *)"asht", 0, (float )18000, (char *)"Asia/Ashkhabad"}, 
        {(char *)"asht", 0, (float )14400, (char *)"Asia/Ashgabat"}, 
        {(char *)"asht", 0, (float )18000, (char *)"Asia/Ashgabat"}, 
        {(char *)"ast", 0, (float )10800, (char *)"Asia/Riyadh"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/Anguilla"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/Antigua"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/Aruba"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/Barbados"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/Blanc-Sablon"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/Curacao"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/Dominica"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/Glace_Bay"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/Goose_Bay"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/Grenada"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/Guadeloupe"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/Halifax"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/Marigot"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/Martinique"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/Miquelon"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/Moncton"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/Montserrat"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/Pangnirtung"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/Port_of_Spain"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/Puerto_Rico"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/Santo_Domingo"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/St_Barthelemy"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/St_Kitts"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/St_Lucia"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/St_Thomas"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/St_Vincent"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/Thule"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/Tortola"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"America/Virgin"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"Atlantic/Bermuda"}, 
        {(char *)"ast", 0, (float )-14400, (char *)"Canada/Atlantic"}, 
        {(char *)"ast", 0, (float )10800, (char *)"Asia/Aden"}, 
        {(char *)"ast", 0, (float )10800, (char *)"Asia/Baghdad"}, 
        {(char *)"ast", 0, (float )10800, (char *)"Asia/Bahrain"}, 
        {(char *)"ast", 0, (float )10800, (char *)"Asia/Kuwait"}, 
        {(char *)"ast", 0, (float )10800, (char *)"Asia/Qatar"}, 
        {(char *)"awt", 1, (float )-10800, (char *)"America/Halifax"}, 
        {(char *)"awt", 1, (float )-10800, (char *)"America/Blanc-Sablon"}, 
        {(char *)"awt", 1, (float )-10800, (char *)"America/Glace_Bay"}, 
        {(char *)"awt", 1, (float )-10800, (char *)"America/Moncton"}, 
        {(char *)"awt", 1, (float )-10800, (char *)"America/Pangnirtung"}, 
        {(char *)"awt", 1, (float )-10800, (char *)"America/Puerto_Rico"}, 
        {(char *)"awt", 1, (float )-10800, (char *)"Canada/Atlantic"}, 
        {(char *)"azomt", 1, (float )0, (char *)"Atlantic/Azores"}, 
        {(char *)"azost", 1, (float )-3600, (char *)"Atlantic/Azores"}, 
        {(char *)"azost", 1, (float )0, (char *)"Atlantic/Azores"}, 
        {(char *)"azot", 0, (float )-3600, (char *)"Atlantic/Azores"}, 
        {(char *)"azot", 0, (float )-7200, (char *)"Atlantic/Azores"}, 
        {(char *)"azst", 1, (float )14400, (char *)"Asia/Baku"}, 
        {(char *)"azst", 1, (float )18000, (char *)"Asia/Baku"}, 
        {(char *)"azt", 0, (float )10800, (char *)"Asia/Baku"}, 
        {(char *)"azt", 0, (float )14400, (char *)"Asia/Baku"}, 
        {(char *)"bakst", 1, (float )14400, (char *)"Asia/Baku"}, 
        {(char *)"bakst", 1, (float )18000, (char *)"Asia/Baku"}, 
        {(char *)"bakt", 0, (float )10800, (char *)"Asia/Baku"}, 
        {(char *)"bakt", 0, (float )14400, (char *)"Asia/Baku"}, 
        {(char *)"bdst", 1, (float )7200, (char *)"Europe/London"}, 
        {(char *)"bdst", 1, (float )25200, (char *)"Asia/Dacca"}, 
        {(char *)"bdst", 1, (float )25200, (char *)"Asia/Dhaka"}, 
        {(char *)"bdst", 1, (float )7200, (char *)"Europe/Belfast"}, 
        {(char *)"bdst", 1, (float )7200, (char *)"Europe/Gibraltar"}, 
        {(char *)"bdst", 1, (float )7200, (char *)"Europe/Guernsey"}, 
        {(char *)"bdst", 1, (float )7200, (char *)"Europe/Isle_of_Man"}, 
        {(char *)"bdst", 1, (float )7200, (char *)"Europe/Jersey"}, 
        {(char *)"bdst", 1, (float )7200, (char *)"GB"}, 
        {(char *)"bdt", 1, (float )-36000, (char *)"America/Adak"}, 
        {(char *)"bdt", 1, (float )-36000, (char *)"America/Atka"}, 
        {(char *)"bdt", 1, (float )-36000, (char *)"America/Nome"}, 
        {(char *)"bdt", 0, (float )21600, (char *)"Asia/Dacca"}, 
        {(char *)"bdt", 0, (float )21600, (char *)"Asia/Dhaka"}, 
        {(char *)"beat", 0, (float )9000, (char *)"Africa/Mogadishu"}, 
        {(char *)"beat", 0, (float )9000, (char *)"Africa/Kampala"}, 
        {(char *)"beat", 0, (float )9000, (char *)"Africa/Nairobi"}, 
        {(char *)"beaut", 0, (float )9885, (char *)"Africa/Nairobi"}, 
        {(char *)"beaut", 0, (float )9885, (char *)"Africa/Dar_es_Salaam"}, 
        {(char *)"beaut", 0, (float )9885, (char *)"Africa/Kampala"}, 
        {(char *)"bmt", 0, (float )-14308, (char *)"America/Barbados"}, 
        {(char *)"bmt", 0, (float )-17780, (char *)"America/Bogota"}, 
        {(char *)"bmt", 0, (float )-3996, (char *)"Africa/Banjul"}, 
        {(char *)"bmt", 0, (float )10656, (char *)"Asia/Baghdad"}, 
        {(char *)"bmt", 0, (float )24124, (char *)"Asia/Bangkok"}, 
        {(char *)"bmt", 0, (float )6264, (char *)"Europe/Tiraspol"}, 
        {(char *)"bmt", 0, (float )6264, (char *)"Europe/Bucharest"}, 
        {(char *)"bmt", 0, (float )6264, (char *)"Europe/Chisinau"}, 
        {(char *)"bnt", 0, (float )27000, (char *)"Asia/Brunei"}, 
        {(char *)"bnt", 0, (float )28800, (char *)"Asia/Brunei"}, 
        {(char *)"bortst", 1, (float )30000, (char *)"Asia/Kuching"}, 
        {(char *)"bort", 0, (float )27000, (char *)"Asia/Kuching"}, 
        {(char *)"bort", 0, (float )28800, (char *)"Asia/Kuching"}, 
        {(char *)"bost", 1, (float )-12756, (char *)"America/La_Paz"}, 
        {(char *)"bot", 0, (float )-14400, (char *)"America/La_Paz"}, 
        {(char *)"brst", 1, (float )-7200, (char *)"America/Sao_Paulo"}, 
        {(char *)"brst", 1, (float )-7200, (char *)"America/Araguaina"}, 
        {(char *)"brst", 1, (float )-7200, (char *)"America/Bahia"}, 
        {(char *)"brst", 1, (float )-7200, (char *)"America/Belem"}, 
        {(char *)"brst", 1, (float )-7200, (char *)"America/Fortaleza"}, 
        {(char *)"brst", 1, (float )-7200, (char *)"America/Maceio"}, 
        {(char *)"brst", 1, (float )-7200, (char *)"America/Recife"}, 
        {(char *)"brst", 1, (float )-7200, (char *)"Brazil/East"}, 
        {(char *)"brt", 0, (float )-10800, (char *)"America/Sao_Paulo"}, 
        {(char *)"brt", 0, (float )-10800, (char *)"America/Araguaina"}, 
        {(char *)"brt", 0, (float )-10800, (char *)"America/Bahia"}, 
        {(char *)"brt", 0, (float )-10800, (char *)"America/Belem"}, 
        {(char *)"brt", 0, (float )-10800, (char *)"America/Fortaleza"}, 
        {(char *)"brt", 0, (float )-10800, (char *)"America/Maceio"}, 
        {(char *)"brt", 0, (float )-10800, (char *)"America/Recife"}, 
        {(char *)"brt", 0, (float )-10800, (char *)"America/Santarem"}, 
        {(char *)"brt", 0, (float )-10800, (char *)"Brazil/East"}, 
        {(char *)"bst", 0, (float )3600, (char *)"Europe/London"}, 
        {(char *)"bst", 1, (float )3600, (char *)"Europe/London"}, 
        {(char *)"bst", 0, (float )-39600, (char *)"America/Adak"}, 
        {(char *)"bst", 0, (float )-39600, (char *)"America/Atka"}, 
        {(char *)"bst", 0, (float )-39600, (char *)"America/Nome"}, 
        {(char *)"bst", 0, (float )-39600, (char *)"Pacific/Midway"}, 
        {(char *)"bst", 0, (float )-39600, (char *)"Pacific/Pago_Pago"}, 
        {(char *)"bst", 0, (float )-39600, (char *)"Pacific/Samoa"}, 
        {(char *)"bst", 0, (float )3600, (char *)"Europe/Belfast"}, 
        {(char *)"bst", 0, (float )3600, (char *)"Europe/Guernsey"}, 
        {(char *)"bst", 0, (float )3600, (char *)"Europe/Isle_of_Man"}, 
        {(char *)"bst", 0, (float )3600, (char *)"Europe/Jersey"}, 
        {(char *)"bst", 0, (float )3600, (char *)"GB"}, 
        {(char *)"bst", 1, (float )3600, (char *)"Europe/Belfast"}, 
        {(char *)"bst", 1, (float )3600, (char *)"Europe/Dublin"}, 
        {(char *)"bst", 1, (float )3600, (char *)"Europe/Gibraltar"}, 
        {(char *)"bst", 1, (float )3600, (char *)"Europe/Guernsey"}, 
        {(char *)"bst", 1, (float )3600, (char *)"Europe/Isle_of_Man"}, 
        {(char *)"bst", 1, (float )3600, (char *)"Europe/Jersey"}, 
        {(char *)"bst", 1, (float )3600, (char *)"GB"}, 
        {(char *)"btt", 0, (float )21600, (char *)"Asia/Thimbu"}, 
        {(char *)"btt", 0, (float )21600, (char *)"Asia/Thimphu"}, 
        {(char *)"burt", 0, (float )23400, (char *)"Asia/Kolkata"}, 
        {(char *)"burt", 0, (float )23400, (char *)"Asia/Calcutta"}, 
        {(char *)"burt", 0, (float )23400, (char *)"Asia/Dacca"}, 
        {(char *)"burt", 0, (float )23400, (char *)"Asia/Dhaka"}, 
        {(char *)"burt", 0, (float )23400, (char *)"Asia/Rangoon"}, 
        {(char *)"cant", 0, (float )-3600, (char *)"Atlantic/Canary"}, 
        {(char *)"capt", 1, (float )-32400, (char *)"America/Anchorage"}, 
        {(char *)"cast", 0, (float )34200, (char *)"Australia/Adelaide"}, 
        {(char *)"cast", 1, (float )10800, (char *)"Africa/Gaborone"}, 
        {(char *)"cast", 1, (float )10800, (char *)"Africa/Khartoum"}, 
        {(char *)"cast", 0, (float )39600, (char *)"Antarctica/Casey"}, 
        {(char *)"cat", 0, (float )-36000, (char *)"America/Anchorage"}, 
        {(char *)"cat", 0, (float )7200, (char *)"Africa/Khartoum"}, 
        {(char *)"cat", 0, (float )7200, (char *)"Africa/Blantyre"}, 
        {(char *)"cat", 0, (float )7200, (char *)"Africa/Bujumbura"}, 
        {(char *)"cat", 0, (float )7200, (char *)"Africa/Gaborone"}, 
        {(char *)"cat", 0, (float )7200, (char *)"Africa/Harare"}, 
        {(char *)"cat", 0, (float )7200, (char *)"Africa/Kigali"}, 
        {(char *)"cat", 0, (float )7200, (char *)"Africa/Lubumbashi"}, 
        {(char *)"cat", 0, (float )7200, (char *)"Africa/Lusaka"}, 
        {(char *)"cat", 0, (float )7200, (char *)"Africa/Maputo"}, 
        {(char *)"cat", 0, (float )7200, (char *)"Africa/Windhoek"}, 
        {(char *)"cawt", 1, (float )-32400, (char *)"America/Anchorage"}, 
        {(char *)"cct", 0, (float )23400, (char *)"Indian/Cocos"}, 
        {(char *)"cddt", 1, (float )-14400, (char *)"America/Rankin_Inlet"}, 
        {(char *)"cddt", 1, (float )-14400, (char *)"America/Resolute"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Chicago"}, 
        {(char *)"cdt", 1, (float )-14400, (char *)"America/Havana"}, 
        {(char *)"cdt", 0, (float )-18000, (char *)"America/Resolute"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Atikokan"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Bahia_Banderas"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Belize"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Cambridge_Bay"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Cancun"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Chihuahua"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Coral_Harbour"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Costa_Rica"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/El_Salvador"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Fort_Wayne"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Guatemala"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Indiana/Indianapolis"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Indiana/Knox"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Indiana/Marengo"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Indiana/Petersburg"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Indiana/Tell_City"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Indiana/Vevay"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Indiana/Vincennes"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Indiana/Winamac"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Indianapolis"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Iqaluit"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Kentucky/Louisville"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Kentucky/Monticello"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Knox_IN"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Louisville"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Managua"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Matamoros"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Menominee"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Merida"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Mexico_City"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Monterrey"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/North_Dakota/Center"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/North_Dakota/New_Salem"}, 
        {(char *)"cdt",
      1, (float )-18000, (char *)"America/Ojinaga"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Pangnirtung"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Rainy_River"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Rankin_Inlet"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Resolute"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Tegucigalpa"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"America/Winnipeg"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"Canada/Central"}, 
        {(char *)"cdt", 1, (float )-18000, (char *)"Mexico/General"}, 
        {(char *)"cdt", 1, (float )32400, (char *)"Asia/Shanghai"}, 
        {(char *)"cdt", 1, (float )32400, (char *)"Asia/Chongqing"}, 
        {(char *)"cdt", 1, (float )32400, (char *)"Asia/Chungking"}, 
        {(char *)"cdt", 1, (float )32400, (char *)"Asia/Harbin"}, 
        {(char *)"cdt", 1, (float )32400, (char *)"Asia/Kashgar"}, 
        {(char *)"cdt", 1, (float )32400, (char *)"Asia/Taipei"}, 
        {(char *)"cdt", 1, (float )32400, (char *)"Asia/Urumqi"}, 
        {(char *)"cdt", 1, (float )32400, (char *)"PRC"}, 
        {(char *)"cdt", 1, (float )32400, (char *)"ROC"}, 
        {(char *)"cemt", 1, (float )10800, (char *)"Europe/Berlin"}, 
        {(char *)"cemt", 1, (float )10800, (char *)"CET"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Berlin"}, 
        {(char *)"cest", 1, (float )10800, (char *)"Europe/Kaliningrad"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Africa/Algiers"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Africa/Ceuta"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Africa/Tripoli"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Africa/Tunis"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Arctic/Longyearbyen"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Atlantic/Jan_Mayen"}, 
        {(char *)"cest", 1, (float )7200, (char *)"CET"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Amsterdam"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Andorra"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Athens"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Belgrade"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Bratislava"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Brussels"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Budapest"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Chisinau"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Copenhagen"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Gibraltar"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Kaliningrad"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Kiev"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Lisbon"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Ljubljana"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Luxembourg"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Madrid"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Malta"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Minsk"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Monaco"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Oslo"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Paris"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Podgorica"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Prague"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Riga"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Rome"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/San_Marino"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Sarajevo"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Simferopol"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Skopje"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Sofia"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Stockholm"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Tallinn"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Tirane"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Tiraspol"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Uzhgorod"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Vaduz"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Vatican"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Vienna"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Vilnius"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Warsaw"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Zagreb"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Zaporozhye"}, 
        {(char *)"cest", 1, (float )7200, (char *)"Europe/Zurich"}, 
        {(char *)"cest", 1, (float )7200, (char *)"WET"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Berlin"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Africa/Algiers"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Africa/Casablanca"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Africa/Ceuta"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Africa/Tripoli"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Africa/Tunis"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Arctic/Longyearbyen"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Atlantic/Jan_Mayen"}, 
        {(char *)"cet", 0, (float )3600, (char *)"CET"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Amsterdam"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Andorra"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Athens"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Belgrade"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Bratislava"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Brussels"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Budapest"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Chisinau"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Copenhagen"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Gibraltar"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Kaliningrad"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Kiev"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Lisbon"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Ljubljana"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Luxembourg"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Madrid"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Malta"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Minsk"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Monaco"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Oslo"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Paris"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Podgorica"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Prague"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Riga"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Rome"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/San_Marino"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Sarajevo"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Simferopol"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Skopje"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Sofia"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Stockholm"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Tallinn"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Tirane"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Tiraspol"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Uzhgorod"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Vaduz"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Vatican"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Vienna"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Vilnius"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Warsaw"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Zagreb"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Zaporozhye"}, 
        {(char *)"cet", 0, (float )3600, (char *)"Europe/Zurich"}, 
        {(char *)"cet", 0, (float )3600, (char *)"WET"}, 
        {(char *)"cet", 0, (float )7200, (char *)"Europe/Kaliningrad"}, 
        {(char *)"cgst", 1, (float )-3600, (char *)"America/Scoresbysund"}, 
        {(char *)"cgt", 0, (float )-7200, (char *)"America/Scoresbysund"}, 
        {(char *)"chadt", 1, (float )49500, (char *)"Pacific/Chatham"}, 
        {(char *)"chast", 0, (float )45900, (char *)"Pacific/Chatham"}, 
        {(char *)"chat", 0, (float )30600, (char *)"Asia/Harbin"}, 
        {(char *)"chat", 0, (float )32400, (char *)"Asia/Harbin"}, 
        {(char *)"chdt", 1, (float )-19800, (char *)"America/Belize"}, 
        {(char *)"chost", 1, (float )36000, (char *)"Asia/Choibalsan"}, 
        {(char *)"chot", 0, (float )28800, (char *)"Asia/Choibalsan"}, 
        {(char *)"chot", 0, (float )32400, (char *)"Asia/Choibalsan"}, 
        {(char *)"chut", 0, (float )36000, (char *)"Pacific/Chuuk"}, 
        {(char *)"chut", 0, (float )36000, (char *)"Pacific/Truk"}, 
        {(char *)"chut", 0, (float )36000, (char *)"Pacific/Yap"}, 
        {(char *)"cit", 0, (float )28800, (char *)"Asia/Dili"}, 
        {(char *)"cit", 0, (float )28800, (char *)"Asia/Makassar"}, 
        {(char *)"cit", 0, (float )28800, (char *)"Asia/Pontianak"}, 
        {(char *)"cit", 0, (float )28800, (char *)"Asia/Ujung_Pandang"}, 
        {(char *)"cjt", 0, (float )32400, (char *)"Asia/Sakhalin"}, 
        {(char *)"cjt", 0, (float )32400, (char *)"Asia/Tokyo"}, 
        {(char *)"ckhst", 1, (float )-34200, (char *)"Pacific/Rarotonga"}, 
        {(char *)"ckt", 0, (float )-36000, (char *)"Pacific/Rarotonga"}, 
        {(char *)"ckt", 0, (float )-37800, (char *)"Pacific/Rarotonga"}, 
        {(char *)"clst", 1, (float )-10800, (char *)"America/Santiago"}, 
        {(char *)"clst", 1, (float )-14400, (char *)"America/Santiago"}, 
        {(char *)"clst", 1, (float )-10800, (char *)"Antarctica/Palmer"}, 
        {(char *)"clst", 1, (float )-10800, (char *)"Chile/Continental"}, 
        {(char *)"clst", 1, (float )-14400, (char *)"Chile/Continental"}, 
        {(char *)"clt", 0, (float )-14400, (char *)"America/Santiago"}, 
        {(char *)"clt", 0, (float )-18000, (char *)"America/Santiago"}, 
        {(char *)"clt", 0, (float )-14400, (char *)"Antarctica/Palmer"}, 
        {(char *)"clt", 0, (float )-14400, (char *)"Chile/Continental"}, 
        {(char *)"clt", 0, (float )-18000, (char *)"Chile/Continental"}, 
        {(char *)"cmt", 0, (float )-14640, (char *)"America/St_Lucia"}, 
        {(char *)"cmt", 0, (float )-15408, (char *)"America/Argentina/Buenos_Aires"}, 
        {(char *)"cmt",
      0, (float )-15408, (char *)"America/Argentina/Catamarca"}, 
        {(char *)"cmt", 0, (float )-15408,
      (char *)"America/Argentina/ComodRivadavia"}, 
        {(char *)"cmt", 0, (float )-15408, (char *)"America/Argentina/Cordoba"}, 
        {(char *)"cmt", 0, (float )-15408, (char *)"America/Argentina/Jujuy"}, 
        {(char *)"cmt", 0, (float )-15408, (char *)"America/Argentina/La_Rioja"}, 
        {(char *)"cmt", 0, (float )-15408, (char *)"America/Argentina/Mendoza"}, 
        {(char *)"cmt", 0, (float )-15408, (char *)"America/Argentina/Rio_Gallegos"}, 
        {(char *)"cmt",
      0, (float )-15408, (char *)"America/Argentina/Salta"}, 
        {(char *)"cmt", 0, (float )-15408, (char *)"America/Argentina/San_Juan"}, 
        {(char *)"cmt", 0, (float )-15408, (char *)"America/Argentina/San_Luis"}, 
        {(char *)"cmt", 0, (float )-15408, (char *)"America/Argentina/Tucuman"}, 
        {(char *)"cmt", 0, (float )-15408, (char *)"America/Argentina/Ushuaia"}, 
        {(char *)"cmt", 0, (float )-15408, (char *)"America/Buenos_Aires"}, 
        {(char *)"cmt", 0, (float )-15408, (char *)"America/Catamarca"}, 
        {(char *)"cmt", 0, (float )-15408, (char *)"America/Cordoba"}, 
        {(char *)"cmt", 0, (float )-15408, (char *)"America/Jujuy"}, 
        {(char *)"cmt", 0, (float )-15408, (char *)"America/Mendoza"}, 
        {(char *)"cmt", 0, (float )-15408, (char *)"America/Rosario"}, 
        {(char *)"cmt", 0, (float )-16060, (char *)"America/Caracas"}, 
        {(char *)"cmt", 0, (float )-16356, (char *)"America/La_Paz"}, 
        {(char *)"cmt", 0, (float )-19176, (char *)"America/Panama"}, 
        {(char *)"cmt", 0, (float )6900, (char *)"Europe/Chisinau"}, 
        {(char *)"cmt", 0, (float )6900, (char *)"Europe/Tiraspol"}, 
        {(char *)"cost", 1, (float )-14400, (char *)"America/Bogota"}, 
        {(char *)"cot", 0, (float )-18000, (char *)"America/Bogota"}, 
        {(char *)"cpt", 1, (float )-18000, (char *)"America/Chicago"}, 
        {(char *)"cpt", 1, (float )-18000, (char *)"America/Atikokan"}, 
        {(char *)"cpt", 1, (float )-18000, (char *)"America/Coral_Harbour"}, 
        {(char *)"cpt", 1, (float )-18000, (char *)"America/Fort_Wayne"}, 
        {(char *)"cpt", 1, (float )-18000, (char *)"America/Indiana/Indianapolis"}, 
        {(char *)"cpt", 1, (float )-18000, (char *)"America/Indiana/Knox"}, 
        {(char *)"cpt", 1, (float )-18000, (char *)"America/Indiana/Marengo"}, 
        {(char *)"cpt", 1, (float )-18000, (char *)"America/Indiana/Petersburg"}, 
        {(char *)"cpt", 1, (float )-18000, (char *)"America/Indiana/Tell_City"}, 
        {(char *)"cpt", 1, (float )-18000, (char *)"America/Indiana/Vevay"}, 
        {(char *)"cpt", 1, (float )-18000, (char *)"America/Indiana/Vincennes"}, 
        {(char *)"cpt", 1, (float )-18000, (char *)"America/Indiana/Winamac"}, 
        {(char *)"cpt", 1, (float )-18000, (char *)"America/Indianapolis"}, 
        {(char *)"cpt", 1, (float )-18000, (char *)"America/Kentucky/Louisville"}, 
        {(char *)"cpt", 1, (float )-18000, (char *)"America/Kentucky/Monticello"}, 
        {(char *)"cpt", 1, (float )-18000, (char *)"America/Knox_IN"}, 
        {(char *)"cpt", 1, (float )-18000, (char *)"America/Louisville"}, 
        {(char *)"cpt", 1, (float )-18000, (char *)"America/Menominee"}, 
        {(char *)"cpt", 1, (float )-18000, (char *)"America/Rainy_River"}, 
        {(char *)"cpt", 1, (float )-18000, (char *)"America/Winnipeg"}, 
        {(char *)"cpt", 1, (float )-18000, (char *)"Canada/Central"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Chicago"}, 
        {(char *)"cst", 0, (float )-18000, (char *)"America/Havana"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Atikokan"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Bahia_Banderas"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Belize"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Cambridge_Bay"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Cancun"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Chihuahua"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Coral_Harbour"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Costa_Rica"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Detroit"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/El_Salvador"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Fort_Wayne"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Guatemala"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Hermosillo"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Indiana/Indianapolis"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Indiana/Knox"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Indiana/Marengo"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Indiana/Petersburg"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Indiana/Tell_City"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Indiana/Vevay"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Indiana/Vincennes"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Indiana/Winamac"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Indianapolis"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Iqaluit"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Kentucky/Louisville"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Kentucky/Monticello"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Knox_IN"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Louisville"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Managua"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Matamoros"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Mazatlan"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Menominee"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Merida"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Mexico_City"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Monterrey"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/North_Dakota/Center"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/North_Dakota/New_Salem"}, 
        {(char *)"cst",
      0, (float )-21600, (char *)"America/Ojinaga"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Pangnirtung"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Rainy_River"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Rankin_Inlet"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Regina"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Resolute"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Swift_Current"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Tegucigalpa"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Thunder_Bay"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"America/Winnipeg"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"Canada/Central"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"Canada/East-Saskatchewan"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"Canada/Saskatchewan"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"Mexico/BajaSur"}, 
        {(char *)"cst", 0, (float )-21600, (char *)"Mexico/General"}, 
        {(char *)"cst", 0, (float )28800, (char *)"Asia/Chongqing"}, 
        {(char *)"cst", 0, (float )28800, (char *)"Asia/Chungking"}, 
        {(char *)"cst", 0, (float )28800, (char *)"Asia/Harbin"}, 
        {(char *)"cst", 0, (float )28800, (char *)"Asia/Kashgar"}, 
        {(char *)"cst", 0, (float )28800, (char *)"Asia/Macao"}, 
        {(char *)"cst", 0, (float )28800, (char *)"Asia/Macau"}, 
        {(char *)"cst", 0, (float )28800, (char *)"Asia/Shanghai"}, 
        {(char *)"cst", 0, (float )28800, (char *)"Asia/Taipei"}, 
        {(char *)"cst", 0, (float )28800, (char *)"Asia/Urumqi"}, 
        {(char *)"cst", 0, (float )28800, (char *)"PRC"}, 
        {(char *)"cst", 0, (float )28800, (char *)"ROC"}, 
        {(char *)"cst", 0, (float )34200, (char *)"Asia/Jayapura"}, 
        {(char *)"cst", 0, (float )34200, (char *)"Australia/Adelaide"}, 
        {(char *)"cst", 0, (float )34200, (char *)"Australia/Broken_Hill"}, 
        {(char *)"cst", 0, (float )34200, (char *)"Australia/Darwin"}, 
        {(char *)"cst", 0, (float )34200, (char *)"Australia/North"}, 
        {(char *)"cst", 0, (float )34200, (char *)"Australia/South"}, 
        {(char *)"cst", 0, (float )34200, (char *)"Australia/Yancowinna"}, 
        {(char *)"cst", 1, (float )37800, (char *)"Australia/Adelaide"}, 
        {(char *)"cst", 1, (float )37800, (char *)"Australia/Broken_Hill"}, 
        {(char *)"cst", 1, (float )37800, (char *)"Australia/Darwin"}, 
        {(char *)"cst", 1, (float )37800, (char *)"Australia/North"}, 
        {(char *)"cst", 1, (float )37800, (char *)"Australia/South"}, 
        {(char *)"cst", 1, (float )37800, (char *)"Australia/Yancowinna"}, 
        {(char *)"cut", 0, (float )8400, (char *)"Europe/Zaporozhye"}, 
        {(char *)"cvst", 1, (float )-3600, (char *)"Atlantic/Cape_Verde"}, 
        {(char *)"cvt", 0, (float )-3600, (char *)"Atlantic/Cape_Verde"}, 
        {(char *)"cvt", 0, (float )-7200, (char *)"Atlantic/Cape_Verde"}, 
        {(char *)"cwst", 0, (float )31500, (char *)"Australia/Eucla"}, 
        {(char *)"cwst", 1, (float )35100, (char *)"Australia/Eucla"}, 
        {(char *)"cwt", 1, (float )-18000, (char *)"America/Chicago"}, 
        {(char *)"cwt", 1, (float )-18000, (char *)"America/Atikokan"}, 
        {(char *)"cwt", 1, (float )-18000, (char *)"America/Coral_Harbour"}, 
        {(char *)"cwt", 1, (float )-18000, (char *)"America/Fort_Wayne"}, 
        {(char *)"cwt", 1, (float )-18000, (char *)"America/Indiana/Indianapolis"}, 
        {(char *)"cwt", 1, (float )-18000, (char *)"America/Indiana/Knox"}, 
        {(char *)"cwt", 1, (float )-18000, (char *)"America/Indiana/Marengo"}, 
        {(char *)"cwt", 1, (float )-18000, (char *)"America/Indiana/Petersburg"}, 
        {(char *)"cwt", 1, (float )-18000, (char *)"America/Indiana/Tell_City"}, 
        {(char *)"cwt", 1, (float )-18000, (char *)"America/Indiana/Vevay"}, 
        {(char *)"cwt", 1, (float )-18000, (char *)"America/Indiana/Vincennes"}, 
        {(char *)"cwt", 1, (float )-18000, (char *)"America/Indiana/Winamac"}, 
        {(char *)"cwt", 1, (float )-18000, (char *)"America/Indianapolis"}, 
        {(char *)"cwt", 1, (float )-18000, (char *)"America/Kentucky/Louisville"}, 
        {(char *)"cwt", 1, (float )-18000, (char *)"America/Kentucky/Monticello"}, 
        {(char *)"cwt", 1, (float )-18000, (char *)"America/Knox_IN"}, 
        {(char *)"cwt", 1, (float )-18000, (char *)"America/Louisville"}, 
        {(char *)"cwt", 1, (float )-18000, (char *)"America/Menominee"}, 
        {(char *)"cwt", 1, (float )-18000, (char *)"America/Mexico_City"}, 
        {(char *)"cwt", 1, (float )-18000, (char *)"America/Rainy_River"}, 
        {(char *)"cwt", 1, (float )-18000, (char *)"America/Winnipeg"}, 
        {(char *)"cwt", 1, (float )-18000, (char *)"Canada/Central"}, 
        {(char *)"cwt", 1, (float )-18000, (char *)"Mexico/General"}, 
        {(char *)"cxt", 0, (float )25200, (char *)"Indian/Christmas"}, 
        {(char *)"chst", 0, (float )36000, (char *)"Pacific/Guam"}, 
        {(char *)"chst", 0, (float )36000, (char *)"Pacific/Saipan"}, 
        {(char *)"dact", 0, (float )21600, (char *)"Asia/Dacca"}, 
        {(char *)"dact", 0, (float )21600, (char *)"Asia/Dhaka"}, 
        {(char *)"davt", 0, (float )18000, (char *)"Antarctica/Davis"}, 
        {(char *)"davt", 0, (float )25200, (char *)"Antarctica/Davis"}, 
        {(char *)"ddut", 0, (float )36000, (char *)"Antarctica/DumontDUrville"}, 
        {(char *)"dmt", 0, (float )-1521, (char *)"Europe/Dublin"}, 
        {(char *)"dusst", 1, (float )21600, (char *)"Asia/Dushanbe"}, 
        {(char *)"dusst", 1, (float )25200, (char *)"Asia/Dushanbe"}, 
        {(char *)"dust", 0, (float )18000, (char *)"Asia/Dushanbe"}, 
        {(char *)"dust", 0, (float )21600, (char *)"Asia/Dushanbe"}, 
        {(char *)"easst", 1, (float )-18000, (char *)"Chile/EasterIsland"}, 
        {(char *)"easst", 1, (float )-21600, (char *)"Chile/EasterIsland"}, 
        {(char *)"easst", 1, (float )-18000, (char *)"Pacific/Easter"}, 
        {(char *)"easst", 1, (float )-21600, (char *)"Pacific/Easter"}, 
        {(char *)"east", 0, (float )-21600, (char *)"Chile/EasterIsland"}, 
        {(char *)"east", 0, (float )-25200, (char *)"Chile/EasterIsland"}, 
        {(char *)"east", 0, (float )-21600, (char *)"Pacific/Easter"}, 
        {(char *)"east", 0, (float )-25200, (char *)"Pacific/Easter"}, 
        {(char *)"east", 1, (float )14400, (char *)"Indian/Antananarivo"}, 
        {(char *)"eat", 0, (float )10800, (char *)"Africa/Khartoum"}, 
        {(char *)"eat", 0, (float )10800, (char *)"Africa/Addis_Ababa"}, 
        {(char *)"eat", 0, (float )10800, (char *)"Africa/Asmara"}, 
        {(char *)"eat", 0, (float )10800, (char *)"Africa/Asmera"}, 
        {(char *)"eat", 0, (float )10800, (char *)"Africa/Dar_es_Salaam"}, 
        {(char *)"eat", 0, (float )10800, (char *)"Africa/Djibouti"}, 
        {(char *)"eat", 0, (float )10800, (char *)"Africa/Kampala"}, 
        {(char *)"eat", 0, (float )10800, (char *)"Africa/Mogadishu"}, 
        {(char *)"eat", 0, (float )10800, (char *)"Africa/Nairobi"}, 
        {(char *)"eat", 0, (float )10800, (char *)"Indian/Antananarivo"}, 
        {(char *)"eat", 0, (float )10800, (char *)"Indian/Comoro"}, 
        {(char *)"eat", 0, (float )10800, (char *)"Indian/Mayotte"}, 
        {(char *)"ect", 0, (float )-18000, (char *)"America/Guayaquil"}, 
        {(char *)"ect", 0, (float )-18000, (char *)"Pacific/Galapagos"}, 
        {(char *)"eddt", 1, (float )-10800, (char *)"America/Iqaluit"}, 
        {(char *)"edt", 1, (float )-14400, (char *)"America/New_York"}, 
        {(char *)"edt", 1, (float )-14400, (char *)"America/Cancun"}, 
        {(char *)"edt", 1, (float )-14400, (char *)"America/Detroit"}, 
        {(char *)"edt", 1, (float )-14400, (char *)"America/Fort_Wayne"}, 
        {(char *)"edt", 1, (float )-14400, (char *)"America/Grand_Turk"}, 
        {(char *)"edt", 1, (float )-14400, (char *)"America/Indiana/Indianapolis"}, 
        {(char *)"edt", 1, (float )-14400, (char *)"America/Indiana/Marengo"}, 
        {(char *)"edt", 1, (float )-14400, (char *)"America/Indiana/Petersburg"}, 
        {(char *)"edt", 1, (float )-14400, (char *)"America/Indiana/Tell_City"}, 
        {(char *)"edt", 1, (float )-14400, (char *)"America/Indiana/Vevay"}, 
        {(char *)"edt", 1, (float )-14400, (char *)"America/Indiana/Vincennes"}, 
        {(char *)"edt", 1, (float )-14400, (char *)"America/Indiana/Winamac"}, 
        {(char *)"edt", 1, (float )-14400, (char *)"America/Indianapolis"}, 
        {(char *)"edt", 1, (float )-14400, (char *)"America/Iqaluit"}, 
        {(char *)"edt", 1, (float )-14400, (char *)"America/Jamaica"}, 
        {(char *)"edt", 1, (float )-14400, (char *)"America/Kentucky/Louisville"}, 
        {(char *)"edt", 1, (float )-14400, (char *)"America/Kentucky/Monticello"}, 
        {(char *)"edt", 1, (float )-14400, (char *)"America/Louisville"}, 
        {(char *)"edt", 1, (float )-14400, (char *)"America/Montreal"}, 
        {(char *)"edt", 1, (float )-14400, (char *)"America/Nassau"}, 
        {(char *)"edt", 1, (float )-14400, (char *)"America/Nipigon"}, 
        {(char *)"edt", 1, (float )-14400, (char *)"America/Pangnirtung"}, 
        {(char *)"edt", 1, (float )-14400, (char *)"America/Port-au-Prince"}, 
        {(char *)"edt", 1, (float )-14400, (char *)"America/Santo_Domingo"}, 
        {(char *)"edt", 1, (float )-14400, (char *)"America/Thunder_Bay"}, 
        {(char *)"edt", 1, (float )-14400, (char *)"America/Toronto"}, 
        {(char *)"edt", 1, (float )-14400, (char *)"Canada/Eastern"}, 
        {(char *)"edt", 1, (float )-14400, (char *)"EST"}, 
        {(char *)"eest", 1, (float )10800, (char *)"Europe/Helsinki"}, 
        {(char *)"eest", 1, (float )10800, (char *)"Africa/Cairo"}, 
        {(char *)"eest", 1, (float )10800, (char *)"Asia/Amman"}, 
        {(char *)"eest", 1, (float )10800, (char *)"Asia/Beirut"}, 
        {(char *)"eest", 1, (float )10800, (char *)"Asia/Damascus"}, 
        {(char *)"eest", 1, (float )10800, (char *)"Asia/Gaza"}, 
        {(char *)"eest", 1, (float )10800, (char *)"Asia/Istanbul"}, 
        {(char *)"eest", 1, (float )10800, (char *)"Asia/Nicosia"}, 
        {(char *)"eest", 1, (float )10800, (char *)"EET"}, 
        {(char *)"eest", 1, (float )10800, (char *)"Europe/Athens"}, 
        {(char *)"eest", 1, (float )10800, (char *)"Europe/Bucharest"}, 
        {(char *)"eest", 1, (float )10800, (char *)"Europe/Chisinau"}, 
        {(char *)"eest", 1, (float )10800, (char *)"Europe/Istanbul"}, 
        {(char *)"eest", 1, (float )10800, (char *)"Europe/Kaliningrad"}, 
        {(char *)"eest", 1, (float )10800, (char *)"Europe/Kiev"}, 
        {(char *)"eest", 1, (float )10800, (char *)"Europe/Mariehamn"}, 
        {(char *)"eest", 1, (float )10800, (char *)"Europe/Minsk"}, 
        {(char *)"eest", 1, (float )10800, (char *)"Europe/Moscow"}, 
        {(char *)"eest", 1, (float )10800, (char *)"Europe/Nicosia"}, 
        {(char *)"eest", 1, (float )10800, (char *)"Europe/Riga"}, 
        {(char *)"eest", 1, (float )10800, (char *)"Europe/Simferopol"}, 
        {(char *)"eest", 1, (float )10800, (char *)"Europe/Sofia"}, 
        {(char *)"eest", 1, (float )10800, (char *)"Europe/Tallinn"}, 
        {(char *)"eest", 1, (float )10800, (char *)"Europe/Tiraspol"}, 
        {(char *)"eest", 1, (float )10800, (char *)"Europe/Uzhgorod"}, 
        {(char *)"eest", 1, (float )10800, (char *)"Europe/Vilnius"}, 
        {(char *)"eest", 1, (float )10800, (char *)"Europe/Warsaw"}, 
        {(char *)"eest", 1, (float )10800, (char *)"Europe/Zaporozhye"}, 
        {(char *)"eet", 0, (float )7200, (char *)"Europe/Helsinki"}, 
        {(char *)"eet", 1, (float )10800, (char *)"Asia/Gaza"}, 
        {(char *)"eet", 0, (float )7200, (char *)"Africa/Cairo"}, 
        {(char *)"eet", 0, (float )7200, (char *)"Africa/Tripoli"}, 
        {(char *)"eet", 0, (float )7200, (char *)"Asia/Amman"}, 
        {(char *)"eet", 0, (float )7200, (char *)"Asia/Beirut"}, 
        {(char *)"eet", 0, (float )7200, (char *)"Asia/Damascus"}, 
        {(char *)"eet", 0, (float )7200, (char *)"Asia/Gaza"}, 
        {(char *)"eet", 0, (float )7200, (char *)"Asia/Istanbul"}, 
        {(char *)"eet", 0, (float )7200, (char *)"Asia/Nicosia"}, 
        {(char *)"eet", 0, (float )7200, (char *)"EET"}, 
        {(char *)"eet", 0, (float )7200, (char *)"Europe/Athens"}, 
        {(char *)"eet", 0, (float )7200, (char *)"Europe/Bucharest"}, 
        {(char *)"eet", 0, (float )7200, (char *)"Europe/Chisinau"}, 
        {(char *)"eet", 0, (float )7200, (char *)"Europe/Istanbul"}, 
        {(char *)"eet", 0, (float )7200, (char *)"Europe/Kaliningrad"}, 
        {(char *)"eet", 0, (float )7200, (char *)"Europe/Kiev"}, 
        {(char *)"eet", 0, (float )7200, (char *)"Europe/Mariehamn"}, 
        {(char *)"eet", 0, (float )7200, (char *)"Europe/Minsk"}, 
        {(char *)"eet", 0, (float )7200, (char *)"Europe/Moscow"}, 
        {(char *)"eet", 0, (float )7200, (char *)"Europe/Nicosia"}, 
        {(char *)"eet", 0, (float )7200, (char *)"Europe/Riga"}, 
        {(char *)"eet", 0, (float )7200, (char *)"Europe/Simferopol"}, 
        {(char *)"eet", 0, (float )7200, (char *)"Europe/Sofia"}, 
        {(char *)"eet", 0, (float )7200, (char *)"Europe/Tallinn"}, 
        {(char *)"eet", 0, (float )7200, (char *)"Europe/Tiraspol"}, 
        {(char *)"eet", 0, (float )7200, (char *)"Europe/Uzhgorod"}, 
        {(char *)"eet", 0, (float )7200, (char *)"Europe/Vilnius"}, 
        {(char *)"eet", 0, (float )7200, (char *)"Europe/Warsaw"}, 
        {(char *)"eet", 0, (float )7200, (char *)"Europe/Zaporozhye"}, 
        {(char *)"egst", 1, (float )0, (char *)"America/Scoresbysund"}, 
        {(char *)"egt", 0, (float )-3600, (char *)"America/Scoresbysund"}, 
        {(char *)"ehdt", 1, (float )-16200, (char *)"America/Santo_Domingo"}, 
        {(char *)"eit", 0, (float )32400, (char *)"Asia/Jayapura"}, 
        {(char *)"emt", 0, (float )-26248, (char *)"Chile/EasterIsland"}, 
        {(char *)"emt", 0, (float )-26248, (char *)"Pacific/Easter"}, 
        {(char *)"ept", 1, (float )-14400, (char *)"America/New_York"}, 
        {(char *)"ept", 1, (float )-14400, (char *)"America/Detroit"}, 
        {(char *)"ept", 1, (float )-14400, (char *)"America/Iqaluit"}, 
        {(char *)"ept", 1, (float )-14400, (char *)"America/Montreal"}, 
        {(char *)"ept", 1, (float )-14400, (char *)"America/Nipigon"}, 
        {(char *)"ept", 1, (float )-14400, (char *)"America/Thunder_Bay"}, 
        {(char *)"ept", 1, (float )-14400, (char *)"America/Toronto"}, 
        {(char *)"ept", 1, (float )-14400, (char *)"Canada/Eastern"}, 
        {(char *)"ept", 1, (float )-14400, (char *)"EST"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/New_York"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Antigua"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Atikokan"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Cambridge_Bay"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Cancun"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Cayman"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Chicago"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Coral_Harbour"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Detroit"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Fort_Wayne"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Grand_Turk"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Indiana/Indianapolis"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Indiana/Knox"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Indiana/Marengo"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Indiana/Petersburg"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Indiana/Tell_City"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Indiana/Vevay"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Indiana/Vincennes"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Indiana/Winamac"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Indianapolis"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Iqaluit"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Jamaica"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Kentucky/Louisville"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Kentucky/Monticello"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Knox_IN"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Louisville"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Managua"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Menominee"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Merida"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Moncton"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Montreal"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Nassau"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Nipigon"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Panama"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Pangnirtung"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Port-au-Prince"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Rankin_Inlet"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Resolute"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Santo_Domingo"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Thunder_Bay"}, 
        {(char *)"est", 0, (float )-18000, (char *)"America/Toronto"}, 
        {(char *)"est", 0, (float )-18000, (char *)"Canada/Eastern"}, 
        {(char *)"est", 0, (float )-18000, (char *)"EST"}, 
        {(char *)"est", 0, (float )36000, (char *)"Antarctica/Macquarie"}, 
        {(char *)"est", 0, (float )36000, (char *)"Australia/ACT"}, 
        {(char *)"est", 0, (float )36000, (char *)"Australia/Brisbane"}, 
        {(char *)"est", 0, (float )36000, (char *)"Australia/Canberra"}, 
        {(char *)"est", 0, (float )36000, (char *)"Australia/Currie"}, 
        {(char *)"est", 0, (float )36000, (char *)"Australia/Hobart"}, 
        {(char *)"est", 0, (float )36000, (char *)"Australia/LHI"}, 
        {(char *)"est", 0, (float )36000, (char *)"Australia/Lindeman"}, 
        {(char *)"est", 0, (float )36000, (char *)"Australia/Lord_Howe"}, 
        {(char *)"est", 0, (float )36000, (char *)"Australia/Melbourne"}, 
        {(char *)"est", 0, (float )36000, (char *)"Australia/NSW"}, 
        {(char *)"est", 0, (float )36000, (char *)"Australia/Queensland"}, 
        {(char *)"est", 0, (float )36000, (char *)"Australia/Sydney"}, 
        {(char *)"est", 0, (float )36000, (char *)"Australia/Tasmania"}, 
        {(char *)"est", 0, (float )36000, (char *)"Australia/Victoria"}, 
        {(char *)"est", 1, (float )39600, (char *)"Australia/Melbourne"}, 
        {(char *)"est", 1, (float )39600, (char *)"Antarctica/Macquarie"}, 
        {(char *)"est", 1, (float )39600, (char *)"Australia/ACT"}, 
        {(char *)"est", 1, (float )39600, (char *)"Australia/Brisbane"}, 
        {(char *)"est", 1, (float )39600, (char *)"Australia/Canberra"}, 
        {(char *)"est", 1, (float )39600, (char *)"Australia/Currie"}, 
        {(char *)"est", 1, (float )39600, (char *)"Australia/Hobart"}, 
        {(char *)"est", 1, (float )39600, (char *)"Australia/Lindeman"}, 
        {(char *)"est", 1, (float )39600, (char *)"Australia/NSW"}, 
        {(char *)"est", 1, (float )39600, (char *)"Australia/Queensland"}, 
        {(char *)"est", 1, (float )39600, (char *)"Australia/Sydney"}, 
        {(char *)"est", 1, (float )39600, (char *)"Australia/Tasmania"}, 
        {(char *)"est", 1, (float )39600, (char *)"Australia/Victoria"}, 
        {(char *)"ewt", 1, (float )-14400, (char *)"America/New_York"}, 
        {(char *)"ewt", 1, (float )-14400, (char *)"America/Detroit"}, 
        {(char *)"ewt", 1, (float )-14400, (char *)"America/Iqaluit"}, 
        {(char *)"ewt", 1, (float )-14400, (char *)"America/Montreal"}, 
        {(char *)"ewt", 1, (float )-14400, (char *)"America/Nipigon"}, 
        {(char *)"ewt", 1, (float )-14400, (char *)"America/Thunder_Bay"}, 
        {(char *)"ewt", 1, (float )-14400, (char *)"America/Toronto"}, 
        {(char *)"ewt", 1, (float )-14400, (char *)"Canada/Eastern"}, 
        {(char *)"ewt", 1, (float )-14400, (char *)"EST"}, 
        {(char *)"ffmt", 0, (float )-14660, (char *)"America/Martinique"}, 
        {(char *)"fjst", 1, (float )46800, (char *)"Pacific/Fiji"}, 
        {(char *)"fjt", 0, (float )43200, (char *)"Pacific/Fiji"}, 
        {(char *)"fkst", 1, (float )-10800, (char *)"Atlantic/Stanley"}, 
        {(char *)"fkst", 1, (float )-7200, (char *)"Atlantic/Stanley"}, 
        {(char *)"fkt", 0, (float )-10800, (char *)"Atlantic/Stanley"}, 
        {(char *)"fkt", 0, (float )-14400, (char *)"Atlantic/Stanley"}, 
        {(char *)"fmt", 0, (float )-3180, (char *)"Africa/Freetown"}, 
        {(char *)"fmt", 0, (float )-4056, (char *)"Atlantic/Madeira"}, 
        {(char *)"fnst", 1, (float )-3600, (char *)"America/Noronha"}, 
        {(char *)"fnst", 1, (float )-3600, (char *)"Brazil/DeNoronha"}, 
        {(char *)"fnt", 0, (float )-7200, (char *)"America/Noronha"}, 
        {(char *)"fnt", 0, (float )-7200, (char *)"Brazil/DeNoronha"}, 
        {(char *)"fort", 0, (float )14400, (char *)"Asia/Aqtau"}, 
        {(char *)"fort", 0, (float )18000, (char *)"Asia/Aqtau"}, 
        {(char *)"frust", 1, (float )21600, (char *)"Asia/Bishkek"}, 
        {(char *)"frust", 1, (float )25200, (char *)"Asia/Bishkek"}, 
        {(char *)"frut", 0, (float )18000, (char *)"Asia/Bishkek"}, 
        {(char *)"frut", 0, (float )21600, (char *)"Asia/Bishkek"}, 
        {(char *)"galt", 0, (float )-21600, (char *)"Pacific/Galapagos"}, 
        {(char *)"gamt", 0, (float )-32400, (char *)"Pacific/Gambier"}, 
        {(char *)"gbgt", 0, (float )-13500, (char *)"America/Guyana"}, 
        {(char *)"gest", 1, (float )14400, (char *)"Asia/Tbilisi"}, 
        {(char *)"gest", 1, (float )18000, (char *)"Asia/Tbilisi"}, 
        {(char *)"get", 0, (float )10800, (char *)"Asia/Tbilisi"}, 
        {(char *)"get", 0, (float )14400, (char *)"Asia/Tbilisi"}, 
        {(char *)"gft", 0, (float )-10800, (char *)"America/Cayenne"}, 
        {(char *)"gft", 0, (float )-14400, (char *)"America/Cayenne"}, 
        {(char *)"ghst", 1, (float )1200, (char *)"Africa/Accra"}, 
        {(char *)"gilt", 0, (float )43200, (char *)"Pacific/Tarawa"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Africa/Abidjan"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Africa/Accra"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Africa/Bamako"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Africa/Banjul"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Africa/Bissau"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Africa/Conakry"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Africa/Dakar"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Africa/Freetown"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Africa/Lome"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Africa/Malabo"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Africa/Monrovia"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Africa/Niamey"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Africa/Nouakchott"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Africa/Ouagadougou"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Africa/Porto-Novo"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Africa/Sao_Tome"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Africa/Timbuktu"}, 
        {(char *)"gmt", 0, (float )0, (char *)"America/Danmarkshavn"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Atlantic/Reykjavik"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Atlantic/St_Helena"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Etc/GMT"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Etc/Greenwich"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Europe/Belfast"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Europe/Dublin"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Europe/Gibraltar"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Europe/Guernsey"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Europe/Isle_of_Man"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Europe/Jersey"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Europe/London"}, 
        {(char *)"gmt", 0, (float )0, (char *)"GB"}, 
        {(char *)"gst", 0, (float )14400, (char *)"Asia/Dubai"}, 
        {(char *)"gst", 0, (float )-7200, (char *)"Atlantic/South_Georgia"}, 
        {(char *)"gst", 0, (float )14400, (char *)"Asia/Bahrain"}, 
        {(char *)"gst", 0, (float )14400, (char *)"Asia/Muscat"}, 
        {(char *)"gst", 0, (float )14400, (char *)"Asia/Qatar"}, 
        {(char *)"gst", 0, (float )36000, (char *)"Pacific/Guam"}, 
        {(char *)"gyt", 0, (float )-10800, (char *)"America/Guyana"}, 
        {(char *)"gyt", 0, (float )-13500, (char *)"America/Guyana"}, 
        {(char *)"gyt", 0, (float )-14400, (char *)"America/Guyana"}, 
        {(char *)"hadt", 1, (float )-32400, (char *)"America/Adak"}, 
        {(char *)"hadt", 1, (float )-32400, (char *)"America/Atka"}, 
        {(char *)"hast", 0, (float )-36000, (char *)"America/Adak"}, 
        {(char *)"hast", 0, (float )-36000, (char *)"America/Atka"}, 
        {(char *)"hdt", 1, (float )-34200, (char *)"Pacific/Honolulu"}, 
        {(char *)"hdt", 1, (float )-34200, (char *)"HST"}, 
        {(char *)"hkst", 1, (float )32400, (char *)"Asia/Hong_Kong"}, 
        {(char *)"hkt", 0, (float )28800, (char *)"Asia/Hong_Kong"}, 
        {(char *)"hmt", 0, (float )-19776, (char *)"America/Havana"}, 
        {(char *)"hmt", 0, (float )-6872, (char *)"Atlantic/Azores"}, 
        {(char *)"hmt", 0, (float )21200, (char *)"Asia/Calcutta"}, 
        {(char *)"hmt", 0, (float )21200, (char *)"Asia/Dacca"}, 
        {(char *)"hmt", 0, (float )21200, (char *)"Asia/Dhaka"}, 
        {(char *)"hmt", 0, (float )21200, (char *)"Asia/Kolkata"}, 
        {(char *)"hmt", 0, (float )5992, (char *)"EET"}, 
        {(char *)"hmt", 0, (float )5992, (char *)"Europe/Helsinki"}, 
        {(char *)"hmt", 0, (float )5992, (char *)"Europe/Mariehamn"}, 
        {(char *)"hovst", 1, (float )28800, (char *)"Asia/Hovd"}, 
        {(char *)"hovt", 0, (float )21600, (char *)"Asia/Hovd"}, 
        {(char *)"hovt", 0, (float )25200, (char *)"Asia/Hovd"}, 
        {(char *)"hpt", 1, (float )-34200, (char *)"Pacific/Honolulu"}, 
        {(char *)"hpt", 1, (float )-34200, (char *)"HST"}, 
        {(char *)"hst", 0, (float )-36000, (char *)"Pacific/Honolulu"}, 
        {(char *)"hst", 0, (float )-37800, (char *)"Pacific/Honolulu"}, 
        {(char *)"hst", 0, (float )-36000, (char *)"HST"}, 
        {(char *)"hst", 0, (float )-36000, (char *)"Pacific/Johnston"}, 
        {(char *)"hst", 0, (float )-37800, (char *)"HST"}, 
        {(char *)"hwt", 1, (float )-34200, (char *)"Pacific/Honolulu"}, 
        {(char *)"hwt", 1, (float )-34200, (char *)"HST"}, 
        {(char *)"ict", 0, (float )25200, (char *)"Asia/Bangkok"}, 
        {(char *)"ict", 0, (float )25200, (char *)"Asia/Ho_Chi_Minh"}, 
        {(char *)"ict", 0, (float )25200, (char *)"Asia/Phnom_Penh"}, 
        {(char *)"ict", 0, (float )25200, (char *)"Asia/Saigon"}, 
        {(char *)"ict", 0, (float )25200, (char *)"Asia/Vientiane"}, 
        {(char *)"ict", 0, (float )28800, (char *)"Asia/Ho_Chi_Minh"}, 
        {(char *)"ict", 0, (float )28800, (char *)"Asia/Phnom_Penh"}, 
        {(char *)"ict", 0, (float )28800, (char *)"Asia/Saigon"}, 
        {(char *)"ict", 0, (float )28800, (char *)"Asia/Vientiane"}, 
        {(char *)"iddt", 1, (float )14400, (char *)"Asia/Jerusalem"}, 
        {(char *)"iddt", 1, (float )14400, (char *)"Asia/Tel_Aviv"}, 
        {(char *)"idt", 1, (float )10800, (char *)"Asia/Jerusalem"}, 
        {(char *)"idt", 1, (float )10800, (char *)"Asia/Gaza"}, 
        {(char *)"idt", 1, (float )10800, (char *)"Asia/Tel_Aviv"}, 
        {(char *)"ihst", 1, (float )21600, (char *)"Asia/Colombo"}, 
        {(char *)"imt", 0, (float )25040, (char *)"Asia/Irkutsk"}, 
        {(char *)"imt", 0, (float )7016, (char *)"Asia/Istanbul"}, 
        {(char *)"imt", 0, (float )7016, (char *)"Europe/Istanbul"}, 
        {(char *)"iot", 0, (float )18000, (char *)"Indian/Chagos"}, 
        {(char *)"iot", 0, (float )21600, (char *)"Indian/Chagos"}, 
        {(char *)"irdt", 1, (float )16200, (char *)"Asia/Tehran"}, 
        {(char *)"irdt", 1, (float )18000, (char *)"Asia/Tehran"}, 
        {(char *)"irkst", 1, (float )28800, (char *)"Asia/Irkutsk"}, 
        {(char *)"irkst", 1, (float )32400, (char *)"Asia/Irkutsk"}, 
        {(char *)"irkt", 0, (float )25200, (char *)"Asia/Irkutsk"}, 
        {(char *)"irkt", 0, (float )28800, (char *)"Asia/Irkutsk"}, 
        {(char *)"irst", 0, (float )12600, (char *)"Asia/Tehran"}, 
        {(char *)"irst", 0, (float )14400, (char *)"Asia/Tehran"}, 
        {(char *)"isst", 1, (float )0, (char *)"Atlantic/Reykjavik"}, 
        {(char *)"ist", 0, (float )7200, (char *)"Asia/Jerusalem"}, 
        {(char *)"ist", 0, (float )-3600, (char *)"Atlantic/Reykjavik"}, 
        {(char *)"ist", 0, (float )19800, (char *)"Asia/Calcutta"}, 
        {(char *)"ist", 0, (float )19800, (char *)"Asia/Colombo"}, 
        {(char *)"ist", 0, (float )19800, (char *)"Asia/Dacca"}, 
        {(char *)"ist", 0, (float )19800, (char *)"Asia/Dhaka"}, 
        {(char *)"ist", 0, (float )19800, (char *)"Asia/Karachi"}, 
        {(char *)"ist", 0, (float )19800, (char *)"Asia/Kathmandu"}, 
        {(char *)"ist", 0, (float )19800, (char *)"Asia/Katmandu"}, 
        {(char *)"ist", 0, (float )19800, (char *)"Asia/Kolkata"}, 
        {(char *)"ist", 0, (float )19800, (char *)"Asia/Thimbu"}, 
        {(char *)"ist", 0, (float )19800, (char *)"Asia/Thimphu"}, 
        {(char *)"ist", 1, (float )2079, (char *)"Europe/Dublin"}, 
        {(char *)"ist", 1, (float )23400, (char *)"Asia/Calcutta"}, 
        {(char *)"ist", 1, (float )23400, (char *)"Asia/Colombo"}, 
        {(char *)"ist", 1, (float )23400, (char *)"Asia/Karachi"}, 
        {(char *)"ist", 1, (float )23400, (char *)"Asia/Kolkata"}, 
        {(char *)"ist", 0, (float )3600, (char *)"Europe/Dublin"}, 
        {(char *)"ist", 1, (float )3600, (char *)"Europe/Dublin"}, 
        {(char *)"ist", 0, (float )7200, (char *)"Asia/Gaza"}, 
        {(char *)"ist", 0, (float )7200, (char *)"Asia/Tel_Aviv"}, 
        {(char *)"javt", 0, (float )26400, (char *)"Asia/Jakarta"}, 
        {(char *)"jdt", 1, (float )36000, (char *)"Asia/Tokyo"}, 
        {(char *)"jmt", 0, (float )-1368, (char *)"Atlantic/St_Helena"}, 
        {(char *)"jmt", 0, (float )25632, (char *)"Asia/Jakarta"}, 
        {(char *)"jmt", 0, (float )8440, (char *)"Asia/Jerusalem"}, 
        {(char *)"jmt", 0, (float )8440, (char *)"Asia/Tel_Aviv"}, 
        {(char *)"jst", 0, (float )32400, (char *)"Asia/Tokyo"}, 
        {(char *)"jst", 0, (float )32400, (char *)"Asia/Dili"}, 
        {(char *)"jst", 0, (float )32400, (char *)"Asia/Hong_Kong"}, 
        {(char *)"jst", 0, (float )32400, (char *)"Asia/Jakarta"}, 
        {(char *)"jst", 0, (float )32400, (char *)"Asia/Kuala_Lumpur"}, 
        {(char *)"jst", 0, (float )32400, (char *)"Asia/Kuching"}, 
        {(char *)"jst", 0, (float )32400, (char *)"Asia/Makassar"}, 
        {(char *)"jst", 0, (float )32400, (char *)"Asia/Manila"}, 
        {(char *)"jst", 0, (float )32400, (char *)"Asia/Pontianak"}, 
        {(char *)"jst", 0, (float )32400, (char *)"Asia/Rangoon"}, 
        {(char *)"jst", 0, (float )32400, (char *)"Asia/Sakhalin"}, 
        {(char *)"jst", 0, (float )32400, (char *)"Asia/Singapore"}, 
        {(char *)"jst", 0, (float )32400, (char *)"Asia/Ujung_Pandang"}, 
        {(char *)"jst", 0, (float )32400, (char *)"Pacific/Nauru"}, 
        {(char *)"kart", 0, (float )18000, (char *)"Asia/Karachi"}, 
        {(char *)"kast", 0, (float )18000, (char *)"Asia/Kashgar"}, 
        {(char *)"kast", 0, (float )19800, (char *)"Asia/Kashgar"}, 
        {(char *)"kdt", 1, (float )32400, (char *)"Asia/Seoul"}, 
        {(char *)"kdt", 1, (float )36000, (char *)"Asia/Seoul"}, 
        {(char *)"kdt", 1, (float )32400, (char *)"ROK"}, 
        {(char *)"kdt", 1, (float )36000, (char *)"ROK"}, 
        {(char *)"kgst", 1, (float )21600, (char *)"Asia/Bishkek"}, 
        {(char *)"kgt", 0, (float )18000, (char *)"Asia/Bishkek"}, 
        {(char *)"kgt", 0, (float )21600, (char *)"Asia/Bishkek"}, 
        {(char *)"kizst", 1, (float )21600, (char *)"Asia/Qyzylorda"}, 
        {(char *)"kizt", 0, (float )14400, (char *)"Asia/Qyzylorda"}, 
        {(char *)"kizt", 0, (float )18000, (char *)"Asia/Qyzylorda"}, 
        {(char *)"kizt", 0, (float )21600, (char *)"Asia/Qyzylorda"}, 
        {(char *)"kmt", 0, (float )5736, (char *)"Europe/Vilnius"}, 
        {(char *)"kmt", 0, (float )-14696, (char *)"America/St_Vincent"}, 
        {(char *)"kmt", 0, (float )-18432, (char *)"America/Cayman"}, 
        {(char *)"kmt", 0, (float )-18432, (char *)"America/Grand_Turk"}, 
        {(char *)"kmt", 0, (float )-18432, (char *)"America/Jamaica"}, 
        {(char *)"kmt", 0, (float )7324, (char *)"Europe/Kiev"}, 
        {(char *)"kost", 0, (float )39600, (char *)"Pacific/Kosrae"}, 
        {(char *)"kost", 0, (float )43200, (char *)"Pacific/Kosrae"}, 
        {(char *)"krast", 1, (float )25200, (char *)"Asia/Krasnoyarsk"}, 
        {(char *)"krast", 1, (float )28800, (char *)"Asia/Krasnoyarsk"}, 
        {(char *)"krast", 1, (float )25200, (char *)"Asia/Novokuznetsk"}, 
        {(char *)"krast", 1, (float )28800, (char *)"Asia/Novokuznetsk"}, 
        {(char *)"krat", 0, (float )21600, (char *)"Asia/Krasnoyarsk"}, 
        {(char *)"krat", 0, (float )25200, (char *)"Asia/Krasnoyarsk"}, 
        {(char *)"krat", 0, (float )21600, (char *)"Asia/Novokuznetsk"}, 
        {(char *)"krat", 0, (float )25200, (char *)"Asia/Novokuznetsk"}, 
        {(char *)"kst", 0, (float )28800, (char *)"Asia/Seoul"}, 
        {(char *)"kst", 0, (float )30600, (char *)"Asia/Seoul"}, 
        {(char *)"kst", 0, (float )32400, (char *)"Asia/Seoul"}, 
        {(char *)"kst", 0, (float )28800, (char *)"Asia/Pyongyang"}, 
        {(char *)"kst", 0, (float )28800, (char *)"ROK"}, 
        {(char *)"kst", 0, (float )30600, (char *)"Asia/Pyongyang"}, 
        {(char *)"kst", 0, (float )30600, (char *)"ROK"}, 
        {(char *)"kst", 0, (float )32400, (char *)"Asia/Pyongyang"}, 
        {(char *)"kst", 0, (float )32400, (char *)"ROK"}, 
        {(char *)"kuyst", 1, (float )10800, (char *)"Europe/Samara"}, 
        {(char *)"kuyst", 1, (float )14400, (char *)"Europe/Samara"}, 
        {(char *)"kuyst", 1, (float )18000, (char *)"Europe/Samara"}, 
        {(char *)"kuyt", 0, (float )10800, (char *)"Europe/Samara"}, 
        {(char *)"kuyt", 0, (float )14400, (char *)"Europe/Samara"}, 
        {(char *)"kwat", 0, (float )-43200, (char *)"Pacific/Kwajalein"}, 
        {(char *)"lhst", 0, (float )37800, (char *)"Australia/Lord_Howe"}, 
        {(char *)"lhst", 1, (float )39600, (char *)"Australia/Lord_Howe"}, 
        {(char *)"lhst", 1, (float )41400, (char *)"Australia/Lord_Howe"}, 
        {(char *)"lhst", 0, (float )37800, (char *)"Australia/LHI"}, 
        {(char *)"lhst", 1, (float )39600, (char *)"Australia/LHI"}, 
        {(char *)"lhst", 1, (float )41400, (char *)"Australia/LHI"}, 
        {(char *)"lint", 0, (float )-36000, (char *)"Pacific/Kiritimati"}, 
        {(char *)"lint", 0, (float )-38400, (char *)"Pacific/Kiritimati"}, 
        {(char *)"lint", 0, (float )50400, (char *)"Pacific/Kiritimati"}, 
        {(char *)"lkt", 0, (float )21600, (char *)"Asia/Colombo"}, 
        {(char *)"lkt", 0, (float )23400, (char *)"Asia/Colombo"}, 
        {(char *)"lmt", 0, (float )-11188, (char *)"America/Sao_Paulo"}, 
        {(char *)"lmt", 0, (float )-11188, (char *)"Brazil/East"}, 
        {(char *)"lmt", 0, (float )-11568, (char *)"America/Araguaina"}, 
        {(char *)"lmt", 0, (float )-11636, (char *)"America/Belem"}, 
        {(char *)"lmt", 0, (float )-12416, (char *)"America/Godthab"}, 
        {(char *)"lmt", 0, (float )-12560, (char *)"America/Cayenne"}, 
        {(char *)"lmt", 0, (float )-13108, (char *)"America/Campo_Grande"}, 
        {(char *)"lmt", 0, (float )-13128, (char *)"America/Santarem"}, 
        {(char *)"lmt", 0, (float )-13240, (char *)"America/Paramaribo"}, 
        {(char *)"lmt", 0, (float )-13460, (char *)"America/Cuiaba"}, 
        {(char *)"lmt", 0, (float )-13480, (char *)"America/Miquelon"}, 
        {(char *)"lmt", 0, (float )-13960, (char *)"America/Guyana"}, 
        {(char *)"lmt", 0, (float )-14308, (char *)"America/Barbados"}, 
        {(char *)"lmt", 0, (float )-14388, (char *)"America/Glace_Bay"}, 
        {(char *)"lmt", 0, (float )-14404, (char *)"America/Manaus"}, 
        {(char *)"lmt", 0, (float )-14404, (char *)"Brazil/West"}, 
        {(char *)"lmt", 0, (float )-14560, (char *)"America/Boa_Vista"}, 
        {(char *)"lmt", 0, (float )-14736, (char *)"America/Dominica"}, 
        {(char *)"lmt", 0, (float )-14764, (char *)"America/Port_of_Spain"}, 
        {(char *)"lmt", 0, (float )-14768, (char *)"America/Guadeloupe"}, 
        {(char *)"lmt", 0, (float )-14768, (char *)"America/Marigot"}, 
        {(char *)"lmt", 0, (float )-14768, (char *)"America/St_Barthelemy"}, 
        {(char *)"lmt", 0, (float )-14820, (char *)"America/Grenada"}, 
        {(char *)"lmt", 0, (float )-14832, (char *)"America/Antigua"}, 
        {(char *)"lmt", 0, (float )-14932, (char *)"America/Montserrat"}, 
        {(char *)"lmt", 0, (float )-15052, (char *)"America/St_Kitts"}, 
        {(char *)"lmt", 0, (float )-15136, (char *)"America/Anguilla"}, 
        {(char *)"lmt", 0, (float )-15264, (char *)"America/Halifax"}, 
        {(char *)"lmt", 0, (float )-15264, (char *)"Canada/Atlantic"}, 
        {(char *)"lmt", 0, (float )-15336, (char *)"America/Porto_Velho"}, 
        {(char *)"lmt", 0, (float )-15508, (char *)"America/Tortola"}, 
        {(char *)"lmt", 0, (float )-15544, (char *)"Atlantic/Bermuda"}, 
        {(char *)"lmt", 0, (float )-15584, (char *)"America/St_Thomas"}, 
        {(char *)"lmt", 0, (float )-15584, (char *)"America/Virgin"}, 
        {(char *)"lmt", 0, (float )-1624, (char *)"Atlantic/Faeroe"}, 
        {(char *)"lmt", 0, (float )-1624, (char *)"Atlantic/Faroe"}, 
        {(char *)"lmt", 0, (float )-16272, (char *)"America/Porto_Acre"}, 
        {(char *)"lmt", 0, (float )-16272, (char *)"America/Rio_Branco"}, 
        {(char *)"lmt", 0, (float )-16272, (char *)"Brazil/Acre"}, 
        {(char *)"lmt", 0, (float )-16508, (char *)"America/Thule"}, 
        {(char *)"lmt", 0, (float )-16544, (char *)"America/Curacao"}, 
        {(char *)"lmt", 0, (float )-16768, (char *)"America/Eirunepe"}, 
        {(char *)"lmt", 0, (float )-16824, (char *)"America/Aruba"}, 
        {(char *)"lmt", 0, (float )-1820, (char *)"Africa/Casablanca"}, 
        {(char *)"lmt", 0, (float )-18516, (char *)"America/Lima"}, 
        {(char *)"lmt", 0, (float )-18564, (char *)"America/Nassau"}, 
        {(char *)"lmt", 0, (float )-1920, (char *)"Africa/Bamako"}, 
        {(char *)"lmt", 0, (float )-1920, (char *)"Africa/Timbuktu"}, 
        {(char *)"lmt", 0, (float )-19931, (char *)"America/Detroit"}, 
        {(char *)"lmt", 0, (float )-20824, (char *)"America/Cancun"}, 
        {(char *)"lmt", 0, (float )-20932, (char *)"America/Tegucigalpa"}, 
        {(char *)"lmt", 0, (float )-21168, (char *)"America/Belize"}, 
        {(char *)"lmt", 0, (float )-21408, (char *)"America/El_Salvador"}, 
        {(char *)"lmt", 0, (float )-21504, (char *)"Pacific/Galapagos"}, 
        {(char *)"lmt", 0, (float )-21508, (char *)"America/Merida"}, 
        {(char *)"lmt", 0, (float )-21724, (char *)"America/Guatemala"}, 
        {(char *)"lmt", 0, (float )-2192, (char *)"Africa/Sao_Tome"}, 
        {(char *)"lmt", 0, (float )-2192, (char *)"Europe/Lisbon"}, 
        {(char *)"lmt", 0, (float )-23796, (char *)"America/Mexico_City"}, 
        {(char *)"lmt", 0, (float )-23796, (char *)"Mexico/General"}, 
        {(char *)"lmt", 0, (float )-24000, (char *)"America/Matamoros"}, 
        {(char *)"lmt", 0, (float )-24076, (char *)"America/Monterrey"}, 
        {(char *)"lmt", 0, (float )-25060, (char *)"America/Ojinaga"}, 
        {(char *)"lmt", 0, (float )-25116, (char *)"America/Regina"}, 
        {(char *)"lmt", 0, (float )-25116, (char *)"Canada/East-Saskatchewan"}, 
        {(char *)"lmt", 0, (float )-25116, (char *)"Canada/Saskatchewan"}, 
        {(char *)"lmt", 0, (float )-25260, (char *)"America/Bahia_Banderas"}, 
        {(char *)"lmt", 0, (float )-25460, (char *)"America/Chihuahua"}, 
        {(char *)"lmt", 0, (float )-25540, (char *)"America/Mazatlan"}, 
        {(char *)"lmt", 0, (float )-25540, (char *)"Mexico/BajaSur"}, 
        {(char *)"lmt", 0, (float )-25880, (char *)"America/Swift_Current"}, 
        {(char *)"lmt", 0, (float )-26632, (char *)"America/Hermosillo"}, 
        {(char *)"lmt", 0, (float )-27232, (char *)"America/Edmonton"}, 
        {(char *)"lmt", 0, (float )-27232, (char *)"Canada/Mountain"}, 
        {(char *)"lmt", 0, (float )-27568, (char *)"America/Santa_Isabel"}, 
        {(char *)"lmt", 0, (float )-28084, (char *)"America/Ensenada"}, 
        {(char *)"lmt", 0, (float )-28084, (char *)"America/Tijuana"}, 
        {(char *)"lmt", 0, (float )-28084, (char *)"Mexico/BajaNorte"}, 
        {(char *)"lmt", 0, (float )-3168, (char *)"Africa/El_Aaiun"}, 
        {(char *)"lmt", 0, (float )-32388, (char *)"Pacific/Gambier"}, 
        {(char *)"lmt", 0, (float )-3292, (char *)"Africa/Conakry"}, 
        {(char *)"lmt", 0, (float )-33480, (char *)"Pacific/Marquesas"}, 
        {(char *)"lmt", 0, (float )-35896, (char *)"Pacific/Tahiti"}, 
        {(char *)"lmt", 0, (float )-364, (char *)"Africa/Ouagadougou"}, 
        {(char *)"lmt", 0, (float )-3696, (char *)"Atlantic/Canary"}, 
        {(char *)"lmt", 0, (float )-3740, (char *)"Africa/Bissau"}, 
        {(char *)"lmt", 0, (float )-3828, (char *)"Africa/Nouakchott"}, 
        {(char *)"lmt", 0, (float )-3996, (char *)"Africa/Banjul"}, 
        {(char *)"lmt", 0, (float )-40968, (char *)"Pacific/Pago_Pago"}, 
        {(char *)"lmt", 0, (float )-40968, (char *)"Pacific/Samoa"}, 
        {(char *)"lmt", 0, (float )-41216, (char *)"Pacific/Apia"}, 
        {(char *)"lmt", 0, (float )-4184, (char *)"Africa/Dakar"}, 
        {(char *)"lmt", 0, (float )-4480, (char *)"America/Danmarkshavn"}, 
        {(char *)"lmt", 0, (float )-5272, (char *)"America/Scoresbysund"}, 
        {(char *)"lmt", 0, (float )-52, (char *)"Africa/Accra"}, 
        {(char *)"lmt", 0, (float )-5644, (char *)"Atlantic/Cape_Verde"}, 
        {(char *)"lmt", 0, (float )-7780, (char *)"America/Noronha"}, 
        {(char *)"lmt", 0, (float )-7780, (char *)"Brazil/DeNoronha"}, 
        {(char *)"lmt", 0, (float )-8376, (char *)"America/Recife"}, 
        {(char *)"lmt", 0, (float )-8572, (char *)"America/Maceio"}, 
        {(char *)"lmt", 0, (float )-9240, (char *)"America/Fortaleza"}, 
        {(char *)"lmt", 0, (float )-9244, (char *)"America/Bahia"}, 
        {(char *)"lmt", 0, (float )-968, (char *)"Africa/Abidjan"}, 
        {(char *)"lmt", 0, (float )10356, (char *)"Africa/Djibouti"}, 
        {(char *)"lmt", 0, (float )10384, (char *)"Indian/Comoro"}, 
        {(char *)"lmt", 0, (float )10660, (char *)"Europe/Volgograd"}, 
        {(char *)"lmt", 0, (float )10680, (char *)"Asia/Yerevan"}, 
        {(char *)"lmt", 0, (float )10848, (char *)"Asia/Aden"}, 
        {(char *)"lmt", 0, (float )10856, (char *)"Indian/Mayotte"}, 
        {(char *)"lmt", 0, (float )11212, (char *)"Asia/Riyadh"}, 
        {(char *)"lmt", 0, (float )11404, (char *)"Indian/Antananarivo"}, 
        {(char *)"lmt", 0, (float )11516, (char *)"Asia/Kuwait"}, 
        {(char *)"lmt", 0, (float )11964, (char *)"Asia/Baku"}, 
        {(char *)"lmt", 0, (float )12036, (char *)"Europe/Samara"}, 
        {(char *)"lmt", 0, (float )12064, (char *)"Asia/Aqtau"}, 
        {(char *)"lmt", 0, (float )12140, (char *)"Asia/Bahrain"}, 
        {(char *)"lmt", 0, (float )12324, (char *)"Asia/Oral"}, 
        {(char *)"lmt", 0, (float )12344, (char *)"Asia/Tehran"}, 
        {(char *)"lmt", 0, (float )12368, (char *)"Asia/Qatar"}, 
        {(char *)"lmt", 0, (float )13272, (char *)"Asia/Dubai"}, 
        {(char *)"lmt", 0, (float )13308, (char *)"Indian/Mahe"}, 
        {(char *)"lmt", 0, (float )13312, (char *)"Indian/Reunion"}, 
        {(char *)"lmt", 0, (float )13720, (char *)"Asia/Aqtobe"}, 
        {(char *)"lmt", 0, (float )13800, (char *)"Indian/Mauritius"}, 
        {(char *)"lmt", 0, (float )14012, (char *)"Asia/Ashgabat"}, 
        {(char *)"lmt", 0, (float )14012, (char *)"Asia/Ashkhabad"}, 
        {(char *)"lmt", 0, (float )14060, (char *)"Asia/Muscat"}, 
        {(char *)"lmt", 0, (float )14544, (char *)"Asia/Yekaterinburg"}, 
        {(char *)"lmt", 0, (float )1476, (char *)"Europe/Luxembourg"}, 
        {(char *)"lmt", 0, (float )15712, (char *)"Asia/Qyzylorda"}, 
        {(char *)"lmt", 0, (float )16032, (char *)"Asia/Samarkand"}, 
        {(char *)"lmt", 0, (float )16092, (char *)"Asia/Karachi"}, 
        {(char *)"lmt", 0, (float )16512, (char *)"Asia/Dushanbe"}, 
        {(char *)"lmt", 0, (float )16632, (char *)"Asia/Tashkent"}, 
        {(char *)"lmt", 0, (float )17380, (char *)"Indian/Chagos"}, 
        {(char *)"lmt", 0, (float )17616, (char *)"Asia/Omsk"}, 
        {(char *)"lmt", 0, (float )17904, (char *)"Asia/Bishkek"}, 
        {(char *)"lmt", 0, (float )18236, (char *)"Asia/Kashgar"}, 
        {(char *)"lmt", 0, (float )18468, (char *)"Asia/Almaty"}, 
        {(char *)"lmt", 0, (float )19900, (char *)"Asia/Novosibirsk"}, 
        {(char *)"lmt", 0, (float )20476, (char *)"Asia/Kathmandu"}, 
        {(char *)"lmt", 0, (float )20476, (char *)"Asia/Katmandu"}, 
        {(char *)"lmt", 0, (float )21020, (char *)"Asia/Urumqi"}, 
        {(char *)"lmt", 0, (float )2108, (char *)"Africa/Malabo"}, 
        {(char *)"lmt", 0, (float )21516, (char *)"Asia/Thimbu"}, 
        {(char *)"lmt", 0, (float )21516, (char *)"Asia/Thimphu"}, 
        {(char *)"lmt", 0, (float )21996, (char *)"Asia/Hovd"}, 
        {(char *)"lmt", 0, (float )22280, (char *)"Asia/Krasnoyarsk"}, 
        {(char *)"lmt", 0, (float )2268, (char *)"Africa/Libreville"}, 
        {(char *)"lmt", 0, (float )2328, (char *)"Africa/Douala"}, 
        {(char *)"lmt", 0, (float )24624, (char *)"Asia/Vientiane"}, 
        {(char *)"lmt", 0, (float )25180, (char *)"Asia/Phnom_Penh"}, 
        {(char *)"lmt", 0, (float )25580, (char *)"Asia/Chongqing"}, 
        {(char *)"lmt", 0, (float )25580, (char *)"Asia/Chungking"}, 
        {(char *)"lmt", 0, (float )25600, (char *)"Asia/Ho_Chi_Minh"}, 
        {(char *)"lmt", 0, (float )25600, (char *)"Asia/Saigon"}, 
        {(char *)"lmt", 0, (float )25652, (char *)"Asia/Ulaanbaatar"}, 
        {(char *)"lmt", 0, (float )25652, (char *)"Asia/Ulan_Bator"}, 
        {(char *)"lmt", 0, (float )26240, (char *)"Asia/Pontianak"}, 
        {(char *)"lmt", 0, (float )26480, (char *)"Asia/Kuching"}, 
        {(char *)"lmt", 0, (float )27260, (char *)"Asia/Macao"}, 
        {(char *)"lmt", 0, (float )27260, (char *)"Asia/Macau"}, 
        {(char *)"lmt", 0, (float )27396, (char *)"Asia/Hong_Kong"}, 
        {(char *)"lmt", 0, (float )27480, (char *)"Asia/Choibalsan"}, 
        {(char *)"lmt", 0, (float )27580, (char *)"Asia/Brunei"}, 
        {(char *)"lmt", 0, (float )28656, (char *)"Asia/Makassar"}, 
        {(char *)"lmt", 0, (float )28656, (char *)"Asia/Ujung_Pandang"}, 
        {(char *)"lmt", 0, (float )29152, (char *)"Asia/Shanghai"}, 
        {(char *)"lmt", 0, (float )29152, (char *)"PRC"}, 
        {(char *)"lmt", 0, (float )30140, (char *)"Asia/Dili"}, 
        {(char *)"lmt", 0, (float )30404, (char *)"Asia/Harbin"}, 
        {(char *)"lmt", 0, (float )31120, (char *)"Asia/Yakutsk"}, 
        {(char *)"lmt", 0, (float )3164, (char *)"Africa/Tripoli"}, 
        {(char *)"lmt", 0, (float )31664, (char *)"Asia/Vladivostok"}, 
        {(char *)"lmt", 0, (float )33768, (char *)"Asia/Jayapura"}, 
        {(char *)"lmt", 0, (float )34248, (char *)"Asia/Sakhalin"}, 
        {(char *)"lmt", 0, (float )3612, (char *)"Africa/Ndjamena"}, 
        {(char *)"lmt", 0, (float )36192, (char *)"Asia/Magadan"}, 
        {(char *)"lmt", 0, (float )3668, (char *)"Africa/Brazzaville"}, 
        {(char *)"lmt", 0, (float )38076, (char *)"Asia/Kamchatka"}, 
        {(char *)"lmt", 0, (float )38388, (char *)"Pacific/Guadalcanal"}, 
        {(char *)"lmt", 0, (float )39948, (char *)"Pacific/Noumea"}, 
        {(char *)"lmt", 0, (float )40060, (char *)"Pacific/Nauru"}, 
        {(char *)"lmt", 0, (float )40396, (char *)"Pacific/Efate"}, 
        {(char *)"lmt", 0, (float )42596, (char *)"Asia/Anadyr"}, 
        {(char *)"lmt", 0, (float )42820, (char *)"Pacific/Fiji"}, 
        {(char *)"lmt", 0, (float )44028, (char *)"Pacific/Chatham"}, 
        {(char *)"lmt", 0, (float )4460, (char *)"Africa/Bangui"}, 
        {(char *)"lmt", 0, (float )4760, (char *)"Europe/Tirane"}, 
        {(char *)"lmt", 0, (float )508, (char *)"Africa/Niamey"}, 
        {(char *)"lmt", 0, (float )628, (char *)"Africa/Porto-Novo"}, 
        {(char *)"lmt", 0, (float )6600, (char *)"Africa/Maseru"}, 
        {(char *)"lmt", 0, (float )6788, (char *)"Africa/Lusaka"}, 
        {(char *)"lmt", 0, (float )7216, (char *)"Africa/Kigali"}, 
        {(char *)"lmt", 0, (float )7452, (char *)"Africa/Harare"}, 
        {(char *)"lmt", 0, (float )7464, (char *)"Africa/Mbabane"}, 
        {(char *)"lmt", 0, (float )7780, (char *)"Africa/Kampala"}, 
        {(char *)"lmt", 0, (float )7808, (char *)"Africa/Khartoum"}, 
        {(char *)"lmt", 0, (float )7820, (char *)"Africa/Maputo"}, 
        {(char *)"lmt", 0, (float )8008, (char *)"Asia/Nicosia"}, 
        {(char *)"lmt", 0, (float )8008, (char *)"Europe/Nicosia"}, 
        {(char *)"lmt", 0, (float )816, (char *)"Africa/Lagos"}, 
        {(char *)"lmt", 0, (float )8400, (char *)"Africa/Blantyre"}, 
        {(char *)"lmt", 0, (float )8624, (char *)"Asia/Amman"}, 
        {(char *)"lmt", 0, (float )8712, (char *)"Asia/Damascus"}, 
        {(char *)"lmt", 0, (float )8836, (char *)"Africa/Nairobi"}, 
        {(char *)"lmt", 0, (float )9428, (char *)"Africa/Dar_es_Salaam"}, 
        {(char *)"lont", 0, (float )25200, (char *)"Asia/Chongqing"}, 
        {(char *)"lont", 0, (float )25200, (char *)"Asia/Chungking"}, 
        {(char *)"lrt", 0, (float )-2670, (char *)"Africa/Monrovia"}, 
        {(char *)"lst", 1, (float )9384, (char *)"Europe/Riga"}, 
        {(char *)"madmt", 1, (float )3600, (char *)"Atlantic/Madeira"}, 
        {(char *)"madst", 1, (float )0, (char *)"Atlantic/Madeira"}, 
        {(char *)"madt", 0, (float )-3600, (char *)"Atlantic/Madeira"}, 
        {(char *)"magst", 1, (float )39600, (char *)"Asia/Magadan"}, 
        {(char *)"magst", 1, (float )43200, (char *)"Asia/Magadan"}, 
        {(char *)"magt", 0, (float )36000, (char *)"Asia/Magadan"}, 
        {(char *)"magt", 0, (float )39600, (char *)"Asia/Magadan"}, 
        {(char *)"malst", 1, (float )26400, (char *)"Asia/Singapore"}, 
        {(char *)"malst", 1, (float )26400, (char *)"Asia/Kuala_Lumpur"}, 
        {(char *)"malt", 0, (float )25200, (char *)"Asia/Singapore"}, 
        {(char *)"malt", 0, (float )26400, (char *)"Asia/Singapore"}, 
        {(char *)"malt", 0, (float )27000, (char *)"Asia/Singapore"}, 
        {(char *)"malt", 0, (float )25200, (char *)"Asia/Kuala_Lumpur"}, 
        {(char *)"malt", 0, (float )26400, (char *)"Asia/Kuala_Lumpur"}, 
        {(char *)"malt", 0, (float )27000, (char *)"Asia/Kuala_Lumpur"}, 
        {(char *)"mart", 0, (float )-34200, (char *)"Pacific/Marquesas"}, 
        {(char *)"mawt", 0, (float )18000, (char *)"Antarctica/Mawson"}, 
        {(char *)"mawt", 0, (float )21600, (char *)"Antarctica/Mawson"}, 
        {(char *)"mddt", 1, (float )-18000, (char *)"America/Cambridge_Bay"}, 
        {(char *)"mddt", 1, (float )-18000, (char *)"America/Yellowknife"}, 
        {(char *)"mdst", 1, (float )16248, (char *)"Europe/Moscow"}, 
        {(char *)"mdt", 1, (float )-21600, (char *)"America/Denver"}, 
        {(char *)"mdt", 1, (float )-21600, (char *)"America/Bahia_Banderas"}, 
        {(char *)"mdt", 1, (float )-21600, (char *)"America/Boise"}, 
        {(char *)"mdt", 1, (float )-21600, (char *)"America/Cambridge_Bay"}, 
        {(char *)"mdt", 1, (float )-21600, (char *)"America/Chihuahua"}, 
        {(char *)"mdt", 1, (float )-21600, (char *)"America/Edmonton"}, 
        {(char *)"mdt", 1, (float )-21600, (char *)"America/Hermosillo"}, 
        {(char *)"mdt", 1, (float )-21600, (char *)"America/Inuvik"}, 
        {(char *)"mdt", 1, (float )-21600, (char *)"America/Mazatlan"}, 
        {(char *)"mdt", 1, (float )-21600, (char *)"America/North_Dakota/Center"}, 
        {(char *)"mdt", 1, (float )-21600, (char *)"America/North_Dakota/New_Salem"}, 
        {(char *)"mdt",
      1, (float )-21600, (char *)"America/Ojinaga"}, 
        {(char *)"mdt", 1, (float )-21600, (char *)"America/Phoenix"}, 
        {(char *)"mdt", 1, (float )-21600, (char *)"America/Regina"}, 
        {(char *)"mdt", 1, (float )-21600, (char *)"America/Shiprock"}, 
        {(char *)"mdt", 1, (float )-21600, (char *)"America/Swift_Current"}, 
        {(char *)"mdt", 1, (float )-21600, (char *)"America/Yellowknife"}, 
        {(char *)"mdt", 1, (float )-21600, (char *)"Canada/East-Saskatchewan"}, 
        {(char *)"mdt", 1, (float )-21600, (char *)"Canada/Mountain"}, 
        {(char *)"mdt", 1, (float )-21600, (char *)"Canada/Saskatchewan"}, 
        {(char *)"mdt", 1, (float )-21600, (char *)"Mexico/BajaSur"}, 
        {(char *)"mdt", 1, (float )-21600, (char *)"MST"}, 
        {(char *)"mest", 1, (float )7200, (char *)"MET"}, 
        {(char *)"met", 0, (float )3600, (char *)"MET"}, 
        {(char *)"mht", 0, (float )39600, (char *)"Pacific/Kwajalein"}, 
        {(char *)"mht", 0, (float )43200, (char *)"Pacific/Kwajalein"}, 
        {(char *)"mht", 0, (float )39600, (char *)"Pacific/Majuro"}, 
        {(char *)"mht", 0, (float )43200, (char *)"Pacific/Majuro"}, 
        {(char *)"mist", 0, (float )39600, (char *)"Antarctica/Macquarie"}, 
        {(char *)"mmt", 0, (float )9000, (char *)"Europe/Moscow"}, 
        {(char *)"mmt", 0, (float )9048, (char *)"Europe/Moscow"}, 
        {(char *)"mmt", 0, (float )-13484, (char *)"America/Montevideo"}, 
        {(char *)"mmt", 0, (float )-20712, (char *)"America/Managua"}, 
        {(char *)"mmt", 0, (float )-2588, (char *)"Africa/Monrovia"}, 
        {(char *)"mmt", 0, (float )17640, (char *)"Indian/Maldives"}, 
        {(char *)"mmt", 0, (float )19172, (char *)"Asia/Colombo"}, 
        {(char *)"mmt", 0, (float )23400, (char *)"Asia/Rangoon"}, 
        {(char *)"mmt", 0, (float )28656, (char *)"Asia/Makassar"}, 
        {(char *)"mmt", 0, (float )28656, (char *)"Asia/Ujung_Pandang"}, 
        {(char *)"mmt", 0, (float )6600, (char *)"Europe/Minsk"}, 
        {(char *)"most", 1, (float )32400, (char *)"Asia/Macao"}, 
        {(char *)"most", 1, (float )32400, (char *)"Asia/Macau"}, 
        {(char *)"mot", 0, (float )28800, (char *)"Asia/Macao"}, 
        {(char *)"mot", 0, (float )28800, (char *)"Asia/Macau"}, 
        {(char *)"mpt", 1, (float )-21600, (char *)"America/Denver"}, 
        {(char *)"mpt", 1, (float )-21600, (char *)"America/Boise"}, 
        {(char *)"mpt", 1, (float )-21600, (char *)"America/Cambridge_Bay"}, 
        {(char *)"mpt", 1, (float )-21600, (char *)"America/Edmonton"}, 
        {(char *)"mpt", 1, (float )-21600, (char *)"America/North_Dakota/Center"}, 
        {(char *)"mpt", 1, (float )-21600, (char *)"America/North_Dakota/New_Salem"}, 
        {(char *)"mpt",
      1, (float )-21600, (char *)"America/Regina"}, 
        {(char *)"mpt", 1, (float )-21600, (char *)"America/Shiprock"}, 
        {(char *)"mpt", 1, (float )-21600, (char *)"America/Swift_Current"}, 
        {(char *)"mpt", 1, (float )-21600, (char *)"America/Yellowknife"}, 
        {(char *)"mpt", 1, (float )-21600, (char *)"Canada/East-Saskatchewan"}, 
        {(char *)"mpt", 1, (float )-21600, (char *)"Canada/Mountain"}, 
        {(char *)"mpt", 1, (float )-21600, (char *)"Canada/Saskatchewan"}, 
        {(char *)"mpt", 1, (float )-21600, (char *)"MST"}, 
        {(char *)"mpt", 0, (float )32400, (char *)"Pacific/Saipan"}, 
        {(char *)"mpt", 0, (float )36000, (char *)"Pacific/Saipan"}, 
        {(char *)"msd", 1, (float )14400, (char *)"Europe/Moscow"}, 
        {(char *)"msd", 1, (float )18000, (char *)"Europe/Moscow"}, 
        {(char *)"msd", 1, (float )14400, (char *)"Europe/Chisinau"}, 
        {(char *)"msd", 1, (float )14400, (char *)"Europe/Kaliningrad"}, 
        {(char *)"msd", 1, (float )14400, (char *)"Europe/Kiev"}, 
        {(char *)"msd", 1, (float )14400, (char *)"Europe/Minsk"}, 
        {(char *)"msd", 1, (float )14400, (char *)"Europe/Riga"}, 
        {(char *)"msd", 1, (float )14400, (char *)"Europe/Simferopol"}, 
        {(char *)"msd", 1, (float )14400, (char *)"Europe/Tallinn"}, 
        {(char *)"msd", 1, (float )14400, (char *)"Europe/Tiraspol"}, 
        {(char *)"msd", 1, (float )14400, (char *)"Europe/Uzhgorod"}, 
        {(char *)"msd", 1, (float )14400, (char *)"Europe/Vilnius"}, 
        {(char *)"msd", 1, (float )14400, (char *)"Europe/Zaporozhye"}, 
        {(char *)"msk", 0, (float )10800, (char *)"Europe/Moscow"}, 
        {(char *)"msk", 0, (float )10800, (char *)"Europe/Chisinau"}, 
        {(char *)"msk", 0, (float )10800, (char *)"Europe/Kaliningrad"}, 
        {(char *)"msk", 0, (float )10800, (char *)"Europe/Kiev"}, 
        {(char *)"msk", 0, (float )10800, (char *)"Europe/Minsk"}, 
        {(char *)"msk", 0, (float )10800, (char *)"Europe/Riga"}, 
        {(char *)"msk", 0, (float )10800, (char *)"Europe/Simferopol"}, 
        {(char *)"msk", 0, (float )10800, (char *)"Europe/Tallinn"}, 
        {(char *)"msk", 0, (float )10800, (char *)"Europe/Tiraspol"}, 
        {(char *)"msk", 0, (float )10800, (char *)"Europe/Uzhgorod"}, 
        {(char *)"msk", 0, (float )10800, (char *)"Europe/Vilnius"}, 
        {(char *)"msk", 0, (float )10800, (char *)"Europe/Zaporozhye"}, 
        {(char *)"mst", 0, (float )-25200, (char *)"America/Denver"}, 
        {(char *)"mst", 0, (float )-25200, (char *)"America/Bahia_Banderas"}, 
        {(char *)"mst", 0, (float )-25200, (char *)"America/Boise"}, 
        {(char *)"mst", 0, (float )-25200, (char *)"America/Cambridge_Bay"}, 
        {(char *)"mst", 0, (float )-25200, (char *)"America/Chihuahua"}, 
        {(char *)"mst", 0, (float )-25200, (char *)"America/Dawson_Creek"}, 
        {(char *)"mst", 0, (float )-25200, (char *)"America/Edmonton"}, 
        {(char *)"mst", 0, (float )-25200, (char *)"America/Ensenada"}, 
        {(char *)"mst", 0, (float )-25200, (char *)"America/Hermosillo"}, 
        {(char *)"mst", 0, (float )-25200, (char *)"America/Inuvik"}, 
        {(char *)"mst", 0, (float )-25200, (char *)"America/Mazatlan"}, 
        {(char *)"mst", 0, (float )-25200, (char *)"America/Mexico_City"}, 
        {(char *)"mst", 0, (float )-25200, (char *)"America/North_Dakota/Center"}, 
        {(char *)"mst", 0, (float )-25200, (char *)"America/North_Dakota/New_Salem"}, 
        {(char *)"mst",
      0, (float )-25200, (char *)"America/Ojinaga"}, 
        {(char *)"mst", 0, (float )-25200, (char *)"America/Phoenix"}, 
        {(char *)"mst", 0, (float )-25200, (char *)"America/Regina"}, 
        {(char *)"mst", 0, (float )-25200, (char *)"America/Santa_Isabel"}, 
        {(char *)"mst", 0, (float )-25200, (char *)"America/Shiprock"}, 
        {(char *)"mst", 0, (float )-25200, (char *)"America/Swift_Current"}, 
        {(char *)"mst", 0, (float )-25200, (char *)"America/Tijuana"}, 
        {(char *)"mst", 0, (float )-25200, (char *)"America/Yellowknife"}, 
        {(char *)"mst", 0, (float )-25200, (char *)"Canada/East-Saskatchewan"}, 
        {(char *)"mst", 0, (float )-25200, (char *)"Canada/Mountain"}, 
        {(char *)"mst", 0, (float )-25200, (char *)"Canada/Saskatchewan"}, 
        {(char *)"mst", 0, (float )-25200, (char *)"Mexico/BajaNorte"}, 
        {(char *)"mst", 0, (float )-25200, (char *)"Mexico/BajaSur"}, 
        {(char *)"mst", 0, (float )-25200, (char *)"Mexico/General"}, 
        {(char *)"mst", 0, (float )-25200, (char *)"MST"}, 
        {(char *)"mst", 1, (float )12648, (char *)"Europe/Moscow"}, 
        {(char *)"must", 1, (float )18000, (char *)"Indian/Mauritius"}, 
        {(char *)"mut", 0, (float )14400, (char *)"Indian/Mauritius"}, 
        {(char *)"mvt", 0, (float )18000, (char *)"Indian/Maldives"}, 
        {(char *)"mwt", 1, (float )-21600, (char *)"America/Denver"}, 
        {(char *)"mwt", 1, (float )-21600, (char *)"America/Boise"}, 
        {(char *)"mwt", 1, (float )-21600, (char *)"America/Cambridge_Bay"}, 
        {(char *)"mwt", 1, (float )-21600, (char *)"America/Edmonton"}, 
        {(char *)"mwt", 1, (float )-21600, (char *)"America/North_Dakota/Center"}, 
        {(char *)"mwt", 1, (float )-21600, (char *)"America/North_Dakota/New_Salem"}, 
        {(char *)"mwt",
      1, (float )-21600, (char *)"America/Phoenix"}, 
        {(char *)"mwt", 1, (float )-21600, (char *)"America/Regina"}, 
        {(char *)"mwt", 1, (float )-21600, (char *)"America/Shiprock"}, 
        {(char *)"mwt", 1, (float )-21600, (char *)"America/Swift_Current"}, 
        {(char *)"mwt", 1, (float )-21600, (char *)"America/Yellowknife"}, 
        {(char *)"mwt", 1, (float )-21600, (char *)"Canada/East-Saskatchewan"}, 
        {(char *)"mwt", 1, (float )-21600, (char *)"Canada/Mountain"}, 
        {(char *)"mwt", 1, (float )-21600, (char *)"Canada/Saskatchewan"}, 
        {(char *)"mwt", 1, (float )-21600, (char *)"MST"}, 
        {(char *)"myt", 0, (float )28800, (char *)"Asia/Kuala_Lumpur"}, 
        {(char *)"myt", 0, (float )28800, (char *)"Asia/Kuching"}, 
        {(char *)"ncst", 1, (float )43200, (char *)"Pacific/Noumea"}, 
        {(char *)"nct", 0, (float )39600, (char *)"Pacific/Noumea"}, 
        {(char *)"nddt", 1, (float )-5400, (char *)"America/St_Johns"}, 
        {(char *)"nddt", 1, (float )-5400, (char *)"Canada/Newfoundland"}, 
        {(char *)"ndt", 1, (float )-9000, (char *)"America/St_Johns"}, 
        {(char *)"ndt", 1, (float )-9052, (char *)"America/St_Johns"}, 
        {(char *)"ndt", 1, (float )-36000, (char *)"Pacific/Midway"}, 
        {(char *)"ndt", 1, (float )-9000, (char *)"America/Goose_Bay"}, 
        {(char *)"ndt", 1, (float )-9000, (char *)"Canada/Newfoundland"}, 
        {(char *)"ndt", 1, (float )-9052, (char *)"America/Goose_Bay"}, 
        {(char *)"ndt", 1, (float )-9052, (char *)"Canada/Newfoundland"}, 
        {(char *)"negt", 0, (float )-12600, (char *)"America/Paramaribo"}, 
        {(char *)"nest", 1, (float )4800, (char *)"Europe/Amsterdam"}, 
        {(char *)"net", 0, (float )1200, (char *)"Europe/Amsterdam"}, 
        {(char *)"nft", 0, (float )41400, (char *)"Pacific/Norfolk"}, 
        {(char *)"nmt", 0, (float )20928, (char *)"Asia/Novokuznetsk"}, 
        {(char *)"nmt", 0, (float )40320, (char *)"Pacific/Norfolk"}, 
        {(char *)"novst", 1, (float )25200, (char *)"Asia/Novosibirsk"}, 
        {(char *)"novst", 1, (float )28800, (char *)"Asia/Novosibirsk"}, 
        {(char *)"novst", 1, (float )25200, (char *)"Asia/Novokuznetsk"}, 
        {(char *)"novt", 0, (float )21600, (char *)"Asia/Novosibirsk"}, 
        {(char *)"novt", 0, (float )25200, (char *)"Asia/Novosibirsk"}, 
        {(char *)"novt", 0, (float )21600, (char *)"Asia/Novokuznetsk"}, 
        {(char *)"npt", 1, (float )-9000, (char *)"America/St_Johns"}, 
        {(char *)"npt", 1, (float )-36000, (char *)"America/Adak"}, 
        {(char *)"npt", 1, (float )-36000, (char *)"America/Atka"}, 
        {(char *)"npt", 1, (float )-36000, (char *)"America/Nome"}, 
        {(char *)"npt", 1, (float )-9000, (char *)"America/Goose_Bay"}, 
        {(char *)"npt", 1, (float )-9000, (char *)"Canada/Newfoundland"}, 
        {(char *)"npt", 0, (float )20700, (char *)"Asia/Katmandu"}, 
        {(char *)"npt", 0, (float )20700, (char *)"Asia/Kathmandu"}, 
        {(char *)"nrt", 0, (float )41400, (char *)"Pacific/Nauru"}, 
        {(char *)"nrt", 0, (float )43200, (char *)"Pacific/Nauru"}, 
        {(char *)"nst", 0, (float )-12600, (char *)"America/St_Johns"}, 
        {(char *)"nst", 0, (float )-12652, (char *)"America/St_Johns"}, 
        {(char *)"nst", 0, (float )-12600, (char *)"America/Goose_Bay"}, 
        {(char *)"nst", 0, (float )-12600, (char *)"Canada/Newfoundland"}, 
        {(char *)"nst", 0, (float )-12652, (char *)"America/Goose_Bay"}, 
        {(char *)"nst", 0, (float )-12652, (char *)"Canada/Newfoundland"}, 
        {(char *)"nst", 0, (float )-39600, (char *)"America/Adak"}, 
        {(char *)"nst", 0, (float )-39600, (char *)"America/Atka"}, 
        {(char *)"nst", 0, (float )-39600, (char *)"America/Nome"}, 
        {(char *)"nst", 0, (float )-39600, (char *)"Pacific/Midway"}, 
        {(char *)"nst", 0, (float )-39600, (char *)"Pacific/Pago_Pago"}, 
        {(char *)"nst", 0, (float )-39600, (char *)"Pacific/Samoa"}, 
        {(char *)"nst", 1, (float )4772, (char *)"Europe/Amsterdam"}, 
        {(char *)"nut", 0, (float )-39600, (char *)"Pacific/Niue"}, 
        {(char *)"nut", 0, (float )-40800, (char *)"Pacific/Niue"}, 
        {(char *)"nut", 0, (float )-41400, (char *)"Pacific/Niue"}, 
        {(char *)"nwt", 1, (float )-9000, (char *)"America/St_Johns"}, 
        {(char *)"nwt", 1, (float )-36000, (char *)"America/Adak"}, 
        {(char *)"nwt", 1, (float )-36000, (char *)"America/Atka"}, 
        {(char *)"nwt", 1, (float )-36000, (char *)"America/Nome"}, 
        {(char *)"nwt", 1, (float )-9000, (char *)"America/Goose_Bay"}, 
        {(char *)"nwt", 1, (float )-9000, (char *)"Canada/Newfoundland"}, 
        {(char *)"nzdt", 1, (float )46800, (char *)"Pacific/Auckland"}, 
        {(char *)"nzdt", 1, (float )46800, (char *)"Antarctica/McMurdo"}, 
        {(char *)"nzdt", 1, (float )46800, (char *)"Antarctica/South_Pole"}, 
        {(char *)"nzdt", 1, (float )46800, (char *)"NZ"}, 
        {(char *)"nzmt", 0, (float )41400, (char *)"Pacific/Auckland"}, 
        {(char *)"nzmt", 0, (float )41400, (char *)"NZ"}, 
        {(char *)"nzst", 0, (float )43200, (char *)"Pacific/Auckland"}, 
        {(char *)"nzst", 1, (float )43200, (char *)"Pacific/Auckland"}, 
        {(char *)"nzst", 1, (float )45000, (char *)"Pacific/Auckland"}, 
        {(char *)"nzst", 0, (float )43200, (char *)"Antarctica/McMurdo"}, 
        {(char *)"nzst", 0, (float )43200, (char *)"Antarctica/South_Pole"}, 
        {(char *)"nzst", 0, (float )43200, (char *)"NZ"}, 
        {(char *)"nzst", 1, (float )43200, (char *)"NZ"}, 
        {(char *)"nzst", 1, (float )45000, (char *)"NZ"}, 
        {(char *)"omsst", 1, (float )21600, (char *)"Asia/Omsk"}, 
        {(char *)"omsst", 1, (float )25200, (char *)"Asia/Omsk"}, 
        {(char *)"omst", 0, (float )18000, (char *)"Asia/Omsk"}, 
        {(char *)"omst", 0, (float )21600, (char *)"Asia/Omsk"}, 
        {(char *)"orast", 1, (float )18000, (char *)"Asia/Oral"}, 
        {(char *)"orat", 0, (float )14400, (char *)"Asia/Oral"}, 
        {(char *)"orat", 0, (float )18000, (char *)"Asia/Oral"}, 
        {(char *)"pddt", 1, (float )-21600, (char *)"America/Inuvik"}, 
        {(char *)"pdt", 1, (float )-25200, (char *)"America/Los_Angeles"}, 
        {(char *)"pdt", 1, (float )-25200, (char *)"America/Boise"}, 
        {(char *)"pdt", 1, (float )-25200, (char *)"America/Dawson"}, 
        {(char *)"pdt", 1, (float )-25200, (char *)"America/Dawson_Creek"}, 
        {(char *)"pdt", 1, (float )-25200, (char *)"America/Ensenada"}, 
        {(char *)"pdt", 1, (float )-25200, (char *)"America/Juneau"}, 
        {(char *)"pdt", 1, (float )-25200, (char *)"America/Santa_Isabel"}, 
        {(char *)"pdt", 1, (float )-25200, (char *)"America/Tijuana"}, 
        {(char *)"pdt", 1, (float )-25200, (char *)"America/Vancouver"}, 
        {(char *)"pdt", 1, (float )-25200, (char *)"America/Whitehorse"}, 
        {(char *)"pdt", 1, (float )-25200, (char *)"Canada/Pacific"}, 
        {(char *)"pdt", 1, (float )-25200, (char *)"Canada/Yukon"}, 
        {(char *)"pdt", 1, (float )-25200, (char *)"Mexico/BajaNorte"}, 
        {(char *)"pest", 1, (float )-14400, (char *)"America/Lima"}, 
        {(char *)"petst", 1, (float )43200, (char *)"Asia/Kamchatka"}, 
        {(char *)"petst", 1, (float )46800, (char *)"Asia/Kamchatka"}, 
        {(char *)"pett", 0, (float )39600, (char *)"Asia/Kamchatka"}, 
        {(char *)"pett", 0, (float )43200, (char *)"Asia/Kamchatka"}, 
        {(char *)"pet", 0, (float )-18000, (char *)"America/Lima"}, 
        {(char *)"pgt", 0, (float )36000, (char *)"Pacific/Port_Moresby"}, 
        {(char *)"phot", 0, (float )-39600, (char *)"Pacific/Enderbury"}, 
        {(char *)"phot", 0, (float )-43200, (char *)"Pacific/Enderbury"}, 
        {(char *)"phot", 0, (float )46800, (char *)"Pacific/Enderbury"}, 
        {(char *)"phst", 1, (float )32400, (char *)"Asia/Manila"}, 
        {(char *)"pht", 0, (float )28800, (char *)"Asia/Manila"}, 
        {(char *)"pkst", 1, (float )21600, (char *)"Asia/Karachi"}, 
        {(char *)"pkt", 0, (float )18000, (char *)"Asia/Karachi"}, 
        {(char *)"pmdt", 1, (float )-7200, (char *)"America/Miquelon"}, 
        {(char *)"pmst", 0, (float )-10800, (char *)"America/Miquelon"}, 
        {(char *)"pmt", 0, (float )-13236, (char *)"America/Paramaribo"}, 
        {(char *)"pmt", 0, (float )-13252, (char *)"America/Paramaribo"}, 
        {(char *)"pmt", 0, (float )26240, (char *)"Asia/Pontianak"}, 
        {(char *)"pmt", 0, (float )36000, (char *)"Antarctica/DumontDUrville"}, 
        {(char *)"pmt", 0, (float )561, (char *)"Africa/Algiers"}, 
        {(char *)"pmt", 0, (float )561, (char *)"Africa/Tunis"}, 
        {(char *)"pmt", 0, (float )561, (char *)"Europe/Monaco"}, 
        {(char *)"pmt", 0, (float )561, (char *)"Europe/Paris"}, 
        {(char *)"pmt", 0, (float )561, (char *)"WET"}, 
        {(char *)"pnt", 0, (float )-30600, (char *)"Pacific/Pitcairn"}, 
        {(char *)"pont", 0, (float )39600, (char *)"Pacific/Pohnpei"}, 
        {(char *)"pont", 0, (float )39600, (char *)"Pacific/Ponape"}, 
        {(char *)"ppmt", 0, (float )-17340, (char *)"America/Port-au-Prince"}, 
        {(char *)"ppt", 1, (float )-25200, (char *)"America/Los_Angeles"}, 
        {(char *)"ppt", 1, (float )-25200, (char *)"America/Dawson_Creek"}, 
        {(char *)"ppt", 1, (float )-25200, (char *)"America/Ensenada"}, 
        {(char *)"ppt", 1, (float )-25200, (char *)"America/Juneau"}, 
        {(char *)"ppt", 1, (float )-25200, (char *)"America/Santa_Isabel"}, 
        {(char *)"ppt", 1, (float )-25200, (char *)"America/Tijuana"}, 
        {(char *)"ppt", 1, (float )-25200, (char *)"America/Vancouver"}, 
        {(char *)"ppt", 1, (float )-25200, (char *)"Canada/Pacific"}, 
        {(char *)"ppt", 1, (float )-25200, (char *)"Mexico/BajaNorte"}, 
        {(char *)"pst", 0, (float )-28800, (char *)"America/Los_Angeles"}, 
        {(char *)"pst", 0, (float )-28800, (char *)"America/Bahia_Banderas"}, 
        {(char *)"pst", 0, (float )-28800, (char *)"America/Boise"}, 
        {(char *)"pst", 0, (float )-28800, (char *)"America/Dawson"}, 
        {(char *)"pst", 0, (float )-28800, (char *)"America/Dawson_Creek"}, 
        {(char *)"pst", 0, (float )-28800, (char *)"America/Ensenada"}, 
        {(char *)"pst", 0, (float )-28800, (char *)"America/Hermosillo"}, 
        {(char *)"pst", 0, (float )-28800, (char *)"America/Inuvik"}, 
        {(char *)"pst", 0, (float )-28800, (char *)"America/Juneau"}, 
        {(char *)"pst", 0, (float )-28800, (char *)"America/Mazatlan"}, 
        {(char *)"pst", 0, (float )-28800, (char *)"America/Santa_Isabel"}, 
        {(char *)"pst", 0, (float )-28800, (char *)"America/Tijuana"}, 
        {(char *)"pst", 0, (float )-28800, (char *)"America/Vancouver"}, 
        {(char *)"pst", 0, (float )-28800, (char *)"America/Whitehorse"}, 
        {(char *)"pst", 0, (float )-28800, (char *)"Canada/Pacific"}, 
        {(char *)"pst", 0, (float )-28800, (char *)"Canada/Yukon"}, 
        {(char *)"pst", 0, (float )-28800, (char *)"Mexico/BajaNorte"}, 
        {(char *)"pst", 0, (float )-28800, (char *)"Mexico/BajaSur"}, 
        {(char *)"pst", 0, (float )-28800, (char *)"Pacific/Pitcairn"}, 
        {(char *)"pwt", 1, (float )-25200, (char *)"America/Los_Angeles"}, 
        {(char *)"pwt", 1, (float )-25200, (char *)"America/Dawson_Creek"}, 
        {(char *)"pwt", 1, (float )-25200, (char *)"America/Ensenada"}, 
        {(char *)"pwt", 1, (float )-25200, (char *)"America/Juneau"}, 
        {(char *)"pwt", 1, (float )-25200, (char *)"America/Santa_Isabel"}, 
        {(char *)"pwt", 1, (float )-25200, (char *)"America/Tijuana"}, 
        {(char *)"pwt", 1, (float )-25200, (char *)"America/Vancouver"}, 
        {(char *)"pwt", 1, (float )-25200, (char *)"Canada/Pacific"}, 
        {(char *)"pwt", 1, (float )-25200, (char *)"Mexico/BajaNorte"}, 
        {(char *)"pwt", 0, (float )32400, (char *)"Pacific/Palau"}, 
        {(char *)"pyst", 1, (float )-10800, (char *)"America/Asuncion"}, 
        {(char *)"pyt", 0, (float )-10800, (char *)"America/Asuncion"}, 
        {(char *)"pyt", 0, (float )-14400, (char *)"America/Asuncion"}, 
        {(char *)"qmt", 0, (float )-18840, (char *)"America/Guayaquil"}, 
        {(char *)"qyzst", 1, (float )25200, (char *)"Asia/Qyzylorda"}, 
        {(char *)"qyzt", 0, (float )18000, (char *)"Asia/Qyzylorda"}, 
        {(char *)"qyzt", 0, (float )21600, (char *)"Asia/Qyzylorda"}, 
        {(char *)"ret", 0, (float )14400, (char *)"Indian/Reunion"}, 
        {(char *)"rmt", 0, (float )5784, (char *)"Europe/Riga"}, 
        {(char *)"rmt", 0, (float )-5268, (char *)"Atlantic/Reykjavik"}, 
        {(char *)"rmt", 0, (float )23076, (char *)"Asia/Rangoon"}, 
        {(char *)"rott", 0, (float )-10800, (char *)"Antarctica/Rothera"}, 
        {(char *)"sakst", 1, (float )39600, (char *)"Asia/Sakhalin"}, 
        {(char *)"sakst", 1, (float )43200, (char *)"Asia/Sakhalin"}, 
        {(char *)"sakt", 0, (float )36000, (char *)"Asia/Sakhalin"}, 
        {(char *)"sakt", 0, (float )39600, (char *)"Asia/Sakhalin"}, 
        {(char *)"samst", 1, (float )21600, (char *)"Asia/Samarkand"}, 
        {(char *)"samst", 1, (float )14400, (char *)"Europe/Samara"}, 
        {(char *)"samst", 1, (float )18000, (char *)"Europe/Samara"}, 
        {(char *)"samt", 0, (float )14400, (char *)"Asia/Samarkand"}, 
        {(char *)"samt", 0, (float )18000, (char *)"Asia/Samarkand"}, 
        {(char *)"samt", 0, (float )-41400, (char *)"Pacific/Apia"}, 
        {(char *)"samt", 0, (float )-41400, (char *)"Pacific/Pago_Pago"}, 
        {(char *)"samt", 0, (float )-41400, (char *)"Pacific/Samoa"}, 
        {(char *)"samt", 0, (float )10800, (char *)"Europe/Samara"}, 
        {(char *)"samt", 0, (float )14400, (char *)"Europe/Samara"}, 
        {(char *)"sast", 1, (float )10800, (char *)"Africa/Johannesburg"}, 
        {(char *)"sast", 0, (float )5400, (char *)"Africa/Johannesburg"}, 
        {(char *)"sast", 0, (float )7200, (char *)"Africa/Johannesburg"}, 
        {(char *)"sast", 1, (float )10800, (char *)"Africa/Maseru"}, 
        {(char *)"sast", 1, (float )10800, (char *)"Africa/Windhoek"}, 
        {(char *)"sast", 0, (float )7200, (char *)"Africa/Maseru"}, 
        {(char *)"sast", 0, (float )7200, (char *)"Africa/Mbabane"}, 
        {(char *)"sast", 0, (float )7200, (char *)"Africa/Windhoek"}, 
        {(char *)"sbt", 0, (float )39600, (char *)"Pacific/Guadalcanal"}, 
        {(char *)"sct", 0, (float )14400, (char *)"Indian/Mahe"}, 
        {(char *)"sdmt", 0, (float )-16800, (char *)"America/Santo_Domingo"}, 
        {(char *)"sgt", 0, (float )27000, (char *)"Asia/Singapore"}, 
        {(char *)"sgt", 0, (float )28800, (char *)"Asia/Singapore"}, 
        {(char *)"shest", 1, (float )21600, (char *)"Asia/Aqtau"}, 
        {(char *)"shet", 0, (float )18000, (char *)"Asia/Aqtau"}, 
        {(char *)"shet", 0, (float )21600, (char *)"Asia/Aqtau"}, 
        {(char *)"sjmt", 0, (float )-20180, (char *)"America/Costa_Rica"}, 
        {(char *)"slst", 1, (float )-1200, (char *)"Africa/Freetown"}, 
        {(char *)"slst", 1, (float )3600, (char *)"Africa/Freetown"}, 
        {(char *)"smt", 0, (float )25580, (char *)"Asia/Saigon"}, 
        {(char *)"smt", 0, (float )-13884, (char *)"Atlantic/Stanley"}, 
        {(char *)"smt", 0, (float )-16966, (char *)"America/Santiago"}, 
        {(char *)"smt", 0, (float )-16966, (char *)"Chile/Continental"}, 
        {(char *)"smt", 0, (float )24925, (char *)"Asia/Kuala_Lumpur"}, 
        {(char *)"smt", 0, (float )24925, (char *)"Asia/Singapore"}, 
        {(char *)"smt", 0, (float )25580, (char *)"Asia/Ho_Chi_Minh"}, 
        {(char *)"smt", 0, (float )25580, (char *)"Asia/Phnom_Penh"}, 
        {(char *)"smt", 0, (float )25580, (char *)"Asia/Vientiane"}, 
        {(char *)"smt", 0, (float )8160, (char *)"Europe/Simferopol"}, 
        {(char *)"srt", 0, (float )-10800, (char *)"America/Paramaribo"}, 
        {(char *)"srt", 0, (float )-12600, (char *)"America/Paramaribo"}, 
        {(char *)"sst", 0, (float )-39600, (char *)"Pacific/Samoa"}, 
        {(char *)"sst", 0, (float )-39600, (char *)"Pacific/Midway"}, 
        {(char *)"sst", 0, (float )-39600, (char *)"Pacific/Pago_Pago"}, 
        {(char *)"stat", 0, (float )10800, (char *)"Europe/Volgograd"}, 
        {(char *)"stat", 0, (float )14400, (char *)"Europe/Volgograd"}, 
        {(char *)"svest", 1, (float )18000, (char *)"Asia/Yekaterinburg"}, 
        {(char *)"svest", 1, (float )21600, (char *)"Asia/Yekaterinburg"}, 
        {(char *)"svet", 0, (float )14400, (char *)"Asia/Yekaterinburg"}, 
        {(char *)"svet", 0, (float )18000, (char *)"Asia/Yekaterinburg"}, 
        {(char *)"swat", 0, (float )5400, (char *)"Africa/Windhoek"}, 
        {(char *)"syot", 0, (float )10800, (char *)"Antarctica/Syowa"}, 
        {(char *)"taht", 0, (float )-36000, (char *)"Pacific/Tahiti"}, 
        {(char *)"tasst", 1, (float )25200, (char *)"Asia/Samarkand"}, 
        {(char *)"tasst", 1, (float )21600, (char *)"Asia/Tashkent"}, 
        {(char *)"tasst", 1, (float )25200, (char *)"Asia/Tashkent"}, 
        {(char *)"tast", 0, (float )21600, (char *)"Asia/Samarkand"}, 
        {(char *)"tast", 0, (float )18000, (char *)"Asia/Tashkent"}, 
        {(char *)"tast", 0, (float )21600, (char *)"Asia/Tashkent"}, 
        {(char *)"tbist", 1, (float )14400, (char *)"Asia/Tbilisi"}, 
        {(char *)"tbist", 1, (float )18000, (char *)"Asia/Tbilisi"}, 
        {(char *)"tbit", 0, (float )10800, (char *)"Asia/Tbilisi"}, 
        {(char *)"tbit", 0, (float )14400, (char *)"Asia/Tbilisi"}, 
        {(char *)"tbmt", 0, (float )10756, (char *)"Asia/Tbilisi"}, 
        {(char *)"tft", 0, (float )18000, (char *)"Indian/Kerguelen"}, 
        {(char *)"tjt", 0, (float )18000, (char *)"Asia/Dushanbe"}, 
        {(char *)"tkt", 0, (float )-36000, (char *)"Pacific/Fakaofo"}, 
        {(char *)"tlt", 0, (float )28800, (char *)"Asia/Dili"}, 
        {(char *)"tlt", 0, (float )32400, (char *)"Asia/Dili"}, 
        {(char *)"tmt", 0, (float )12344, (char *)"Asia/Tehran"}, 
        {(char *)"tmt", 0, (float )14400, (char *)"Asia/Ashgabat"}, 
        {(char *)"tmt", 0, (float )14400, (char *)"Asia/Ashkhabad"}, 
        {(char *)"tmt", 0, (float )18000, (char *)"Asia/Ashgabat"}, 
        {(char *)"tmt", 0, (float )18000, (char *)"Asia/Ashkhabad"}, 
        {(char *)"tmt", 0, (float )5940, (char *)"Europe/Tallinn"}, 
        {(char *)"tost", 1, (float )50400, (char *)"Pacific/Tongatapu"}, 
        {(char *)"tot", 0, (float )44400, (char *)"Pacific/Tongatapu"}, 
        {(char *)"tot", 0, (float )46800, (char *)"Pacific/Tongatapu"}, 
        {(char *)"trst", 1, (float )14400, (char *)"Europe/Istanbul"}, 
        {(char *)"trst", 1, (float )14400, (char *)"Asia/Istanbul"}, 
        {(char *)"trt", 0, (float )10800, (char *)"Europe/Istanbul"}, 
        {(char *)"trt", 0, (float )10800, (char *)"Asia/Istanbul"}, 
        {(char *)"tsat", 0, (float )10800, (char *)"Europe/Volgograd"}, 
        {(char *)"tvt", 0, (float )43200, (char *)"Pacific/Funafuti"}, 
        {(char *)"uct", 0, (float )0, (char *)"Etc/UCT"}, 
        {(char *)"uct", 0, (float )0, (char *)"UCT"}, 
        {(char *)"ulast", 1, (float )32400, (char *)"Asia/Ulaanbaatar"}, 
        {(char *)"ulast", 1, (float )32400, (char *)"Asia/Ulan_Bator"}, 
        {(char *)"ulat", 0, (float )25200, (char *)"Asia/Ulaanbaatar"}, 
        {(char *)"ulat", 0, (float )28800, (char *)"Asia/Ulaanbaatar"}, 
        {(char *)"ulat", 0, (float )25200, (char *)"Asia/Choibalsan"}, 
        {(char *)"ulat", 0, (float )25200, (char *)"Asia/Ulan_Bator"}, 
        {(char *)"ulat", 0, (float )28800, (char *)"Asia/Choibalsan"}, 
        {(char *)"ulat", 0, (float )28800, (char *)"Asia/Ulan_Bator"}, 
        {(char *)"urast", 1, (float )18000, (char *)"Asia/Oral"}, 
        {(char *)"urast", 1, (float )21600, (char *)"Asia/Oral"}, 
        {(char *)"urat", 0, (float )14400, (char *)"Asia/Oral"}, 
        {(char *)"urat", 0, (float )18000, (char *)"Asia/Oral"}, 
        {(char *)"urat", 0, (float )21600, (char *)"Asia/Oral"}, 
        {(char *)"urut", 0, (float )21600, (char *)"Asia/Urumqi"}, 
        {(char *)"utc", 0, (float )0, (char *)"Etc/Universal"}, 
        {(char *)"utc", 0, (float )0, (char *)"Etc/UTC"}, 
        {(char *)"utc", 0, (float )0, (char *)"Etc/Zulu"}, 
        {(char *)"utc", 0, (float )0, (char *)"GMT"}, 
        {(char *)"utc", 0, (float )0, (char *)"UTC"}, 
        {(char *)"utc", 0, (float )0, (char *)"UTC"}, 
        {(char *)"uyhst", 1, (float )-10800, (char *)"America/Montevideo"}, 
        {(char *)"uyhst", 1, (float )-9000, (char *)"America/Montevideo"}, 
        {(char *)"uyst", 1, (float )-7200, (char *)"America/Montevideo"}, 
        {(char *)"uyt", 0, (float )-10800, (char *)"America/Montevideo"}, 
        {(char *)"uyt", 0, (float )-12600, (char *)"America/Montevideo"}, 
        {(char *)"uzst", 1, (float )21600, (char *)"Asia/Samarkand"}, 
        {(char *)"uzst", 1, (float )21600, (char *)"Asia/Tashkent"}, 
        {(char *)"uzt", 0, (float )18000, (char *)"Asia/Samarkand"}, 
        {(char *)"uzt", 0, (float )18000, (char *)"Asia/Tashkent"}, 
        {(char *)"vet", 0, (float )-14400, (char *)"America/Caracas"}, 
        {(char *)"vet", 0, (float )-16200, (char *)"America/Caracas"}, 
        {(char *)"vlasst", 1, (float )36000, (char *)"Asia/Vladivostok"}, 
        {(char *)"vlast", 0, (float )32400, (char *)"Asia/Vladivostok"}, 
        {(char *)"vlast", 1, (float )39600, (char *)"Asia/Vladivostok"}, 
        {(char *)"vlat", 0, (float )32400, (char *)"Asia/Vladivostok"}, 
        {(char *)"vlat", 0, (float )36000, (char *)"Asia/Vladivostok"}, 
        {(char *)"volst", 1, (float )14400, (char *)"Europe/Volgograd"}, 
        {(char *)"volst", 1, (float )18000, (char *)"Europe/Volgograd"}, 
        {(char *)"volt", 0, (float )10800, (char *)"Europe/Volgograd"}, 
        {(char *)"volt", 0, (float )14400, (char *)"Europe/Volgograd"}, 
        {(char *)"vost", 0, (float )21600, (char *)"Antarctica/Vostok"}, 
        {(char *)"vust", 1, (float )43200, (char *)"Pacific/Efate"}, 
        {(char *)"vut", 0, (float )39600, (char *)"Pacific/Efate"}, 
        {(char *)"wakt", 0, (float )43200, (char *)"Pacific/Wake"}, 
        {(char *)"warst", 1, (float )-10800, (char *)"America/Mendoza"}, 
        {(char *)"warst", 1, (float )-10800, (char *)"America/Argentina/Jujuy"}, 
        {(char *)"warst", 1, (float )-10800, (char *)"America/Argentina/Mendoza"}, 
        {(char *)"warst", 1, (float )-10800, (char *)"America/Argentina/San_Luis"}, 
        {(char *)"warst", 1, (float )-10800, (char *)"America/Jujuy"}, 
        {(char *)"wart", 0, (float )-14400, (char *)"America/Mendoza"}, 
        {(char *)"wart", 0, (float )-14400, (char *)"America/Argentina/Catamarca"}, 
        {(char *)"wart", 0, (float )-14400,
      (char *)"America/Argentina/ComodRivadavia"}, 
        {(char *)"wart", 0, (float )-14400, (char *)"America/Argentina/Cordoba"}, 
        {(char *)"wart", 0, (float )-14400, (char *)"America/Argentina/Jujuy"}, 
        {(char *)"wart", 0, (float )-14400, (char *)"America/Argentina/La_Rioja"}, 
        {(char *)"wart", 0, (float )-14400, (char *)"America/Argentina/Mendoza"}, 
        {(char *)"wart", 0, (float )-14400,
      (char *)"America/Argentina/Rio_Gallegos"}, 
        {(char *)"wart", 0, (float )-14400, (char *)"America/Argentina/Salta"}, 
        {(char *)"wart", 0, (float )-14400, (char *)"America/Argentina/San_Juan"}, 
        {(char *)"wart", 0, (float )-14400, (char *)"America/Argentina/San_Luis"}, 
        {(char *)"wart", 0, (float )-14400, (char *)"America/Argentina/Tucuman"}, 
        {(char *)"wart", 0, (float )-14400, (char *)"America/Argentina/Ushuaia"}, 
        {(char *)"wart", 0, (float )-14400, (char *)"America/Catamarca"}, 
        {(char *)"wart", 0, (float )-14400, (char *)"America/Cordoba"}, 
        {(char *)"wart", 0, (float )-14400, (char *)"America/Jujuy"}, 
        {(char *)"wart", 0, (float )-14400, (char *)"America/Rosario"}, 
        {(char *)"wast", 1, (float )7200, (char *)"Africa/Windhoek"}, 
        {(char *)"wast", 1, (float )7200, (char *)"Africa/Ndjamena"}, 
        {(char *)"wat", 0, (float )-3600, (char *)"Africa/Dakar"}, 
        {(char *)"wat", 0, (float )-3600, (char *)"Africa/Bamako"}, 
        {(char *)"wat", 0, (float )-3600, (char *)"Africa/Banjul"}, 
        {(char *)"wat", 0, (float )-3600, (char *)"Africa/Bissau"}, 
        {(char *)"wat", 0, (float )-3600, (char *)"Africa/Conakry"}, 
        {(char *)"wat", 0, (float )-3600, (char *)"Africa/El_Aaiun"}, 
        {(char *)"wat", 0, (float )-3600, (char *)"Africa/Freetown"}, 
        {(char *)"wat", 0, (float )-3600, (char *)"Africa/Niamey"}, 
        {(char *)"wat", 0, (float )-3600, (char *)"Africa/Nouakchott"}, 
        {(char *)"wat", 0, (float )-3600, (char *)"Africa/Timbuktu"}, 
        {(char *)"wat", 0, (float )0, (char *)"Africa/Freetown"}, 
        {(char *)"wat", 0, (float )3600, (char *)"Africa/Brazzaville"}, 
        {(char *)"wat", 0, (float )3600, (char *)"Africa/Bangui"}, 
        {(char *)"wat", 0, (float )3600, (char *)"Africa/Douala"}, 
        {(char *)"wat", 0, (float )3600, (char *)"Africa/Kinshasa"}, 
        {(char *)"wat", 0, (float )3600, (char *)"Africa/Lagos"}, 
        {(char *)"wat", 0, (float )3600, (char *)"Africa/Libreville"}, 
        {(char *)"wat", 0, (float )3600, (char *)"Africa/Luanda"}, 
        {(char *)"wat", 0, (float )3600, (char *)"Africa/Malabo"}, 
        {(char *)"wat", 0, (float )3600, (char *)"Africa/Ndjamena"}, 
        {(char *)"wat", 0, (float )3600, (char *)"Africa/Niamey"}, 
        {(char *)"wat", 0, (float )3600, (char *)"Africa/Porto-Novo"}, 
        {(char *)"wat", 0, (float )3600, (char *)"Africa/Windhoek"}, 
        {(char *)"wemt", 1, (float )7200, (char *)"Europe/Lisbon"}, 
        {(char *)"wemt", 1, (float )7200, (char *)"Europe/Madrid"}, 
        {(char *)"wemt", 1, (float )7200, (char *)"Europe/Monaco"}, 
        {(char *)"wemt", 1, (float )7200, (char *)"Europe/Paris"}, 
        {(char *)"wemt", 1, (float )7200, (char *)"WET"}, 
        {(char *)"west", 1, (float )3600, (char *)"Europe/Paris"}, 
        {(char *)"west", 1, (float )3600, (char *)"Africa/Algiers"}, 
        {(char *)"west", 1, (float )3600, (char *)"Africa/Casablanca"}, 
        {(char *)"west", 1, (float )3600, (char *)"Africa/Ceuta"}, 
        {(char *)"west", 1, (float )3600, (char *)"Atlantic/Canary"}, 
        {(char *)"west", 1, (float )3600, (char *)"Atlantic/Faeroe"}, 
        {(char *)"west", 1, (float )3600, (char *)"Atlantic/Faroe"}, 
        {(char *)"west", 1, (float )3600, (char *)"Atlantic/Madeira"}, 
        {(char *)"west", 1, (float )3600, (char *)"Europe/Brussels"}, 
        {(char *)"west", 1, (float )3600, (char *)"Europe/Lisbon"}, 
        {(char *)"west", 1, (float )3600, (char *)"Europe/Luxembourg"}, 
        {(char *)"west", 1, (float )3600, (char *)"Europe/Madrid"}, 
        {(char *)"west", 1, (float )3600, (char *)"Europe/Monaco"}, 
        {(char *)"west", 1, (float )3600, (char *)"WET"}, 
        {(char *)"west", 1, (float )7200, (char *)"Europe/Luxembourg"}, 
        {(char *)"wet", 0, (float )0, (char *)"Europe/Paris"}, 
        {(char *)"wet", 0, (float )0, (char *)"Africa/Algiers"}, 
        {(char *)"wet", 0, (float )0, (char *)"Africa/Casablanca"}, 
        {(char *)"wet", 0, (float )0, (char *)"Africa/Ceuta"}, 
        {(char *)"wet", 0, (float )0, (char *)"Africa/El_Aaiun"}, 
        {(char *)"wet", 0, (float )0, (char *)"Atlantic/Azores"}, 
        {(char *)"wet", 0, (float )0, (char *)"Atlantic/Canary"}, 
        {(char *)"wet", 0, (float )0, (char *)"Atlantic/Faeroe"}, 
        {(char *)"wet", 0, (float )0, (char *)"Atlantic/Faroe"}, 
        {(char *)"wet", 0, (float )0, (char *)"Atlantic/Madeira"}, 
        {(char *)"wet", 0, (float )0, (char *)"Europe/Andorra"}, 
        {(char *)"wet", 0, (float )0, (char *)"Europe/Brussels"}, 
        {(char *)"wet", 0, (float )0, (char *)"Europe/Lisbon"}, 
        {(char *)"wet", 0, (float )0, (char *)"Europe/Luxembourg"}, 
        {(char *)"wet", 0, (float )0, (char *)"Europe/Madrid"}, 
        {(char *)"wet", 0, (float )0, (char *)"Europe/Monaco"}, 
        {(char *)"wet", 0, (float )0, (char *)"WET"}, 
        {(char *)"wet", 0, (float )3600, (char *)"Europe/Luxembourg"}, 
        {(char *)"wft", 0, (float )43200, (char *)"Pacific/Wallis"}, 
        {(char *)"wgst", 1, (float )-7200, (char *)"America/Godthab"}, 
        {(char *)"wgst", 1, (float )-7200, (char *)"America/Danmarkshavn"}, 
        {(char *)"wgt", 0, (float )-10800, (char *)"America/Godthab"}, 
        {(char *)"wgt", 0, (float )-10800, (char *)"America/Danmarkshavn"}, 
        {(char *)"wit", 0, (float )25200, (char *)"Asia/Jakarta"}, 
        {(char *)"wit", 0, (float )27000, (char *)"Asia/Jakarta"}, 
        {(char *)"wit", 0, (float )28800, (char *)"Asia/Jakarta"}, 
        {(char *)"wit", 0, (float )25200, (char *)"Asia/Pontianak"}, 
        {(char *)"wit", 0, (float )27000, (char *)"Asia/Pontianak"}, 
        {(char *)"wit", 0, (float )28800, (char *)"Asia/Pontianak"}, 
        {(char *)"wmt", 0, (float )5040, (char *)"Europe/Vilnius"}, 
        {(char *)"wmt", 0, (float )5040, (char *)"Europe/Warsaw"}, 
        {(char *)"wsdt", 1, (float )-36000, (char *)"Pacific/Apia"}, 
        {(char *)"wst", 0, (float )28800, (char *)"Australia/Perth"}, 
        {(char *)"wst", 1, (float )32400, (char *)"Australia/Perth"}, 
        {(char *)"wst", 0, (float )-39600, (char *)"Pacific/Apia"}, 
        {(char *)"wst", 0, (float )28800, (char *)"Antarctica/Casey"}, 
        {(char *)"wst", 0, (float )28800, (char *)"Australia/West"}, 
        {(char *)"wst", 1, (float )32400, (char *)"Australia/West"}, 
        {(char *)"yakst", 1, (float )32400, (char *)"Asia/Yakutsk"}, 
        {(char *)"yakst", 1, (float )36000, (char *)"Asia/Yakutsk"}, 
        {(char *)"yakt", 0, (float )28800, (char *)"Asia/Yakutsk"}, 
        {(char *)"yakt", 0, (float )32400, (char *)"Asia/Yakutsk"}, 
        {(char *)"yddt", 1, (float )-25200, (char *)"America/Dawson"}, 
        {(char *)"yddt", 1, (float )-25200, (char *)"America/Whitehorse"}, 
        {(char *)"yddt", 1, (float )-25200, (char *)"Canada/Yukon"}, 
        {(char *)"ydt", 1, (float )-28800, (char *)"America/Dawson"}, 
        {(char *)"ydt", 1, (float )-28800, (char *)"America/Whitehorse"}, 
        {(char *)"ydt", 1, (float )-28800, (char *)"America/Yakutat"}, 
        {(char *)"ydt", 1, (float )-28800, (char *)"Canada/Yukon"}, 
        {(char *)"yekst", 1, (float )21600, (char *)"Asia/Yekaterinburg"}, 
        {(char *)"yekt", 0, (float )18000, (char *)"Asia/Yekaterinburg"}, 
        {(char *)"yerst", 1, (float )14400, (char *)"Asia/Yerevan"}, 
        {(char *)"yerst", 1, (float )18000, (char *)"Asia/Yerevan"}, 
        {(char *)"yert", 0, (float )10800, (char *)"Asia/Yerevan"}, 
        {(char *)"yert", 0, (float )14400, (char *)"Asia/Yerevan"}, 
        {(char *)"ypt", 1, (float )-28800, (char *)"America/Dawson"}, 
        {(char *)"ypt", 1, (float )-28800, (char *)"America/Whitehorse"}, 
        {(char *)"ypt", 1, (float )-28800, (char *)"America/Yakutat"}, 
        {(char *)"ypt", 1, (float )-28800, (char *)"Canada/Yukon"}, 
        {(char *)"yst", 0, (float )-32400, (char *)"America/Anchorage"}, 
        {(char *)"yst", 0, (float )-32400, (char *)"America/Dawson"}, 
        {(char *)"yst", 0, (float )-32400, (char *)"America/Juneau"}, 
        {(char *)"yst", 0, (float )-32400, (char *)"America/Nome"}, 
        {(char *)"yst", 0, (float )-32400, (char *)"America/Whitehorse"}, 
        {(char *)"yst", 0, (float )-32400, (char *)"America/Yakutat"}, 
        {(char *)"yst", 0, (float )-32400, (char *)"Canada/Yukon"}, 
        {(char *)"ywt", 1, (float )-28800, (char *)"America/Dawson"}, 
        {(char *)"ywt", 1, (float )-28800, (char *)"America/Whitehorse"}, 
        {(char *)"ywt", 1, (float )-28800, (char *)"America/Yakutat"}, 
        {(char *)"ywt", 1, (float )-28800, (char *)"Canada/Yukon"}, 
        {(char *)"a", 0, (float )3600, (char *)((void *)0)}, 
        {(char *)"b", 0, (float )7200, (char *)((void *)0)}, 
        {(char *)"c", 0, (float )10800, (char *)((void *)0)}, 
        {(char *)"d", 0, (float )14400, (char *)((void *)0)}, 
        {(char *)"e", 0, (float )18000, (char *)((void *)0)}, 
        {(char *)"f", 0, (float )21600, (char *)((void *)0)}, 
        {(char *)"g", 0, (float )25200, (char *)((void *)0)}, 
        {(char *)"h", 0, (float )28800, (char *)((void *)0)}, 
        {(char *)"i", 0, (float )32400, (char *)((void *)0)}, 
        {(char *)"k", 0, (float )36000, (char *)((void *)0)}, 
        {(char *)"l", 0, (float )39600, (char *)((void *)0)}, 
        {(char *)"m", 0, (float )43200, (char *)((void *)0)}, 
        {(char *)"n", 0, (float )-3600, (char *)((void *)0)}, 
        {(char *)"o", 0, (float )-7200, (char *)((void *)0)}, 
        {(char *)"p", 0, (float )-10800, (char *)((void *)0)}, 
        {(char *)"q", 0, (float )-14400, (char *)((void *)0)}, 
        {(char *)"r", 0, (float )-18000, (char *)((void *)0)}, 
        {(char *)"s", 0, (float )-21600, (char *)((void *)0)}, 
        {(char *)"t", 0, (float )-25200, (char *)((void *)0)}, 
        {(char *)"u", 0, (float )-28800, (char *)((void *)0)}, 
        {(char *)"v", 0, (float )-32400, (char *)((void *)0)}, 
        {(char *)"w", 0, (float )-36000, (char *)((void *)0)}, 
        {(char *)"x", 0, (float )-39600, (char *)((void *)0)}, 
        {(char *)"y", 0, (float )-43200, (char *)((void *)0)}, 
        {(char *)"zzz", 0, (float )0, (char *)"Antarctica/Davis"}, 
        {(char *)"zzz", 0, (float )0, (char *)"America/Cambridge_Bay"}, 
        {(char *)"zzz", 0, (float )0, (char *)"America/Inuvik"}, 
        {(char *)"zzz", 0, (float )0, (char *)"America/Iqaluit"}, 
        {(char *)"zzz", 0, (float )0, (char *)"America/Pangnirtung"}, 
        {(char *)"zzz", 0, (float )0, (char *)"America/Rankin_Inlet"}, 
        {(char *)"zzz", 0, (float )0, (char *)"America/Resolute"}, 
        {(char *)"zzz", 0, (float )0, (char *)"America/Yellowknife"}, 
        {(char *)"zzz", 0, (float )0, (char *)"Antarctica/Casey"}, 
        {(char *)"zzz", 0, (float )0, (char *)"Antarctica/DumontDUrville"}, 
        {(char *)"zzz", 0, (float )0, (char *)"Antarctica/Macquarie"}, 
        {(char *)"zzz", 0, (float )0, (char *)"Antarctica/Mawson"}, 
        {(char *)"zzz", 0, (float )0, (char *)"Antarctica/McMurdo"}, 
        {(char *)"zzz", 0, (float )0, (char *)"Antarctica/Palmer"}, 
        {(char *)"zzz", 0, (float )0, (char *)"Antarctica/Rothera"}, 
        {(char *)"zzz", 0, (float )0, (char *)"Antarctica/South_Pole"}, 
        {(char *)"zzz", 0, (float )0, (char *)"Antarctica/Syowa"}, 
        {(char *)"zzz", 0, (float )0, (char *)"Antarctica/Vostok"}, 
        {(char *)"zzz", 0, (float )0, (char *)"Indian/Kerguelen"}, 
        {(char *)"z", 0, (float )0, (char *)((void *)0)}, 
        {(char *)((void *)0), 0, (float )0, (char *)((void *)0)}};
static timelib_tz_lookup_table const   timelib_timezone_fallbackmap[41]  = 
  {      {(char *)"sst", 0, (float )-11, (char *)"Pacific/Apia"}, 
        {(char *)"hst", 0, (float )-10, (char *)"Pacific/Honolulu"}, 
        {(char *)"akst", 0, (float )-9, (char *)"America/Anchorage"}, 
        {(char *)"akdt", 1, (float )-8, (char *)"America/Anchorage"}, 
        {(char *)"pst", 0, (float )-8, (char *)"America/Los_Angeles"}, 
        {(char *)"pdt", 1, (float )-7, (char *)"America/Los_Angeles"}, 
        {(char *)"mst", 0, (float )-7, (char *)"America/Denver"}, 
        {(char *)"mdt", 1, (float )-6, (char *)"America/Denver"}, 
        {(char *)"cst", 0, (float )-6, (char *)"America/Chicago"}, 
        {(char *)"cdt", 1, (float )-5, (char *)"America/Chicago"}, 
        {(char *)"est", 0, (float )-5, (char *)"America/New_York"}, 
        {(char *)"edt", 1, (float )-4, (char *)"America/New_York"}, 
        {(char *)"ast", 0, (float )-4, (char *)"America/Halifax"}, 
        {(char *)"adt", 1, (float )-3, (char *)"America/Halifax"}, 
        {(char *)"brt", 0, (float )-3, (char *)"America/Sao_Paulo"}, 
        {(char *)"brst", 1, (float )-2, (char *)"America/Sao_Paulo"}, 
        {(char *)"azost", 0, (float )-1, (char *)"Atlantic/Azores"}, 
        {(char *)"azodt", 1, (float )0, (char *)"Atlantic/Azores"}, 
        {(char *)"gmt", 0, (float )0, (char *)"Europe/London"}, 
        {(char *)"bst", 1, (float )1, (char *)"Europe/London"}, 
        {(char *)"cet", 0, (float )1, (char *)"Europe/Paris"}, 
        {(char *)"cest", 1, (float )2, (char *)"Europe/Paris"}, 
        {(char *)"eet", 0, (float )2, (char *)"Europe/Helsinki"}, 
        {(char *)"eest", 1, (float )3, (char *)"Europe/Helsinki"}, 
        {(char *)"msk", 0, (float )3, (char *)"Europe/Moscow"}, 
        {(char *)"msd", 1, (float )4, (char *)"Europe/Moscow"}, 
        {(char *)"gst", 0, (float )4, (char *)"Asia/Dubai"}, 
        {(char *)"pkt", 0, (float )5, (char *)"Asia/Karachi"}, 
        {(char *)"ist", 0, (float )5.5, (char *)"Asia/Kolkata"}, 
        {(char *)"npt", 0, (float )5.75, (char *)"Asia/Katmandu"}, 
        {(char *)"yekt", 1, (float )6, (char *)"Asia/Yekaterinburg"}, 
        {(char *)"novst", 1, (float )7, (char *)"Asia/Novosibirsk"}, 
        {(char *)"krat", 0, (float )7, (char *)"Asia/Krasnoyarsk"}, 
        {(char *)"krast", 1, (float )8, (char *)"Asia/Krasnoyarsk"}, 
        {(char *)"jst", 0, (float )9, (char *)"Asia/Tokyo"}, 
        {(char *)"est", 0, (float )10, (char *)"Australia/Melbourne"}, 
        {(char *)"cst", 1, (float )10.5, (char *)"Australia/Adelaide"}, 
        {(char *)"est", 1, (float )11, (char *)"Australia/Melbourne"}, 
        {(char *)"nzst", 0, (float )12, (char *)"Pacific/Auckland"}, 
        {(char *)"nzdt", 1, (float )13, (char *)"Pacific/Auckland"}, 
        {(char *)((void *)0), 0, (float )0, (char *)((void *)0)}};
static timelib_tz_lookup_table const   timelib_timezone_utc[1]  = {      {(char *)"utc", 0, (float )0, (char *)"UTC"}};
static timelib_relunit const   timelib_relunit_lookup[39]  = 
  {      {"sec", 1, 1}, 
        {"secs", 1, 1}, 
        {"second", 1, 1}, 
        {"seconds", 1, 1}, 
        {"min", 2, 1}, 
        {"mins", 2, 1}, 
        {"minute", 2, 1}, 
        {"minutes", 2, 1}, 
        {"hour", 3, 1}, 
        {"hours", 3, 1}, 
        {"day", 4, 1}, 
        {"days", 4, 1}, 
        {"week", 4, 7}, 
        {"weeks", 4, 7}, 
        {"fortnight", 4, 14}, 
        {"fortnights", 4, 14}, 
        {"forthnight", 4, 14}, 
        {"forthnights", 4, 14}, 
        {"month", 5, 1}, 
        {"months", 5, 1}, 
        {"year", 6, 1}, 
        {"years", 6, 1}, 
        {"monday", 7, 1}, 
        {"mon", 7, 1}, 
        {"tuesday", 7, 2}, 
        {"tue", 7, 2}, 
        {"wednesday", 7, 3}, 
        {"wed", 7, 3}, 
        {"thursday", 7, 4}, 
        {"thu", 7, 4}, 
        {"friday", 7, 5}, 
        {"fri", 7, 5}, 
        {"saturday", 7, 6}, 
        {"sat", 7, 6}, 
        {"sunday", 7, 0}, 
        {"sun", 7, 0}, 
        {"weekday", 8, 1}, 
        {"weekdays", 8, 1}, 
        {(char const   *)((void *)0), 0, 0}};
static timelib_lookup_table const   timelib_reltext_lookup[18]  = 
  {      {"first", 0, 1}, 
        {"next", 0, 1}, 
        {"second", 0, 2}, 
        {"third", 0, 3}, 
        {"fourth", 0, 4}, 
        {"fifth", 0, 5}, 
        {"sixth", 0, 6}, 
        {"seventh", 0, 7}, 
        {"eight", 0, 8}, 
        {"eighth", 0, 8}, 
        {"ninth", 0, 9}, 
        {"tenth", 0, 10}, 
        {"eleventh", 0, 11}, 
        {"twelfth", 0, 12}, 
        {"last", 0, -1}, 
        {"previous", 0, -1}, 
        {"this", 1, 0}, 
        {(char const   *)((void *)0), 1, 0}};
static timelib_lookup_table const   timelib_month_lookup[38]  = 
  {      {"jan", 0, 1}, 
        {"feb", 0, 2}, 
        {"mar", 0, 3}, 
        {"apr", 0, 4}, 
        {"may", 0, 5}, 
        {"jun", 0, 6}, 
        {"jul", 0, 7}, 
        {"aug", 0, 8}, 
        {"sep", 0, 9}, 
        {"sept", 0, 9}, 
        {"oct", 0, 10}, 
        {"nov", 0, 11}, 
        {"dec", 0, 12}, 
        {"i", 0, 1}, 
        {"ii", 0, 2}, 
        {"iii", 0, 3}, 
        {"iv", 0, 4}, 
        {"v", 0, 5}, 
        {"vi", 0, 6}, 
        {"vii", 0, 7}, 
        {"viii", 0, 8}, 
        {"ix", 0, 9}, 
        {"x", 0, 10}, 
        {"xi", 0, 11}, 
        {"xii", 0, 12}, 
        {"january", 0, 1}, 
        {"february", 0, 2}, 
        {"march", 0, 3}, 
        {"april", 0, 4}, 
        {"may", 0, 5}, 
        {"june", 0, 6}, 
        {"july", 0, 7}, 
        {"august", 0, 8}, 
        {"september", 0, 9}, 
        {"october", 0, 10}, 
        {"november", 0, 11}, 
        {"december", 0, 12}, 
        {(char const   *)((void *)0), 0, 0}};
static void add_warning(Scanner *s , char *error ) 
{ 
  void *tmp ;
  void *tmp___0 ;
  size_t __len ;
  size_t tmp___1 ;
  char *__retval ;
  void *tmp___2 ;
  void *tmp___3 ;
  char *tmp___4 ;
  char *tmp___5 ;

  {
  ((s->errors)->warning_count) ++;
  tmp = realloc((void *)(s->errors)->warning_messages,
                (unsigned int )(s->errors)->warning_count * sizeof(timelib_error_message ));
  (s->errors)->warning_messages = (struct timelib_error_message *)tmp;
  if (s->tok) {
    ((s->errors)->warning_messages + ((s->errors)->warning_count - 1))->position = s->tok - s->str;
  } else {
    ((s->errors)->warning_messages + ((s->errors)->warning_count - 1))->position = 0;
  }
  if (s->tok) {
    ((s->errors)->warning_messages + ((s->errors)->warning_count - 1))->character = (char )*(s->tok);
  } else {
    ((s->errors)->warning_messages + ((s->errors)->warning_count - 1))->character = (char)0;
  }
  tmp___5 = __strdup((char const   *)error);
  ((s->errors)->warning_messages + ((s->errors)->warning_count - 1))->message = tmp___5;
  return;
}
}
static void add_error(Scanner *s , char *error ) 
{ 
  void *tmp ;
  void *tmp___0 ;
  size_t __len ;
  size_t tmp___1 ;
  char *__retval ;
  void *tmp___2 ;
  void *tmp___3 ;
  char *tmp___4 ;
  char *tmp___5 ;

  {
  ((s->errors)->error_count) ++;
  tmp = realloc((void *)(s->errors)->error_messages,
                (unsigned int )(s->errors)->error_count * sizeof(timelib_error_message ));
  (s->errors)->error_messages = (struct timelib_error_message *)tmp;
  if (s->tok) {
    ((s->errors)->error_messages + ((s->errors)->error_count - 1))->position = s->tok - s->str;
  } else {
    ((s->errors)->error_messages + ((s->errors)->error_count - 1))->position = 0;
  }
  if (s->tok) {
    ((s->errors)->error_messages + ((s->errors)->error_count - 1))->character = (char )*(s->tok);
  } else {
    ((s->errors)->error_messages + ((s->errors)->error_count - 1))->character = (char)0;
  }
  tmp___5 = __strdup((char const   *)error);
  ((s->errors)->error_messages + ((s->errors)->error_count - 1))->message = tmp___5;
  return;
}
}
static void add_pbf_warning(Scanner *s , char *error , char *sptr , char *cptr ) 
{ 
  void *tmp ;
  void *tmp___0 ;
  size_t __len ;
  size_t tmp___1 ;
  char *__retval ;
  void *tmp___2 ;
  void *tmp___3 ;
  char *tmp___4 ;
  char *tmp___5 ;

  {
  ((s->errors)->warning_count) ++;
  tmp = realloc((void *)(s->errors)->warning_messages,
                (unsigned int )(s->errors)->warning_count * sizeof(timelib_error_message ));
  (s->errors)->warning_messages = (struct timelib_error_message *)tmp;
  ((s->errors)->warning_messages + ((s->errors)->warning_count - 1))->position = cptr - sptr;
  ((s->errors)->warning_messages + ((s->errors)->warning_count - 1))->character = *cptr;
  tmp___5 = __strdup((char const   *)error);
  ((s->errors)->warning_messages + ((s->errors)->warning_count - 1))->message = tmp___5;
  return;
}
}
static void add_pbf_error(Scanner *s , char *error , char *sptr , char *cptr ) 
{ 
  void *tmp ;
  void *tmp___0 ;
  size_t __len ;
  size_t tmp___1 ;
  char *__retval ;
  void *tmp___2 ;
  void *tmp___3 ;
  char *tmp___4 ;
  char *tmp___5 ;

  {
  ((s->errors)->error_count) ++;
  tmp = realloc((void *)(s->errors)->error_messages,
                (unsigned int )(s->errors)->error_count * sizeof(timelib_error_message ));
  (s->errors)->error_messages = (struct timelib_error_message *)tmp;
  ((s->errors)->error_messages + ((s->errors)->error_count - 1))->position = cptr - sptr;
  ((s->errors)->error_messages + ((s->errors)->error_count - 1))->character = *cptr;
  tmp___5 = __strdup((char const   *)error);
  ((s->errors)->error_messages + ((s->errors)->error_count - 1))->message = tmp___5;
  return;
}
}
static timelib_sll timelib_meridian(char **ptr , timelib_sll h ) 
{ 
  timelib_sll retval ;
  void *tmp ;
  char *tmp___0 ;

  {
  retval = (timelib_sll )0;
  while (1) {
    tmp___0 = __builtin_strchr((char *)"AaPp", (int )*(*ptr));
    if (tmp___0) {
      break;
    } else {

    }
    (*ptr) ++;
  }
  if ((int )*(*ptr) == 97) {
    goto _L;
  } else
  if ((int )*(*ptr) == 65) {
    _L: /* CIL Label */ 
    if (h == 12LL) {
      retval = (timelib_sll )-12;
    } else {

    }
  } else
  if (h != 12LL) {
    retval = (timelib_sll )12;
  } else {

  }
  (*ptr) ++;
  if ((int )*(*ptr) == 46) {
    *ptr += 3;
  } else {
    (*ptr) ++;
  }
  return (retval);
}
}
static timelib_sll timelib_meridian_with_check(char **ptr , timelib_sll h ) 
{ 
  timelib_sll retval ;
  void *tmp ;
  char *tmp___0 ;

  {
  retval = (timelib_sll )0;
  while (1) {
    tmp___0 = __builtin_strchr((char *)"AaPp", (int )*(*ptr));
    if (tmp___0) {
      break;
    } else {

    }
    (*ptr) ++;
  }
  if ((int )*(*ptr) == 97) {
    goto _L;
  } else
  if ((int )*(*ptr) == 65) {
    _L: /* CIL Label */ 
    if (h == 12LL) {
      retval = (timelib_sll )-12;
    } else {

    }
  } else
  if (h != 12LL) {
    retval = (timelib_sll )12;
  } else {

  }
  (*ptr) ++;
  if ((int )*(*ptr) == 46) {
    (*ptr) ++;
    if ((int )*(*ptr) != 109) {
      if ((int )*(*ptr) != 77) {
        return ((timelib_sll )-99999);
      } else {

      }
    } else {

    }
    (*ptr) ++;
    if ((int )*(*ptr) != 46) {
      return ((timelib_sll )-99999);
    } else {

    }
    (*ptr) ++;
  } else
  if ((int )*(*ptr) == 109) {
    (*ptr) ++;
  } else
  if ((int )*(*ptr) == 77) {
    (*ptr) ++;
  } else {
    return ((timelib_sll )-99999);
  }
  return (retval);
}
}
static char *timelib_string(Scanner *s ) 
{ 
  char *tmp ;
  void *tmp___0 ;

  {
  tmp___0 = calloc((size_t )1, (size_t )((s->cur - s->tok) + 1));
  tmp = (char *)tmp___0;
  memcpy((void */* __restrict  */)tmp, (void const   */* __restrict  */)s->tok,
         (size_t )(s->cur - s->tok));
  return (tmp);
}
}
static timelib_sll timelib_get_nr(char **ptr , int max_length ) 
{ 
  char *begin ;
  char *end ;
  char *str ;
  timelib_sll tmp_nr ;
  int len ;
  void *tmp ;

  {
  tmp_nr = (timelib_sll )-99999;
  len = 0;
  while (1) {
    if ((int )*(*ptr) < 48) {

    } else
    if ((int )*(*ptr) > 57) {

    } else {
      break;
    }
    if ((int )*(*ptr) == 0) {
      return ((timelib_sll )-99999);
    } else {

    }
    (*ptr) ++;
  }
  begin = *ptr;
  while (1) {
    if ((int )*(*ptr) >= 48) {
      if ((int )*(*ptr) <= 57) {
        if (len < max_length) {

        } else {
          break;
        }
      } else {
        break;
      }
    } else {
      break;
    }
    (*ptr) ++;
    len ++;
  }
  end = *ptr;
  tmp = calloc((size_t )1, (size_t )((end - begin) + 1));
  str = (char *)tmp;
  memcpy((void */* __restrict  */)str, (void const   */* __restrict  */)begin,
         (size_t )(end - begin));
  tmp_nr = strtoll((char const   */* __restrict  */)str,
                   (char **/* __restrict  */)((void *)0), 10);
  free((void *)str);
  return (tmp_nr);
}
}
static void timelib_skip_day_suffix(char **ptr ) 
{ 
  unsigned short const   **tmp ;
  int tmp___0 ;
  int tmp___1 ;
  int tmp___2 ;
  int tmp___3 ;

  {
  tmp = __ctype_b_loc();
  if ((int const   )*(*tmp + (int )*(*ptr)) & 8192) {
    return;
  } else {

  }
  tmp___0 = strncasecmp((char const   *)*ptr, "nd", (size_t )2);
  if (tmp___0) {
    tmp___1 = strncasecmp((char const   *)*ptr, "rd", (size_t )2);
    if (tmp___1) {
      tmp___2 = strncasecmp((char const   *)*ptr, "st", (size_t )2);
      if (tmp___2) {
        tmp___3 = strncasecmp((char const   *)*ptr, "th", (size_t )2);
        if (tmp___3) {

        } else {
          *ptr += 2;
        }
      } else {
        *ptr += 2;
      }
    } else {
      *ptr += 2;
    }
  } else {
    *ptr += 2;
  }
  return;
}
}
static double timelib_get_frac_nr(char **ptr , int max_length ) 
{ 
  char *begin ;
  char *end ;
  char *str ;
  double tmp_nr ;
  int len ;
  void *tmp ;

  {
  tmp_nr = (double )-99999;
  len = 0;
  while (1) {
    if ((int )*(*ptr) != 46) {
      if ((int )*(*ptr) != 58) {
        if ((int )*(*ptr) < 48) {

        } else
        if ((int )*(*ptr) > 57) {

        } else {
          break;
        }
      } else {
        break;
      }
    } else {
      break;
    }
    if ((int )*(*ptr) == 0) {
      return ((double )-99999);
    } else {

    }
    (*ptr) ++;
  }
  begin = *ptr;
  while (1) {
    if ((int )*(*ptr) == 46) {
      goto _L;
    } else
    if ((int )*(*ptr) == 58) {
      goto _L;
    } else
    if ((int )*(*ptr) >= 48) {
      if ((int )*(*ptr) <= 57) {
        _L: /* CIL Label */ 
        if (len < max_length) {

        } else {
          break;
        }
      } else {
        break;
      }
    } else {
      break;
    }
    (*ptr) ++;
    len ++;
  }
  end = *ptr;
  tmp = calloc((size_t )1, (size_t )((end - begin) + 1));
  str = (char *)tmp;
  memcpy((void */* __restrict  */)str, (void const   */* __restrict  */)begin,
         (size_t )(end - begin));
  if ((int )*(str + 0) == 58) {
    *(str + 0) = (char )'.';
  } else {

  }
  tmp_nr = strtod((char const   */* __restrict  */)str,
                  (char **/* __restrict  */)((void *)0));
  free((void *)str);
  return (tmp_nr);
}
}
static timelib_ull timelib_get_unsigned_nr(char **ptr , int max_length ) 
{ 
  timelib_ull dir ;
  timelib_sll tmp ;

  {
  dir = (timelib_ull )1;
  while (1) {
    if ((int )*(*ptr) < 48) {
      goto _L;
    } else
    if ((int )*(*ptr) > 57) {
      _L: /* CIL Label */ 
      if ((int )*(*ptr) != 43) {
        if ((int )*(*ptr) != 45) {

        } else {
          break;
        }
      } else {
        break;
      }
    } else {
      break;
    }
    if ((int )*(*ptr) == 0) {
      return ((timelib_ull )-99999);
    } else {

    }
    (*ptr) ++;
  }
  while (1) {
    if ((int )*(*ptr) == 43) {

    } else
    if ((int )*(*ptr) == 45) {

    } else {
      break;
    }
    if ((int )*(*ptr) == 45) {
      dir *= 0xffffffffffffffffULL;
    } else {

    }
    (*ptr) ++;
  }
  tmp = timelib_get_nr(ptr, max_length);
  return (dir * (timelib_ull )tmp);
}
}
static long timelib_parse_tz_cor(char **ptr ) 
{ 
  char *begin ;
  char *end ;
  long tmp ;
  unsigned short const   **tmp___0 ;
  long tmp___1 ;
  long tmp___2 ;
  long tmp___3 ;
  long tmp___4 ;
  long tmp___5 ;
  long tmp___6 ;
  long tmp___7 ;

  {
  begin = *ptr;
  while (1) {
    tmp___0 = __ctype_b_loc();
    if ((int const   )*(*tmp___0 + (int )*(*ptr)) & 2048) {

    } else
    if ((int )*(*ptr) == 58) {

    } else {
      break;
    }
    (*ptr) ++;
  }
  end = *ptr;
  switch (end - begin) {
  case 2: 
  case 1: 
  tmp___1 = strtol((char const   */* __restrict  */)begin,
                   (char **/* __restrict  */)((void *)0), 10);
  return ((long )((int )(tmp___1 * 60L)));
  break;
  case 4: 
  case 3: 
  if ((int )*(begin + 1) == 58) {
    tmp___2 = strtol((char const   */* __restrict  */)begin,
                     (char **/* __restrict  */)((void *)0), 10);
    tmp___3 = strtol((char const   */* __restrict  */)(begin + 2),
                     (char **/* __restrict  */)((void *)0), 10);
    tmp = (long )((int )(tmp___2 * 60L)) + tmp___3;
    return (tmp);
  } else
  if ((int )*(begin + 2) == 58) {
    tmp___4 = strtol((char const   */* __restrict  */)begin,
                     (char **/* __restrict  */)((void *)0), 10);
    tmp___5 = strtol((char const   */* __restrict  */)(begin + 3),
                     (char **/* __restrict  */)((void *)0), 10);
    tmp = (long )((int )(tmp___4 * 60L)) + tmp___5;
    return (tmp);
  } else {
    tmp = strtol((char const   */* __restrict  */)begin,
                 (char **/* __restrict  */)((void *)0), 10);
    return ((long )((int )((tmp / 100L) * 60L)) + tmp % 100L);
  }
  case 5: 
  tmp___6 = strtol((char const   */* __restrict  */)begin,
                   (char **/* __restrict  */)((void *)0), 10);
  tmp___7 = strtol((char const   */* __restrict  */)(begin + 3),
                   (char **/* __restrict  */)((void *)0), 10);
  tmp = (long )((int )(tmp___6 * 60L)) + tmp___7;
  return (tmp);
  }
  return (0L);
}
}
static timelib_sll timelib_lookup_relative_text(char **ptr , int *behavior ) 
{ 
  char *word ;
  char *begin ;
  char *end ;
  timelib_sll value ;
  timelib_lookup_table const   *tp ;
  void *tmp ;
  int tmp___0 ;

  {
  begin = *ptr;
  value = (timelib_sll )0;
  while (1) {
    if ((int )*(*ptr) >= 65) {
      if ((int )*(*ptr) <= 90) {

      } else {
        goto _L;
      }
    } else
    _L: /* CIL Label */ 
    if ((int )*(*ptr) >= 97) {
      if ((int )*(*ptr) <= 122) {

      } else {
        break;
      }
    } else {
      break;
    }
    (*ptr) ++;
  }
  end = *ptr;
  tmp = calloc((size_t )1, (size_t )((end - begin) + 1));
  word = (char *)tmp;
  memcpy((void */* __restrict  */)word, (void const   */* __restrict  */)begin,
         (size_t )(end - begin));
  tp = timelib_reltext_lookup;
  while (tp->name) {
    tmp___0 = strcasecmp((char const   *)word, (char const   *)tp->name);
    if (tmp___0 == 0) {
      value = (timelib_sll )tp->value;
      *behavior = (int )tp->type;
    } else {

    }
    tp ++;
  }
  free((void *)word);
  return (value);
}
}
static timelib_sll timelib_get_relative_text(char **ptr , int *behavior ) 
{ 
  timelib_sll tmp ;

  {
  while (1) {
    if ((int )*(*ptr) == 32) {

    } else
    if ((int )*(*ptr) == 9) {

    } else
    if ((int )*(*ptr) == 45) {

    } else
    if ((int )*(*ptr) == 47) {

    } else {
      break;
    }
    (*ptr) ++;
  }
  tmp = timelib_lookup_relative_text(ptr, behavior);
  return (tmp);
}
}
static long timelib_lookup_month(char **ptr ) 
{ 
  char *word ;
  char *begin ;
  char *end ;
  long value ;
  timelib_lookup_table const   *tp ;
  void *tmp ;
  int tmp___0 ;

  {
  begin = *ptr;
  value = 0L;
  while (1) {
    if ((int )*(*ptr) >= 65) {
      if ((int )*(*ptr) <= 90) {

      } else {
        goto _L;
      }
    } else
    _L: /* CIL Label */ 
    if ((int )*(*ptr) >= 97) {
      if ((int )*(*ptr) <= 122) {

      } else {
        break;
      }
    } else {
      break;
    }
    (*ptr) ++;
  }
  end = *ptr;
  tmp = calloc((size_t )1, (size_t )((end - begin) + 1));
  word = (char *)tmp;
  memcpy((void */* __restrict  */)word, (void const   */* __restrict  */)begin,
         (size_t )(end - begin));
  tp = timelib_month_lookup;
  while (tp->name) {
    tmp___0 = strcasecmp((char const   *)word, (char const   *)tp->name);
    if (tmp___0 == 0) {
      value = (long )tp->value;
    } else {

    }
    tp ++;
  }
  free((void *)word);
  return (value);
}
}
static long timelib_get_month(char **ptr ) 
{ 
  long tmp ;

  {
  while (1) {
    if ((int )*(*ptr) == 32) {

    } else
    if ((int )*(*ptr) == 9) {

    } else
    if ((int )*(*ptr) == 45) {

    } else
    if ((int )*(*ptr) == 46) {

    } else
    if ((int )*(*ptr) == 47) {

    } else {
      break;
    }
    (*ptr) ++;
  }
  tmp = timelib_lookup_month(ptr);
  return (tmp);
}
}
static void timelib_eat_spaces(char **ptr ) 
{ 


  {
  while (1) {
    if ((int )*(*ptr) == 32) {

    } else
    if ((int )*(*ptr) == 9) {

    } else {
      break;
    }
    (*ptr) ++;
  }
  return;
}
}
static void timelib_eat_until_separator(char **ptr ) 
{ 
  void *tmp ;
  char *tmp___0 ;

  {
  (*ptr) ++;
  while (1) {
    tmp___0 = __builtin_strchr((char *)" \t.,:;/-0123456789", (int )*(*ptr));
    if ((unsigned int )tmp___0 == (unsigned int )((void *)0)) {

    } else {
      break;
    }
    (*ptr) ++;
  }
  return;
}
}
static timelib_relunit const   *timelib_lookup_relunit(char **ptr ) 
{ 
  char *word ;
  char *begin ;
  char *end ;
  timelib_relunit const   *tp ;
  timelib_relunit const   *value ;
  void *tmp ;
  int tmp___0 ;

  {
  begin = *ptr;
  value = (timelib_relunit const   *)((void *)0);
  while (1) {
    if ((int )*(*ptr) != 0) {
      if ((int )*(*ptr) != 32) {
        if ((int )*(*ptr) != 44) {
          if ((int )*(*ptr) != 9) {

          } else {
            break;
          }
        } else {
          break;
        }
      } else {
        break;
      }
    } else {
      break;
    }
    (*ptr) ++;
  }
  end = *ptr;
  tmp = calloc((size_t )1, (size_t )((end - begin) + 1));
  word = (char *)tmp;
  memcpy((void */* __restrict  */)word, (void const   */* __restrict  */)begin,
         (size_t )(end - begin));
  tp = timelib_relunit_lookup;
  while (tp->name) {
    tmp___0 = strcasecmp((char const   *)word, (char const   *)tp->name);
    if (tmp___0 == 0) {
      value = tp;
      break;
    } else {

    }
    tp ++;
  }
  free((void *)word);
  return (value);
}
}
static void timelib_set_relative(char **ptr , timelib_sll amount ,
                                 int behavior , Scanner *s ) 
{ 
  timelib_relunit const   *relunit ;
  timelib_sll tmp ;

  {
  relunit = timelib_lookup_relunit(ptr);
  if (relunit) {

  } else {
    return;
  }
  switch (relunit->unit) {
  case 1: 
  (s->time)->relative.s += amount * (timelib_sll )relunit->multiplier;
  break;
  case 2: 
  (s->time)->relative.i += amount * (timelib_sll )relunit->multiplier;
  break;
  case 3: 
  (s->time)->relative.h += amount * (timelib_sll )relunit->multiplier;
  break;
  case 4: 
  (s->time)->relative.d += amount * (timelib_sll )relunit->multiplier;
  break;
  case 5: 
  (s->time)->relative.m += amount * (timelib_sll )relunit->multiplier;
  break;
  case 6: 
  (s->time)->relative.y += amount * (timelib_sll )relunit->multiplier;
  break;
  case 7: 
  (s->time)->have_relative = 1U;
  (s->time)->relative.have_weekday_relative = 1U;
  (s->time)->have_time = 0U;
  (s->time)->h = (timelib_sll )0;
  (s->time)->i = (timelib_sll )0;
  (s->time)->s = (timelib_sll )0;
  (s->time)->f = (double )0;
  if (amount > 0LL) {
    tmp = amount - 1LL;
  } else {
    tmp = amount;
  }
  (s->time)->relative.d += tmp * 7LL;
  (s->time)->relative.weekday = (int )relunit->multiplier;
  (s->time)->relative.weekday_behavior = behavior;
  break;
  case 8: 
  (s->time)->have_relative = 1U;
  (s->time)->relative.have_special_relative = 1U;
  (s->time)->have_time = 0U;
  (s->time)->h = (timelib_sll )0;
  (s->time)->i = (timelib_sll )0;
  (s->time)->s = (timelib_sll )0;
  (s->time)->f = (double )0;
  (s->time)->relative.special.type = (unsigned int )relunit->multiplier;
  (s->time)->relative.special.amount = amount;
  }
  return;
}
}
static timelib_tz_lookup_table const   *zone_search(char const   *word ,
                                                    long gmtoffset , int isdst ) 
{ 
  int first_found ;
  timelib_tz_lookup_table const   *tp ;
  timelib_tz_lookup_table const   *first_found_elem ;
  timelib_tz_lookup_table const   *fmp ;
  int tmp ;
  int tmp___0 ;
  int tmp___1 ;
  int tmp___2 ;

  {
  first_found = 0;
  first_found_elem = (timelib_tz_lookup_table const   *)((void *)0);
  tmp = strcasecmp("utc", word);
  if (tmp == 0) {
    return (timelib_timezone_utc);
  } else {
    tmp___0 = strcasecmp("gmt", word);
    if (tmp___0 == 0) {
      return (timelib_timezone_utc);
    } else {

    }
  }
  tp = timelib_timezone_lookup;
  while (tp->name) {
    tmp___1 = strcasecmp(word, (char const   *)tp->name);
    if (tmp___1 == 0) {
      if (! first_found) {
        first_found = 1;
        first_found_elem = tp;
        if (gmtoffset == -1L) {
          return (tp);
        } else {

        }
      } else {

      }
      if (tp->gmtoffset == (float const   )gmtoffset) {
        return (tp);
      } else {

      }
    } else {

    }
    tp ++;
  }
  if (first_found) {
    return (first_found_elem);
  } else {

  }
  tp = timelib_timezone_lookup;
  while (tp->name) {
    if (tp->full_tz_name) {
      tmp___2 = strcasecmp(word, (char const   *)tp->full_tz_name);
      if (tmp___2 == 0) {
        if (! first_found) {
          first_found = 1;
          first_found_elem = tp;
          if (gmtoffset == -1L) {
            return (tp);
          } else {

          }
        } else {

        }
        if (tp->gmtoffset == (float const   )gmtoffset) {
          return (tp);
        } else {

        }
      } else {

      }
    } else {

    }
    tp ++;
  }
  if (first_found) {
    return (first_found_elem);
  } else {

  }
  fmp = timelib_timezone_fallbackmap;
  while (fmp->name) {
    if (fmp->gmtoffset * (float const   )3600 == (float const   )gmtoffset) {
      if (fmp->type == (int const   )isdst) {
        return (fmp);
      } else {

      }
    } else {

    }
    fmp ++;
  }
  return ((timelib_tz_lookup_table const   *)((void *)0));
}
}
static long timelib_lookup_zone(char **ptr , int *dst , char **tz_abbr ,
                                int *found ) 
{ 
  char *word ;
  char *begin ;
  char *end ;
  long value ;
  timelib_tz_lookup_table const   *tp ;
  void *tmp ;

  {
  begin = *ptr;
  value = 0L;
  while (1) {
    if ((int )*(*ptr) != 0) {
      if ((int )*(*ptr) != 41) {
        if ((int )*(*ptr) != 32) {

        } else {
          break;
        }
      } else {
        break;
      }
    } else {
      break;
    }
    (*ptr) ++;
  }
  end = *ptr;
  tmp = calloc((size_t )1, (size_t )((end - begin) + 1));
  word = (char *)tmp;
  memcpy((void */* __restrict  */)word, (void const   */* __restrict  */)begin,
         (size_t )(end - begin));
  tp = zone_search((char const   *)word, -1L, 0);
  if (tp) {
    value = (long )(- tp->gmtoffset / (float const   )60);
    *dst = (int )tp->type;
    value += (long )(tp->type * 60);
    *found = 1;
  } else {
    *found = 0;
  }
  *tz_abbr = word;
  return (value);
}
}
static long timelib_get_zone(char **ptr , int *dst , timelib_time *t ,
                             int *tz_not_found , timelib_tzdb const   *tzdb ) 
{ 
  timelib_tzinfo *res ;
  long retval ;
  long tmp ;
  int found ;
  long offset ;
  char *tz_abbr ;
  char *tmp___0 ;
  size_t __s1_len ;
  size_t __s2_len ;
  int tmp___1 ;
  int tmp___2 ;
  int tmp___3 ;
  unsigned char const   *__s2 ;
  register int __result ;
  int tmp___4 ;
  unsigned char const   *__s1 ;
  register int __result___0 ;
  int tmp___5 ;
  int tmp___6 ;
  int tmp___7 ;

  {
  retval = 0L;
  *tz_not_found = 0;
  while (1) {
    if ((int )*(*ptr) == 32) {

    } else
    if ((int )*(*ptr) == 9) {

    } else
    if ((int )*(*ptr) == 40) {

    } else {
      break;
    }
    (*ptr) ++;
  }
  if ((int )*(*ptr + 0) == 71) {
    if ((int )*(*ptr + 1) == 77) {
      if ((int )*(*ptr + 2) == 84) {
        if ((int )*(*ptr + 3) == 43) {
          *ptr += 3;
        } else
        if ((int )*(*ptr + 3) == 45) {
          *ptr += 3;
        } else {

        }
      } else {

      }
    } else {

    }
  } else {

  }
  if ((int )*(*ptr) == 43) {
    (*ptr) ++;
    t->is_localtime = 1U;
    t->zone_type = 1U;
    *tz_not_found = 0;
    t->dst = 0;
    tmp = timelib_parse_tz_cor(ptr);
    retval = -1L * tmp;
  } else
  if ((int )*(*ptr) == 45) {
    (*ptr) ++;
    t->is_localtime = 1U;
    t->zone_type = 1U;
    *tz_not_found = 0;
    t->dst = 0;
    retval = timelib_parse_tz_cor(ptr);
  } else {
    found = 0;
    t->is_localtime = 1U;
    offset = timelib_lookup_zone(ptr, dst, & tz_abbr, & found);
    if (found) {
      t->zone_type = 2U;
    } else {

    }
    tmp___0 = strstr((char const   *)tz_abbr, "/");
    if (tmp___0) {
      goto _L___1;
    } else {
      if (0) {
        __s1_len = strlen((char const   *)tz_abbr);
        __s2_len = strlen("UTC");
        if (! ((size_t )((void const   *)(tz_abbr + 1)) - (size_t )((void const   *)tz_abbr) == 1U)) {
          goto _L___0;
        } else
        if (__s1_len >= 4U) {
          _L___0: /* CIL Label */ 
          if (! ((size_t )((void const   *)("UTC" + 1)) - (size_t )((void const   *)"UTC") == 1U)) {
            tmp___7 = 1;
          } else
          if (__s2_len >= 4U) {
            tmp___7 = 1;
          } else {
            tmp___7 = 0;
          }
        } else {
          tmp___7 = 0;
        }
        if (tmp___7) {
          tmp___2 = __builtin_strcmp((char const   *)tz_abbr, "UTC");
          tmp___6 = tmp___2;
        } else {
          tmp___5 = __builtin_strcmp((char const   *)tz_abbr, "UTC");
          tmp___6 = tmp___5;
        }
      } else {
        tmp___5 = __builtin_strcmp((char const   *)tz_abbr, "UTC");
        tmp___6 = tmp___5;
      }
      if (tmp___6 == 0) {
        _L___1: /* CIL Label */ 
        res = timelib_parse_tzfile(tz_abbr, tzdb);
        if ((unsigned int )res != (unsigned int )((void *)0)) {
          t->tz_info = res;
          t->zone_type = 3U;
          found ++;
        } else {

        }
      } else {

      }
    }
    if (found) {
      if (t->zone_type != 3U) {
        timelib_time_tz_abbr_update(t, tz_abbr);
      } else {

      }
    } else {

    }
    free((void *)tz_abbr);
    *tz_not_found = found == 0;
    retval = offset;
  }
  while ((int )*(*ptr) == 41) {
    (*ptr) ++;
  }
  return (retval);
}
}
static int scan(Scanner *s ) ;
static unsigned char const   yybm[256]  = 
  {      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )100,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )100,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )128, 
        (unsigned char const   )64,      (unsigned char const   )160,      (unsigned char const   )96,      (unsigned char const   )0, 
        (unsigned char const   )2,      (unsigned char const   )2,      (unsigned char const   )2,      (unsigned char const   )2, 
        (unsigned char const   )2,      (unsigned char const   )2,      (unsigned char const   )2,      (unsigned char const   )2, 
        (unsigned char const   )2,      (unsigned char const   )2,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )8,      (unsigned char const   )8,      (unsigned char const   )8, 
        (unsigned char const   )8,      (unsigned char const   )8,      (unsigned char const   )8,      (unsigned char const   )8, 
        (unsigned char const   )8,      (unsigned char const   )8,      (unsigned char const   )8,      (unsigned char const   )8, 
        (unsigned char const   )8,      (unsigned char const   )8,      (unsigned char const   )8,      (unsigned char const   )8, 
        (unsigned char const   )8,      (unsigned char const   )8,      (unsigned char const   )8,      (unsigned char const   )8, 
        (unsigned char const   )8,      (unsigned char const   )8,      (unsigned char const   )8,      (unsigned char const   )8, 
        (unsigned char const   )8,      (unsigned char const   )8,      (unsigned char const   )8,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )24,      (unsigned char const   )24,      (unsigned char const   )24, 
        (unsigned char const   )88,      (unsigned char const   )24,      (unsigned char const   )24,      (unsigned char const   )24, 
        (unsigned char const   )88,      (unsigned char const   )24,      (unsigned char const   )24,      (unsigned char const   )24, 
        (unsigned char const   )24,      (unsigned char const   )24,      (unsigned char const   )88,      (unsigned char const   )24, 
        (unsigned char const   )24,      (unsigned char const   )24,      (unsigned char const   )88,      (unsigned char const   )88, 
        (unsigned char const   )88,      (unsigned char const   )24,      (unsigned char const   )24,      (unsigned char const   )24, 
        (unsigned char const   )24,      (unsigned char const   )24,      (unsigned char const   )24,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0};
static int scan(Scanner *s ) 
{ 
  uchar *cursor ;
  char *str ;
  char *ptr ;
  uchar yych ;
  unsigned int yyaccept ;
  int tz_not_found ;
  long tmp ;
  uchar *tmp___0 ;
  uchar *tmp___1 ;
  uchar *tmp___2 ;
  uchar *tmp___3 ;
  uchar *tmp___4 ;
  uchar *tmp___5 ;
  uchar *tmp___6 ;
  uchar *tmp___7 ;
  uchar *tmp___8 ;
  uchar *tmp___9 ;
  uchar *tmp___10 ;
  uchar *tmp___11 ;
  uchar *tmp___12 ;
  uchar *tmp___13 ;
  timelib_ull i ;
  uchar *tmp___14 ;
  uchar *tmp___15 ;
  uchar *tmp___16 ;
  uchar *tmp___17 ;
  uchar *tmp___18 ;
  uchar *tmp___19 ;
  uchar *tmp___20 ;
  uchar *tmp___21 ;
  uchar *tmp___22 ;
  uchar *tmp___23 ;
  uchar *tmp___24 ;
  uchar *tmp___25 ;
  uchar *tmp___26 ;
  uchar *tmp___27 ;
  uchar *tmp___28 ;
  timelib_relunit const   *relunit ;
  uchar *tmp___29 ;
  uchar *tmp___30 ;
  uchar *tmp___31 ;
  uchar *tmp___32 ;
  uchar *tmp___33 ;
  uchar *tmp___34 ;
  uchar *tmp___35 ;
  uchar *tmp___36 ;
  uchar *tmp___37 ;
  uchar *tmp___38 ;
  uchar *tmp___39 ;
  uchar *tmp___40 ;
  uchar *tmp___41 ;
  long tmp___42 ;
  uchar *tmp___43 ;
  long tmp___44 ;
  uchar *tmp___45 ;
  uchar *tmp___46 ;
  uchar *tmp___47 ;
  uchar *tmp___48 ;
  uchar *tmp___49 ;
  uchar *tmp___50 ;
  uchar *tmp___51 ;
  uchar *tmp___52 ;
  uchar *tmp___53 ;
  int tz_not_found___0 ;
  long tmp___54 ;
  long tmp___55 ;
  uchar *tmp___56 ;
  uchar *tmp___57 ;
  uchar *tmp___58 ;
  uchar *tmp___59 ;
  uchar *tmp___60 ;
  uchar *tmp___61 ;
  uchar *tmp___62 ;
  uchar *tmp___63 ;
  uchar *tmp___64 ;
  uchar *tmp___65 ;
  uchar *tmp___66 ;
  uchar *tmp___67 ;
  uchar *tmp___68 ;
  uchar *tmp___69 ;
  uchar *tmp___70 ;
  uchar *tmp___71 ;
  uchar *tmp___72 ;
  uchar *tmp___73 ;
  long tmp___74 ;
  timelib_sll tmp___75 ;
  uchar *tmp___76 ;
  uchar *tmp___77 ;
  uchar *tmp___78 ;
  uchar *tmp___79 ;
  long tmp___80 ;
  uchar *tmp___81 ;
  uchar *tmp___82 ;
  uchar *tmp___83 ;
  uchar *tmp___84 ;
  uchar *tmp___85 ;
  uchar *tmp___86 ;
  uchar *tmp___87 ;
  uchar *tmp___88 ;
  uchar *tmp___89 ;
  uchar *tmp___90 ;
  uchar *tmp___91 ;
  uchar *tmp___92 ;
  uchar *tmp___93 ;
  uchar *tmp___94 ;
  uchar *tmp___95 ;
  uchar *tmp___96 ;
  uchar *tmp___97 ;
  uchar *tmp___98 ;
  uchar *tmp___99 ;
  uchar *tmp___100 ;
  uchar *tmp___101 ;
  uchar *tmp___102 ;
  uchar *tmp___103 ;
  uchar *tmp___104 ;
  uchar *tmp___105 ;
  uchar *tmp___106 ;
  uchar *tmp___107 ;
  uchar *tmp___108 ;
  uchar *tmp___109 ;
  uchar *tmp___110 ;
  uchar *tmp___111 ;
  uchar *tmp___112 ;
  uchar *tmp___113 ;
  uchar *tmp___114 ;
  uchar *tmp___115 ;
  uchar *tmp___116 ;
  uchar *tmp___117 ;
  uchar *tmp___118 ;
  uchar *tmp___119 ;
  uchar *tmp___120 ;
  uchar *tmp___121 ;
  uchar *tmp___122 ;
  uchar *tmp___123 ;
  uchar *tmp___124 ;
  uchar *tmp___125 ;
  uchar *tmp___126 ;
  uchar *tmp___127 ;
  uchar *tmp___128 ;
  uchar *tmp___129 ;
  long tmp___130 ;
  uchar *tmp___131 ;
  uchar *tmp___132 ;
  uchar *tmp___133 ;
  uchar *tmp___134 ;
  uchar *tmp___135 ;
  uchar *tmp___136 ;
  uchar *tmp___137 ;
  uchar *tmp___138 ;
  uchar *tmp___139 ;
  uchar *tmp___140 ;
  uchar *tmp___141 ;
  uchar *tmp___142 ;
  uchar *tmp___143 ;
  uchar *tmp___144 ;
  uchar *tmp___145 ;
  uchar *tmp___146 ;
  uchar *tmp___147 ;
  uchar *tmp___148 ;
  uchar *tmp___149 ;
  uchar *tmp___150 ;
  uchar *tmp___151 ;
  uchar *tmp___152 ;
  uchar *tmp___153 ;
  uchar *tmp___154 ;
  uchar *tmp___155 ;
  uchar *tmp___156 ;
  uchar *tmp___157 ;
  uchar *tmp___158 ;
  uchar *tmp___159 ;
  uchar *tmp___160 ;
  uchar *tmp___161 ;
  uchar *tmp___162 ;
  uchar *tmp___163 ;
  uchar *tmp___164 ;
  uchar *tmp___165 ;
  uchar *tmp___166 ;
  uchar *tmp___167 ;
  uchar *tmp___168 ;
  uchar *tmp___169 ;
  uchar *tmp___170 ;
  uchar *tmp___171 ;
  uchar *tmp___172 ;
  uchar *tmp___173 ;
  uchar *tmp___174 ;
  uchar *tmp___175 ;
  uchar *tmp___176 ;
  uchar *tmp___177 ;
  uchar *tmp___178 ;
  uchar *tmp___179 ;
  uchar *tmp___180 ;
  uchar *tmp___181 ;
  uchar *tmp___182 ;
  uchar *tmp___183 ;
  uchar *tmp___184 ;
  timelib_ull tmp___185 ;
  uchar *tmp___186 ;
  uchar *tmp___187 ;
  uchar *tmp___188 ;
  uchar *tmp___189 ;
  uchar *tmp___190 ;
  long tmp___191 ;
  uchar *tmp___192 ;
  uchar *tmp___193 ;
  timelib_sll tmp___194 ;
  uchar *tmp___195 ;
  int tz_not_found___1 ;
  long tmp___196 ;
  uchar *tmp___197 ;
  uchar *tmp___198 ;
  uchar *tmp___199 ;
  uchar *tmp___200 ;
  uchar *tmp___201 ;
  uchar *tmp___202 ;
  uchar *tmp___203 ;
  uchar *tmp___204 ;
  uchar *tmp___205 ;
  timelib_sll tmp___206 ;
  uchar *tmp___207 ;
  long tmp___208 ;
  uchar *tmp___209 ;
  uchar *tmp___210 ;
  uchar *tmp___211 ;
  uchar *tmp___212 ;
  uchar *tmp___213 ;
  uchar *tmp___214 ;
  uchar *tmp___215 ;
  uchar *tmp___216 ;
  uchar *tmp___217 ;
  uchar *tmp___218 ;
  uchar *tmp___219 ;
  uchar *tmp___220 ;
  uchar *tmp___221 ;
  uchar *tmp___222 ;
  uchar *tmp___223 ;
  uchar *tmp___224 ;
  uchar *tmp___225 ;
  uchar *tmp___226 ;
  uchar *tmp___227 ;
  uchar *tmp___228 ;
  uchar *tmp___229 ;
  uchar *tmp___230 ;
  uchar *tmp___231 ;
  uchar *tmp___232 ;
  uchar *tmp___233 ;
  uchar *tmp___234 ;
  uchar *tmp___235 ;
  uchar *tmp___236 ;
  uchar *tmp___237 ;
  uchar *tmp___238 ;
  uchar *tmp___239 ;
  uchar *tmp___240 ;
  uchar *tmp___241 ;
  uchar *tmp___242 ;
  uchar *tmp___243 ;
  uchar *tmp___244 ;
  uchar *tmp___245 ;
  uchar *tmp___246 ;
  uchar *tmp___247 ;
  uchar *tmp___248 ;
  uchar *tmp___249 ;
  uchar *tmp___250 ;
  uchar *tmp___251 ;
  uchar *tmp___252 ;
  uchar *tmp___253 ;
  uchar *tmp___254 ;
  uchar *tmp___255 ;
  int tz_not_found___2 ;
  long tmp___256 ;
  long tmp___257 ;
  uchar *tmp___258 ;
  uchar *tmp___259 ;
  uchar *tmp___260 ;
  uchar *tmp___261 ;
  uchar *tmp___262 ;
  uchar *tmp___263 ;
  uchar *tmp___264 ;
  uchar *tmp___265 ;
  long tmp___266 ;
  uchar *tmp___267 ;
  long tmp___268 ;
  uchar *tmp___269 ;
  timelib_sll w ;
  timelib_sll d ;
  timelib_sll w___0 ;
  timelib_sll d___0 ;
  uchar *tmp___270 ;
  int tz_not_found___3 ;
  long tmp___271 ;
  uchar *tmp___272 ;
  uchar *tmp___273 ;
  uchar *tmp___274 ;
  uchar *tmp___275 ;
  uchar *tmp___276 ;
  uchar *tmp___277 ;
  uchar *tmp___278 ;
  uchar *tmp___279 ;
  uchar *tmp___280 ;
  uchar *tmp___281 ;
  uchar *tmp___282 ;
  uchar *tmp___283 ;
  uchar *tmp___284 ;
  uchar *tmp___285 ;
  uchar *tmp___286 ;
  uchar *tmp___287 ;
  uchar *tmp___288 ;
  uchar *tmp___289 ;
  uchar *tmp___290 ;
  uchar *tmp___291 ;
  uchar *tmp___292 ;
  uchar *tmp___293 ;
  uchar *tmp___294 ;
  uchar *tmp___295 ;
  uchar *tmp___296 ;
  uchar *tmp___297 ;
  uchar *tmp___298 ;
  uchar *tmp___299 ;
  uchar *tmp___300 ;
  uchar *tmp___301 ;
  uchar *tmp___302 ;
  uchar *tmp___303 ;
  uchar *tmp___304 ;
  uchar *tmp___305 ;
  uchar *tmp___306 ;
  uchar *tmp___307 ;
  uchar *tmp___308 ;
  uchar *tmp___309 ;
  uchar *tmp___310 ;
  uchar *tmp___311 ;
  uchar *tmp___312 ;
  uchar *tmp___313 ;
  uchar *tmp___314 ;
  uchar *tmp___315 ;
  uchar *tmp___316 ;
  uchar *tmp___317 ;
  uchar *tmp___318 ;
  uchar *tmp___319 ;
  uchar *tmp___320 ;
  uchar *tmp___321 ;
  uchar *tmp___322 ;
  uchar *tmp___323 ;
  int tz_not_found___4 ;
  long tmp___324 ;
  uchar *tmp___325 ;
  uchar *tmp___326 ;
  uchar *tmp___327 ;
  uchar *tmp___328 ;
  uchar *tmp___329 ;
  uchar *tmp___330 ;
  uchar *tmp___331 ;
  uchar *tmp___332 ;
  uchar *tmp___333 ;
  uchar *tmp___334 ;
  uchar *tmp___335 ;
  uchar *tmp___336 ;
  uchar *tmp___337 ;
  uchar *tmp___338 ;
  uchar *tmp___339 ;
  uchar *tmp___340 ;
  uchar *tmp___341 ;
  uchar *tmp___342 ;
  uchar *tmp___343 ;
  timelib_sll i___0 ;
  int behavior ;
  uchar *tmp___344 ;
  timelib_sll i___1 ;
  int behavior___0 ;
  uchar *tmp___345 ;
  uchar *tmp___346 ;
  uchar *tmp___347 ;
  timelib_sll i___2 ;
  int behavior___1 ;
  uchar *tmp___348 ;
  uchar *tmp___349 ;
  uchar *tmp___350 ;
  uchar *tmp___351 ;
  uchar *tmp___352 ;
  uchar *tmp___353 ;
  uchar *tmp___354 ;
  uchar *tmp___355 ;
  uchar *tmp___356 ;
  uchar *tmp___357 ;
  uchar *tmp___358 ;
  uchar *tmp___359 ;
  uchar *tmp___360 ;
  uchar *tmp___361 ;
  uchar *tmp___362 ;
  uchar *tmp___363 ;
  uchar *tmp___364 ;
  uchar *tmp___365 ;
  uchar *tmp___366 ;
  uchar *tmp___367 ;
  uchar *tmp___368 ;
  uchar *tmp___369 ;
  uchar *tmp___370 ;
  uchar *tmp___371 ;
  uchar *tmp___372 ;
  uchar *tmp___373 ;
  uchar *tmp___374 ;
  uchar *tmp___375 ;
  uchar *tmp___376 ;
  uchar *tmp___377 ;
  uchar *tmp___378 ;
  uchar *tmp___379 ;
  uchar *tmp___380 ;
  uchar *tmp___381 ;
  uchar *tmp___382 ;
  uchar *tmp___383 ;
  uchar *tmp___384 ;
  uchar *tmp___385 ;
  uchar *tmp___386 ;
  uchar *tmp___387 ;
  uchar *tmp___388 ;
  uchar *tmp___389 ;
  uchar *tmp___390 ;
  uchar *tmp___391 ;
  uchar *tmp___392 ;
  uchar *tmp___393 ;
  uchar *tmp___394 ;
  uchar *tmp___395 ;
  uchar *tmp___396 ;
  uchar *tmp___397 ;
  uchar *tmp___398 ;
  uchar *tmp___399 ;
  uchar *tmp___400 ;
  uchar *tmp___401 ;
  uchar *tmp___402 ;
  uchar *tmp___403 ;
  uchar *tmp___404 ;
  uchar *tmp___405 ;
  timelib_sll tmp___406 ;
  timelib_sll tmp___407 ;
  uchar *tmp___408 ;
  uchar *tmp___409 ;
  uchar *tmp___410 ;
  uchar *tmp___411 ;
  uchar *tmp___412 ;
  uchar *tmp___413 ;
  uchar *tmp___414 ;
  uchar *tmp___415 ;
  uchar *tmp___416 ;
  uchar *tmp___417 ;
  uchar *tmp___418 ;
  uchar *tmp___419 ;
  uchar *tmp___420 ;
  uchar *tmp___421 ;
  uchar *tmp___422 ;
  uchar *tmp___423 ;
  uchar *tmp___424 ;
  uchar *tmp___425 ;
  uchar *tmp___426 ;
  uchar *tmp___427 ;
  uchar *tmp___428 ;
  uchar *tmp___429 ;
  uchar *tmp___430 ;
  uchar *tmp___431 ;
  uchar *tmp___432 ;
  uchar *tmp___433 ;
  uchar *tmp___434 ;
  uchar *tmp___435 ;
  uchar *tmp___436 ;
  uchar *tmp___437 ;
  uchar *tmp___438 ;
  uchar *tmp___439 ;
  uchar *tmp___440 ;
  uchar *tmp___441 ;
  uchar *tmp___442 ;
  uchar *tmp___443 ;
  uchar *tmp___444 ;
  uchar *tmp___445 ;
  timelib_ull i___3 ;
  timelib_sll tmp___446 ;
  timelib_sll tmp___447 ;
  uchar *tmp___448 ;
  uchar *tmp___449 ;
  uchar *tmp___450 ;
  uchar *tmp___451 ;
  uchar *tmp___452 ;
  uchar *tmp___453 ;
  uchar *tmp___454 ;
  uchar *tmp___455 ;
  uchar *tmp___456 ;
  uchar *tmp___457 ;
  uchar *tmp___458 ;
  uchar *tmp___459 ;
  uchar *tmp___460 ;
  uchar *tmp___461 ;
  uchar *tmp___462 ;
  uchar *tmp___463 ;
  uchar *tmp___464 ;
  uchar *tmp___465 ;
  uchar *tmp___466 ;
  uchar *tmp___467 ;
  uchar *tmp___468 ;
  uchar *tmp___469 ;
  uchar *tmp___470 ;
  uchar *tmp___471 ;
  uchar *tmp___472 ;
  uchar *tmp___473 ;
  uchar *tmp___474 ;
  uchar *tmp___475 ;
  uchar *tmp___476 ;
  uchar *tmp___477 ;
  uchar *tmp___478 ;
  uchar *tmp___479 ;
  uchar *tmp___480 ;
  uchar *tmp___481 ;
  uchar *tmp___482 ;
  uchar *tmp___483 ;
  uchar *tmp___484 ;
  uchar *tmp___485 ;
  uchar *tmp___486 ;
  uchar *tmp___487 ;
  uchar *tmp___488 ;
  uchar *tmp___489 ;
  uchar *tmp___490 ;
  uchar *tmp___491 ;
  uchar *tmp___492 ;
  uchar *tmp___493 ;
  uchar *tmp___494 ;
  uchar *tmp___495 ;
  uchar *tmp___496 ;
  uchar *tmp___497 ;
  uchar *tmp___498 ;
  uchar *tmp___499 ;
  uchar *tmp___500 ;
  uchar *tmp___501 ;
  uchar *tmp___502 ;
  uchar *tmp___503 ;
  uchar *tmp___504 ;
  uchar *tmp___505 ;
  uchar *tmp___506 ;
  uchar *tmp___507 ;
  uchar *tmp___508 ;
  uchar *tmp___509 ;
  uchar *tmp___510 ;
  uchar *tmp___511 ;
  uchar *tmp___512 ;
  uchar *tmp___513 ;
  uchar *tmp___514 ;
  uchar *tmp___515 ;
  uchar *tmp___516 ;
  uchar *tmp___517 ;
  uchar *tmp___518 ;
  uchar *tmp___519 ;
  uchar *tmp___520 ;
  uchar *tmp___521 ;

  {
  cursor = s->cur;
  ptr = (char *)((void *)0);
  std: 
  s->tok = cursor;
  s->len = 0U;
  yyaccept = 0U;
  if (s->lim - cursor < 31) {
    return (257);
  } else {

  }
  yych = *cursor;
  switch ((int )yych) {
  case 10: 
  case 0: 
  goto yy52;
  case 32: 
  case 9: 
  goto yy49;
  case 40: 
  goto yy46;
  case 45: 
  case 43: 
  goto yy31;
  case 46: 
  case 44: 
  goto yy51;
  case 48: 
  goto yy26;
  case 49: 
  goto yy27;
  case 50: 
  goto yy28;
  case 51: 
  goto yy29;
  case 57: 
  case 56: 
  case 55: 
  case 54: 
  case 53: 
  case 52: 
  goto yy30;
  case 64: 
  goto yy12;
  case 65: 
  goto yy37;
  case 66: 
  goto yy18;
  case 90: 
  case 85: 
  case 82: 
  case 81: 
  case 75: 
  case 72: 
  case 67: 
  goto yy47;
  case 68: 
  goto yy41;
  case 69: 
  goto yy22;
  case 70: 
  goto yy14;
  case 71: 
  goto yy45;
  case 73: 
  goto yy32;
  case 74: 
  goto yy35;
  case 76: 
  goto yy16;
  case 77: 
  goto yy8;
  case 78: 
  goto yy6;
  case 79: 
  goto yy39;
  case 80: 
  goto yy24;
  case 83: 
  goto yy20;
  case 84: 
  goto yy10;
  case 86: 
  goto yy33;
  case 87: 
  goto yy43;
  case 88: 
  goto yy34;
  case 89: 
  goto yy3;
  case 97: 
  goto yy38;
  case 98: 
  goto yy19;
  case 122: 
  case 120: 
  case 118: 
  case 117: 
  case 114: 
  case 113: 
  case 107: 
  case 105: 
  case 104: 
  case 103: 
  case 99: 
  goto yy48;
  case 100: 
  goto yy42;
  case 101: 
  goto yy23;
  case 102: 
  goto yy15;
  case 106: 
  goto yy36;
  case 108: 
  goto yy17;
  case 109: 
  goto yy9;
  case 110: 
  goto yy7;
  case 111: 
  goto yy40;
  case 112: 
  goto yy25;
  case 115: 
  goto yy21;
  case 116: 
  goto yy11;
  case 119: 
  goto yy44;
  case 121: 
  goto yy5;
  default: 
  goto yy54;
  }
  yy2: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  (s->time)->have_relative = 1U;
  if ((int )*ptr == 108) {
    (s->time)->relative.first_last_day_of = 2;
  } else {
    (s->time)->relative.first_last_day_of = 1;
  }
  free((void *)str);
  return (280);
  yy3: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 69) {
    if ((int )yych <= 41) {
      if ((int )yych >= 41) {
        goto yy140;
      } else {

      }
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 68) {
        goto yy141;
      } else {

      }
      goto yy1523;
    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 90) {
      goto yy141;
    } else {

    }
    if ((int )yych >= 97) {
      goto yy146;
    } else {

    }
  } else {
    if ((int )yych <= 101) {
      goto yy1532;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy146;
    } else {

    }
  }
  yy4: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  s->cur = cursor;
  if ((s->time)->have_zone) {
    if ((s->time)->have_zone > 1U) {
      add_error(s, (char *)"Double timezone specification");
    } else {
      add_warning(s, (char *)"Double timezone specification");
    }
    free((void *)str);
    ((s->time)->have_zone) ++;
    return (999);
  } else {
    ((s->time)->have_zone) ++;
  }
  tmp = timelib_get_zone(& ptr, & (s->time)->dst, s->time, & tz_not_found,
                         s->tzdb);
  (s->time)->z = (int )tmp;
  if (tz_not_found) {
    add_error(s, (char *)"The timezone could not be found in the database");
  } else {

  }
  free((void *)str);
  return (300);
  yy5: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 69) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 68) {
        goto yy141;
      } else {

      }
      goto yy1523;
    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 90) {
      goto yy141;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy141;
  } else {
    if ((int )yych <= 101) {
      goto yy1523;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy141;
    } else {

    }
    goto yy4;
  }
  yy6: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 79) {
    if ((int )yych <= 68) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy141;
    } else
    if ((int )yych <= 72) {
      if ((int )yych <= 69) {
        goto yy1494;
      } else {

      }
      goto yy141;
    } else {
      if ((int )yych <= 73) {
        goto yy1495;
      } else {

      }
      if ((int )yych <= 78) {
        goto yy141;
      } else {

      }
      goto yy1493;
    }
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 96) {
      if ((int )yych <= 90) {
        goto yy141;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 101) {
        goto yy1510;
      } else {

      }
      goto yy146;
    }
  } else
  if ((int )yych <= 110) {
    if ((int )yych <= 105) {
      goto yy1511;
    } else {

    }
    goto yy146;
  } else {
    if ((int )yych <= 111) {
      goto yy1509;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy146;
    } else {

    }
    goto yy4;
  }
  yy7: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 79) {
    if ((int )yych <= 68) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy141;
    } else
    if ((int )yych <= 72) {
      if ((int )yych <= 69) {
        goto yy1494;
      } else {

      }
      goto yy141;
    } else {
      if ((int )yych <= 73) {
        goto yy1495;
      } else {

      }
      if ((int )yych <= 78) {
        goto yy141;
      } else {

      }
      goto yy1493;
    }
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 96) {
      if ((int )yych <= 90) {
        goto yy141;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 101) {
        goto yy1494;
      } else {

      }
      goto yy141;
    }
  } else
  if ((int )yych <= 110) {
    if ((int )yych <= 105) {
      goto yy1495;
    } else {

    }
    goto yy141;
  } else {
    if ((int )yych <= 111) {
      goto yy1493;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy141;
    } else {

    }
    goto yy4;
  }
  yy8: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 79) {
    if ((int )yych <= 65) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy1463;
    } else {
      if ((int )yych == 73) {
        goto yy1464;
      } else {

      }
      if ((int )yych <= 78) {
        goto yy141;
      } else {

      }
      goto yy1465;
    }
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 90) {
      goto yy141;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 97) {
      goto yy1478;
    } else {

    }
    goto yy146;
  } else
  if ((int )yych <= 110) {
    if ((int )yych <= 105) {
      goto yy1479;
    } else {

    }
    goto yy146;
  } else {
    if ((int )yych <= 111) {
      goto yy1480;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy146;
    } else {

    }
    goto yy4;
  }
  yy9: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 79) {
    if ((int )yych <= 65) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy1463;
    } else {
      if ((int )yych == 73) {
        goto yy1464;
      } else {

      }
      if ((int )yych <= 78) {
        goto yy141;
      } else {

      }
      goto yy1465;
    }
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 90) {
      goto yy141;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 97) {
      goto yy1463;
    } else {

    }
    goto yy141;
  } else
  if ((int )yych <= 110) {
    if ((int )yych <= 105) {
      goto yy1464;
    } else {

    }
    goto yy141;
  } else {
    if ((int )yych <= 111) {
      goto yy1465;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy141;
    } else {

    }
    goto yy4;
  }
  yy10: 
  yyaccept = 0U;
  cursor ++;
  tmp___0 = cursor;
  s->ptr = tmp___0;
  yych = *tmp___0;
  switch ((int )yych) {
  case 41: 
  goto yy140;
  case 49: 
  case 48: 
  goto yy1393;
  case 50: 
  goto yy1394;
  case 57: 
  case 56: 
  case 55: 
  case 54: 
  case 53: 
  case 52: 
  case 51: 
  goto yy1395;
  case 90: 
  case 89: 
  case 88: 
  case 86: 
  case 84: 
  case 83: 
  case 82: 
  case 81: 
  case 80: 
  case 78: 
  case 77: 
  case 76: 
  case 75: 
  case 74: 
  case 73: 
  case 71: 
  case 70: 
  case 68: 
  case 67: 
  case 66: 
  case 65: 
  goto yy141;
  case 69: 
  goto yy1388;
  case 72: 
  goto yy1389;
  case 79: 
  goto yy1390;
  case 85: 
  goto yy1391;
  case 87: 
  goto yy1392;
  case 122: 
  case 121: 
  case 120: 
  case 118: 
  case 116: 
  case 115: 
  case 114: 
  case 113: 
  case 112: 
  case 110: 
  case 109: 
  case 108: 
  case 107: 
  case 106: 
  case 105: 
  case 103: 
  case 102: 
  case 100: 
  case 99: 
  case 98: 
  case 97: 
  goto yy146;
  case 101: 
  goto yy1431;
  case 104: 
  goto yy1432;
  case 111: 
  goto yy1433;
  case 117: 
  goto yy1434;
  case 119: 
  goto yy1435;
  default: 
  goto yy4;
  }
  yy11: 
  yyaccept = 0U;
  cursor ++;
  tmp___1 = cursor;
  s->ptr = tmp___1;
  yych = *tmp___1;
  switch ((int )yych) {
  case 41: 
  goto yy140;
  case 49: 
  case 48: 
  goto yy1393;
  case 50: 
  goto yy1394;
  case 57: 
  case 56: 
  case 55: 
  case 54: 
  case 53: 
  case 52: 
  case 51: 
  goto yy1395;
  case 122: 
  case 121: 
  case 120: 
  case 118: 
  case 116: 
  case 115: 
  case 114: 
  case 113: 
  case 112: 
  case 110: 
  case 109: 
  case 108: 
  case 107: 
  case 106: 
  case 105: 
  case 103: 
  case 102: 
  case 100: 
  case 99: 
  case 98: 
  case 97: 
  case 90: 
  case 89: 
  case 88: 
  case 86: 
  case 84: 
  case 83: 
  case 82: 
  case 81: 
  case 80: 
  case 78: 
  case 77: 
  case 76: 
  case 75: 
  case 74: 
  case 73: 
  case 71: 
  case 70: 
  case 68: 
  case 67: 
  case 66: 
  case 65: 
  goto yy141;
  case 101: 
  case 69: 
  goto yy1388;
  case 104: 
  case 72: 
  goto yy1389;
  case 111: 
  case 79: 
  goto yy1390;
  case 117: 
  case 85: 
  goto yy1391;
  case 119: 
  case 87: 
  goto yy1392;
  default: 
  goto yy4;
  }
  yy12: 
  yyaccept = 1U;
  cursor ++;
  tmp___2 = cursor;
  s->ptr = tmp___2;
  yych = *tmp___2;
  if ((int )yych == 45) {
    goto yy1384;
  } else {

  }
  if ((int )yych <= 47) {
    goto yy13;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy1385;
  } else {

  }
  yy13: 
  add_error(s, (char *)"Unexpected character");
  goto std;
  yy14: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 82) {
    if ((int )yych <= 69) {
      if ((int )yych <= 41) {
        if ((int )yych <= 40) {
          goto yy4;
        } else {

        }
        goto yy140;
      } else {
        if ((int )yych <= 64) {
          goto yy4;
        } else {

        }
        if ((int )yych <= 68) {
          goto yy141;
        } else {

        }
        goto yy1320;
      }
    } else
    if ((int )yych <= 78) {
      if ((int )yych == 73) {
        goto yy1321;
      } else {

      }
      goto yy141;
    } else {
      if ((int )yych <= 79) {
        goto yy1322;
      } else {

      }
      if ((int )yych <= 81) {
        goto yy141;
      } else {

      }
      goto yy1323;
    }
  } else
  if ((int )yych <= 105) {
    if ((int )yych <= 100) {
      if ((int )yych <= 90) {
        goto yy141;
      } else {

      }
      if ((int )yych <= 96) {
        goto yy4;
      } else {

      }
      goto yy146;
    } else {
      if ((int )yych <= 101) {
        goto yy1361;
      } else {

      }
      if ((int )yych <= 104) {
        goto yy146;
      } else {

      }
      goto yy1362;
    }
  } else
  if ((int )yych <= 113) {
    if ((int )yych == 111) {
      goto yy1363;
    } else {

    }
    goto yy146;
  } else {
    if ((int )yych <= 114) {
      goto yy1364;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy146;
    } else {

    }
    goto yy4;
  }
  yy15: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 82) {
    if ((int )yych <= 69) {
      if ((int )yych <= 41) {
        if ((int )yych <= 40) {
          goto yy4;
        } else {

        }
        goto yy140;
      } else {
        if ((int )yych <= 64) {
          goto yy4;
        } else {

        }
        if ((int )yych <= 68) {
          goto yy141;
        } else {

        }
        goto yy1320;
      }
    } else
    if ((int )yych <= 78) {
      if ((int )yych == 73) {
        goto yy1321;
      } else {

      }
      goto yy141;
    } else {
      if ((int )yych <= 79) {
        goto yy1322;
      } else {

      }
      if ((int )yych <= 81) {
        goto yy141;
      } else {

      }
      goto yy1323;
    }
  } else
  if ((int )yych <= 105) {
    if ((int )yych <= 100) {
      if ((int )yych <= 90) {
        goto yy141;
      } else {

      }
      if ((int )yych <= 96) {
        goto yy4;
      } else {

      }
      goto yy141;
    } else {
      if ((int )yych <= 101) {
        goto yy1320;
      } else {

      }
      if ((int )yych <= 104) {
        goto yy141;
      } else {

      }
      goto yy1321;
    }
  } else
  if ((int )yych <= 113) {
    if ((int )yych == 111) {
      goto yy1322;
    } else {

    }
    goto yy141;
  } else {
    if ((int )yych <= 114) {
      goto yy1323;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy141;
    } else {

    }
    goto yy4;
  }
  yy16: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 65) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    goto yy1307;
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 90) {
      goto yy141;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 97) {
      goto yy1317;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy146;
    } else {

    }
    goto yy4;
  }
  yy17: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 65) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    goto yy1307;
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 90) {
      goto yy141;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 97) {
      goto yy1307;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy141;
    } else {

    }
    goto yy4;
  }
  yy18: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 65) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    goto yy1287;
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 90) {
      goto yy141;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 97) {
      goto yy1304;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy146;
    } else {

    }
    goto yy4;
  }
  yy19: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 65) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    goto yy1287;
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 90) {
      goto yy141;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 97) {
      goto yy1287;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy141;
    } else {

    }
    goto yy4;
  }
  yy20: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych <= 68) {
      if ((int )yych <= 41) {
        if ((int )yych <= 40) {
          goto yy4;
        } else {

        }
        goto yy140;
      } else {
        if ((int )yych <= 64) {
          goto yy4;
        } else {

        }
        if ((int )yych <= 65) {
          goto yy1230;
        } else {

        }
        goto yy141;
      }
    } else
    if ((int )yych <= 72) {
      if ((int )yych <= 69) {
        goto yy1229;
      } else {

      }
      goto yy141;
    } else {
      if ((int )yych <= 73) {
        goto yy1231;
      } else {

      }
      if ((int )yych <= 84) {
        goto yy141;
      } else {

      }
      goto yy1232;
    }
  } else
  if ((int )yych <= 101) {
    if ((int )yych <= 96) {
      if ((int )yych <= 90) {
        goto yy141;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych <= 97) {
        goto yy1259;
      } else {

      }
      if ((int )yych <= 100) {
        goto yy146;
      } else {

      }
      goto yy1258;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych == 105) {
      goto yy1260;
    } else {

    }
    goto yy146;
  } else {
    if ((int )yych <= 117) {
      goto yy1261;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy146;
    } else {

    }
    goto yy4;
  }
  yy21: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych <= 68) {
      if ((int )yych <= 41) {
        if ((int )yych <= 40) {
          goto yy4;
        } else {

        }
        goto yy140;
      } else {
        if ((int )yych <= 64) {
          goto yy4;
        } else {

        }
        if ((int )yych <= 65) {
          goto yy1230;
        } else {

        }
        goto yy141;
      }
    } else
    if ((int )yych <= 72) {
      if ((int )yych <= 69) {
        goto yy1229;
      } else {

      }
      goto yy141;
    } else {
      if ((int )yych <= 73) {
        goto yy1231;
      } else {

      }
      if ((int )yych <= 84) {
        goto yy141;
      } else {

      }
      goto yy1232;
    }
  } else
  if ((int )yych <= 101) {
    if ((int )yych <= 96) {
      if ((int )yych <= 90) {
        goto yy141;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych <= 97) {
        goto yy1230;
      } else {

      }
      if ((int )yych <= 100) {
        goto yy141;
      } else {

      }
      goto yy1229;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych == 105) {
      goto yy1231;
    } else {

    }
    goto yy141;
  } else {
    if ((int )yych <= 117) {
      goto yy1232;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy141;
    } else {

    }
    goto yy4;
  }
  yy22: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 76) {
    if ((int )yych <= 64) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 73) {
        goto yy1199;
      } else {

      }
      if ((int )yych <= 75) {
        goto yy141;
      } else {

      }
      goto yy1200;
    }
  } else
  if ((int )yych <= 105) {
    if ((int )yych <= 90) {
      goto yy141;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 104) {
      goto yy146;
    } else {

    }
    goto yy1217;
  } else {
    if ((int )yych == 108) {
      goto yy1218;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy146;
    } else {

    }
    goto yy4;
  }
  yy23: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 76) {
    if ((int )yych <= 64) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 73) {
        goto yy1199;
      } else {

      }
      if ((int )yych <= 75) {
        goto yy141;
      } else {

      }
      goto yy1200;
    }
  } else
  if ((int )yych <= 105) {
    if ((int )yych <= 90) {
      goto yy141;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 104) {
      goto yy141;
    } else {

    }
    goto yy1199;
  } else {
    if ((int )yych == 108) {
      goto yy1200;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy141;
    } else {

    }
    goto yy4;
  }
  yy24: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 82) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 81) {
        goto yy141;
      } else {

      }
      goto yy1098;
    }
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 90) {
      goto yy141;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy146;
  } else {
    if ((int )yych <= 114) {
      goto yy1192;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy146;
    } else {

    }
    goto yy4;
  }
  yy25: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 82) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 81) {
        goto yy141;
      } else {

      }
      goto yy1098;
    }
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 90) {
      goto yy141;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy141;
  } else {
    if ((int )yych <= 114) {
      goto yy1098;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy141;
    } else {

    }
    goto yy4;
  }
  yy26: 
  yyaccept = 1U;
  cursor ++;
  tmp___3 = cursor;
  s->ptr = tmp___3;
  yych = *tmp___3;
  switch ((int )yych) {
  case 9: 
  goto yy1052;
  case 121: 
  case 119: 
  case 111: 
  case 109: 
  case 106: 
  case 104: 
  case 102: 
  case 100: 
  case 97: 
  case 89: 
  case 88: 
  case 87: 
  case 86: 
  case 84: 
  case 83: 
  case 79: 
  case 78: 
  case 77: 
  case 74: 
  case 73: 
  case 72: 
  case 70: 
  case 68: 
  case 65: 
  case 32: 
  goto yy1054;
  case 45: 
  goto yy473;
  case 46: 
  goto yy1064;
  case 47: 
  goto yy472;
  case 48: 
  goto yy1097;
  case 57: 
  case 56: 
  case 55: 
  case 54: 
  case 53: 
  case 52: 
  case 51: 
  case 50: 
  case 49: 
  goto yy1096;
  case 58: 
  goto yy1065;
  case 110: 
  goto yy470;
  case 114: 
  goto yy471;
  case 115: 
  goto yy464;
  case 116: 
  goto yy468;
  default: 
  goto yy13;
  }
  yy27: 
  yyaccept = 1U;
  cursor ++;
  tmp___4 = cursor;
  s->ptr = tmp___4;
  yych = *tmp___4;
  switch ((int )yych) {
  case 9: 
  goto yy460;
  case 121: 
  case 119: 
  case 112: 
  case 111: 
  case 109: 
  case 106: 
  case 104: 
  case 102: 
  case 100: 
  case 97: 
  case 89: 
  case 88: 
  case 87: 
  case 86: 
  case 84: 
  case 83: 
  case 80: 
  case 79: 
  case 78: 
  case 77: 
  case 74: 
  case 73: 
  case 72: 
  case 70: 
  case 68: 
  case 65: 
  case 32: 
  goto yy462;
  case 45: 
  goto yy473;
  case 46: 
  goto yy474;
  case 47: 
  goto yy472;
  case 50: 
  case 49: 
  case 48: 
  goto yy1096;
  case 57: 
  case 56: 
  case 55: 
  case 54: 
  case 53: 
  case 52: 
  case 51: 
  goto yy1063;
  case 58: 
  goto yy483;
  case 110: 
  goto yy470;
  case 114: 
  goto yy471;
  case 115: 
  goto yy464;
  case 116: 
  goto yy468;
  default: 
  goto yy13;
  }
  yy28: 
  yyaccept = 1U;
  cursor ++;
  tmp___5 = cursor;
  s->ptr = tmp___5;
  yych = *tmp___5;
  switch ((int )yych) {
  case 9: 
  goto yy460;
  case 121: 
  case 119: 
  case 112: 
  case 111: 
  case 109: 
  case 106: 
  case 104: 
  case 102: 
  case 100: 
  case 97: 
  case 89: 
  case 88: 
  case 87: 
  case 86: 
  case 84: 
  case 83: 
  case 80: 
  case 79: 
  case 78: 
  case 77: 
  case 74: 
  case 73: 
  case 72: 
  case 70: 
  case 68: 
  case 65: 
  case 32: 
  goto yy462;
  case 45: 
  goto yy473;
  case 46: 
  goto yy474;
  case 47: 
  goto yy472;
  case 52: 
  case 51: 
  case 50: 
  case 49: 
  case 48: 
  goto yy1063;
  case 57: 
  case 56: 
  case 55: 
  case 54: 
  case 53: 
  goto yy1050;
  case 58: 
  goto yy483;
  case 110: 
  goto yy470;
  case 114: 
  goto yy471;
  case 115: 
  goto yy464;
  case 116: 
  goto yy468;
  default: 
  goto yy13;
  }
  yy29: 
  yyaccept = 1U;
  cursor ++;
  tmp___6 = cursor;
  s->ptr = tmp___6;
  yych = *tmp___6;
  switch ((int )yych) {
  case 9: 
  goto yy460;
  case 121: 
  case 119: 
  case 112: 
  case 111: 
  case 109: 
  case 106: 
  case 104: 
  case 102: 
  case 100: 
  case 97: 
  case 89: 
  case 88: 
  case 87: 
  case 86: 
  case 84: 
  case 83: 
  case 80: 
  case 79: 
  case 78: 
  case 77: 
  case 74: 
  case 73: 
  case 72: 
  case 70: 
  case 68: 
  case 65: 
  case 32: 
  goto yy462;
  case 45: 
  goto yy473;
  case 46: 
  goto yy474;
  case 47: 
  goto yy472;
  case 49: 
  case 48: 
  goto yy1050;
  case 57: 
  case 56: 
  case 55: 
  case 54: 
  case 53: 
  case 52: 
  case 51: 
  case 50: 
  goto yy469;
  case 58: 
  goto yy483;
  case 110: 
  goto yy470;
  case 114: 
  goto yy471;
  case 115: 
  goto yy464;
  case 116: 
  goto yy468;
  default: 
  goto yy13;
  }
  yy30: 
  yyaccept = 1U;
  cursor ++;
  tmp___7 = cursor;
  s->ptr = tmp___7;
  yych = *tmp___7;
  switch ((int )yych) {
  case 9: 
  goto yy460;
  case 121: 
  case 119: 
  case 112: 
  case 111: 
  case 109: 
  case 106: 
  case 104: 
  case 102: 
  case 100: 
  case 97: 
  case 89: 
  case 88: 
  case 87: 
  case 86: 
  case 84: 
  case 83: 
  case 80: 
  case 79: 
  case 78: 
  case 77: 
  case 74: 
  case 73: 
  case 72: 
  case 70: 
  case 68: 
  case 65: 
  case 32: 
  goto yy462;
  case 45: 
  goto yy473;
  case 46: 
  goto yy474;
  case 47: 
  goto yy472;
  case 57: 
  case 56: 
  case 55: 
  case 54: 
  case 53: 
  case 52: 
  case 51: 
  case 50: 
  case 49: 
  case 48: 
  goto yy469;
  case 58: 
  goto yy483;
  case 110: 
  goto yy470;
  case 114: 
  goto yy471;
  case 115: 
  goto yy464;
  case 116: 
  goto yy468;
  default: 
  goto yy13;
  }
  yy31: 
  yyaccept = 1U;
  cursor ++;
  tmp___8 = cursor;
  s->ptr = tmp___8;
  yych = *tmp___8;
  if ((int const   )yybm[(int )yych] & 4) {
    goto yy58;
  } else {

  }
  switch ((int )yych) {
  case 45: 
  case 43: 
  goto yy440;
  case 49: 
  case 48: 
  goto yy437;
  case 50: 
  goto yy438;
  case 57: 
  case 56: 
  case 55: 
  case 54: 
  case 53: 
  case 52: 
  case 51: 
  goto yy439;
  default: 
  goto yy13;
  }
  yy32: 
  yyaccept = 0U;
  cursor ++;
  tmp___9 = cursor;
  s->ptr = tmp___9;
  yych = *tmp___9;
  if ((int )yych <= 57) {
    if ((int )yych <= 40) {
      if ((int )yych <= 9) {
        if ((int )yych <= 8) {
          goto yy4;
        } else {

        }
        goto yy196;
      } else {
        if ((int )yych == 32) {
          goto yy196;
        } else {

        }
        goto yy4;
      }
    } else
    if ((int )yych <= 44) {
      if ((int )yych <= 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 47) {
        goto yy4;
      } else {

      }
      goto yy196;
    }
  } else
  if ((int )yych <= 86) {
    if ((int )yych <= 72) {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy141;
    } else {
      if ((int )yych <= 73) {
        goto yy436;
      } else {

      }
      if ((int )yych <= 85) {
        goto yy141;
      } else {

      }
      goto yy435;
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych == 88) {
      goto yy435;
    } else {

    }
    goto yy141;
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy146;
    } else {

    }
    goto yy4;
  }
  yy33: 
  yyaccept = 0U;
  cursor ++;
  tmp___10 = cursor;
  s->ptr = tmp___10;
  yych = *tmp___10;
  if ((int )yych <= 46) {
    if ((int )yych <= 32) {
      if ((int )yych == 9) {
        goto yy196;
      } else {

      }
      if ((int )yych <= 31) {
        goto yy4;
      } else {

      }
      goto yy196;
    } else {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy196;
    }
  } else
  if ((int )yych <= 72) {
    if ((int )yych <= 47) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy196;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    goto yy141;
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 73) {
      goto yy432;
    } else {

    }
    goto yy141;
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy146;
    } else {

    }
    goto yy4;
  }
  yy34: 
  yyaccept = 0U;
  cursor ++;
  tmp___11 = cursor;
  s->ptr = tmp___11;
  yych = *tmp___11;
  if ((int )yych <= 46) {
    if ((int )yych <= 32) {
      if ((int )yych == 9) {
        goto yy196;
      } else {

      }
      if ((int )yych <= 31) {
        goto yy4;
      } else {

      }
      goto yy196;
    } else {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy196;
    }
  } else
  if ((int )yych <= 72) {
    if ((int )yych <= 47) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy196;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    goto yy141;
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 73) {
      goto yy430;
    } else {

    }
    goto yy141;
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy146;
    } else {

    }
    goto yy4;
  }
  yy35: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych <= 64) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych <= 65) {
        goto yy413;
      } else {

      }
      if ((int )yych <= 84) {
        goto yy141;
      } else {

      }
      goto yy412;
    }
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 90) {
      goto yy141;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy422;
  } else {
    if ((int )yych == 117) {
      goto yy421;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy146;
    } else {

    }
    goto yy4;
  }
  yy36: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych <= 64) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych <= 65) {
        goto yy413;
      } else {

      }
      if ((int )yych <= 84) {
        goto yy141;
      } else {

      }
      goto yy412;
    }
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 90) {
      goto yy141;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy413;
  } else {
    if ((int )yych == 117) {
      goto yy412;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy141;
    } else {

    }
    goto yy4;
  }
  yy37: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych <= 70) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy141;
    } else
    if ((int )yych <= 79) {
      if ((int )yych <= 71) {
        goto yy391;
      } else {

      }
      goto yy141;
    } else {
      if ((int )yych <= 80) {
        goto yy390;
      } else {

      }
      if ((int )yych <= 84) {
        goto yy141;
      } else {

      }
      goto yy389;
    }
  } else
  if ((int )yych <= 111) {
    if ((int )yych <= 96) {
      if ((int )yych <= 90) {
        goto yy141;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 103) {
        goto yy403;
      } else {

      }
      goto yy146;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych <= 112) {
      goto yy402;
    } else {

    }
    goto yy146;
  } else {
    if ((int )yych <= 117) {
      goto yy401;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy146;
    } else {

    }
    goto yy4;
  }
  yy38: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych <= 70) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy141;
    } else
    if ((int )yych <= 79) {
      if ((int )yych <= 71) {
        goto yy391;
      } else {

      }
      goto yy141;
    } else {
      if ((int )yych <= 80) {
        goto yy390;
      } else {

      }
      if ((int )yych <= 84) {
        goto yy141;
      } else {

      }
      goto yy389;
    }
  } else
  if ((int )yych <= 111) {
    if ((int )yych <= 96) {
      if ((int )yych <= 90) {
        goto yy141;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 103) {
        goto yy391;
      } else {

      }
      goto yy141;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych <= 112) {
      goto yy390;
    } else {

    }
    goto yy141;
  } else {
    if ((int )yych <= 117) {
      goto yy389;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy141;
    } else {

    }
    goto yy4;
  }
  yy39: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 67) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 66) {
        goto yy141;
      } else {

      }
      goto yy379;
    }
  } else
  if ((int )yych <= 98) {
    if ((int )yych <= 90) {
      goto yy141;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy146;
  } else {
    if ((int )yych <= 99) {
      goto yy384;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy146;
    } else {

    }
    goto yy4;
  }
  yy40: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 67) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 66) {
        goto yy141;
      } else {

      }
      goto yy379;
    }
  } else
  if ((int )yych <= 98) {
    if ((int )yych <= 90) {
      goto yy141;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy141;
  } else {
    if ((int )yych <= 99) {
      goto yy379;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy141;
    } else {

    }
    goto yy4;
  }
  yy41: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 69) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 68) {
        goto yy141;
      } else {

      }
      goto yy192;
    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 90) {
      goto yy141;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy146;
  } else {
    if ((int )yych <= 101) {
      goto yy370;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy146;
    } else {

    }
    goto yy4;
  }
  yy42: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 69) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 68) {
        goto yy141;
      } else {

      }
      goto yy192;
    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 90) {
      goto yy141;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy141;
  } else {
    if ((int )yych <= 101) {
      goto yy192;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy141;
    } else {

    }
    goto yy4;
  }
  yy43: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 69) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 68) {
        goto yy141;
      } else {

      }
      goto yy165;
    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 90) {
      goto yy141;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy146;
  } else {
    if ((int )yych <= 101) {
      goto yy179;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy146;
    } else {

    }
    goto yy4;
  }
  yy44: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 69) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 68) {
        goto yy141;
      } else {

      }
      goto yy165;
    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 90) {
      goto yy141;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy141;
  } else {
    if ((int )yych <= 101) {
      goto yy165;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy141;
    } else {

    }
    goto yy4;
  }
  yy45: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 76) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    goto yy141;
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 77) {
      goto yy157;
    } else {

    }
    goto yy141;
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy146;
    } else {

    }
    goto yy4;
  }
  yy46: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 64) {
    goto yy13;
  } else {

  }
  if ((int )yych <= 90) {
    goto yy156;
  } else {

  }
  if ((int )yych <= 96) {
    goto yy13;
  } else {

  }
  if ((int )yych <= 122) {
    goto yy156;
  } else {

  }
  goto yy13;
  yy47: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 64) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 90) {
      goto yy141;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy146;
    } else {

    }
    goto yy4;
  }
  yy48: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 64) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 90) {
      goto yy141;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy141;
    } else {

    }
    goto yy4;
  }
  yy49: 
  yyaccept = 2U;
  cursor ++;
  tmp___12 = cursor;
  s->ptr = tmp___12;
  yych = *tmp___12;
  if ((int const   )yybm[(int )yych] & 4) {
    goto yy58;
  } else {

  }
  if ((int )yych <= 47) {
    goto yy50;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy55;
  } else {

  }
  yy50: ;
  goto std;
  yy51: 
  cursor ++;
  yych = *cursor;
  goto yy50;
  yy52: 
  cursor ++;
  s->pos = cursor;
  (s->line) ++;
  goto std;
  yy54: 
  cursor ++;
  yych = *cursor;
  goto yy13;
  yy55: 
  cursor ++;
  if (s->lim - cursor < 11) {
    return (257);
  } else {

  }
  yych = *cursor;
  if ((int const   )yybm[(int )yych] & 2) {
    goto yy55;
  } else {

  }
  if ((int )yych <= 87) {
    if ((int )yych <= 70) {
      if ((int )yych <= 32) {
        if ((int )yych == 9) {
          goto yy60;
        } else {

        }
        if ((int )yych >= 32) {
          goto yy60;
        } else {

        }
      } else {
        if ((int )yych == 68) {
          goto yy65;
        } else {

        }
        if ((int )yych >= 70) {
          goto yy66;
        } else {

        }
      }
    } else
    if ((int )yych <= 77) {
      if ((int )yych == 72) {
        goto yy64;
      } else {

      }
      if ((int )yych >= 77) {
        goto yy63;
      } else {

      }
    } else
    if ((int )yych <= 83) {
      if ((int )yych >= 83) {
        goto yy62;
      } else {

      }
    } else {
      if ((int )yych <= 84) {
        goto yy69;
      } else {

      }
      if ((int )yych >= 87) {
        goto yy68;
      } else {

      }
    }
  } else
  if ((int )yych <= 108) {
    if ((int )yych <= 100) {
      if ((int )yych == 89) {
        goto yy67;
      } else {

      }
      if ((int )yych >= 100) {
        goto yy65;
      } else {

      }
    } else
    if ((int )yych <= 102) {
      if ((int )yych >= 102) {
        goto yy66;
      } else {

      }
    } else
    if ((int )yych == 104) {
      goto yy64;
    } else {

    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych <= 109) {
      goto yy63;
    } else {

    }
    if ((int )yych <= 114) {
      goto yy57;
    } else {

    }
    if ((int )yych <= 115) {
      goto yy62;
    } else {

    }
    goto yy69;
  } else
  if ((int )yych <= 119) {
    if ((int )yych >= 119) {
      goto yy68;
    } else {

    }
  } else
  if ((int )yych == 121) {
    goto yy67;
  } else {

  }
  yy57: 
  cursor = s->ptr;
  if (yyaccept <= 16U) {
    if (yyaccept <= 8U) {
      if (yyaccept <= 4U) {
        if (yyaccept <= 2U) {
          if (yyaccept <= 1U) {
            if (yyaccept <= 0U) {
              goto yy4;
            } else {
              goto yy13;
            }
          } else {
            goto yy50;
          }
        } else
        if (yyaccept <= 3U) {
          goto yy73;
        } else {
          goto yy167;
        }
      } else
      if (yyaccept <= 6U) {
        if (yyaccept <= 5U) {
          goto yy194;
        } else {
          goto yy199;
        }
      } else
      if (yyaccept <= 7U) {
        goto yy223;
      } else {
        goto yy295;
      }
    } else
    if (yyaccept <= 12U) {
      if (yyaccept <= 10U) {
        if (yyaccept <= 9U) {
          goto yy393;
        } else {
          goto yy476;
        }
      } else
      if (yyaccept <= 11U) {
        goto yy491;
      } else {
        goto yy612;
      }
    } else
    if (yyaccept <= 14U) {
      if (yyaccept <= 13U) {
        goto yy657;
      } else {
        goto yy667;
      }
    } else
    if (yyaccept <= 15U) {
      goto yy764;
    } else {
      goto yy784;
    }
  } else
  if (yyaccept <= 25U) {
    if (yyaccept <= 21U) {
      if (yyaccept <= 19U) {
        if (yyaccept <= 18U) {
          if (yyaccept <= 17U) {
            goto yy815;
          } else {
            goto yy822;
          }
        } else {
          goto yy849;
        }
      } else
      if (yyaccept <= 20U) {
        goto yy794;
      } else {
        goto yy455;
      }
    } else
    if (yyaccept <= 23U) {
      if (yyaccept <= 22U) {
        goto yy974;
      } else {
        goto yy843;
      }
    } else
    if (yyaccept <= 24U) {
      goto yy1068;
    } else {
      goto yy1076;
    }
  } else
  if (yyaccept <= 29U) {
    if (yyaccept <= 27U) {
      if (yyaccept <= 26U) {
        goto yy1118;
      } else {
        goto yy1142;
      }
    } else
    if (yyaccept <= 28U) {
      goto yy1295;
    } else {
      goto yy1417;
    }
  } else
  if (yyaccept <= 31U) {
    if (yyaccept <= 30U) {
      goto yy1420;
    } else {
      goto yy1500;
    }
  } else
  if (yyaccept <= 32U) {
    goto yy1508;
  } else {
    goto yy1531;
  }
  yy58: 
  cursor ++;
  if ((unsigned int )s->lim <= (unsigned int )cursor) {
    return (257);
  } else {

  }
  yych = *cursor;
  if ((int const   )yybm[(int )yych] & 4) {
    goto yy58;
  } else {

  }
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy55;
  } else {

  }
  goto yy57;
  yy60: 
  cursor ++;
  if (s->lim - cursor < 11) {
    return (257);
  } else {

  }
  yych = *cursor;
  yy61: ;
  if ((int )yych <= 87) {
    if ((int )yych <= 70) {
      if ((int )yych <= 32) {
        if ((int )yych == 9) {
          goto yy60;
        } else {

        }
        if ((int )yych <= 31) {
          goto yy57;
        } else {

        }
        goto yy60;
      } else {
        if ((int )yych == 68) {
          goto yy65;
        } else {

        }
        if ((int )yych <= 69) {
          goto yy57;
        } else {

        }
        goto yy66;
      }
    } else
    if ((int )yych <= 77) {
      if ((int )yych == 72) {
        goto yy64;
      } else {

      }
      if ((int )yych <= 76) {
        goto yy57;
      } else {

      }
      goto yy63;
    } else
    if ((int )yych <= 83) {
      if ((int )yych <= 82) {
        goto yy57;
      } else {

      }
    } else {
      if ((int )yych <= 84) {
        goto yy69;
      } else {

      }
      if ((int )yych <= 86) {
        goto yy57;
      } else {

      }
      goto yy68;
    }
  } else
  if ((int )yych <= 108) {
    if ((int )yych <= 100) {
      if ((int )yych == 89) {
        goto yy67;
      } else {

      }
      if ((int )yych <= 99) {
        goto yy57;
      } else {

      }
      goto yy65;
    } else
    if ((int )yych <= 102) {
      if ((int )yych <= 101) {
        goto yy57;
      } else {

      }
      goto yy66;
    } else {
      if ((int )yych == 104) {
        goto yy64;
      } else {

      }
      goto yy57;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych <= 109) {
      goto yy63;
    } else {

    }
    if ((int )yych <= 114) {
      goto yy57;
    } else {

    }
    if ((int )yych >= 116) {
      goto yy69;
    } else {

    }
  } else
  if ((int )yych <= 119) {
    if ((int )yych <= 118) {
      goto yy57;
    } else {

    }
    goto yy68;
  } else {
    if ((int )yych == 121) {
      goto yy67;
    } else {

    }
    goto yy57;
  }
  yy62: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych <= 68) {
      if ((int )yych == 65) {
        goto yy127;
      } else {

      }
      goto yy57;
    } else {
      if ((int )yych <= 69) {
        goto yy128;
      } else {

      }
      if ((int )yych <= 84) {
        goto yy57;
      } else {

      }
      goto yy126;
    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych == 97) {
      goto yy127;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 101) {
      goto yy128;
    } else {

    }
    if ((int )yych == 117) {
      goto yy126;
    } else {

    }
    goto yy57;
  }
  yy63: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 79) {
    if ((int )yych == 73) {
      goto yy118;
    } else {

    }
    if ((int )yych <= 78) {
      goto yy57;
    } else {

    }
    goto yy117;
  } else
  if ((int )yych <= 105) {
    if ((int )yych <= 104) {
      goto yy57;
    } else {

    }
    goto yy118;
  } else {
    if ((int )yych == 111) {
      goto yy117;
    } else {

    }
    goto yy57;
  }
  yy64: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 79) {
    goto yy115;
  } else {

  }
  if ((int )yych == 111) {
    goto yy115;
  } else {

  }
  goto yy57;
  yy65: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy114;
  } else {

  }
  if ((int )yych == 97) {
    goto yy114;
  } else {

  }
  goto yy57;
  yy66: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 82) {
    if ((int )yych == 79) {
      goto yy99;
    } else {

    }
    if ((int )yych <= 81) {
      goto yy57;
    } else {

    }
    goto yy98;
  } else
  if ((int )yych <= 111) {
    if ((int )yych <= 110) {
      goto yy57;
    } else {

    }
    goto yy99;
  } else {
    if ((int )yych == 114) {
      goto yy98;
    } else {

    }
    goto yy57;
  }
  yy67: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy95;
  } else {

  }
  if ((int )yych == 101) {
    goto yy95;
  } else {

  }
  goto yy57;
  yy68: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy83;
  } else {

  }
  if ((int )yych == 101) {
    goto yy83;
  } else {

  }
  goto yy57;
  yy69: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych == 72) {
      goto yy70;
    } else {

    }
    if ((int )yych <= 84) {
      goto yy57;
    } else {

    }
    goto yy71;
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 103) {
      goto yy57;
    } else {

    }
  } else {
    if ((int )yych == 117) {
      goto yy71;
    } else {

    }
    goto yy57;
  }
  yy70: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 85) {
    goto yy78;
  } else {

  }
  if ((int )yych == 117) {
    goto yy78;
  } else {

  }
  goto yy57;
  yy71: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy72;
  } else {

  }
  if ((int )yych != 101) {
    goto yy57;
  } else {

  }
  yy72: 
  yyaccept = 3U;
  cursor ++;
  tmp___13 = cursor;
  s->ptr = tmp___13;
  yych = *tmp___13;
  if ((int )yych == 83) {
    goto yy74;
  } else {

  }
  if ((int )yych == 115) {
    goto yy74;
  } else {

  }
  yy73: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  (s->time)->have_relative = 1U;
  while (*ptr) {
    i = timelib_get_unsigned_nr(& ptr, 24);
    timelib_eat_spaces(& ptr);
    timelib_set_relative(& ptr, (timelib_sll )i, 1, s);
  }
  free((void *)str);
  return (310);
  yy74: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 68) {
    goto yy75;
  } else {

  }
  if ((int )yych != 100) {
    goto yy57;
  } else {

  }
  yy75: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy76;
  } else {

  }
  if ((int )yych != 97) {
    goto yy57;
  } else {

  }
  yy76: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy77;
  } else {

  }
  if ((int )yych != 121) {
    goto yy57;
  } else {

  }
  yy77: 
  cursor ++;
  yych = *cursor;
  goto yy73;
  yy78: 
  yyaccept = 3U;
  cursor ++;
  tmp___14 = cursor;
  s->ptr = tmp___14;
  yych = *tmp___14;
  if ((int )yych == 82) {
    goto yy79;
  } else {

  }
  if ((int )yych != 114) {
    goto yy73;
  } else {

  }
  yy79: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 83) {
    goto yy80;
  } else {

  }
  if ((int )yych != 115) {
    goto yy57;
  } else {

  }
  yy80: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 68) {
    goto yy81;
  } else {

  }
  if ((int )yych != 100) {
    goto yy57;
  } else {

  }
  yy81: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy82;
  } else {

  }
  if ((int )yych != 97) {
    goto yy57;
  } else {

  }
  yy82: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy77;
  } else {

  }
  if ((int )yych == 121) {
    goto yy77;
  } else {

  }
  goto yy57;
  yy83: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 69) {
    if ((int )yych <= 67) {
      goto yy57;
    } else {

    }
    if ((int )yych <= 68) {
      goto yy85;
    } else {

    }
  } else {
    if ((int )yych <= 99) {
      goto yy57;
    } else {

    }
    if ((int )yych <= 100) {
      goto yy85;
    } else {

    }
    if ((int )yych >= 102) {
      goto yy57;
    } else {

    }
  }
  cursor ++;
  yych = *cursor;
  if ((int )yych == 75) {
    goto yy91;
  } else {

  }
  if ((int )yych == 107) {
    goto yy91;
  } else {

  }
  goto yy57;
  yy85: 
  yyaccept = 3U;
  cursor ++;
  tmp___15 = cursor;
  s->ptr = tmp___15;
  yych = *tmp___15;
  if ((int )yych == 78) {
    goto yy86;
  } else {

  }
  if ((int )yych != 110) {
    goto yy73;
  } else {

  }
  yy86: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy87;
  } else {

  }
  if ((int )yych != 101) {
    goto yy57;
  } else {

  }
  yy87: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 83) {
    goto yy88;
  } else {

  }
  if ((int )yych != 115) {
    goto yy57;
  } else {

  }
  yy88: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 68) {
    goto yy89;
  } else {

  }
  if ((int )yych != 100) {
    goto yy57;
  } else {

  }
  yy89: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy90;
  } else {

  }
  if ((int )yych != 97) {
    goto yy57;
  } else {

  }
  yy90: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy77;
  } else {

  }
  if ((int )yych == 121) {
    goto yy77;
  } else {

  }
  goto yy57;
  yy91: 
  yyaccept = 3U;
  cursor ++;
  tmp___16 = cursor;
  s->ptr = tmp___16;
  yych = *tmp___16;
  if ((int )yych <= 83) {
    if ((int )yych == 68) {
      goto yy92;
    } else {

    }
    if ((int )yych <= 82) {
      goto yy73;
    } else {

    }
    goto yy77;
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 99) {
      goto yy73;
    } else {

    }
  } else {
    if ((int )yych == 115) {
      goto yy77;
    } else {

    }
    goto yy73;
  }
  yy92: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy93;
  } else {

  }
  if ((int )yych != 97) {
    goto yy57;
  } else {

  }
  yy93: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy94;
  } else {

  }
  if ((int )yych != 121) {
    goto yy57;
  } else {

  }
  yy94: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 83) {
    goto yy77;
  } else {

  }
  if ((int )yych == 115) {
    goto yy77;
  } else {

  }
  goto yy73;
  yy95: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy96;
  } else {

  }
  if ((int )yych != 97) {
    goto yy57;
  } else {

  }
  yy96: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy97;
  } else {

  }
  if ((int )yych != 114) {
    goto yy57;
  } else {

  }
  yy97: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 83) {
    goto yy77;
  } else {

  }
  if ((int )yych == 115) {
    goto yy77;
  } else {

  }
  goto yy73;
  yy98: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 73) {
    goto yy111;
  } else {

  }
  if ((int )yych == 105) {
    goto yy111;
  } else {

  }
  goto yy57;
  yy99: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy100;
  } else {

  }
  if ((int )yych != 114) {
    goto yy57;
  } else {

  }
  yy100: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 84) {
    goto yy101;
  } else {

  }
  if ((int )yych != 116) {
    goto yy57;
  } else {

  }
  yy101: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 78) {
    if ((int )yych == 72) {
      goto yy103;
    } else {

    }
    if ((int )yych <= 77) {
      goto yy57;
    } else {

    }
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 103) {
      goto yy57;
    } else {

    }
    goto yy103;
  } else
  if ((int )yych != 110) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych == 73) {
    goto yy108;
  } else {

  }
  if ((int )yych == 105) {
    goto yy108;
  } else {

  }
  goto yy57;
  yy103: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 78) {
    goto yy104;
  } else {

  }
  if ((int )yych != 110) {
    goto yy57;
  } else {

  }
  yy104: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 73) {
    goto yy105;
  } else {

  }
  if ((int )yych != 105) {
    goto yy57;
  } else {

  }
  yy105: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 71) {
    goto yy106;
  } else {

  }
  if ((int )yych != 103) {
    goto yy57;
  } else {

  }
  yy106: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 72) {
    goto yy107;
  } else {

  }
  if ((int )yych != 104) {
    goto yy57;
  } else {

  }
  yy107: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 84) {
    goto yy97;
  } else {

  }
  if ((int )yych == 116) {
    goto yy97;
  } else {

  }
  goto yy57;
  yy108: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 71) {
    goto yy109;
  } else {

  }
  if ((int )yych != 103) {
    goto yy57;
  } else {

  }
  yy109: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 72) {
    goto yy110;
  } else {

  }
  if ((int )yych != 104) {
    goto yy57;
  } else {

  }
  yy110: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 84) {
    goto yy97;
  } else {

  }
  if ((int )yych == 116) {
    goto yy97;
  } else {

  }
  goto yy57;
  yy111: 
  yyaccept = 3U;
  cursor ++;
  tmp___17 = cursor;
  s->ptr = tmp___17;
  yych = *tmp___17;
  if ((int )yych == 68) {
    goto yy112;
  } else {

  }
  if ((int )yych != 100) {
    goto yy73;
  } else {

  }
  yy112: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy113;
  } else {

  }
  if ((int )yych != 97) {
    goto yy57;
  } else {

  }
  yy113: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy77;
  } else {

  }
  if ((int )yych == 121) {
    goto yy77;
  } else {

  }
  goto yy57;
  yy114: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy97;
  } else {

  }
  if ((int )yych == 121) {
    goto yy97;
  } else {

  }
  goto yy57;
  yy115: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 85) {
    goto yy116;
  } else {

  }
  if ((int )yych != 117) {
    goto yy57;
  } else {

  }
  yy116: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy97;
  } else {

  }
  if ((int )yych == 114) {
    goto yy97;
  } else {

  }
  goto yy57;
  yy117: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 78) {
    goto yy122;
  } else {

  }
  if ((int )yych == 110) {
    goto yy122;
  } else {

  }
  goto yy57;
  yy118: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 78) {
    goto yy119;
  } else {

  }
  if ((int )yych != 110) {
    goto yy57;
  } else {

  }
  yy119: 
  yyaccept = 3U;
  cursor ++;
  tmp___18 = cursor;
  s->ptr = tmp___18;
  yych = *tmp___18;
  if ((int )yych <= 85) {
    if ((int )yych == 83) {
      goto yy77;
    } else {

    }
    if ((int )yych <= 84) {
      goto yy73;
    } else {

    }
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 114) {
      goto yy73;
    } else {

    }
    goto yy77;
  } else
  if ((int )yych != 117) {
    goto yy73;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych == 84) {
    goto yy121;
  } else {

  }
  if ((int )yych != 116) {
    goto yy57;
  } else {

  }
  yy121: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy97;
  } else {

  }
  if ((int )yych == 101) {
    goto yy97;
  } else {

  }
  goto yy57;
  yy122: 
  yyaccept = 3U;
  cursor ++;
  tmp___19 = cursor;
  s->ptr = tmp___19;
  yych = *tmp___19;
  if ((int )yych <= 84) {
    if ((int )yych == 68) {
      goto yy123;
    } else {

    }
    if ((int )yych <= 83) {
      goto yy73;
    } else {

    }
    goto yy124;
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 99) {
      goto yy73;
    } else {

    }
  } else {
    if ((int )yych == 116) {
      goto yy124;
    } else {

    }
    goto yy73;
  }
  yy123: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy125;
  } else {

  }
  if ((int )yych == 97) {
    goto yy125;
  } else {

  }
  goto yy57;
  yy124: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 72) {
    goto yy97;
  } else {

  }
  if ((int )yych == 104) {
    goto yy97;
  } else {

  }
  goto yy57;
  yy125: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy77;
  } else {

  }
  if ((int )yych == 121) {
    goto yy77;
  } else {

  }
  goto yy57;
  yy126: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 78) {
    goto yy137;
  } else {

  }
  if ((int )yych == 110) {
    goto yy137;
  } else {

  }
  goto yy57;
  yy127: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 84) {
    goto yy132;
  } else {

  }
  if ((int )yych == 116) {
    goto yy132;
  } else {

  }
  goto yy57;
  yy128: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 67) {
    goto yy129;
  } else {

  }
  if ((int )yych != 99) {
    goto yy57;
  } else {

  }
  yy129: 
  yyaccept = 3U;
  cursor ++;
  tmp___20 = cursor;
  s->ptr = tmp___20;
  yych = *tmp___20;
  if ((int )yych <= 83) {
    if ((int )yych == 79) {
      goto yy130;
    } else {

    }
    if ((int )yych <= 82) {
      goto yy73;
    } else {

    }
    goto yy77;
  } else
  if ((int )yych <= 111) {
    if ((int )yych <= 110) {
      goto yy73;
    } else {

    }
  } else {
    if ((int )yych == 115) {
      goto yy77;
    } else {

    }
    goto yy73;
  }
  yy130: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 78) {
    goto yy131;
  } else {

  }
  if ((int )yych != 110) {
    goto yy57;
  } else {

  }
  yy131: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 68) {
    goto yy97;
  } else {

  }
  if ((int )yych == 100) {
    goto yy97;
  } else {

  }
  goto yy57;
  yy132: 
  yyaccept = 3U;
  cursor ++;
  tmp___21 = cursor;
  s->ptr = tmp___21;
  yych = *tmp___21;
  if ((int )yych == 85) {
    goto yy133;
  } else {

  }
  if ((int )yych != 117) {
    goto yy73;
  } else {

  }
  yy133: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy134;
  } else {

  }
  if ((int )yych != 114) {
    goto yy57;
  } else {

  }
  yy134: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 68) {
    goto yy135;
  } else {

  }
  if ((int )yych != 100) {
    goto yy57;
  } else {

  }
  yy135: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy136;
  } else {

  }
  if ((int )yych != 97) {
    goto yy57;
  } else {

  }
  yy136: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy77;
  } else {

  }
  if ((int )yych == 121) {
    goto yy77;
  } else {

  }
  goto yy57;
  yy137: 
  yyaccept = 3U;
  cursor ++;
  tmp___22 = cursor;
  s->ptr = tmp___22;
  yych = *tmp___22;
  if ((int )yych == 68) {
    goto yy138;
  } else {

  }
  if ((int )yych != 100) {
    goto yy73;
  } else {

  }
  yy138: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy139;
  } else {

  }
  if ((int )yych != 97) {
    goto yy57;
  } else {

  }
  yy139: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy77;
  } else {

  }
  if ((int )yych == 121) {
    goto yy77;
  } else {

  }
  goto yy57;
  yy140: 
  cursor ++;
  yych = *cursor;
  goto yy4;
  yy141: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 64) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych >= 123) {
      goto yy4;
    } else {

    }
  }
  yy142: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 64) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych >= 123) {
      goto yy4;
    } else {

    }
  }
  yy143: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 64) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych >= 123) {
      goto yy4;
    } else {

    }
  }
  yy144: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 64) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych >= 123) {
      goto yy4;
    } else {

    }
  }
  yy145: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 41) {
    goto yy140;
  } else {

  }
  goto yy4;
  yy146: 
  yyaccept = 0U;
  cursor ++;
  tmp___23 = cursor;
  s->ptr = tmp___23;
  yych = *tmp___23;
  if ((int )yych <= 47) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      goto yy148;
    }
  } else
  if ((int )yych <= 94) {
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 95) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych >= 123) {
      goto yy4;
    } else {

    }
  }
  yy147: 
  yyaccept = 0U;
  cursor ++;
  tmp___24 = cursor;
  s->ptr = tmp___24;
  yych = *tmp___24;
  if ((int )yych <= 47) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else
    if ((int )yych == 46) {
      goto yy4;
    } else {

    }
  } else
  if ((int )yych <= 94) {
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 95) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy4;
  }
  yy148: 
  cursor ++;
  if ((unsigned int )s->lim <= (unsigned int )cursor) {
    return (257);
  } else {

  }
  yych = *cursor;
  if ((int const   )yybm[(int )yych] & 8) {
    goto yy149;
  } else {

  }
  goto yy57;
  yy149: 
  yyaccept = 0U;
  cursor ++;
  s->ptr = cursor;
  if ((unsigned int )s->lim <= (unsigned int )cursor) {
    return (257);
  } else {

  }
  yych = *cursor;
  if ((int const   )yybm[(int )yych] & 8) {
    goto yy149;
  } else {

  }
  if ((int )yych <= 46) {
    if ((int )yych == 45) {
      goto yy148;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 47) {
      goto yy148;
    } else {

    }
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy4;
  }
  yy151: 
  yyaccept = 0U;
  cursor ++;
  tmp___25 = cursor;
  s->ptr = tmp___25;
  yych = *tmp___25;
  if ((int )yych <= 47) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      goto yy148;
    }
  } else
  if ((int )yych <= 94) {
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 95) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych >= 123) {
      goto yy4;
    } else {

    }
  }
  yy152: 
  yyaccept = 0U;
  cursor ++;
  tmp___26 = cursor;
  s->ptr = tmp___26;
  yych = *tmp___26;
  if ((int )yych <= 47) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      goto yy148;
    }
  } else
  if ((int )yych <= 94) {
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 95) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych >= 123) {
      goto yy4;
    } else {

    }
  }
  yy153: 
  yyaccept = 0U;
  cursor ++;
  tmp___27 = cursor;
  s->ptr = tmp___27;
  yych = *tmp___27;
  if ((int const   )yybm[(int )yych] & 16) {
    goto yy154;
  } else {

  }
  if ((int )yych <= 45) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    if ((int )yych <= 44) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 47) {
    if ((int )yych <= 46) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else {
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy4;
  }
  yy154: 
  cursor ++;
  if ((unsigned int )s->lim <= (unsigned int )cursor) {
    return (257);
  } else {

  }
  yych = *cursor;
  yy155: ;
  if ((int const   )yybm[(int )yych] & 16) {
    goto yy154;
  } else {

  }
  if ((int )yych <= 46) {
    if ((int )yych == 45) {
      goto yy148;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 47) {
      goto yy148;
    } else {

    }
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy57;
  }
  yy156: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 64) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 90) {
      goto yy141;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy141;
    } else {

    }
    goto yy4;
  }
  yy157: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 83) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else
  if ((int )yych <= 90) {
    if ((int )yych >= 85) {
      goto yy142;
    } else {

    }
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yyaccept = 0U;
  cursor ++;
  tmp___28 = cursor;
  s->ptr = tmp___28;
  yych = *tmp___28;
  if ((int )yych <= 44) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else
    if ((int )yych != 43) {
      goto yy4;
    } else {

    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 45) {
      goto yy159;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy4;
  }
  yy159: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 49) {
    goto yy160;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy161;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy162;
  } else {

  }
  goto yy57;
  yy160: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy4;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy162;
  } else {

  }
  if ((int )yych <= 58) {
    goto yy163;
  } else {

  }
  goto yy4;
  yy161: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 53) {
    if ((int )yych <= 47) {
      goto yy4;
    } else {

    }
    if ((int )yych >= 53) {
      goto yy164;
    } else {

    }
  } else {
    if ((int )yych <= 57) {
      goto yy140;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy163;
    } else {

    }
    goto yy4;
  }
  yy162: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy4;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy164;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy140;
  } else {

  }
  if ((int )yych >= 59) {
    goto yy4;
  } else {

  }
  yy163: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy4;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy164;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy140;
  } else {

  }
  goto yy4;
  yy164: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy4;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy140;
  } else {

  }
  goto yy4;
  yy165: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 69) {
    if ((int )yych <= 64) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych <= 67) {
        goto yy142;
      } else {

      }
      if ((int )yych >= 69) {
        goto yy168;
      } else {

      }
    }
  } else
  if ((int )yych <= 99) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else {
    if ((int )yych <= 100) {
      goto yy166;
    } else {

    }
    if ((int )yych <= 101) {
      goto yy168;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy166: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 78) {
    if ((int )yych <= 41) {
      if ((int )yych >= 41) {
        goto yy140;
      } else {

      }
    } else {
      if ((int )yych <= 64) {
        goto yy167;
      } else {

      }
      if ((int )yych <= 77) {
        goto yy143;
      } else {

      }
      goto yy174;
    }
  } else
  if ((int )yych <= 109) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych >= 97) {
      goto yy143;
    } else {

    }
  } else {
    if ((int )yych <= 110) {
      goto yy174;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
  }
  yy167: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  (s->time)->have_relative = 1U;
  (s->time)->have_relative = 1U;
  (s->time)->relative.have_weekday_relative = 1U;
  (s->time)->have_time = 0U;
  (s->time)->h = (timelib_sll )0;
  (s->time)->i = (timelib_sll )0;
  (s->time)->s = (timelib_sll )0;
  (s->time)->f = (double )0;
  relunit = timelib_lookup_relunit(& ptr);
  (s->time)->relative.weekday = (int )relunit->multiplier;
  if ((s->time)->relative.weekday_behavior != 2) {
    (s->time)->relative.weekday_behavior = 1;
  } else {

  }
  free((void *)str);
  return (7);
  yy168: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 75) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 74) {
        goto yy143;
      } else {

      }
    }
  } else
  if ((int )yych <= 106) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 107) {
      goto yy169;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy4;
  }
  yy169: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 68) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 67) {
        goto yy144;
      } else {

      }
    }
  } else
  if ((int )yych <= 99) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 100) {
      goto yy170;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy170: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 65) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 97) {
      goto yy171;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy4;
  }
  yy171: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 88) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 89) {
      goto yy172;
    } else {

    }
    if ((int )yych != 121) {
      goto yy4;
    } else {

    }
  }
  yy172: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 83) {
    goto yy173;
  } else {

  }
  if ((int )yych != 115) {
    goto yy167;
  } else {

  }
  yy173: 
  cursor ++;
  yych = *cursor;
  goto yy167;
  yy174: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 69) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 68) {
        goto yy144;
      } else {

      }
    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 101) {
      goto yy175;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy175: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 83) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 82) {
        goto yy145;
      } else {

      }
    }
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy145;
  } else {
    if ((int )yych <= 115) {
      goto yy176;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy4;
  }
  yy176: 
  yyaccept = 0U;
  cursor ++;
  tmp___29 = cursor;
  s->ptr = tmp___29;
  yych = *tmp___29;
  if ((int )yych <= 67) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 68) {
      goto yy177;
    } else {

    }
    if ((int )yych != 100) {
      goto yy4;
    } else {

    }
  }
  yy177: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy178;
  } else {

  }
  if ((int )yych != 97) {
    goto yy57;
  } else {

  }
  yy178: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy173;
  } else {

  }
  if ((int )yych == 121) {
    goto yy173;
  } else {

  }
  goto yy57;
  yy179: 
  yyaccept = 0U;
  cursor ++;
  tmp___30 = cursor;
  s->ptr = tmp___30;
  yych = *tmp___30;
  if ((int )yych <= 68) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else
    if ((int )yych <= 47) {
      if ((int )yych <= 46) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 67) {
        goto yy142;
      } else {

      }
      goto yy166;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 90) {
      if ((int )yych <= 69) {
        goto yy168;
      } else {

      }
      goto yy142;
    } else {
      if ((int )yych == 95) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 99) {
      goto yy147;
    } else {

    }
  } else {
    if ((int )yych <= 101) {
      goto yy181;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yyaccept = 4U;
  cursor ++;
  tmp___31 = cursor;
  s->ptr = tmp___31;
  yych = *tmp___31;
  if ((int )yych <= 77) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy167;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy167;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 78) {
      goto yy174;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy167;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 109) {
    if ((int )yych <= 96) {
      goto yy167;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 110) {
      goto yy187;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy167;
  }
  yy181: 
  yyaccept = 0U;
  cursor ++;
  tmp___32 = cursor;
  s->ptr = tmp___32;
  yych = *tmp___32;
  if ((int )yych <= 74) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 75) {
      goto yy169;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 106) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 107) {
      goto yy182;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy4;
  }
  yy182: 
  yyaccept = 0U;
  cursor ++;
  tmp___33 = cursor;
  s->ptr = tmp___33;
  yych = *tmp___33;
  if ((int )yych <= 67) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 68) {
      goto yy170;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 99) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 100) {
      goto yy183;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy183: 
  yyaccept = 0U;
  cursor ++;
  tmp___34 = cursor;
  s->ptr = tmp___34;
  yych = *tmp___34;
  if ((int )yych <= 64) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 65) {
      goto yy171;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 97) {
      goto yy184;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy4;
  }
  yy184: 
  yyaccept = 0U;
  cursor ++;
  tmp___35 = cursor;
  s->ptr = tmp___35;
  yych = *tmp___35;
  if ((int )yych <= 88) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 89) {
      goto yy172;
    } else {

    }
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych == 121) {
      goto yy185;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy154;
    } else {

    }
    goto yy4;
  }
  yy185: 
  yyaccept = 4U;
  cursor ++;
  tmp___36 = cursor;
  s->ptr = tmp___36;
  yych = *tmp___36;
  if ((int )yych <= 83) {
    if ((int )yych <= 46) {
      if ((int )yych == 45) {
        goto yy148;
      } else {

      }
      goto yy167;
    } else {
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 82) {
        goto yy167;
      } else {

      }
      goto yy173;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy167;
  } else {
    if ((int )yych == 115) {
      goto yy186;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy154;
    } else {

    }
    goto yy167;
  }
  yy186: 
  yyaccept = 4U;
  cursor ++;
  tmp___37 = cursor;
  s->ptr = tmp___37;
  yych = *tmp___37;
  if ((int const   )yybm[(int )yych] & 16) {
    goto yy154;
  } else {

  }
  if ((int )yych <= 46) {
    if ((int )yych == 45) {
      goto yy148;
    } else {

    }
    goto yy167;
  } else {
    if ((int )yych <= 47) {
      goto yy148;
    } else {

    }
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy167;
  }
  yy187: 
  yyaccept = 0U;
  cursor ++;
  tmp___38 = cursor;
  s->ptr = tmp___38;
  yych = *tmp___38;
  if ((int )yych <= 68) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 69) {
      goto yy175;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 101) {
      goto yy188;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy188: 
  yyaccept = 0U;
  cursor ++;
  tmp___39 = cursor;
  s->ptr = tmp___39;
  yych = *tmp___39;
  if ((int )yych <= 82) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy145;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 83) {
      goto yy176;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy153;
  } else {
    if ((int )yych <= 115) {
      goto yy189;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy4;
  }
  yy189: 
  yyaccept = 0U;
  cursor ++;
  tmp___40 = cursor;
  s->ptr = tmp___40;
  yych = *tmp___40;
  if ((int )yych <= 67) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 68) {
      goto yy177;
    } else {

    }
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych == 100) {
      goto yy190;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy154;
    } else {

    }
    goto yy4;
  }
  yy190: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy178;
  } else {

  }
  if ((int )yych != 97) {
    goto yy155;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy173;
  } else {

  }
  if ((int )yych == 121) {
    goto yy186;
  } else {

  }
  goto yy155;
  yy192: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 67) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 66) {
        goto yy142;
      } else {

      }
    }
  } else
  if ((int )yych <= 98) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else {
    if ((int )yych <= 99) {
      goto yy193;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy193: 
  yyaccept = 5U;
  cursor ++;
  tmp___41 = cursor;
  s->ptr = tmp___41;
  yych = *tmp___41;
  if ((int )yych <= 47) {
    if ((int )yych <= 40) {
      if ((int )yych <= 9) {
        if ((int )yych >= 9) {
          goto yy196;
        } else {

        }
      } else
      if ((int )yych == 32) {
        goto yy196;
      } else {

      }
    } else
    if ((int )yych <= 44) {
      if ((int )yych <= 41) {
        goto yy140;
      } else {

      }
    } else {
      if ((int )yych <= 45) {
        goto yy197;
      } else {

      }
      if ((int )yych <= 46) {
        goto yy196;
      } else {

      }
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 64) {
      if ((int )yych <= 57) {
        goto yy196;
      } else {

      }
    } else {
      if ((int )yych == 69) {
        goto yy202;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych >= 97) {
      goto yy143;
    } else {

    }
  } else {
    if ((int )yych <= 101) {
      goto yy202;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
  }
  yy194: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  if ((s->time)->have_date) {
    add_error(s, (char *)"Double date specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_date = 1U;
  }
  tmp___42 = timelib_lookup_month(& ptr);
  (s->time)->m = (timelib_sll )tmp___42;
  free((void *)str);
  return (269);
  yy195: 
  cursor ++;
  if (s->lim - cursor < 21) {
    return (257);
  } else {

  }
  yych = *cursor;
  yy196: ;
  if ((int const   )yybm[(int )yych] & 32) {
    goto yy195;
  } else {

  }
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy198;
  } else {

  }
  if ((int )yych <= 51) {
    goto yy200;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy201;
  } else {

  }
  goto yy57;
  yy197: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy196;
  } else {

  }
  if ((int )yych <= 48) {
    goto yy357;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy358;
  } else {

  }
  if ((int )yych <= 51) {
    goto yy359;
  } else {

  }
  goto yy196;
  yy198: 
  yyaccept = 6U;
  cursor ++;
  tmp___43 = cursor;
  s->ptr = tmp___43;
  yych = *tmp___43;
  if ((int )yych <= 109) {
    if ((int )yych <= 49) {
      if ((int )yych <= 47) {
        goto yy216;
      } else {

      }
      if ((int )yych <= 48) {
        goto yy298;
      } else {

      }
      goto yy299;
    } else {
      if ((int )yych <= 50) {
        goto yy355;
      } else {

      }
      if ((int )yych <= 57) {
        goto yy356;
      } else {

      }
      goto yy216;
    }
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 110) {
      goto yy212;
    } else {

    }
    if ((int )yych <= 113) {
      goto yy216;
    } else {

    }
    goto yy213;
  } else {
    if ((int )yych <= 115) {
      goto yy211;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy214;
    } else {

    }
    goto yy216;
  }
  yy199: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  if ((s->time)->have_date) {
    add_error(s, (char *)"Double date specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_date = 1U;
  }
  tmp___44 = timelib_get_month(& ptr);
  (s->time)->m = (timelib_sll )tmp___44;
  (s->time)->d = timelib_get_nr(& ptr, 2);
  (s->time)->y = timelib_get_nr(& ptr, 4);
  if ((s->time)->y == -99999LL) {

  } else
  if ((s->time)->y < 100LL) {
    if ((s->time)->y < 70LL) {
      (s->time)->y += 2000LL;
    } else {
      (s->time)->y += 1900LL;
    }
  } else {

  }
  free((void *)str);
  return (269);
  yy200: 
  yyaccept = 6U;
  cursor ++;
  tmp___45 = cursor;
  s->ptr = tmp___45;
  yych = *tmp___45;
  if ((int )yych <= 109) {
    if ((int )yych <= 49) {
      if ((int )yych <= 47) {
        goto yy216;
      } else {

      }
      if ((int )yych <= 48) {
        goto yy298;
      } else {

      }
      goto yy299;
    } else {
      if ((int )yych <= 50) {
        goto yy209;
      } else {

      }
      if ((int )yych <= 57) {
        goto yy210;
      } else {

      }
      goto yy216;
    }
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 110) {
      goto yy212;
    } else {

    }
    if ((int )yych <= 113) {
      goto yy216;
    } else {

    }
    goto yy213;
  } else {
    if ((int )yych <= 115) {
      goto yy211;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy214;
    } else {

    }
    goto yy216;
  }
  yy201: 
  yyaccept = 6U;
  cursor ++;
  tmp___46 = cursor;
  s->ptr = tmp___46;
  yych = *tmp___46;
  if ((int )yych <= 109) {
    if ((int )yych <= 49) {
      if ((int )yych <= 47) {
        goto yy216;
      } else {

      }
      if ((int )yych <= 48) {
        goto yy207;
      } else {

      }
      goto yy208;
    } else {
      if ((int )yych <= 50) {
        goto yy209;
      } else {

      }
      if ((int )yych <= 57) {
        goto yy210;
      } else {

      }
      goto yy216;
    }
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 110) {
      goto yy212;
    } else {

    }
    if ((int )yych <= 113) {
      goto yy216;
    } else {

    }
    goto yy213;
  } else {
    if ((int )yych <= 115) {
      goto yy211;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy214;
    } else {

    }
    goto yy216;
  }
  yy202: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 77) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 76) {
        goto yy144;
      } else {

      }
    }
  } else
  if ((int )yych <= 108) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 109) {
      goto yy203;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy203: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 66) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 65) {
        goto yy145;
      } else {

      }
    }
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy145;
  } else {
    if ((int )yych <= 98) {
      goto yy204;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy4;
  }
  yy204: 
  yyaccept = 0U;
  cursor ++;
  tmp___47 = cursor;
  s->ptr = tmp___47;
  yych = *tmp___47;
  if ((int )yych <= 68) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 69) {
      goto yy205;
    } else {

    }
    if ((int )yych != 101) {
      goto yy4;
    } else {

    }
  }
  yy205: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy206;
  } else {

  }
  if ((int )yych != 114) {
    goto yy57;
  } else {

  }
  yy206: 
  yyaccept = 5U;
  cursor ++;
  tmp___48 = cursor;
  s->ptr = tmp___48;
  yych = *tmp___48;
  if ((int )yych <= 32) {
    if ((int )yych == 9) {
      goto yy196;
    } else {

    }
    if ((int )yych <= 31) {
      goto yy194;
    } else {

    }
    goto yy196;
  } else
  if ((int )yych <= 46) {
    if ((int )yych <= 44) {
      goto yy194;
    } else {

    }
    goto yy196;
  } else {
    if ((int )yych <= 47) {
      goto yy194;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy196;
    } else {

    }
    goto yy194;
  }
  yy207: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy221;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 48) {
      goto yy296;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy297;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy221;
    } else {

    }
    goto yy57;
  }
  yy208: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy264;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 50) {
      goto yy297;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy296;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy264;
    } else {

    }
    goto yy57;
  }
  yy209: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy264;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 52) {
      goto yy296;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy293;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy264;
    } else {

    }
    goto yy57;
  }
  yy210: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy264;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 57) {
      goto yy293;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy264;
    } else {

    }
    goto yy57;
  }
  yy211: 
  yyaccept = 6U;
  cursor ++;
  tmp___49 = cursor;
  s->ptr = tmp___49;
  yych = *tmp___49;
  goto yy216;
  yy212: 
  yyaccept = 6U;
  cursor ++;
  tmp___50 = cursor;
  s->ptr = tmp___50;
  yych = *tmp___50;
  goto yy216;
  yy213: 
  yyaccept = 6U;
  cursor ++;
  tmp___51 = cursor;
  s->ptr = tmp___51;
  yych = *tmp___51;
  goto yy216;
  yy214: 
  yyaccept = 6U;
  cursor ++;
  tmp___52 = cursor;
  s->ptr = tmp___52;
  yych = *tmp___52;
  goto yy216;
  yy215: 
  yyaccept = 6U;
  cursor ++;
  s->ptr = cursor;
  if (s->lim - cursor < 18) {
    return (257);
  } else {

  }
  yych = *cursor;
  yy216: ;
  if ((int const   )yybm[(int )yych] & 64) {
    goto yy215;
  } else {

  }
  if ((int )yych <= 50) {
    if ((int )yych <= 47) {
      goto yy199;
    } else {

    }
    if ((int )yych <= 48) {
      goto yy259;
    } else {

    }
    if ((int )yych <= 49) {
      goto yy260;
    } else {

    }
    goto yy261;
  } else {
    if ((int )yych <= 57) {
      goto yy262;
    } else {

    }
    if ((int )yych != 84) {
      goto yy199;
    } else {

    }
  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 49) {
    goto yy218;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy219;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy220;
  } else {

  }
  goto yy57;
  yy218: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy221;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 57) {
      goto yy220;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy221;
    } else {

    }
    goto yy57;
  }
  yy219: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy221;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 52) {
      goto yy220;
    } else {

    }
    if ((int )yych == 58) {
      goto yy221;
    } else {

    }
    goto yy57;
  }
  yy220: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 46) {
    goto yy221;
  } else {

  }
  if ((int )yych != 58) {
    goto yy57;
  } else {

  }
  yy221: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy222;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy224;
  } else {

  }
  goto yy57;
  yy222: 
  yyaccept = 7U;
  cursor ++;
  tmp___53 = cursor;
  s->ptr = tmp___53;
  yych = *tmp___53;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy225;
    } else {

    }
  } else {
    if ((int )yych <= 57) {
      goto yy224;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy225;
    } else {

    }
  }
  yy223: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  if ((s->time)->have_date) {
    add_error(s, (char *)"Double date specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_date = 1U;
  }
  tmp___54 = timelib_get_month(& ptr);
  (s->time)->m = (timelib_sll )tmp___54;
  (s->time)->d = timelib_get_nr(& ptr, 2);
  if ((s->time)->have_time) {
    add_error(s, (char *)"Double time specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_time = 1U;
    (s->time)->h = (timelib_sll )0;
    (s->time)->i = (timelib_sll )0;
    (s->time)->s = (timelib_sll )0;
    (s->time)->f = (double )0;
  }
  (s->time)->h = timelib_get_nr(& ptr, 2);
  (s->time)->i = timelib_get_nr(& ptr, 2);
  if ((int )*ptr == 58) {
    (s->time)->s = timelib_get_nr(& ptr, 2);
    if ((int )*ptr == 46) {
      (s->time)->f = timelib_get_frac_nr(& ptr, 8);
    } else {

    }
  } else {

  }
  if ((int )*ptr != 0) {
    tmp___55 = timelib_get_zone(& ptr, & (s->time)->dst, s->time,
                                & tz_not_found___0, s->tzdb);
    (s->time)->z = (int )tmp___55;
    if (tz_not_found___0) {
      add_error(s, (char *)"The timezone could not be found in the database");
    } else {

    }
  } else {

  }
  free((void *)str);
  return (276);
  yy224: 
  yyaccept = 7U;
  cursor ++;
  tmp___56 = cursor;
  s->ptr = tmp___56;
  yych = *tmp___56;
  if ((int )yych == 46) {
    goto yy225;
  } else {

  }
  if ((int )yych != 58) {
    goto yy223;
  } else {

  }
  yy225: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy226;
  } else {

  }
  if ((int )yych <= 54) {
    goto yy227;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy228;
  } else {

  }
  goto yy57;
  yy226: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy223;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy229;
  } else {

  }
  goto yy223;
  yy227: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 48) {
    goto yy229;
  } else {

  }
  goto yy223;
  yy228: 
  cursor ++;
  yych = *cursor;
  goto yy223;
  yy229: 
  yyaccept = 7U;
  cursor ++;
  tmp___57 = cursor;
  s->ptr = tmp___57;
  yych = *tmp___57;
  if ((int )yych <= 42) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy231;
      } else {

      }
      goto yy223;
    } else {
      if ((int )yych <= 32) {
        goto yy231;
      } else {

      }
      if ((int )yych == 40) {
        goto yy231;
      } else {

      }
      goto yy223;
    }
  } else
  if ((int )yych <= 64) {
    if ((int )yych == 44) {
      goto yy223;
    } else {

    }
    if ((int )yych <= 45) {
      goto yy231;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych <= 90) {
      goto yy231;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy223;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy231;
    } else {

    }
    goto yy223;
  }
  yy230: 
  cursor ++;
  if (s->lim - cursor < 9) {
    return (257);
  } else {

  }
  yych = *cursor;
  yy231: ;
  if ((int )yych <= 43) {
    if ((int )yych <= 32) {
      if ((int )yych == 9) {
        goto yy230;
      } else {

      }
      if ((int )yych <= 31) {
        goto yy57;
      } else {

      }
      goto yy230;
    } else {
      if ((int )yych == 40) {
        goto yy234;
      } else {

      }
      if ((int )yych <= 42) {
        goto yy57;
      } else {

      }
      goto yy233;
    }
  } else
  if ((int )yych <= 70) {
    if ((int )yych == 45) {
      goto yy233;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy57;
    } else {

    }
    goto yy235;
  } else
  if ((int )yych <= 90) {
    if ((int )yych >= 72) {
      goto yy235;
    } else {

    }
  } else {
    if ((int )yych <= 96) {
      goto yy57;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy236;
    } else {

    }
    goto yy57;
  }
  yy232: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 76) {
    if ((int )yych == 41) {
      goto yy228;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy223;
    } else {

    }
    goto yy237;
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 77) {
      goto yy257;
    } else {

    }
    goto yy237;
  } else {
    if ((int )yych <= 96) {
      goto yy223;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy242;
    } else {

    }
    goto yy223;
  }
  yy233: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 49) {
    goto yy252;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy253;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy254;
  } else {

  }
  goto yy57;
  yy234: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 64) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 90) {
    goto yy236;
  } else {

  }
  if ((int )yych <= 96) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 122) {
    goto yy236;
  } else {

  }
  goto yy57;
  yy235: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 64) {
    if ((int )yych == 41) {
      goto yy228;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych <= 90) {
      goto yy237;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy223;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy242;
    } else {

    }
    goto yy223;
  }
  yy236: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 64) {
    if ((int )yych == 41) {
      goto yy228;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych <= 90) {
      goto yy237;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy223;
    } else {

    }
    if ((int )yych >= 123) {
      goto yy223;
    } else {

    }
  }
  yy237: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 64) {
    if ((int )yych == 41) {
      goto yy228;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych <= 90) {
      goto yy238;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy223;
    } else {

    }
    if ((int )yych >= 123) {
      goto yy223;
    } else {

    }
  }
  yy238: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 64) {
    if ((int )yych == 41) {
      goto yy228;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych <= 90) {
      goto yy239;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy223;
    } else {

    }
    if ((int )yych >= 123) {
      goto yy223;
    } else {

    }
  }
  yy239: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 64) {
    if ((int )yych == 41) {
      goto yy228;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych <= 90) {
      goto yy240;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy223;
    } else {

    }
    if ((int )yych >= 123) {
      goto yy223;
    } else {

    }
  }
  yy240: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 64) {
    if ((int )yych == 41) {
      goto yy228;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych <= 90) {
      goto yy241;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy223;
    } else {

    }
    if ((int )yych >= 123) {
      goto yy223;
    } else {

    }
  }
  yy241: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 41) {
    goto yy228;
  } else {

  }
  goto yy223;
  yy242: 
  yyaccept = 7U;
  cursor ++;
  tmp___58 = cursor;
  s->ptr = tmp___58;
  yych = *tmp___58;
  if ((int )yych <= 47) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy228;
      } else {

      }
      goto yy223;
    } else {
      if ((int )yych == 46) {
        goto yy223;
      } else {

      }
      goto yy244;
    }
  } else
  if ((int )yych <= 94) {
    if ((int )yych <= 64) {
      goto yy223;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy238;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych <= 95) {
      goto yy244;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy223;
    } else {

    }
    if ((int )yych >= 123) {
      goto yy223;
    } else {

    }
  }
  yy243: 
  yyaccept = 7U;
  cursor ++;
  tmp___59 = cursor;
  s->ptr = tmp___59;
  yych = *tmp___59;
  if ((int )yych <= 47) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy228;
      } else {

      }
      goto yy223;
    } else
    if ((int )yych == 46) {
      goto yy223;
    } else {

    }
  } else
  if ((int )yych <= 94) {
    if ((int )yych <= 64) {
      goto yy223;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy239;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych <= 95) {
      goto yy244;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy223;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy247;
    } else {

    }
    goto yy223;
  }
  yy244: 
  cursor ++;
  if ((unsigned int )s->lim <= (unsigned int )cursor) {
    return (257);
  } else {

  }
  yych = *cursor;
  if ((int )yych <= 64) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 90) {
    goto yy245;
  } else {

  }
  if ((int )yych <= 96) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 123) {
    goto yy57;
  } else {

  }
  yy245: 
  yyaccept = 7U;
  cursor ++;
  s->ptr = cursor;
  if ((unsigned int )s->lim <= (unsigned int )cursor) {
    return (257);
  } else {

  }
  yych = *cursor;
  if ((int )yych <= 64) {
    if ((int )yych <= 45) {
      if ((int )yych <= 44) {
        goto yy223;
      } else {

      }
      goto yy244;
    } else {
      if ((int )yych == 47) {
        goto yy244;
      } else {

      }
      goto yy223;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 90) {
      goto yy245;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy223;
    } else {

    }
    goto yy244;
  } else {
    if ((int )yych <= 96) {
      goto yy223;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy245;
    } else {

    }
    goto yy223;
  }
  yy247: 
  yyaccept = 7U;
  cursor ++;
  tmp___60 = cursor;
  s->ptr = tmp___60;
  yych = *tmp___60;
  if ((int )yych <= 47) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy228;
      } else {

      }
      goto yy223;
    } else {
      if ((int )yych == 46) {
        goto yy223;
      } else {

      }
      goto yy244;
    }
  } else
  if ((int )yych <= 94) {
    if ((int )yych <= 64) {
      goto yy223;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy240;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych <= 95) {
      goto yy244;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy223;
    } else {

    }
    if ((int )yych >= 123) {
      goto yy223;
    } else {

    }
  }
  yyaccept = 7U;
  cursor ++;
  tmp___61 = cursor;
  s->ptr = tmp___61;
  yych = *tmp___61;
  if ((int )yych <= 47) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy228;
      } else {

      }
      goto yy223;
    } else {
      if ((int )yych == 46) {
        goto yy223;
      } else {

      }
      goto yy244;
    }
  } else
  if ((int )yych <= 94) {
    if ((int )yych <= 64) {
      goto yy223;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy241;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych <= 95) {
      goto yy244;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy223;
    } else {

    }
    if ((int )yych >= 123) {
      goto yy223;
    } else {

    }
  }
  yyaccept = 7U;
  cursor ++;
  tmp___62 = cursor;
  s->ptr = tmp___62;
  yych = *tmp___62;
  if ((int )yych <= 46) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy223;
      } else {

      }
      goto yy228;
    } else {
      if ((int )yych == 45) {
        goto yy244;
      } else {

      }
      goto yy223;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 47) {
      goto yy244;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy223;
    } else {

    }
    goto yy244;
  } else {
    if ((int )yych <= 96) {
      goto yy223;
    } else {

    }
    if ((int )yych >= 123) {
      goto yy223;
    } else {

    }
  }
  yy250: 
  cursor ++;
  if ((unsigned int )s->lim <= (unsigned int )cursor) {
    return (257);
  } else {

  }
  yych = *cursor;
  if ((int )yych <= 47) {
    if ((int )yych == 45) {
      goto yy244;
    } else {

    }
    if ((int )yych <= 46) {
      goto yy57;
    } else {

    }
    goto yy244;
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 94) {
      goto yy57;
    } else {

    }
    goto yy244;
  } else {
    if ((int )yych <= 96) {
      goto yy57;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy250;
    } else {

    }
    goto yy57;
  }
  yy252: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy223;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy254;
  } else {

  }
  if ((int )yych <= 58) {
    goto yy255;
  } else {

  }
  goto yy223;
  yy253: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 53) {
    if ((int )yych <= 47) {
      goto yy223;
    } else {

    }
    if ((int )yych >= 53) {
      goto yy256;
    } else {

    }
  } else {
    if ((int )yych <= 57) {
      goto yy228;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy255;
    } else {

    }
    goto yy223;
  }
  yy254: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy223;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy256;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy228;
  } else {

  }
  if ((int )yych >= 59) {
    goto yy223;
  } else {

  }
  yy255: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy223;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy256;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy228;
  } else {

  }
  goto yy223;
  yy256: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy223;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy228;
  } else {

  }
  goto yy223;
  yy257: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 83) {
    if ((int )yych == 41) {
      goto yy228;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy223;
    } else {

    }
    goto yy238;
  } else
  if ((int )yych <= 90) {
    if ((int )yych >= 85) {
      goto yy238;
    } else {

    }
  } else {
    if ((int )yych <= 96) {
      goto yy223;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy238;
    } else {

    }
    goto yy223;
  }
  yyaccept = 7U;
  cursor ++;
  tmp___63 = cursor;
  s->ptr = tmp___63;
  yych = *tmp___63;
  if ((int )yych <= 44) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy223;
      } else {

      }
      goto yy228;
    } else {
      if ((int )yych == 43) {
        goto yy233;
      } else {

      }
      goto yy223;
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 45) {
      goto yy233;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy223;
    } else {

    }
    goto yy239;
  } else {
    if ((int )yych <= 96) {
      goto yy223;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy239;
    } else {

    }
    goto yy223;
  }
  yy259: 
  yyaccept = 6U;
  cursor ++;
  tmp___64 = cursor;
  s->ptr = tmp___64;
  yych = *tmp___64;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy221;
    } else {

    }
    goto yy199;
  } else {
    if ((int )yych <= 48) {
      goto yy291;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy292;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy221;
    } else {

    }
    goto yy199;
  }
  yy260: 
  yyaccept = 6U;
  cursor ++;
  tmp___65 = cursor;
  s->ptr = tmp___65;
  yych = *tmp___65;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy264;
    } else {

    }
    goto yy199;
  } else {
    if ((int )yych <= 50) {
      goto yy292;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy291;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy264;
    } else {

    }
    goto yy199;
  }
  yy261: 
  yyaccept = 6U;
  cursor ++;
  tmp___66 = cursor;
  s->ptr = tmp___66;
  yych = *tmp___66;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy264;
    } else {

    }
    goto yy199;
  } else {
    if ((int )yych <= 52) {
      goto yy291;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy263;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy264;
    } else {

    }
    goto yy199;
  }
  yy262: 
  yyaccept = 6U;
  cursor ++;
  tmp___67 = cursor;
  s->ptr = tmp___67;
  yych = *tmp___67;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy264;
    } else {

    }
    goto yy199;
  } else {
    if ((int )yych <= 57) {
      goto yy263;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy264;
    } else {

    }
    goto yy199;
  }
  yy263: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy199;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy289;
  } else {

  }
  goto yy199;
  yy264: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy265;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy266;
  } else {

  }
  goto yy57;
  yy265: 
  yyaccept = 7U;
  cursor ++;
  tmp___68 = cursor;
  s->ptr = tmp___68;
  yych = *tmp___68;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy267;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych <= 57) {
      goto yy282;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy267;
    } else {

    }
    goto yy223;
  }
  yy266: 
  yyaccept = 7U;
  cursor ++;
  tmp___69 = cursor;
  s->ptr = tmp___69;
  yych = *tmp___69;
  if ((int )yych == 46) {
    goto yy267;
  } else {

  }
  if ((int )yych != 58) {
    goto yy223;
  } else {

  }
  yy267: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy268;
  } else {

  }
  if ((int )yych <= 54) {
    goto yy269;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy228;
  } else {

  }
  goto yy57;
  yy268: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy223;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy270;
  } else {

  }
  goto yy223;
  yy269: 
  cursor ++;
  yych = *cursor;
  if ((int )yych != 48) {
    goto yy223;
  } else {

  }
  yy270: 
  yyaccept = 7U;
  cursor ++;
  tmp___70 = cursor;
  s->ptr = tmp___70;
  yych = *tmp___70;
  if ((int )yych <= 42) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy272;
      } else {

      }
      goto yy223;
    } else {
      if ((int )yych <= 32) {
        goto yy272;
      } else {

      }
      if ((int )yych == 40) {
        goto yy272;
      } else {

      }
      goto yy223;
    }
  } else
  if ((int )yych <= 64) {
    if ((int )yych == 44) {
      goto yy223;
    } else {

    }
    if ((int )yych <= 45) {
      goto yy272;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych <= 90) {
      goto yy272;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy223;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy272;
    } else {

    }
    goto yy223;
  }
  yy271: 
  cursor ++;
  if (s->lim - cursor < 9) {
    return (257);
  } else {

  }
  yych = *cursor;
  yy272: ;
  if ((int )yych <= 64) {
    if ((int )yych <= 39) {
      if ((int )yych <= 9) {
        if ((int )yych <= 8) {
          goto yy57;
        } else {

        }
        goto yy271;
      } else {
        if ((int )yych == 32) {
          goto yy271;
        } else {

        }
        goto yy57;
      }
    } else
    if ((int )yych <= 43) {
      if ((int )yych <= 40) {
        goto yy234;
      } else {

      }
      if ((int )yych <= 42) {
        goto yy57;
      } else {

      }
      goto yy233;
    } else {
      if ((int )yych == 45) {
        goto yy233;
      } else {

      }
      goto yy57;
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 71) {
      if ((int )yych <= 65) {
        goto yy273;
      } else {

      }
      if ((int )yych <= 70) {
        goto yy235;
      } else {

      }
      goto yy232;
    } else
    if ((int )yych != 80) {
      goto yy235;
    } else {

    }
  } else
  if ((int )yych <= 111) {
    if ((int )yych <= 96) {
      goto yy57;
    } else {

    }
    if ((int )yych <= 97) {
      goto yy274;
    } else {

    }
    goto yy236;
  } else {
    if ((int )yych <= 112) {
      goto yy274;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy236;
    } else {

    }
    goto yy57;
  }
  yy273: 
  yyaccept = 7U;
  cursor ++;
  tmp___71 = cursor;
  s->ptr = tmp___71;
  yych = *tmp___71;
  if ((int )yych <= 76) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy228;
      } else {

      }
      goto yy223;
    } else {
      if ((int )yych <= 46) {
        goto yy275;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy223;
      } else {

      }
      goto yy237;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 77) {
      goto yy276;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy237;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych == 109) {
      goto yy281;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy242;
    } else {

    }
    goto yy223;
  }
  yy274: 
  yyaccept = 7U;
  cursor ++;
  tmp___72 = cursor;
  s->ptr = tmp___72;
  yych = *tmp___72;
  if ((int )yych <= 76) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy228;
      } else {

      }
      goto yy223;
    } else {
      if ((int )yych <= 46) {
        goto yy275;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy223;
      } else {

      }
      goto yy237;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 77) {
      goto yy276;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy237;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych == 109) {
      goto yy276;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy237;
    } else {

    }
    goto yy223;
  }
  yy275: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 77) {
    goto yy280;
  } else {

  }
  if ((int )yych == 109) {
    goto yy280;
  } else {

  }
  goto yy57;
  yy276: 
  yyaccept = 7U;
  cursor ++;
  tmp___73 = cursor;
  s->ptr = tmp___73;
  yych = *tmp___73;
  if ((int )yych <= 41) {
    if ((int )yych <= 9) {
      if ((int )yych <= 0) {
        goto yy278;
      } else {

      }
      if ((int )yych <= 8) {
        goto yy223;
      } else {

      }
      goto yy278;
    } else {
      if ((int )yych == 32) {
        goto yy278;
      } else {

      }
      if ((int )yych <= 40) {
        goto yy223;
      } else {

      }
      goto yy228;
    }
  } else
  if ((int )yych <= 64) {
    if ((int )yych != 46) {
      goto yy223;
    } else {

    }
  } else {
    if ((int )yych <= 90) {
      goto yy238;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy223;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy238;
    } else {

    }
    goto yy223;
  }
  yy277: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 9) {
    if ((int )yych <= 0) {
      goto yy278;
    } else {

    }
    if ((int )yych <= 8) {
      goto yy57;
    } else {

    }
  } else
  if ((int )yych != 32) {
    goto yy57;
  } else {

  }
  yy278: 
  cursor ++;
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  if ((s->time)->have_date) {
    add_error(s, (char *)"Double date specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_date = 1U;
  }
  tmp___74 = timelib_get_month(& ptr);
  (s->time)->m = (timelib_sll )tmp___74;
  (s->time)->d = timelib_get_nr(& ptr, 2);
  if ((s->time)->have_time) {
    add_error(s, (char *)"Double time specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_time = 1U;
    (s->time)->h = (timelib_sll )0;
    (s->time)->i = (timelib_sll )0;
    (s->time)->s = (timelib_sll )0;
    (s->time)->f = (double )0;
  }
  (s->time)->h = timelib_get_nr(& ptr, 2);
  (s->time)->i = timelib_get_nr(& ptr, 2);
  if ((int )*ptr == 58) {
    goto _L;
  } else
  if ((int )*ptr == 46) {
    _L: /* CIL Label */ 
    (s->time)->s = timelib_get_nr(& ptr, 2);
    if ((int )*ptr == 46) {
      (s->time)->f = timelib_get_frac_nr(& ptr, 8);
    } else {

    }
  } else {

  }
  tmp___75 = timelib_meridian(& ptr, (s->time)->h);
  (s->time)->h += tmp___75;
  free((void *)str);
  return (276);
  yy280: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 31) {
    if ((int )yych <= 0) {
      goto yy278;
    } else {

    }
    if ((int )yych == 9) {
      goto yy278;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 32) {
      goto yy278;
    } else {

    }
    if ((int )yych == 46) {
      goto yy277;
    } else {

    }
    goto yy57;
  }
  yy281: 
  yyaccept = 7U;
  cursor ++;
  tmp___76 = cursor;
  s->ptr = tmp___76;
  yych = *tmp___76;
  if ((int )yych <= 45) {
    if ((int )yych <= 31) {
      if ((int )yych <= 0) {
        goto yy278;
      } else {

      }
      if ((int )yych == 9) {
        goto yy278;
      } else {

      }
      goto yy223;
    } else
    if ((int )yych <= 40) {
      if ((int )yych <= 32) {
        goto yy278;
      } else {

      }
      goto yy223;
    } else {
      if ((int )yych <= 41) {
        goto yy228;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy223;
      } else {

      }
      goto yy244;
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 46) {
      goto yy277;
    } else {

    }
    if ((int )yych <= 47) {
      goto yy244;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy223;
    } else {

    }
    goto yy238;
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 94) {
      goto yy223;
    } else {

    }
    goto yy244;
  } else {
    if ((int )yych <= 96) {
      goto yy223;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy243;
    } else {

    }
    goto yy223;
  }
  yy282: 
  yyaccept = 7U;
  cursor ++;
  tmp___77 = cursor;
  s->ptr = tmp___77;
  yych = *tmp___77;
  if ((int )yych <= 58) {
    if ((int )yych <= 32) {
      if ((int )yych == 9) {
        goto yy283;
      } else {

      }
      if ((int )yych <= 31) {
        goto yy223;
      } else {

      }
    } else {
      if ((int )yych == 46) {
        goto yy267;
      } else {

      }
      if ((int )yych <= 57) {
        goto yy223;
      } else {

      }
      goto yy267;
    }
  } else
  if ((int )yych <= 80) {
    if ((int )yych == 65) {
      goto yy285;
    } else {

    }
    if ((int )yych <= 79) {
      goto yy223;
    } else {

    }
    goto yy285;
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 96) {
      goto yy223;
    } else {

    }
    goto yy285;
  } else {
    if ((int )yych == 112) {
      goto yy285;
    } else {

    }
    goto yy223;
  }
  yy283: 
  cursor ++;
  if (s->lim - cursor < 5) {
    return (257);
  } else {

  }
  yych = *cursor;
  if ((int )yych <= 65) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy283;
      } else {

      }
      goto yy57;
    } else {
      if ((int )yych <= 32) {
        goto yy283;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy57;
      } else {

      }
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych != 80) {
      goto yy57;
    } else {

    }
  } else {
    if ((int )yych <= 97) {
      goto yy285;
    } else {

    }
    if ((int )yych != 112) {
      goto yy57;
    } else {

    }
  }
  yy285: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 76) {
    if ((int )yych != 46) {
      goto yy57;
    } else {

    }
  } else {
    if ((int )yych <= 77) {
      goto yy287;
    } else {

    }
    if ((int )yych == 109) {
      goto yy287;
    } else {

    }
    goto yy57;
  }
  yy286: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 77) {
    goto yy287;
  } else {

  }
  if ((int )yych != 109) {
    goto yy57;
  } else {

  }
  yy287: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 31) {
    if ((int )yych <= 0) {
      goto yy278;
    } else {

    }
    if ((int )yych == 9) {
      goto yy278;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 32) {
      goto yy278;
    } else {

    }
    if ((int )yych != 46) {
      goto yy57;
    } else {

    }
  }
  yy288: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 9) {
    if ((int )yych <= 0) {
      goto yy278;
    } else {

    }
    if ((int )yych <= 8) {
      goto yy57;
    } else {

    }
    goto yy278;
  } else {
    if ((int )yych == 32) {
      goto yy278;
    } else {

    }
    goto yy57;
  }
  yy289: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy199;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy199;
  } else {

  }
  cursor ++;
  yych = *cursor;
  goto yy199;
  yy291: 
  yyaccept = 6U;
  cursor ++;
  tmp___78 = cursor;
  s->ptr = tmp___78;
  yych = *tmp___78;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy221;
    } else {

    }
    goto yy199;
  } else {
    if ((int )yych <= 57) {
      goto yy289;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy221;
    } else {

    }
    goto yy199;
  }
  yy292: 
  yyaccept = 6U;
  cursor ++;
  tmp___79 = cursor;
  s->ptr = tmp___79;
  yych = *tmp___79;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy264;
    } else {

    }
    goto yy199;
  } else {
    if ((int )yych <= 57) {
      goto yy289;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy264;
    } else {

    }
    goto yy199;
  }
  yy293: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy57;
  } else {

  }
  yy294: 
  cursor ++;
  yy295: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  if ((s->time)->have_date) {
    add_error(s, (char *)"Double date specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_date = 1U;
  }
  tmp___80 = timelib_get_month(& ptr);
  (s->time)->m = (timelib_sll )tmp___80;
  (s->time)->y = timelib_get_nr(& ptr, 4);
  (s->time)->d = (timelib_sll )1;
  if ((s->time)->y == -99999LL) {

  } else
  if ((s->time)->y < 100LL) {
    if ((s->time)->y < 70LL) {
      (s->time)->y += 2000LL;
    } else {
      (s->time)->y += 1900LL;
    }
  } else {

  }
  free((void *)str);
  return (275);
  yy296: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy221;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 57) {
      goto yy294;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy221;
    } else {

    }
    goto yy57;
  }
  yy297: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy264;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 57) {
      goto yy294;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy264;
    } else {

    }
    goto yy57;
  }
  yy298: 
  yyaccept = 6U;
  cursor ++;
  tmp___81 = cursor;
  s->ptr = tmp___81;
  yych = *tmp___81;
  if ((int )yych <= 57) {
    if ((int )yych <= 47) {
      if ((int )yych == 46) {
        goto yy331;
      } else {

      }
      goto yy216;
    } else {
      if ((int )yych <= 48) {
        goto yy332;
      } else {

      }
      if ((int )yych <= 49) {
        goto yy302;
      } else {

      }
      if ((int )yych <= 50) {
        goto yy303;
      } else {

      }
      goto yy297;
    }
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 58) {
      goto yy221;
    } else {

    }
    if ((int )yych == 110) {
      goto yy212;
    } else {

    }
    goto yy216;
  } else {
    if ((int )yych <= 114) {
      goto yy213;
    } else {

    }
    if ((int )yych <= 115) {
      goto yy211;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy214;
    } else {

    }
    goto yy216;
  }
  yy299: 
  yyaccept = 6U;
  cursor ++;
  tmp___82 = cursor;
  s->ptr = tmp___82;
  yych = *tmp___82;
  if ((int )yych <= 57) {
    if ((int )yych <= 47) {
      if ((int )yych != 46) {
        goto yy216;
      } else {

      }
    } else {
      if ((int )yych <= 48) {
        goto yy301;
      } else {

      }
      if ((int )yych <= 49) {
        goto yy302;
      } else {

      }
      if ((int )yych <= 50) {
        goto yy303;
      } else {

      }
      goto yy297;
    }
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 58) {
      goto yy264;
    } else {

    }
    if ((int )yych == 110) {
      goto yy212;
    } else {

    }
    goto yy216;
  } else {
    if ((int )yych <= 114) {
      goto yy213;
    } else {

    }
    if ((int )yych <= 115) {
      goto yy211;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy214;
    } else {

    }
    goto yy216;
  }
  yy300: 
  yyaccept = 6U;
  cursor ++;
  tmp___83 = cursor;
  s->ptr = tmp___83;
  yych = *tmp___83;
  if ((int )yych <= 49) {
    if ((int )yych <= 47) {
      goto yy216;
    } else {

    }
    if ((int )yych <= 48) {
      goto yy306;
    } else {

    }
    goto yy307;
  } else {
    if ((int )yych <= 50) {
      goto yy308;
    } else {

    }
    if ((int )yych <= 53) {
      goto yy309;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy310;
    } else {

    }
    goto yy216;
  }
  yy301: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy264;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 48) {
      goto yy304;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy305;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy264;
    } else {

    }
    goto yy57;
  }
  yy302: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy264;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 50) {
      goto yy305;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy304;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy264;
    } else {

    }
    goto yy57;
  }
  yy303: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy264;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 52) {
      goto yy304;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy294;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy264;
    } else {

    }
    goto yy57;
  }
  yy304: 
  yyaccept = 8U;
  cursor ++;
  tmp___84 = cursor;
  s->ptr = tmp___84;
  yych = *tmp___84;
  if ((int )yych == 46) {
    goto yy221;
  } else {

  }
  if ((int )yych == 58) {
    goto yy221;
  } else {

  }
  goto yy295;
  yy305: 
  yyaccept = 8U;
  cursor ++;
  tmp___85 = cursor;
  s->ptr = tmp___85;
  yych = *tmp___85;
  if ((int )yych == 46) {
    goto yy264;
  } else {

  }
  if ((int )yych == 58) {
    goto yy264;
  } else {

  }
  goto yy295;
  yy306: 
  yyaccept = 6U;
  cursor ++;
  tmp___86 = cursor;
  s->ptr = tmp___86;
  yych = *tmp___86;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy326;
    } else {

    }
    goto yy199;
  } else {
    if ((int )yych <= 48) {
      goto yy325;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy330;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy326;
    } else {

    }
    goto yy199;
  }
  yy307: 
  yyaccept = 6U;
  cursor ++;
  tmp___87 = cursor;
  s->ptr = tmp___87;
  yych = *tmp___87;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy311;
    } else {

    }
    goto yy199;
  } else {
    if ((int )yych <= 50) {
      goto yy330;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy325;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy311;
    } else {

    }
    goto yy199;
  }
  yy308: 
  yyaccept = 6U;
  cursor ++;
  tmp___88 = cursor;
  s->ptr = tmp___88;
  yych = *tmp___88;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy311;
    } else {

    }
    goto yy199;
  } else {
    if ((int )yych <= 52) {
      goto yy325;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy324;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy311;
    } else {

    }
    goto yy199;
  }
  yy309: 
  yyaccept = 6U;
  cursor ++;
  tmp___89 = cursor;
  s->ptr = tmp___89;
  yych = *tmp___89;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy311;
    } else {

    }
    goto yy199;
  } else {
    if ((int )yych <= 57) {
      goto yy324;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy311;
    } else {

    }
    goto yy199;
  }
  yy310: 
  yyaccept = 6U;
  cursor ++;
  tmp___90 = cursor;
  s->ptr = tmp___90;
  yych = *tmp___90;
  if ((int )yych <= 47) {
    if ((int )yych != 46) {
      goto yy199;
    } else {

    }
  } else {
    if ((int )yych <= 57) {
      goto yy263;
    } else {

    }
    if ((int )yych >= 59) {
      goto yy199;
    } else {

    }
  }
  yy311: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy312;
  } else {

  }
  if ((int )yych <= 54) {
    goto yy313;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy266;
  } else {

  }
  goto yy57;
  yy312: 
  yyaccept = 7U;
  cursor ++;
  tmp___91 = cursor;
  s->ptr = tmp___91;
  yych = *tmp___91;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy267;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych <= 57) {
      goto yy314;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy267;
    } else {

    }
    goto yy223;
  }
  yy313: 
  yyaccept = 7U;
  cursor ++;
  tmp___92 = cursor;
  s->ptr = tmp___92;
  yych = *tmp___92;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy267;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych <= 48) {
      goto yy270;
    } else {

    }
    if ((int )yych == 58) {
      goto yy267;
    } else {

    }
    goto yy223;
  }
  yy314: 
  yyaccept = 7U;
  cursor ++;
  tmp___93 = cursor;
  s->ptr = tmp___93;
  yych = *tmp___93;
  if ((int )yych <= 44) {
    if ((int )yych <= 32) {
      if ((int )yych == 9) {
        goto yy316;
      } else {

      }
      if ((int )yych <= 31) {
        goto yy223;
      } else {

      }
      goto yy316;
    } else
    if ((int )yych <= 40) {
      if ((int )yych <= 39) {
        goto yy223;
      } else {

      }
      goto yy316;
    } else {
      if ((int )yych == 43) {
        goto yy316;
      } else {

      }
      goto yy223;
    }
  } else
  if ((int )yych <= 58) {
    if ((int )yych <= 45) {
      goto yy316;
    } else {

    }
    if ((int )yych <= 46) {
      goto yy267;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy223;
    } else {

    }
    goto yy267;
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 64) {
      goto yy223;
    } else {

    }
    goto yy316;
  } else {
    if ((int )yych <= 96) {
      goto yy223;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy316;
    } else {

    }
    goto yy223;
  }
  yy315: 
  cursor ++;
  if (s->lim - cursor < 9) {
    return (257);
  } else {

  }
  yych = *cursor;
  yy316: ;
  if ((int )yych <= 64) {
    if ((int )yych <= 39) {
      if ((int )yych <= 9) {
        if ((int )yych <= 8) {
          goto yy57;
        } else {

        }
        goto yy315;
      } else {
        if ((int )yych == 32) {
          goto yy315;
        } else {

        }
        goto yy57;
      }
    } else
    if ((int )yych <= 43) {
      if ((int )yych <= 40) {
        goto yy234;
      } else {

      }
      if ((int )yych <= 42) {
        goto yy57;
      } else {

      }
      goto yy233;
    } else {
      if ((int )yych == 45) {
        goto yy233;
      } else {

      }
      goto yy57;
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 71) {
      if ((int )yych <= 65) {
        goto yy317;
      } else {

      }
      if ((int )yych <= 70) {
        goto yy235;
      } else {

      }
      goto yy232;
    } else
    if ((int )yych != 80) {
      goto yy235;
    } else {

    }
  } else
  if ((int )yych <= 111) {
    if ((int )yych <= 96) {
      goto yy57;
    } else {

    }
    if ((int )yych <= 97) {
      goto yy318;
    } else {

    }
    goto yy236;
  } else {
    if ((int )yych <= 112) {
      goto yy318;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy236;
    } else {

    }
    goto yy57;
  }
  yy317: 
  yyaccept = 7U;
  cursor ++;
  tmp___94 = cursor;
  s->ptr = tmp___94;
  yych = *tmp___94;
  if ((int )yych <= 76) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy228;
      } else {

      }
      goto yy223;
    } else {
      if ((int )yych <= 46) {
        goto yy320;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy223;
      } else {

      }
      goto yy237;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 77) {
      goto yy319;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy237;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych == 109) {
      goto yy323;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy242;
    } else {

    }
    goto yy223;
  }
  yy318: 
  yyaccept = 7U;
  cursor ++;
  tmp___95 = cursor;
  s->ptr = tmp___95;
  yych = *tmp___95;
  if ((int )yych <= 76) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy228;
      } else {

      }
      goto yy223;
    } else {
      if ((int )yych <= 46) {
        goto yy320;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy223;
      } else {

      }
      goto yy237;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 77) {
      goto yy319;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy237;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych == 109) {
      goto yy319;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy237;
    } else {

    }
    goto yy223;
  }
  yy319: 
  yyaccept = 7U;
  cursor ++;
  tmp___96 = cursor;
  s->ptr = tmp___96;
  yych = *tmp___96;
  if ((int )yych <= 41) {
    if ((int )yych <= 9) {
      if ((int )yych <= 0) {
        goto yy278;
      } else {

      }
      if ((int )yych <= 8) {
        goto yy223;
      } else {

      }
      goto yy278;
    } else {
      if ((int )yych == 32) {
        goto yy278;
      } else {

      }
      if ((int )yych <= 40) {
        goto yy223;
      } else {

      }
      goto yy228;
    }
  } else
  if ((int )yych <= 64) {
    if ((int )yych == 46) {
      goto yy322;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych <= 90) {
      goto yy238;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy223;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy238;
    } else {

    }
    goto yy223;
  }
  yy320: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 77) {
    goto yy321;
  } else {

  }
  if ((int )yych != 109) {
    goto yy57;
  } else {

  }
  yy321: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 31) {
    if ((int )yych <= 0) {
      goto yy278;
    } else {

    }
    if ((int )yych == 9) {
      goto yy278;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 32) {
      goto yy278;
    } else {

    }
    if ((int )yych != 46) {
      goto yy57;
    } else {

    }
  }
  yy322: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 9) {
    if ((int )yych <= 0) {
      goto yy278;
    } else {

    }
    if ((int )yych <= 8) {
      goto yy57;
    } else {

    }
    goto yy278;
  } else {
    if ((int )yych == 32) {
      goto yy278;
    } else {

    }
    goto yy57;
  }
  yy323: 
  yyaccept = 7U;
  cursor ++;
  tmp___97 = cursor;
  s->ptr = tmp___97;
  yych = *tmp___97;
  if ((int )yych <= 45) {
    if ((int )yych <= 31) {
      if ((int )yych <= 0) {
        goto yy278;
      } else {

      }
      if ((int )yych == 9) {
        goto yy278;
      } else {

      }
      goto yy223;
    } else
    if ((int )yych <= 40) {
      if ((int )yych <= 32) {
        goto yy278;
      } else {

      }
      goto yy223;
    } else {
      if ((int )yych <= 41) {
        goto yy228;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy223;
      } else {

      }
      goto yy244;
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 46) {
      goto yy322;
    } else {

    }
    if ((int )yych <= 47) {
      goto yy244;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy223;
    } else {

    }
    goto yy238;
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 94) {
      goto yy223;
    } else {

    }
    goto yy244;
  } else {
    if ((int )yych <= 96) {
      goto yy223;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy243;
    } else {

    }
    goto yy223;
  }
  yy324: 
  yyaccept = 6U;
  cursor ++;
  tmp___98 = cursor;
  s->ptr = tmp___98;
  yych = *tmp___98;
  if ((int )yych <= 58) {
    if ((int )yych <= 32) {
      if ((int )yych == 9) {
        goto yy283;
      } else {

      }
      if ((int )yych <= 31) {
        goto yy199;
      } else {

      }
      goto yy283;
    } else
    if ((int )yych <= 46) {
      if ((int )yych <= 45) {
        goto yy199;
      } else {

      }
      goto yy267;
    } else {
      if ((int )yych <= 47) {
        goto yy199;
      } else {

      }
      if ((int )yych <= 57) {
        goto yy289;
      } else {

      }
      goto yy267;
    }
  } else
  if ((int )yych <= 80) {
    if ((int )yych == 65) {
      goto yy285;
    } else {

    }
    if ((int )yych <= 79) {
      goto yy199;
    } else {

    }
    goto yy285;
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 96) {
      goto yy199;
    } else {

    }
    goto yy285;
  } else {
    if ((int )yych == 112) {
      goto yy285;
    } else {

    }
    goto yy199;
  }
  yy325: 
  yyaccept = 6U;
  cursor ++;
  tmp___99 = cursor;
  s->ptr = tmp___99;
  yych = *tmp___99;
  if ((int )yych <= 58) {
    if ((int )yych <= 32) {
      if ((int )yych == 9) {
        goto yy283;
      } else {

      }
      if ((int )yych <= 31) {
        goto yy199;
      } else {

      }
      goto yy283;
    } else
    if ((int )yych <= 46) {
      if ((int )yych <= 45) {
        goto yy199;
      } else {

      }
    } else {
      if ((int )yych <= 47) {
        goto yy199;
      } else {

      }
      if ((int )yych <= 57) {
        goto yy289;
      } else {

      }
    }
  } else
  if ((int )yych <= 80) {
    if ((int )yych == 65) {
      goto yy285;
    } else {

    }
    if ((int )yych <= 79) {
      goto yy199;
    } else {

    }
    goto yy285;
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 96) {
      goto yy199;
    } else {

    }
    goto yy285;
  } else {
    if ((int )yych == 112) {
      goto yy285;
    } else {

    }
    goto yy199;
  }
  yy326: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy327;
  } else {

  }
  if ((int )yych <= 54) {
    goto yy328;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy224;
  } else {

  }
  goto yy57;
  yy327: 
  yyaccept = 7U;
  cursor ++;
  tmp___100 = cursor;
  s->ptr = tmp___100;
  yych = *tmp___100;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy225;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych <= 57) {
      goto yy329;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy225;
    } else {

    }
    goto yy223;
  }
  yy328: 
  yyaccept = 7U;
  cursor ++;
  tmp___101 = cursor;
  s->ptr = tmp___101;
  yych = *tmp___101;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy225;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych <= 48) {
      goto yy270;
    } else {

    }
    if ((int )yych == 58) {
      goto yy225;
    } else {

    }
    goto yy223;
  }
  yy329: 
  yyaccept = 7U;
  cursor ++;
  tmp___102 = cursor;
  s->ptr = tmp___102;
  yych = *tmp___102;
  if ((int )yych <= 44) {
    if ((int )yych <= 32) {
      if ((int )yych == 9) {
        goto yy272;
      } else {

      }
      if ((int )yych <= 31) {
        goto yy223;
      } else {

      }
      goto yy272;
    } else
    if ((int )yych <= 40) {
      if ((int )yych <= 39) {
        goto yy223;
      } else {

      }
      goto yy272;
    } else {
      if ((int )yych == 43) {
        goto yy272;
      } else {

      }
      goto yy223;
    }
  } else
  if ((int )yych <= 58) {
    if ((int )yych <= 45) {
      goto yy272;
    } else {

    }
    if ((int )yych <= 46) {
      goto yy225;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy223;
    } else {

    }
    goto yy225;
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 64) {
      goto yy223;
    } else {

    }
    goto yy272;
  } else {
    if ((int )yych <= 96) {
      goto yy223;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy272;
    } else {

    }
    goto yy223;
  }
  yy330: 
  yyaccept = 6U;
  cursor ++;
  tmp___103 = cursor;
  s->ptr = tmp___103;
  yych = *tmp___103;
  if ((int )yych <= 58) {
    if ((int )yych <= 32) {
      if ((int )yych == 9) {
        goto yy283;
      } else {

      }
      if ((int )yych <= 31) {
        goto yy199;
      } else {

      }
      goto yy283;
    } else
    if ((int )yych <= 46) {
      if ((int )yych <= 45) {
        goto yy199;
      } else {

      }
      goto yy311;
    } else {
      if ((int )yych <= 47) {
        goto yy199;
      } else {

      }
      if ((int )yych <= 57) {
        goto yy289;
      } else {

      }
      goto yy311;
    }
  } else
  if ((int )yych <= 80) {
    if ((int )yych == 65) {
      goto yy285;
    } else {

    }
    if ((int )yych <= 79) {
      goto yy199;
    } else {

    }
    goto yy285;
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 96) {
      goto yy199;
    } else {

    }
    goto yy285;
  } else {
    if ((int )yych == 112) {
      goto yy285;
    } else {

    }
    goto yy199;
  }
  yy331: 
  yyaccept = 6U;
  cursor ++;
  tmp___104 = cursor;
  s->ptr = tmp___104;
  yych = *tmp___104;
  if ((int )yych <= 49) {
    if ((int )yych <= 47) {
      goto yy216;
    } else {

    }
    if ((int )yych <= 48) {
      goto yy333;
    } else {

    }
    goto yy334;
  } else {
    if ((int )yych <= 50) {
      goto yy335;
    } else {

    }
    if ((int )yych <= 53) {
      goto yy336;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy337;
    } else {

    }
    goto yy216;
  }
  yy332: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy221;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 48) {
      goto yy304;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy305;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy221;
    } else {

    }
    goto yy57;
  }
  yy333: 
  yyaccept = 6U;
  cursor ++;
  tmp___105 = cursor;
  s->ptr = tmp___105;
  yych = *tmp___105;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy350;
    } else {

    }
    goto yy199;
  } else {
    if ((int )yych <= 48) {
      goto yy349;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy354;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy350;
    } else {

    }
    goto yy199;
  }
  yy334: 
  yyaccept = 6U;
  cursor ++;
  tmp___106 = cursor;
  s->ptr = tmp___106;
  yych = *tmp___106;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy338;
    } else {

    }
    goto yy199;
  } else {
    if ((int )yych <= 50) {
      goto yy354;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy349;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy338;
    } else {

    }
    goto yy199;
  }
  yy335: 
  yyaccept = 6U;
  cursor ++;
  tmp___107 = cursor;
  s->ptr = tmp___107;
  yych = *tmp___107;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy338;
    } else {

    }
    goto yy199;
  } else {
    if ((int )yych <= 52) {
      goto yy349;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy348;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy338;
    } else {

    }
    goto yy199;
  }
  yy336: 
  yyaccept = 6U;
  cursor ++;
  tmp___108 = cursor;
  s->ptr = tmp___108;
  yych = *tmp___108;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy338;
    } else {

    }
    goto yy199;
  } else {
    if ((int )yych <= 57) {
      goto yy348;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy338;
    } else {

    }
    goto yy199;
  }
  yy337: 
  yyaccept = 6U;
  cursor ++;
  tmp___109 = cursor;
  s->ptr = tmp___109;
  yych = *tmp___109;
  if ((int )yych <= 47) {
    if ((int )yych != 46) {
      goto yy199;
    } else {

    }
  } else {
    if ((int )yych <= 57) {
      goto yy263;
    } else {

    }
    if ((int )yych >= 59) {
      goto yy199;
    } else {

    }
  }
  yy338: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy339;
  } else {

  }
  if ((int )yych <= 54) {
    goto yy340;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy266;
  } else {

  }
  goto yy57;
  yy339: 
  yyaccept = 7U;
  cursor ++;
  tmp___110 = cursor;
  s->ptr = tmp___110;
  yych = *tmp___110;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy267;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych <= 57) {
      goto yy341;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy267;
    } else {

    }
    goto yy223;
  }
  yy340: 
  yyaccept = 7U;
  cursor ++;
  tmp___111 = cursor;
  s->ptr = tmp___111;
  yych = *tmp___111;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy267;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych <= 48) {
      goto yy229;
    } else {

    }
    if ((int )yych == 58) {
      goto yy267;
    } else {

    }
    goto yy223;
  }
  yy341: 
  yyaccept = 7U;
  cursor ++;
  tmp___112 = cursor;
  s->ptr = tmp___112;
  yych = *tmp___112;
  if ((int )yych <= 44) {
    if ((int )yych <= 32) {
      if ((int )yych == 9) {
        goto yy343;
      } else {

      }
      if ((int )yych <= 31) {
        goto yy223;
      } else {

      }
      goto yy343;
    } else
    if ((int )yych <= 40) {
      if ((int )yych <= 39) {
        goto yy223;
      } else {

      }
      goto yy343;
    } else {
      if ((int )yych == 43) {
        goto yy343;
      } else {

      }
      goto yy223;
    }
  } else
  if ((int )yych <= 58) {
    if ((int )yych <= 45) {
      goto yy343;
    } else {

    }
    if ((int )yych <= 46) {
      goto yy267;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy223;
    } else {

    }
    goto yy267;
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 64) {
      goto yy223;
    } else {

    }
    goto yy343;
  } else {
    if ((int )yych <= 96) {
      goto yy223;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy343;
    } else {

    }
    goto yy223;
  }
  yy342: 
  cursor ++;
  if (s->lim - cursor < 9) {
    return (257);
  } else {

  }
  yych = *cursor;
  yy343: ;
  if ((int )yych <= 64) {
    if ((int )yych <= 39) {
      if ((int )yych <= 9) {
        if ((int )yych <= 8) {
          goto yy57;
        } else {

        }
        goto yy342;
      } else {
        if ((int )yych == 32) {
          goto yy342;
        } else {

        }
        goto yy57;
      }
    } else
    if ((int )yych <= 43) {
      if ((int )yych <= 40) {
        goto yy234;
      } else {

      }
      if ((int )yych <= 42) {
        goto yy57;
      } else {

      }
      goto yy233;
    } else {
      if ((int )yych == 45) {
        goto yy233;
      } else {

      }
      goto yy57;
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 71) {
      if ((int )yych <= 65) {
        goto yy344;
      } else {

      }
      if ((int )yych <= 70) {
        goto yy235;
      } else {

      }
      goto yy232;
    } else
    if ((int )yych != 80) {
      goto yy235;
    } else {

    }
  } else
  if ((int )yych <= 111) {
    if ((int )yych <= 96) {
      goto yy57;
    } else {

    }
    if ((int )yych <= 97) {
      goto yy345;
    } else {

    }
    goto yy236;
  } else {
    if ((int )yych <= 112) {
      goto yy345;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy236;
    } else {

    }
    goto yy57;
  }
  yy344: 
  yyaccept = 7U;
  cursor ++;
  tmp___113 = cursor;
  s->ptr = tmp___113;
  yych = *tmp___113;
  if ((int )yych <= 76) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy228;
      } else {

      }
      goto yy223;
    } else {
      if ((int )yych <= 46) {
        goto yy286;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy223;
      } else {

      }
      goto yy237;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 77) {
      goto yy346;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy237;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych == 109) {
      goto yy347;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy242;
    } else {

    }
    goto yy223;
  }
  yy345: 
  yyaccept = 7U;
  cursor ++;
  tmp___114 = cursor;
  s->ptr = tmp___114;
  yych = *tmp___114;
  if ((int )yych <= 76) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy228;
      } else {

      }
      goto yy223;
    } else {
      if ((int )yych <= 46) {
        goto yy286;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy223;
      } else {

      }
      goto yy237;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 77) {
      goto yy346;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy237;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych == 109) {
      goto yy346;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy237;
    } else {

    }
    goto yy223;
  }
  yy346: 
  yyaccept = 7U;
  cursor ++;
  tmp___115 = cursor;
  s->ptr = tmp___115;
  yych = *tmp___115;
  if ((int )yych <= 41) {
    if ((int )yych <= 9) {
      if ((int )yych <= 0) {
        goto yy278;
      } else {

      }
      if ((int )yych <= 8) {
        goto yy223;
      } else {

      }
      goto yy278;
    } else {
      if ((int )yych == 32) {
        goto yy278;
      } else {

      }
      if ((int )yych <= 40) {
        goto yy223;
      } else {

      }
      goto yy228;
    }
  } else
  if ((int )yych <= 64) {
    if ((int )yych == 46) {
      goto yy288;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych <= 90) {
      goto yy238;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy223;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy238;
    } else {

    }
    goto yy223;
  }
  yy347: 
  yyaccept = 7U;
  cursor ++;
  tmp___116 = cursor;
  s->ptr = tmp___116;
  yych = *tmp___116;
  if ((int )yych <= 45) {
    if ((int )yych <= 31) {
      if ((int )yych <= 0) {
        goto yy278;
      } else {

      }
      if ((int )yych == 9) {
        goto yy278;
      } else {

      }
      goto yy223;
    } else
    if ((int )yych <= 40) {
      if ((int )yych <= 32) {
        goto yy278;
      } else {

      }
      goto yy223;
    } else {
      if ((int )yych <= 41) {
        goto yy228;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy223;
      } else {

      }
      goto yy244;
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 46) {
      goto yy288;
    } else {

    }
    if ((int )yych <= 47) {
      goto yy244;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy223;
    } else {

    }
    goto yy238;
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 94) {
      goto yy223;
    } else {

    }
    goto yy244;
  } else {
    if ((int )yych <= 96) {
      goto yy223;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy243;
    } else {

    }
    goto yy223;
  }
  yy348: 
  yyaccept = 6U;
  cursor ++;
  tmp___117 = cursor;
  s->ptr = tmp___117;
  yych = *tmp___117;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy225;
    } else {

    }
    goto yy199;
  } else {
    if ((int )yych <= 57) {
      goto yy289;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy225;
    } else {

    }
    goto yy199;
  }
  yy349: 
  yyaccept = 6U;
  cursor ++;
  tmp___118 = cursor;
  s->ptr = tmp___118;
  yych = *tmp___118;
  if ((int )yych <= 47) {
    if ((int )yych != 46) {
      goto yy199;
    } else {

    }
  } else {
    if ((int )yych <= 57) {
      goto yy289;
    } else {

    }
    if ((int )yych >= 59) {
      goto yy199;
    } else {

    }
  }
  yy350: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy351;
  } else {

  }
  if ((int )yych <= 54) {
    goto yy352;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy224;
  } else {

  }
  goto yy57;
  yy351: 
  yyaccept = 7U;
  cursor ++;
  tmp___119 = cursor;
  s->ptr = tmp___119;
  yych = *tmp___119;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy225;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych <= 57) {
      goto yy353;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy225;
    } else {

    }
    goto yy223;
  }
  yy352: 
  yyaccept = 7U;
  cursor ++;
  tmp___120 = cursor;
  s->ptr = tmp___120;
  yych = *tmp___120;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy225;
    } else {

    }
    goto yy223;
  } else {
    if ((int )yych <= 48) {
      goto yy229;
    } else {

    }
    if ((int )yych == 58) {
      goto yy225;
    } else {

    }
    goto yy223;
  }
  yy353: 
  yyaccept = 7U;
  cursor ++;
  tmp___121 = cursor;
  s->ptr = tmp___121;
  yych = *tmp___121;
  if ((int )yych <= 44) {
    if ((int )yych <= 32) {
      if ((int )yych == 9) {
        goto yy231;
      } else {

      }
      if ((int )yych <= 31) {
        goto yy223;
      } else {

      }
      goto yy231;
    } else
    if ((int )yych <= 40) {
      if ((int )yych <= 39) {
        goto yy223;
      } else {

      }
      goto yy231;
    } else {
      if ((int )yych == 43) {
        goto yy231;
      } else {

      }
      goto yy223;
    }
  } else
  if ((int )yych <= 58) {
    if ((int )yych <= 45) {
      goto yy231;
    } else {

    }
    if ((int )yych <= 46) {
      goto yy225;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy223;
    } else {

    }
    goto yy225;
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 64) {
      goto yy223;
    } else {

    }
    goto yy231;
  } else {
    if ((int )yych <= 96) {
      goto yy223;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy231;
    } else {

    }
    goto yy223;
  }
  yy354: 
  yyaccept = 6U;
  cursor ++;
  tmp___122 = cursor;
  s->ptr = tmp___122;
  yych = *tmp___122;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy338;
    } else {

    }
    goto yy199;
  } else {
    if ((int )yych <= 57) {
      goto yy289;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy338;
    } else {

    }
    goto yy199;
  }
  yy355: 
  yyaccept = 6U;
  cursor ++;
  tmp___123 = cursor;
  s->ptr = tmp___123;
  yych = *tmp___123;
  if ((int )yych <= 57) {
    if ((int )yych <= 47) {
      if ((int )yych == 46) {
        goto yy300;
      } else {

      }
      goto yy216;
    } else {
      if ((int )yych <= 48) {
        goto yy332;
      } else {

      }
      if ((int )yych <= 49) {
        goto yy302;
      } else {

      }
      if ((int )yych <= 50) {
        goto yy303;
      } else {

      }
      goto yy297;
    }
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 58) {
      goto yy264;
    } else {

    }
    if ((int )yych == 110) {
      goto yy212;
    } else {

    }
    goto yy216;
  } else {
    if ((int )yych <= 114) {
      goto yy213;
    } else {

    }
    if ((int )yych <= 115) {
      goto yy211;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy214;
    } else {

    }
    goto yy216;
  }
  yy356: 
  yyaccept = 6U;
  cursor ++;
  tmp___124 = cursor;
  s->ptr = tmp___124;
  yych = *tmp___124;
  if ((int )yych <= 57) {
    if ((int )yych <= 47) {
      if ((int )yych == 46) {
        goto yy300;
      } else {

      }
      goto yy216;
    } else {
      if ((int )yych <= 48) {
        goto yy332;
      } else {

      }
      if ((int )yych <= 49) {
        goto yy302;
      } else {

      }
      if ((int )yych <= 50) {
        goto yy303;
      } else {

      }
      goto yy297;
    }
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 58) {
      goto yy264;
    } else {

    }
    if ((int )yych == 110) {
      goto yy212;
    } else {

    }
    goto yy216;
  } else {
    if ((int )yych <= 114) {
      goto yy213;
    } else {

    }
    if ((int )yych <= 115) {
      goto yy211;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy214;
    } else {

    }
    goto yy216;
  }
  yy357: 
  yyaccept = 6U;
  cursor ++;
  tmp___125 = cursor;
  s->ptr = tmp___125;
  yych = *tmp___125;
  if ((int )yych <= 109) {
    if ((int )yych <= 49) {
      if ((int )yych <= 47) {
        goto yy216;
      } else {

      }
      if ((int )yych <= 48) {
        goto yy360;
      } else {

      }
      goto yy361;
    } else {
      if ((int )yych <= 50) {
        goto yy368;
      } else {

      }
      if ((int )yych <= 57) {
        goto yy369;
      } else {

      }
      goto yy216;
    }
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 110) {
      goto yy212;
    } else {

    }
    if ((int )yych <= 113) {
      goto yy216;
    } else {

    }
    goto yy213;
  } else {
    if ((int )yych <= 115) {
      goto yy211;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy214;
    } else {

    }
    goto yy216;
  }
  yy358: 
  yyaccept = 6U;
  cursor ++;
  tmp___126 = cursor;
  s->ptr = tmp___126;
  yych = *tmp___126;
  if ((int )yych <= 109) {
    if ((int )yych <= 49) {
      if ((int )yych <= 47) {
        goto yy216;
      } else {

      }
      if ((int )yych <= 48) {
        goto yy360;
      } else {

      }
      goto yy361;
    } else {
      if ((int )yych <= 50) {
        goto yy368;
      } else {

      }
      if ((int )yych <= 57) {
        goto yy369;
      } else {

      }
      goto yy216;
    }
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 110) {
      goto yy212;
    } else {

    }
    if ((int )yych <= 113) {
      goto yy216;
    } else {

    }
    goto yy213;
  } else {
    if ((int )yych <= 115) {
      goto yy211;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy214;
    } else {

    }
    goto yy216;
  }
  yy359: 
  yyaccept = 6U;
  cursor ++;
  tmp___127 = cursor;
  s->ptr = tmp___127;
  yych = *tmp___127;
  if ((int )yych <= 109) {
    if ((int )yych <= 49) {
      if ((int )yych <= 47) {
        goto yy216;
      } else {

      }
      if ((int )yych >= 49) {
        goto yy361;
      } else {

      }
    } else {
      if ((int )yych <= 50) {
        goto yy209;
      } else {

      }
      if ((int )yych <= 57) {
        goto yy210;
      } else {

      }
      goto yy216;
    }
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 110) {
      goto yy212;
    } else {

    }
    if ((int )yych <= 113) {
      goto yy216;
    } else {

    }
    goto yy213;
  } else {
    if ((int )yych <= 115) {
      goto yy211;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy214;
    } else {

    }
    goto yy216;
  }
  yy360: 
  yyaccept = 6U;
  cursor ++;
  tmp___128 = cursor;
  s->ptr = tmp___128;
  yych = *tmp___128;
  if ((int )yych <= 57) {
    if ((int )yych <= 47) {
      if ((int )yych <= 44) {
        goto yy216;
      } else {

      }
      if ((int )yych <= 45) {
        goto yy362;
      } else {

      }
      if ((int )yych <= 46) {
        goto yy331;
      } else {

      }
      goto yy216;
    } else {
      if ((int )yych <= 48) {
        goto yy332;
      } else {

      }
      if ((int )yych <= 49) {
        goto yy302;
      } else {

      }
      if ((int )yych <= 50) {
        goto yy303;
      } else {

      }
      goto yy297;
    }
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 58) {
      goto yy221;
    } else {

    }
    if ((int )yych == 110) {
      goto yy212;
    } else {

    }
    goto yy216;
  } else {
    if ((int )yych <= 114) {
      goto yy213;
    } else {

    }
    if ((int )yych <= 115) {
      goto yy211;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy214;
    } else {

    }
    goto yy216;
  }
  yy361: 
  yyaccept = 6U;
  cursor ++;
  tmp___129 = cursor;
  s->ptr = tmp___129;
  yych = *tmp___129;
  if ((int )yych <= 57) {
    if ((int )yych <= 47) {
      if ((int )yych <= 44) {
        goto yy216;
      } else {

      }
      if ((int )yych <= 45) {
        goto yy362;
      } else {

      }
      if ((int )yych <= 46) {
        goto yy300;
      } else {

      }
      goto yy216;
    } else {
      if ((int )yych <= 48) {
        goto yy301;
      } else {

      }
      if ((int )yych <= 49) {
        goto yy302;
      } else {

      }
      if ((int )yych <= 50) {
        goto yy303;
      } else {

      }
      goto yy297;
    }
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 58) {
      goto yy264;
    } else {

    }
    if ((int )yych == 110) {
      goto yy212;
    } else {

    }
    goto yy216;
  } else {
    if ((int )yych <= 114) {
      goto yy213;
    } else {

    }
    if ((int )yych <= 115) {
      goto yy211;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy214;
    } else {

    }
    goto yy216;
  }
  yy362: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy364;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy365;
  } else {

  }
  yy364: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  if ((s->time)->have_date) {
    add_error(s, (char *)"Double date specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_date = 1U;
  }
  tmp___130 = timelib_get_month(& ptr);
  (s->time)->m = (timelib_sll )tmp___130;
  (s->time)->d = timelib_get_nr(& ptr, 2);
  (s->time)->y = timelib_get_nr(& ptr, 4);
  if ((s->time)->y == -99999LL) {

  } else
  if ((s->time)->y < 100LL) {
    if ((s->time)->y < 70LL) {
      (s->time)->y += 2000LL;
    } else {
      (s->time)->y += 1900LL;
    }
  } else {

  }
  free((void *)str);
  return (272);
  yy365: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy364;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy364;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy364;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy364;
  } else {

  }
  cursor ++;
  yych = *cursor;
  goto yy364;
  yy368: 
  yyaccept = 6U;
  cursor ++;
  tmp___131 = cursor;
  s->ptr = tmp___131;
  yych = *tmp___131;
  if ((int )yych <= 57) {
    if ((int )yych <= 47) {
      if ((int )yych <= 44) {
        goto yy216;
      } else {

      }
      if ((int )yych <= 45) {
        goto yy362;
      } else {

      }
      if ((int )yych <= 46) {
        goto yy300;
      } else {

      }
      goto yy216;
    } else {
      if ((int )yych <= 48) {
        goto yy332;
      } else {

      }
      if ((int )yych <= 49) {
        goto yy302;
      } else {

      }
      if ((int )yych <= 50) {
        goto yy303;
      } else {

      }
      goto yy297;
    }
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 58) {
      goto yy264;
    } else {

    }
    if ((int )yych == 110) {
      goto yy212;
    } else {

    }
    goto yy216;
  } else {
    if ((int )yych <= 114) {
      goto yy213;
    } else {

    }
    if ((int )yych <= 115) {
      goto yy211;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy214;
    } else {

    }
    goto yy216;
  }
  yy369: 
  yyaccept = 6U;
  cursor ++;
  tmp___132 = cursor;
  s->ptr = tmp___132;
  yych = *tmp___132;
  if ((int )yych <= 57) {
    if ((int )yych <= 47) {
      if ((int )yych <= 44) {
        goto yy216;
      } else {

      }
      if ((int )yych <= 45) {
        goto yy362;
      } else {

      }
      if ((int )yych <= 46) {
        goto yy300;
      } else {

      }
      goto yy216;
    } else {
      if ((int )yych <= 48) {
        goto yy332;
      } else {

      }
      if ((int )yych <= 49) {
        goto yy302;
      } else {

      }
      if ((int )yych <= 50) {
        goto yy303;
      } else {

      }
      goto yy297;
    }
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 58) {
      goto yy264;
    } else {

    }
    if ((int )yych == 110) {
      goto yy212;
    } else {

    }
    goto yy216;
  } else {
    if ((int )yych <= 114) {
      goto yy213;
    } else {

    }
    if ((int )yych <= 115) {
      goto yy211;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy214;
    } else {

    }
    goto yy216;
  }
  yy370: 
  yyaccept = 0U;
  cursor ++;
  tmp___133 = cursor;
  s->ptr = tmp___133;
  yych = *tmp___133;
  if ((int )yych <= 66) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 67) {
      goto yy193;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 98) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 99) {
      goto yy371;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy371: 
  yyaccept = 5U;
  cursor ++;
  tmp___134 = cursor;
  s->ptr = tmp___134;
  yych = *tmp___134;
  if ((int )yych <= 57) {
    if ((int )yych <= 40) {
      if ((int )yych <= 9) {
        if ((int )yych <= 8) {
          goto yy194;
        } else {

        }
        goto yy196;
      } else {
        if ((int )yych == 32) {
          goto yy196;
        } else {

        }
        goto yy194;
      }
    } else
    if ((int )yych <= 45) {
      if ((int )yych <= 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy194;
      } else {

      }
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      goto yy196;
    }
  } else
  if ((int )yych <= 94) {
    if ((int )yych <= 68) {
      if ((int )yych <= 64) {
        goto yy194;
      } else {

      }
      goto yy143;
    } else {
      if ((int )yych <= 69) {
        goto yy202;
      } else {

      }
      if ((int )yych <= 90) {
        goto yy143;
      } else {

      }
      goto yy194;
    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 95) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 101) {
      goto yy373;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy194;
  }
  yy372: 
  cursor ++;
  yych = *cursor;
  if ((int const   )yybm[(int )yych] & 8) {
    goto yy149;
  } else {

  }
  if ((int )yych <= 47) {
    goto yy196;
  } else {

  }
  if ((int )yych <= 48) {
    goto yy357;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy358;
  } else {

  }
  if ((int )yych <= 51) {
    goto yy359;
  } else {

  }
  goto yy196;
  yy373: 
  yyaccept = 0U;
  cursor ++;
  tmp___135 = cursor;
  s->ptr = tmp___135;
  yych = *tmp___135;
  if ((int )yych <= 76) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 77) {
      goto yy203;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 108) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 109) {
      goto yy374;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy374: 
  yyaccept = 0U;
  cursor ++;
  tmp___136 = cursor;
  s->ptr = tmp___136;
  yych = *tmp___136;
  if ((int )yych <= 65) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy145;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 66) {
      goto yy204;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy153;
  } else {
    if ((int )yych <= 98) {
      goto yy375;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy4;
  }
  yy375: 
  yyaccept = 0U;
  cursor ++;
  tmp___137 = cursor;
  s->ptr = tmp___137;
  yych = *tmp___137;
  if ((int )yych <= 68) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 69) {
      goto yy205;
    } else {

    }
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych == 101) {
      goto yy376;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy154;
    } else {

    }
    goto yy4;
  }
  yy376: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy206;
  } else {

  }
  if ((int )yych != 114) {
    goto yy155;
  } else {

  }
  yy377: 
  yyaccept = 5U;
  cursor ++;
  tmp___138 = cursor;
  s->ptr = tmp___138;
  yych = *tmp___138;
  if ((int const   )yybm[(int )yych] & 16) {
    goto yy154;
  } else {

  }
  if ((int )yych <= 45) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy196;
      } else {

      }
      goto yy194;
    } else {
      if ((int )yych <= 32) {
        goto yy196;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy194;
      } else {

      }
    }
  } else
  if ((int )yych <= 57) {
    if ((int )yych == 47) {
      goto yy148;
    } else {

    }
    goto yy196;
  } else {
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy194;
  }
  yy378: 
  cursor ++;
  yych = *cursor;
  if ((int const   )yybm[(int )yych] & 8) {
    goto yy149;
  } else {

  }
  goto yy196;
  yy379: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 84) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 83) {
        goto yy142;
      } else {

      }
    }
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else {
    if ((int )yych <= 116) {
      goto yy380;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy380: 
  yyaccept = 5U;
  cursor ++;
  tmp___139 = cursor;
  s->ptr = tmp___139;
  yych = *tmp___139;
  if ((int )yych <= 47) {
    if ((int )yych <= 40) {
      if ((int )yych <= 9) {
        if ((int )yych <= 8) {
          goto yy194;
        } else {

        }
        goto yy196;
      } else {
        if ((int )yych == 32) {
          goto yy196;
        } else {

        }
        goto yy194;
      }
    } else
    if ((int )yych <= 44) {
      if ((int )yych <= 41) {
        goto yy140;
      } else {

      }
      goto yy194;
    } else {
      if ((int )yych <= 45) {
        goto yy197;
      } else {

      }
      if ((int )yych <= 46) {
        goto yy196;
      } else {

      }
      goto yy194;
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 64) {
      if ((int )yych <= 57) {
        goto yy196;
      } else {

      }
      goto yy194;
    } else
    if ((int )yych != 79) {
      goto yy143;
    } else {

    }
  } else
  if ((int )yych <= 110) {
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 111) {
      goto yy381;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy194;
  }
  yy381: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 66) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 65) {
        goto yy144;
      } else {

      }
    }
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 98) {
      goto yy382;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy382: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 69) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 68) {
        goto yy145;
      } else {

      }
    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy145;
  } else {
    if ((int )yych <= 101) {
      goto yy383;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy4;
  }
  yy383: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 81) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 82) {
      goto yy206;
    } else {

    }
    if ((int )yych == 114) {
      goto yy206;
    } else {

    }
    goto yy4;
  }
  yy384: 
  yyaccept = 0U;
  cursor ++;
  tmp___140 = cursor;
  s->ptr = tmp___140;
  yych = *tmp___140;
  if ((int )yych <= 83) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 84) {
      goto yy380;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 116) {
      goto yy385;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy385: 
  yyaccept = 5U;
  cursor ++;
  tmp___141 = cursor;
  s->ptr = tmp___141;
  yych = *tmp___141;
  if ((int )yych <= 57) {
    if ((int )yych <= 40) {
      if ((int )yych <= 9) {
        if ((int )yych <= 8) {
          goto yy194;
        } else {

        }
        goto yy196;
      } else {
        if ((int )yych == 32) {
          goto yy196;
        } else {

        }
        goto yy194;
      }
    } else
    if ((int )yych <= 45) {
      if ((int )yych <= 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy194;
      } else {

      }
      goto yy372;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      goto yy196;
    }
  } else
  if ((int )yych <= 94) {
    if ((int )yych <= 78) {
      if ((int )yych <= 64) {
        goto yy194;
      } else {

      }
      goto yy143;
    } else {
      if ((int )yych <= 79) {
        goto yy381;
      } else {

      }
      if ((int )yych <= 90) {
        goto yy143;
      } else {

      }
      goto yy194;
    }
  } else
  if ((int )yych <= 110) {
    if ((int )yych <= 95) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 111) {
      goto yy386;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy194;
  }
  yy386: 
  yyaccept = 0U;
  cursor ++;
  tmp___142 = cursor;
  s->ptr = tmp___142;
  yych = *tmp___142;
  if ((int )yych <= 65) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 66) {
      goto yy382;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 98) {
      goto yy387;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy387: 
  yyaccept = 0U;
  cursor ++;
  tmp___143 = cursor;
  s->ptr = tmp___143;
  yych = *tmp___143;
  if ((int )yych <= 68) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy145;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 69) {
      goto yy383;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy153;
  } else {
    if ((int )yych <= 101) {
      goto yy388;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy4;
  }
  yy388: 
  yyaccept = 0U;
  cursor ++;
  tmp___144 = cursor;
  s->ptr = tmp___144;
  yych = *tmp___144;
  if ((int )yych <= 81) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 82) {
      goto yy206;
    } else {

    }
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych == 114) {
      goto yy377;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy154;
    } else {

    }
    goto yy4;
  }
  yy389: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 71) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 70) {
        goto yy142;
      } else {

      }
      goto yy397;
    }
  } else
  if ((int )yych <= 102) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else {
    if ((int )yych <= 103) {
      goto yy397;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy390: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 82) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 81) {
        goto yy142;
      } else {

      }
      goto yy394;
    }
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else {
    if ((int )yych <= 114) {
      goto yy394;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy391: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 79) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 78) {
        goto yy142;
      } else {

      }
    }
  } else
  if ((int )yych <= 110) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else {
    if ((int )yych <= 111) {
      goto yy392;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy392: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 64) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
  } else {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy393;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
  }
  yy393: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  (s->time)->relative.y = 0LL - (s->time)->relative.y;
  (s->time)->relative.m = 0LL - (s->time)->relative.m;
  (s->time)->relative.d = 0LL - (s->time)->relative.d;
  (s->time)->relative.h = 0LL - (s->time)->relative.h;
  (s->time)->relative.i = 0LL - (s->time)->relative.i;
  (s->time)->relative.s = 0LL - (s->time)->relative.s;
  (s->time)->relative.weekday = 0 - (s->time)->relative.weekday;
  if ((s->time)->relative.weekday == 0) {
    (s->time)->relative.weekday = -7;
  } else {

  }
  if ((s->time)->relative.have_special_relative) {
    if ((s->time)->relative.special.type == 1U) {
      (s->time)->relative.special.amount = 0LL - (s->time)->relative.special.amount;
    } else {

    }
  } else {

  }
  free((void *)str);
  return (301);
  yy394: 
  yyaccept = 5U;
  cursor ++;
  tmp___145 = cursor;
  s->ptr = tmp___145;
  yych = *tmp___145;
  if ((int )yych <= 47) {
    if ((int )yych <= 40) {
      if ((int )yych <= 9) {
        if ((int )yych <= 8) {
          goto yy194;
        } else {

        }
        goto yy196;
      } else {
        if ((int )yych == 32) {
          goto yy196;
        } else {

        }
        goto yy194;
      }
    } else
    if ((int )yych <= 44) {
      if ((int )yych <= 41) {
        goto yy140;
      } else {

      }
      goto yy194;
    } else {
      if ((int )yych <= 45) {
        goto yy197;
      } else {

      }
      if ((int )yych <= 46) {
        goto yy196;
      } else {

      }
      goto yy194;
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 64) {
      if ((int )yych <= 57) {
        goto yy196;
      } else {

      }
      goto yy194;
    } else
    if ((int )yych != 73) {
      goto yy143;
    } else {

    }
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 105) {
      goto yy395;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy194;
  }
  yy395: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 76) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 75) {
        goto yy144;
      } else {

      }
    }
  } else
  if ((int )yych <= 107) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 108) {
      goto yy396;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy396: 
  yyaccept = 5U;
  cursor ++;
  tmp___146 = cursor;
  s->ptr = tmp___146;
  yych = *tmp___146;
  if ((int )yych <= 44) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy196;
      } else {

      }
      goto yy194;
    } else {
      if ((int )yych <= 32) {
        goto yy196;
      } else {

      }
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy194;
    }
  } else
  if ((int )yych <= 64) {
    if ((int )yych == 47) {
      goto yy194;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy196;
    } else {

    }
    goto yy194;
  } else {
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy194;
  }
  yy397: 
  yyaccept = 5U;
  cursor ++;
  tmp___147 = cursor;
  s->ptr = tmp___147;
  yych = *tmp___147;
  if ((int )yych <= 47) {
    if ((int )yych <= 40) {
      if ((int )yych <= 9) {
        if ((int )yych <= 8) {
          goto yy194;
        } else {

        }
        goto yy196;
      } else {
        if ((int )yych == 32) {
          goto yy196;
        } else {

        }
        goto yy194;
      }
    } else
    if ((int )yych <= 44) {
      if ((int )yych <= 41) {
        goto yy140;
      } else {

      }
      goto yy194;
    } else {
      if ((int )yych <= 45) {
        goto yy197;
      } else {

      }
      if ((int )yych <= 46) {
        goto yy196;
      } else {

      }
      goto yy194;
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 64) {
      if ((int )yych <= 57) {
        goto yy196;
      } else {

      }
      goto yy194;
    } else
    if ((int )yych != 85) {
      goto yy143;
    } else {

    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 117) {
      goto yy398;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy194;
  }
  yy398: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 83) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 82) {
        goto yy144;
      } else {

      }
    }
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 115) {
      goto yy399;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy399: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 84) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 83) {
        goto yy145;
      } else {

      }
    }
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy145;
  } else {
    if ((int )yych <= 116) {
      goto yy400;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy4;
  }
  yy400: 
  yyaccept = 5U;
  cursor ++;
  tmp___148 = cursor;
  s->ptr = tmp___148;
  yych = *tmp___148;
  if ((int )yych <= 40) {
    if ((int )yych <= 9) {
      if ((int )yych <= 8) {
        goto yy194;
      } else {

      }
      goto yy196;
    } else {
      if ((int )yych == 32) {
        goto yy196;
      } else {

      }
      goto yy194;
    }
  } else
  if ((int )yych <= 46) {
    if ((int )yych <= 41) {
      goto yy140;
    } else {

    }
    if ((int )yych <= 44) {
      goto yy194;
    } else {

    }
    goto yy196;
  } else {
    if ((int )yych <= 47) {
      goto yy194;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy196;
    } else {

    }
    goto yy194;
  }
  yy401: 
  yyaccept = 0U;
  cursor ++;
  tmp___149 = cursor;
  s->ptr = tmp___149;
  yych = *tmp___149;
  if ((int )yych <= 70) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 71) {
      goto yy397;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 102) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 103) {
      goto yy408;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy402: 
  yyaccept = 0U;
  cursor ++;
  tmp___150 = cursor;
  s->ptr = tmp___150;
  yych = *tmp___150;
  if ((int )yych <= 81) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 82) {
      goto yy394;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 114) {
      goto yy405;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy403: 
  yyaccept = 0U;
  cursor ++;
  tmp___151 = cursor;
  s->ptr = tmp___151;
  yych = *tmp___151;
  if ((int )yych <= 78) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 79) {
      goto yy392;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 110) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 111) {
      goto yy404;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy404: 
  yyaccept = 9U;
  cursor ++;
  tmp___152 = cursor;
  s->ptr = tmp___152;
  yych = *tmp___152;
  if ((int )yych <= 47) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy393;
    } else {
      if ((int )yych == 46) {
        goto yy393;
      } else {

      }
      goto yy148;
    }
  } else
  if ((int )yych <= 94) {
    if ((int )yych <= 64) {
      goto yy393;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    goto yy393;
  } else {
    if ((int )yych <= 95) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy393;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy393;
  }
  yy405: 
  yyaccept = 5U;
  cursor ++;
  tmp___153 = cursor;
  s->ptr = tmp___153;
  yych = *tmp___153;
  if ((int )yych <= 57) {
    if ((int )yych <= 40) {
      if ((int )yych <= 9) {
        if ((int )yych <= 8) {
          goto yy194;
        } else {

        }
        goto yy196;
      } else {
        if ((int )yych == 32) {
          goto yy196;
        } else {

        }
        goto yy194;
      }
    } else
    if ((int )yych <= 45) {
      if ((int )yych <= 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy194;
      } else {

      }
      goto yy372;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      goto yy196;
    }
  } else
  if ((int )yych <= 94) {
    if ((int )yych <= 72) {
      if ((int )yych <= 64) {
        goto yy194;
      } else {

      }
      goto yy143;
    } else {
      if ((int )yych <= 73) {
        goto yy395;
      } else {

      }
      if ((int )yych <= 90) {
        goto yy143;
      } else {

      }
      goto yy194;
    }
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 95) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 105) {
      goto yy406;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy194;
  }
  yy406: 
  yyaccept = 0U;
  cursor ++;
  tmp___154 = cursor;
  s->ptr = tmp___154;
  yych = *tmp___154;
  if ((int )yych <= 75) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 76) {
      goto yy396;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 107) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 108) {
      goto yy407;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy407: 
  yyaccept = 5U;
  cursor ++;
  tmp___155 = cursor;
  s->ptr = tmp___155;
  yych = *tmp___155;
  if ((int )yych <= 46) {
    if ((int )yych <= 32) {
      if ((int )yych == 9) {
        goto yy196;
      } else {

      }
      if ((int )yych <= 31) {
        goto yy194;
      } else {

      }
      goto yy196;
    } else
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy194;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 44) {
        goto yy194;
      } else {

      }
      if ((int )yych <= 45) {
        goto yy378;
      } else {

      }
      goto yy196;
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 47) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy196;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy194;
    } else {

    }
    goto yy145;
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 94) {
      goto yy194;
    } else {

    }
    goto yy148;
  } else {
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy194;
  }
  yy408: 
  yyaccept = 5U;
  cursor ++;
  tmp___156 = cursor;
  s->ptr = tmp___156;
  yych = *tmp___156;
  if ((int )yych <= 57) {
    if ((int )yych <= 40) {
      if ((int )yych <= 9) {
        if ((int )yych <= 8) {
          goto yy194;
        } else {

        }
        goto yy196;
      } else {
        if ((int )yych == 32) {
          goto yy196;
        } else {

        }
        goto yy194;
      }
    } else
    if ((int )yych <= 45) {
      if ((int )yych <= 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy194;
      } else {

      }
      goto yy372;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      goto yy196;
    }
  } else
  if ((int )yych <= 94) {
    if ((int )yych <= 84) {
      if ((int )yych <= 64) {
        goto yy194;
      } else {

      }
      goto yy143;
    } else {
      if ((int )yych <= 85) {
        goto yy398;
      } else {

      }
      if ((int )yych <= 90) {
        goto yy143;
      } else {

      }
      goto yy194;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych <= 95) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 117) {
      goto yy409;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy194;
  }
  yy409: 
  yyaccept = 0U;
  cursor ++;
  tmp___157 = cursor;
  s->ptr = tmp___157;
  yych = *tmp___157;
  if ((int )yych <= 82) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 83) {
      goto yy399;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 115) {
      goto yy410;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy410: 
  yyaccept = 0U;
  cursor ++;
  tmp___158 = cursor;
  s->ptr = tmp___158;
  yych = *tmp___158;
  if ((int )yych <= 83) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy145;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 84) {
      goto yy400;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy153;
  } else {
    if ((int )yych <= 116) {
      goto yy411;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy4;
  }
  yy411: 
  yyaccept = 5U;
  cursor ++;
  tmp___159 = cursor;
  s->ptr = tmp___159;
  yych = *tmp___159;
  if ((int const   )yybm[(int )yych] & 16) {
    goto yy154;
  } else {

  }
  if ((int )yych <= 44) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy196;
      } else {

      }
      goto yy194;
    } else {
      if ((int )yych <= 32) {
        goto yy196;
      } else {

      }
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy194;
    }
  } else
  if ((int )yych <= 47) {
    if ((int )yych <= 45) {
      goto yy378;
    } else {

    }
    if ((int )yych <= 46) {
      goto yy196;
    } else {

    }
    goto yy148;
  } else {
    if ((int )yych <= 57) {
      goto yy196;
    } else {

    }
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy194;
  }
  yy412: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 78) {
    if ((int )yych <= 64) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 76) {
        goto yy419;
      } else {

      }
      if ((int )yych <= 77) {
        goto yy142;
      } else {

      }
      goto yy418;
    }
  } else
  if ((int )yych <= 108) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 107) {
      goto yy142;
    } else {

    }
    goto yy419;
  } else {
    if ((int )yych == 110) {
      goto yy418;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy413: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 78) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 77) {
        goto yy142;
      } else {

      }
    }
  } else
  if ((int )yych <= 109) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else {
    if ((int )yych <= 110) {
      goto yy414;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy414: 
  yyaccept = 5U;
  cursor ++;
  tmp___160 = cursor;
  s->ptr = tmp___160;
  yych = *tmp___160;
  if ((int )yych <= 47) {
    if ((int )yych <= 40) {
      if ((int )yych <= 9) {
        if ((int )yych <= 8) {
          goto yy194;
        } else {

        }
        goto yy196;
      } else {
        if ((int )yych == 32) {
          goto yy196;
        } else {

        }
        goto yy194;
      }
    } else
    if ((int )yych <= 44) {
      if ((int )yych <= 41) {
        goto yy140;
      } else {

      }
      goto yy194;
    } else {
      if ((int )yych <= 45) {
        goto yy197;
      } else {

      }
      if ((int )yych <= 46) {
        goto yy196;
      } else {

      }
      goto yy194;
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 64) {
      if ((int )yych <= 57) {
        goto yy196;
      } else {

      }
      goto yy194;
    } else
    if ((int )yych != 85) {
      goto yy143;
    } else {

    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 117) {
      goto yy415;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy194;
  }
  yy415: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 65) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 97) {
      goto yy416;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy416: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 82) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 81) {
        goto yy145;
      } else {

      }
    }
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy145;
  } else {
    if ((int )yych <= 114) {
      goto yy417;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy4;
  }
  yy417: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 88) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 89) {
      goto yy206;
    } else {

    }
    if ((int )yych == 121) {
      goto yy206;
    } else {

    }
    goto yy4;
  }
  yy418: 
  yyaccept = 5U;
  cursor ++;
  tmp___161 = cursor;
  s->ptr = tmp___161;
  yych = *tmp___161;
  if ((int )yych <= 47) {
    if ((int )yych <= 40) {
      if ((int )yych <= 9) {
        if ((int )yych <= 8) {
          goto yy194;
        } else {

        }
        goto yy196;
      } else {
        if ((int )yych == 32) {
          goto yy196;
        } else {

        }
        goto yy194;
      }
    } else
    if ((int )yych <= 44) {
      if ((int )yych <= 41) {
        goto yy140;
      } else {

      }
      goto yy194;
    } else {
      if ((int )yych <= 45) {
        goto yy197;
      } else {

      }
      if ((int )yych <= 46) {
        goto yy196;
      } else {

      }
      goto yy194;
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 64) {
      if ((int )yych <= 57) {
        goto yy196;
      } else {

      }
      goto yy194;
    } else {
      if ((int )yych == 69) {
        goto yy420;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 101) {
      goto yy420;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy194;
  }
  yy419: 
  yyaccept = 5U;
  cursor ++;
  tmp___162 = cursor;
  s->ptr = tmp___162;
  yych = *tmp___162;
  if ((int )yych <= 47) {
    if ((int )yych <= 40) {
      if ((int )yych <= 9) {
        if ((int )yych <= 8) {
          goto yy194;
        } else {

        }
        goto yy196;
      } else {
        if ((int )yych == 32) {
          goto yy196;
        } else {

        }
        goto yy194;
      }
    } else
    if ((int )yych <= 44) {
      if ((int )yych <= 41) {
        goto yy140;
      } else {

      }
      goto yy194;
    } else {
      if ((int )yych <= 45) {
        goto yy197;
      } else {

      }
      if ((int )yych <= 46) {
        goto yy196;
      } else {

      }
      goto yy194;
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 64) {
      if ((int )yych <= 57) {
        goto yy196;
      } else {

      }
      goto yy194;
    } else
    if ((int )yych != 89) {
      goto yy143;
    } else {

    }
  } else
  if ((int )yych <= 120) {
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 121) {
      goto yy420;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy194;
  }
  yy420: 
  yyaccept = 5U;
  cursor ++;
  tmp___163 = cursor;
  s->ptr = tmp___163;
  yych = *tmp___163;
  if ((int )yych <= 44) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy196;
      } else {

      }
      goto yy194;
    } else {
      if ((int )yych <= 32) {
        goto yy196;
      } else {

      }
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy194;
    }
  } else
  if ((int )yych <= 64) {
    if ((int )yych == 47) {
      goto yy194;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy196;
    } else {

    }
    goto yy194;
  } else {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy194;
  }
  yy421: 
  yyaccept = 0U;
  cursor ++;
  tmp___164 = cursor;
  s->ptr = tmp___164;
  yych = *tmp___164;
  if ((int )yych <= 77) {
    if ((int )yych <= 46) {
      if ((int )yych <= 41) {
        if ((int )yych <= 40) {
          goto yy4;
        } else {

        }
        goto yy140;
      } else {
        if ((int )yych == 45) {
          goto yy148;
        } else {

        }
        goto yy4;
      }
    } else
    if ((int )yych <= 64) {
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 76) {
        goto yy419;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 90) {
      if ((int )yych <= 78) {
        goto yy418;
      } else {

      }
      goto yy142;
    } else {
      if ((int )yych == 95) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 109) {
    if ((int )yych == 108) {
      goto yy428;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 110) {
      goto yy427;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy422: 
  yyaccept = 0U;
  cursor ++;
  tmp___165 = cursor;
  s->ptr = tmp___165;
  yych = *tmp___165;
  if ((int )yych <= 77) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 78) {
      goto yy414;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 109) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 110) {
      goto yy423;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy423: 
  yyaccept = 5U;
  cursor ++;
  tmp___166 = cursor;
  s->ptr = tmp___166;
  yych = *tmp___166;
  if ((int )yych <= 57) {
    if ((int )yych <= 40) {
      if ((int )yych <= 9) {
        if ((int )yych <= 8) {
          goto yy194;
        } else {

        }
        goto yy196;
      } else {
        if ((int )yych == 32) {
          goto yy196;
        } else {

        }
        goto yy194;
      }
    } else
    if ((int )yych <= 45) {
      if ((int )yych <= 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy194;
      } else {

      }
      goto yy372;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      goto yy196;
    }
  } else
  if ((int )yych <= 94) {
    if ((int )yych <= 84) {
      if ((int )yych <= 64) {
        goto yy194;
      } else {

      }
      goto yy143;
    } else {
      if ((int )yych <= 85) {
        goto yy415;
      } else {

      }
      if ((int )yych <= 90) {
        goto yy143;
      } else {

      }
      goto yy194;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych <= 95) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 117) {
      goto yy424;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy194;
  }
  yy424: 
  yyaccept = 0U;
  cursor ++;
  tmp___167 = cursor;
  s->ptr = tmp___167;
  yych = *tmp___167;
  if ((int )yych <= 64) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 65) {
      goto yy416;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 97) {
      goto yy425;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy425: 
  yyaccept = 0U;
  cursor ++;
  tmp___168 = cursor;
  s->ptr = tmp___168;
  yych = *tmp___168;
  if ((int )yych <= 81) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy145;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 82) {
      goto yy417;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy153;
  } else {
    if ((int )yych <= 114) {
      goto yy426;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy4;
  }
  yy426: 
  yyaccept = 0U;
  cursor ++;
  tmp___169 = cursor;
  s->ptr = tmp___169;
  yych = *tmp___169;
  if ((int )yych <= 88) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 89) {
      goto yy206;
    } else {

    }
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych == 121) {
      goto yy377;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy154;
    } else {

    }
    goto yy4;
  }
  yy427: 
  yyaccept = 5U;
  cursor ++;
  tmp___170 = cursor;
  s->ptr = tmp___170;
  yych = *tmp___170;
  if ((int )yych <= 57) {
    if ((int )yych <= 40) {
      if ((int )yych <= 9) {
        if ((int )yych <= 8) {
          goto yy194;
        } else {

        }
        goto yy196;
      } else {
        if ((int )yych == 32) {
          goto yy196;
        } else {

        }
        goto yy194;
      }
    } else
    if ((int )yych <= 45) {
      if ((int )yych <= 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy194;
      } else {

      }
      goto yy372;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      goto yy196;
    }
  } else
  if ((int )yych <= 94) {
    if ((int )yych <= 68) {
      if ((int )yych <= 64) {
        goto yy194;
      } else {

      }
      goto yy143;
    } else {
      if ((int )yych <= 69) {
        goto yy420;
      } else {

      }
      if ((int )yych <= 90) {
        goto yy143;
      } else {

      }
      goto yy194;
    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 95) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 101) {
      goto yy429;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy194;
  }
  yy428: 
  yyaccept = 5U;
  cursor ++;
  tmp___171 = cursor;
  s->ptr = tmp___171;
  yych = *tmp___171;
  if ((int )yych <= 57) {
    if ((int )yych <= 40) {
      if ((int )yych <= 9) {
        if ((int )yych <= 8) {
          goto yy194;
        } else {

        }
        goto yy196;
      } else {
        if ((int )yych == 32) {
          goto yy196;
        } else {

        }
        goto yy194;
      }
    } else
    if ((int )yych <= 45) {
      if ((int )yych <= 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy194;
      } else {

      }
      goto yy372;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      goto yy196;
    }
  } else
  if ((int )yych <= 94) {
    if ((int )yych <= 88) {
      if ((int )yych <= 64) {
        goto yy194;
      } else {

      }
      goto yy143;
    } else {
      if ((int )yych <= 89) {
        goto yy420;
      } else {

      }
      if ((int )yych <= 90) {
        goto yy143;
      } else {

      }
      goto yy194;
    }
  } else
  if ((int )yych <= 120) {
    if ((int )yych <= 95) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 121) {
      goto yy429;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy194;
  }
  yy429: 
  yyaccept = 5U;
  cursor ++;
  tmp___172 = cursor;
  s->ptr = tmp___172;
  yych = *tmp___172;
  if ((int )yych <= 46) {
    if ((int )yych <= 32) {
      if ((int )yych == 9) {
        goto yy196;
      } else {

      }
      if ((int )yych <= 31) {
        goto yy194;
      } else {

      }
      goto yy196;
    } else
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy194;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 44) {
        goto yy194;
      } else {

      }
      if ((int )yych <= 45) {
        goto yy378;
      } else {

      }
      goto yy196;
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 47) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy196;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy194;
    } else {

    }
    goto yy144;
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 94) {
      goto yy194;
    } else {

    }
    goto yy148;
  } else {
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy194;
  }
  yy430: 
  yyaccept = 0U;
  cursor ++;
  tmp___173 = cursor;
  s->ptr = tmp___173;
  yych = *tmp___173;
  if ((int )yych <= 46) {
    if ((int )yych <= 32) {
      if ((int )yych == 9) {
        goto yy196;
      } else {

      }
      if ((int )yych <= 31) {
        goto yy4;
      } else {

      }
      goto yy196;
    } else {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy196;
    }
  } else
  if ((int )yych <= 72) {
    if ((int )yych <= 47) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy196;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else
  if ((int )yych <= 90) {
    if ((int )yych >= 74) {
      goto yy142;
    } else {

    }
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy431: 
  yyaccept = 0U;
  cursor ++;
  tmp___174 = cursor;
  s->ptr = tmp___174;
  yych = *tmp___174;
  if ((int )yych <= 44) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy196;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych <= 32) {
        goto yy196;
      } else {

      }
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 64) {
    if ((int )yych == 47) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy196;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy4;
  }
  yy432: 
  yyaccept = 0U;
  cursor ++;
  tmp___175 = cursor;
  s->ptr = tmp___175;
  yych = *tmp___175;
  if ((int )yych <= 46) {
    if ((int )yych <= 32) {
      if ((int )yych == 9) {
        goto yy196;
      } else {

      }
      if ((int )yych <= 31) {
        goto yy4;
      } else {

      }
      goto yy196;
    } else {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy196;
    }
  } else
  if ((int )yych <= 72) {
    if ((int )yych <= 47) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy196;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else
  if ((int )yych <= 90) {
    if ((int )yych >= 74) {
      goto yy142;
    } else {

    }
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yyaccept = 0U;
  cursor ++;
  tmp___176 = cursor;
  s->ptr = tmp___176;
  yych = *tmp___176;
  if ((int )yych <= 46) {
    if ((int )yych <= 32) {
      if ((int )yych == 9) {
        goto yy196;
      } else {

      }
      if ((int )yych <= 31) {
        goto yy4;
      } else {

      }
      goto yy196;
    } else {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy196;
    }
  } else
  if ((int )yych <= 72) {
    if ((int )yych <= 47) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy196;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    goto yy143;
  } else
  if ((int )yych <= 90) {
    if ((int )yych >= 74) {
      goto yy143;
    } else {

    }
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy4;
  }
  yyaccept = 0U;
  cursor ++;
  tmp___177 = cursor;
  s->ptr = tmp___177;
  yych = *tmp___177;
  if ((int )yych <= 44) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy196;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych <= 32) {
        goto yy196;
      } else {

      }
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 64) {
    if ((int )yych == 47) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy196;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy435: 
  yyaccept = 0U;
  cursor ++;
  tmp___178 = cursor;
  s->ptr = tmp___178;
  yych = *tmp___178;
  if ((int )yych <= 44) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy196;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych <= 32) {
        goto yy196;
      } else {

      }
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 64) {
    if ((int )yych == 47) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy196;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy436: 
  yyaccept = 0U;
  cursor ++;
  tmp___179 = cursor;
  s->ptr = tmp___179;
  yych = *tmp___179;
  if ((int )yych <= 46) {
    if ((int )yych <= 32) {
      if ((int )yych == 9) {
        goto yy196;
      } else {

      }
      if ((int )yych <= 31) {
        goto yy4;
      } else {

      }
      goto yy196;
    } else {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy196;
    }
  } else
  if ((int )yych <= 72) {
    if ((int )yych <= 47) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy196;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 73) {
      goto yy431;
    } else {

    }
    goto yy142;
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy437: 
  yyaccept = 0U;
  cursor ++;
  tmp___180 = cursor;
  s->ptr = tmp___180;
  yych = *tmp___180;
  if ((int )yych <= 86) {
    if ((int )yych <= 68) {
      if ((int )yych <= 32) {
        if ((int )yych == 9) {
          goto yy61;
        } else {

        }
        if ((int )yych <= 31) {
          goto yy4;
        } else {

        }
        goto yy61;
      } else
      if ((int )yych <= 57) {
        if ((int )yych <= 47) {
          goto yy4;
        } else {

        }
        goto yy457;
      } else {
        if ((int )yych <= 58) {
          goto yy163;
        } else {

        }
        if ((int )yych <= 67) {
          goto yy4;
        } else {

        }
        goto yy61;
      }
    } else
    if ((int )yych <= 72) {
      if ((int )yych == 70) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 71) {
        goto yy4;
      } else {

      }
      goto yy61;
    } else
    if ((int )yych <= 77) {
      if ((int )yych <= 76) {
        goto yy4;
      } else {

      }
      goto yy61;
    } else {
      if ((int )yych <= 82) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 84) {
        goto yy61;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 99) {
      if ((int )yych == 88) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 89) {
        goto yy61;
      } else {

      }
      goto yy4;
    } else
    if ((int )yych <= 101) {
      if ((int )yych <= 100) {
        goto yy61;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 103) {
        goto yy4;
      } else {

      }
      goto yy61;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych == 109) {
      goto yy61;
    } else {

    }
    if ((int )yych <= 114) {
      goto yy4;
    } else {

    }
    goto yy61;
  } else
  if ((int )yych <= 119) {
    if ((int )yych <= 118) {
      goto yy4;
    } else {

    }
    goto yy61;
  } else {
    if ((int )yych == 121) {
      goto yy61;
    } else {

    }
    goto yy4;
  }
  yy438: 
  yyaccept = 0U;
  cursor ++;
  tmp___181 = cursor;
  s->ptr = tmp___181;
  yych = *tmp___181;
  if ((int )yych <= 84) {
    if ((int )yych <= 58) {
      if ((int )yych <= 32) {
        if ((int )yych == 9) {
          goto yy61;
        } else {

        }
        if ((int )yych <= 31) {
          goto yy4;
        } else {

        }
        goto yy61;
      } else
      if ((int )yych <= 52) {
        if ((int )yych <= 47) {
          goto yy4;
        } else {

        }
        goto yy457;
      } else {
        if ((int )yych <= 53) {
          goto yy442;
        } else {

        }
        if ((int )yych <= 57) {
          goto yy443;
        } else {

        }
        goto yy163;
      }
    } else
    if ((int )yych <= 71) {
      if ((int )yych <= 68) {
        if ((int )yych <= 67) {
          goto yy4;
        } else {

        }
        goto yy61;
      } else {
        if ((int )yych == 70) {
          goto yy61;
        } else {

        }
        goto yy4;
      }
    } else
    if ((int )yych <= 76) {
      if ((int )yych <= 72) {
        goto yy61;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych <= 77) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 82) {
        goto yy4;
      } else {

      }
      goto yy61;
    }
  } else
  if ((int )yych <= 103) {
    if ((int )yych <= 89) {
      if ((int )yych == 87) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 88) {
        goto yy4;
      } else {

      }
      goto yy61;
    } else
    if ((int )yych <= 100) {
      if ((int )yych <= 99) {
        goto yy4;
      } else {

      }
      goto yy61;
    } else {
      if ((int )yych == 102) {
        goto yy61;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych <= 108) {
      if ((int )yych <= 104) {
        goto yy61;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych <= 109) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 114) {
        goto yy4;
      } else {

      }
      goto yy61;
    }
  } else
  if ((int )yych <= 119) {
    if ((int )yych <= 118) {
      goto yy4;
    } else {

    }
    goto yy61;
  } else {
    if ((int )yych == 121) {
      goto yy61;
    } else {

    }
    goto yy4;
  }
  yy439: 
  yyaccept = 0U;
  cursor ++;
  tmp___182 = cursor;
  s->ptr = tmp___182;
  yych = *tmp___182;
  if ((int )yych <= 84) {
    if ((int )yych <= 67) {
      if ((int )yych <= 32) {
        if ((int )yych == 9) {
          goto yy61;
        } else {

        }
        if ((int )yych <= 31) {
          goto yy4;
        } else {

        }
        goto yy61;
      } else
      if ((int )yych <= 53) {
        if ((int )yych <= 47) {
          goto yy4;
        } else {

        }
        goto yy442;
      } else {
        if ((int )yych <= 57) {
          goto yy443;
        } else {

        }
        if ((int )yych <= 58) {
          goto yy163;
        } else {

        }
        goto yy4;
      }
    } else
    if ((int )yych <= 71) {
      if ((int )yych == 69) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 70) {
        goto yy61;
      } else {

      }
      goto yy4;
    } else
    if ((int )yych <= 76) {
      if ((int )yych <= 72) {
        goto yy61;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych <= 77) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 82) {
        goto yy4;
      } else {

      }
      goto yy61;
    }
  } else
  if ((int )yych <= 103) {
    if ((int )yych <= 89) {
      if ((int )yych == 87) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 88) {
        goto yy4;
      } else {

      }
      goto yy61;
    } else
    if ((int )yych <= 100) {
      if ((int )yych <= 99) {
        goto yy4;
      } else {

      }
      goto yy61;
    } else {
      if ((int )yych == 102) {
        goto yy61;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych <= 108) {
      if ((int )yych <= 104) {
        goto yy61;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych <= 109) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 114) {
        goto yy4;
      } else {

      }
      goto yy61;
    }
  } else
  if ((int )yych <= 119) {
    if ((int )yych <= 118) {
      goto yy4;
    } else {

    }
    goto yy61;
  } else {
    if ((int )yych == 121) {
      goto yy61;
    } else {

    }
    goto yy4;
  }
  yy440: 
  cursor ++;
  if ((unsigned int )s->lim <= (unsigned int )cursor) {
    return (257);
  } else {

  }
  yych = *cursor;
  if ((int const   )yybm[(int )yych] & 4) {
    goto yy58;
  } else {

  }
  if ((int )yych <= 44) {
    if ((int )yych == 43) {
      goto yy440;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 45) {
      goto yy440;
    } else {

    }
    if ((int )yych <= 47) {
      goto yy57;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy55;
    } else {

    }
    goto yy57;
  }
  yy442: 
  yyaccept = 0U;
  cursor ++;
  tmp___183 = cursor;
  s->ptr = tmp___183;
  yych = *tmp___183;
  if ((int )yych <= 86) {
    if ((int )yych <= 68) {
      if ((int )yych <= 32) {
        if ((int )yych == 9) {
          goto yy61;
        } else {

        }
        if ((int )yych <= 31) {
          goto yy4;
        } else {

        }
        goto yy61;
      } else {
        if ((int )yych <= 47) {
          goto yy4;
        } else {

        }
        if ((int )yych <= 57) {
          goto yy456;
        } else {

        }
        if ((int )yych <= 67) {
          goto yy4;
        } else {

        }
        goto yy61;
      }
    } else
    if ((int )yych <= 72) {
      if ((int )yych == 70) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 71) {
        goto yy4;
      } else {

      }
      goto yy61;
    } else
    if ((int )yych <= 77) {
      if ((int )yych <= 76) {
        goto yy4;
      } else {

      }
      goto yy61;
    } else {
      if ((int )yych <= 82) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 84) {
        goto yy61;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 99) {
      if ((int )yych == 88) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 89) {
        goto yy61;
      } else {

      }
      goto yy4;
    } else
    if ((int )yych <= 101) {
      if ((int )yych <= 100) {
        goto yy61;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 103) {
        goto yy4;
      } else {

      }
      goto yy61;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych == 109) {
      goto yy61;
    } else {

    }
    if ((int )yych <= 114) {
      goto yy4;
    } else {

    }
    goto yy61;
  } else
  if ((int )yych <= 119) {
    if ((int )yych <= 118) {
      goto yy4;
    } else {

    }
    goto yy61;
  } else {
    if ((int )yych == 121) {
      goto yy61;
    } else {

    }
    goto yy4;
  }
  yy443: 
  yyaccept = 0U;
  cursor ++;
  tmp___184 = cursor;
  s->ptr = tmp___184;
  yych = *tmp___184;
  if ((int )yych <= 86) {
    if ((int )yych <= 68) {
      if ((int )yych <= 32) {
        if ((int )yych == 9) {
          goto yy61;
        } else {

        }
        if ((int )yych <= 31) {
          goto yy4;
        } else {

        }
        goto yy61;
      } else {
        if ((int )yych <= 47) {
          goto yy4;
        } else {

        }
        if ((int )yych <= 57) {
          goto yy444;
        } else {

        }
        if ((int )yych <= 67) {
          goto yy4;
        } else {

        }
        goto yy61;
      }
    } else
    if ((int )yych <= 72) {
      if ((int )yych == 70) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 71) {
        goto yy4;
      } else {

      }
      goto yy61;
    } else
    if ((int )yych <= 77) {
      if ((int )yych <= 76) {
        goto yy4;
      } else {

      }
      goto yy61;
    } else {
      if ((int )yych <= 82) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 84) {
        goto yy61;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 99) {
      if ((int )yych == 88) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 89) {
        goto yy61;
      } else {

      }
      goto yy4;
    } else
    if ((int )yych <= 101) {
      if ((int )yych <= 100) {
        goto yy61;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 103) {
        goto yy4;
      } else {

      }
      goto yy61;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych == 109) {
      goto yy61;
    } else {

    }
    if ((int )yych <= 114) {
      goto yy4;
    } else {

    }
    goto yy61;
  } else
  if ((int )yych <= 119) {
    if ((int )yych <= 118) {
      goto yy4;
    } else {

    }
    goto yy61;
  } else {
    if ((int )yych == 121) {
      goto yy61;
    } else {

    }
    goto yy4;
  }
  yy444: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy61;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy61;
  } else {

  }
  yy445: 
  cursor ++;
  yych = *cursor;
  if ((int const   )yybm[(int )yych] & 2) {
    goto yy55;
  } else {

  }
  if ((int )yych != 45) {
    goto yy61;
  } else {

  }
  yy446: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 48) {
    goto yy447;
  } else {

  }
  if ((int )yych <= 49) {
    goto yy448;
  } else {

  }
  goto yy57;
  yy447: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy449;
  } else {

  }
  goto yy57;
  yy448: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 51) {
    goto yy57;
  } else {

  }
  yy449: 
  cursor ++;
  yych = *cursor;
  if ((int )yych != 45) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 48) {
    goto yy451;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy452;
  } else {

  }
  if ((int )yych <= 51) {
    goto yy453;
  } else {

  }
  goto yy57;
  yy451: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy454;
  } else {

  }
  goto yy57;
  yy452: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy454;
  } else {

  }
  goto yy57;
  yy453: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 50) {
    goto yy57;
  } else {

  }
  yy454: 
  cursor ++;
  yy455: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  if ((s->time)->have_date) {
    add_error(s, (char *)"Double date specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_date = 1U;
  }
  tmp___185 = timelib_get_unsigned_nr(& ptr, 4);
  (s->time)->y = (timelib_sll )tmp___185;
  (s->time)->m = timelib_get_nr(& ptr, 2);
  (s->time)->d = timelib_get_nr(& ptr, 2);
  free((void *)str);
  return (267);
  yy456: 
  yyaccept = 0U;
  cursor ++;
  tmp___186 = cursor;
  s->ptr = tmp___186;
  yych = *tmp___186;
  if ((int )yych <= 86) {
    if ((int )yych <= 68) {
      if ((int )yych <= 32) {
        if ((int )yych == 9) {
          goto yy61;
        } else {

        }
        if ((int )yych <= 31) {
          goto yy4;
        } else {

        }
        goto yy61;
      } else {
        if ((int )yych <= 47) {
          goto yy4;
        } else {

        }
        if ((int )yych <= 57) {
          goto yy445;
        } else {

        }
        if ((int )yych <= 67) {
          goto yy4;
        } else {

        }
        goto yy61;
      }
    } else
    if ((int )yych <= 72) {
      if ((int )yych == 70) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 71) {
        goto yy4;
      } else {

      }
      goto yy61;
    } else
    if ((int )yych <= 77) {
      if ((int )yych <= 76) {
        goto yy4;
      } else {

      }
      goto yy61;
    } else {
      if ((int )yych <= 82) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 84) {
        goto yy61;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 99) {
      if ((int )yych == 88) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 89) {
        goto yy61;
      } else {

      }
      goto yy4;
    } else
    if ((int )yych <= 101) {
      if ((int )yych <= 100) {
        goto yy61;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 103) {
        goto yy4;
      } else {

      }
      goto yy61;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych == 109) {
      goto yy61;
    } else {

    }
    if ((int )yych <= 114) {
      goto yy4;
    } else {

    }
    goto yy61;
  } else
  if ((int )yych <= 119) {
    if ((int )yych <= 118) {
      goto yy4;
    } else {

    }
    goto yy61;
  } else {
    if ((int )yych == 121) {
      goto yy61;
    } else {

    }
    goto yy4;
  }
  yy457: 
  yyaccept = 0U;
  cursor ++;
  tmp___187 = cursor;
  s->ptr = tmp___187;
  yych = *tmp___187;
  if ((int )yych <= 84) {
    if ((int )yych <= 67) {
      if ((int )yych <= 32) {
        if ((int )yych == 9) {
          goto yy61;
        } else {

        }
        if ((int )yych <= 31) {
          goto yy4;
        } else {

        }
        goto yy61;
      } else
      if ((int )yych <= 53) {
        if ((int )yych <= 47) {
          goto yy4;
        } else {

        }
      } else {
        if ((int )yych <= 57) {
          goto yy456;
        } else {

        }
        if ((int )yych <= 58) {
          goto yy163;
        } else {

        }
        goto yy4;
      }
    } else
    if ((int )yych <= 71) {
      if ((int )yych == 69) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 70) {
        goto yy61;
      } else {

      }
      goto yy4;
    } else
    if ((int )yych <= 76) {
      if ((int )yych <= 72) {
        goto yy61;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych <= 77) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 82) {
        goto yy4;
      } else {

      }
      goto yy61;
    }
  } else
  if ((int )yych <= 103) {
    if ((int )yych <= 89) {
      if ((int )yych == 87) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 88) {
        goto yy4;
      } else {

      }
      goto yy61;
    } else
    if ((int )yych <= 100) {
      if ((int )yych <= 99) {
        goto yy4;
      } else {

      }
      goto yy61;
    } else {
      if ((int )yych == 102) {
        goto yy61;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych <= 108) {
      if ((int )yych <= 104) {
        goto yy61;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych <= 109) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 114) {
        goto yy4;
      } else {

      }
      goto yy61;
    }
  } else
  if ((int )yych <= 119) {
    if ((int )yych <= 118) {
      goto yy4;
    } else {

    }
    goto yy61;
  } else {
    if ((int )yych == 121) {
      goto yy61;
    } else {

    }
    goto yy4;
  }
  yyaccept = 0U;
  cursor ++;
  tmp___188 = cursor;
  s->ptr = tmp___188;
  yych = *tmp___188;
  if ((int )yych <= 86) {
    if ((int )yych <= 68) {
      if ((int )yych <= 32) {
        if ((int )yych == 9) {
          goto yy61;
        } else {

        }
        if ((int )yych <= 31) {
          goto yy4;
        } else {

        }
        goto yy61;
      } else {
        if ((int )yych <= 47) {
          goto yy4;
        } else {

        }
        if ((int )yych <= 57) {
          goto yy459;
        } else {

        }
        if ((int )yych <= 67) {
          goto yy4;
        } else {

        }
        goto yy61;
      }
    } else
    if ((int )yych <= 72) {
      if ((int )yych == 70) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 71) {
        goto yy4;
      } else {

      }
      goto yy61;
    } else
    if ((int )yych <= 77) {
      if ((int )yych <= 76) {
        goto yy4;
      } else {

      }
      goto yy61;
    } else {
      if ((int )yych <= 82) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 84) {
        goto yy61;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 99) {
      if ((int )yych == 88) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 89) {
        goto yy61;
      } else {

      }
      goto yy4;
    } else
    if ((int )yych <= 101) {
      if ((int )yych <= 100) {
        goto yy61;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 103) {
        goto yy4;
      } else {

      }
      goto yy61;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych == 109) {
      goto yy61;
    } else {

    }
    if ((int )yych <= 114) {
      goto yy4;
    } else {

    }
    goto yy61;
  } else
  if ((int )yych <= 119) {
    if ((int )yych <= 118) {
      goto yy4;
    } else {

    }
    goto yy61;
  } else {
    if ((int )yych == 121) {
      goto yy61;
    } else {

    }
    goto yy4;
  }
  yy459: 
  yyaccept = 0U;
  cursor ++;
  tmp___189 = cursor;
  s->ptr = tmp___189;
  yych = *tmp___189;
  if ((int const   )yybm[(int )yych] & 2) {
    goto yy55;
  } else {

  }
  if ((int )yych <= 86) {
    if ((int )yych <= 68) {
      if ((int )yych <= 32) {
        if ((int )yych == 9) {
          goto yy61;
        } else {

        }
        if ((int )yych <= 31) {
          goto yy4;
        } else {

        }
        goto yy61;
      } else {
        if ((int )yych == 45) {
          goto yy446;
        } else {

        }
        if ((int )yych <= 67) {
          goto yy4;
        } else {

        }
        goto yy61;
      }
    } else
    if ((int )yych <= 72) {
      if ((int )yych == 70) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 71) {
        goto yy4;
      } else {

      }
      goto yy61;
    } else
    if ((int )yych <= 77) {
      if ((int )yych <= 76) {
        goto yy4;
      } else {

      }
      goto yy61;
    } else {
      if ((int )yych <= 82) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 84) {
        goto yy61;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 99) {
      if ((int )yych == 88) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 89) {
        goto yy61;
      } else {

      }
      goto yy4;
    } else
    if ((int )yych <= 101) {
      if ((int )yych <= 100) {
        goto yy61;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 103) {
        goto yy4;
      } else {

      }
      goto yy61;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych == 109) {
      goto yy61;
    } else {

    }
    if ((int )yych <= 114) {
      goto yy4;
    } else {

    }
    goto yy61;
  } else
  if ((int )yych <= 119) {
    if ((int )yych <= 118) {
      goto yy4;
    } else {

    }
    goto yy61;
  } else {
    if ((int )yych == 121) {
      goto yy61;
    } else {

    }
    goto yy4;
  }
  yy460: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy462;
  } else {

  }
  if ((int )yych <= 48) {
    goto yy736;
  } else {

  }
  if ((int )yych <= 49) {
    goto yy737;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy738;
  } else {

  }
  goto yy462;
  yy461: 
  cursor ++;
  if (s->lim - cursor < 13) {
    return (257);
  } else {

  }
  yych = *cursor;
  yy462: ;
  switch ((int )yych) {
  case 32: 
  case 9: 
  goto yy461;
  case 46: 
  case 45: 
  goto yy577;
  case 97: 
  case 65: 
  goto yy480;
  case 100: 
  case 68: 
  goto yy466;
  case 102: 
  case 70: 
  goto yy467;
  case 104: 
  case 72: 
  goto yy64;
  case 73: 
  goto yy475;
  case 106: 
  case 74: 
  goto yy479;
  case 109: 
  case 77: 
  goto yy465;
  case 110: 
  case 78: 
  goto yy482;
  case 111: 
  case 79: 
  goto yy481;
  case 112: 
  case 80: 
  goto yy484;
  case 115: 
  case 83: 
  goto yy463;
  case 116: 
  case 84: 
  goto yy69;
  case 86: 
  goto yy477;
  case 119: 
  case 87: 
  goto yy68;
  case 88: 
  goto yy478;
  case 121: 
  case 89: 
  goto yy67;
  default: 
  goto yy57;
  }
  yy463: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych <= 68) {
      if ((int )yych == 65) {
        goto yy127;
      } else {

      }
      goto yy57;
    } else {
      if ((int )yych <= 69) {
        goto yy1049;
      } else {

      }
      if ((int )yych <= 84) {
        goto yy57;
      } else {

      }
      goto yy126;
    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych == 97) {
      goto yy127;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 101) {
      goto yy1049;
    } else {

    }
    if ((int )yych == 117) {
      goto yy126;
    } else {

    }
    goto yy57;
  }
  yy464: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 96) {
    if ((int )yych <= 68) {
      if ((int )yych == 65) {
        goto yy127;
      } else {

      }
      goto yy57;
    } else {
      if ((int )yych <= 69) {
        goto yy1049;
      } else {

      }
      if ((int )yych == 85) {
        goto yy126;
      } else {

      }
      goto yy57;
    }
  } else
  if ((int )yych <= 101) {
    if ((int )yych <= 97) {
      goto yy127;
    } else {

    }
    if ((int )yych <= 100) {
      goto yy57;
    } else {

    }
    goto yy1049;
  } else {
    if ((int )yych <= 115) {
      goto yy57;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy729;
    } else {

    }
    if ((int )yych <= 117) {
      goto yy126;
    } else {

    }
    goto yy57;
  }
  yy465: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 79) {
    if ((int )yych <= 72) {
      if ((int )yych == 65) {
        goto yy592;
      } else {

      }
      goto yy57;
    } else {
      if ((int )yych <= 73) {
        goto yy118;
      } else {

      }
      if ((int )yych <= 78) {
        goto yy57;
      } else {

      }
      goto yy117;
    }
  } else
  if ((int )yych <= 104) {
    if ((int )yych == 97) {
      goto yy592;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 105) {
      goto yy118;
    } else {

    }
    if ((int )yych == 111) {
      goto yy117;
    } else {

    }
    goto yy57;
  }
  yy466: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 69) {
    if ((int )yych == 65) {
      goto yy114;
    } else {

    }
    if ((int )yych <= 68) {
      goto yy57;
    } else {

    }
    goto yy579;
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 96) {
      goto yy57;
    } else {

    }
    goto yy114;
  } else {
    if ((int )yych == 101) {
      goto yy579;
    } else {

    }
    goto yy57;
  }
  yy467: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 82) {
    if ((int )yych <= 78) {
      if ((int )yych == 69) {
        goto yy595;
      } else {

      }
      goto yy57;
    } else {
      if ((int )yych <= 79) {
        goto yy99;
      } else {

      }
      if ((int )yych <= 81) {
        goto yy57;
      } else {

      }
      goto yy98;
    }
  } else
  if ((int )yych <= 110) {
    if ((int )yych == 101) {
      goto yy595;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 111) {
      goto yy99;
    } else {

    }
    if ((int )yych == 114) {
      goto yy98;
    } else {

    }
    goto yy57;
  }
  yy468: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych == 72) {
      goto yy70;
    } else {

    }
    if ((int )yych <= 84) {
      goto yy57;
    } else {

    }
    goto yy71;
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 103) {
      goto yy57;
    } else {

    }
    goto yy1048;
  } else {
    if ((int )yych == 117) {
      goto yy71;
    } else {

    }
    goto yy57;
  }
  yy469: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 45) {
    goto yy742;
  } else {

  }
  if ((int )yych <= 47) {
    goto yy61;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy741;
  } else {

  }
  goto yy61;
  yy470: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 99) {
    if ((int )yych == 79) {
      goto yy530;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 100) {
      goto yy729;
    } else {

    }
    if ((int )yych == 111) {
      goto yy530;
    } else {

    }
    goto yy57;
  }
  yy471: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 100) {
    goto yy729;
  } else {

  }
  goto yy57;
  yy472: 
  cursor ++;
  yych = *cursor;
  switch ((int )yych) {
  case 50: 
  case 49: 
  case 48: 
  goto yy666;
  case 51: 
  goto yy668;
  case 57: 
  case 56: 
  case 55: 
  case 54: 
  case 53: 
  case 52: 
  goto yy669;
  case 97: 
  case 65: 
  goto yy673;
  case 100: 
  case 68: 
  goto yy677;
  case 102: 
  case 70: 
  goto yy671;
  case 106: 
  case 74: 
  goto yy670;
  case 109: 
  case 77: 
  goto yy672;
  case 110: 
  case 78: 
  goto yy676;
  case 111: 
  case 79: 
  goto yy675;
  case 115: 
  case 83: 
  goto yy674;
  default: 
  goto yy57;
  }
  yy473: 
  cursor ++;
  yych = *cursor;
  switch ((int )yych) {
  case 48: 
  goto yy616;
  case 49: 
  goto yy617;
  case 57: 
  case 56: 
  case 55: 
  case 54: 
  case 53: 
  case 52: 
  case 51: 
  case 50: 
  goto yy618;
  case 97: 
  case 65: 
  goto yy622;
  case 100: 
  case 68: 
  goto yy626;
  case 102: 
  case 70: 
  goto yy620;
  case 106: 
  case 74: 
  goto yy619;
  case 109: 
  case 77: 
  goto yy621;
  case 110: 
  case 78: 
  goto yy625;
  case 111: 
  case 79: 
  goto yy624;
  case 115: 
  case 83: 
  goto yy623;
  default: 
  goto yy578;
  }
  yy474: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 49) {
    if ((int )yych <= 47) {
      goto yy578;
    } else {

    }
    if ((int )yych <= 48) {
      goto yy568;
    } else {

    }
    goto yy569;
  } else {
    if ((int )yych <= 53) {
      goto yy570;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy571;
    } else {

    }
    goto yy578;
  }
  yy475: 
  yyaccept = 10U;
  cursor ++;
  tmp___190 = cursor;
  s->ptr = tmp___190;
  yych = *tmp___190;
  if ((int )yych <= 47) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy532;
      } else {

      }
    } else {
      if ((int )yych <= 32) {
        goto yy532;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy476;
      } else {

      }
      if ((int )yych <= 46) {
        goto yy532;
      } else {

      }
    }
  } else
  if ((int )yych <= 85) {
    if ((int )yych <= 57) {
      goto yy534;
    } else {

    }
    if ((int )yych == 73) {
      goto yy567;
    } else {

    }
  } else {
    if ((int )yych == 87) {
      goto yy476;
    } else {

    }
    if ((int )yych <= 88) {
      goto yy540;
    } else {

    }
  }
  yy476: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  if ((s->time)->have_date) {
    add_error(s, (char *)"Double date specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_date = 1U;
  }
  (s->time)->d = timelib_get_nr(& ptr, 2);
  timelib_skip_day_suffix(& ptr);
  tmp___191 = timelib_get_month(& ptr);
  (s->time)->m = (timelib_sll )tmp___191;
  free((void *)str);
  return (269);
  yy477: 
  yyaccept = 10U;
  cursor ++;
  tmp___192 = cursor;
  s->ptr = tmp___192;
  yych = *tmp___192;
  if ((int )yych <= 44) {
    if ((int )yych <= 9) {
      if ((int )yych <= 8) {
        goto yy476;
      } else {

      }
      goto yy532;
    } else {
      if ((int )yych == 32) {
        goto yy532;
      } else {

      }
      goto yy476;
    }
  } else
  if ((int )yych <= 57) {
    if ((int )yych <= 46) {
      goto yy532;
    } else {

    }
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    goto yy534;
  } else {
    if ((int )yych == 73) {
      goto yy565;
    } else {

    }
    goto yy476;
  }
  yy478: 
  yyaccept = 10U;
  cursor ++;
  tmp___193 = cursor;
  s->ptr = tmp___193;
  yych = *tmp___193;
  if ((int )yych <= 44) {
    if ((int )yych <= 9) {
      if ((int )yych <= 8) {
        goto yy476;
      } else {

      }
      goto yy532;
    } else {
      if ((int )yych == 32) {
        goto yy532;
      } else {

      }
      goto yy476;
    }
  } else
  if ((int )yych <= 57) {
    if ((int )yych <= 46) {
      goto yy532;
    } else {

    }
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    goto yy534;
  } else {
    if ((int )yych == 73) {
      goto yy564;
    } else {

    }
    goto yy476;
  }
  yy479: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych == 65) {
      goto yy557;
    } else {

    }
    if ((int )yych <= 84) {
      goto yy57;
    } else {

    }
    goto yy556;
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 96) {
      goto yy57;
    } else {

    }
    goto yy557;
  } else {
    if ((int )yych == 117) {
      goto yy556;
    } else {

    }
    goto yy57;
  }
  yy480: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 84) {
    if ((int )yych <= 76) {
      if ((int )yych == 46) {
        goto yy485;
      } else {

      }
      goto yy57;
    } else {
      if ((int )yych <= 77) {
        goto yy486;
      } else {

      }
      if ((int )yych == 80) {
        goto yy550;
      } else {

      }
      goto yy57;
    }
  } else
  if ((int )yych <= 111) {
    if ((int )yych <= 85) {
      goto yy549;
    } else {

    }
    if ((int )yych == 109) {
      goto yy486;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 112) {
      goto yy550;
    } else {

    }
    if ((int )yych == 117) {
      goto yy549;
    } else {

    }
    goto yy57;
  }
  yy481: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 67) {
    goto yy544;
  } else {

  }
  if ((int )yych == 99) {
    goto yy544;
  } else {

  }
  goto yy57;
  yy482: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 79) {
    goto yy530;
  } else {

  }
  if ((int )yych == 111) {
    goto yy530;
  } else {

  }
  goto yy57;
  yy483: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy490;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy492;
  } else {

  }
  goto yy57;
  yy484: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 76) {
    if ((int )yych != 46) {
      goto yy57;
    } else {

    }
  } else {
    if ((int )yych <= 77) {
      goto yy486;
    } else {

    }
    if ((int )yych == 109) {
      goto yy486;
    } else {

    }
    goto yy57;
  }
  yy485: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 77) {
    goto yy486;
  } else {

  }
  if ((int )yych != 109) {
    goto yy57;
  } else {

  }
  yy486: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 31) {
    if ((int )yych <= 0) {
      goto yy488;
    } else {

    }
    if ((int )yych == 9) {
      goto yy488;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 32) {
      goto yy488;
    } else {

    }
    if ((int )yych != 46) {
      goto yy57;
    } else {

    }
  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 9) {
    if ((int )yych <= 0) {
      goto yy488;
    } else {

    }
    if ((int )yych <= 8) {
      goto yy57;
    } else {

    }
  } else
  if ((int )yych != 32) {
    goto yy57;
  } else {

  }
  yy488: 
  cursor ++;
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  if ((s->time)->have_time) {
    add_error(s, (char *)"Double time specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_time = 1U;
    (s->time)->h = (timelib_sll )0;
    (s->time)->i = (timelib_sll )0;
    (s->time)->s = (timelib_sll )0;
    (s->time)->f = (double )0;
  }
  (s->time)->h = timelib_get_nr(& ptr, 2);
  if ((int )*ptr == 58) {
    goto _L___0;
  } else
  if ((int )*ptr == 46) {
    _L___0: /* CIL Label */ 
    (s->time)->i = timelib_get_nr(& ptr, 2);
    if ((int )*ptr == 58) {
      (s->time)->s = timelib_get_nr(& ptr, 2);
    } else
    if ((int )*ptr == 46) {
      (s->time)->s = timelib_get_nr(& ptr, 2);
    } else {

    }
  } else {

  }
  tmp___194 = timelib_meridian(& ptr, (s->time)->h);
  (s->time)->h += tmp___194;
  free((void *)str);
  return (261);
  yy490: 
  yyaccept = 11U;
  cursor ++;
  tmp___195 = cursor;
  s->ptr = tmp___195;
  yych = *tmp___195;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy493;
    } else {

    }
  } else {
    if ((int )yych <= 57) {
      goto yy507;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy493;
    } else {

    }
  }
  yy491: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  if ((s->time)->have_time) {
    add_error(s, (char *)"Double time specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_time = 1U;
    (s->time)->h = (timelib_sll )0;
    (s->time)->i = (timelib_sll )0;
    (s->time)->s = (timelib_sll )0;
    (s->time)->f = (double )0;
  }
  (s->time)->h = timelib_get_nr(& ptr, 2);
  (s->time)->i = timelib_get_nr(& ptr, 2);
  if ((int )*ptr == 58) {
    goto _L___1;
  } else
  if ((int )*ptr == 46) {
    _L___1: /* CIL Label */ 
    (s->time)->s = timelib_get_nr(& ptr, 2);
    if ((int )*ptr == 46) {
      (s->time)->f = timelib_get_frac_nr(& ptr, 8);
    } else {

    }
  } else {

  }
  if ((int )*ptr != 0) {
    tmp___196 = timelib_get_zone(& ptr, & (s->time)->dst, s->time,
                                 & tz_not_found___1, s->tzdb);
    (s->time)->z = (int )tmp___196;
    if (tz_not_found___1) {
      add_error(s, (char *)"The timezone could not be found in the database");
    } else {

    }
  } else {

  }
  free((void *)str);
  return (278);
  yy492: 
  yyaccept = 11U;
  cursor ++;
  tmp___197 = cursor;
  s->ptr = tmp___197;
  yych = *tmp___197;
  if ((int )yych == 46) {
    goto yy493;
  } else {

  }
  if ((int )yych != 58) {
    goto yy491;
  } else {

  }
  yy493: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy494;
  } else {

  }
  if ((int )yych <= 54) {
    goto yy495;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy496;
  } else {

  }
  goto yy57;
  yy494: 
  yyaccept = 11U;
  cursor ++;
  tmp___198 = cursor;
  s->ptr = tmp___198;
  yych = *tmp___198;
  if ((int )yych == 46) {
    goto yy497;
  } else {

  }
  if ((int )yych <= 47) {
    goto yy491;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy500;
  } else {

  }
  goto yy491;
  yy495: 
  yyaccept = 11U;
  cursor ++;
  tmp___199 = cursor;
  s->ptr = tmp___199;
  yych = *tmp___199;
  if ((int )yych == 46) {
    goto yy497;
  } else {

  }
  if ((int )yych == 48) {
    goto yy500;
  } else {

  }
  goto yy491;
  yy496: 
  yyaccept = 11U;
  cursor ++;
  tmp___200 = cursor;
  s->ptr = tmp___200;
  yych = *tmp___200;
  if ((int )yych != 46) {
    goto yy491;
  } else {

  }
  yy497: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy57;
  } else {

  }
  yy498: 
  cursor ++;
  if ((unsigned int )s->lim <= (unsigned int )cursor) {
    return (257);
  } else {

  }
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy491;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy498;
  } else {

  }
  goto yy491;
  yy500: 
  yyaccept = 11U;
  cursor ++;
  tmp___201 = cursor;
  s->ptr = tmp___201;
  yych = *tmp___201;
  if ((int )yych <= 64) {
    if ((int )yych <= 31) {
      if ((int )yych != 9) {
        goto yy491;
      } else {

      }
    } else {
      if ((int )yych <= 32) {
        goto yy501;
      } else {

      }
      if ((int )yych == 46) {
        goto yy497;
      } else {

      }
      goto yy491;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 65) {
      goto yy503;
    } else {

    }
    if ((int )yych == 80) {
      goto yy503;
    } else {

    }
    goto yy491;
  } else {
    if ((int )yych <= 97) {
      goto yy503;
    } else {

    }
    if ((int )yych == 112) {
      goto yy503;
    } else {

    }
    goto yy491;
  }
  yy501: 
  cursor ++;
  if (s->lim - cursor < 5) {
    return (257);
  } else {

  }
  yych = *cursor;
  if ((int )yych <= 65) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy501;
      } else {

      }
      goto yy57;
    } else {
      if ((int )yych <= 32) {
        goto yy501;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy57;
      } else {

      }
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych != 80) {
      goto yy57;
    } else {

    }
  } else {
    if ((int )yych <= 97) {
      goto yy503;
    } else {

    }
    if ((int )yych != 112) {
      goto yy57;
    } else {

    }
  }
  yy503: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 76) {
    if ((int )yych != 46) {
      goto yy57;
    } else {

    }
  } else {
    if ((int )yych <= 77) {
      goto yy505;
    } else {

    }
    if ((int )yych == 109) {
      goto yy505;
    } else {

    }
    goto yy57;
  }
  cursor ++;
  yych = *cursor;
  if ((int )yych == 77) {
    goto yy505;
  } else {

  }
  if ((int )yych != 109) {
    goto yy57;
  } else {

  }
  yy505: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 31) {
    if ((int )yych <= 0) {
      goto yy488;
    } else {

    }
    if ((int )yych == 9) {
      goto yy488;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 32) {
      goto yy488;
    } else {

    }
    if ((int )yych != 46) {
      goto yy57;
    } else {

    }
  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 9) {
    if ((int )yych <= 0) {
      goto yy488;
    } else {

    }
    if ((int )yych <= 8) {
      goto yy57;
    } else {

    }
    goto yy488;
  } else {
    if ((int )yych == 32) {
      goto yy488;
    } else {

    }
    goto yy57;
  }
  yy507: 
  yyaccept = 11U;
  cursor ++;
  tmp___202 = cursor;
  s->ptr = tmp___202;
  yych = *tmp___202;
  if ((int )yych <= 58) {
    if ((int )yych <= 32) {
      if ((int )yych == 9) {
        goto yy508;
      } else {

      }
      if ((int )yych <= 31) {
        goto yy491;
      } else {

      }
    } else {
      if ((int )yych == 46) {
        goto yy493;
      } else {

      }
      if ((int )yych <= 57) {
        goto yy491;
      } else {

      }
      goto yy511;
    }
  } else
  if ((int )yych <= 80) {
    if ((int )yych == 65) {
      goto yy510;
    } else {

    }
    if ((int )yych <= 79) {
      goto yy491;
    } else {

    }
    goto yy510;
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 96) {
      goto yy491;
    } else {

    }
    goto yy510;
  } else {
    if ((int )yych == 112) {
      goto yy510;
    } else {

    }
    goto yy491;
  }
  yy508: 
  cursor ++;
  if (s->lim - cursor < 5) {
    return (257);
  } else {

  }
  yych = *cursor;
  if ((int )yych <= 65) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy508;
      } else {

      }
      goto yy57;
    } else {
      if ((int )yych <= 32) {
        goto yy508;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy57;
      } else {

      }
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych != 80) {
      goto yy57;
    } else {

    }
  } else {
    if ((int )yych <= 97) {
      goto yy510;
    } else {

    }
    if ((int )yych != 112) {
      goto yy57;
    } else {

    }
  }
  yy510: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 76) {
    if ((int )yych == 46) {
      goto yy527;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 77) {
      goto yy528;
    } else {

    }
    if ((int )yych == 109) {
      goto yy528;
    } else {

    }
    goto yy57;
  }
  yy511: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy512;
  } else {

  }
  if ((int )yych <= 54) {
    goto yy513;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy496;
  } else {

  }
  goto yy57;
  yy512: 
  yyaccept = 11U;
  cursor ++;
  tmp___203 = cursor;
  s->ptr = tmp___203;
  yych = *tmp___203;
  if ((int )yych == 46) {
    goto yy497;
  } else {

  }
  if ((int )yych <= 47) {
    goto yy491;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy514;
  } else {

  }
  goto yy491;
  yy513: 
  yyaccept = 11U;
  cursor ++;
  tmp___204 = cursor;
  s->ptr = tmp___204;
  yych = *tmp___204;
  if ((int )yych == 46) {
    goto yy497;
  } else {

  }
  if ((int )yych != 48) {
    goto yy491;
  } else {

  }
  yy514: 
  yyaccept = 11U;
  cursor ++;
  tmp___205 = cursor;
  s->ptr = tmp___205;
  yych = *tmp___205;
  if ((int )yych <= 58) {
    if ((int )yych <= 32) {
      if ((int )yych == 9) {
        goto yy501;
      } else {

      }
      if ((int )yych <= 31) {
        goto yy491;
      } else {

      }
      goto yy501;
    } else {
      if ((int )yych == 46) {
        goto yy515;
      } else {

      }
      if ((int )yych <= 57) {
        goto yy491;
      } else {

      }
      goto yy516;
    }
  } else
  if ((int )yych <= 80) {
    if ((int )yych == 65) {
      goto yy503;
    } else {

    }
    if ((int )yych <= 79) {
      goto yy491;
    } else {

    }
    goto yy503;
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 96) {
      goto yy491;
    } else {

    }
    goto yy503;
  } else {
    if ((int )yych == 112) {
      goto yy503;
    } else {

    }
    goto yy491;
  }
  yy515: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy525;
  } else {

  }
  goto yy57;
  yy516: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy57;
  } else {

  }
  yy517: 
  cursor ++;
  if (s->lim - cursor < 5) {
    return (257);
  } else {

  }
  yych = *cursor;
  if ((int )yych <= 79) {
    if ((int )yych <= 57) {
      if ((int )yych <= 47) {
        goto yy57;
      } else {

      }
      goto yy517;
    } else
    if ((int )yych != 65) {
      goto yy57;
    } else {

    }
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 80) {
      goto yy519;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy57;
    } else {

    }
  } else
  if ((int )yych != 112) {
    goto yy57;
  } else {

  }
  yy519: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 76) {
    if ((int )yych != 46) {
      goto yy57;
    } else {

    }
  } else {
    if ((int )yych <= 77) {
      goto yy521;
    } else {

    }
    if ((int )yych == 109) {
      goto yy521;
    } else {

    }
    goto yy57;
  }
  cursor ++;
  yych = *cursor;
  if ((int )yych == 77) {
    goto yy521;
  } else {

  }
  if ((int )yych != 109) {
    goto yy57;
  } else {

  }
  yy521: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 31) {
    if ((int )yych <= 0) {
      goto yy523;
    } else {

    }
    if ((int )yych == 9) {
      goto yy523;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 32) {
      goto yy523;
    } else {

    }
    if ((int )yych != 46) {
      goto yy57;
    } else {

    }
  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 9) {
    if ((int )yych <= 0) {
      goto yy523;
    } else {

    }
    if ((int )yych <= 8) {
      goto yy57;
    } else {

    }
  } else
  if ((int )yych != 32) {
    goto yy57;
  } else {

  }
  yy523: 
  cursor ++;
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  if ((s->time)->have_time) {
    add_error(s, (char *)"Double time specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_time = 1U;
    (s->time)->h = (timelib_sll )0;
    (s->time)->i = (timelib_sll )0;
    (s->time)->s = (timelib_sll )0;
    (s->time)->f = (double )0;
  }
  (s->time)->h = timelib_get_nr(& ptr, 2);
  (s->time)->i = timelib_get_nr(& ptr, 2);
  if ((int )*ptr == 58) {
    goto _L___2;
  } else
  if ((int )*ptr == 46) {
    _L___2: /* CIL Label */ 
    (s->time)->s = timelib_get_nr(& ptr, 2);
    if ((int )*ptr == 58) {
      (s->time)->f = timelib_get_frac_nr(& ptr, 8);
    } else
    if ((int )*ptr == 46) {
      (s->time)->f = timelib_get_frac_nr(& ptr, 8);
    } else {

    }
  } else {

  }
  timelib_eat_spaces(& ptr);
  tmp___206 = timelib_meridian(& ptr, (s->time)->h);
  (s->time)->h += tmp___206;
  free((void *)str);
  return (278);
  yy525: 
  yyaccept = 11U;
  cursor ++;
  s->ptr = cursor;
  if (s->lim - cursor < 5) {
    return (257);
  } else {

  }
  yych = *cursor;
  if ((int )yych <= 79) {
    if ((int )yych <= 57) {
      if ((int )yych <= 47) {
        goto yy491;
      } else {

      }
      goto yy525;
    } else {
      if ((int )yych == 65) {
        goto yy519;
      } else {

      }
      goto yy491;
    }
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 80) {
      goto yy519;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy491;
    } else {

    }
    goto yy519;
  } else {
    if ((int )yych == 112) {
      goto yy519;
    } else {

    }
    goto yy491;
  }
  yy527: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 77) {
    goto yy528;
  } else {

  }
  if ((int )yych != 109) {
    goto yy57;
  } else {

  }
  yy528: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 31) {
    if ((int )yych <= 0) {
      goto yy488;
    } else {

    }
    if ((int )yych == 9) {
      goto yy488;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 32) {
      goto yy488;
    } else {

    }
    if ((int )yych != 46) {
      goto yy57;
    } else {

    }
  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 9) {
    if ((int )yych <= 0) {
      goto yy488;
    } else {

    }
    if ((int )yych <= 8) {
      goto yy57;
    } else {

    }
    goto yy488;
  } else {
    if ((int )yych == 32) {
      goto yy488;
    } else {

    }
    goto yy57;
  }
  yy530: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 86) {
    goto yy531;
  } else {

  }
  if ((int )yych != 118) {
    goto yy57;
  } else {

  }
  yy531: 
  yyaccept = 10U;
  cursor ++;
  tmp___207 = cursor;
  s->ptr = tmp___207;
  yych = *tmp___207;
  if ((int )yych <= 46) {
    if ((int )yych <= 31) {
      if ((int )yych != 9) {
        goto yy476;
      } else {

      }
    } else {
      if ((int )yych <= 32) {
        goto yy532;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy476;
      } else {

      }
    }
  } else
  if ((int )yych <= 68) {
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy534;
    } else {

    }
    goto yy476;
  } else {
    if ((int )yych <= 69) {
      goto yy536;
    } else {

    }
    if ((int )yych == 101) {
      goto yy536;
    } else {

    }
    goto yy476;
  }
  yy532: 
  cursor ++;
  if (s->lim - cursor < 4) {
    return (257);
  } else {

  }
  yych = *cursor;
  yy533: ;
  if ((int )yych <= 32) {
    if ((int )yych == 9) {
      goto yy532;
    } else {

    }
    if ((int )yych <= 31) {
      goto yy57;
    } else {

    }
    goto yy532;
  } else
  if ((int )yych <= 46) {
    if ((int )yych <= 44) {
      goto yy57;
    } else {

    }
    goto yy532;
  } else {
    if ((int )yych <= 47) {
      goto yy57;
    } else {

    }
    if ((int )yych >= 58) {
      goto yy57;
    } else {

    }
  }
  yy534: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy535;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy541;
  } else {

  }
  yy535: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  if ((s->time)->have_date) {
    add_error(s, (char *)"Double date specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_date = 1U;
  }
  (s->time)->d = timelib_get_nr(& ptr, 2);
  timelib_skip_day_suffix(& ptr);
  tmp___208 = timelib_get_month(& ptr);
  (s->time)->m = (timelib_sll )tmp___208;
  (s->time)->y = timelib_get_nr(& ptr, 4);
  if ((s->time)->y == -99999LL) {

  } else
  if ((s->time)->y < 100LL) {
    if ((s->time)->y < 70LL) {
      (s->time)->y += 2000LL;
    } else {
      (s->time)->y += 1900LL;
    }
  } else {

  }
  free((void *)str);
  return (268);
  yy536: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 77) {
    goto yy537;
  } else {

  }
  if ((int )yych != 109) {
    goto yy57;
  } else {

  }
  yy537: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 66) {
    goto yy538;
  } else {

  }
  if ((int )yych != 98) {
    goto yy57;
  } else {

  }
  yy538: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy539;
  } else {

  }
  if ((int )yych != 101) {
    goto yy57;
  } else {

  }
  yy539: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy540;
  } else {

  }
  if ((int )yych != 114) {
    goto yy57;
  } else {

  }
  yy540: 
  yyaccept = 10U;
  cursor ++;
  tmp___209 = cursor;
  s->ptr = tmp___209;
  yych = *tmp___209;
  if ((int )yych <= 32) {
    if ((int )yych == 9) {
      goto yy532;
    } else {

    }
    if ((int )yych <= 31) {
      goto yy476;
    } else {

    }
    goto yy532;
  } else
  if ((int )yych <= 46) {
    if ((int )yych <= 44) {
      goto yy476;
    } else {

    }
    goto yy532;
  } else {
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy534;
    } else {

    }
    goto yy476;
  }
  yy541: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy535;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy535;
  } else {

  }
  yy542: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy535;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy535;
  } else {

  }
  cursor ++;
  yych = *cursor;
  goto yy535;
  yy544: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 84) {
    goto yy545;
  } else {

  }
  if ((int )yych != 116) {
    goto yy57;
  } else {

  }
  yy545: 
  yyaccept = 10U;
  cursor ++;
  tmp___210 = cursor;
  s->ptr = tmp___210;
  yych = *tmp___210;
  if ((int )yych <= 46) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy532;
      } else {

      }
      goto yy476;
    } else {
      if ((int )yych <= 32) {
        goto yy532;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy476;
      } else {

      }
      goto yy532;
    }
  } else
  if ((int )yych <= 78) {
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy534;
    } else {

    }
    goto yy476;
  } else {
    if ((int )yych <= 79) {
      goto yy546;
    } else {

    }
    if ((int )yych != 111) {
      goto yy476;
    } else {

    }
  }
  yy546: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 66) {
    goto yy547;
  } else {

  }
  if ((int )yych != 98) {
    goto yy57;
  } else {

  }
  yy547: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy548;
  } else {

  }
  if ((int )yych != 101) {
    goto yy57;
  } else {

  }
  yy548: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy540;
  } else {

  }
  if ((int )yych == 114) {
    goto yy540;
  } else {

  }
  goto yy57;
  yy549: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 71) {
    goto yy553;
  } else {

  }
  if ((int )yych == 103) {
    goto yy553;
  } else {

  }
  goto yy57;
  yy550: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy551;
  } else {

  }
  if ((int )yych != 114) {
    goto yy57;
  } else {

  }
  yy551: 
  yyaccept = 10U;
  cursor ++;
  tmp___211 = cursor;
  s->ptr = tmp___211;
  yych = *tmp___211;
  if ((int )yych <= 46) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy532;
      } else {

      }
      goto yy476;
    } else {
      if ((int )yych <= 32) {
        goto yy532;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy476;
      } else {

      }
      goto yy532;
    }
  } else
  if ((int )yych <= 72) {
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy534;
    } else {

    }
    goto yy476;
  } else {
    if ((int )yych <= 73) {
      goto yy552;
    } else {

    }
    if ((int )yych != 105) {
      goto yy476;
    } else {

    }
  }
  yy552: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 76) {
    goto yy540;
  } else {

  }
  if ((int )yych == 108) {
    goto yy540;
  } else {

  }
  goto yy57;
  yy553: 
  yyaccept = 10U;
  cursor ++;
  tmp___212 = cursor;
  s->ptr = tmp___212;
  yych = *tmp___212;
  if ((int )yych <= 46) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy532;
      } else {

      }
      goto yy476;
    } else {
      if ((int )yych <= 32) {
        goto yy532;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy476;
      } else {

      }
      goto yy532;
    }
  } else
  if ((int )yych <= 84) {
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy534;
    } else {

    }
    goto yy476;
  } else {
    if ((int )yych <= 85) {
      goto yy554;
    } else {

    }
    if ((int )yych != 117) {
      goto yy476;
    } else {

    }
  }
  yy554: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 83) {
    goto yy555;
  } else {

  }
  if ((int )yych != 115) {
    goto yy57;
  } else {

  }
  yy555: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 84) {
    goto yy540;
  } else {

  }
  if ((int )yych == 116) {
    goto yy540;
  } else {

  }
  goto yy57;
  yy556: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 78) {
    if ((int )yych == 76) {
      goto yy563;
    } else {

    }
    if ((int )yych <= 77) {
      goto yy57;
    } else {

    }
    goto yy562;
  } else
  if ((int )yych <= 108) {
    if ((int )yych <= 107) {
      goto yy57;
    } else {

    }
    goto yy563;
  } else {
    if ((int )yych == 110) {
      goto yy562;
    } else {

    }
    goto yy57;
  }
  yy557: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 78) {
    goto yy558;
  } else {

  }
  if ((int )yych != 110) {
    goto yy57;
  } else {

  }
  yy558: 
  yyaccept = 10U;
  cursor ++;
  tmp___213 = cursor;
  s->ptr = tmp___213;
  yych = *tmp___213;
  if ((int )yych <= 46) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy532;
      } else {

      }
      goto yy476;
    } else {
      if ((int )yych <= 32) {
        goto yy532;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy476;
      } else {

      }
      goto yy532;
    }
  } else
  if ((int )yych <= 84) {
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy534;
    } else {

    }
    goto yy476;
  } else {
    if ((int )yych <= 85) {
      goto yy559;
    } else {

    }
    if ((int )yych != 117) {
      goto yy476;
    } else {

    }
  }
  yy559: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy560;
  } else {

  }
  if ((int )yych != 97) {
    goto yy57;
  } else {

  }
  yy560: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy561;
  } else {

  }
  if ((int )yych != 114) {
    goto yy57;
  } else {

  }
  yy561: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy540;
  } else {

  }
  if ((int )yych == 121) {
    goto yy540;
  } else {

  }
  goto yy57;
  yy562: 
  yyaccept = 10U;
  cursor ++;
  tmp___214 = cursor;
  s->ptr = tmp___214;
  yych = *tmp___214;
  if ((int )yych <= 46) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy532;
      } else {

      }
      goto yy476;
    } else {
      if ((int )yych <= 32) {
        goto yy532;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy476;
      } else {

      }
      goto yy532;
    }
  } else
  if ((int )yych <= 68) {
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy534;
    } else {

    }
    goto yy476;
  } else {
    if ((int )yych <= 69) {
      goto yy540;
    } else {

    }
    if ((int )yych == 101) {
      goto yy540;
    } else {

    }
    goto yy476;
  }
  yy563: 
  yyaccept = 10U;
  cursor ++;
  tmp___215 = cursor;
  s->ptr = tmp___215;
  yych = *tmp___215;
  if ((int )yych <= 46) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy532;
      } else {

      }
      goto yy476;
    } else {
      if ((int )yych <= 32) {
        goto yy532;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy476;
      } else {

      }
      goto yy532;
    }
  } else
  if ((int )yych <= 88) {
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy534;
    } else {

    }
    goto yy476;
  } else {
    if ((int )yych <= 89) {
      goto yy540;
    } else {

    }
    if ((int )yych == 121) {
      goto yy540;
    } else {

    }
    goto yy476;
  }
  yy564: 
  yyaccept = 10U;
  cursor ++;
  tmp___216 = cursor;
  s->ptr = tmp___216;
  yych = *tmp___216;
  if ((int )yych <= 44) {
    if ((int )yych <= 9) {
      if ((int )yych <= 8) {
        goto yy476;
      } else {

      }
      goto yy532;
    } else {
      if ((int )yych == 32) {
        goto yy532;
      } else {

      }
      goto yy476;
    }
  } else
  if ((int )yych <= 57) {
    if ((int )yych <= 46) {
      goto yy532;
    } else {

    }
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    goto yy534;
  } else {
    if ((int )yych == 73) {
      goto yy540;
    } else {

    }
    goto yy476;
  }
  yy565: 
  yyaccept = 10U;
  cursor ++;
  tmp___217 = cursor;
  s->ptr = tmp___217;
  yych = *tmp___217;
  if ((int )yych <= 44) {
    if ((int )yych <= 9) {
      if ((int )yych <= 8) {
        goto yy476;
      } else {

      }
      goto yy532;
    } else {
      if ((int )yych == 32) {
        goto yy532;
      } else {

      }
      goto yy476;
    }
  } else
  if ((int )yych <= 57) {
    if ((int )yych <= 46) {
      goto yy532;
    } else {

    }
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    goto yy534;
  } else
  if ((int )yych != 73) {
    goto yy476;
  } else {

  }
  yyaccept = 10U;
  cursor ++;
  tmp___218 = cursor;
  s->ptr = tmp___218;
  yych = *tmp___218;
  if ((int )yych <= 44) {
    if ((int )yych <= 9) {
      if ((int )yych <= 8) {
        goto yy476;
      } else {

      }
      goto yy532;
    } else {
      if ((int )yych == 32) {
        goto yy532;
      } else {

      }
      goto yy476;
    }
  } else
  if ((int )yych <= 57) {
    if ((int )yych <= 46) {
      goto yy532;
    } else {

    }
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    goto yy534;
  } else {
    if ((int )yych == 73) {
      goto yy540;
    } else {

    }
    goto yy476;
  }
  yy567: 
  yyaccept = 10U;
  cursor ++;
  tmp___219 = cursor;
  s->ptr = tmp___219;
  yych = *tmp___219;
  if ((int )yych <= 44) {
    if ((int )yych <= 9) {
      if ((int )yych <= 8) {
        goto yy476;
      } else {

      }
      goto yy532;
    } else {
      if ((int )yych == 32) {
        goto yy532;
      } else {

      }
      goto yy476;
    }
  } else
  if ((int )yych <= 57) {
    if ((int )yych <= 46) {
      goto yy532;
    } else {

    }
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    goto yy534;
  } else {
    if ((int )yych == 73) {
      goto yy540;
    } else {

    }
    goto yy476;
  }
  yy568: 
  yyaccept = 11U;
  cursor ++;
  tmp___220 = cursor;
  s->ptr = tmp___220;
  yych = *tmp___220;
  if ((int )yych <= 46) {
    if ((int )yych <= 44) {
      goto yy491;
    } else {

    }
    if ((int )yych <= 45) {
      goto yy602;
    } else {

    }
    goto yy601;
  } else {
    if ((int )yych <= 47) {
      goto yy491;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy615;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy493;
    } else {

    }
    goto yy491;
  }
  yy569: 
  yyaccept = 11U;
  cursor ++;
  tmp___221 = cursor;
  s->ptr = tmp___221;
  yych = *tmp___221;
  if ((int )yych <= 47) {
    if ((int )yych <= 44) {
      goto yy491;
    } else {

    }
    if ((int )yych <= 45) {
      goto yy602;
    } else {

    }
    if ((int )yych <= 46) {
      goto yy601;
    } else {

    }
    goto yy491;
  } else {
    if ((int )yych <= 50) {
      goto yy615;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy614;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy493;
    } else {

    }
    goto yy491;
  }
  yy570: 
  yyaccept = 11U;
  cursor ++;
  tmp___222 = cursor;
  s->ptr = tmp___222;
  yych = *tmp___222;
  if ((int )yych <= 46) {
    if ((int )yych <= 44) {
      goto yy491;
    } else {

    }
    if ((int )yych <= 45) {
      goto yy602;
    } else {

    }
    goto yy601;
  } else {
    if ((int )yych <= 47) {
      goto yy491;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy614;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy493;
    } else {

    }
    goto yy491;
  }
  yy571: 
  yyaccept = 11U;
  cursor ++;
  tmp___223 = cursor;
  s->ptr = tmp___223;
  yych = *tmp___223;
  if ((int )yych <= 46) {
    if ((int )yych <= 44) {
      goto yy491;
    } else {

    }
    if ((int )yych <= 45) {
      goto yy602;
    } else {

    }
    goto yy601;
  } else {
    if ((int )yych == 58) {
      goto yy493;
    } else {

    }
    goto yy491;
  }
  yy572: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy595;
  } else {

  }
  if ((int )yych == 101) {
    goto yy595;
  } else {

  }
  goto yy57;
  yy573: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy592;
  } else {

  }
  if ((int )yych == 97) {
    goto yy592;
  } else {

  }
  goto yy57;
  yy574: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych == 80) {
      goto yy550;
    } else {

    }
    if ((int )yych <= 84) {
      goto yy57;
    } else {

    }
    goto yy549;
  } else
  if ((int )yych <= 112) {
    if ((int )yych <= 111) {
      goto yy57;
    } else {

    }
    goto yy550;
  } else {
    if ((int )yych == 117) {
      goto yy549;
    } else {

    }
    goto yy57;
  }
  yy575: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy585;
  } else {

  }
  if ((int )yych == 101) {
    goto yy585;
  } else {

  }
  goto yy57;
  yy576: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy579;
  } else {

  }
  if ((int )yych == 101) {
    goto yy579;
  } else {

  }
  goto yy57;
  yy577: 
  cursor ++;
  if (s->lim - cursor < 13) {
    return (257);
  } else {

  }
  yych = *cursor;
  yy578: ;
  switch ((int )yych) {
  case 46: 
  case 45: 
  case 32: 
  case 9: 
  goto yy577;
  case 97: 
  case 65: 
  goto yy574;
  case 100: 
  case 68: 
  goto yy576;
  case 102: 
  case 70: 
  goto yy572;
  case 73: 
  goto yy475;
  case 106: 
  case 74: 
  goto yy479;
  case 109: 
  case 77: 
  goto yy573;
  case 110: 
  case 78: 
  goto yy482;
  case 111: 
  case 79: 
  goto yy481;
  case 115: 
  case 83: 
  goto yy575;
  case 86: 
  goto yy477;
  case 88: 
  goto yy478;
  default: 
  goto yy57;
  }
  yy579: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 67) {
    goto yy580;
  } else {

  }
  if ((int )yych != 99) {
    goto yy57;
  } else {

  }
  yy580: 
  yyaccept = 10U;
  cursor ++;
  tmp___224 = cursor;
  s->ptr = tmp___224;
  yych = *tmp___224;
  if ((int )yych <= 46) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy532;
      } else {

      }
      goto yy476;
    } else {
      if ((int )yych <= 32) {
        goto yy532;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy476;
      } else {

      }
      goto yy532;
    }
  } else
  if ((int )yych <= 68) {
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy534;
    } else {

    }
    goto yy476;
  } else {
    if ((int )yych <= 69) {
      goto yy581;
    } else {

    }
    if ((int )yych != 101) {
      goto yy476;
    } else {

    }
  }
  yy581: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 77) {
    goto yy582;
  } else {

  }
  if ((int )yych != 109) {
    goto yy57;
  } else {

  }
  yy582: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 66) {
    goto yy583;
  } else {

  }
  if ((int )yych != 98) {
    goto yy57;
  } else {

  }
  yy583: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy584;
  } else {

  }
  if ((int )yych != 101) {
    goto yy57;
  } else {

  }
  yy584: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy540;
  } else {

  }
  if ((int )yych == 114) {
    goto yy540;
  } else {

  }
  goto yy57;
  yy585: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 80) {
    goto yy586;
  } else {

  }
  if ((int )yych != 112) {
    goto yy57;
  } else {

  }
  yy586: 
  yyaccept = 10U;
  cursor ++;
  tmp___225 = cursor;
  s->ptr = tmp___225;
  yych = *tmp___225;
  if ((int )yych <= 46) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy532;
      } else {

      }
      goto yy476;
    } else {
      if ((int )yych <= 32) {
        goto yy532;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy476;
      } else {

      }
      goto yy532;
    }
  } else
  if ((int )yych <= 83) {
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy534;
    } else {

    }
    goto yy476;
  } else {
    if ((int )yych <= 84) {
      goto yy587;
    } else {

    }
    if ((int )yych != 116) {
      goto yy476;
    } else {

    }
  }
  yy587: 
  yyaccept = 10U;
  cursor ++;
  tmp___226 = cursor;
  s->ptr = tmp___226;
  yych = *tmp___226;
  if ((int )yych <= 46) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy532;
      } else {

      }
      goto yy476;
    } else {
      if ((int )yych <= 32) {
        goto yy532;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy476;
      } else {

      }
      goto yy532;
    }
  } else
  if ((int )yych <= 68) {
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy534;
    } else {

    }
    goto yy476;
  } else {
    if ((int )yych <= 69) {
      goto yy588;
    } else {

    }
    if ((int )yych != 101) {
      goto yy476;
    } else {

    }
  }
  yy588: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 77) {
    goto yy589;
  } else {

  }
  if ((int )yych != 109) {
    goto yy57;
  } else {

  }
  yy589: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 66) {
    goto yy590;
  } else {

  }
  if ((int )yych != 98) {
    goto yy57;
  } else {

  }
  yy590: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy591;
  } else {

  }
  if ((int )yych != 101) {
    goto yy57;
  } else {

  }
  yy591: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy540;
  } else {

  }
  if ((int )yych == 114) {
    goto yy540;
  } else {

  }
  goto yy57;
  yy592: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 89) {
    if ((int )yych == 82) {
      goto yy593;
    } else {

    }
    if ((int )yych <= 88) {
      goto yy57;
    } else {

    }
    goto yy540;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy57;
    } else {

    }
  } else {
    if ((int )yych == 121) {
      goto yy540;
    } else {

    }
    goto yy57;
  }
  yy593: 
  yyaccept = 10U;
  cursor ++;
  tmp___227 = cursor;
  s->ptr = tmp___227;
  yych = *tmp___227;
  if ((int )yych <= 46) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy532;
      } else {

      }
      goto yy476;
    } else {
      if ((int )yych <= 32) {
        goto yy532;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy476;
      } else {

      }
      goto yy532;
    }
  } else
  if ((int )yych <= 66) {
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy534;
    } else {

    }
    goto yy476;
  } else {
    if ((int )yych <= 67) {
      goto yy594;
    } else {

    }
    if ((int )yych != 99) {
      goto yy476;
    } else {

    }
  }
  yy594: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 72) {
    goto yy540;
  } else {

  }
  if ((int )yych == 104) {
    goto yy540;
  } else {

  }
  goto yy57;
  yy595: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 66) {
    goto yy596;
  } else {

  }
  if ((int )yych != 98) {
    goto yy57;
  } else {

  }
  yy596: 
  yyaccept = 10U;
  cursor ++;
  tmp___228 = cursor;
  s->ptr = tmp___228;
  yych = *tmp___228;
  if ((int )yych <= 46) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy532;
      } else {

      }
      goto yy476;
    } else {
      if ((int )yych <= 32) {
        goto yy532;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy476;
      } else {

      }
      goto yy532;
    }
  } else
  if ((int )yych <= 81) {
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy534;
    } else {

    }
    goto yy476;
  } else {
    if ((int )yych <= 82) {
      goto yy597;
    } else {

    }
    if ((int )yych != 114) {
      goto yy476;
    } else {

    }
  }
  yy597: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 85) {
    goto yy598;
  } else {

  }
  if ((int )yych != 117) {
    goto yy57;
  } else {

  }
  yy598: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy599;
  } else {

  }
  if ((int )yych != 97) {
    goto yy57;
  } else {

  }
  yy599: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy600;
  } else {

  }
  if ((int )yych != 114) {
    goto yy57;
  } else {

  }
  yy600: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy540;
  } else {

  }
  if ((int )yych == 121) {
    goto yy540;
  } else {

  }
  goto yy57;
  yy601: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy608;
  } else {

  }
  if ((int )yych <= 54) {
    goto yy609;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy610;
  } else {

  }
  goto yy57;
  yy602: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy57;
  } else {

  }
  yy604: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy57;
  } else {

  }
  yy605: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy57;
  } else {

  }
  cursor ++;
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  if ((s->time)->have_date) {
    add_error(s, (char *)"Double date specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_date = 1U;
  }
  (s->time)->d = timelib_get_nr(& ptr, 2);
  (s->time)->m = timelib_get_nr(& ptr, 2);
  (s->time)->y = timelib_get_nr(& ptr, 4);
  free((void *)str);
  return (277);
  yy608: 
  yyaccept = 11U;
  cursor ++;
  tmp___229 = cursor;
  s->ptr = tmp___229;
  yych = *tmp___229;
  if ((int )yych == 46) {
    goto yy497;
  } else {

  }
  if ((int )yych <= 47) {
    goto yy491;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy613;
  } else {

  }
  goto yy491;
  yy609: 
  yyaccept = 11U;
  cursor ++;
  tmp___230 = cursor;
  s->ptr = tmp___230;
  yych = *tmp___230;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy497;
    } else {

    }
    goto yy491;
  } else {
    if ((int )yych <= 48) {
      goto yy613;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy611;
    } else {

    }
    goto yy491;
  }
  yy610: 
  yyaccept = 11U;
  cursor ++;
  tmp___231 = cursor;
  s->ptr = tmp___231;
  yych = *tmp___231;
  if ((int )yych == 46) {
    goto yy497;
  } else {

  }
  if ((int )yych <= 47) {
    goto yy491;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy491;
  } else {

  }
  yy611: 
  yyaccept = 12U;
  cursor ++;
  tmp___232 = cursor;
  s->ptr = tmp___232;
  yych = *tmp___232;
  if ((int )yych <= 47) {
    goto yy612;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy605;
  } else {

  }
  yy612: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  if ((s->time)->have_date) {
    add_error(s, (char *)"Double date specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_date = 1U;
  }
  (s->time)->d = timelib_get_nr(& ptr, 2);
  (s->time)->m = timelib_get_nr(& ptr, 2);
  (s->time)->y = timelib_get_nr(& ptr, 2);
  if ((s->time)->y == -99999LL) {

  } else
  if ((s->time)->y < 100LL) {
    if ((s->time)->y < 70LL) {
      (s->time)->y += 2000LL;
    } else {
      (s->time)->y += 1900LL;
    }
  } else {

  }
  free((void *)str);
  return (277);
  yy613: 
  yyaccept = 11U;
  cursor ++;
  tmp___233 = cursor;
  s->ptr = tmp___233;
  yych = *tmp___233;
  if ((int )yych <= 57) {
    if ((int )yych <= 32) {
      if ((int )yych == 9) {
        goto yy501;
      } else {

      }
      if ((int )yych <= 31) {
        goto yy491;
      } else {

      }
      goto yy501;
    } else {
      if ((int )yych == 46) {
        goto yy497;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy491;
      } else {

      }
      goto yy605;
    }
  } else
  if ((int )yych <= 80) {
    if ((int )yych == 65) {
      goto yy503;
    } else {

    }
    if ((int )yych <= 79) {
      goto yy491;
    } else {

    }
    goto yy503;
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 96) {
      goto yy491;
    } else {

    }
    goto yy503;
  } else {
    if ((int )yych == 112) {
      goto yy503;
    } else {

    }
    goto yy491;
  }
  yy614: 
  yyaccept = 11U;
  cursor ++;
  tmp___234 = cursor;
  s->ptr = tmp___234;
  yych = *tmp___234;
  if ((int )yych <= 58) {
    if ((int )yych <= 32) {
      if ((int )yych == 9) {
        goto yy508;
      } else {

      }
      if ((int )yych <= 31) {
        goto yy491;
      } else {

      }
      goto yy508;
    } else {
      if ((int )yych == 46) {
        goto yy493;
      } else {

      }
      if ((int )yych <= 57) {
        goto yy491;
      } else {

      }
      goto yy493;
    }
  } else
  if ((int )yych <= 80) {
    if ((int )yych == 65) {
      goto yy510;
    } else {

    }
    if ((int )yych <= 79) {
      goto yy491;
    } else {

    }
    goto yy510;
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 96) {
      goto yy491;
    } else {

    }
    goto yy510;
  } else {
    if ((int )yych == 112) {
      goto yy510;
    } else {

    }
    goto yy491;
  }
  yy615: 
  yyaccept = 11U;
  cursor ++;
  tmp___235 = cursor;
  s->ptr = tmp___235;
  yych = *tmp___235;
  if ((int )yych <= 58) {
    if ((int )yych <= 32) {
      if ((int )yych == 9) {
        goto yy508;
      } else {

      }
      if ((int )yych <= 31) {
        goto yy491;
      } else {

      }
      goto yy508;
    } else
    if ((int )yych <= 45) {
      if ((int )yych <= 44) {
        goto yy491;
      } else {

      }
      goto yy602;
    } else {
      if ((int )yych <= 46) {
        goto yy601;
      } else {

      }
      if ((int )yych <= 57) {
        goto yy491;
      } else {

      }
      goto yy493;
    }
  } else
  if ((int )yych <= 80) {
    if ((int )yych == 65) {
      goto yy510;
    } else {

    }
    if ((int )yych <= 79) {
      goto yy491;
    } else {

    }
    goto yy510;
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 96) {
      goto yy491;
    } else {

    }
    goto yy510;
  } else {
    if ((int )yych == 112) {
      goto yy510;
    } else {

    }
    goto yy491;
  }
  yy616: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 46) {
    if ((int )yych <= 44) {
      goto yy57;
    } else {

    }
    if ((int )yych <= 45) {
      goto yy655;
    } else {

    }
    goto yy602;
  } else {
    if ((int )yych <= 47) {
      goto yy57;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy618;
    } else {

    }
    goto yy57;
  }
  yy617: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 46) {
    if ((int )yych <= 44) {
      goto yy57;
    } else {

    }
    if ((int )yych <= 45) {
      goto yy655;
    } else {

    }
    goto yy602;
  } else {
    if ((int )yych <= 47) {
      goto yy57;
    } else {

    }
    if ((int )yych >= 51) {
      goto yy57;
    } else {

    }
  }
  yy618: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 44) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 45) {
    goto yy655;
  } else {

  }
  if ((int )yych <= 46) {
    goto yy602;
  } else {

  }
  goto yy57;
  yy619: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych == 65) {
      goto yy651;
    } else {

    }
    if ((int )yych <= 84) {
      goto yy57;
    } else {

    }
    goto yy650;
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 96) {
      goto yy57;
    } else {

    }
    goto yy651;
  } else {
    if ((int )yych == 117) {
      goto yy650;
    } else {

    }
    goto yy57;
  }
  yy620: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy648;
  } else {

  }
  if ((int )yych == 101) {
    goto yy648;
  } else {

  }
  goto yy57;
  yy621: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy645;
  } else {

  }
  if ((int )yych == 97) {
    goto yy645;
  } else {

  }
  goto yy57;
  yy622: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych == 80) {
      goto yy642;
    } else {

    }
    if ((int )yych <= 84) {
      goto yy57;
    } else {

    }
    goto yy641;
  } else
  if ((int )yych <= 112) {
    if ((int )yych <= 111) {
      goto yy57;
    } else {

    }
    goto yy642;
  } else {
    if ((int )yych == 117) {
      goto yy641;
    } else {

    }
    goto yy57;
  }
  yy623: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy638;
  } else {

  }
  if ((int )yych == 101) {
    goto yy638;
  } else {

  }
  goto yy57;
  yy624: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 67) {
    goto yy636;
  } else {

  }
  if ((int )yych == 99) {
    goto yy636;
  } else {

  }
  goto yy57;
  yy625: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 79) {
    goto yy634;
  } else {

  }
  if ((int )yych == 111) {
    goto yy634;
  } else {

  }
  goto yy57;
  yy626: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy627;
  } else {

  }
  if ((int )yych != 101) {
    goto yy57;
  } else {

  }
  yy627: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 67) {
    goto yy628;
  } else {

  }
  if ((int )yych != 99) {
    goto yy57;
  } else {

  }
  yy628: 
  yyaccept = 10U;
  cursor ++;
  tmp___236 = cursor;
  s->ptr = tmp___236;
  yych = *tmp___236;
  if ((int )yych <= 46) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy532;
      } else {

      }
      goto yy476;
    } else {
      if ((int )yych <= 32) {
        goto yy532;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy476;
      } else {

      }
      if ((int )yych >= 46) {
        goto yy532;
      } else {

      }
    }
  } else
  if ((int )yych <= 68) {
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy534;
    } else {

    }
    goto yy476;
  } else {
    if ((int )yych <= 69) {
      goto yy581;
    } else {

    }
    if ((int )yych == 101) {
      goto yy581;
    } else {

    }
    goto yy476;
  }
  yy629: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy533;
  } else {

  }
  if ((int )yych <= 48) {
    goto yy630;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy631;
  } else {

  }
  if ((int )yych <= 51) {
    goto yy632;
  } else {

  }
  goto yy533;
  yy630: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy535;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy633;
  } else {

  }
  goto yy535;
  yy631: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy535;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy633;
  } else {

  }
  goto yy535;
  yy632: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy535;
  } else {

  }
  if ((int )yych <= 49) {
    goto yy633;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy541;
  } else {

  }
  goto yy535;
  yy633: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy535;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy542;
  } else {

  }
  goto yy535;
  yy634: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 86) {
    goto yy635;
  } else {

  }
  if ((int )yych != 118) {
    goto yy57;
  } else {

  }
  yy635: 
  yyaccept = 10U;
  cursor ++;
  tmp___237 = cursor;
  s->ptr = tmp___237;
  yych = *tmp___237;
  if ((int )yych <= 46) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy532;
      } else {

      }
      goto yy476;
    } else {
      if ((int )yych <= 32) {
        goto yy532;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy476;
      } else {

      }
      if ((int )yych <= 45) {
        goto yy629;
      } else {

      }
      goto yy532;
    }
  } else
  if ((int )yych <= 68) {
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy534;
    } else {

    }
    goto yy476;
  } else {
    if ((int )yych <= 69) {
      goto yy536;
    } else {

    }
    if ((int )yych == 101) {
      goto yy536;
    } else {

    }
    goto yy476;
  }
  yy636: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 84) {
    goto yy637;
  } else {

  }
  if ((int )yych != 116) {
    goto yy57;
  } else {

  }
  yy637: 
  yyaccept = 10U;
  cursor ++;
  tmp___238 = cursor;
  s->ptr = tmp___238;
  yych = *tmp___238;
  if ((int )yych <= 46) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy532;
      } else {

      }
      goto yy476;
    } else {
      if ((int )yych <= 32) {
        goto yy532;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy476;
      } else {

      }
      if ((int )yych <= 45) {
        goto yy629;
      } else {

      }
      goto yy532;
    }
  } else
  if ((int )yych <= 78) {
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy534;
    } else {

    }
    goto yy476;
  } else {
    if ((int )yych <= 79) {
      goto yy546;
    } else {

    }
    if ((int )yych == 111) {
      goto yy546;
    } else {

    }
    goto yy476;
  }
  yy638: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 80) {
    goto yy639;
  } else {

  }
  if ((int )yych != 112) {
    goto yy57;
  } else {

  }
  yy639: 
  yyaccept = 10U;
  cursor ++;
  tmp___239 = cursor;
  s->ptr = tmp___239;
  yych = *tmp___239;
  if ((int )yych <= 46) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy532;
      } else {

      }
      goto yy476;
    } else {
      if ((int )yych <= 32) {
        goto yy532;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy476;
      } else {

      }
      if ((int )yych <= 45) {
        goto yy629;
      } else {

      }
      goto yy532;
    }
  } else
  if ((int )yych <= 83) {
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy534;
    } else {

    }
    goto yy476;
  } else {
    if ((int )yych <= 84) {
      goto yy640;
    } else {

    }
    if ((int )yych != 116) {
      goto yy476;
    } else {

    }
  }
  yy640: 
  yyaccept = 10U;
  cursor ++;
  tmp___240 = cursor;
  s->ptr = tmp___240;
  yych = *tmp___240;
  if ((int )yych <= 46) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy532;
      } else {

      }
      goto yy476;
    } else {
      if ((int )yych <= 32) {
        goto yy532;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy476;
      } else {

      }
      if ((int )yych <= 45) {
        goto yy629;
      } else {

      }
      goto yy532;
    }
  } else
  if ((int )yych <= 68) {
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy534;
    } else {

    }
    goto yy476;
  } else {
    if ((int )yych <= 69) {
      goto yy588;
    } else {

    }
    if ((int )yych == 101) {
      goto yy588;
    } else {

    }
    goto yy476;
  }
  yy641: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 71) {
    goto yy644;
  } else {

  }
  if ((int )yych == 103) {
    goto yy644;
  } else {

  }
  goto yy57;
  yy642: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy643;
  } else {

  }
  if ((int )yych != 114) {
    goto yy57;
  } else {

  }
  yy643: 
  yyaccept = 10U;
  cursor ++;
  tmp___241 = cursor;
  s->ptr = tmp___241;
  yych = *tmp___241;
  if ((int )yych <= 46) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy532;
      } else {

      }
      goto yy476;
    } else {
      if ((int )yych <= 32) {
        goto yy532;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy476;
      } else {

      }
      if ((int )yych <= 45) {
        goto yy629;
      } else {

      }
      goto yy532;
    }
  } else
  if ((int )yych <= 72) {
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy534;
    } else {

    }
    goto yy476;
  } else {
    if ((int )yych <= 73) {
      goto yy552;
    } else {

    }
    if ((int )yych == 105) {
      goto yy552;
    } else {

    }
    goto yy476;
  }
  yy644: 
  yyaccept = 10U;
  cursor ++;
  tmp___242 = cursor;
  s->ptr = tmp___242;
  yych = *tmp___242;
  if ((int )yych <= 46) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy532;
      } else {

      }
      goto yy476;
    } else {
      if ((int )yych <= 32) {
        goto yy532;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy476;
      } else {

      }
      if ((int )yych <= 45) {
        goto yy629;
      } else {

      }
      goto yy532;
    }
  } else
  if ((int )yych <= 84) {
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy534;
    } else {

    }
    goto yy476;
  } else {
    if ((int )yych <= 85) {
      goto yy554;
    } else {

    }
    if ((int )yych == 117) {
      goto yy554;
    } else {

    }
    goto yy476;
  }
  yy645: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 89) {
    if ((int )yych == 82) {
      goto yy646;
    } else {

    }
    if ((int )yych <= 88) {
      goto yy57;
    } else {

    }
    goto yy647;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy57;
    } else {

    }
  } else {
    if ((int )yych == 121) {
      goto yy647;
    } else {

    }
    goto yy57;
  }
  yy646: 
  yyaccept = 10U;
  cursor ++;
  tmp___243 = cursor;
  s->ptr = tmp___243;
  yych = *tmp___243;
  if ((int )yych <= 46) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy532;
      } else {

      }
      goto yy476;
    } else {
      if ((int )yych <= 32) {
        goto yy532;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy476;
      } else {

      }
      if ((int )yych <= 45) {
        goto yy629;
      } else {

      }
      goto yy532;
    }
  } else
  if ((int )yych <= 66) {
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy534;
    } else {

    }
    goto yy476;
  } else {
    if ((int )yych <= 67) {
      goto yy594;
    } else {

    }
    if ((int )yych == 99) {
      goto yy594;
    } else {

    }
    goto yy476;
  }
  yy647: 
  yyaccept = 10U;
  cursor ++;
  tmp___244 = cursor;
  s->ptr = tmp___244;
  yych = *tmp___244;
  if ((int )yych <= 44) {
    if ((int )yych <= 9) {
      if ((int )yych <= 8) {
        goto yy476;
      } else {

      }
      goto yy532;
    } else {
      if ((int )yych == 32) {
        goto yy532;
      } else {

      }
      goto yy476;
    }
  } else
  if ((int )yych <= 46) {
    if ((int )yych <= 45) {
      goto yy629;
    } else {

    }
    goto yy532;
  } else {
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy534;
    } else {

    }
    goto yy476;
  }
  yy648: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 66) {
    goto yy649;
  } else {

  }
  if ((int )yych != 98) {
    goto yy57;
  } else {

  }
  yy649: 
  yyaccept = 10U;
  cursor ++;
  tmp___245 = cursor;
  s->ptr = tmp___245;
  yych = *tmp___245;
  if ((int )yych <= 46) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy532;
      } else {

      }
      goto yy476;
    } else {
      if ((int )yych <= 32) {
        goto yy532;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy476;
      } else {

      }
      if ((int )yych <= 45) {
        goto yy629;
      } else {

      }
      goto yy532;
    }
  } else
  if ((int )yych <= 81) {
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy534;
    } else {

    }
    goto yy476;
  } else {
    if ((int )yych <= 82) {
      goto yy597;
    } else {

    }
    if ((int )yych == 114) {
      goto yy597;
    } else {

    }
    goto yy476;
  }
  yy650: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 78) {
    if ((int )yych == 76) {
      goto yy654;
    } else {

    }
    if ((int )yych <= 77) {
      goto yy57;
    } else {

    }
    goto yy653;
  } else
  if ((int )yych <= 108) {
    if ((int )yych <= 107) {
      goto yy57;
    } else {

    }
    goto yy654;
  } else {
    if ((int )yych == 110) {
      goto yy653;
    } else {

    }
    goto yy57;
  }
  yy651: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 78) {
    goto yy652;
  } else {

  }
  if ((int )yych != 110) {
    goto yy57;
  } else {

  }
  yy652: 
  yyaccept = 10U;
  cursor ++;
  tmp___246 = cursor;
  s->ptr = tmp___246;
  yych = *tmp___246;
  if ((int )yych <= 46) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy532;
      } else {

      }
      goto yy476;
    } else {
      if ((int )yych <= 32) {
        goto yy532;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy476;
      } else {

      }
      if ((int )yych <= 45) {
        goto yy629;
      } else {

      }
      goto yy532;
    }
  } else
  if ((int )yych <= 84) {
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy534;
    } else {

    }
    goto yy476;
  } else {
    if ((int )yych <= 85) {
      goto yy559;
    } else {

    }
    if ((int )yych == 117) {
      goto yy559;
    } else {

    }
    goto yy476;
  }
  yy653: 
  yyaccept = 10U;
  cursor ++;
  tmp___247 = cursor;
  s->ptr = tmp___247;
  yych = *tmp___247;
  if ((int )yych <= 46) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy532;
      } else {

      }
      goto yy476;
    } else {
      if ((int )yych <= 32) {
        goto yy532;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy476;
      } else {

      }
      if ((int )yych <= 45) {
        goto yy629;
      } else {

      }
      goto yy532;
    }
  } else
  if ((int )yych <= 68) {
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy534;
    } else {

    }
    goto yy476;
  } else {
    if ((int )yych <= 69) {
      goto yy540;
    } else {

    }
    if ((int )yych == 101) {
      goto yy540;
    } else {

    }
    goto yy476;
  }
  yy654: 
  yyaccept = 10U;
  cursor ++;
  tmp___248 = cursor;
  s->ptr = tmp___248;
  yych = *tmp___248;
  if ((int )yych <= 46) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy532;
      } else {

      }
      goto yy476;
    } else {
      if ((int )yych <= 32) {
        goto yy532;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy476;
      } else {

      }
      if ((int )yych <= 45) {
        goto yy629;
      } else {

      }
      goto yy532;
    }
  } else
  if ((int )yych <= 88) {
    if ((int )yych <= 47) {
      goto yy476;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy534;
    } else {

    }
    goto yy476;
  } else {
    if ((int )yych <= 89) {
      goto yy540;
    } else {

    }
    if ((int )yych == 121) {
      goto yy540;
    } else {

    }
    goto yy476;
  }
  yy655: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy656;
  } else {

  }
  if ((int )yych <= 51) {
    goto yy658;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy659;
  } else {

  }
  goto yy57;
  yy656: 
  yyaccept = 13U;
  cursor ++;
  tmp___249 = cursor;
  s->ptr = tmp___249;
  yych = *tmp___249;
  if ((int )yych <= 110) {
    if ((int )yych <= 47) {
      goto yy657;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy665;
    } else {

    }
    if ((int )yych >= 110) {
      goto yy661;
    } else {

    }
  } else
  if ((int )yych <= 114) {
    if ((int )yych >= 114) {
      goto yy662;
    } else {

    }
  } else {
    if ((int )yych <= 115) {
      goto yy660;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy663;
    } else {

    }
  }
  yy657: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  if ((s->time)->have_date) {
    add_error(s, (char *)"Double date specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_date = 1U;
  }
  (s->time)->y = timelib_get_nr(& ptr, 4);
  (s->time)->m = timelib_get_nr(& ptr, 2);
  (s->time)->d = timelib_get_nr(& ptr, 2);
  if ((s->time)->y == -99999LL) {

  } else
  if ((s->time)->y < 100LL) {
    if ((s->time)->y < 70LL) {
      (s->time)->y += 2000LL;
    } else {
      (s->time)->y += 1900LL;
    }
  } else {

  }
  free((void *)str);
  return (267);
  yy658: 
  yyaccept = 13U;
  cursor ++;
  tmp___250 = cursor;
  s->ptr = tmp___250;
  yych = *tmp___250;
  if ((int )yych <= 110) {
    if ((int )yych <= 49) {
      if ((int )yych <= 47) {
        goto yy657;
      } else {

      }
      goto yy665;
    } else {
      if ((int )yych <= 57) {
        goto yy604;
      } else {

      }
      if ((int )yych <= 109) {
        goto yy657;
      } else {

      }
      goto yy661;
    }
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy657;
    } else {

    }
    goto yy662;
  } else {
    if ((int )yych <= 115) {
      goto yy660;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy663;
    } else {

    }
    goto yy657;
  }
  yy659: 
  yyaccept = 13U;
  cursor ++;
  tmp___251 = cursor;
  s->ptr = tmp___251;
  yych = *tmp___251;
  if ((int )yych <= 110) {
    if ((int )yych <= 47) {
      goto yy657;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy604;
    } else {

    }
    if ((int )yych <= 109) {
      goto yy657;
    } else {

    }
    goto yy661;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy657;
    } else {

    }
    goto yy662;
  } else {
    if ((int )yych <= 115) {
      goto yy660;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy663;
    } else {

    }
    goto yy657;
  }
  yy660: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 116) {
    goto yy664;
  } else {

  }
  goto yy57;
  yy661: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 100) {
    goto yy664;
  } else {

  }
  goto yy57;
  yy662: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 100) {
    goto yy664;
  } else {

  }
  goto yy57;
  yy663: 
  cursor ++;
  yych = *cursor;
  if ((int )yych != 104) {
    goto yy57;
  } else {

  }
  yy664: 
  cursor ++;
  yych = *cursor;
  goto yy657;
  yy665: 
  yyaccept = 13U;
  cursor ++;
  tmp___252 = cursor;
  s->ptr = tmp___252;
  yych = *tmp___252;
  if ((int )yych <= 110) {
    if ((int )yych <= 47) {
      goto yy657;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy605;
    } else {

    }
    if ((int )yych <= 109) {
      goto yy657;
    } else {

    }
    goto yy661;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy657;
    } else {

    }
    goto yy662;
  } else {
    if ((int )yych <= 115) {
      goto yy660;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy663;
    } else {

    }
    goto yy657;
  }
  yy666: 
  yyaccept = 14U;
  cursor ++;
  tmp___253 = cursor;
  s->ptr = tmp___253;
  yych = *tmp___253;
  if ((int )yych <= 110) {
    if ((int )yych <= 47) {
      if ((int )yych >= 47) {
        goto yy723;
      } else {

      }
    } else {
      if ((int )yych <= 57) {
        goto yy669;
      } else {

      }
      if ((int )yych >= 110) {
        goto yy720;
      } else {

      }
    }
  } else
  if ((int )yych <= 114) {
    if ((int )yych >= 114) {
      goto yy721;
    } else {

    }
  } else {
    if ((int )yych <= 115) {
      goto yy719;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy722;
    } else {

    }
  }
  yy667: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  if ((s->time)->have_date) {
    add_error(s, (char *)"Double date specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_date = 1U;
  }
  (s->time)->m = timelib_get_nr(& ptr, 2);
  (s->time)->d = timelib_get_nr(& ptr, 2);
  if ((int )*ptr == 47) {
    (s->time)->y = timelib_get_nr(& ptr, 4);
    if ((s->time)->y == -99999LL) {

    } else
    if ((s->time)->y < 100LL) {
      if ((s->time)->y < 70LL) {
        (s->time)->y += 2000LL;
      } else {
        (s->time)->y += 1900LL;
      }
    } else {

    }
  } else {

  }
  free((void *)str);
  return (266);
  yy668: 
  yyaccept = 14U;
  cursor ++;
  tmp___254 = cursor;
  s->ptr = tmp___254;
  yych = *tmp___254;
  if ((int )yych <= 110) {
    if ((int )yych <= 47) {
      if ((int )yych <= 46) {
        goto yy667;
      } else {

      }
      goto yy723;
    } else {
      if ((int )yych <= 49) {
        goto yy669;
      } else {

      }
      if ((int )yych <= 109) {
        goto yy667;
      } else {

      }
      goto yy720;
    }
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy667;
    } else {

    }
    goto yy721;
  } else {
    if ((int )yych <= 115) {
      goto yy719;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy722;
    } else {

    }
    goto yy667;
  }
  yy669: 
  yyaccept = 14U;
  cursor ++;
  tmp___255 = cursor;
  s->ptr = tmp___255;
  yych = *tmp___255;
  if ((int )yych <= 110) {
    if ((int )yych == 47) {
      goto yy723;
    } else {

    }
    if ((int )yych <= 109) {
      goto yy667;
    } else {

    }
    goto yy720;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy667;
    } else {

    }
    goto yy721;
  } else {
    if ((int )yych <= 115) {
      goto yy719;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy722;
    } else {

    }
    goto yy667;
  }
  yy670: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych == 65) {
      goto yy718;
    } else {

    }
    if ((int )yych <= 84) {
      goto yy57;
    } else {

    }
    goto yy717;
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 96) {
      goto yy57;
    } else {

    }
    goto yy718;
  } else {
    if ((int )yych == 117) {
      goto yy717;
    } else {

    }
    goto yy57;
  }
  yy671: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy716;
  } else {

  }
  if ((int )yych == 101) {
    goto yy716;
  } else {

  }
  goto yy57;
  yy672: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy715;
  } else {

  }
  if ((int )yych == 97) {
    goto yy715;
  } else {

  }
  goto yy57;
  yy673: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych == 80) {
      goto yy714;
    } else {

    }
    if ((int )yych <= 84) {
      goto yy57;
    } else {

    }
    goto yy713;
  } else
  if ((int )yych <= 112) {
    if ((int )yych <= 111) {
      goto yy57;
    } else {

    }
    goto yy714;
  } else {
    if ((int )yych == 117) {
      goto yy713;
    } else {

    }
    goto yy57;
  }
  yy674: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy711;
  } else {

  }
  if ((int )yych == 101) {
    goto yy711;
  } else {

  }
  goto yy57;
  yy675: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 67) {
    goto yy710;
  } else {

  }
  if ((int )yych == 99) {
    goto yy710;
  } else {

  }
  goto yy57;
  yy676: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 79) {
    goto yy709;
  } else {

  }
  if ((int )yych == 111) {
    goto yy709;
  } else {

  }
  goto yy57;
  yy677: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy678;
  } else {

  }
  if ((int )yych != 101) {
    goto yy57;
  } else {

  }
  yy678: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 67) {
    goto yy679;
  } else {

  }
  if ((int )yych != 99) {
    goto yy57;
  } else {

  }
  yy679: 
  cursor ++;
  yych = *cursor;
  if ((int )yych != 47) {
    goto yy57;
  } else {

  }
  yy680: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych != 58) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 49) {
    goto yy686;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy687;
  } else {

  }
  goto yy57;
  yy686: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy688;
  } else {

  }
  goto yy57;
  yy687: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 53) {
    goto yy57;
  } else {

  }
  yy688: 
  cursor ++;
  yych = *cursor;
  if ((int )yych != 58) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 54) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych != 58) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy693;
  } else {

  }
  if ((int )yych <= 54) {
    goto yy694;
  } else {

  }
  goto yy57;
  yy693: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy695;
  } else {

  }
  goto yy57;
  yy694: 
  cursor ++;
  yych = *cursor;
  if ((int )yych != 48) {
    goto yy57;
  } else {

  }
  yy695: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 9) {
    goto yy696;
  } else {

  }
  if ((int )yych != 32) {
    goto yy57;
  } else {

  }
  yy696: 
  cursor ++;
  if (s->lim - cursor < 9) {
    return (257);
  } else {

  }
  yych = *cursor;
  if ((int )yych <= 42) {
    if ((int )yych <= 9) {
      if ((int )yych <= 8) {
        goto yy57;
      } else {

      }
      goto yy696;
    } else {
      if ((int )yych == 32) {
        goto yy696;
      } else {

      }
      goto yy57;
    }
  } else
  if ((int )yych <= 45) {
    if ((int )yych == 44) {
      goto yy57;
    } else {

    }
    goto yy699;
  } else
  if ((int )yych != 71) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych == 77) {
    goto yy707;
  } else {

  }
  goto yy57;
  yy699: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 49) {
    goto yy700;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy702;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy703;
  } else {

  }
  goto yy57;
  yy700: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy701;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy703;
  } else {

  }
  if ((int )yych <= 58) {
    goto yy704;
  } else {

  }
  yy701: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  if ((s->time)->have_time) {
    add_error(s, (char *)"Double time specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_time = 1U;
    (s->time)->h = (timelib_sll )0;
    (s->time)->i = (timelib_sll )0;
    (s->time)->s = (timelib_sll )0;
    (s->time)->f = (double )0;
  }
  if ((s->time)->have_date) {
    add_error(s, (char *)"Double date specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_date = 1U;
  }
  (s->time)->d = timelib_get_nr(& ptr, 2);
  tmp___256 = timelib_get_month(& ptr);
  (s->time)->m = (timelib_sll )tmp___256;
  (s->time)->y = timelib_get_nr(& ptr, 4);
  (s->time)->h = timelib_get_nr(& ptr, 2);
  (s->time)->i = timelib_get_nr(& ptr, 2);
  (s->time)->s = timelib_get_nr(& ptr, 2);
  tmp___257 = timelib_get_zone(& ptr, & (s->time)->dst, s->time,
                               & tz_not_found___2, s->tzdb);
  (s->time)->z = (int )tmp___257;
  if (tz_not_found___2) {
    add_error(s, (char *)"The timezone could not be found in the database");
  } else {

  }
  free((void *)str);
  return (274);
  yy702: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 53) {
    if ((int )yych <= 47) {
      goto yy701;
    } else {

    }
    if ((int )yych >= 53) {
      goto yy705;
    } else {

    }
  } else {
    if ((int )yych <= 57) {
      goto yy706;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy704;
    } else {

    }
    goto yy701;
  }
  yy703: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy701;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy705;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy706;
  } else {

  }
  if ((int )yych >= 59) {
    goto yy701;
  } else {

  }
  yy704: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy701;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy705;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy706;
  } else {

  }
  goto yy701;
  yy705: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy701;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy701;
  } else {

  }
  yy706: 
  cursor ++;
  yych = *cursor;
  goto yy701;
  yy707: 
  cursor ++;
  yych = *cursor;
  if ((int )yych != 84) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych == 43) {
    goto yy699;
  } else {

  }
  if ((int )yych == 45) {
    goto yy699;
  } else {

  }
  goto yy57;
  yy709: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 86) {
    goto yy679;
  } else {

  }
  if ((int )yych == 118) {
    goto yy679;
  } else {

  }
  goto yy57;
  yy710: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 84) {
    goto yy679;
  } else {

  }
  if ((int )yych == 116) {
    goto yy679;
  } else {

  }
  goto yy57;
  yy711: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 80) {
    goto yy712;
  } else {

  }
  if ((int )yych != 112) {
    goto yy57;
  } else {

  }
  yy712: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 83) {
    if ((int )yych == 47) {
      goto yy680;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 84) {
      goto yy679;
    } else {

    }
    if ((int )yych == 116) {
      goto yy679;
    } else {

    }
    goto yy57;
  }
  yy713: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 71) {
    goto yy679;
  } else {

  }
  if ((int )yych == 103) {
    goto yy679;
  } else {

  }
  goto yy57;
  yy714: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy679;
  } else {

  }
  if ((int )yych == 114) {
    goto yy679;
  } else {

  }
  goto yy57;
  yy715: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 89) {
    if ((int )yych == 82) {
      goto yy679;
    } else {

    }
    if ((int )yych <= 88) {
      goto yy57;
    } else {

    }
    goto yy679;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy57;
    } else {

    }
    goto yy679;
  } else {
    if ((int )yych == 121) {
      goto yy679;
    } else {

    }
    goto yy57;
  }
  yy716: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 66) {
    goto yy679;
  } else {

  }
  if ((int )yych == 98) {
    goto yy679;
  } else {

  }
  goto yy57;
  yy717: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 78) {
    if ((int )yych == 76) {
      goto yy679;
    } else {

    }
    if ((int )yych <= 77) {
      goto yy57;
    } else {

    }
    goto yy679;
  } else
  if ((int )yych <= 108) {
    if ((int )yych <= 107) {
      goto yy57;
    } else {

    }
    goto yy679;
  } else {
    if ((int )yych == 110) {
      goto yy679;
    } else {

    }
    goto yy57;
  }
  yy718: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 78) {
    goto yy679;
  } else {

  }
  if ((int )yych == 110) {
    goto yy679;
  } else {

  }
  goto yy57;
  yy719: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 116) {
    goto yy728;
  } else {

  }
  goto yy57;
  yy720: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 100) {
    goto yy728;
  } else {

  }
  goto yy57;
  yy721: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 100) {
    goto yy728;
  } else {

  }
  goto yy57;
  yy722: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 104) {
    goto yy728;
  } else {

  }
  goto yy57;
  yy723: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy667;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy667;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy667;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy667;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy667;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy667;
  } else {

  }
  cursor ++;
  yych = *cursor;
  goto yy667;
  yy728: 
  yyaccept = 14U;
  cursor ++;
  tmp___258 = cursor;
  s->ptr = tmp___258;
  yych = *tmp___258;
  if ((int )yych == 47) {
    goto yy723;
  } else {

  }
  goto yy667;
  yy729: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 44) {
    if ((int )yych == 9) {
      goto yy731;
    } else {

    }
    goto yy578;
  } else {
    if ((int )yych <= 45) {
      goto yy732;
    } else {

    }
    if ((int )yych <= 46) {
      goto yy731;
    } else {

    }
    if ((int )yych >= 48) {
      goto yy578;
    } else {

    }
  }
  yy730: 
  cursor ++;
  yych = *cursor;
  switch ((int )yych) {
  case 97: 
  case 65: 
  goto yy673;
  case 100: 
  case 68: 
  goto yy677;
  case 102: 
  case 70: 
  goto yy671;
  case 106: 
  case 74: 
  goto yy670;
  case 109: 
  case 77: 
  goto yy672;
  case 110: 
  case 78: 
  goto yy676;
  case 111: 
  case 79: 
  goto yy675;
  case 115: 
  case 83: 
  goto yy674;
  default: 
  goto yy57;
  }
  yy731: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy578;
  } else {

  }
  if ((int )yych <= 48) {
    goto yy736;
  } else {

  }
  if ((int )yych <= 49) {
    goto yy737;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy738;
  } else {

  }
  goto yy578;
  yy732: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy578;
  } else {

  }
  if ((int )yych <= 48) {
    goto yy733;
  } else {

  }
  if ((int )yych <= 49) {
    goto yy734;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy735;
  } else {

  }
  goto yy578;
  yy733: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 44) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 46) {
    goto yy602;
  } else {

  }
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy735;
  } else {

  }
  goto yy57;
  yy734: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 44) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 46) {
    goto yy602;
  } else {

  }
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 51) {
    goto yy57;
  } else {

  }
  yy735: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 44) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 46) {
    goto yy602;
  } else {

  }
  goto yy57;
  yy736: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 46) {
    if ((int )yych <= 44) {
      goto yy57;
    } else {

    }
    if ((int )yych <= 45) {
      goto yy602;
    } else {

    }
    goto yy739;
  } else {
    if ((int )yych <= 47) {
      goto yy57;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy738;
    } else {

    }
    goto yy57;
  }
  yy737: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 46) {
    if ((int )yych <= 44) {
      goto yy57;
    } else {

    }
    if ((int )yych <= 45) {
      goto yy602;
    } else {

    }
    goto yy739;
  } else {
    if ((int )yych <= 47) {
      goto yy57;
    } else {

    }
    if ((int )yych >= 51) {
      goto yy57;
    } else {

    }
  }
  yy738: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 44) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 45) {
    goto yy602;
  } else {

  }
  if ((int )yych >= 47) {
    goto yy57;
  } else {

  }
  yy739: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy611;
  } else {

  }
  goto yy57;
  yy741: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 45) {
    goto yy785;
  } else {

  }
  if ((int )yych <= 47) {
    goto yy61;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy783;
  } else {

  }
  goto yy61;
  yy742: 
  cursor ++;
  yych = *cursor;
  switch ((int )yych) {
  case 48: 
  goto yy751;
  case 49: 
  goto yy752;
  case 57: 
  case 56: 
  case 55: 
  case 54: 
  case 53: 
  case 52: 
  case 51: 
  case 50: 
  goto yy753;
  case 97: 
  case 65: 
  goto yy746;
  case 100: 
  case 68: 
  goto yy750;
  case 102: 
  case 70: 
  goto yy744;
  case 106: 
  case 74: 
  goto yy743;
  case 109: 
  case 77: 
  goto yy745;
  case 110: 
  case 78: 
  goto yy749;
  case 111: 
  case 79: 
  goto yy748;
  case 115: 
  case 83: 
  goto yy747;
  default: 
  goto yy57;
  }
  yy743: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych == 65) {
      goto yy782;
    } else {

    }
    if ((int )yych <= 84) {
      goto yy57;
    } else {

    }
    goto yy781;
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 96) {
      goto yy57;
    } else {

    }
    goto yy782;
  } else {
    if ((int )yych == 117) {
      goto yy781;
    } else {

    }
    goto yy57;
  }
  yy744: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy780;
  } else {

  }
  if ((int )yych == 101) {
    goto yy780;
  } else {

  }
  goto yy57;
  yy745: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy779;
  } else {

  }
  if ((int )yych == 97) {
    goto yy779;
  } else {

  }
  goto yy57;
  yy746: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych == 80) {
      goto yy778;
    } else {

    }
    if ((int )yych <= 84) {
      goto yy57;
    } else {

    }
    goto yy777;
  } else
  if ((int )yych <= 112) {
    if ((int )yych <= 111) {
      goto yy57;
    } else {

    }
    goto yy778;
  } else {
    if ((int )yych == 117) {
      goto yy777;
    } else {

    }
    goto yy57;
  }
  yy747: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy775;
  } else {

  }
  if ((int )yych == 101) {
    goto yy775;
  } else {

  }
  goto yy57;
  yy748: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 67) {
    goto yy774;
  } else {

  }
  if ((int )yych == 99) {
    goto yy774;
  } else {

  }
  goto yy57;
  yy749: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 79) {
    goto yy773;
  } else {

  }
  if ((int )yych == 111) {
    goto yy773;
  } else {

  }
  goto yy57;
  yy750: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy765;
  } else {

  }
  if ((int )yych == 101) {
    goto yy765;
  } else {

  }
  goto yy57;
  yy751: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 45) {
    goto yy754;
  } else {

  }
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy758;
  } else {

  }
  goto yy57;
  yy752: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 45) {
    goto yy754;
  } else {

  }
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy758;
  } else {

  }
  goto yy57;
  yy753: 
  cursor ++;
  yych = *cursor;
  if ((int )yych != 45) {
    goto yy57;
  } else {

  }
  yy754: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy755;
  } else {

  }
  if ((int )yych <= 51) {
    goto yy756;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy757;
  } else {

  }
  goto yy57;
  yy755: 
  yyaccept = 13U;
  cursor ++;
  tmp___259 = cursor;
  s->ptr = tmp___259;
  yych = *tmp___259;
  if ((int )yych <= 110) {
    if ((int )yych <= 47) {
      goto yy657;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy757;
    } else {

    }
    if ((int )yych <= 109) {
      goto yy657;
    } else {

    }
    goto yy661;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy657;
    } else {

    }
    goto yy662;
  } else {
    if ((int )yych <= 115) {
      goto yy660;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy663;
    } else {

    }
    goto yy657;
  }
  yy756: 
  yyaccept = 13U;
  cursor ++;
  tmp___260 = cursor;
  s->ptr = tmp___260;
  yych = *tmp___260;
  if ((int )yych <= 110) {
    if ((int )yych <= 47) {
      goto yy657;
    } else {

    }
    if ((int )yych <= 49) {
      goto yy757;
    } else {

    }
    if ((int )yych <= 109) {
      goto yy657;
    } else {

    }
    goto yy661;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy657;
    } else {

    }
    goto yy662;
  } else {
    if ((int )yych <= 115) {
      goto yy660;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy663;
    } else {

    }
    goto yy657;
  }
  yy757: 
  yyaccept = 13U;
  cursor ++;
  tmp___261 = cursor;
  s->ptr = tmp___261;
  yych = *tmp___261;
  if ((int )yych <= 113) {
    if ((int )yych == 110) {
      goto yy661;
    } else {

    }
    goto yy657;
  } else {
    if ((int )yych <= 114) {
      goto yy662;
    } else {

    }
    if ((int )yych <= 115) {
      goto yy660;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy663;
    } else {

    }
    goto yy657;
  }
  yy758: 
  cursor ++;
  yych = *cursor;
  if ((int )yych != 45) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 50) {
    if ((int )yych <= 47) {
      goto yy57;
    } else {

    }
    if ((int )yych >= 49) {
      goto yy761;
    } else {

    }
  } else {
    if ((int )yych <= 51) {
      goto yy762;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy757;
    } else {

    }
    goto yy57;
  }
  yyaccept = 13U;
  cursor ++;
  tmp___262 = cursor;
  s->ptr = tmp___262;
  yych = *tmp___262;
  if ((int )yych <= 110) {
    if ((int )yych <= 47) {
      goto yy657;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy763;
    } else {

    }
    if ((int )yych <= 109) {
      goto yy657;
    } else {

    }
    goto yy661;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy657;
    } else {

    }
    goto yy662;
  } else {
    if ((int )yych <= 115) {
      goto yy660;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy663;
    } else {

    }
    goto yy657;
  }
  yy761: 
  yyaccept = 13U;
  cursor ++;
  tmp___263 = cursor;
  s->ptr = tmp___263;
  yych = *tmp___263;
  if ((int )yych <= 110) {
    if ((int )yych <= 47) {
      goto yy657;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy763;
    } else {

    }
    if ((int )yych <= 109) {
      goto yy657;
    } else {

    }
    goto yy661;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy657;
    } else {

    }
    goto yy662;
  } else {
    if ((int )yych <= 115) {
      goto yy660;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy663;
    } else {

    }
    goto yy657;
  }
  yy762: 
  yyaccept = 13U;
  cursor ++;
  tmp___264 = cursor;
  s->ptr = tmp___264;
  yych = *tmp___264;
  if ((int )yych <= 110) {
    if ((int )yych <= 47) {
      goto yy657;
    } else {

    }
    if ((int )yych <= 49) {
      goto yy763;
    } else {

    }
    if ((int )yych <= 109) {
      goto yy657;
    } else {

    }
    goto yy661;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy657;
    } else {

    }
    goto yy662;
  } else {
    if ((int )yych <= 115) {
      goto yy660;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy663;
    } else {

    }
    goto yy657;
  }
  yy763: 
  yyaccept = 15U;
  cursor ++;
  tmp___265 = cursor;
  s->ptr = tmp___265;
  yych = *tmp___265;
  if ((int )yych <= 113) {
    if ((int )yych == 110) {
      goto yy661;
    } else {

    }
  } else {
    if ((int )yych <= 114) {
      goto yy662;
    } else {

    }
    if ((int )yych <= 115) {
      goto yy660;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy663;
    } else {

    }
  }
  yy764: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  if ((s->time)->have_date) {
    add_error(s, (char *)"Double date specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_date = 1U;
  }
  (s->time)->y = timelib_get_nr(& ptr, 4);
  (s->time)->m = timelib_get_nr(& ptr, 2);
  (s->time)->d = timelib_get_nr(& ptr, 2);
  if ((s->time)->y == -99999LL) {

  } else
  if ((s->time)->y < 100LL) {
    if ((s->time)->y < 70LL) {
      (s->time)->y += 2000LL;
    } else {
      (s->time)->y += 1900LL;
    }
  } else {

  }
  free((void *)str);
  return (267);
  yy765: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 67) {
    goto yy766;
  } else {

  }
  if ((int )yych != 99) {
    goto yy57;
  } else {

  }
  yy766: 
  cursor ++;
  yych = *cursor;
  if ((int )yych != 45) {
    goto yy57;
  } else {

  }
  yy767: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 48) {
    goto yy768;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy769;
  } else {

  }
  if ((int )yych <= 51) {
    goto yy770;
  } else {

  }
  goto yy57;
  yy768: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy771;
  } else {

  }
  goto yy57;
  yy769: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy771;
  } else {

  }
  goto yy57;
  yy770: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 50) {
    goto yy57;
  } else {

  }
  yy771: 
  cursor ++;
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  if ((s->time)->have_date) {
    add_error(s, (char *)"Double date specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_date = 1U;
  }
  (s->time)->y = timelib_get_nr(& ptr, 4);
  tmp___266 = timelib_get_month(& ptr);
  (s->time)->m = (timelib_sll )tmp___266;
  (s->time)->d = timelib_get_nr(& ptr, 2);
  if ((s->time)->y == -99999LL) {

  } else
  if ((s->time)->y < 100LL) {
    if ((s->time)->y < 70LL) {
      (s->time)->y += 2000LL;
    } else {
      (s->time)->y += 1900LL;
    }
  } else {

  }
  free((void *)str);
  return (272);
  yy773: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 86) {
    goto yy766;
  } else {

  }
  if ((int )yych == 118) {
    goto yy766;
  } else {

  }
  goto yy57;
  yy774: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 84) {
    goto yy766;
  } else {

  }
  if ((int )yych == 116) {
    goto yy766;
  } else {

  }
  goto yy57;
  yy775: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 80) {
    goto yy776;
  } else {

  }
  if ((int )yych != 112) {
    goto yy57;
  } else {

  }
  yy776: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 83) {
    if ((int )yych == 45) {
      goto yy767;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 84) {
      goto yy766;
    } else {

    }
    if ((int )yych == 116) {
      goto yy766;
    } else {

    }
    goto yy57;
  }
  yy777: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 71) {
    goto yy766;
  } else {

  }
  if ((int )yych == 103) {
    goto yy766;
  } else {

  }
  goto yy57;
  yy778: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy766;
  } else {

  }
  if ((int )yych == 114) {
    goto yy766;
  } else {

  }
  goto yy57;
  yy779: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 89) {
    if ((int )yych == 82) {
      goto yy766;
    } else {

    }
    if ((int )yych <= 88) {
      goto yy57;
    } else {

    }
    goto yy766;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy57;
    } else {

    }
    goto yy766;
  } else {
    if ((int )yych == 121) {
      goto yy766;
    } else {

    }
    goto yy57;
  }
  yy780: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 66) {
    goto yy766;
  } else {

  }
  if ((int )yych == 98) {
    goto yy766;
  } else {

  }
  goto yy57;
  yy781: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 78) {
    if ((int )yych == 76) {
      goto yy766;
    } else {

    }
    if ((int )yych <= 77) {
      goto yy57;
    } else {

    }
    goto yy766;
  } else
  if ((int )yych <= 108) {
    if ((int )yych <= 107) {
      goto yy57;
    } else {

    }
    goto yy766;
  } else {
    if ((int )yych == 110) {
      goto yy766;
    } else {

    }
    goto yy57;
  }
  yy782: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 78) {
    goto yy766;
  } else {

  }
  if ((int )yych == 110) {
    goto yy766;
  } else {

  }
  goto yy57;
  yy783: 
  yyaccept = 16U;
  cursor ++;
  tmp___267 = cursor;
  s->ptr = tmp___267;
  yych = *tmp___267;
  switch ((int )yych) {
  case 121: 
  case 119: 
  case 116: 
  case 115: 
  case 111: 
  case 110: 
  case 109: 
  case 106: 
  case 104: 
  case 102: 
  case 100: 
  case 97: 
  case 89: 
  case 88: 
  case 86: 
  case 84: 
  case 83: 
  case 79: 
  case 78: 
  case 77: 
  case 74: 
  case 73: 
  case 72: 
  case 70: 
  case 68: 
  case 65: 
  case 32: 
  case 9: 
  goto yy791;
  case 45: 
  goto yy788;
  case 46: 
  goto yy792;
  case 47: 
  goto yy789;
  case 48: 
  goto yy805;
  case 49: 
  goto yy806;
  case 50: 
  goto yy808;
  case 51: 
  goto yy809;
  case 57: 
  case 56: 
  case 55: 
  case 54: 
  case 53: 
  case 52: 
  goto yy55;
  case 58: 
  goto yy807;
  case 87: 
  goto yy810;
  default: 
  goto yy784;
  }
  yy784: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  (s->time)->y = timelib_get_nr(& ptr, 4);
  free((void *)str);
  return (274);
  yy785: 
  cursor ++;
  yych = *cursor;
  switch ((int )yych) {
  case 48: 
  goto yy786;
  case 49: 
  goto yy787;
  case 57: 
  case 56: 
  case 55: 
  case 54: 
  case 53: 
  case 52: 
  case 51: 
  case 50: 
  goto yy753;
  case 97: 
  case 65: 
  goto yy746;
  case 100: 
  case 68: 
  goto yy750;
  case 102: 
  case 70: 
  goto yy744;
  case 106: 
  case 74: 
  goto yy743;
  case 109: 
  case 77: 
  goto yy745;
  case 110: 
  case 78: 
  goto yy749;
  case 111: 
  case 79: 
  goto yy748;
  case 115: 
  case 83: 
  goto yy747;
  default: 
  goto yy57;
  }
  yy786: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 45) {
    goto yy754;
  } else {

  }
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy753;
  } else {

  }
  goto yy57;
  yy787: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 45) {
    goto yy754;
  } else {

  }
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy753;
  } else {

  }
  goto yy57;
  yy788: 
  cursor ++;
  yych = *cursor;
  switch ((int )yych) {
  case 48: 
  goto yy973;
  case 49: 
  goto yy975;
  case 57: 
  case 56: 
  case 55: 
  case 54: 
  case 53: 
  case 52: 
  case 51: 
  case 50: 
  goto yy976;
  case 97: 
  case 65: 
  goto yy967;
  case 100: 
  case 68: 
  goto yy971;
  case 102: 
  case 70: 
  goto yy965;
  case 106: 
  case 74: 
  goto yy964;
  case 109: 
  case 77: 
  goto yy966;
  case 110: 
  case 78: 
  goto yy970;
  case 111: 
  case 79: 
  goto yy969;
  case 115: 
  case 83: 
  goto yy968;
  case 87: 
  goto yy972;
  default: 
  goto yy939;
  }
  yy789: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 48) {
    goto yy947;
  } else {

  }
  if ((int )yych <= 49) {
    goto yy948;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy949;
  } else {

  }
  goto yy57;
  yy790: 
  cursor ++;
  if (s->lim - cursor < 11) {
    return (257);
  } else {

  }
  yych = *cursor;
  yy791: ;
  switch ((int )yych) {
  case 32: 
  case 9: 
  goto yy790;
  case 46: 
  case 45: 
  goto yy938;
  case 97: 
  case 65: 
  goto yy800;
  case 100: 
  case 68: 
  goto yy804;
  case 102: 
  case 70: 
  goto yy798;
  case 104: 
  case 72: 
  goto yy64;
  case 73: 
  goto yy793;
  case 106: 
  case 74: 
  goto yy797;
  case 109: 
  case 77: 
  goto yy799;
  case 110: 
  case 78: 
  goto yy803;
  case 111: 
  case 79: 
  goto yy802;
  case 115: 
  case 83: 
  goto yy801;
  case 116: 
  case 84: 
  goto yy69;
  case 86: 
  goto yy795;
  case 119: 
  case 87: 
  goto yy68;
  case 88: 
  goto yy796;
  case 121: 
  case 89: 
  goto yy67;
  default: 
  goto yy57;
  }
  yy792: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy939;
  } else {

  }
  if ((int )yych <= 48) {
    goto yy931;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy932;
  } else {

  }
  if ((int )yych <= 51) {
    goto yy933;
  } else {

  }
  goto yy939;
  yy793: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych == 73) {
      goto yy930;
    } else {

    }
  } else {
    if ((int )yych == 87) {
      goto yy794;
    } else {

    }
    if ((int )yych <= 88) {
      goto yy884;
    } else {

    }
  }
  yy794: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  if ((s->time)->have_date) {
    add_error(s, (char *)"Double date specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_date = 1U;
  }
  (s->time)->y = timelib_get_nr(& ptr, 4);
  tmp___268 = timelib_get_month(& ptr);
  (s->time)->m = (timelib_sll )tmp___268;
  (s->time)->d = (timelib_sll )1;
  if ((s->time)->y == -99999LL) {

  } else
  if ((s->time)->y < 100LL) {
    if ((s->time)->y < 70LL) {
      (s->time)->y += 2000LL;
    } else {
      (s->time)->y += 1900LL;
    }
  } else {

  }
  free((void *)str);
  return (275);
  yy795: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 73) {
    goto yy928;
  } else {

  }
  goto yy794;
  yy796: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 73) {
    goto yy927;
  } else {

  }
  goto yy794;
  yy797: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych == 65) {
      goto yy920;
    } else {

    }
    if ((int )yych <= 84) {
      goto yy57;
    } else {

    }
    goto yy919;
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 96) {
      goto yy57;
    } else {

    }
    goto yy920;
  } else {
    if ((int )yych == 117) {
      goto yy919;
    } else {

    }
    goto yy57;
  }
  yy798: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 82) {
    if ((int )yych <= 78) {
      if ((int )yych == 69) {
        goto yy913;
      } else {

      }
      goto yy57;
    } else {
      if ((int )yych <= 79) {
        goto yy99;
      } else {

      }
      if ((int )yych <= 81) {
        goto yy57;
      } else {

      }
      goto yy98;
    }
  } else
  if ((int )yych <= 110) {
    if ((int )yych == 101) {
      goto yy913;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 111) {
      goto yy99;
    } else {

    }
    if ((int )yych == 114) {
      goto yy98;
    } else {

    }
    goto yy57;
  }
  yy799: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 79) {
    if ((int )yych <= 72) {
      if ((int )yych == 65) {
        goto yy910;
      } else {

      }
      goto yy57;
    } else {
      if ((int )yych <= 73) {
        goto yy118;
      } else {

      }
      if ((int )yych <= 78) {
        goto yy57;
      } else {

      }
      goto yy117;
    }
  } else
  if ((int )yych <= 104) {
    if ((int )yych == 97) {
      goto yy910;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 105) {
      goto yy118;
    } else {

    }
    if ((int )yych == 111) {
      goto yy117;
    } else {

    }
    goto yy57;
  }
  yy800: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych == 80) {
      goto yy904;
    } else {

    }
    if ((int )yych <= 84) {
      goto yy57;
    } else {

    }
    goto yy903;
  } else
  if ((int )yych <= 112) {
    if ((int )yych <= 111) {
      goto yy57;
    } else {

    }
    goto yy904;
  } else {
    if ((int )yych == 117) {
      goto yy903;
    } else {

    }
    goto yy57;
  }
  yy801: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych <= 68) {
      if ((int )yych == 65) {
        goto yy127;
      } else {

      }
      goto yy57;
    } else {
      if ((int )yych <= 69) {
        goto yy896;
      } else {

      }
      if ((int )yych <= 84) {
        goto yy57;
      } else {

      }
      goto yy126;
    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych == 97) {
      goto yy127;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 101) {
      goto yy896;
    } else {

    }
    if ((int )yych == 117) {
      goto yy126;
    } else {

    }
    goto yy57;
  }
  yy802: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 67) {
    goto yy891;
  } else {

  }
  if ((int )yych == 99) {
    goto yy891;
  } else {

  }
  goto yy57;
  yy803: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 79) {
    goto yy885;
  } else {

  }
  if ((int )yych == 111) {
    goto yy885;
  } else {

  }
  goto yy57;
  yy804: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 69) {
    if ((int )yych == 65) {
      goto yy114;
    } else {

    }
    if ((int )yych <= 68) {
      goto yy57;
    } else {

    }
    goto yy878;
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 96) {
      goto yy57;
    } else {

    }
    goto yy114;
  } else {
    if ((int )yych == 101) {
      goto yy878;
    } else {

    }
    goto yy57;
  }
  yy805: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy61;
  } else {

  }
  if ((int )yych <= 48) {
    goto yy875;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy876;
  } else {

  }
  goto yy61;
  yy806: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy61;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy844;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy823;
  } else {

  }
  goto yy61;
  yy807: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 48) {
    goto yy824;
  } else {

  }
  if ((int )yych <= 49) {
    goto yy825;
  } else {

  }
  goto yy57;
  yy808: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy61;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy823;
  } else {

  }
  goto yy61;
  yy809: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy61;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy819;
  } else {

  }
  if ((int )yych <= 54) {
    goto yy820;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy55;
  } else {

  }
  goto yy61;
  yy810: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 53) {
    if ((int )yych <= 47) {
      goto yy57;
    } else {

    }
    if ((int )yych <= 48) {
      goto yy811;
    } else {

    }
    if ((int )yych <= 52) {
      goto yy812;
    } else {

    }
    goto yy813;
  } else
  if ((int )yych <= 69) {
    if ((int )yych <= 68) {
      goto yy57;
    } else {

    }
    goto yy83;
  } else {
    if ((int )yych == 101) {
      goto yy83;
    } else {

    }
    goto yy57;
  }
  yy811: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 48) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy814;
  } else {

  }
  goto yy57;
  yy812: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy814;
  } else {

  }
  goto yy57;
  yy813: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 52) {
    goto yy57;
  } else {

  }
  yy814: 
  yyaccept = 17U;
  cursor ++;
  tmp___269 = cursor;
  s->ptr = tmp___269;
  yych = *tmp___269;
  if ((int )yych == 45) {
    goto yy816;
  } else {

  }
  if ((int )yych <= 47) {
    goto yy815;
  } else {

  }
  if ((int )yych <= 55) {
    goto yy817;
  } else {

  }
  yy815: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  if ((s->time)->have_date) {
    add_error(s, (char *)"Double date specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_date = 1U;
  }
  (s->time)->have_relative = 1U;
  (s->time)->y = timelib_get_nr(& ptr, 4);
  w = timelib_get_nr(& ptr, 2);
  d = (timelib_sll )1;
  (s->time)->m = (timelib_sll )1;
  (s->time)->d = (timelib_sll )1;
  (s->time)->relative.d = timelib_daynr_from_weeknr((s->time)->y, w, d);
  free((void *)str);
  return (279);
  yy816: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 56) {
    goto yy57;
  } else {

  }
  yy817: 
  cursor ++;
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  if ((s->time)->have_date) {
    add_error(s, (char *)"Double date specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_date = 1U;
  }
  (s->time)->have_relative = 1U;
  (s->time)->y = timelib_get_nr(& ptr, 4);
  w___0 = timelib_get_nr(& ptr, 2);
  d___0 = timelib_get_nr(& ptr, 1);
  (s->time)->m = (timelib_sll )1;
  (s->time)->d = (timelib_sll )1;
  (s->time)->relative.d = timelib_daynr_from_weeknr((s->time)->y, w___0, d___0);
  free((void *)str);
  return (279);
  yy819: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy61;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy821;
  } else {

  }
  goto yy61;
  yy820: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy61;
  } else {

  }
  if ((int )yych <= 54) {
    goto yy821;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy55;
  } else {

  }
  goto yy61;
  yy821: 
  yyaccept = 18U;
  cursor ++;
  tmp___270 = cursor;
  s->ptr = tmp___270;
  yych = *tmp___270;
  if ((int const   )yybm[(int )yych] & 2) {
    goto yy55;
  } else {

  }
  if ((int )yych <= 87) {
    if ((int )yych <= 70) {
      if ((int )yych <= 32) {
        if ((int )yych == 9) {
          goto yy61;
        } else {

        }
        if ((int )yych >= 32) {
          goto yy61;
        } else {

        }
      } else {
        if ((int )yych == 68) {
          goto yy61;
        } else {

        }
        if ((int )yych >= 70) {
          goto yy61;
        } else {

        }
      }
    } else
    if ((int )yych <= 77) {
      if ((int )yych == 72) {
        goto yy61;
      } else {

      }
      if ((int )yych >= 77) {
        goto yy61;
      } else {

      }
    } else {
      if ((int )yych <= 82) {
        goto yy822;
      } else {

      }
      if ((int )yych <= 84) {
        goto yy61;
      } else {

      }
      if ((int )yych >= 87) {
        goto yy61;
      } else {

      }
    }
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 100) {
      if ((int )yych == 89) {
        goto yy61;
      } else {

      }
      if ((int )yych >= 100) {
        goto yy61;
      } else {

      }
    } else {
      if ((int )yych == 102) {
        goto yy61;
      } else {

      }
      if ((int )yych >= 104) {
        goto yy61;
      } else {

      }
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych == 109) {
      goto yy61;
    } else {

    }
    if ((int )yych >= 115) {
      goto yy61;
    } else {

    }
  } else
  if ((int )yych <= 119) {
    if ((int )yych >= 119) {
      goto yy61;
    } else {

    }
  } else
  if ((int )yych == 121) {
    goto yy61;
  } else {

  }
  yy822: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  if ((s->time)->have_date) {
    add_error(s, (char *)"Double date specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_date = 1U;
  }
  (s->time)->y = timelib_get_nr(& ptr, 4);
  (s->time)->d = timelib_get_nr(& ptr, 3);
  (s->time)->m = (timelib_sll )1;
  if ((s->time)->y == -99999LL) {

  } else
  if ((s->time)->y < 100LL) {
    if ((s->time)->y < 70LL) {
      (s->time)->y += 2000LL;
    } else {
      (s->time)->y += 1900LL;
    }
  } else {

  }
  free((void *)str);
  return (271);
  yy823: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy61;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy821;
  } else {

  }
  goto yy61;
  yy824: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy826;
  } else {

  }
  goto yy57;
  yy825: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 51) {
    goto yy57;
  } else {

  }
  yy826: 
  cursor ++;
  yych = *cursor;
  if ((int )yych != 58) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 48) {
    goto yy828;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy829;
  } else {

  }
  if ((int )yych <= 51) {
    goto yy830;
  } else {

  }
  goto yy57;
  yy828: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy831;
  } else {

  }
  goto yy57;
  yy829: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy831;
  } else {

  }
  goto yy57;
  yy830: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 50) {
    goto yy57;
  } else {

  }
  yy831: 
  cursor ++;
  yych = *cursor;
  if ((int )yych != 32) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 49) {
    goto yy833;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy834;
  } else {

  }
  goto yy57;
  yy833: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy835;
  } else {

  }
  goto yy57;
  yy834: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 53) {
    goto yy57;
  } else {

  }
  yy835: 
  cursor ++;
  yych = *cursor;
  if ((int )yych != 58) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 54) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych != 58) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy840;
  } else {

  }
  if ((int )yych <= 54) {
    goto yy841;
  } else {

  }
  goto yy57;
  yy840: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy842;
  } else {

  }
  goto yy57;
  yy841: 
  cursor ++;
  yych = *cursor;
  if ((int )yych != 48) {
    goto yy57;
  } else {

  }
  yy842: 
  cursor ++;
  yy843: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  if ((s->time)->have_time) {
    add_error(s, (char *)"Double time specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_time = 1U;
    (s->time)->h = (timelib_sll )0;
    (s->time)->i = (timelib_sll )0;
    (s->time)->s = (timelib_sll )0;
    (s->time)->f = (double )0;
  }
  if ((s->time)->have_date) {
    add_error(s, (char *)"Double date specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_date = 1U;
  }
  (s->time)->y = timelib_get_nr(& ptr, 4);
  (s->time)->m = timelib_get_nr(& ptr, 2);
  (s->time)->d = timelib_get_nr(& ptr, 2);
  (s->time)->h = timelib_get_nr(& ptr, 2);
  (s->time)->i = timelib_get_nr(& ptr, 2);
  (s->time)->s = timelib_get_nr(& ptr, 2);
  if ((int )*ptr == 46) {
    (s->time)->f = timelib_get_frac_nr(& ptr, 9);
    if (*ptr) {
      tmp___271 = timelib_get_zone(& ptr, & (s->time)->dst, s->time,
                                   & tz_not_found___3, s->tzdb);
      (s->time)->z = (int )tmp___271;
      if (tz_not_found___3) {
        add_error(s, (char *)"The timezone could not be found in the database");
      } else {

      }
    } else {

    }
  } else {

  }
  free((void *)str);
  return (260);
  yy844: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 50) {
    if ((int )yych <= 47) {
      goto yy61;
    } else {

    }
    if ((int )yych >= 49) {
      goto yy846;
    } else {

    }
  } else {
    if ((int )yych <= 51) {
      goto yy847;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy821;
    } else {

    }
    goto yy61;
  }
  yy845: 
  yyaccept = 18U;
  cursor ++;
  tmp___272 = cursor;
  s->ptr = tmp___272;
  yych = *tmp___272;
  if ((int )yych <= 86) {
    if ((int )yych <= 68) {
      if ((int )yych <= 32) {
        if ((int )yych == 9) {
          goto yy61;
        } else {

        }
        if ((int )yych <= 31) {
          goto yy822;
        } else {

        }
        goto yy61;
      } else {
        if ((int )yych <= 47) {
          goto yy822;
        } else {

        }
        if ((int )yych <= 57) {
          goto yy848;
        } else {

        }
        if ((int )yych <= 67) {
          goto yy822;
        } else {

        }
        goto yy61;
      }
    } else
    if ((int )yych <= 72) {
      if ((int )yych == 70) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 71) {
        goto yy822;
      } else {

      }
      goto yy61;
    } else
    if ((int )yych <= 77) {
      if ((int )yych <= 76) {
        goto yy822;
      } else {

      }
      goto yy61;
    } else {
      if ((int )yych <= 82) {
        goto yy822;
      } else {

      }
      if ((int )yych <= 84) {
        goto yy61;
      } else {

      }
      goto yy822;
    }
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 99) {
      if ((int )yych == 88) {
        goto yy822;
      } else {

      }
      if ((int )yych <= 89) {
        goto yy61;
      } else {

      }
      goto yy822;
    } else
    if ((int )yych <= 101) {
      if ((int )yych <= 100) {
        goto yy61;
      } else {

      }
      goto yy822;
    } else {
      if ((int )yych == 103) {
        goto yy822;
      } else {

      }
      goto yy61;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych == 109) {
      goto yy61;
    } else {

    }
    if ((int )yych <= 114) {
      goto yy822;
    } else {

    }
    goto yy61;
  } else
  if ((int )yych <= 119) {
    if ((int )yych <= 118) {
      goto yy822;
    } else {

    }
    goto yy61;
  } else {
    if ((int )yych == 121) {
      goto yy61;
    } else {

    }
    goto yy822;
  }
  yy846: 
  yyaccept = 18U;
  cursor ++;
  tmp___273 = cursor;
  s->ptr = tmp___273;
  yych = *tmp___273;
  if ((int )yych <= 86) {
    if ((int )yych <= 68) {
      if ((int )yych <= 32) {
        if ((int )yych == 9) {
          goto yy61;
        } else {

        }
        if ((int )yych <= 31) {
          goto yy822;
        } else {

        }
        goto yy61;
      } else {
        if ((int )yych <= 47) {
          goto yy822;
        } else {

        }
        if ((int )yych <= 57) {
          goto yy848;
        } else {

        }
        if ((int )yych <= 67) {
          goto yy822;
        } else {

        }
        goto yy61;
      }
    } else
    if ((int )yych <= 72) {
      if ((int )yych == 70) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 71) {
        goto yy822;
      } else {

      }
      goto yy61;
    } else
    if ((int )yych <= 77) {
      if ((int )yych <= 76) {
        goto yy822;
      } else {

      }
      goto yy61;
    } else {
      if ((int )yych <= 82) {
        goto yy822;
      } else {

      }
      if ((int )yych <= 84) {
        goto yy61;
      } else {

      }
      goto yy822;
    }
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 99) {
      if ((int )yych == 88) {
        goto yy822;
      } else {

      }
      if ((int )yych <= 89) {
        goto yy61;
      } else {

      }
      goto yy822;
    } else
    if ((int )yych <= 101) {
      if ((int )yych <= 100) {
        goto yy61;
      } else {

      }
      goto yy822;
    } else {
      if ((int )yych == 103) {
        goto yy822;
      } else {

      }
      goto yy61;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych == 109) {
      goto yy61;
    } else {

    }
    if ((int )yych <= 114) {
      goto yy822;
    } else {

    }
    goto yy61;
  } else
  if ((int )yych <= 119) {
    if ((int )yych <= 118) {
      goto yy822;
    } else {

    }
    goto yy61;
  } else {
    if ((int )yych == 121) {
      goto yy61;
    } else {

    }
    goto yy822;
  }
  yy847: 
  yyaccept = 18U;
  cursor ++;
  tmp___274 = cursor;
  s->ptr = tmp___274;
  yych = *tmp___274;
  if ((int )yych <= 86) {
    if ((int )yych <= 68) {
      if ((int )yych <= 32) {
        if ((int )yych == 9) {
          goto yy61;
        } else {

        }
        if ((int )yych <= 31) {
          goto yy822;
        } else {

        }
        goto yy61;
      } else
      if ((int )yych <= 49) {
        if ((int )yych <= 47) {
          goto yy822;
        } else {

        }
      } else {
        if ((int )yych <= 57) {
          goto yy55;
        } else {

        }
        if ((int )yych <= 67) {
          goto yy822;
        } else {

        }
        goto yy61;
      }
    } else
    if ((int )yych <= 72) {
      if ((int )yych == 70) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 71) {
        goto yy822;
      } else {

      }
      goto yy61;
    } else
    if ((int )yych <= 77) {
      if ((int )yych <= 76) {
        goto yy822;
      } else {

      }
      goto yy61;
    } else {
      if ((int )yych <= 82) {
        goto yy822;
      } else {

      }
      if ((int )yych <= 84) {
        goto yy61;
      } else {

      }
      goto yy822;
    }
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 99) {
      if ((int )yych == 88) {
        goto yy822;
      } else {

      }
      if ((int )yych <= 89) {
        goto yy61;
      } else {

      }
      goto yy822;
    } else
    if ((int )yych <= 101) {
      if ((int )yych <= 100) {
        goto yy61;
      } else {

      }
      goto yy822;
    } else {
      if ((int )yych == 103) {
        goto yy822;
      } else {

      }
      goto yy61;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych == 109) {
      goto yy61;
    } else {

    }
    if ((int )yych <= 114) {
      goto yy822;
    } else {

    }
    goto yy61;
  } else
  if ((int )yych <= 119) {
    if ((int )yych <= 118) {
      goto yy822;
    } else {

    }
    goto yy61;
  } else {
    if ((int )yych == 121) {
      goto yy61;
    } else {

    }
    goto yy822;
  }
  yy848: 
  yyaccept = 19U;
  cursor ++;
  tmp___275 = cursor;
  s->ptr = tmp___275;
  yych = *tmp___275;
  if ((int const   )yybm[(int )yych] & 2) {
    goto yy55;
  } else {

  }
  if ((int )yych <= 87) {
    if ((int )yych <= 70) {
      if ((int )yych <= 32) {
        if ((int )yych == 9) {
          goto yy60;
        } else {

        }
        if ((int )yych >= 32) {
          goto yy60;
        } else {

        }
      } else {
        if ((int )yych == 68) {
          goto yy65;
        } else {

        }
        if ((int )yych >= 70) {
          goto yy66;
        } else {

        }
      }
    } else
    if ((int )yych <= 77) {
      if ((int )yych == 72) {
        goto yy64;
      } else {

      }
      if ((int )yych >= 77) {
        goto yy63;
      } else {

      }
    } else
    if ((int )yych <= 83) {
      if ((int )yych >= 83) {
        goto yy62;
      } else {

      }
    } else {
      if ((int )yych <= 84) {
        goto yy850;
      } else {

      }
      if ((int )yych >= 87) {
        goto yy68;
      } else {

      }
    }
  } else
  if ((int )yych <= 108) {
    if ((int )yych <= 100) {
      if ((int )yych == 89) {
        goto yy67;
      } else {

      }
      if ((int )yych >= 100) {
        goto yy65;
      } else {

      }
    } else
    if ((int )yych <= 102) {
      if ((int )yych >= 102) {
        goto yy66;
      } else {

      }
    } else
    if ((int )yych == 104) {
      goto yy64;
    } else {

    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych <= 109) {
      goto yy63;
    } else {

    }
    if ((int )yych <= 114) {
      goto yy849;
    } else {

    }
    if ((int )yych <= 115) {
      goto yy62;
    } else {

    }
    goto yy851;
  } else
  if ((int )yych <= 119) {
    if ((int )yych >= 119) {
      goto yy68;
    } else {

    }
  } else
  if ((int )yych == 121) {
    goto yy67;
  } else {

  }
  yy849: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  if ((s->time)->have_date) {
    add_error(s, (char *)"Double date specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_date = 1U;
  }
  (s->time)->y = timelib_get_nr(& ptr, 4);
  (s->time)->m = timelib_get_nr(& ptr, 2);
  (s->time)->d = timelib_get_nr(& ptr, 2);
  free((void *)str);
  return (270);
  yy850: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 72) {
    if ((int )yych <= 50) {
      if ((int )yych <= 47) {
        goto yy57;
      } else {

      }
      if ((int )yych <= 49) {
        goto yy865;
      } else {

      }
      goto yy866;
    } else {
      if ((int )yych <= 57) {
        goto yy867;
      } else {

      }
      if ((int )yych <= 71) {
        goto yy57;
      } else {

      }
      goto yy70;
    }
  } else
  if ((int )yych <= 103) {
    if ((int )yych == 85) {
      goto yy71;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 104) {
      goto yy70;
    } else {

    }
    if ((int )yych == 117) {
      goto yy71;
    } else {

    }
    goto yy57;
  }
  yy851: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 72) {
    if ((int )yych <= 50) {
      if ((int )yych <= 47) {
        goto yy57;
      } else {

      }
      if ((int )yych >= 50) {
        goto yy853;
      } else {

      }
    } else {
      if ((int )yych <= 57) {
        goto yy854;
      } else {

      }
      if ((int )yych <= 71) {
        goto yy57;
      } else {

      }
      goto yy70;
    }
  } else
  if ((int )yych <= 103) {
    if ((int )yych == 85) {
      goto yy71;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 104) {
      goto yy70;
    } else {

    }
    if ((int )yych == 117) {
      goto yy71;
    } else {

    }
    goto yy57;
  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy859;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy854;
  } else {

  }
  goto yy57;
  yy853: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 52) {
    goto yy859;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy855;
  } else {

  }
  goto yy57;
  yy854: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 54) {
    goto yy57;
  } else {

  }
  yy855: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy57;
  } else {

  }
  yy856: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy857;
  } else {

  }
  if ((int )yych <= 54) {
    goto yy858;
  } else {

  }
  goto yy57;
  yy857: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy842;
  } else {

  }
  goto yy57;
  yy858: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 48) {
    goto yy842;
  } else {

  }
  goto yy57;
  yy859: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy860;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy856;
  } else {

  }
  goto yy57;
  yy860: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy861;
  } else {

  }
  if ((int )yych <= 54) {
    goto yy862;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy856;
  } else {

  }
  goto yy57;
  yy861: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy863;
  } else {

  }
  if ((int )yych <= 54) {
    goto yy864;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy842;
  } else {

  }
  goto yy57;
  yy862: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 48) {
    goto yy863;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy857;
  } else {

  }
  if ((int )yych <= 54) {
    goto yy858;
  } else {

  }
  goto yy57;
  yy863: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy843;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy842;
  } else {

  }
  goto yy843;
  yy864: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 48) {
    goto yy842;
  } else {

  }
  goto yy843;
  yy865: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy874;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy867;
  } else {

  }
  if ((int )yych <= 58) {
    goto yy868;
  } else {

  }
  goto yy57;
  yy866: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 53) {
    if ((int )yych <= 47) {
      goto yy57;
    } else {

    }
    if ((int )yych <= 52) {
      goto yy874;
    } else {

    }
    goto yy855;
  } else {
    if ((int )yych == 58) {
      goto yy868;
    } else {

    }
    goto yy57;
  }
  yy867: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy855;
  } else {

  }
  if ((int )yych != 58) {
    goto yy57;
  } else {

  }
  yy868: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 54) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych != 58) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy872;
  } else {

  }
  if ((int )yych <= 54) {
    goto yy873;
  } else {

  }
  goto yy57;
  yy872: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy842;
  } else {

  }
  goto yy57;
  yy873: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 48) {
    goto yy842;
  } else {

  }
  goto yy57;
  yy874: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy860;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy856;
  } else {

  }
  if ((int )yych <= 58) {
    goto yy868;
  } else {

  }
  goto yy57;
  yy875: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 50) {
    if ((int )yych <= 47) {
      goto yy61;
    } else {

    }
    if ((int )yych <= 48) {
      goto yy877;
    } else {

    }
    goto yy846;
  } else {
    if ((int )yych <= 51) {
      goto yy847;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy821;
    } else {

    }
    goto yy61;
  }
  yy876: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 50) {
    if ((int )yych <= 47) {
      goto yy61;
    } else {

    }
    if ((int )yych <= 48) {
      goto yy845;
    } else {

    }
    goto yy846;
  } else {
    if ((int )yych <= 51) {
      goto yy847;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy821;
    } else {

    }
    goto yy61;
  }
  yy877: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy61;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy848;
  } else {

  }
  goto yy61;
  yy878: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 67) {
    goto yy879;
  } else {

  }
  if ((int )yych != 99) {
    goto yy57;
  } else {

  }
  yy879: 
  yyaccept = 20U;
  cursor ++;
  tmp___276 = cursor;
  s->ptr = tmp___276;
  yych = *tmp___276;
  if ((int )yych == 69) {
    goto yy880;
  } else {

  }
  if ((int )yych != 101) {
    goto yy794;
  } else {

  }
  yy880: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 77) {
    goto yy881;
  } else {

  }
  if ((int )yych != 109) {
    goto yy57;
  } else {

  }
  yy881: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 66) {
    goto yy882;
  } else {

  }
  if ((int )yych != 98) {
    goto yy57;
  } else {

  }
  yy882: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy883;
  } else {

  }
  if ((int )yych != 101) {
    goto yy57;
  } else {

  }
  yy883: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy884;
  } else {

  }
  if ((int )yych != 114) {
    goto yy57;
  } else {

  }
  yy884: 
  cursor ++;
  yych = *cursor;
  goto yy794;
  yy885: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 86) {
    goto yy886;
  } else {

  }
  if ((int )yych != 118) {
    goto yy57;
  } else {

  }
  yy886: 
  yyaccept = 20U;
  cursor ++;
  tmp___277 = cursor;
  s->ptr = tmp___277;
  yych = *tmp___277;
  if ((int )yych == 69) {
    goto yy887;
  } else {

  }
  if ((int )yych != 101) {
    goto yy794;
  } else {

  }
  yy887: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 77) {
    goto yy888;
  } else {

  }
  if ((int )yych != 109) {
    goto yy57;
  } else {

  }
  yy888: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 66) {
    goto yy889;
  } else {

  }
  if ((int )yych != 98) {
    goto yy57;
  } else {

  }
  yy889: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy890;
  } else {

  }
  if ((int )yych != 101) {
    goto yy57;
  } else {

  }
  yy890: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy884;
  } else {

  }
  if ((int )yych == 114) {
    goto yy884;
  } else {

  }
  goto yy57;
  yy891: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 84) {
    goto yy892;
  } else {

  }
  if ((int )yych != 116) {
    goto yy57;
  } else {

  }
  yy892: 
  yyaccept = 20U;
  cursor ++;
  tmp___278 = cursor;
  s->ptr = tmp___278;
  yych = *tmp___278;
  if ((int )yych == 79) {
    goto yy893;
  } else {

  }
  if ((int )yych != 111) {
    goto yy794;
  } else {

  }
  yy893: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 66) {
    goto yy894;
  } else {

  }
  if ((int )yych != 98) {
    goto yy57;
  } else {

  }
  yy894: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy895;
  } else {

  }
  if ((int )yych != 101) {
    goto yy57;
  } else {

  }
  yy895: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy884;
  } else {

  }
  if ((int )yych == 114) {
    goto yy884;
  } else {

  }
  goto yy57;
  yy896: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 80) {
    if ((int )yych == 67) {
      goto yy129;
    } else {

    }
    if ((int )yych <= 79) {
      goto yy57;
    } else {

    }
  } else
  if ((int )yych <= 99) {
    if ((int )yych <= 98) {
      goto yy57;
    } else {

    }
    goto yy129;
  } else
  if ((int )yych != 112) {
    goto yy57;
  } else {

  }
  yy897: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 84) {
    goto yy898;
  } else {

  }
  if ((int )yych != 116) {
    goto yy794;
  } else {

  }
  yy898: 
  yyaccept = 20U;
  cursor ++;
  tmp___279 = cursor;
  s->ptr = tmp___279;
  yych = *tmp___279;
  if ((int )yych == 69) {
    goto yy899;
  } else {

  }
  if ((int )yych != 101) {
    goto yy794;
  } else {

  }
  yy899: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 77) {
    goto yy900;
  } else {

  }
  if ((int )yych != 109) {
    goto yy57;
  } else {

  }
  yy900: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 66) {
    goto yy901;
  } else {

  }
  if ((int )yych != 98) {
    goto yy57;
  } else {

  }
  yy901: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy902;
  } else {

  }
  if ((int )yych != 101) {
    goto yy57;
  } else {

  }
  yy902: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy884;
  } else {

  }
  if ((int )yych == 114) {
    goto yy884;
  } else {

  }
  goto yy57;
  yy903: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 71) {
    goto yy907;
  } else {

  }
  if ((int )yych == 103) {
    goto yy907;
  } else {

  }
  goto yy57;
  yy904: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy905;
  } else {

  }
  if ((int )yych != 114) {
    goto yy57;
  } else {

  }
  yy905: 
  yyaccept = 20U;
  cursor ++;
  tmp___280 = cursor;
  s->ptr = tmp___280;
  yych = *tmp___280;
  if ((int )yych == 73) {
    goto yy906;
  } else {

  }
  if ((int )yych != 105) {
    goto yy794;
  } else {

  }
  yy906: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 76) {
    goto yy884;
  } else {

  }
  if ((int )yych == 108) {
    goto yy884;
  } else {

  }
  goto yy57;
  yy907: 
  yyaccept = 20U;
  cursor ++;
  tmp___281 = cursor;
  s->ptr = tmp___281;
  yych = *tmp___281;
  if ((int )yych == 85) {
    goto yy908;
  } else {

  }
  if ((int )yych != 117) {
    goto yy794;
  } else {

  }
  yy908: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 83) {
    goto yy909;
  } else {

  }
  if ((int )yych != 115) {
    goto yy57;
  } else {

  }
  yy909: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 84) {
    goto yy884;
  } else {

  }
  if ((int )yych == 116) {
    goto yy884;
  } else {

  }
  goto yy57;
  yy910: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 89) {
    if ((int )yych == 82) {
      goto yy911;
    } else {

    }
    if ((int )yych <= 88) {
      goto yy57;
    } else {

    }
    goto yy884;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy57;
    } else {

    }
  } else {
    if ((int )yych == 121) {
      goto yy884;
    } else {

    }
    goto yy57;
  }
  yy911: 
  yyaccept = 20U;
  cursor ++;
  tmp___282 = cursor;
  s->ptr = tmp___282;
  yych = *tmp___282;
  if ((int )yych == 67) {
    goto yy912;
  } else {

  }
  if ((int )yych != 99) {
    goto yy794;
  } else {

  }
  yy912: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 72) {
    goto yy884;
  } else {

  }
  if ((int )yych == 104) {
    goto yy884;
  } else {

  }
  goto yy57;
  yy913: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 66) {
    goto yy914;
  } else {

  }
  if ((int )yych != 98) {
    goto yy57;
  } else {

  }
  yy914: 
  yyaccept = 20U;
  cursor ++;
  tmp___283 = cursor;
  s->ptr = tmp___283;
  yych = *tmp___283;
  if ((int )yych == 82) {
    goto yy915;
  } else {

  }
  if ((int )yych != 114) {
    goto yy794;
  } else {

  }
  yy915: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 85) {
    goto yy916;
  } else {

  }
  if ((int )yych != 117) {
    goto yy57;
  } else {

  }
  yy916: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy917;
  } else {

  }
  if ((int )yych != 97) {
    goto yy57;
  } else {

  }
  yy917: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy918;
  } else {

  }
  if ((int )yych != 114) {
    goto yy57;
  } else {

  }
  yy918: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy884;
  } else {

  }
  if ((int )yych == 121) {
    goto yy884;
  } else {

  }
  goto yy57;
  yy919: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 78) {
    if ((int )yych == 76) {
      goto yy926;
    } else {

    }
    if ((int )yych <= 77) {
      goto yy57;
    } else {

    }
    goto yy925;
  } else
  if ((int )yych <= 108) {
    if ((int )yych <= 107) {
      goto yy57;
    } else {

    }
    goto yy926;
  } else {
    if ((int )yych == 110) {
      goto yy925;
    } else {

    }
    goto yy57;
  }
  yy920: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 78) {
    goto yy921;
  } else {

  }
  if ((int )yych != 110) {
    goto yy57;
  } else {

  }
  yy921: 
  yyaccept = 20U;
  cursor ++;
  tmp___284 = cursor;
  s->ptr = tmp___284;
  yych = *tmp___284;
  if ((int )yych == 85) {
    goto yy922;
  } else {

  }
  if ((int )yych != 117) {
    goto yy794;
  } else {

  }
  yy922: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy923;
  } else {

  }
  if ((int )yych != 97) {
    goto yy57;
  } else {

  }
  yy923: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy924;
  } else {

  }
  if ((int )yych != 114) {
    goto yy57;
  } else {

  }
  yy924: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy884;
  } else {

  }
  if ((int )yych == 121) {
    goto yy884;
  } else {

  }
  goto yy57;
  yy925: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy884;
  } else {

  }
  if ((int )yych == 101) {
    goto yy884;
  } else {

  }
  goto yy794;
  yy926: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy884;
  } else {

  }
  if ((int )yych == 121) {
    goto yy884;
  } else {

  }
  goto yy794;
  yy927: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 73) {
    goto yy884;
  } else {

  }
  goto yy794;
  yy928: 
  cursor ++;
  yych = *cursor;
  if ((int )yych != 73) {
    goto yy794;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych == 73) {
    goto yy884;
  } else {

  }
  goto yy794;
  yy930: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 73) {
    goto yy884;
  } else {

  }
  goto yy794;
  yy931: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 48) {
    goto yy946;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy945;
  } else {

  }
  goto yy57;
  yy932: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy944;
  } else {

  }
  goto yy57;
  yy933: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy942;
  } else {

  }
  if ((int )yych <= 54) {
    goto yy941;
  } else {

  }
  goto yy57;
  yy934: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy913;
  } else {

  }
  if ((int )yych == 101) {
    goto yy913;
  } else {

  }
  goto yy57;
  yy935: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy910;
  } else {

  }
  if ((int )yych == 97) {
    goto yy910;
  } else {

  }
  goto yy57;
  yy936: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy940;
  } else {

  }
  if ((int )yych == 101) {
    goto yy940;
  } else {

  }
  goto yy57;
  yy937: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy878;
  } else {

  }
  if ((int )yych == 101) {
    goto yy878;
  } else {

  }
  goto yy57;
  yy938: 
  cursor ++;
  if (s->lim - cursor < 9) {
    return (257);
  } else {

  }
  yych = *cursor;
  yy939: ;
  switch ((int )yych) {
  case 46: 
  case 45: 
  case 32: 
  case 9: 
  goto yy938;
  case 97: 
  case 65: 
  goto yy800;
  case 100: 
  case 68: 
  goto yy937;
  case 102: 
  case 70: 
  goto yy934;
  case 73: 
  goto yy793;
  case 106: 
  case 74: 
  goto yy797;
  case 109: 
  case 77: 
  goto yy935;
  case 110: 
  case 78: 
  goto yy803;
  case 111: 
  case 79: 
  goto yy802;
  case 115: 
  case 83: 
  goto yy936;
  case 86: 
  goto yy795;
  case 88: 
  goto yy796;
  default: 
  goto yy57;
  }
  yy940: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 80) {
    goto yy897;
  } else {

  }
  if ((int )yych == 112) {
    goto yy897;
  } else {

  }
  goto yy57;
  yy941: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 54) {
    goto yy943;
  } else {

  }
  goto yy57;
  yy942: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy57;
  } else {

  }
  yy943: 
  cursor ++;
  yych = *cursor;
  goto yy822;
  yy944: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy943;
  } else {

  }
  goto yy57;
  yy945: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy943;
  } else {

  }
  goto yy57;
  yy946: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 48) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy943;
  } else {

  }
  goto yy57;
  yy947: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 46) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 47) {
    goto yy950;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy958;
  } else {

  }
  goto yy57;
  yy948: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 46) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 47) {
    goto yy950;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy958;
  } else {

  }
  goto yy57;
  yy949: 
  cursor ++;
  yych = *cursor;
  if ((int )yych != 47) {
    goto yy57;
  } else {

  }
  yy950: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy951;
  } else {

  }
  if ((int )yych <= 51) {
    goto yy952;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy953;
  } else {

  }
  goto yy57;
  yy951: 
  yyaccept = 21U;
  cursor ++;
  tmp___285 = cursor;
  s->ptr = tmp___285;
  yych = *tmp___285;
  if ((int )yych <= 110) {
    if ((int )yych <= 47) {
      goto yy455;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy953;
    } else {

    }
    if ((int )yych <= 109) {
      goto yy455;
    } else {

    }
    goto yy955;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy455;
    } else {

    }
    goto yy956;
  } else {
    if ((int )yych <= 115) {
      goto yy954;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy957;
    } else {

    }
    goto yy455;
  }
  yy952: 
  yyaccept = 21U;
  cursor ++;
  tmp___286 = cursor;
  s->ptr = tmp___286;
  yych = *tmp___286;
  if ((int )yych <= 110) {
    if ((int )yych <= 47) {
      goto yy455;
    } else {

    }
    if ((int )yych <= 49) {
      goto yy953;
    } else {

    }
    if ((int )yych <= 109) {
      goto yy455;
    } else {

    }
    goto yy955;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy455;
    } else {

    }
    goto yy956;
  } else {
    if ((int )yych <= 115) {
      goto yy954;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy957;
    } else {

    }
    goto yy455;
  }
  yy953: 
  yyaccept = 21U;
  cursor ++;
  tmp___287 = cursor;
  s->ptr = tmp___287;
  yych = *tmp___287;
  if ((int )yych <= 113) {
    if ((int )yych == 110) {
      goto yy955;
    } else {

    }
    goto yy455;
  } else {
    if ((int )yych <= 114) {
      goto yy956;
    } else {

    }
    if ((int )yych <= 115) {
      goto yy954;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy957;
    } else {

    }
    goto yy455;
  }
  yy954: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 116) {
    goto yy454;
  } else {

  }
  goto yy57;
  yy955: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 100) {
    goto yy454;
  } else {

  }
  goto yy57;
  yy956: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 100) {
    goto yy454;
  } else {

  }
  goto yy57;
  yy957: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 104) {
    goto yy454;
  } else {

  }
  goto yy57;
  yy958: 
  cursor ++;
  yych = *cursor;
  if ((int )yych != 47) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 50) {
    if ((int )yych <= 47) {
      goto yy57;
    } else {

    }
    if ((int )yych >= 49) {
      goto yy961;
    } else {

    }
  } else {
    if ((int )yych <= 51) {
      goto yy962;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy953;
    } else {

    }
    goto yy57;
  }
  yyaccept = 21U;
  cursor ++;
  tmp___288 = cursor;
  s->ptr = tmp___288;
  yych = *tmp___288;
  if ((int )yych <= 110) {
    if ((int )yych <= 47) {
      goto yy455;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy963;
    } else {

    }
    if ((int )yych <= 109) {
      goto yy455;
    } else {

    }
    goto yy955;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy455;
    } else {

    }
    goto yy956;
  } else {
    if ((int )yych <= 115) {
      goto yy954;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy957;
    } else {

    }
    goto yy455;
  }
  yy961: 
  yyaccept = 21U;
  cursor ++;
  tmp___289 = cursor;
  s->ptr = tmp___289;
  yych = *tmp___289;
  if ((int )yych <= 110) {
    if ((int )yych <= 47) {
      goto yy455;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy963;
    } else {

    }
    if ((int )yych <= 109) {
      goto yy455;
    } else {

    }
    goto yy955;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy455;
    } else {

    }
    goto yy956;
  } else {
    if ((int )yych <= 115) {
      goto yy954;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy957;
    } else {

    }
    goto yy455;
  }
  yy962: 
  yyaccept = 21U;
  cursor ++;
  tmp___290 = cursor;
  s->ptr = tmp___290;
  yych = *tmp___290;
  if ((int )yych <= 110) {
    if ((int )yych <= 47) {
      goto yy455;
    } else {

    }
    if ((int )yych <= 49) {
      goto yy963;
    } else {

    }
    if ((int )yych <= 109) {
      goto yy455;
    } else {

    }
    goto yy955;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy455;
    } else {

    }
    goto yy956;
  } else {
    if ((int )yych <= 115) {
      goto yy954;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy957;
    } else {

    }
    goto yy455;
  }
  yy963: 
  yyaccept = 21U;
  cursor ++;
  tmp___291 = cursor;
  s->ptr = tmp___291;
  yych = *tmp___291;
  if ((int )yych <= 110) {
    if ((int )yych == 47) {
      goto yy454;
    } else {

    }
    if ((int )yych <= 109) {
      goto yy455;
    } else {

    }
    goto yy955;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy455;
    } else {

    }
    goto yy956;
  } else {
    if ((int )yych <= 115) {
      goto yy954;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy957;
    } else {

    }
    goto yy455;
  }
  yy964: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych == 65) {
      goto yy1044;
    } else {

    }
    if ((int )yych <= 84) {
      goto yy57;
    } else {

    }
    goto yy1043;
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 96) {
      goto yy57;
    } else {

    }
    goto yy1044;
  } else {
    if ((int )yych == 117) {
      goto yy1043;
    } else {

    }
    goto yy57;
  }
  yy965: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy1041;
  } else {

  }
  if ((int )yych == 101) {
    goto yy1041;
  } else {

  }
  goto yy57;
  yy966: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy1038;
  } else {

  }
  if ((int )yych == 97) {
    goto yy1038;
  } else {

  }
  goto yy57;
  yy967: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych == 80) {
      goto yy1035;
    } else {

    }
    if ((int )yych <= 84) {
      goto yy57;
    } else {

    }
    goto yy1034;
  } else
  if ((int )yych <= 112) {
    if ((int )yych <= 111) {
      goto yy57;
    } else {

    }
    goto yy1035;
  } else {
    if ((int )yych == 117) {
      goto yy1034;
    } else {

    }
    goto yy57;
  }
  yy968: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy1031;
  } else {

  }
  if ((int )yych == 101) {
    goto yy1031;
  } else {

  }
  goto yy57;
  yy969: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 67) {
    goto yy1029;
  } else {

  }
  if ((int )yych == 99) {
    goto yy1029;
  } else {

  }
  goto yy57;
  yy970: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 79) {
    goto yy1027;
  } else {

  }
  if ((int )yych == 111) {
    goto yy1027;
  } else {

  }
  goto yy57;
  yy971: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy1025;
  } else {

  }
  if ((int )yych == 101) {
    goto yy1025;
  } else {

  }
  goto yy57;
  yy972: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 48) {
    goto yy811;
  } else {

  }
  if ((int )yych <= 52) {
    goto yy812;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy813;
  } else {

  }
  goto yy57;
  yy973: 
  yyaccept = 22U;
  cursor ++;
  tmp___292 = cursor;
  s->ptr = tmp___292;
  yych = *tmp___292;
  if ((int )yych == 45) {
    goto yy977;
  } else {

  }
  if ((int )yych <= 47) {
    goto yy974;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy996;
  } else {

  }
  yy974: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  if ((s->time)->have_date) {
    add_error(s, (char *)"Double date specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_date = 1U;
  }
  (s->time)->y = timelib_get_nr(& ptr, 4);
  (s->time)->m = timelib_get_nr(& ptr, 2);
  (s->time)->d = (timelib_sll )1;
  if ((s->time)->y == -99999LL) {

  } else
  if ((s->time)->y < 100LL) {
    if ((s->time)->y < 70LL) {
      (s->time)->y += 2000LL;
    } else {
      (s->time)->y += 1900LL;
    }
  } else {

  }
  free((void *)str);
  return (267);
  yy975: 
  yyaccept = 22U;
  cursor ++;
  tmp___293 = cursor;
  s->ptr = tmp___293;
  yych = *tmp___293;
  if ((int )yych == 45) {
    goto yy977;
  } else {

  }
  if ((int )yych <= 47) {
    goto yy974;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy996;
  } else {

  }
  goto yy974;
  yy976: 
  yyaccept = 22U;
  cursor ++;
  tmp___294 = cursor;
  s->ptr = tmp___294;
  yych = *tmp___294;
  if ((int )yych != 45) {
    goto yy974;
  } else {

  }
  yy977: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy978;
  } else {

  }
  if ((int )yych <= 51) {
    goto yy979;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy980;
  } else {

  }
  goto yy57;
  yy978: 
  yyaccept = 13U;
  cursor ++;
  tmp___295 = cursor;
  s->ptr = tmp___295;
  yych = *tmp___295;
  if ((int )yych <= 109) {
    if ((int )yych <= 57) {
      if ((int )yych <= 47) {
        goto yy657;
      } else {

      }
      goto yy980;
    } else {
      if ((int )yych == 84) {
        goto yy985;
      } else {

      }
      goto yy657;
    }
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 110) {
      goto yy982;
    } else {

    }
    if ((int )yych <= 113) {
      goto yy657;
    } else {

    }
    goto yy983;
  } else {
    if ((int )yych <= 115) {
      goto yy981;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy984;
    } else {

    }
    goto yy657;
  }
  yy979: 
  yyaccept = 13U;
  cursor ++;
  tmp___296 = cursor;
  s->ptr = tmp___296;
  yych = *tmp___296;
  if ((int )yych <= 109) {
    if ((int )yych <= 49) {
      if ((int )yych <= 47) {
        goto yy657;
      } else {

      }
    } else {
      if ((int )yych == 84) {
        goto yy985;
      } else {

      }
      goto yy657;
    }
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 110) {
      goto yy982;
    } else {

    }
    if ((int )yych <= 113) {
      goto yy657;
    } else {

    }
    goto yy983;
  } else {
    if ((int )yych <= 115) {
      goto yy981;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy984;
    } else {

    }
    goto yy657;
  }
  yy980: 
  yyaccept = 13U;
  cursor ++;
  tmp___297 = cursor;
  s->ptr = tmp___297;
  yych = *tmp___297;
  if ((int )yych <= 110) {
    if ((int )yych == 84) {
      goto yy985;
    } else {

    }
    if ((int )yych <= 109) {
      goto yy657;
    } else {

    }
    goto yy982;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy657;
    } else {

    }
    goto yy983;
  } else {
    if ((int )yych <= 115) {
      goto yy981;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy984;
    } else {

    }
    goto yy657;
  }
  yy981: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 116) {
    goto yy995;
  } else {

  }
  goto yy57;
  yy982: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 100) {
    goto yy995;
  } else {

  }
  goto yy57;
  yy983: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 100) {
    goto yy995;
  } else {

  }
  goto yy57;
  yy984: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 104) {
    goto yy995;
  } else {

  }
  goto yy57;
  yy985: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 49) {
    goto yy986;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy987;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy988;
  } else {

  }
  goto yy57;
  yy986: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy988;
  } else {

  }
  if ((int )yych <= 58) {
    goto yy989;
  } else {

  }
  goto yy57;
  yy987: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 52) {
    goto yy988;
  } else {

  }
  if ((int )yych == 58) {
    goto yy989;
  } else {

  }
  goto yy57;
  yy988: 
  cursor ++;
  yych = *cursor;
  if ((int )yych != 58) {
    goto yy57;
  } else {

  }
  yy989: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy990;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy991;
  } else {

  }
  goto yy57;
  yy990: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy991;
  } else {

  }
  if ((int )yych <= 58) {
    goto yy992;
  } else {

  }
  goto yy57;
  yy991: 
  cursor ++;
  yych = *cursor;
  if ((int )yych != 58) {
    goto yy57;
  } else {

  }
  yy992: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy993;
  } else {

  }
  if ((int )yych <= 54) {
    goto yy994;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy842;
  } else {

  }
  goto yy57;
  yy993: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy843;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy842;
  } else {

  }
  goto yy843;
  yy994: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 48) {
    goto yy842;
  } else {

  }
  goto yy843;
  yy995: 
  yyaccept = 13U;
  cursor ++;
  tmp___298 = cursor;
  s->ptr = tmp___298;
  yych = *tmp___298;
  if ((int )yych == 84) {
    goto yy985;
  } else {

  }
  goto yy657;
  yy996: 
  yyaccept = 22U;
  cursor ++;
  tmp___299 = cursor;
  s->ptr = tmp___299;
  yych = *tmp___299;
  if ((int )yych != 45) {
    goto yy974;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 50) {
    if ((int )yych <= 47) {
      goto yy57;
    } else {

    }
    if ((int )yych >= 49) {
      goto yy999;
    } else {

    }
  } else {
    if ((int )yych <= 51) {
      goto yy1000;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy980;
    } else {

    }
    goto yy57;
  }
  yyaccept = 13U;
  cursor ++;
  tmp___300 = cursor;
  s->ptr = tmp___300;
  yych = *tmp___300;
  if ((int )yych <= 109) {
    if ((int )yych <= 57) {
      if ((int )yych <= 47) {
        goto yy657;
      } else {

      }
      goto yy1001;
    } else {
      if ((int )yych == 84) {
        goto yy985;
      } else {

      }
      goto yy657;
    }
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 110) {
      goto yy982;
    } else {

    }
    if ((int )yych <= 113) {
      goto yy657;
    } else {

    }
    goto yy983;
  } else {
    if ((int )yych <= 115) {
      goto yy981;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy984;
    } else {

    }
    goto yy657;
  }
  yy999: 
  yyaccept = 13U;
  cursor ++;
  tmp___301 = cursor;
  s->ptr = tmp___301;
  yych = *tmp___301;
  if ((int )yych <= 109) {
    if ((int )yych <= 57) {
      if ((int )yych <= 47) {
        goto yy657;
      } else {

      }
      goto yy1001;
    } else {
      if ((int )yych == 84) {
        goto yy985;
      } else {

      }
      goto yy657;
    }
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 110) {
      goto yy982;
    } else {

    }
    if ((int )yych <= 113) {
      goto yy657;
    } else {

    }
    goto yy983;
  } else {
    if ((int )yych <= 115) {
      goto yy981;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy984;
    } else {

    }
    goto yy657;
  }
  yy1000: 
  yyaccept = 13U;
  cursor ++;
  tmp___302 = cursor;
  s->ptr = tmp___302;
  yych = *tmp___302;
  if ((int )yych <= 109) {
    if ((int )yych <= 49) {
      if ((int )yych <= 47) {
        goto yy657;
      } else {

      }
    } else {
      if ((int )yych == 84) {
        goto yy985;
      } else {

      }
      goto yy657;
    }
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 110) {
      goto yy982;
    } else {

    }
    if ((int )yych <= 113) {
      goto yy657;
    } else {

    }
    goto yy983;
  } else {
    if ((int )yych <= 115) {
      goto yy981;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy984;
    } else {

    }
    goto yy657;
  }
  yy1001: 
  yyaccept = 21U;
  cursor ++;
  tmp___303 = cursor;
  s->ptr = tmp___303;
  yych = *tmp___303;
  if ((int )yych <= 110) {
    if ((int )yych == 84) {
      goto yy1002;
    } else {

    }
    if ((int )yych <= 109) {
      goto yy455;
    } else {

    }
    goto yy982;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy455;
    } else {

    }
    goto yy983;
  } else {
    if ((int )yych <= 115) {
      goto yy981;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy984;
    } else {

    }
    goto yy455;
  }
  yy1002: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 49) {
    goto yy1003;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy1004;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy988;
  } else {

  }
  goto yy57;
  yy1003: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy1005;
  } else {

  }
  if ((int )yych <= 58) {
    goto yy989;
  } else {

  }
  goto yy57;
  yy1004: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 52) {
    goto yy1005;
  } else {

  }
  if ((int )yych == 58) {
    goto yy989;
  } else {

  }
  goto yy57;
  yy1005: 
  cursor ++;
  yych = *cursor;
  if ((int )yych != 58) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy1007;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy991;
  } else {

  }
  goto yy57;
  yy1007: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy1008;
  } else {

  }
  if ((int )yych <= 58) {
    goto yy992;
  } else {

  }
  goto yy57;
  yy1008: 
  cursor ++;
  yych = *cursor;
  if ((int )yych != 58) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy1010;
  } else {

  }
  if ((int )yych <= 54) {
    goto yy1011;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy842;
  } else {

  }
  goto yy57;
  yy1010: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy843;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy1012;
  } else {

  }
  goto yy843;
  yy1011: 
  cursor ++;
  yych = *cursor;
  if ((int )yych != 48) {
    goto yy843;
  } else {

  }
  yy1012: 
  yyaccept = 23U;
  cursor ++;
  tmp___304 = cursor;
  s->ptr = tmp___304;
  yych = *tmp___304;
  if ((int )yych != 46) {
    goto yy843;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy57;
  } else {

  }
  yy1014: 
  yyaccept = 23U;
  cursor ++;
  s->ptr = cursor;
  if (s->lim - cursor < 9) {
    return (257);
  } else {

  }
  yych = *cursor;
  if ((int )yych <= 45) {
    if ((int )yych == 43) {
      goto yy1017;
    } else {

    }
    if ((int )yych <= 44) {
      goto yy843;
    } else {

    }
    goto yy1017;
  } else
  if ((int )yych <= 57) {
    if ((int )yych <= 47) {
      goto yy843;
    } else {

    }
    goto yy1014;
  } else
  if ((int )yych != 71) {
    goto yy843;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych == 77) {
    goto yy1023;
  } else {

  }
  goto yy57;
  yy1017: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 49) {
    goto yy1018;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy1019;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy1020;
  } else {

  }
  goto yy57;
  yy1018: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy843;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy1020;
  } else {

  }
  if ((int )yych <= 58) {
    goto yy1021;
  } else {

  }
  goto yy843;
  yy1019: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 53) {
    if ((int )yych <= 47) {
      goto yy843;
    } else {

    }
    if ((int )yych >= 53) {
      goto yy1022;
    } else {

    }
  } else {
    if ((int )yych <= 57) {
      goto yy842;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy1021;
    } else {

    }
    goto yy843;
  }
  yy1020: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy843;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy1022;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy842;
  } else {

  }
  if ((int )yych >= 59) {
    goto yy843;
  } else {

  }
  yy1021: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy843;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy1022;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy842;
  } else {

  }
  goto yy843;
  yy1022: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy843;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy842;
  } else {

  }
  goto yy843;
  yy1023: 
  cursor ++;
  yych = *cursor;
  if ((int )yych != 84) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych == 43) {
    goto yy1017;
  } else {

  }
  if ((int )yych == 45) {
    goto yy1017;
  } else {

  }
  goto yy57;
  yy1025: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 67) {
    goto yy1026;
  } else {

  }
  if ((int )yych != 99) {
    goto yy57;
  } else {

  }
  yy1026: 
  yyaccept = 20U;
  cursor ++;
  tmp___305 = cursor;
  s->ptr = tmp___305;
  yych = *tmp___305;
  if ((int )yych <= 68) {
    if ((int )yych == 45) {
      goto yy767;
    } else {

    }
    goto yy794;
  } else {
    if ((int )yych <= 69) {
      goto yy880;
    } else {

    }
    if ((int )yych == 101) {
      goto yy880;
    } else {

    }
    goto yy794;
  }
  yy1027: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 86) {
    goto yy1028;
  } else {

  }
  if ((int )yych != 118) {
    goto yy57;
  } else {

  }
  yy1028: 
  yyaccept = 20U;
  cursor ++;
  tmp___306 = cursor;
  s->ptr = tmp___306;
  yych = *tmp___306;
  if ((int )yych <= 68) {
    if ((int )yych == 45) {
      goto yy767;
    } else {

    }
    goto yy794;
  } else {
    if ((int )yych <= 69) {
      goto yy887;
    } else {

    }
    if ((int )yych == 101) {
      goto yy887;
    } else {

    }
    goto yy794;
  }
  yy1029: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 84) {
    goto yy1030;
  } else {

  }
  if ((int )yych != 116) {
    goto yy57;
  } else {

  }
  yy1030: 
  yyaccept = 20U;
  cursor ++;
  tmp___307 = cursor;
  s->ptr = tmp___307;
  yych = *tmp___307;
  if ((int )yych <= 78) {
    if ((int )yych == 45) {
      goto yy767;
    } else {

    }
    goto yy794;
  } else {
    if ((int )yych <= 79) {
      goto yy893;
    } else {

    }
    if ((int )yych == 111) {
      goto yy893;
    } else {

    }
    goto yy794;
  }
  yy1031: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 80) {
    goto yy1032;
  } else {

  }
  if ((int )yych != 112) {
    goto yy57;
  } else {

  }
  yy1032: 
  yyaccept = 20U;
  cursor ++;
  tmp___308 = cursor;
  s->ptr = tmp___308;
  yych = *tmp___308;
  if ((int )yych <= 83) {
    if ((int )yych == 45) {
      goto yy767;
    } else {

    }
    goto yy794;
  } else {
    if ((int )yych <= 84) {
      goto yy1033;
    } else {

    }
    if ((int )yych != 116) {
      goto yy794;
    } else {

    }
  }
  yy1033: 
  yyaccept = 20U;
  cursor ++;
  tmp___309 = cursor;
  s->ptr = tmp___309;
  yych = *tmp___309;
  if ((int )yych <= 68) {
    if ((int )yych == 45) {
      goto yy767;
    } else {

    }
    goto yy794;
  } else {
    if ((int )yych <= 69) {
      goto yy899;
    } else {

    }
    if ((int )yych == 101) {
      goto yy899;
    } else {

    }
    goto yy794;
  }
  yy1034: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 71) {
    goto yy1037;
  } else {

  }
  if ((int )yych == 103) {
    goto yy1037;
  } else {

  }
  goto yy57;
  yy1035: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy1036;
  } else {

  }
  if ((int )yych != 114) {
    goto yy57;
  } else {

  }
  yy1036: 
  yyaccept = 20U;
  cursor ++;
  tmp___310 = cursor;
  s->ptr = tmp___310;
  yych = *tmp___310;
  if ((int )yych <= 72) {
    if ((int )yych == 45) {
      goto yy767;
    } else {

    }
    goto yy794;
  } else {
    if ((int )yych <= 73) {
      goto yy906;
    } else {

    }
    if ((int )yych == 105) {
      goto yy906;
    } else {

    }
    goto yy794;
  }
  yy1037: 
  yyaccept = 20U;
  cursor ++;
  tmp___311 = cursor;
  s->ptr = tmp___311;
  yych = *tmp___311;
  if ((int )yych <= 84) {
    if ((int )yych == 45) {
      goto yy767;
    } else {

    }
    goto yy794;
  } else {
    if ((int )yych <= 85) {
      goto yy908;
    } else {

    }
    if ((int )yych == 117) {
      goto yy908;
    } else {

    }
    goto yy794;
  }
  yy1038: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 89) {
    if ((int )yych == 82) {
      goto yy1039;
    } else {

    }
    if ((int )yych <= 88) {
      goto yy57;
    } else {

    }
    goto yy1040;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy57;
    } else {

    }
  } else {
    if ((int )yych == 121) {
      goto yy1040;
    } else {

    }
    goto yy57;
  }
  yy1039: 
  yyaccept = 20U;
  cursor ++;
  tmp___312 = cursor;
  s->ptr = tmp___312;
  yych = *tmp___312;
  if ((int )yych <= 66) {
    if ((int )yych == 45) {
      goto yy767;
    } else {

    }
    goto yy794;
  } else {
    if ((int )yych <= 67) {
      goto yy912;
    } else {

    }
    if ((int )yych == 99) {
      goto yy912;
    } else {

    }
    goto yy794;
  }
  yy1040: 
  yyaccept = 20U;
  cursor ++;
  tmp___313 = cursor;
  s->ptr = tmp___313;
  yych = *tmp___313;
  if ((int )yych == 45) {
    goto yy767;
  } else {

  }
  goto yy794;
  yy1041: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 66) {
    goto yy1042;
  } else {

  }
  if ((int )yych != 98) {
    goto yy57;
  } else {

  }
  yy1042: 
  yyaccept = 20U;
  cursor ++;
  tmp___314 = cursor;
  s->ptr = tmp___314;
  yych = *tmp___314;
  if ((int )yych <= 81) {
    if ((int )yych == 45) {
      goto yy767;
    } else {

    }
    goto yy794;
  } else {
    if ((int )yych <= 82) {
      goto yy915;
    } else {

    }
    if ((int )yych == 114) {
      goto yy915;
    } else {

    }
    goto yy794;
  }
  yy1043: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 78) {
    if ((int )yych == 76) {
      goto yy1047;
    } else {

    }
    if ((int )yych <= 77) {
      goto yy57;
    } else {

    }
    goto yy1046;
  } else
  if ((int )yych <= 108) {
    if ((int )yych <= 107) {
      goto yy57;
    } else {

    }
    goto yy1047;
  } else {
    if ((int )yych == 110) {
      goto yy1046;
    } else {

    }
    goto yy57;
  }
  yy1044: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 78) {
    goto yy1045;
  } else {

  }
  if ((int )yych != 110) {
    goto yy57;
  } else {

  }
  yy1045: 
  yyaccept = 20U;
  cursor ++;
  tmp___315 = cursor;
  s->ptr = tmp___315;
  yych = *tmp___315;
  if ((int )yych <= 84) {
    if ((int )yych == 45) {
      goto yy767;
    } else {

    }
    goto yy794;
  } else {
    if ((int )yych <= 85) {
      goto yy922;
    } else {

    }
    if ((int )yych == 117) {
      goto yy922;
    } else {

    }
    goto yy794;
  }
  yy1046: 
  yyaccept = 20U;
  cursor ++;
  tmp___316 = cursor;
  s->ptr = tmp___316;
  yych = *tmp___316;
  if ((int )yych <= 68) {
    if ((int )yych == 45) {
      goto yy767;
    } else {

    }
    goto yy794;
  } else {
    if ((int )yych <= 69) {
      goto yy884;
    } else {

    }
    if ((int )yych == 101) {
      goto yy884;
    } else {

    }
    goto yy794;
  }
  yy1047: 
  yyaccept = 20U;
  cursor ++;
  tmp___317 = cursor;
  s->ptr = tmp___317;
  yych = *tmp___317;
  if ((int )yych <= 88) {
    if ((int )yych == 45) {
      goto yy767;
    } else {

    }
    goto yy794;
  } else {
    if ((int )yych <= 89) {
      goto yy884;
    } else {

    }
    if ((int )yych == 121) {
      goto yy884;
    } else {

    }
    goto yy794;
  }
  yy1048: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 46) {
    if ((int )yych <= 9) {
      if ((int )yych <= 8) {
        goto yy578;
      } else {

      }
      goto yy731;
    } else {
      if ((int )yych <= 44) {
        goto yy578;
      } else {

      }
      if ((int )yych <= 45) {
        goto yy732;
      } else {

      }
      goto yy731;
    }
  } else
  if ((int )yych <= 85) {
    if ((int )yych <= 47) {
      goto yy730;
    } else {

    }
    if ((int )yych <= 84) {
      goto yy578;
    } else {

    }
    goto yy78;
  } else {
    if ((int )yych == 117) {
      goto yy78;
    } else {

    }
    goto yy578;
  }
  yy1049: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 80) {
    if ((int )yych == 67) {
      goto yy129;
    } else {

    }
    if ((int )yych <= 79) {
      goto yy57;
    } else {

    }
    goto yy586;
  } else
  if ((int )yych <= 99) {
    if ((int )yych <= 98) {
      goto yy57;
    } else {

    }
    goto yy129;
  } else {
    if ((int )yych == 112) {
      goto yy586;
    } else {

    }
    goto yy57;
  }
  yy1050: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 57) {
    if ((int )yych <= 44) {
      if ((int )yych == 9) {
        goto yy1052;
      } else {

      }
      goto yy1054;
    } else {
      if ((int )yych <= 45) {
        goto yy1051;
      } else {

      }
      if ((int )yych <= 46) {
        goto yy731;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy730;
      } else {

      }
      goto yy741;
    }
  } else
  if ((int )yych <= 113) {
    if ((int )yych == 110) {
      goto yy470;
    } else {

    }
    goto yy1054;
  } else {
    if ((int )yych <= 114) {
      goto yy471;
    } else {

    }
    if ((int )yych <= 115) {
      goto yy464;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy468;
    } else {

    }
    goto yy1054;
  }
  yy1051: 
  cursor ++;
  yych = *cursor;
  switch ((int )yych) {
  case 48: 
  goto yy1055;
  case 49: 
  goto yy1056;
  case 57: 
  case 56: 
  case 55: 
  case 54: 
  case 53: 
  case 52: 
  case 51: 
  case 50: 
  goto yy618;
  case 97: 
  case 65: 
  goto yy622;
  case 100: 
  case 68: 
  goto yy626;
  case 102: 
  case 70: 
  goto yy620;
  case 106: 
  case 74: 
  goto yy619;
  case 109: 
  case 77: 
  goto yy621;
  case 110: 
  case 78: 
  goto yy625;
  case 111: 
  case 79: 
  goto yy624;
  case 115: 
  case 83: 
  goto yy623;
  default: 
  goto yy578;
  }
  yy1052: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy1054;
  } else {

  }
  if ((int )yych <= 48) {
    goto yy736;
  } else {

  }
  if ((int )yych <= 49) {
    goto yy737;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy738;
  } else {

  }
  goto yy1054;
  yy1053: 
  cursor ++;
  if (s->lim - cursor < 13) {
    return (257);
  } else {

  }
  yych = *cursor;
  yy1054: ;
  switch ((int )yych) {
  case 32: 
  case 9: 
  goto yy1053;
  case 46: 
  case 45: 
  goto yy577;
  case 97: 
  case 65: 
  goto yy574;
  case 100: 
  case 68: 
  goto yy466;
  case 102: 
  case 70: 
  goto yy467;
  case 104: 
  case 72: 
  goto yy64;
  case 73: 
  goto yy475;
  case 106: 
  case 74: 
  goto yy479;
  case 109: 
  case 77: 
  goto yy465;
  case 110: 
  case 78: 
  goto yy482;
  case 111: 
  case 79: 
  goto yy481;
  case 115: 
  case 83: 
  goto yy463;
  case 116: 
  case 84: 
  goto yy69;
  case 86: 
  goto yy477;
  case 119: 
  case 87: 
  goto yy68;
  case 88: 
  goto yy478;
  case 121: 
  case 89: 
  goto yy67;
  default: 
  goto yy57;
  }
  yy1055: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 46) {
    if ((int )yych <= 44) {
      goto yy57;
    } else {

    }
    if ((int )yych <= 45) {
      goto yy655;
    } else {

    }
    goto yy602;
  } else {
    if ((int )yych <= 47) {
      goto yy57;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy1057;
    } else {

    }
    goto yy57;
  }
  yy1056: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 46) {
    if ((int )yych <= 44) {
      goto yy57;
    } else {

    }
    if ((int )yych <= 45) {
      goto yy655;
    } else {

    }
    goto yy602;
  } else {
    if ((int )yych <= 47) {
      goto yy57;
    } else {

    }
    if ((int )yych >= 51) {
      goto yy57;
    } else {

    }
  }
  yy1057: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 44) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 45) {
    goto yy1058;
  } else {

  }
  if ((int )yych <= 46) {
    goto yy602;
  } else {

  }
  goto yy57;
  yy1058: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 50) {
    if ((int )yych <= 47) {
      goto yy57;
    } else {

    }
    if ((int )yych >= 49) {
      goto yy1060;
    } else {

    }
  } else {
    if ((int )yych <= 51) {
      goto yy1061;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy659;
    } else {

    }
    goto yy57;
  }
  yyaccept = 13U;
  cursor ++;
  tmp___318 = cursor;
  s->ptr = tmp___318;
  yych = *tmp___318;
  if ((int )yych <= 110) {
    if ((int )yych <= 47) {
      goto yy657;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy1062;
    } else {

    }
    if ((int )yych <= 109) {
      goto yy657;
    } else {

    }
    goto yy661;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy657;
    } else {

    }
    goto yy662;
  } else {
    if ((int )yych <= 115) {
      goto yy660;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy663;
    } else {

    }
    goto yy657;
  }
  yy1060: 
  yyaccept = 13U;
  cursor ++;
  tmp___319 = cursor;
  s->ptr = tmp___319;
  yych = *tmp___319;
  if ((int )yych <= 110) {
    if ((int )yych <= 47) {
      goto yy657;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy1062;
    } else {

    }
    if ((int )yych <= 109) {
      goto yy657;
    } else {

    }
    goto yy661;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy657;
    } else {

    }
    goto yy662;
  } else {
    if ((int )yych <= 115) {
      goto yy660;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy663;
    } else {

    }
    goto yy657;
  }
  yy1061: 
  yyaccept = 13U;
  cursor ++;
  tmp___320 = cursor;
  s->ptr = tmp___320;
  yych = *tmp___320;
  if ((int )yych <= 110) {
    if ((int )yych <= 49) {
      if ((int )yych <= 47) {
        goto yy657;
      } else {

      }
    } else {
      if ((int )yych <= 57) {
        goto yy604;
      } else {

      }
      if ((int )yych <= 109) {
        goto yy657;
      } else {

      }
      goto yy661;
    }
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy657;
    } else {

    }
    goto yy662;
  } else {
    if ((int )yych <= 115) {
      goto yy660;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy663;
    } else {

    }
    goto yy657;
  }
  yy1062: 
  yyaccept = 15U;
  cursor ++;
  tmp___321 = cursor;
  s->ptr = tmp___321;
  yych = *tmp___321;
  if ((int )yych <= 110) {
    if ((int )yych <= 47) {
      goto yy764;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy605;
    } else {

    }
    if ((int )yych <= 109) {
      goto yy764;
    } else {

    }
    goto yy661;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy764;
    } else {

    }
    goto yy662;
  } else {
    if ((int )yych <= 115) {
      goto yy660;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy663;
    } else {

    }
    goto yy764;
  }
  yy1063: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 57) {
    if ((int )yych <= 45) {
      if ((int )yych == 9) {
        goto yy1052;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy1054;
      } else {

      }
      goto yy1051;
    } else {
      if ((int )yych <= 46) {
        goto yy1064;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy730;
      } else {

      }
      if ((int )yych <= 53) {
        goto yy1066;
      } else {

      }
      goto yy741;
    }
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 58) {
      goto yy1065;
    } else {

    }
    if ((int )yych == 110) {
      goto yy470;
    } else {

    }
    goto yy1054;
  } else {
    if ((int )yych <= 114) {
      goto yy471;
    } else {

    }
    if ((int )yych <= 115) {
      goto yy464;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy468;
    } else {

    }
    goto yy1054;
  }
  yy1064: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 49) {
    if ((int )yych <= 47) {
      goto yy578;
    } else {

    }
    if ((int )yych <= 48) {
      goto yy1088;
    } else {

    }
    goto yy1089;
  } else {
    if ((int )yych <= 53) {
      goto yy1090;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy1091;
    } else {

    }
    goto yy578;
  }
  yy1065: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy1083;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy1084;
  } else {

  }
  goto yy57;
  yy1066: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 45) {
    goto yy785;
  } else {

  }
  if ((int )yych <= 47) {
    goto yy61;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy61;
  } else {

  }
  yyaccept = 24U;
  cursor ++;
  tmp___322 = cursor;
  s->ptr = tmp___322;
  yych = *tmp___322;
  switch ((int )yych) {
  case 121: 
  case 119: 
  case 116: 
  case 115: 
  case 111: 
  case 110: 
  case 109: 
  case 106: 
  case 104: 
  case 102: 
  case 100: 
  case 97: 
  case 89: 
  case 88: 
  case 86: 
  case 84: 
  case 83: 
  case 79: 
  case 78: 
  case 77: 
  case 74: 
  case 73: 
  case 72: 
  case 70: 
  case 68: 
  case 65: 
  case 32: 
  case 9: 
  goto yy791;
  case 45: 
  goto yy788;
  case 46: 
  goto yy792;
  case 47: 
  goto yy789;
  case 48: 
  goto yy1069;
  case 49: 
  goto yy1070;
  case 50: 
  goto yy1071;
  case 51: 
  goto yy1072;
  case 53: 
  case 52: 
  goto yy1073;
  case 54: 
  goto yy1074;
  case 57: 
  case 56: 
  case 55: 
  goto yy55;
  case 58: 
  goto yy807;
  case 87: 
  goto yy810;
  default: 
  goto yy1068;
  }
  yy1068: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  switch ((s->time)->have_time) {
  case 0U: 
  (s->time)->h = timelib_get_nr(& ptr, 2);
  (s->time)->i = timelib_get_nr(& ptr, 2);
  (s->time)->s = (timelib_sll )0;
  break;
  case 1U: 
  (s->time)->y = timelib_get_nr(& ptr, 4);
  break;
  default: 
  free((void *)str);
  add_error(s, (char *)"Double time specification");
  return (999);
  }
  ((s->time)->have_time) ++;
  free((void *)str);
  return (263);
  yy1069: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy61;
  } else {

  }
  if ((int )yych <= 48) {
    goto yy1081;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy1082;
  } else {

  }
  goto yy61;
  yy1070: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy61;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy1080;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy1079;
  } else {

  }
  goto yy61;
  yy1071: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy61;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy1079;
  } else {

  }
  goto yy61;
  yy1072: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy61;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy1077;
  } else {

  }
  if ((int )yych <= 54) {
    goto yy1078;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy1075;
  } else {

  }
  goto yy61;
  yy1073: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy61;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy1075;
  } else {

  }
  goto yy61;
  yy1074: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy61;
  } else {

  }
  if ((int )yych <= 48) {
    goto yy1075;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy55;
  } else {

  }
  goto yy61;
  yy1075: 
  yyaccept = 25U;
  cursor ++;
  tmp___323 = cursor;
  s->ptr = tmp___323;
  yych = *tmp___323;
  if ((int const   )yybm[(int )yych] & 2) {
    goto yy55;
  } else {

  }
  if ((int )yych <= 87) {
    if ((int )yych <= 70) {
      if ((int )yych <= 32) {
        if ((int )yych == 9) {
          goto yy61;
        } else {

        }
        if ((int )yych >= 32) {
          goto yy61;
        } else {

        }
      } else {
        if ((int )yych == 68) {
          goto yy61;
        } else {

        }
        if ((int )yych >= 70) {
          goto yy61;
        } else {

        }
      }
    } else
    if ((int )yych <= 77) {
      if ((int )yych == 72) {
        goto yy61;
      } else {

      }
      if ((int )yych >= 77) {
        goto yy61;
      } else {

      }
    } else {
      if ((int )yych <= 82) {
        goto yy1076;
      } else {

      }
      if ((int )yych <= 84) {
        goto yy61;
      } else {

      }
      if ((int )yych >= 87) {
        goto yy61;
      } else {

      }
    }
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 100) {
      if ((int )yych == 89) {
        goto yy61;
      } else {

      }
      if ((int )yych >= 100) {
        goto yy61;
      } else {

      }
    } else {
      if ((int )yych == 102) {
        goto yy61;
      } else {

      }
      if ((int )yych >= 104) {
        goto yy61;
      } else {

      }
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych == 109) {
      goto yy61;
    } else {

    }
    if ((int )yych >= 115) {
      goto yy61;
    } else {

    }
  } else
  if ((int )yych <= 119) {
    if ((int )yych >= 119) {
      goto yy61;
    } else {

    }
  } else
  if ((int )yych == 121) {
    goto yy61;
  } else {

  }
  yy1076: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  if ((s->time)->have_time) {
    add_error(s, (char *)"Double time specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_time = 1U;
    (s->time)->h = (timelib_sll )0;
    (s->time)->i = (timelib_sll )0;
    (s->time)->s = (timelib_sll )0;
    (s->time)->f = (double )0;
  }
  (s->time)->h = timelib_get_nr(& ptr, 2);
  (s->time)->i = timelib_get_nr(& ptr, 2);
  (s->time)->s = timelib_get_nr(& ptr, 2);
  if ((int )*ptr != 0) {
    tmp___324 = timelib_get_zone(& ptr, & (s->time)->dst, s->time,
                                 & tz_not_found___4, s->tzdb);
    (s->time)->z = (int )tmp___324;
    if (tz_not_found___4) {
      add_error(s, (char *)"The timezone could not be found in the database");
    } else {

    }
  } else {

  }
  free((void *)str);
  return (265);
  yy1077: 
  yyaccept = 25U;
  cursor ++;
  tmp___325 = cursor;
  s->ptr = tmp___325;
  yych = *tmp___325;
  if ((int )yych <= 86) {
    if ((int )yych <= 68) {
      if ((int )yych <= 32) {
        if ((int )yych == 9) {
          goto yy61;
        } else {

        }
        if ((int )yych <= 31) {
          goto yy1076;
        } else {

        }
        goto yy61;
      } else {
        if ((int )yych <= 47) {
          goto yy1076;
        } else {

        }
        if ((int )yych <= 57) {
          goto yy821;
        } else {

        }
        if ((int )yych <= 67) {
          goto yy1076;
        } else {

        }
        goto yy61;
      }
    } else
    if ((int )yych <= 72) {
      if ((int )yych == 70) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 71) {
        goto yy1076;
      } else {

      }
      goto yy61;
    } else
    if ((int )yych <= 77) {
      if ((int )yych <= 76) {
        goto yy1076;
      } else {

      }
      goto yy61;
    } else {
      if ((int )yych <= 82) {
        goto yy1076;
      } else {

      }
      if ((int )yych <= 84) {
        goto yy61;
      } else {

      }
      goto yy1076;
    }
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 99) {
      if ((int )yych == 88) {
        goto yy1076;
      } else {

      }
      if ((int )yych <= 89) {
        goto yy61;
      } else {

      }
      goto yy1076;
    } else
    if ((int )yych <= 101) {
      if ((int )yych <= 100) {
        goto yy61;
      } else {

      }
      goto yy1076;
    } else {
      if ((int )yych == 103) {
        goto yy1076;
      } else {

      }
      goto yy61;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych == 109) {
      goto yy61;
    } else {

    }
    if ((int )yych <= 114) {
      goto yy1076;
    } else {

    }
    goto yy61;
  } else
  if ((int )yych <= 119) {
    if ((int )yych <= 118) {
      goto yy1076;
    } else {

    }
    goto yy61;
  } else {
    if ((int )yych == 121) {
      goto yy61;
    } else {

    }
    goto yy1076;
  }
  yy1078: 
  yyaccept = 25U;
  cursor ++;
  tmp___326 = cursor;
  s->ptr = tmp___326;
  yych = *tmp___326;
  if ((int )yych <= 86) {
    if ((int )yych <= 68) {
      if ((int )yych <= 32) {
        if ((int )yych == 9) {
          goto yy61;
        } else {

        }
        if ((int )yych <= 31) {
          goto yy1076;
        } else {

        }
        goto yy61;
      } else
      if ((int )yych <= 54) {
        if ((int )yych <= 47) {
          goto yy1076;
        } else {

        }
        goto yy821;
      } else {
        if ((int )yych <= 57) {
          goto yy55;
        } else {

        }
        if ((int )yych <= 67) {
          goto yy1076;
        } else {

        }
        goto yy61;
      }
    } else
    if ((int )yych <= 72) {
      if ((int )yych == 70) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 71) {
        goto yy1076;
      } else {

      }
      goto yy61;
    } else
    if ((int )yych <= 77) {
      if ((int )yych <= 76) {
        goto yy1076;
      } else {

      }
      goto yy61;
    } else {
      if ((int )yych <= 82) {
        goto yy1076;
      } else {

      }
      if ((int )yych <= 84) {
        goto yy61;
      } else {

      }
      goto yy1076;
    }
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 99) {
      if ((int )yych == 88) {
        goto yy1076;
      } else {

      }
      if ((int )yych <= 89) {
        goto yy61;
      } else {

      }
      goto yy1076;
    } else
    if ((int )yych <= 101) {
      if ((int )yych <= 100) {
        goto yy61;
      } else {

      }
      goto yy1076;
    } else {
      if ((int )yych == 103) {
        goto yy1076;
      } else {

      }
      goto yy61;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych == 109) {
      goto yy61;
    } else {

    }
    if ((int )yych <= 114) {
      goto yy1076;
    } else {

    }
    goto yy61;
  } else
  if ((int )yych <= 119) {
    if ((int )yych <= 118) {
      goto yy1076;
    } else {

    }
    goto yy61;
  } else {
    if ((int )yych == 121) {
      goto yy61;
    } else {

    }
    goto yy1076;
  }
  yy1079: 
  yyaccept = 25U;
  cursor ++;
  tmp___327 = cursor;
  s->ptr = tmp___327;
  yych = *tmp___327;
  if ((int )yych <= 86) {
    if ((int )yych <= 68) {
      if ((int )yych <= 32) {
        if ((int )yych == 9) {
          goto yy61;
        } else {

        }
        if ((int )yych <= 31) {
          goto yy1076;
        } else {

        }
        goto yy61;
      } else {
        if ((int )yych <= 47) {
          goto yy1076;
        } else {

        }
        if ((int )yych <= 57) {
          goto yy821;
        } else {

        }
        if ((int )yych <= 67) {
          goto yy1076;
        } else {

        }
        goto yy61;
      }
    } else
    if ((int )yych <= 72) {
      if ((int )yych == 70) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 71) {
        goto yy1076;
      } else {

      }
      goto yy61;
    } else
    if ((int )yych <= 77) {
      if ((int )yych <= 76) {
        goto yy1076;
      } else {

      }
      goto yy61;
    } else {
      if ((int )yych <= 82) {
        goto yy1076;
      } else {

      }
      if ((int )yych <= 84) {
        goto yy61;
      } else {

      }
      goto yy1076;
    }
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 99) {
      if ((int )yych == 88) {
        goto yy1076;
      } else {

      }
      if ((int )yych <= 89) {
        goto yy61;
      } else {

      }
      goto yy1076;
    } else
    if ((int )yych <= 101) {
      if ((int )yych <= 100) {
        goto yy61;
      } else {

      }
      goto yy1076;
    } else {
      if ((int )yych == 103) {
        goto yy1076;
      } else {

      }
      goto yy61;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych == 109) {
      goto yy61;
    } else {

    }
    if ((int )yych <= 114) {
      goto yy1076;
    } else {

    }
    goto yy61;
  } else
  if ((int )yych <= 119) {
    if ((int )yych <= 118) {
      goto yy1076;
    } else {

    }
    goto yy61;
  } else {
    if ((int )yych == 121) {
      goto yy61;
    } else {

    }
    goto yy1076;
  }
  yy1080: 
  yyaccept = 25U;
  cursor ++;
  tmp___328 = cursor;
  s->ptr = tmp___328;
  yych = *tmp___328;
  if ((int )yych <= 84) {
    if ((int )yych <= 57) {
      if ((int )yych <= 32) {
        if ((int )yych == 9) {
          goto yy61;
        } else {

        }
        if ((int )yych <= 31) {
          goto yy1076;
        } else {

        }
        goto yy61;
      } else
      if ((int )yych <= 48) {
        if ((int )yych <= 47) {
          goto yy1076;
        } else {

        }
        goto yy845;
      } else {
        if ((int )yych <= 50) {
          goto yy846;
        } else {

        }
        if ((int )yych <= 51) {
          goto yy847;
        } else {

        }
        goto yy821;
      }
    } else
    if ((int )yych <= 71) {
      if ((int )yych <= 68) {
        if ((int )yych <= 67) {
          goto yy1076;
        } else {

        }
        goto yy61;
      } else {
        if ((int )yych == 70) {
          goto yy61;
        } else {

        }
        goto yy1076;
      }
    } else
    if ((int )yych <= 76) {
      if ((int )yych <= 72) {
        goto yy61;
      } else {

      }
      goto yy1076;
    } else {
      if ((int )yych <= 77) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 82) {
        goto yy1076;
      } else {

      }
      goto yy61;
    }
  } else
  if ((int )yych <= 103) {
    if ((int )yych <= 89) {
      if ((int )yych == 87) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 88) {
        goto yy1076;
      } else {

      }
      goto yy61;
    } else
    if ((int )yych <= 100) {
      if ((int )yych <= 99) {
        goto yy1076;
      } else {

      }
      goto yy61;
    } else {
      if ((int )yych == 102) {
        goto yy61;
      } else {

      }
      goto yy1076;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych <= 108) {
      if ((int )yych <= 104) {
        goto yy61;
      } else {

      }
      goto yy1076;
    } else {
      if ((int )yych <= 109) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 114) {
        goto yy1076;
      } else {

      }
      goto yy61;
    }
  } else
  if ((int )yych <= 119) {
    if ((int )yych <= 118) {
      goto yy1076;
    } else {

    }
    goto yy61;
  } else {
    if ((int )yych == 121) {
      goto yy61;
    } else {

    }
    goto yy1076;
  }
  yy1081: 
  yyaccept = 25U;
  cursor ++;
  tmp___329 = cursor;
  s->ptr = tmp___329;
  yych = *tmp___329;
  if ((int )yych <= 84) {
    if ((int )yych <= 57) {
      if ((int )yych <= 32) {
        if ((int )yych == 9) {
          goto yy61;
        } else {

        }
        if ((int )yych <= 31) {
          goto yy1076;
        } else {

        }
        goto yy61;
      } else
      if ((int )yych <= 48) {
        if ((int )yych <= 47) {
          goto yy1076;
        } else {

        }
        goto yy877;
      } else {
        if ((int )yych <= 50) {
          goto yy846;
        } else {

        }
        if ((int )yych <= 51) {
          goto yy847;
        } else {

        }
        goto yy821;
      }
    } else
    if ((int )yych <= 71) {
      if ((int )yych <= 68) {
        if ((int )yych <= 67) {
          goto yy1076;
        } else {

        }
        goto yy61;
      } else {
        if ((int )yych == 70) {
          goto yy61;
        } else {

        }
        goto yy1076;
      }
    } else
    if ((int )yych <= 76) {
      if ((int )yych <= 72) {
        goto yy61;
      } else {

      }
      goto yy1076;
    } else {
      if ((int )yych <= 77) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 82) {
        goto yy1076;
      } else {

      }
      goto yy61;
    }
  } else
  if ((int )yych <= 103) {
    if ((int )yych <= 89) {
      if ((int )yych == 87) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 88) {
        goto yy1076;
      } else {

      }
      goto yy61;
    } else
    if ((int )yych <= 100) {
      if ((int )yych <= 99) {
        goto yy1076;
      } else {

      }
      goto yy61;
    } else {
      if ((int )yych == 102) {
        goto yy61;
      } else {

      }
      goto yy1076;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych <= 108) {
      if ((int )yych <= 104) {
        goto yy61;
      } else {

      }
      goto yy1076;
    } else {
      if ((int )yych <= 109) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 114) {
        goto yy1076;
      } else {

      }
      goto yy61;
    }
  } else
  if ((int )yych <= 119) {
    if ((int )yych <= 118) {
      goto yy1076;
    } else {

    }
    goto yy61;
  } else {
    if ((int )yych == 121) {
      goto yy61;
    } else {

    }
    goto yy1076;
  }
  yy1082: 
  yyaccept = 25U;
  cursor ++;
  tmp___330 = cursor;
  s->ptr = tmp___330;
  yych = *tmp___330;
  if ((int )yych <= 84) {
    if ((int )yych <= 57) {
      if ((int )yych <= 32) {
        if ((int )yych == 9) {
          goto yy61;
        } else {

        }
        if ((int )yych <= 31) {
          goto yy1076;
        } else {

        }
        goto yy61;
      } else
      if ((int )yych <= 48) {
        if ((int )yych <= 47) {
          goto yy1076;
        } else {

        }
        goto yy845;
      } else {
        if ((int )yych <= 50) {
          goto yy846;
        } else {

        }
        if ((int )yych <= 51) {
          goto yy847;
        } else {

        }
        goto yy821;
      }
    } else
    if ((int )yych <= 71) {
      if ((int )yych <= 68) {
        if ((int )yych <= 67) {
          goto yy1076;
        } else {

        }
        goto yy61;
      } else {
        if ((int )yych == 70) {
          goto yy61;
        } else {

        }
        goto yy1076;
      }
    } else
    if ((int )yych <= 76) {
      if ((int )yych <= 72) {
        goto yy61;
      } else {

      }
      goto yy1076;
    } else {
      if ((int )yych <= 77) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 82) {
        goto yy1076;
      } else {

      }
      goto yy61;
    }
  } else
  if ((int )yych <= 103) {
    if ((int )yych <= 89) {
      if ((int )yych == 87) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 88) {
        goto yy1076;
      } else {

      }
      goto yy61;
    } else
    if ((int )yych <= 100) {
      if ((int )yych <= 99) {
        goto yy1076;
      } else {

      }
      goto yy61;
    } else {
      if ((int )yych == 102) {
        goto yy61;
      } else {

      }
      goto yy1076;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych <= 108) {
      if ((int )yych <= 104) {
        goto yy61;
      } else {

      }
      goto yy1076;
    } else {
      if ((int )yych <= 109) {
        goto yy61;
      } else {

      }
      if ((int )yych <= 114) {
        goto yy1076;
      } else {

      }
      goto yy61;
    }
  } else
  if ((int )yych <= 119) {
    if ((int )yych <= 118) {
      goto yy1076;
    } else {

    }
    goto yy61;
  } else {
    if ((int )yych == 121) {
      goto yy61;
    } else {

    }
    goto yy1076;
  }
  yy1083: 
  yyaccept = 11U;
  cursor ++;
  tmp___331 = cursor;
  s->ptr = tmp___331;
  yych = *tmp___331;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy1085;
    } else {

    }
    goto yy491;
  } else {
    if ((int )yych <= 57) {
      goto yy1084;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy1085;
    } else {

    }
    goto yy491;
  }
  yy1084: 
  yyaccept = 11U;
  cursor ++;
  tmp___332 = cursor;
  s->ptr = tmp___332;
  yych = *tmp___332;
  if ((int )yych == 46) {
    goto yy1085;
  } else {

  }
  if ((int )yych != 58) {
    goto yy491;
  } else {

  }
  yy1085: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy1086;
  } else {

  }
  if ((int )yych <= 54) {
    goto yy1087;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy496;
  } else {

  }
  goto yy57;
  yy1086: 
  yyaccept = 11U;
  cursor ++;
  tmp___333 = cursor;
  s->ptr = tmp___333;
  yych = *tmp___333;
  if ((int )yych == 46) {
    goto yy497;
  } else {

  }
  if ((int )yych <= 47) {
    goto yy491;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy496;
  } else {

  }
  goto yy491;
  yy1087: 
  yyaccept = 11U;
  cursor ++;
  tmp___334 = cursor;
  s->ptr = tmp___334;
  yych = *tmp___334;
  if ((int )yych == 46) {
    goto yy497;
  } else {

  }
  if ((int )yych == 48) {
    goto yy496;
  } else {

  }
  goto yy491;
  yy1088: 
  yyaccept = 11U;
  cursor ++;
  tmp___335 = cursor;
  s->ptr = tmp___335;
  yych = *tmp___335;
  if ((int )yych <= 46) {
    if ((int )yych <= 44) {
      goto yy491;
    } else {

    }
    if ((int )yych <= 45) {
      goto yy602;
    } else {

    }
    goto yy1092;
  } else {
    if ((int )yych <= 47) {
      goto yy491;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy1091;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy1085;
    } else {

    }
    goto yy491;
  }
  yy1089: 
  yyaccept = 11U;
  cursor ++;
  tmp___336 = cursor;
  s->ptr = tmp___336;
  yych = *tmp___336;
  if ((int )yych <= 47) {
    if ((int )yych <= 44) {
      goto yy491;
    } else {

    }
    if ((int )yych <= 45) {
      goto yy602;
    } else {

    }
    if ((int )yych <= 46) {
      goto yy1092;
    } else {

    }
    goto yy491;
  } else {
    if ((int )yych <= 50) {
      goto yy1091;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy1084;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy1085;
    } else {

    }
    goto yy491;
  }
  yy1090: 
  yyaccept = 11U;
  cursor ++;
  tmp___337 = cursor;
  s->ptr = tmp___337;
  yych = *tmp___337;
  if ((int )yych <= 46) {
    if ((int )yych <= 44) {
      goto yy491;
    } else {

    }
    if ((int )yych <= 45) {
      goto yy602;
    } else {

    }
    goto yy1092;
  } else {
    if ((int )yych <= 47) {
      goto yy491;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy1084;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy1085;
    } else {

    }
    goto yy491;
  }
  yy1091: 
  yyaccept = 11U;
  cursor ++;
  tmp___338 = cursor;
  s->ptr = tmp___338;
  yych = *tmp___338;
  if ((int )yych <= 46) {
    if ((int )yych <= 44) {
      goto yy491;
    } else {

    }
    if ((int )yych <= 45) {
      goto yy602;
    } else {

    }
  } else {
    if ((int )yych == 58) {
      goto yy1085;
    } else {

    }
    goto yy491;
  }
  yy1092: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy1093;
  } else {

  }
  if ((int )yych <= 54) {
    goto yy1094;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy610;
  } else {

  }
  goto yy57;
  yy1093: 
  yyaccept = 11U;
  cursor ++;
  tmp___339 = cursor;
  s->ptr = tmp___339;
  yych = *tmp___339;
  if ((int )yych == 46) {
    goto yy497;
  } else {

  }
  if ((int )yych <= 47) {
    goto yy491;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy1095;
  } else {

  }
  goto yy491;
  yy1094: 
  yyaccept = 11U;
  cursor ++;
  tmp___340 = cursor;
  s->ptr = tmp___340;
  yych = *tmp___340;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy497;
    } else {

    }
    goto yy491;
  } else {
    if ((int )yych <= 48) {
      goto yy1095;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy611;
    } else {

    }
    goto yy491;
  }
  yy1095: 
  yyaccept = 11U;
  cursor ++;
  tmp___341 = cursor;
  s->ptr = tmp___341;
  yych = *tmp___341;
  if ((int )yych == 46) {
    goto yy497;
  } else {

  }
  if ((int )yych <= 47) {
    goto yy491;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy605;
  } else {

  }
  goto yy491;
  yy1096: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 57) {
    if ((int )yych <= 45) {
      if ((int )yych == 9) {
        goto yy460;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy462;
      } else {

      }
      goto yy1051;
    } else {
      if ((int )yych <= 46) {
        goto yy474;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy472;
      } else {

      }
      if ((int )yych <= 53) {
        goto yy1066;
      } else {

      }
      goto yy741;
    }
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 58) {
      goto yy483;
    } else {

    }
    if ((int )yych == 110) {
      goto yy470;
    } else {

    }
    goto yy462;
  } else {
    if ((int )yych <= 114) {
      goto yy471;
    } else {

    }
    if ((int )yych <= 115) {
      goto yy464;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy468;
    } else {

    }
    goto yy462;
  }
  yy1097: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 57) {
    if ((int )yych <= 45) {
      if ((int )yych == 9) {
        goto yy1052;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy1054;
      } else {

      }
      goto yy1051;
    } else {
      if ((int )yych <= 46) {
        goto yy1064;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy472;
      } else {

      }
      if ((int )yych <= 53) {
        goto yy1066;
      } else {

      }
      goto yy741;
    }
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 58) {
      goto yy1065;
    } else {

    }
    if ((int )yych == 110) {
      goto yy470;
    } else {

    }
    goto yy1054;
  } else {
    if ((int )yych <= 114) {
      goto yy471;
    } else {

    }
    if ((int )yych <= 115) {
      goto yy464;
    } else {

    }
    if ((int )yych <= 116) {
      goto yy468;
    } else {

    }
    goto yy1054;
  }
  yy1098: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 69) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 68) {
        goto yy142;
      } else {

      }
    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else {
    if ((int )yych <= 101) {
      goto yy1099;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy1099: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 86) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 85) {
        goto yy143;
      } else {

      }
    }
  } else
  if ((int )yych <= 117) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 118) {
      goto yy1100;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy4;
  }
  yy1100: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 73) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 72) {
        goto yy144;
      } else {

      }
    }
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 105) {
      goto yy1101;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1101: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 79) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 78) {
        goto yy145;
      } else {

      }
    }
  } else
  if ((int )yych <= 110) {
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy145;
  } else {
    if ((int )yych <= 111) {
      goto yy1102;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy4;
  }
  yy1102: 
  yyaccept = 0U;
  cursor ++;
  tmp___342 = cursor;
  s->ptr = tmp___342;
  yych = *tmp___342;
  if ((int )yych <= 84) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 85) {
      goto yy1103;
    } else {

    }
    if ((int )yych != 117) {
      goto yy4;
    } else {

    }
  }
  yy1103: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 83) {
    goto yy1104;
  } else {

  }
  if ((int )yych != 115) {
    goto yy57;
  } else {

  }
  yy1104: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 9) {
    goto yy1105;
  } else {

  }
  if ((int )yych != 32) {
    goto yy57;
  } else {

  }
  yy1105: 
  cursor ++;
  if (s->lim - cursor < 11) {
    return (257);
  } else {

  }
  yych = *cursor;
  yy1106: ;
  if ((int )yych <= 87) {
    if ((int )yych <= 70) {
      if ((int )yych <= 32) {
        if ((int )yych == 9) {
          goto yy1105;
        } else {

        }
        if ((int )yych <= 31) {
          goto yy57;
        } else {

        }
        goto yy1105;
      } else {
        if ((int )yych == 68) {
          goto yy1110;
        } else {

        }
        if ((int )yych <= 69) {
          goto yy57;
        } else {

        }
        goto yy1111;
      }
    } else
    if ((int )yych <= 77) {
      if ((int )yych == 72) {
        goto yy1109;
      } else {

      }
      if ((int )yych <= 76) {
        goto yy57;
      } else {

      }
      goto yy1108;
    } else
    if ((int )yych <= 83) {
      if ((int )yych <= 82) {
        goto yy57;
      } else {

      }
    } else {
      if ((int )yych <= 84) {
        goto yy1114;
      } else {

      }
      if ((int )yych <= 86) {
        goto yy57;
      } else {

      }
      goto yy1113;
    }
  } else
  if ((int )yych <= 108) {
    if ((int )yych <= 100) {
      if ((int )yych == 89) {
        goto yy1112;
      } else {

      }
      if ((int )yych <= 99) {
        goto yy57;
      } else {

      }
      goto yy1110;
    } else
    if ((int )yych <= 102) {
      if ((int )yych <= 101) {
        goto yy57;
      } else {

      }
      goto yy1111;
    } else {
      if ((int )yych == 104) {
        goto yy1109;
      } else {

      }
      goto yy57;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych <= 109) {
      goto yy1108;
    } else {

    }
    if ((int )yych <= 114) {
      goto yy57;
    } else {

    }
    if ((int )yych >= 116) {
      goto yy1114;
    } else {

    }
  } else
  if ((int )yych <= 119) {
    if ((int )yych <= 118) {
      goto yy57;
    } else {

    }
    goto yy1113;
  } else {
    if ((int )yych == 121) {
      goto yy1112;
    } else {

    }
    goto yy57;
  }
  yy1107: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych <= 68) {
      if ((int )yych == 65) {
        goto yy1179;
      } else {

      }
      goto yy57;
    } else {
      if ((int )yych <= 69) {
        goto yy1180;
      } else {

      }
      if ((int )yych <= 84) {
        goto yy57;
      } else {

      }
      goto yy1178;
    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych == 97) {
      goto yy1179;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 101) {
      goto yy1180;
    } else {

    }
    if ((int )yych == 117) {
      goto yy1178;
    } else {

    }
    goto yy57;
  }
  yy1108: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 79) {
    if ((int )yych == 73) {
      goto yy1170;
    } else {

    }
    if ((int )yych <= 78) {
      goto yy57;
    } else {

    }
    goto yy1169;
  } else
  if ((int )yych <= 105) {
    if ((int )yych <= 104) {
      goto yy57;
    } else {

    }
    goto yy1170;
  } else {
    if ((int )yych == 111) {
      goto yy1169;
    } else {

    }
    goto yy57;
  }
  yy1109: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 79) {
    goto yy1167;
  } else {

  }
  if ((int )yych == 111) {
    goto yy1167;
  } else {

  }
  goto yy57;
  yy1110: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy1166;
  } else {

  }
  if ((int )yych == 97) {
    goto yy1166;
  } else {

  }
  goto yy57;
  yy1111: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 82) {
    if ((int )yych == 79) {
      goto yy1151;
    } else {

    }
    if ((int )yych <= 81) {
      goto yy57;
    } else {

    }
    goto yy1150;
  } else
  if ((int )yych <= 111) {
    if ((int )yych <= 110) {
      goto yy57;
    } else {

    }
    goto yy1151;
  } else {
    if ((int )yych == 114) {
      goto yy1150;
    } else {

    }
    goto yy57;
  }
  yy1112: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy1147;
  } else {

  }
  if ((int )yych == 101) {
    goto yy1147;
  } else {

  }
  goto yy57;
  yy1113: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy1133;
  } else {

  }
  if ((int )yych == 101) {
    goto yy1133;
  } else {

  }
  goto yy57;
  yy1114: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych == 72) {
      goto yy1115;
    } else {

    }
    if ((int )yych <= 84) {
      goto yy57;
    } else {

    }
    goto yy1116;
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 103) {
      goto yy57;
    } else {

    }
  } else {
    if ((int )yych == 117) {
      goto yy1116;
    } else {

    }
    goto yy57;
  }
  yy1115: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 85) {
    goto yy1128;
  } else {

  }
  if ((int )yych == 117) {
    goto yy1128;
  } else {

  }
  goto yy57;
  yy1116: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy1117;
  } else {

  }
  if ((int )yych != 101) {
    goto yy57;
  } else {

  }
  yy1117: 
  yyaccept = 26U;
  cursor ++;
  tmp___343 = cursor;
  s->ptr = tmp___343;
  yych = *tmp___343;
  if ((int )yych <= 32) {
    if ((int )yych == 9) {
      goto yy1119;
    } else {

    }
    if ((int )yych >= 32) {
      goto yy1119;
    } else {

    }
  } else
  if ((int )yych <= 83) {
    if ((int )yych >= 83) {
      goto yy1121;
    } else {

    }
  } else
  if ((int )yych == 115) {
    goto yy1121;
  } else {

  }
  yy1118: 
  behavior = 0;
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  (s->time)->have_relative = 1U;
  while (*ptr) {
    i___0 = timelib_get_relative_text(& ptr, & behavior);
    timelib_eat_spaces(& ptr);
    timelib_set_relative(& ptr, i___0, behavior, s);
  }
  free((void *)str);
  return (310);
  yy1119: 
  cursor ++;
  if (s->lim - cursor < 2) {
    return (257);
  } else {

  }
  yych = *cursor;
  if ((int )yych <= 32) {
    if ((int )yych == 9) {
      goto yy1119;
    } else {

    }
    if ((int )yych <= 31) {
      goto yy57;
    } else {

    }
    goto yy1119;
  } else
  if ((int )yych <= 79) {
    if ((int )yych <= 78) {
      goto yy57;
    } else {

    }
    goto yy1125;
  } else {
    if ((int )yych == 111) {
      goto yy1125;
    } else {

    }
    goto yy57;
  }
  yy1121: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 68) {
    goto yy1122;
  } else {

  }
  if ((int )yych != 100) {
    goto yy57;
  } else {

  }
  yy1122: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy1123;
  } else {

  }
  if ((int )yych != 97) {
    goto yy57;
  } else {

  }
  yy1123: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy1124;
  } else {

  }
  if ((int )yych != 121) {
    goto yy57;
  } else {

  }
  yy1124: 
  yyaccept = 26U;
  cursor ++;
  tmp___344 = cursor;
  s->ptr = tmp___344;
  yych = *tmp___344;
  if ((int )yych == 9) {
    goto yy1119;
  } else {

  }
  if ((int )yych == 32) {
    goto yy1119;
  } else {

  }
  goto yy1118;
  yy1125: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 70) {
    goto yy1126;
  } else {

  }
  if ((int )yych != 102) {
    goto yy57;
  } else {

  }
  yy1126: 
  cursor ++;
  behavior___0 = 0;
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  (s->time)->have_relative = 1U;
  (s->time)->have_relative = 1U;
  (s->time)->relative.have_special_relative = 1U;
  i___1 = timelib_get_relative_text(& ptr, & behavior___0);
  timelib_eat_spaces(& ptr);
  if (i___1 > 0LL) {
    (s->time)->relative.special.type = 2U;
    timelib_set_relative(& ptr, i___1, 1, s);
  } else {
    (s->time)->relative.special.type = 3U;
    timelib_set_relative(& ptr, i___1, behavior___0, s);
  }
  free((void *)str);
  return (281);
  yy1128: 
  yyaccept = 26U;
  cursor ++;
  tmp___345 = cursor;
  s->ptr = tmp___345;
  yych = *tmp___345;
  if ((int )yych <= 32) {
    if ((int )yych == 9) {
      goto yy1119;
    } else {

    }
    if ((int )yych <= 31) {
      goto yy1118;
    } else {

    }
    goto yy1119;
  } else
  if ((int )yych <= 82) {
    if ((int )yych <= 81) {
      goto yy1118;
    } else {

    }
  } else
  if ((int )yych != 114) {
    goto yy1118;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych == 83) {
    goto yy1130;
  } else {

  }
  if ((int )yych != 115) {
    goto yy57;
  } else {

  }
  yy1130: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 68) {
    goto yy1131;
  } else {

  }
  if ((int )yych != 100) {
    goto yy57;
  } else {

  }
  yy1131: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy1132;
  } else {

  }
  if ((int )yych != 97) {
    goto yy57;
  } else {

  }
  yy1132: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy1124;
  } else {

  }
  if ((int )yych == 121) {
    goto yy1124;
  } else {

  }
  goto yy57;
  yy1133: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 69) {
    if ((int )yych <= 67) {
      goto yy57;
    } else {

    }
    if ((int )yych <= 68) {
      goto yy1135;
    } else {

    }
  } else {
    if ((int )yych <= 99) {
      goto yy57;
    } else {

    }
    if ((int )yych <= 100) {
      goto yy1135;
    } else {

    }
    if ((int )yych >= 102) {
      goto yy57;
    } else {

    }
  }
  cursor ++;
  yych = *cursor;
  if ((int )yych == 75) {
    goto yy1141;
  } else {

  }
  if ((int )yych == 107) {
    goto yy1141;
  } else {

  }
  goto yy57;
  yy1135: 
  yyaccept = 26U;
  cursor ++;
  tmp___346 = cursor;
  s->ptr = tmp___346;
  yych = *tmp___346;
  if ((int )yych <= 32) {
    if ((int )yych == 9) {
      goto yy1119;
    } else {

    }
    if ((int )yych <= 31) {
      goto yy1118;
    } else {

    }
    goto yy1119;
  } else
  if ((int )yych <= 78) {
    if ((int )yych <= 77) {
      goto yy1118;
    } else {

    }
  } else
  if ((int )yych != 110) {
    goto yy1118;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy1137;
  } else {

  }
  if ((int )yych != 101) {
    goto yy57;
  } else {

  }
  yy1137: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 83) {
    goto yy1138;
  } else {

  }
  if ((int )yych != 115) {
    goto yy57;
  } else {

  }
  yy1138: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 68) {
    goto yy1139;
  } else {

  }
  if ((int )yych != 100) {
    goto yy57;
  } else {

  }
  yy1139: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy1140;
  } else {

  }
  if ((int )yych != 97) {
    goto yy57;
  } else {

  }
  yy1140: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy1124;
  } else {

  }
  if ((int )yych == 121) {
    goto yy1124;
  } else {

  }
  goto yy57;
  yy1141: 
  yyaccept = 27U;
  cursor ++;
  tmp___347 = cursor;
  s->ptr = tmp___347;
  yych = *tmp___347;
  if ((int )yych <= 83) {
    if ((int )yych == 68) {
      goto yy1144;
    } else {

    }
    if ((int )yych >= 83) {
      goto yy1143;
    } else {

    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych >= 100) {
      goto yy1144;
    } else {

    }
  } else
  if ((int )yych == 115) {
    goto yy1143;
  } else {

  }
  yy1142: 
  behavior___1 = 0;
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  (s->time)->have_relative = 1U;
  while (*ptr) {
    i___2 = timelib_get_relative_text(& ptr, & behavior___1);
    timelib_eat_spaces(& ptr);
    timelib_set_relative(& ptr, i___2, behavior___1, s);
    (s->time)->relative.weekday_behavior = 2;
    if ((s->time)->relative.have_weekday_relative == 0U) {
      (s->time)->have_relative = 1U;
      (s->time)->relative.have_weekday_relative = 1U;
      (s->time)->relative.weekday = 1;
    } else {

    }
  }
  free((void *)str);
  return (310);
  yy1143: 
  cursor ++;
  yych = *cursor;
  goto yy1118;
  yy1144: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy1145;
  } else {

  }
  if ((int )yych != 97) {
    goto yy57;
  } else {

  }
  yy1145: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy1146;
  } else {

  }
  if ((int )yych != 121) {
    goto yy57;
  } else {

  }
  yy1146: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 83) {
    goto yy1143;
  } else {

  }
  if ((int )yych == 115) {
    goto yy1143;
  } else {

  }
  goto yy1118;
  yy1147: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy1148;
  } else {

  }
  if ((int )yych != 97) {
    goto yy57;
  } else {

  }
  yy1148: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy1149;
  } else {

  }
  if ((int )yych != 114) {
    goto yy57;
  } else {

  }
  yy1149: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 83) {
    goto yy1143;
  } else {

  }
  if ((int )yych == 115) {
    goto yy1143;
  } else {

  }
  goto yy1118;
  yy1150: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 73) {
    goto yy1163;
  } else {

  }
  if ((int )yych == 105) {
    goto yy1163;
  } else {

  }
  goto yy57;
  yy1151: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy1152;
  } else {

  }
  if ((int )yych != 114) {
    goto yy57;
  } else {

  }
  yy1152: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 84) {
    goto yy1153;
  } else {

  }
  if ((int )yych != 116) {
    goto yy57;
  } else {

  }
  yy1153: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 78) {
    if ((int )yych == 72) {
      goto yy1155;
    } else {

    }
    if ((int )yych <= 77) {
      goto yy57;
    } else {

    }
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 103) {
      goto yy57;
    } else {

    }
    goto yy1155;
  } else
  if ((int )yych != 110) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych == 73) {
    goto yy1160;
  } else {

  }
  if ((int )yych == 105) {
    goto yy1160;
  } else {

  }
  goto yy57;
  yy1155: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 78) {
    goto yy1156;
  } else {

  }
  if ((int )yych != 110) {
    goto yy57;
  } else {

  }
  yy1156: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 73) {
    goto yy1157;
  } else {

  }
  if ((int )yych != 105) {
    goto yy57;
  } else {

  }
  yy1157: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 71) {
    goto yy1158;
  } else {

  }
  if ((int )yych != 103) {
    goto yy57;
  } else {

  }
  yy1158: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 72) {
    goto yy1159;
  } else {

  }
  if ((int )yych != 104) {
    goto yy57;
  } else {

  }
  yy1159: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 84) {
    goto yy1149;
  } else {

  }
  if ((int )yych == 116) {
    goto yy1149;
  } else {

  }
  goto yy57;
  yy1160: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 71) {
    goto yy1161;
  } else {

  }
  if ((int )yych != 103) {
    goto yy57;
  } else {

  }
  yy1161: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 72) {
    goto yy1162;
  } else {

  }
  if ((int )yych != 104) {
    goto yy57;
  } else {

  }
  yy1162: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 84) {
    goto yy1149;
  } else {

  }
  if ((int )yych == 116) {
    goto yy1149;
  } else {

  }
  goto yy57;
  yy1163: 
  yyaccept = 26U;
  cursor ++;
  tmp___348 = cursor;
  s->ptr = tmp___348;
  yych = *tmp___348;
  if ((int )yych <= 32) {
    if ((int )yych == 9) {
      goto yy1119;
    } else {

    }
    if ((int )yych <= 31) {
      goto yy1118;
    } else {

    }
    goto yy1119;
  } else
  if ((int )yych <= 68) {
    if ((int )yych <= 67) {
      goto yy1118;
    } else {

    }
  } else
  if ((int )yych != 100) {
    goto yy1118;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy1165;
  } else {

  }
  if ((int )yych != 97) {
    goto yy57;
  } else {

  }
  yy1165: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy1124;
  } else {

  }
  if ((int )yych == 121) {
    goto yy1124;
  } else {

  }
  goto yy57;
  yy1166: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy1149;
  } else {

  }
  if ((int )yych == 121) {
    goto yy1149;
  } else {

  }
  goto yy57;
  yy1167: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 85) {
    goto yy1168;
  } else {

  }
  if ((int )yych != 117) {
    goto yy57;
  } else {

  }
  yy1168: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy1149;
  } else {

  }
  if ((int )yych == 114) {
    goto yy1149;
  } else {

  }
  goto yy57;
  yy1169: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 78) {
    goto yy1174;
  } else {

  }
  if ((int )yych == 110) {
    goto yy1174;
  } else {

  }
  goto yy57;
  yy1170: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 78) {
    goto yy1171;
  } else {

  }
  if ((int )yych != 110) {
    goto yy57;
  } else {

  }
  yy1171: 
  yyaccept = 26U;
  cursor ++;
  tmp___349 = cursor;
  s->ptr = tmp___349;
  yych = *tmp___349;
  if ((int )yych <= 85) {
    if ((int )yych == 83) {
      goto yy1143;
    } else {

    }
    if ((int )yych <= 84) {
      goto yy1118;
    } else {

    }
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 114) {
      goto yy1118;
    } else {

    }
    goto yy1143;
  } else
  if ((int )yych != 117) {
    goto yy1118;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych == 84) {
    goto yy1173;
  } else {

  }
  if ((int )yych != 116) {
    goto yy57;
  } else {

  }
  yy1173: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy1149;
  } else {

  }
  if ((int )yych == 101) {
    goto yy1149;
  } else {

  }
  goto yy57;
  yy1174: 
  yyaccept = 26U;
  cursor ++;
  tmp___350 = cursor;
  s->ptr = tmp___350;
  yych = *tmp___350;
  if ((int )yych <= 68) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy1119;
      } else {

      }
      goto yy1118;
    } else {
      if ((int )yych <= 32) {
        goto yy1119;
      } else {

      }
      if ((int )yych <= 67) {
        goto yy1118;
      } else {

      }
    }
  } else
  if ((int )yych <= 99) {
    if ((int )yych == 84) {
      goto yy1176;
    } else {

    }
    goto yy1118;
  } else {
    if ((int )yych <= 100) {
      goto yy1175;
    } else {

    }
    if ((int )yych == 116) {
      goto yy1176;
    } else {

    }
    goto yy1118;
  }
  yy1175: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy1177;
  } else {

  }
  if ((int )yych == 97) {
    goto yy1177;
  } else {

  }
  goto yy57;
  yy1176: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 72) {
    goto yy1149;
  } else {

  }
  if ((int )yych == 104) {
    goto yy1149;
  } else {

  }
  goto yy57;
  yy1177: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy1124;
  } else {

  }
  if ((int )yych == 121) {
    goto yy1124;
  } else {

  }
  goto yy57;
  yy1178: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 78) {
    goto yy1189;
  } else {

  }
  if ((int )yych == 110) {
    goto yy1189;
  } else {

  }
  goto yy57;
  yy1179: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 84) {
    goto yy1184;
  } else {

  }
  if ((int )yych == 116) {
    goto yy1184;
  } else {

  }
  goto yy57;
  yy1180: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 67) {
    goto yy1181;
  } else {

  }
  if ((int )yych != 99) {
    goto yy57;
  } else {

  }
  yy1181: 
  yyaccept = 26U;
  cursor ++;
  tmp___351 = cursor;
  s->ptr = tmp___351;
  yych = *tmp___351;
  if ((int )yych <= 83) {
    if ((int )yych == 79) {
      goto yy1182;
    } else {

    }
    if ((int )yych <= 82) {
      goto yy1118;
    } else {

    }
    goto yy1143;
  } else
  if ((int )yych <= 111) {
    if ((int )yych <= 110) {
      goto yy1118;
    } else {

    }
  } else {
    if ((int )yych == 115) {
      goto yy1143;
    } else {

    }
    goto yy1118;
  }
  yy1182: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 78) {
    goto yy1183;
  } else {

  }
  if ((int )yych != 110) {
    goto yy57;
  } else {

  }
  yy1183: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 68) {
    goto yy1149;
  } else {

  }
  if ((int )yych == 100) {
    goto yy1149;
  } else {

  }
  goto yy57;
  yy1184: 
  yyaccept = 26U;
  cursor ++;
  tmp___352 = cursor;
  s->ptr = tmp___352;
  yych = *tmp___352;
  if ((int )yych <= 32) {
    if ((int )yych == 9) {
      goto yy1119;
    } else {

    }
    if ((int )yych <= 31) {
      goto yy1118;
    } else {

    }
    goto yy1119;
  } else
  if ((int )yych <= 85) {
    if ((int )yych <= 84) {
      goto yy1118;
    } else {

    }
  } else
  if ((int )yych != 117) {
    goto yy1118;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy1186;
  } else {

  }
  if ((int )yych != 114) {
    goto yy57;
  } else {

  }
  yy1186: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 68) {
    goto yy1187;
  } else {

  }
  if ((int )yych != 100) {
    goto yy57;
  } else {

  }
  yy1187: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy1188;
  } else {

  }
  if ((int )yych != 97) {
    goto yy57;
  } else {

  }
  yy1188: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy1124;
  } else {

  }
  if ((int )yych == 121) {
    goto yy1124;
  } else {

  }
  goto yy57;
  yy1189: 
  yyaccept = 26U;
  cursor ++;
  tmp___353 = cursor;
  s->ptr = tmp___353;
  yych = *tmp___353;
  if ((int )yych <= 32) {
    if ((int )yych == 9) {
      goto yy1119;
    } else {

    }
    if ((int )yych <= 31) {
      goto yy1118;
    } else {

    }
    goto yy1119;
  } else
  if ((int )yych <= 68) {
    if ((int )yych <= 67) {
      goto yy1118;
    } else {

    }
  } else
  if ((int )yych != 100) {
    goto yy1118;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy1191;
  } else {

  }
  if ((int )yych != 97) {
    goto yy57;
  } else {

  }
  yy1191: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy1124;
  } else {

  }
  if ((int )yych == 121) {
    goto yy1124;
  } else {

  }
  goto yy57;
  yy1192: 
  yyaccept = 0U;
  cursor ++;
  tmp___354 = cursor;
  s->ptr = tmp___354;
  yych = *tmp___354;
  if ((int )yych <= 68) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 69) {
      goto yy1099;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 101) {
      goto yy1193;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy1193: 
  yyaccept = 0U;
  cursor ++;
  tmp___355 = cursor;
  s->ptr = tmp___355;
  yych = *tmp___355;
  if ((int )yych <= 85) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 86) {
      goto yy1100;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 117) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 118) {
      goto yy1194;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy4;
  }
  yy1194: 
  yyaccept = 0U;
  cursor ++;
  tmp___356 = cursor;
  s->ptr = tmp___356;
  yych = *tmp___356;
  if ((int )yych <= 72) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 73) {
      goto yy1101;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 105) {
      goto yy1195;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1195: 
  yyaccept = 0U;
  cursor ++;
  tmp___357 = cursor;
  s->ptr = tmp___357;
  yych = *tmp___357;
  if ((int )yych <= 78) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy145;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 79) {
      goto yy1102;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 110) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy153;
  } else {
    if ((int )yych <= 111) {
      goto yy1196;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy4;
  }
  yy1196: 
  yyaccept = 0U;
  cursor ++;
  tmp___358 = cursor;
  s->ptr = tmp___358;
  yych = *tmp___358;
  if ((int )yych <= 84) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 85) {
      goto yy1103;
    } else {

    }
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych == 117) {
      goto yy1197;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy154;
    } else {

    }
    goto yy4;
  }
  yy1197: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 83) {
    goto yy1104;
  } else {

  }
  if ((int )yych != 115) {
    goto yy155;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int const   )yybm[(int )yych] & 16) {
    goto yy154;
  } else {

  }
  if ((int )yych <= 44) {
    if ((int )yych <= 9) {
      if ((int )yych <= 8) {
        goto yy57;
      } else {

      }
      goto yy1105;
    } else {
      if ((int )yych == 32) {
        goto yy1105;
      } else {

      }
      goto yy57;
    }
  } else
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy57;
    } else {

    }
    goto yy148;
  } else {
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy57;
  }
  yy1199: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 71) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 70) {
        goto yy142;
      } else {

      }
      goto yy1213;
    }
  } else
  if ((int )yych <= 102) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else {
    if ((int )yych <= 103) {
      goto yy1213;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy1200: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 69) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 68) {
        goto yy142;
      } else {

      }
    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else {
    if ((int )yych <= 101) {
      goto yy1201;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy1201: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 86) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 85) {
        goto yy143;
      } else {

      }
    }
  } else
  if ((int )yych <= 117) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 118) {
      goto yy1202;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy4;
  }
  yy1202: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 69) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 68) {
        goto yy144;
      } else {

      }
    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 101) {
      goto yy1203;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1203: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 78) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 77) {
        goto yy145;
      } else {

      }
    }
  } else
  if ((int )yych <= 109) {
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy145;
  } else {
    if ((int )yych <= 110) {
      goto yy1204;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy4;
  }
  yy1204: 
  yyaccept = 0U;
  cursor ++;
  tmp___359 = cursor;
  s->ptr = tmp___359;
  yych = *tmp___359;
  if ((int )yych <= 83) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 84) {
      goto yy1205;
    } else {

    }
    if ((int )yych != 116) {
      goto yy4;
    } else {

    }
  }
  yy1205: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 72) {
    goto yy1206;
  } else {

  }
  if ((int )yych != 104) {
    goto yy57;
  } else {

  }
  yy1206: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 9) {
    goto yy1207;
  } else {

  }
  if ((int )yych != 32) {
    goto yy57;
  } else {

  }
  yy1207: 
  cursor ++;
  if (s->lim - cursor < 11) {
    return (257);
  } else {

  }
  yych = *cursor;
  yy1208: ;
  if ((int )yych <= 87) {
    if ((int )yych <= 70) {
      if ((int )yych <= 32) {
        if ((int )yych == 9) {
          goto yy1207;
        } else {

        }
        if ((int )yych <= 31) {
          goto yy57;
        } else {

        }
        goto yy1207;
      } else {
        if ((int )yych == 68) {
          goto yy1110;
        } else {

        }
        if ((int )yych <= 69) {
          goto yy57;
        } else {

        }
        goto yy1111;
      }
    } else
    if ((int )yych <= 77) {
      if ((int )yych == 72) {
        goto yy1109;
      } else {

      }
      if ((int )yych <= 76) {
        goto yy57;
      } else {

      }
      goto yy1108;
    } else
    if ((int )yych <= 83) {
      if ((int )yych <= 82) {
        goto yy57;
      } else {

      }
      goto yy1107;
    } else {
      if ((int )yych <= 84) {
        goto yy1114;
      } else {

      }
      if ((int )yych <= 86) {
        goto yy57;
      } else {

      }
    }
  } else
  if ((int )yych <= 108) {
    if ((int )yych <= 100) {
      if ((int )yych == 89) {
        goto yy1112;
      } else {

      }
      if ((int )yych <= 99) {
        goto yy57;
      } else {

      }
      goto yy1110;
    } else
    if ((int )yych <= 102) {
      if ((int )yych <= 101) {
        goto yy57;
      } else {

      }
      goto yy1111;
    } else {
      if ((int )yych == 104) {
        goto yy1109;
      } else {

      }
      goto yy57;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych <= 109) {
      goto yy1108;
    } else {

    }
    if ((int )yych <= 114) {
      goto yy57;
    } else {

    }
    if ((int )yych <= 115) {
      goto yy1107;
    } else {

    }
    goto yy1114;
  } else
  if ((int )yych <= 119) {
    if ((int )yych <= 118) {
      goto yy57;
    } else {

    }
  } else {
    if ((int )yych == 121) {
      goto yy1112;
    } else {

    }
    goto yy57;
  }
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy1210;
  } else {

  }
  if ((int )yych != 101) {
    goto yy57;
  } else {

  }
  yy1210: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 69) {
    if ((int )yych <= 67) {
      goto yy57;
    } else {

    }
    if ((int )yych <= 68) {
      goto yy1135;
    } else {

    }
  } else {
    if ((int )yych <= 99) {
      goto yy57;
    } else {

    }
    if ((int )yych <= 100) {
      goto yy1135;
    } else {

    }
    if ((int )yych >= 102) {
      goto yy57;
    } else {

    }
  }
  cursor ++;
  yych = *cursor;
  if ((int )yych == 75) {
    goto yy1212;
  } else {

  }
  if ((int )yych != 107) {
    goto yy57;
  } else {

  }
  yy1212: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 83) {
    if ((int )yych == 68) {
      goto yy1144;
    } else {

    }
    if ((int )yych <= 82) {
      goto yy57;
    } else {

    }
    goto yy1143;
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 99) {
      goto yy57;
    } else {

    }
    goto yy1144;
  } else {
    if ((int )yych == 115) {
      goto yy1143;
    } else {

    }
    goto yy57;
  }
  yy1213: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 72) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 71) {
        goto yy143;
      } else {

      }
    }
  } else
  if ((int )yych <= 103) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 104) {
      goto yy1214;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy4;
  }
  yy1214: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 84) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 83) {
        goto yy144;
      } else {

      }
    }
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 116) {
      goto yy1215;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1215: 
  yyaccept = 0U;
  cursor ++;
  tmp___360 = cursor;
  s->ptr = tmp___360;
  yych = *tmp___360;
  if ((int )yych <= 64) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy1207;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych <= 32) {
        goto yy1207;
      } else {

      }
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych == 72) {
      goto yy1216;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych == 104) {
      goto yy1216;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy4;
  }
  yy1216: 
  yyaccept = 0U;
  cursor ++;
  tmp___361 = cursor;
  s->ptr = tmp___361;
  yych = *tmp___361;
  if ((int )yych <= 31) {
    if ((int )yych == 9) {
      goto yy1207;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 32) {
      goto yy1207;
    } else {

    }
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    goto yy4;
  }
  yy1217: 
  yyaccept = 0U;
  cursor ++;
  tmp___362 = cursor;
  s->ptr = tmp___362;
  yych = *tmp___362;
  if ((int )yych <= 70) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 71) {
      goto yy1213;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 102) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 103) {
      goto yy1225;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy1218: 
  yyaccept = 0U;
  cursor ++;
  tmp___363 = cursor;
  s->ptr = tmp___363;
  yych = *tmp___363;
  if ((int )yych <= 68) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 69) {
      goto yy1201;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 101) {
      goto yy1219;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy1219: 
  yyaccept = 0U;
  cursor ++;
  tmp___364 = cursor;
  s->ptr = tmp___364;
  yych = *tmp___364;
  if ((int )yych <= 85) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 86) {
      goto yy1202;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 117) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 118) {
      goto yy1220;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy4;
  }
  yy1220: 
  yyaccept = 0U;
  cursor ++;
  tmp___365 = cursor;
  s->ptr = tmp___365;
  yych = *tmp___365;
  if ((int )yych <= 68) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 69) {
      goto yy1203;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 101) {
      goto yy1221;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1221: 
  yyaccept = 0U;
  cursor ++;
  tmp___366 = cursor;
  s->ptr = tmp___366;
  yych = *tmp___366;
  if ((int )yych <= 77) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy145;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 78) {
      goto yy1204;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 109) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy153;
  } else {
    if ((int )yych <= 110) {
      goto yy1222;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy4;
  }
  yy1222: 
  yyaccept = 0U;
  cursor ++;
  tmp___367 = cursor;
  s->ptr = tmp___367;
  yych = *tmp___367;
  if ((int )yych <= 83) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 84) {
      goto yy1205;
    } else {

    }
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych == 116) {
      goto yy1223;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy154;
    } else {

    }
    goto yy4;
  }
  yy1223: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 72) {
    goto yy1206;
  } else {

  }
  if ((int )yych != 104) {
    goto yy155;
  } else {

  }
  yy1224: 
  cursor ++;
  yych = *cursor;
  if ((int const   )yybm[(int )yych] & 16) {
    goto yy154;
  } else {

  }
  if ((int )yych <= 44) {
    if ((int )yych <= 9) {
      if ((int )yych <= 8) {
        goto yy57;
      } else {

      }
      goto yy1207;
    } else {
      if ((int )yych == 32) {
        goto yy1207;
      } else {

      }
      goto yy57;
    }
  } else
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy57;
    } else {

    }
    goto yy148;
  } else {
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy57;
  }
  yy1225: 
  yyaccept = 0U;
  cursor ++;
  tmp___368 = cursor;
  s->ptr = tmp___368;
  yych = *tmp___368;
  if ((int )yych <= 71) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 72) {
      goto yy1214;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 103) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 104) {
      goto yy1226;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy4;
  }
  yy1226: 
  yyaccept = 0U;
  cursor ++;
  tmp___369 = cursor;
  s->ptr = tmp___369;
  yych = *tmp___369;
  if ((int )yych <= 83) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 84) {
      goto yy1215;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 116) {
      goto yy1227;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1227: 
  yyaccept = 0U;
  cursor ++;
  tmp___370 = cursor;
  s->ptr = tmp___370;
  yych = *tmp___370;
  if ((int )yych <= 47) {
    if ((int )yych <= 40) {
      if ((int )yych <= 9) {
        if ((int )yych <= 8) {
          goto yy4;
        } else {

        }
        goto yy1207;
      } else {
        if ((int )yych == 32) {
          goto yy1207;
        } else {

        }
        goto yy4;
      }
    } else
    if ((int )yych <= 44) {
      if ((int )yych <= 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      goto yy148;
    }
  } else
  if ((int )yych <= 94) {
    if ((int )yych <= 71) {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy145;
    } else {
      if ((int )yych <= 72) {
        goto yy1216;
      } else {

      }
      if ((int )yych <= 90) {
        goto yy145;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 103) {
    if ((int )yych <= 95) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy153;
  } else {
    if ((int )yych <= 104) {
      goto yy1228;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy4;
  }
  yy1228: 
  yyaccept = 0U;
  cursor ++;
  tmp___371 = cursor;
  s->ptr = tmp___371;
  yych = *tmp___371;
  if ((int const   )yybm[(int )yych] & 16) {
    goto yy154;
  } else {

  }
  if ((int )yych <= 41) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy1207;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych <= 32) {
        goto yy1207;
      } else {

      }
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    }
  } else
  if ((int )yych <= 46) {
    if ((int )yych == 45) {
      goto yy148;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 47) {
      goto yy148;
    } else {

    }
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy4;
  }
  yy1229: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 86) {
    if ((int )yych <= 66) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy142;
    } else
    if ((int )yych <= 79) {
      if ((int )yych <= 67) {
        goto yy1245;
      } else {

      }
      goto yy142;
    } else {
      if ((int )yych <= 80) {
        goto yy1247;
      } else {

      }
      if ((int )yych <= 85) {
        goto yy142;
      } else {

      }
      goto yy1246;
    }
  } else
  if ((int )yych <= 111) {
    if ((int )yych <= 96) {
      if ((int )yych <= 90) {
        goto yy142;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 99) {
        goto yy1245;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 117) {
    if ((int )yych <= 112) {
      goto yy1247;
    } else {

    }
    goto yy142;
  } else {
    if ((int )yych <= 118) {
      goto yy1246;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy1230: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 84) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 83) {
        goto yy142;
      } else {

      }
      goto yy1240;
    }
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else {
    if ((int )yych <= 116) {
      goto yy1240;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy1231: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 88) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 87) {
        goto yy142;
      } else {

      }
      goto yy1237;
    }
  } else
  if ((int )yych <= 119) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else {
    if ((int )yych <= 120) {
      goto yy1237;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy1232: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 78) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 77) {
        goto yy142;
      } else {

      }
    }
  } else
  if ((int )yych <= 109) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else {
    if ((int )yych <= 110) {
      goto yy1233;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy1233: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 68) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy167;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy167;
      } else {

      }
      if ((int )yych <= 67) {
        goto yy143;
      } else {

      }
    }
  } else
  if ((int )yych <= 99) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy167;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 100) {
      goto yy1234;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy167;
  }
  yy1234: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 65) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 97) {
      goto yy1235;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1235: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 89) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 88) {
        goto yy145;
      } else {

      }
    }
  } else
  if ((int )yych <= 120) {
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy145;
  } else {
    if ((int )yych <= 121) {
      goto yy1236;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy4;
  }
  yy1236: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 41) {
    goto yy140;
  } else {

  }
  goto yy167;
  yy1237: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 84) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 83) {
        goto yy143;
      } else {

      }
    }
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 116) {
      goto yy1238;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy4;
  }
  yy1238: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 72) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 71) {
        goto yy144;
      } else {

      }
    }
  } else
  if ((int )yych <= 103) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 104) {
      goto yy1239;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1239: 
  yyaccept = 0U;
  cursor ++;
  tmp___372 = cursor;
  s->ptr = tmp___372;
  yych = *tmp___372;
  if ((int )yych <= 40) {
    if ((int )yych <= 9) {
      if ((int )yych <= 8) {
        goto yy4;
      } else {

      }
      goto yy1207;
    } else {
      if ((int )yych == 32) {
        goto yy1207;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 41) {
      goto yy140;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    goto yy145;
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy4;
  }
  yy1240: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy167;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy167;
      } else {

      }
      if ((int )yych <= 84) {
        goto yy143;
      } else {

      }
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy167;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 117) {
      goto yy1241;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy167;
  }
  yy1241: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 82) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 81) {
        goto yy144;
      } else {

      }
    }
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 114) {
      goto yy1242;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1242: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 68) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 67) {
        goto yy145;
      } else {

      }
    }
  } else
  if ((int )yych <= 99) {
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy145;
  } else {
    if ((int )yych <= 100) {
      goto yy1243;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy4;
  }
  yy1243: 
  yyaccept = 0U;
  cursor ++;
  tmp___373 = cursor;
  s->ptr = tmp___373;
  yych = *tmp___373;
  if ((int )yych <= 64) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 65) {
      goto yy1244;
    } else {

    }
    if ((int )yych != 97) {
      goto yy4;
    } else {

    }
  }
  yy1244: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy173;
  } else {

  }
  if ((int )yych == 121) {
    goto yy173;
  } else {

  }
  goto yy57;
  yy1245: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 79) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 78) {
        goto yy143;
      } else {

      }
      goto yy1256;
    }
  } else
  if ((int )yych <= 110) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 111) {
      goto yy1256;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy4;
  }
  yy1246: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 69) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 68) {
        goto yy143;
      } else {

      }
      goto yy1253;
    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 101) {
      goto yy1253;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy4;
  }
  yy1247: 
  yyaccept = 5U;
  cursor ++;
  tmp___374 = cursor;
  s->ptr = tmp___374;
  yych = *tmp___374;
  if ((int )yych <= 47) {
    if ((int )yych <= 40) {
      if ((int )yych <= 9) {
        if ((int )yych <= 8) {
          goto yy194;
        } else {

        }
        goto yy196;
      } else {
        if ((int )yych == 32) {
          goto yy196;
        } else {

        }
        goto yy194;
      }
    } else
    if ((int )yych <= 44) {
      if ((int )yych <= 41) {
        goto yy140;
      } else {

      }
      goto yy194;
    } else {
      if ((int )yych <= 45) {
        goto yy197;
      } else {

      }
      if ((int )yych <= 46) {
        goto yy196;
      } else {

      }
      goto yy194;
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 64) {
      if ((int )yych <= 57) {
        goto yy196;
      } else {

      }
      goto yy194;
    } else
    if ((int )yych != 84) {
      goto yy143;
    } else {

    }
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 116) {
      goto yy1248;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy194;
  }
  yy1248: 
  yyaccept = 5U;
  cursor ++;
  tmp___375 = cursor;
  s->ptr = tmp___375;
  yych = *tmp___375;
  if ((int )yych <= 47) {
    if ((int )yych <= 40) {
      if ((int )yych <= 9) {
        if ((int )yych <= 8) {
          goto yy194;
        } else {

        }
        goto yy196;
      } else {
        if ((int )yych == 32) {
          goto yy196;
        } else {

        }
        goto yy194;
      }
    } else
    if ((int )yych <= 44) {
      if ((int )yych <= 41) {
        goto yy140;
      } else {

      }
      goto yy194;
    } else {
      if ((int )yych <= 45) {
        goto yy197;
      } else {

      }
      if ((int )yych <= 46) {
        goto yy196;
      } else {

      }
      goto yy194;
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 64) {
      if ((int )yych <= 57) {
        goto yy196;
      } else {

      }
      goto yy194;
    } else
    if ((int )yych != 69) {
      goto yy144;
    } else {

    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 101) {
      goto yy1249;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy194;
  }
  yy1249: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 77) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 76) {
        goto yy145;
      } else {

      }
    }
  } else
  if ((int )yych <= 108) {
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy145;
  } else {
    if ((int )yych <= 109) {
      goto yy1250;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy4;
  }
  yy1250: 
  yyaccept = 0U;
  cursor ++;
  tmp___376 = cursor;
  s->ptr = tmp___376;
  yych = *tmp___376;
  if ((int )yych <= 65) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 66) {
      goto yy1251;
    } else {

    }
    if ((int )yych != 98) {
      goto yy4;
    } else {

    }
  }
  yy1251: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy1252;
  } else {

  }
  if ((int )yych != 101) {
    goto yy57;
  } else {

  }
  yy1252: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy206;
  } else {

  }
  if ((int )yych == 114) {
    goto yy206;
  } else {

  }
  goto yy57;
  yy1253: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 78) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 77) {
        goto yy144;
      } else {

      }
    }
  } else
  if ((int )yych <= 109) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 110) {
      goto yy1254;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1254: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 84) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 83) {
        goto yy145;
      } else {

      }
    }
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy145;
  } else {
    if ((int )yych <= 116) {
      goto yy1255;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy4;
  }
  yy1255: 
  yyaccept = 0U;
  cursor ++;
  tmp___377 = cursor;
  s->ptr = tmp___377;
  yych = *tmp___377;
  if ((int )yych <= 71) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 72) {
      goto yy1206;
    } else {

    }
    if ((int )yych == 104) {
      goto yy1206;
    } else {

    }
    goto yy4;
  }
  yy1256: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 78) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 77) {
        goto yy144;
      } else {

      }
    }
  } else
  if ((int )yych <= 109) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 110) {
      goto yy1257;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1257: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 68) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 67) {
        goto yy145;
      } else {

      }
      goto yy1216;
    }
  } else
  if ((int )yych <= 99) {
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy145;
  } else {
    if ((int )yych <= 100) {
      goto yy1216;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy4;
  }
  yy1258: 
  yyaccept = 0U;
  cursor ++;
  tmp___378 = cursor;
  s->ptr = tmp___378;
  yych = *tmp___378;
  if ((int )yych <= 85) {
    if ((int )yych <= 47) {
      if ((int )yych <= 44) {
        if ((int )yych == 41) {
          goto yy140;
        } else {

        }
        goto yy4;
      } else {
        if ((int )yych == 46) {
          goto yy4;
        } else {

        }
        goto yy148;
      }
    } else
    if ((int )yych <= 67) {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 66) {
        goto yy142;
      } else {

      }
      goto yy1245;
    } else {
      if ((int )yych == 80) {
        goto yy1247;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 98) {
    if ((int )yych <= 94) {
      if ((int )yych <= 86) {
        goto yy1246;
      } else {

      }
      if ((int )yych <= 90) {
        goto yy142;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych <= 95) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 96) {
        goto yy4;
      } else {

      }
      goto yy147;
    }
  } else
  if ((int )yych <= 112) {
    if ((int )yych <= 99) {
      goto yy1274;
    } else {

    }
    if ((int )yych <= 111) {
      goto yy147;
    } else {

    }
    goto yy1276;
  } else {
    if ((int )yych == 118) {
      goto yy1275;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy1259: 
  yyaccept = 0U;
  cursor ++;
  tmp___379 = cursor;
  s->ptr = tmp___379;
  yych = *tmp___379;
  if ((int )yych <= 83) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 84) {
      goto yy1240;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 116) {
      goto yy1269;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy1260: 
  yyaccept = 0U;
  cursor ++;
  tmp___380 = cursor;
  s->ptr = tmp___380;
  yych = *tmp___380;
  if ((int )yych <= 87) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 88) {
      goto yy1237;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 119) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 120) {
      goto yy1266;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy1261: 
  yyaccept = 0U;
  cursor ++;
  tmp___381 = cursor;
  s->ptr = tmp___381;
  yych = *tmp___381;
  if ((int )yych <= 77) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 78) {
      goto yy1233;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 109) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 110) {
      goto yy1262;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy1262: 
  yyaccept = 4U;
  cursor ++;
  tmp___382 = cursor;
  s->ptr = tmp___382;
  yych = *tmp___382;
  if ((int )yych <= 67) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy167;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy167;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 68) {
      goto yy1234;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy167;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 99) {
    if ((int )yych <= 96) {
      goto yy167;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 100) {
      goto yy1263;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy167;
  }
  yy1263: 
  yyaccept = 0U;
  cursor ++;
  tmp___383 = cursor;
  s->ptr = tmp___383;
  yych = *tmp___383;
  if ((int )yych <= 64) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 65) {
      goto yy1235;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 97) {
      goto yy1264;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1264: 
  yyaccept = 0U;
  cursor ++;
  tmp___384 = cursor;
  s->ptr = tmp___384;
  yych = *tmp___384;
  if ((int )yych <= 88) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy145;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 89) {
      goto yy1236;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 120) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy153;
  } else {
    if ((int )yych <= 121) {
      goto yy1265;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy4;
  }
  yy1265: 
  yyaccept = 4U;
  cursor ++;
  tmp___385 = cursor;
  s->ptr = tmp___385;
  yych = *tmp___385;
  if ((int const   )yybm[(int )yych] & 16) {
    goto yy154;
  } else {

  }
  if ((int )yych <= 45) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    if ((int )yych <= 44) {
      goto yy167;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 47) {
    if ((int )yych <= 46) {
      goto yy167;
    } else {

    }
    goto yy148;
  } else {
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy167;
  }
  yy1266: 
  yyaccept = 0U;
  cursor ++;
  tmp___386 = cursor;
  s->ptr = tmp___386;
  yych = *tmp___386;
  if ((int )yych <= 83) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 84) {
      goto yy1238;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 116) {
      goto yy1267;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy4;
  }
  yy1267: 
  yyaccept = 0U;
  cursor ++;
  tmp___387 = cursor;
  s->ptr = tmp___387;
  yych = *tmp___387;
  if ((int )yych <= 71) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 72) {
      goto yy1239;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 103) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 104) {
      goto yy1268;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1268: 
  yyaccept = 0U;
  cursor ++;
  tmp___388 = cursor;
  s->ptr = tmp___388;
  yych = *tmp___388;
  if ((int )yych <= 45) {
    if ((int )yych <= 32) {
      if ((int )yych == 9) {
        goto yy1207;
      } else {

      }
      if ((int )yych <= 31) {
        goto yy4;
      } else {

      }
      goto yy1207;
    } else {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych == 47) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    goto yy145;
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy4;
  }
  yy1269: 
  yyaccept = 4U;
  cursor ++;
  tmp___389 = cursor;
  s->ptr = tmp___389;
  yych = *tmp___389;
  if ((int )yych <= 84) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy167;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy167;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 85) {
      goto yy1241;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy167;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 116) {
    if ((int )yych <= 96) {
      goto yy167;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 117) {
      goto yy1270;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy167;
  }
  yy1270: 
  yyaccept = 0U;
  cursor ++;
  tmp___390 = cursor;
  s->ptr = tmp___390;
  yych = *tmp___390;
  if ((int )yych <= 81) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 82) {
      goto yy1242;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 114) {
      goto yy1271;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1271: 
  yyaccept = 0U;
  cursor ++;
  tmp___391 = cursor;
  s->ptr = tmp___391;
  yych = *tmp___391;
  if ((int )yych <= 67) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy145;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 68) {
      goto yy1243;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 99) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy153;
  } else {
    if ((int )yych <= 100) {
      goto yy1272;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy4;
  }
  yy1272: 
  yyaccept = 0U;
  cursor ++;
  tmp___392 = cursor;
  s->ptr = tmp___392;
  yych = *tmp___392;
  if ((int )yych <= 64) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 65) {
      goto yy1244;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 97) {
      goto yy1273;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy154;
    } else {

    }
    goto yy4;
  }
  yy1273: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy173;
  } else {

  }
  if ((int )yych == 121) {
    goto yy186;
  } else {

  }
  goto yy155;
  yy1274: 
  yyaccept = 0U;
  cursor ++;
  tmp___393 = cursor;
  s->ptr = tmp___393;
  yych = *tmp___393;
  if ((int )yych <= 78) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 79) {
      goto yy1256;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 110) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 111) {
      goto yy1285;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy4;
  }
  yy1275: 
  yyaccept = 0U;
  cursor ++;
  tmp___394 = cursor;
  s->ptr = tmp___394;
  yych = *tmp___394;
  if ((int )yych <= 68) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 69) {
      goto yy1253;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 101) {
      goto yy1282;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy4;
  }
  yy1276: 
  yyaccept = 5U;
  cursor ++;
  tmp___395 = cursor;
  s->ptr = tmp___395;
  yych = *tmp___395;
  if ((int )yych <= 57) {
    if ((int )yych <= 40) {
      if ((int )yych <= 9) {
        if ((int )yych <= 8) {
          goto yy194;
        } else {

        }
        goto yy196;
      } else {
        if ((int )yych == 32) {
          goto yy196;
        } else {

        }
        goto yy194;
      }
    } else
    if ((int )yych <= 45) {
      if ((int )yych <= 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy194;
      } else {

      }
      goto yy372;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      goto yy196;
    }
  } else
  if ((int )yych <= 94) {
    if ((int )yych <= 83) {
      if ((int )yych <= 64) {
        goto yy194;
      } else {

      }
      goto yy143;
    } else {
      if ((int )yych <= 84) {
        goto yy1248;
      } else {

      }
      if ((int )yych <= 90) {
        goto yy143;
      } else {

      }
      goto yy194;
    }
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 95) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 116) {
      goto yy1277;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy194;
  }
  yy1277: 
  yyaccept = 5U;
  cursor ++;
  tmp___396 = cursor;
  s->ptr = tmp___396;
  yych = *tmp___396;
  if ((int )yych <= 57) {
    if ((int )yych <= 40) {
      if ((int )yych <= 9) {
        if ((int )yych <= 8) {
          goto yy194;
        } else {

        }
        goto yy196;
      } else {
        if ((int )yych == 32) {
          goto yy196;
        } else {

        }
        goto yy194;
      }
    } else
    if ((int )yych <= 45) {
      if ((int )yych <= 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy194;
      } else {

      }
      goto yy372;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      goto yy196;
    }
  } else
  if ((int )yych <= 94) {
    if ((int )yych <= 68) {
      if ((int )yych <= 64) {
        goto yy194;
      } else {

      }
      goto yy144;
    } else {
      if ((int )yych <= 69) {
        goto yy1249;
      } else {

      }
      if ((int )yych <= 90) {
        goto yy144;
      } else {

      }
      goto yy194;
    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 95) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 101) {
      goto yy1278;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy194;
  }
  yy1278: 
  yyaccept = 0U;
  cursor ++;
  tmp___397 = cursor;
  s->ptr = tmp___397;
  yych = *tmp___397;
  if ((int )yych <= 76) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy145;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 77) {
      goto yy1250;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 108) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy153;
  } else {
    if ((int )yych <= 109) {
      goto yy1279;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy4;
  }
  yy1279: 
  yyaccept = 0U;
  cursor ++;
  tmp___398 = cursor;
  s->ptr = tmp___398;
  yych = *tmp___398;
  if ((int )yych <= 65) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 66) {
      goto yy1251;
    } else {

    }
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych == 98) {
      goto yy1280;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy154;
    } else {

    }
    goto yy4;
  }
  yy1280: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 69) {
    goto yy1252;
  } else {

  }
  if ((int )yych != 101) {
    goto yy155;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy206;
  } else {

  }
  if ((int )yych == 114) {
    goto yy377;
  } else {

  }
  goto yy155;
  yy1282: 
  yyaccept = 0U;
  cursor ++;
  tmp___399 = cursor;
  s->ptr = tmp___399;
  yych = *tmp___399;
  if ((int )yych <= 77) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 78) {
      goto yy1254;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 109) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 110) {
      goto yy1283;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1283: 
  yyaccept = 0U;
  cursor ++;
  tmp___400 = cursor;
  s->ptr = tmp___400;
  yych = *tmp___400;
  if ((int )yych <= 83) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy145;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 84) {
      goto yy1255;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy153;
  } else {
    if ((int )yych <= 116) {
      goto yy1284;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy4;
  }
  yy1284: 
  yyaccept = 0U;
  cursor ++;
  tmp___401 = cursor;
  s->ptr = tmp___401;
  yych = *tmp___401;
  if ((int )yych <= 71) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 72) {
      goto yy1206;
    } else {

    }
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych == 104) {
      goto yy1224;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy154;
    } else {

    }
    goto yy4;
  }
  yy1285: 
  yyaccept = 0U;
  cursor ++;
  tmp___402 = cursor;
  s->ptr = tmp___402;
  yych = *tmp___402;
  if ((int )yych <= 77) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 78) {
      goto yy1257;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 109) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 110) {
      goto yy1286;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1286: 
  yyaccept = 0U;
  cursor ++;
  tmp___403 = cursor;
  s->ptr = tmp___403;
  yych = *tmp___403;
  if ((int )yych <= 67) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy145;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 68) {
      goto yy1216;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 99) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy153;
  } else {
    if ((int )yych <= 100) {
      goto yy1228;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy4;
  }
  yy1287: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 67) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 66) {
        goto yy142;
      } else {

      }
    }
  } else
  if ((int )yych <= 98) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else {
    if ((int )yych <= 99) {
      goto yy1288;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy1288: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 75) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 74) {
        goto yy143;
      } else {

      }
    }
  } else
  if ((int )yych <= 106) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 107) {
      goto yy1289;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy4;
  }
  yy1289: 
  yyaccept = 0U;
  cursor ++;
  tmp___404 = cursor;
  s->ptr = tmp___404;
  yych = *tmp___404;
  if ((int )yych <= 41) {
    if ((int )yych == 32) {
      goto yy1290;
    } else {

    }
    if ((int )yych <= 40) {
      goto yy4;
    } else {

    }
    goto yy140;
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1290: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 79) {
    goto yy1291;
  } else {

  }
  if ((int )yych != 111) {
    goto yy57;
  } else {

  }
  yy1291: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 70) {
    goto yy1292;
  } else {

  }
  if ((int )yych != 102) {
    goto yy57;
  } else {

  }
  yy1292: 
  cursor ++;
  yych = *cursor;
  if ((int )yych != 32) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 49) {
    goto yy1294;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy1296;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy1297;
  } else {

  }
  goto yy57;
  yy1294: 
  yyaccept = 28U;
  cursor ++;
  tmp___405 = cursor;
  s->ptr = tmp___405;
  yych = *tmp___405;
  if ((int )yych <= 47) {
    goto yy1298;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy1297;
  } else {

  }
  goto yy1298;
  yy1295: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  (s->time)->have_time = 0U;
  (s->time)->h = (timelib_sll )0;
  (s->time)->i = (timelib_sll )0;
  (s->time)->s = (timelib_sll )0;
  (s->time)->f = (double )0;
  if ((s->time)->have_time) {
    add_error(s, (char *)"Double time specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_time = 1U;
    (s->time)->h = (timelib_sll )0;
    (s->time)->i = (timelib_sll )0;
    (s->time)->s = (timelib_sll )0;
    (s->time)->f = (double )0;
  }
  if ((int )*ptr == 98) {
    (s->time)->h = timelib_get_nr(& ptr, 2);
    (s->time)->i = (timelib_sll )15;
  } else {
    tmp___406 = timelib_get_nr(& ptr, 2);
    (s->time)->h = tmp___406 - 1LL;
    (s->time)->i = (timelib_sll )45;
  }
  if ((int )*ptr != 0) {
    timelib_eat_spaces(& ptr);
    tmp___407 = timelib_meridian(& ptr, (s->time)->h);
    (s->time)->h += tmp___407;
  } else {

  }
  free((void *)str);
  return (280);
  yy1296: 
  yyaccept = 28U;
  cursor ++;
  tmp___408 = cursor;
  s->ptr = tmp___408;
  yych = *tmp___408;
  if ((int )yych <= 47) {
    goto yy1298;
  } else {

  }
  if ((int )yych >= 53) {
    goto yy1298;
  } else {

  }
  yy1297: 
  yyaccept = 28U;
  cursor ++;
  s->ptr = cursor;
  if (s->lim - cursor < 5) {
    return (257);
  } else {

  }
  yych = *cursor;
  yy1298: ;
  if ((int )yych <= 65) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy1297;
      } else {

      }
      goto yy1295;
    } else {
      if ((int )yych <= 32) {
        goto yy1297;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy1295;
      } else {

      }
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych != 80) {
      goto yy1295;
    } else {

    }
  } else {
    if ((int )yych <= 97) {
      goto yy1299;
    } else {

    }
    if ((int )yych != 112) {
      goto yy1295;
    } else {

    }
  }
  yy1299: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 76) {
    if ((int )yych != 46) {
      goto yy57;
    } else {

    }
  } else {
    if ((int )yych <= 77) {
      goto yy1301;
    } else {

    }
    if ((int )yych == 109) {
      goto yy1301;
    } else {

    }
    goto yy57;
  }
  cursor ++;
  yych = *cursor;
  if ((int )yych == 77) {
    goto yy1301;
  } else {

  }
  if ((int )yych != 109) {
    goto yy57;
  } else {

  }
  yy1301: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 31) {
    if ((int )yych <= 0) {
      goto yy1303;
    } else {

    }
    if ((int )yych == 9) {
      goto yy1303;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 32) {
      goto yy1303;
    } else {

    }
    if ((int )yych != 46) {
      goto yy57;
    } else {

    }
  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 9) {
    if ((int )yych <= 0) {
      goto yy1303;
    } else {

    }
    if ((int )yych <= 8) {
      goto yy57;
    } else {

    }
  } else
  if ((int )yych != 32) {
    goto yy57;
  } else {

  }
  yy1303: 
  cursor ++;
  yych = *cursor;
  goto yy1295;
  yy1304: 
  yyaccept = 0U;
  cursor ++;
  tmp___409 = cursor;
  s->ptr = tmp___409;
  yych = *tmp___409;
  if ((int )yych <= 66) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 67) {
      goto yy1288;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 98) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 99) {
      goto yy1305;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy1305: 
  yyaccept = 0U;
  cursor ++;
  tmp___410 = cursor;
  s->ptr = tmp___410;
  yych = *tmp___410;
  if ((int )yych <= 74) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 75) {
      goto yy1289;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 106) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 107) {
      goto yy1306;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy4;
  }
  yy1306: 
  yyaccept = 0U;
  cursor ++;
  tmp___411 = cursor;
  s->ptr = tmp___411;
  yych = *tmp___411;
  if ((int )yych <= 46) {
    if ((int )yych <= 40) {
      if ((int )yych == 32) {
        goto yy1290;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych <= 41) {
        goto yy140;
      } else {

      }
      if ((int )yych == 45) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 94) {
    if ((int )yych <= 47) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 95) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1307: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 83) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 82) {
        goto yy142;
      } else {

      }
    }
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else {
    if ((int )yych <= 115) {
      goto yy1308;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy1308: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 84) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 83) {
        goto yy143;
      } else {

      }
    }
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 116) {
      goto yy1309;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy4;
  }
  yy1309: 
  yyaccept = 0U;
  cursor ++;
  tmp___412 = cursor;
  s->ptr = tmp___412;
  yych = *tmp___412;
  if ((int )yych <= 40) {
    if ((int )yych <= 9) {
      if ((int )yych <= 8) {
        goto yy4;
      } else {

      }
      goto yy1105;
    } else
    if ((int )yych != 32) {
      goto yy4;
    } else {

    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 41) {
      goto yy140;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1310: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 68) {
    goto yy1311;
  } else {

  }
  if ((int )yych != 100) {
    goto yy1106;
  } else {

  }
  yy1311: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy1312;
  } else {

  }
  if ((int )yych != 97) {
    goto yy57;
  } else {

  }
  yy1312: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy1313;
  } else {

  }
  if ((int )yych != 121) {
    goto yy57;
  } else {

  }
  yy1313: 
  yyaccept = 26U;
  cursor ++;
  tmp___413 = cursor;
  s->ptr = tmp___413;
  yych = *tmp___413;
  if ((int )yych <= 82) {
    if ((int )yych != 32) {
      goto yy1118;
    } else {

    }
  } else {
    if ((int )yych <= 83) {
      goto yy1143;
    } else {

    }
    if ((int )yych == 115) {
      goto yy1143;
    } else {

    }
    goto yy1118;
  }
  cursor ++;
  yych = *cursor;
  if ((int )yych == 79) {
    goto yy1315;
  } else {

  }
  if ((int )yych != 111) {
    goto yy57;
  } else {

  }
  yy1315: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 70) {
    goto yy1316;
  } else {

  }
  if ((int )yych != 102) {
    goto yy57;
  } else {

  }
  yy1316: 
  cursor ++;
  yych = *cursor;
  goto yy2;
  yy1317: 
  yyaccept = 0U;
  cursor ++;
  tmp___414 = cursor;
  s->ptr = tmp___414;
  yych = *tmp___414;
  if ((int )yych <= 82) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 83) {
      goto yy1308;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 115) {
      goto yy1318;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy1318: 
  yyaccept = 0U;
  cursor ++;
  tmp___415 = cursor;
  s->ptr = tmp___415;
  yych = *tmp___415;
  if ((int )yych <= 83) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 84) {
      goto yy1309;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 116) {
      goto yy1319;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy4;
  }
  yy1319: 
  yyaccept = 0U;
  cursor ++;
  tmp___416 = cursor;
  s->ptr = tmp___416;
  yych = *tmp___416;
  if ((int )yych <= 45) {
    if ((int )yych <= 32) {
      if ((int )yych == 9) {
        goto yy1105;
      } else {

      }
      if ((int )yych <= 31) {
        goto yy4;
      } else {

      }
      goto yy1310;
    } else {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych == 47) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1320: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 66) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 65) {
        goto yy142;
      } else {

      }
      goto yy1356;
    }
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else {
    if ((int )yych <= 98) {
      goto yy1356;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy1321: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 82) {
    if ((int )yych <= 64) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 70) {
        goto yy1346;
      } else {

      }
      if ((int )yych <= 81) {
        goto yy142;
      } else {

      }
      goto yy1345;
    }
  } else
  if ((int )yych <= 102) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 101) {
      goto yy142;
    } else {

    }
    goto yy1346;
  } else {
    if ((int )yych == 114) {
      goto yy1345;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy1322: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 84) {
        goto yy142;
      } else {

      }
      goto yy1342;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else {
    if ((int )yych <= 117) {
      goto yy1342;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy1323: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 79) {
    if ((int )yych <= 64) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 73) {
        goto yy1325;
      } else {

      }
      if ((int )yych <= 78) {
        goto yy142;
      } else {

      }
    }
  } else
  if ((int )yych <= 105) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 104) {
      goto yy142;
    } else {

    }
    goto yy1325;
  } else {
    if ((int )yych == 111) {
      goto yy1324;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy1324: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 78) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 77) {
        goto yy143;
      } else {

      }
      goto yy1328;
    }
  } else
  if ((int )yych <= 109) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 110) {
      goto yy1328;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy4;
  }
  yy1325: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 68) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy167;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy167;
      } else {

      }
      if ((int )yych <= 67) {
        goto yy143;
      } else {

      }
    }
  } else
  if ((int )yych <= 99) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy167;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 100) {
      goto yy1326;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy167;
  }
  yy1326: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 65) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 97) {
      goto yy1327;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1327: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 89) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 88) {
        goto yy145;
      } else {

      }
      goto yy1236;
    }
  } else
  if ((int )yych <= 120) {
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy145;
  } else {
    if ((int )yych <= 121) {
      goto yy1236;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy4;
  }
  yy1328: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 84) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 83) {
        goto yy144;
      } else {

      }
    }
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 116) {
      goto yy1329;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1329: 
  yyaccept = 0U;
  cursor ++;
  tmp___417 = cursor;
  s->ptr = tmp___417;
  yych = *tmp___417;
  if ((int )yych <= 41) {
    if ((int )yych == 32) {
      goto yy1330;
    } else {

    }
    if ((int )yych <= 40) {
      goto yy4;
    } else {

    }
    goto yy140;
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    goto yy145;
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy4;
  }
  yy1330: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 79) {
    goto yy1331;
  } else {

  }
  if ((int )yych != 111) {
    goto yy57;
  } else {

  }
  yy1331: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 70) {
    goto yy1332;
  } else {

  }
  if ((int )yych != 102) {
    goto yy57;
  } else {

  }
  yy1332: 
  cursor ++;
  yych = *cursor;
  if ((int )yych != 32) {
    goto yy57;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 49) {
    goto yy1334;
  } else {

  }
  if ((int )yych <= 50) {
    goto yy1335;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy1336;
  } else {

  }
  goto yy57;
  yy1334: 
  yyaccept = 28U;
  cursor ++;
  tmp___418 = cursor;
  s->ptr = tmp___418;
  yych = *tmp___418;
  if ((int )yych <= 47) {
    goto yy1337;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy1336;
  } else {

  }
  goto yy1337;
  yy1335: 
  yyaccept = 28U;
  cursor ++;
  tmp___419 = cursor;
  s->ptr = tmp___419;
  yych = *tmp___419;
  if ((int )yych <= 47) {
    goto yy1337;
  } else {

  }
  if ((int )yych >= 53) {
    goto yy1337;
  } else {

  }
  yy1336: 
  yyaccept = 28U;
  cursor ++;
  s->ptr = cursor;
  if (s->lim - cursor < 5) {
    return (257);
  } else {

  }
  yych = *cursor;
  yy1337: ;
  if ((int )yych <= 65) {
    if ((int )yych <= 31) {
      if ((int )yych == 9) {
        goto yy1336;
      } else {

      }
      goto yy1295;
    } else {
      if ((int )yych <= 32) {
        goto yy1336;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy1295;
      } else {

      }
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych != 80) {
      goto yy1295;
    } else {

    }
  } else {
    if ((int )yych <= 97) {
      goto yy1338;
    } else {

    }
    if ((int )yych != 112) {
      goto yy1295;
    } else {

    }
  }
  yy1338: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 76) {
    if ((int )yych != 46) {
      goto yy57;
    } else {

    }
  } else {
    if ((int )yych <= 77) {
      goto yy1340;
    } else {

    }
    if ((int )yych == 109) {
      goto yy1340;
    } else {

    }
    goto yy57;
  }
  cursor ++;
  yych = *cursor;
  if ((int )yych == 77) {
    goto yy1340;
  } else {

  }
  if ((int )yych != 109) {
    goto yy57;
  } else {

  }
  yy1340: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 31) {
    if ((int )yych <= 0) {
      goto yy1303;
    } else {

    }
    if ((int )yych == 9) {
      goto yy1303;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 32) {
      goto yy1303;
    } else {

    }
    if ((int )yych != 46) {
      goto yy57;
    } else {

    }
  }
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 9) {
    if ((int )yych <= 0) {
      goto yy1303;
    } else {

    }
    if ((int )yych <= 8) {
      goto yy57;
    } else {

    }
    goto yy1303;
  } else {
    if ((int )yych == 32) {
      goto yy1303;
    } else {

    }
    goto yy57;
  }
  yy1342: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 82) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 81) {
        goto yy143;
      } else {

      }
    }
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 114) {
      goto yy1343;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy4;
  }
  yy1343: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 84) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 83) {
        goto yy144;
      } else {

      }
    }
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 116) {
      goto yy1344;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1344: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 72) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 71) {
        goto yy145;
      } else {

      }
      goto yy1216;
    }
  } else
  if ((int )yych <= 103) {
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy145;
  } else {
    if ((int )yych <= 104) {
      goto yy1216;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy4;
  }
  yy1345: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 83) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 82) {
        goto yy143;
      } else {

      }
      goto yy1348;
    }
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 115) {
      goto yy1348;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy4;
  }
  yy1346: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 84) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 83) {
        goto yy143;
      } else {

      }
    }
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 116) {
      goto yy1347;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy4;
  }
  yy1347: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 72) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 71) {
        goto yy144;
      } else {

      }
      goto yy1239;
    }
  } else
  if ((int )yych <= 103) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 104) {
      goto yy1239;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1348: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 84) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 83) {
        goto yy144;
      } else {

      }
    }
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 116) {
      goto yy1349;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1349: 
  yyaccept = 0U;
  cursor ++;
  tmp___420 = cursor;
  s->ptr = tmp___420;
  yych = *tmp___420;
  if ((int )yych <= 40) {
    if ((int )yych <= 9) {
      if ((int )yych <= 8) {
        goto yy4;
      } else {

      }
      goto yy1207;
    } else
    if ((int )yych != 32) {
      goto yy4;
    } else {

    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 41) {
      goto yy140;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    goto yy145;
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy4;
  }
  yy1350: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 68) {
    goto yy1351;
  } else {

  }
  if ((int )yych != 100) {
    goto yy1208;
  } else {

  }
  yy1351: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy1352;
  } else {

  }
  if ((int )yych != 97) {
    goto yy57;
  } else {

  }
  yy1352: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy1353;
  } else {

  }
  if ((int )yych != 121) {
    goto yy57;
  } else {

  }
  yy1353: 
  yyaccept = 26U;
  cursor ++;
  tmp___421 = cursor;
  s->ptr = tmp___421;
  yych = *tmp___421;
  if ((int )yych <= 82) {
    if ((int )yych != 32) {
      goto yy1118;
    } else {

    }
  } else {
    if ((int )yych <= 83) {
      goto yy1143;
    } else {

    }
    if ((int )yych == 115) {
      goto yy1143;
    } else {

    }
    goto yy1118;
  }
  cursor ++;
  yych = *cursor;
  if ((int )yych == 79) {
    goto yy1355;
  } else {

  }
  if ((int )yych != 111) {
    goto yy57;
  } else {

  }
  yy1355: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 70) {
    goto yy1316;
  } else {

  }
  if ((int )yych == 102) {
    goto yy1316;
  } else {

  }
  goto yy57;
  yy1356: 
  yyaccept = 5U;
  cursor ++;
  tmp___422 = cursor;
  s->ptr = tmp___422;
  yych = *tmp___422;
  if ((int )yych <= 47) {
    if ((int )yych <= 40) {
      if ((int )yych <= 9) {
        if ((int )yych <= 8) {
          goto yy194;
        } else {

        }
        goto yy196;
      } else {
        if ((int )yych == 32) {
          goto yy196;
        } else {

        }
        goto yy194;
      }
    } else
    if ((int )yych <= 44) {
      if ((int )yych <= 41) {
        goto yy140;
      } else {

      }
      goto yy194;
    } else {
      if ((int )yych <= 45) {
        goto yy197;
      } else {

      }
      if ((int )yych <= 46) {
        goto yy196;
      } else {

      }
      goto yy194;
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 64) {
      if ((int )yych <= 57) {
        goto yy196;
      } else {

      }
      goto yy194;
    } else
    if ((int )yych != 82) {
      goto yy143;
    } else {

    }
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 114) {
      goto yy1357;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy194;
  }
  yy1357: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 84) {
        goto yy144;
      } else {

      }
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 117) {
      goto yy1358;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1358: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 65) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 97) {
      goto yy1359;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy4;
  }
  yy1359: 
  yyaccept = 0U;
  cursor ++;
  tmp___423 = cursor;
  s->ptr = tmp___423;
  yych = *tmp___423;
  if ((int )yych <= 81) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 82) {
      goto yy1360;
    } else {

    }
    if ((int )yych != 114) {
      goto yy4;
    } else {

    }
  }
  yy1360: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy206;
  } else {

  }
  if ((int )yych == 121) {
    goto yy206;
  } else {

  }
  goto yy57;
  yy1361: 
  yyaccept = 0U;
  cursor ++;
  tmp___424 = cursor;
  s->ptr = tmp___424;
  yych = *tmp___424;
  if ((int )yych <= 65) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 66) {
      goto yy1356;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 98) {
      goto yy1379;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy1362: 
  yyaccept = 0U;
  cursor ++;
  tmp___425 = cursor;
  s->ptr = tmp___425;
  yych = *tmp___425;
  if ((int )yych <= 81) {
    if ((int )yych <= 46) {
      if ((int )yych <= 41) {
        if ((int )yych <= 40) {
          goto yy4;
        } else {

        }
        goto yy140;
      } else {
        if ((int )yych == 45) {
          goto yy148;
        } else {

        }
        goto yy4;
      }
    } else
    if ((int )yych <= 64) {
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 70) {
        goto yy1346;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 90) {
      if ((int )yych <= 82) {
        goto yy1345;
      } else {

      }
      goto yy142;
    } else {
      if ((int )yych == 95) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 113) {
    if ((int )yych == 102) {
      goto yy1375;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 114) {
      goto yy1374;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy1363: 
  yyaccept = 0U;
  cursor ++;
  tmp___426 = cursor;
  s->ptr = tmp___426;
  yych = *tmp___426;
  if ((int )yych <= 84) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 85) {
      goto yy1342;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 116) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 117) {
      goto yy1371;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy1364: 
  yyaccept = 0U;
  cursor ++;
  tmp___427 = cursor;
  s->ptr = tmp___427;
  yych = *tmp___427;
  if ((int )yych <= 78) {
    if ((int )yych <= 46) {
      if ((int )yych <= 41) {
        if ((int )yych <= 40) {
          goto yy4;
        } else {

        }
        goto yy140;
      } else {
        if ((int )yych == 45) {
          goto yy148;
        } else {

        }
        goto yy4;
      }
    } else
    if ((int )yych <= 64) {
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 73) {
        goto yy1325;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 90) {
      if ((int )yych <= 79) {
        goto yy1324;
      } else {

      }
      goto yy142;
    } else {
      if ((int )yych == 95) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 110) {
    if ((int )yych == 105) {
      goto yy1366;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 111) {
      goto yy1365;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy1365: 
  yyaccept = 0U;
  cursor ++;
  tmp___428 = cursor;
  s->ptr = tmp___428;
  yych = *tmp___428;
  if ((int )yych <= 77) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 78) {
      goto yy1328;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 109) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 110) {
      goto yy1369;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy4;
  }
  yy1366: 
  yyaccept = 4U;
  cursor ++;
  tmp___429 = cursor;
  s->ptr = tmp___429;
  yych = *tmp___429;
  if ((int )yych <= 67) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy167;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy167;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 68) {
      goto yy1326;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy167;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 99) {
    if ((int )yych <= 96) {
      goto yy167;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 100) {
      goto yy1367;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy167;
  }
  yy1367: 
  yyaccept = 0U;
  cursor ++;
  tmp___430 = cursor;
  s->ptr = tmp___430;
  yych = *tmp___430;
  if ((int )yych <= 64) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 65) {
      goto yy1327;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 97) {
      goto yy1368;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1368: 
  yyaccept = 0U;
  cursor ++;
  tmp___431 = cursor;
  s->ptr = tmp___431;
  yych = *tmp___431;
  if ((int )yych <= 88) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy145;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 89) {
      goto yy1236;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 120) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy153;
  } else {
    if ((int )yych <= 121) {
      goto yy1265;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy4;
  }
  yy1369: 
  yyaccept = 0U;
  cursor ++;
  tmp___432 = cursor;
  s->ptr = tmp___432;
  yych = *tmp___432;
  if ((int )yych <= 83) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 84) {
      goto yy1329;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 116) {
      goto yy1370;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1370: 
  yyaccept = 0U;
  cursor ++;
  tmp___433 = cursor;
  s->ptr = tmp___433;
  yych = *tmp___433;
  if ((int )yych <= 46) {
    if ((int )yych <= 40) {
      if ((int )yych == 32) {
        goto yy1330;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych <= 41) {
        goto yy140;
      } else {

      }
      if ((int )yych == 45) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 94) {
    if ((int )yych <= 47) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 95) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy4;
  }
  yy1371: 
  yyaccept = 0U;
  cursor ++;
  tmp___434 = cursor;
  s->ptr = tmp___434;
  yych = *tmp___434;
  if ((int )yych <= 81) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 82) {
      goto yy1343;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 114) {
      goto yy1372;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy4;
  }
  yy1372: 
  yyaccept = 0U;
  cursor ++;
  tmp___435 = cursor;
  s->ptr = tmp___435;
  yych = *tmp___435;
  if ((int )yych <= 83) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 84) {
      goto yy1344;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 116) {
      goto yy1373;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1373: 
  yyaccept = 0U;
  cursor ++;
  tmp___436 = cursor;
  s->ptr = tmp___436;
  yych = *tmp___436;
  if ((int )yych <= 71) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy145;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 72) {
      goto yy1216;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 103) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy153;
  } else {
    if ((int )yych <= 104) {
      goto yy1228;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy4;
  }
  yy1374: 
  yyaccept = 0U;
  cursor ++;
  tmp___437 = cursor;
  s->ptr = tmp___437;
  yych = *tmp___437;
  if ((int )yych <= 82) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 83) {
      goto yy1348;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 115) {
      goto yy1377;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy4;
  }
  yy1375: 
  yyaccept = 0U;
  cursor ++;
  tmp___438 = cursor;
  s->ptr = tmp___438;
  yych = *tmp___438;
  if ((int )yych <= 83) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 84) {
      goto yy1347;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 116) {
      goto yy1376;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy4;
  }
  yy1376: 
  yyaccept = 0U;
  cursor ++;
  tmp___439 = cursor;
  s->ptr = tmp___439;
  yych = *tmp___439;
  if ((int )yych <= 71) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 72) {
      goto yy1239;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 103) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 104) {
      goto yy1268;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1377: 
  yyaccept = 0U;
  cursor ++;
  tmp___440 = cursor;
  s->ptr = tmp___440;
  yych = *tmp___440;
  if ((int )yych <= 83) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 84) {
      goto yy1349;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 116) {
      goto yy1378;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1378: 
  yyaccept = 0U;
  cursor ++;
  tmp___441 = cursor;
  s->ptr = tmp___441;
  yych = *tmp___441;
  if ((int )yych <= 45) {
    if ((int )yych <= 32) {
      if ((int )yych == 9) {
        goto yy1207;
      } else {

      }
      if ((int )yych <= 31) {
        goto yy4;
      } else {

      }
      goto yy1350;
    } else {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych == 47) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    goto yy145;
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy4;
  }
  yy1379: 
  yyaccept = 5U;
  cursor ++;
  tmp___442 = cursor;
  s->ptr = tmp___442;
  yych = *tmp___442;
  if ((int )yych <= 57) {
    if ((int )yych <= 40) {
      if ((int )yych <= 9) {
        if ((int )yych <= 8) {
          goto yy194;
        } else {

        }
        goto yy196;
      } else {
        if ((int )yych == 32) {
          goto yy196;
        } else {

        }
        goto yy194;
      }
    } else
    if ((int )yych <= 45) {
      if ((int )yych <= 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy194;
      } else {

      }
      goto yy372;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      goto yy196;
    }
  } else
  if ((int )yych <= 94) {
    if ((int )yych <= 81) {
      if ((int )yych <= 64) {
        goto yy194;
      } else {

      }
      goto yy143;
    } else {
      if ((int )yych <= 82) {
        goto yy1357;
      } else {

      }
      if ((int )yych <= 90) {
        goto yy143;
      } else {

      }
      goto yy194;
    }
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 95) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 114) {
      goto yy1380;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy194;
  }
  yy1380: 
  yyaccept = 0U;
  cursor ++;
  tmp___443 = cursor;
  s->ptr = tmp___443;
  yych = *tmp___443;
  if ((int )yych <= 84) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 85) {
      goto yy1358;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 116) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 117) {
      goto yy1381;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1381: 
  yyaccept = 0U;
  cursor ++;
  tmp___444 = cursor;
  s->ptr = tmp___444;
  yych = *tmp___444;
  if ((int )yych <= 64) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 65) {
      goto yy1359;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 97) {
      goto yy1382;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy4;
  }
  yy1382: 
  yyaccept = 0U;
  cursor ++;
  tmp___445 = cursor;
  s->ptr = tmp___445;
  yych = *tmp___445;
  if ((int )yych <= 81) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 82) {
      goto yy1360;
    } else {

    }
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych == 114) {
      goto yy1383;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy154;
    } else {

    }
    goto yy4;
  }
  yy1383: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy206;
  } else {

  }
  if ((int )yych == 121) {
    goto yy377;
  } else {

  }
  goto yy155;
  yy1384: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy57;
  } else {

  }
  yy1385: 
  cursor ++;
  if ((unsigned int )s->lim <= (unsigned int )cursor) {
    return (257);
  } else {

  }
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy1387;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy1385;
  } else {

  }
  yy1387: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  (s->time)->have_relative = 1U;
  (s->time)->have_date = 0U;
  (s->time)->d = (timelib_sll )0;
  (s->time)->m = (timelib_sll )0;
  (s->time)->y = (timelib_sll )0;
  (s->time)->have_time = 0U;
  (s->time)->h = (timelib_sll )0;
  (s->time)->i = (timelib_sll )0;
  (s->time)->s = (timelib_sll )0;
  (s->time)->f = (double )0;
  s->cur = cursor;
  if ((s->time)->have_zone) {
    if ((s->time)->have_zone > 1U) {
      add_error(s, (char *)"Double timezone specification");
    } else {
      add_warning(s, (char *)"Double timezone specification");
    }
    free((void *)str);
    ((s->time)->have_zone) ++;
    return (999);
  } else {
    ((s->time)->have_zone) ++;
  }
  i___3 = timelib_get_unsigned_nr(& ptr, 24);
  (s->time)->y = (timelib_sll )1970;
  (s->time)->m = (timelib_sll )1;
  (s->time)->d = (timelib_sll )1;
  tmp___447 = (timelib_sll )0;
  (s->time)->s = tmp___447;
  tmp___446 = tmp___447;
  (s->time)->i = tmp___446;
  (s->time)->h = tmp___446;
  (s->time)->f = 0.0;
  (s->time)->relative.s = (timelib_sll )((timelib_ull )(s->time)->relative.s + i___3);
  (s->time)->is_localtime = 1U;
  (s->time)->zone_type = 1U;
  (s->time)->z = 0;
  free((void *)str);
  return (310);
  yy1388: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 78) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 77) {
        goto yy142;
      } else {

      }
      goto yy1429;
    }
  } else
  if ((int )yych <= 109) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else {
    if ((int )yych <= 110) {
      goto yy1429;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy1389: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 85) {
    if ((int )yych <= 64) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 73) {
        goto yy1421;
      } else {

      }
      if ((int )yych <= 84) {
        goto yy142;
      } else {

      }
      goto yy1422;
    }
  } else
  if ((int )yych <= 105) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 104) {
      goto yy142;
    } else {

    }
    goto yy1421;
  } else {
    if ((int )yych == 117) {
      goto yy1422;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy1390: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 77) {
    if ((int )yych <= 64) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 68) {
        goto yy1410;
      } else {

      }
      if ((int )yych <= 76) {
        goto yy142;
      } else {

      }
      goto yy1411;
    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 99) {
      goto yy142;
    } else {

    }
    goto yy1410;
  } else {
    if ((int )yych == 109) {
      goto yy1411;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy1391: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 69) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 68) {
        goto yy142;
      } else {

      }
      goto yy1406;
    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else {
    if ((int )yych <= 101) {
      goto yy1406;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy1392: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 69) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 68) {
        goto yy142;
      } else {

      }
      goto yy1402;
    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else {
    if ((int )yych <= 101) {
      goto yy1402;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy1393: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy1065;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 57) {
      goto yy1396;
    } else {

    }
    if ((int )yych <= 58) {
      goto yy1065;
    } else {

    }
    goto yy57;
  }
  yy1394: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy1065;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 52) {
      goto yy1396;
    } else {

    }
    if ((int )yych == 58) {
      goto yy1065;
    } else {

    }
    goto yy57;
  }
  yy1395: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 46) {
    goto yy1065;
  } else {

  }
  if ((int )yych == 58) {
    goto yy1065;
  } else {

  }
  goto yy57;
  yy1396: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    if ((int )yych == 46) {
      goto yy1065;
    } else {

    }
    goto yy57;
  } else {
    if ((int )yych <= 53) {
      goto yy1397;
    } else {

    }
    if ((int )yych == 58) {
      goto yy1065;
    } else {

    }
    goto yy57;
  }
  yy1397: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych >= 58) {
    goto yy57;
  } else {

  }
  yyaccept = 24U;
  cursor ++;
  tmp___448 = cursor;
  s->ptr = tmp___448;
  yych = *tmp___448;
  if ((int )yych <= 47) {
    goto yy1068;
  } else {

  }
  if ((int )yych <= 53) {
    goto yy1399;
  } else {

  }
  if ((int )yych <= 54) {
    goto yy1400;
  } else {

  }
  goto yy1068;
  yy1399: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 47) {
    goto yy57;
  } else {

  }
  if ((int )yych <= 57) {
    goto yy1401;
  } else {

  }
  goto yy57;
  yy1400: 
  cursor ++;
  yych = *cursor;
  if ((int )yych != 48) {
    goto yy57;
  } else {

  }
  yy1401: 
  cursor ++;
  yych = *cursor;
  goto yy1076;
  yy1402: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 76) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 75) {
        goto yy143;
      } else {

      }
    }
  } else
  if ((int )yych <= 107) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 108) {
      goto yy1403;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy4;
  }
  yy1403: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 70) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 69) {
        goto yy144;
      } else {

      }
    }
  } else
  if ((int )yych <= 101) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 102) {
      goto yy1404;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1404: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 84) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 83) {
        goto yy145;
      } else {

      }
    }
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy145;
  } else {
    if ((int )yych <= 116) {
      goto yy1405;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy4;
  }
  yy1405: 
  yyaccept = 0U;
  cursor ++;
  tmp___449 = cursor;
  s->ptr = tmp___449;
  yych = *tmp___449;
  if ((int )yych <= 71) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 72) {
      goto yy1206;
    } else {

    }
    if ((int )yych == 104) {
      goto yy1206;
    } else {

    }
    goto yy4;
  }
  yy1406: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 83) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy167;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy167;
      } else {

      }
      if ((int )yych <= 82) {
        goto yy143;
      } else {

      }
    }
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy167;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 115) {
      goto yy1407;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy167;
  }
  yy1407: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 68) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 67) {
        goto yy144;
      } else {

      }
    }
  } else
  if ((int )yych <= 99) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 100) {
      goto yy1408;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1408: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 65) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 97) {
      goto yy1409;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy4;
  }
  yy1409: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 88) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 89) {
      goto yy173;
    } else {

    }
    if ((int )yych == 121) {
      goto yy173;
    } else {

    }
    goto yy4;
  }
  yy1410: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 65) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    goto yy1418;
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 97) {
      goto yy1418;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy4;
  }
  yy1411: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 79) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 78) {
        goto yy143;
      } else {

      }
    }
  } else
  if ((int )yych <= 110) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 111) {
      goto yy1412;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy4;
  }
  yy1412: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 82) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 81) {
        goto yy144;
      } else {

      }
    }
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 114) {
      goto yy1413;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1413: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 82) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 81) {
        goto yy145;
      } else {

      }
    }
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy145;
  } else {
    if ((int )yych <= 114) {
      goto yy1414;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy4;
  }
  yy1414: 
  yyaccept = 0U;
  cursor ++;
  tmp___450 = cursor;
  s->ptr = tmp___450;
  yych = *tmp___450;
  if ((int )yych <= 78) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 79) {
      goto yy1415;
    } else {

    }
    if ((int )yych != 111) {
      goto yy4;
    } else {

    }
  }
  yy1415: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 87) {
    goto yy1416;
  } else {

  }
  if ((int )yych != 119) {
    goto yy57;
  } else {

  }
  yy1416: 
  cursor ++;
  yy1417: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  (s->time)->have_relative = 1U;
  (s->time)->have_time = 0U;
  (s->time)->h = (timelib_sll )0;
  (s->time)->i = (timelib_sll )0;
  (s->time)->s = (timelib_sll )0;
  (s->time)->f = (double )0;
  (s->time)->relative.d = (timelib_sll )1;
  free((void *)str);
  return (310);
  yy1418: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 89) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 88) {
        goto yy144;
      } else {

      }
    }
  } else
  if ((int )yych <= 120) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 121) {
      goto yy1419;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1419: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 64) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
  } else {
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy1420;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
  }
  yy1420: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  (s->time)->have_time = 0U;
  (s->time)->h = (timelib_sll )0;
  (s->time)->i = (timelib_sll )0;
  (s->time)->s = (timelib_sll )0;
  (s->time)->f = (double )0;
  free((void *)str);
  return (310);
  yy1421: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 83) {
    if ((int )yych <= 64) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych <= 81) {
        goto yy143;
      } else {

      }
      if ((int )yych <= 82) {
        goto yy1427;
      } else {

      }
      goto yy1428;
    }
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 114) {
      goto yy1427;
    } else {

    }
    if ((int )yych <= 115) {
      goto yy1428;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy4;
  }
  yy1422: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 82) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy167;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy167;
      } else {

      }
      if ((int )yych <= 81) {
        goto yy143;
      } else {

      }
    }
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy167;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 114) {
      goto yy1423;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy167;
  }
  yy1423: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 83) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 82) {
        goto yy144;
      } else {

      }
    }
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 115) {
      goto yy1424;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1424: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 68) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 67) {
        goto yy145;
      } else {

      }
    }
  } else
  if ((int )yych <= 99) {
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy145;
  } else {
    if ((int )yych <= 100) {
      goto yy1425;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy4;
  }
  yy1425: 
  yyaccept = 0U;
  cursor ++;
  tmp___451 = cursor;
  s->ptr = tmp___451;
  yych = *tmp___451;
  if ((int )yych <= 64) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 65) {
      goto yy1426;
    } else {

    }
    if ((int )yych != 97) {
      goto yy4;
    } else {

    }
  }
  yy1426: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy173;
  } else {

  }
  if ((int )yych == 121) {
    goto yy173;
  } else {

  }
  goto yy57;
  yy1427: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 68) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 67) {
        goto yy144;
      } else {

      }
      goto yy1239;
    }
  } else
  if ((int )yych <= 99) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 100) {
      goto yy1239;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1428: 
  yyaccept = 0U;
  cursor ++;
  tmp___452 = cursor;
  s->ptr = tmp___452;
  yych = *tmp___452;
  if ((int )yych <= 40) {
    if ((int )yych <= 9) {
      if ((int )yych <= 8) {
        goto yy4;
      } else {

      }
      goto yy1105;
    } else {
      if ((int )yych == 32) {
        goto yy1105;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 41) {
      goto yy140;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1429: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 84) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 83) {
        goto yy143;
      } else {

      }
    }
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 116) {
      goto yy1430;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy4;
  }
  yy1430: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 72) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 71) {
        goto yy144;
      } else {

      }
      goto yy1239;
    }
  } else
  if ((int )yych <= 103) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 104) {
      goto yy1239;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1431: 
  yyaccept = 0U;
  cursor ++;
  tmp___453 = cursor;
  s->ptr = tmp___453;
  yych = *tmp___453;
  if ((int )yych <= 77) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 78) {
      goto yy1429;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 109) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 110) {
      goto yy1461;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy1432: 
  yyaccept = 0U;
  cursor ++;
  tmp___454 = cursor;
  s->ptr = tmp___454;
  yych = *tmp___454;
  if ((int )yych <= 84) {
    if ((int )yych <= 46) {
      if ((int )yych <= 41) {
        if ((int )yych <= 40) {
          goto yy4;
        } else {

        }
        goto yy140;
      } else {
        if ((int )yych == 45) {
          goto yy148;
        } else {

        }
        goto yy4;
      }
    } else
    if ((int )yych <= 64) {
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 73) {
        goto yy1421;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 90) {
      if ((int )yych <= 85) {
        goto yy1422;
      } else {

      }
      goto yy142;
    } else {
      if ((int )yych == 95) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 116) {
    if ((int )yych == 105) {
      goto yy1453;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 117) {
      goto yy1454;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy1433: 
  yyaccept = 0U;
  cursor ++;
  tmp___455 = cursor;
  s->ptr = tmp___455;
  yych = *tmp___455;
  if ((int )yych <= 76) {
    if ((int )yych <= 46) {
      if ((int )yych <= 41) {
        if ((int )yych <= 40) {
          goto yy4;
        } else {

        }
        goto yy140;
      } else {
        if ((int )yych == 45) {
          goto yy148;
        } else {

        }
        goto yy4;
      }
    } else
    if ((int )yych <= 64) {
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 68) {
        goto yy1410;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 90) {
      if ((int )yych <= 77) {
        goto yy1411;
      } else {

      }
      goto yy142;
    } else {
      if ((int )yych == 95) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 108) {
    if ((int )yych == 100) {
      goto yy1444;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 109) {
      goto yy1445;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy1434: 
  yyaccept = 0U;
  cursor ++;
  tmp___456 = cursor;
  s->ptr = tmp___456;
  yych = *tmp___456;
  if ((int )yych <= 68) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 69) {
      goto yy1406;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 101) {
      goto yy1440;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy1435: 
  yyaccept = 0U;
  cursor ++;
  tmp___457 = cursor;
  s->ptr = tmp___457;
  yych = *tmp___457;
  if ((int )yych <= 68) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 69) {
      goto yy1402;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 101) {
      goto yy1436;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy1436: 
  yyaccept = 0U;
  cursor ++;
  tmp___458 = cursor;
  s->ptr = tmp___458;
  yych = *tmp___458;
  if ((int )yych <= 75) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 76) {
      goto yy1403;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 107) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 108) {
      goto yy1437;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy4;
  }
  yy1437: 
  yyaccept = 0U;
  cursor ++;
  tmp___459 = cursor;
  s->ptr = tmp___459;
  yych = *tmp___459;
  if ((int )yych <= 69) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 70) {
      goto yy1404;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 101) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 102) {
      goto yy1438;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1438: 
  yyaccept = 0U;
  cursor ++;
  tmp___460 = cursor;
  s->ptr = tmp___460;
  yych = *tmp___460;
  if ((int )yych <= 83) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy145;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 84) {
      goto yy1405;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy153;
  } else {
    if ((int )yych <= 116) {
      goto yy1439;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy4;
  }
  yy1439: 
  yyaccept = 0U;
  cursor ++;
  tmp___461 = cursor;
  s->ptr = tmp___461;
  yych = *tmp___461;
  if ((int )yych <= 71) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 72) {
      goto yy1206;
    } else {

    }
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych == 104) {
      goto yy1224;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy154;
    } else {

    }
    goto yy4;
  }
  yy1440: 
  yyaccept = 4U;
  cursor ++;
  tmp___462 = cursor;
  s->ptr = tmp___462;
  yych = *tmp___462;
  if ((int )yych <= 82) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy167;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy167;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 83) {
      goto yy1407;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy167;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 96) {
      goto yy167;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 115) {
      goto yy1441;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy167;
  }
  yy1441: 
  yyaccept = 0U;
  cursor ++;
  tmp___463 = cursor;
  s->ptr = tmp___463;
  yych = *tmp___463;
  if ((int )yych <= 67) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 68) {
      goto yy1408;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 99) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 100) {
      goto yy1442;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1442: 
  yyaccept = 0U;
  cursor ++;
  tmp___464 = cursor;
  s->ptr = tmp___464;
  yych = *tmp___464;
  if ((int )yych <= 64) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 65) {
      goto yy1409;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 97) {
      goto yy1443;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy4;
  }
  yy1443: 
  yyaccept = 0U;
  cursor ++;
  tmp___465 = cursor;
  s->ptr = tmp___465;
  yych = *tmp___465;
  if ((int )yych <= 88) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 89) {
      goto yy173;
    } else {

    }
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych == 121) {
      goto yy186;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy154;
    } else {

    }
    goto yy4;
  }
  yy1444: 
  yyaccept = 0U;
  cursor ++;
  tmp___466 = cursor;
  s->ptr = tmp___466;
  yych = *tmp___466;
  if ((int )yych <= 64) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 65) {
      goto yy1418;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 97) {
      goto yy1451;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy4;
  }
  yy1445: 
  yyaccept = 0U;
  cursor ++;
  tmp___467 = cursor;
  s->ptr = tmp___467;
  yych = *tmp___467;
  if ((int )yych <= 78) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 79) {
      goto yy1412;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 110) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 111) {
      goto yy1446;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy4;
  }
  yy1446: 
  yyaccept = 0U;
  cursor ++;
  tmp___468 = cursor;
  s->ptr = tmp___468;
  yych = *tmp___468;
  if ((int )yych <= 81) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 82) {
      goto yy1413;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 114) {
      goto yy1447;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1447: 
  yyaccept = 0U;
  cursor ++;
  tmp___469 = cursor;
  s->ptr = tmp___469;
  yych = *tmp___469;
  if ((int )yych <= 81) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy145;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 82) {
      goto yy1414;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy153;
  } else {
    if ((int )yych <= 114) {
      goto yy1448;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy4;
  }
  yy1448: 
  yyaccept = 0U;
  cursor ++;
  tmp___470 = cursor;
  s->ptr = tmp___470;
  yych = *tmp___470;
  if ((int )yych <= 78) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 79) {
      goto yy1415;
    } else {

    }
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych == 111) {
      goto yy1449;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy154;
    } else {

    }
    goto yy4;
  }
  yy1449: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 87) {
    goto yy1416;
  } else {

  }
  if ((int )yych != 119) {
    goto yy155;
  } else {

  }
  yyaccept = 29U;
  cursor ++;
  tmp___471 = cursor;
  s->ptr = tmp___471;
  yych = *tmp___471;
  if ((int const   )yybm[(int )yych] & 16) {
    goto yy154;
  } else {

  }
  if ((int )yych <= 46) {
    if ((int )yych == 45) {
      goto yy148;
    } else {

    }
    goto yy1417;
  } else {
    if ((int )yych <= 47) {
      goto yy148;
    } else {

    }
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy1417;
  }
  yy1451: 
  yyaccept = 0U;
  cursor ++;
  tmp___472 = cursor;
  s->ptr = tmp___472;
  yych = *tmp___472;
  if ((int )yych <= 88) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 89) {
      goto yy1419;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 120) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 121) {
      goto yy1452;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1452: 
  yyaccept = 30U;
  cursor ++;
  tmp___473 = cursor;
  s->ptr = tmp___473;
  yych = *tmp___473;
  if ((int )yych <= 47) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy1420;
    } else {
      if ((int )yych == 46) {
        goto yy1420;
      } else {

      }
      goto yy148;
    }
  } else
  if ((int )yych <= 94) {
    if ((int )yych <= 64) {
      goto yy1420;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    goto yy1420;
  } else {
    if ((int )yych <= 95) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy1420;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy1420;
  }
  yy1453: 
  yyaccept = 0U;
  cursor ++;
  tmp___474 = cursor;
  s->ptr = tmp___474;
  yych = *tmp___474;
  if ((int )yych <= 82) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else
    if ((int )yych <= 47) {
      if ((int )yych <= 46) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 81) {
        goto yy143;
      } else {

      }
      goto yy1427;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 90) {
      if ((int )yych <= 83) {
        goto yy1428;
      } else {

      }
      goto yy143;
    } else {
      if ((int )yych == 95) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 113) {
      goto yy151;
    } else {

    }
    goto yy1459;
  } else {
    if ((int )yych <= 115) {
      goto yy1460;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy4;
  }
  yy1454: 
  yyaccept = 4U;
  cursor ++;
  tmp___475 = cursor;
  s->ptr = tmp___475;
  yych = *tmp___475;
  if ((int )yych <= 81) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy167;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy167;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 82) {
      goto yy1423;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy167;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 96) {
      goto yy167;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 114) {
      goto yy1455;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy167;
  }
  yy1455: 
  yyaccept = 0U;
  cursor ++;
  tmp___476 = cursor;
  s->ptr = tmp___476;
  yych = *tmp___476;
  if ((int )yych <= 82) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 83) {
      goto yy1424;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 115) {
      goto yy1456;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1456: 
  yyaccept = 0U;
  cursor ++;
  tmp___477 = cursor;
  s->ptr = tmp___477;
  yych = *tmp___477;
  if ((int )yych <= 67) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy145;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 68) {
      goto yy1425;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 99) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy153;
  } else {
    if ((int )yych <= 100) {
      goto yy1457;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy4;
  }
  yy1457: 
  yyaccept = 0U;
  cursor ++;
  tmp___478 = cursor;
  s->ptr = tmp___478;
  yych = *tmp___478;
  if ((int )yych <= 64) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 65) {
      goto yy1426;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 97) {
      goto yy1458;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy154;
    } else {

    }
    goto yy4;
  }
  yy1458: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy173;
  } else {

  }
  if ((int )yych == 121) {
    goto yy186;
  } else {

  }
  goto yy155;
  yy1459: 
  yyaccept = 0U;
  cursor ++;
  tmp___479 = cursor;
  s->ptr = tmp___479;
  yych = *tmp___479;
  if ((int )yych <= 67) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 68) {
      goto yy1239;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 99) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 100) {
      goto yy1268;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1460: 
  yyaccept = 0U;
  cursor ++;
  tmp___480 = cursor;
  s->ptr = tmp___480;
  yych = *tmp___480;
  if ((int )yych <= 45) {
    if ((int )yych <= 32) {
      if ((int )yych == 9) {
        goto yy1105;
      } else {

      }
      if ((int )yych <= 31) {
        goto yy4;
      } else {

      }
      goto yy1105;
    } else {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych == 47) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1461: 
  yyaccept = 0U;
  cursor ++;
  tmp___481 = cursor;
  s->ptr = tmp___481;
  yych = *tmp___481;
  if ((int )yych <= 83) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 84) {
      goto yy1430;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 116) {
      goto yy1462;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy4;
  }
  yy1462: 
  yyaccept = 0U;
  cursor ++;
  tmp___482 = cursor;
  s->ptr = tmp___482;
  yych = *tmp___482;
  if ((int )yych <= 71) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 72) {
      goto yy1239;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 103) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 104) {
      goto yy1268;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1463: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 89) {
    if ((int )yych <= 64) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 82) {
        goto yy1475;
      } else {

      }
      if ((int )yych <= 88) {
        goto yy142;
      } else {

      }
      goto yy1476;
    }
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 113) {
      goto yy142;
    } else {

    }
    goto yy1475;
  } else {
    if ((int )yych == 121) {
      goto yy1476;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy1464: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 68) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 67) {
        goto yy142;
      } else {

      }
      goto yy1469;
    }
  } else
  if ((int )yych <= 99) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else {
    if ((int )yych <= 100) {
      goto yy1469;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy1465: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 78) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 77) {
        goto yy142;
      } else {

      }
    }
  } else
  if ((int )yych <= 109) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else {
    if ((int )yych <= 110) {
      goto yy1466;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy1466: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 68) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy167;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy167;
      } else {

      }
      if ((int )yych <= 67) {
        goto yy143;
      } else {

      }
    }
  } else
  if ((int )yych <= 99) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy167;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 100) {
      goto yy1467;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy167;
  }
  yy1467: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 65) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy4;
    } else {

    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 97) {
      goto yy1468;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1468: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 89) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 88) {
        goto yy145;
      } else {

      }
      goto yy1236;
    }
  } else
  if ((int )yych <= 120) {
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy145;
  } else {
    if ((int )yych <= 121) {
      goto yy1236;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy4;
  }
  yy1469: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 78) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 77) {
        goto yy143;
      } else {

      }
    }
  } else
  if ((int )yych <= 109) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 110) {
      goto yy1470;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy4;
  }
  yy1470: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 73) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 72) {
        goto yy144;
      } else {

      }
    }
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 105) {
      goto yy1471;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1471: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 71) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 70) {
        goto yy145;
      } else {

      }
    }
  } else
  if ((int )yych <= 102) {
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy145;
  } else {
    if ((int )yych <= 103) {
      goto yy1472;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy4;
  }
  yy1472: 
  yyaccept = 0U;
  cursor ++;
  tmp___483 = cursor;
  s->ptr = tmp___483;
  yych = *tmp___483;
  if ((int )yych <= 71) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 72) {
      goto yy1473;
    } else {

    }
    if ((int )yych != 104) {
      goto yy4;
    } else {

    }
  }
  yy1473: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 84) {
    goto yy1474;
  } else {

  }
  if ((int )yych != 116) {
    goto yy57;
  } else {

  }
  yy1474: 
  cursor ++;
  yych = *cursor;
  goto yy1420;
  yy1475: 
  yyaccept = 5U;
  cursor ++;
  tmp___484 = cursor;
  s->ptr = tmp___484;
  yych = *tmp___484;
  if ((int )yych <= 47) {
    if ((int )yych <= 40) {
      if ((int )yych <= 9) {
        if ((int )yych <= 8) {
          goto yy194;
        } else {

        }
        goto yy196;
      } else {
        if ((int )yych == 32) {
          goto yy196;
        } else {

        }
        goto yy194;
      }
    } else
    if ((int )yych <= 44) {
      if ((int )yych <= 41) {
        goto yy140;
      } else {

      }
      goto yy194;
    } else {
      if ((int )yych <= 45) {
        goto yy197;
      } else {

      }
      if ((int )yych <= 46) {
        goto yy196;
      } else {

      }
      goto yy194;
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 64) {
      if ((int )yych <= 57) {
        goto yy196;
      } else {

      }
      goto yy194;
    } else {
      if ((int )yych == 67) {
        goto yy1477;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 98) {
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 99) {
      goto yy1477;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy194;
  }
  yy1476: 
  yyaccept = 5U;
  cursor ++;
  tmp___485 = cursor;
  s->ptr = tmp___485;
  yych = *tmp___485;
  if ((int )yych <= 45) {
    if ((int )yych <= 32) {
      if ((int )yych == 9) {
        goto yy196;
      } else {

      }
      if ((int )yych <= 31) {
        goto yy194;
      } else {

      }
      goto yy196;
    } else {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy194;
      } else {

      }
      goto yy197;
    }
  } else
  if ((int )yych <= 64) {
    if ((int )yych == 47) {
      goto yy194;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy196;
    } else {

    }
    goto yy194;
  } else {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy194;
  }
  yy1477: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 72) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 71) {
        goto yy144;
      } else {

      }
      goto yy396;
    }
  } else
  if ((int )yych <= 103) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 104) {
      goto yy396;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1478: 
  yyaccept = 0U;
  cursor ++;
  tmp___486 = cursor;
  s->ptr = tmp___486;
  yych = *tmp___486;
  if ((int )yych <= 88) {
    if ((int )yych <= 46) {
      if ((int )yych <= 41) {
        if ((int )yych <= 40) {
          goto yy4;
        } else {

        }
        goto yy140;
      } else {
        if ((int )yych == 45) {
          goto yy148;
        } else {

        }
        goto yy4;
      }
    } else
    if ((int )yych <= 64) {
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 82) {
        goto yy1475;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 90) {
      if ((int )yych <= 89) {
        goto yy1476;
      } else {

      }
      goto yy142;
    } else {
      if ((int )yych == 95) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 120) {
    if ((int )yych == 114) {
      goto yy1490;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 121) {
      goto yy1491;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy1479: 
  yyaccept = 0U;
  cursor ++;
  tmp___487 = cursor;
  s->ptr = tmp___487;
  yych = *tmp___487;
  if ((int )yych <= 67) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 68) {
      goto yy1469;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 99) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 100) {
      goto yy1484;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy1480: 
  yyaccept = 0U;
  cursor ++;
  tmp___488 = cursor;
  s->ptr = tmp___488;
  yych = *tmp___488;
  if ((int )yych <= 77) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 78) {
      goto yy1466;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 109) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 110) {
      goto yy1481;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy1481: 
  yyaccept = 4U;
  cursor ++;
  tmp___489 = cursor;
  s->ptr = tmp___489;
  yych = *tmp___489;
  if ((int )yych <= 67) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy167;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy167;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 68) {
      goto yy1467;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy167;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 99) {
    if ((int )yych <= 96) {
      goto yy167;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 100) {
      goto yy1482;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy167;
  }
  yy1482: 
  yyaccept = 0U;
  cursor ++;
  tmp___490 = cursor;
  s->ptr = tmp___490;
  yych = *tmp___490;
  if ((int )yych <= 64) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 65) {
      goto yy1468;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 97) {
      goto yy1483;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1483: 
  yyaccept = 0U;
  cursor ++;
  tmp___491 = cursor;
  s->ptr = tmp___491;
  yych = *tmp___491;
  if ((int )yych <= 88) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy145;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 89) {
      goto yy1236;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 120) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy153;
  } else {
    if ((int )yych <= 121) {
      goto yy1265;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy4;
  }
  yy1484: 
  yyaccept = 0U;
  cursor ++;
  tmp___492 = cursor;
  s->ptr = tmp___492;
  yych = *tmp___492;
  if ((int )yych <= 77) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 78) {
      goto yy1470;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 109) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 110) {
      goto yy1485;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy4;
  }
  yy1485: 
  yyaccept = 0U;
  cursor ++;
  tmp___493 = cursor;
  s->ptr = tmp___493;
  yych = *tmp___493;
  if ((int )yych <= 72) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 73) {
      goto yy1471;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 104) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 105) {
      goto yy1486;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1486: 
  yyaccept = 0U;
  cursor ++;
  tmp___494 = cursor;
  s->ptr = tmp___494;
  yych = *tmp___494;
  if ((int )yych <= 70) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy145;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 71) {
      goto yy1472;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 102) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy153;
  } else {
    if ((int )yych <= 103) {
      goto yy1487;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy4;
  }
  yy1487: 
  yyaccept = 0U;
  cursor ++;
  tmp___495 = cursor;
  s->ptr = tmp___495;
  yych = *tmp___495;
  if ((int )yych <= 71) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 72) {
      goto yy1473;
    } else {

    }
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych == 104) {
      goto yy1488;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy154;
    } else {

    }
    goto yy4;
  }
  yy1488: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 84) {
    goto yy1474;
  } else {

  }
  if ((int )yych != 116) {
    goto yy155;
  } else {

  }
  yyaccept = 30U;
  cursor ++;
  tmp___496 = cursor;
  s->ptr = tmp___496;
  yych = *tmp___496;
  if ((int const   )yybm[(int )yych] & 16) {
    goto yy154;
  } else {

  }
  if ((int )yych <= 46) {
    if ((int )yych == 45) {
      goto yy148;
    } else {

    }
    goto yy1420;
  } else {
    if ((int )yych <= 47) {
      goto yy148;
    } else {

    }
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy1420;
  }
  yy1490: 
  yyaccept = 5U;
  cursor ++;
  tmp___497 = cursor;
  s->ptr = tmp___497;
  yych = *tmp___497;
  if ((int )yych <= 57) {
    if ((int )yych <= 40) {
      if ((int )yych <= 9) {
        if ((int )yych <= 8) {
          goto yy194;
        } else {

        }
        goto yy196;
      } else {
        if ((int )yych == 32) {
          goto yy196;
        } else {

        }
        goto yy194;
      }
    } else
    if ((int )yych <= 45) {
      if ((int )yych <= 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy194;
      } else {

      }
      goto yy372;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      goto yy196;
    }
  } else
  if ((int )yych <= 94) {
    if ((int )yych <= 66) {
      if ((int )yych <= 64) {
        goto yy194;
      } else {

      }
      goto yy143;
    } else {
      if ((int )yych <= 67) {
        goto yy1477;
      } else {

      }
      if ((int )yych <= 90) {
        goto yy143;
      } else {

      }
      goto yy194;
    }
  } else
  if ((int )yych <= 98) {
    if ((int )yych <= 95) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 99) {
      goto yy1492;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy194;
  }
  yy1491: 
  yyaccept = 5U;
  cursor ++;
  tmp___498 = cursor;
  s->ptr = tmp___498;
  yych = *tmp___498;
  if ((int )yych <= 46) {
    if ((int )yych <= 32) {
      if ((int )yych == 9) {
        goto yy196;
      } else {

      }
      if ((int )yych <= 31) {
        goto yy194;
      } else {

      }
      goto yy196;
    } else
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy194;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 44) {
        goto yy194;
      } else {

      }
      if ((int )yych <= 45) {
        goto yy372;
      } else {

      }
      goto yy196;
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 47) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 57) {
      goto yy196;
    } else {

    }
    if ((int )yych <= 64) {
      goto yy194;
    } else {

    }
    goto yy143;
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 94) {
      goto yy194;
    } else {

    }
    goto yy148;
  } else {
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy194;
  }
  yy1492: 
  yyaccept = 0U;
  cursor ++;
  tmp___499 = cursor;
  s->ptr = tmp___499;
  yych = *tmp___499;
  if ((int )yych <= 71) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 72) {
      goto yy396;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 103) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 104) {
      goto yy407;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1493: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 87) {
    if ((int )yych <= 78) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy142;
    } else {
      if ((int )yych <= 79) {
        goto yy1501;
      } else {

      }
      if ((int )yych <= 85) {
        goto yy142;
      } else {

      }
      if ((int )yych <= 86) {
        goto yy1502;
      } else {

      }
      goto yy1499;
    }
  } else
  if ((int )yych <= 111) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    if ((int )yych <= 110) {
      goto yy142;
    } else {

    }
    goto yy1501;
  } else
  if ((int )yych <= 118) {
    if ((int )yych <= 117) {
      goto yy142;
    } else {

    }
    goto yy1502;
  } else {
    if ((int )yych <= 119) {
      goto yy1499;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy1494: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 88) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 87) {
        goto yy142;
      } else {

      }
      goto yy1498;
    }
  } else
  if ((int )yych <= 119) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else {
    if ((int )yych <= 120) {
      goto yy1498;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy1495: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 78) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 77) {
        goto yy142;
      } else {

      }
    }
  } else
  if ((int )yych <= 109) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else {
    if ((int )yych <= 110) {
      goto yy1496;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy1496: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 84) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 83) {
        goto yy143;
      } else {

      }
    }
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 116) {
      goto yy1497;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy4;
  }
  yy1497: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 72) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 71) {
        goto yy144;
      } else {

      }
      goto yy1239;
    }
  } else
  if ((int )yych <= 103) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 104) {
      goto yy1239;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1498: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 84) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 83) {
        goto yy143;
      } else {

      }
      goto yy1428;
    }
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 116) {
      goto yy1428;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy4;
  }
  yy1499: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 64) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
  } else {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy1500;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
  }
  yy1500: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  free((void *)str);
  return (310);
  yy1501: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 78) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 77) {
        goto yy143;
      } else {

      }
      goto yy1507;
    }
  } else
  if ((int )yych <= 109) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 110) {
      goto yy1507;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy4;
  }
  yy1502: 
  yyaccept = 5U;
  cursor ++;
  tmp___500 = cursor;
  s->ptr = tmp___500;
  yych = *tmp___500;
  if ((int )yych <= 47) {
    if ((int )yych <= 40) {
      if ((int )yych <= 9) {
        if ((int )yych <= 8) {
          goto yy194;
        } else {

        }
        goto yy196;
      } else {
        if ((int )yych == 32) {
          goto yy196;
        } else {

        }
        goto yy194;
      }
    } else
    if ((int )yych <= 44) {
      if ((int )yych <= 41) {
        goto yy140;
      } else {

      }
      goto yy194;
    } else {
      if ((int )yych <= 45) {
        goto yy197;
      } else {

      }
      if ((int )yych <= 46) {
        goto yy196;
      } else {

      }
      goto yy194;
    }
  } else
  if ((int )yych <= 90) {
    if ((int )yych <= 64) {
      if ((int )yych <= 57) {
        goto yy196;
      } else {

      }
      goto yy194;
    } else
    if ((int )yych != 69) {
      goto yy143;
    } else {

    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 101) {
      goto yy1503;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy194;
  }
  yy1503: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 77) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 76) {
        goto yy144;
      } else {

      }
    }
  } else
  if ((int )yych <= 108) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 109) {
      goto yy1504;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1504: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 66) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 65) {
        goto yy145;
      } else {

      }
    }
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy145;
  } else {
    if ((int )yych <= 98) {
      goto yy1505;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy4;
  }
  yy1505: 
  yyaccept = 0U;
  cursor ++;
  tmp___501 = cursor;
  s->ptr = tmp___501;
  yych = *tmp___501;
  if ((int )yych <= 68) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 69) {
      goto yy1506;
    } else {

    }
    if ((int )yych != 101) {
      goto yy4;
    } else {

    }
  }
  yy1506: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy206;
  } else {

  }
  if ((int )yych == 114) {
    goto yy206;
  } else {

  }
  goto yy57;
  yy1507: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 64) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
  } else {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy1508;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
  }
  yy1508: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  (s->time)->have_time = 0U;
  (s->time)->h = (timelib_sll )0;
  (s->time)->i = (timelib_sll )0;
  (s->time)->s = (timelib_sll )0;
  (s->time)->f = (double )0;
  if ((s->time)->have_time) {
    add_error(s, (char *)"Double time specification");
    free((void *)str);
    return (999);
  } else {
    (s->time)->have_time = 1U;
    (s->time)->h = (timelib_sll )0;
    (s->time)->i = (timelib_sll )0;
    (s->time)->s = (timelib_sll )0;
    (s->time)->f = (double )0;
  }
  (s->time)->h = (timelib_sll )12;
  free((void *)str);
  return (310);
  yy1509: 
  yyaccept = 0U;
  cursor ++;
  tmp___502 = cursor;
  s->ptr = tmp___502;
  yych = *tmp___502;
  if ((int )yych <= 86) {
    if ((int )yych <= 46) {
      if ((int )yych <= 41) {
        if ((int )yych <= 40) {
          goto yy4;
        } else {

        }
        goto yy140;
      } else {
        if ((int )yych == 45) {
          goto yy148;
        } else {

        }
        goto yy4;
      }
    } else
    if ((int )yych <= 78) {
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy142;
    } else {
      if ((int )yych <= 79) {
        goto yy1501;
      } else {

      }
      if ((int )yych <= 85) {
        goto yy142;
      } else {

      }
      goto yy1502;
    }
  } else
  if ((int )yych <= 110) {
    if ((int )yych <= 94) {
      if ((int )yych <= 87) {
        goto yy1499;
      } else {

      }
      if ((int )yych <= 90) {
        goto yy142;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych <= 95) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 96) {
        goto yy4;
      } else {

      }
      goto yy147;
    }
  } else
  if ((int )yych <= 118) {
    if ((int )yych <= 111) {
      goto yy1516;
    } else {

    }
    if ((int )yych <= 117) {
      goto yy147;
    } else {

    }
    goto yy1517;
  } else {
    if ((int )yych <= 119) {
      goto yy1515;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy1510: 
  yyaccept = 0U;
  cursor ++;
  tmp___503 = cursor;
  s->ptr = tmp___503;
  yych = *tmp___503;
  if ((int )yych <= 87) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 88) {
      goto yy1498;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 119) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 120) {
      goto yy1514;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy1511: 
  yyaccept = 0U;
  cursor ++;
  tmp___504 = cursor;
  s->ptr = tmp___504;
  yych = *tmp___504;
  if ((int )yych <= 77) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 78) {
      goto yy1496;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 109) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 110) {
      goto yy1512;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy1512: 
  yyaccept = 0U;
  cursor ++;
  tmp___505 = cursor;
  s->ptr = tmp___505;
  yych = *tmp___505;
  if ((int )yych <= 83) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 84) {
      goto yy1497;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 116) {
      goto yy1513;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy4;
  }
  yy1513: 
  yyaccept = 0U;
  cursor ++;
  tmp___506 = cursor;
  s->ptr = tmp___506;
  yych = *tmp___506;
  if ((int )yych <= 71) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 72) {
      goto yy1239;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 103) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 104) {
      goto yy1268;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1514: 
  yyaccept = 0U;
  cursor ++;
  tmp___507 = cursor;
  s->ptr = tmp___507;
  yych = *tmp___507;
  if ((int )yych <= 83) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 84) {
      goto yy1428;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 116) {
      goto yy1460;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy4;
  }
  yy1515: 
  yyaccept = 31U;
  cursor ++;
  tmp___508 = cursor;
  s->ptr = tmp___508;
  yych = *tmp___508;
  if ((int )yych <= 47) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy1500;
    } else {
      if ((int )yych == 46) {
        goto yy1500;
      } else {

      }
      goto yy148;
    }
  } else
  if ((int )yych <= 94) {
    if ((int )yych <= 64) {
      goto yy1500;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    goto yy1500;
  } else {
    if ((int )yych <= 95) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy1500;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy1500;
  }
  yy1516: 
  yyaccept = 0U;
  cursor ++;
  tmp___509 = cursor;
  s->ptr = tmp___509;
  yych = *tmp___509;
  if ((int )yych <= 77) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 78) {
      goto yy1507;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 109) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 110) {
      goto yy1522;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy4;
  }
  yy1517: 
  yyaccept = 5U;
  cursor ++;
  tmp___510 = cursor;
  s->ptr = tmp___510;
  yych = *tmp___510;
  if ((int )yych <= 57) {
    if ((int )yych <= 40) {
      if ((int )yych <= 9) {
        if ((int )yych <= 8) {
          goto yy194;
        } else {

        }
        goto yy196;
      } else {
        if ((int )yych == 32) {
          goto yy196;
        } else {

        }
        goto yy194;
      }
    } else
    if ((int )yych <= 45) {
      if ((int )yych <= 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy194;
      } else {

      }
      goto yy372;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      goto yy196;
    }
  } else
  if ((int )yych <= 94) {
    if ((int )yych <= 68) {
      if ((int )yych <= 64) {
        goto yy194;
      } else {

      }
      goto yy143;
    } else {
      if ((int )yych <= 69) {
        goto yy1503;
      } else {

      }
      if ((int )yych <= 90) {
        goto yy143;
      } else {

      }
      goto yy194;
    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 95) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy194;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 101) {
      goto yy1518;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy194;
  }
  yy1518: 
  yyaccept = 0U;
  cursor ++;
  tmp___511 = cursor;
  s->ptr = tmp___511;
  yych = *tmp___511;
  if ((int )yych <= 76) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 77) {
      goto yy1504;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 108) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 109) {
      goto yy1519;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1519: 
  yyaccept = 0U;
  cursor ++;
  tmp___512 = cursor;
  s->ptr = tmp___512;
  yych = *tmp___512;
  if ((int )yych <= 65) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy145;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 66) {
      goto yy1505;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 97) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy153;
  } else {
    if ((int )yych <= 98) {
      goto yy1520;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy4;
  }
  yy1520: 
  yyaccept = 0U;
  cursor ++;
  tmp___513 = cursor;
  s->ptr = tmp___513;
  yych = *tmp___513;
  if ((int )yych <= 68) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 69) {
      goto yy1506;
    } else {

    }
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych == 101) {
      goto yy1521;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy154;
    } else {

    }
    goto yy4;
  }
  yy1521: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 82) {
    goto yy206;
  } else {

  }
  if ((int )yych == 114) {
    goto yy377;
  } else {

  }
  goto yy155;
  yy1522: 
  yyaccept = 32U;
  cursor ++;
  tmp___514 = cursor;
  s->ptr = tmp___514;
  yych = *tmp___514;
  if ((int )yych <= 47) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy1508;
    } else {
      if ((int )yych == 46) {
        goto yy1508;
      } else {

      }
      goto yy148;
    }
  } else
  if ((int )yych <= 94) {
    if ((int )yych <= 64) {
      goto yy1508;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    goto yy1508;
  } else {
    if ((int )yych <= 95) {
      goto yy148;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy1508;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy1508;
  }
  yy1523: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 83) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 82) {
        goto yy142;
      } else {

      }
    }
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy142;
  } else {
    if ((int )yych <= 115) {
      goto yy1524;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy142;
    } else {

    }
    goto yy4;
  }
  yy1524: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 84) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 83) {
        goto yy143;
      } else {

      }
    }
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy143;
  } else {
    if ((int )yych <= 116) {
      goto yy1525;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy143;
    } else {

    }
    goto yy4;
  }
  yy1525: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 69) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 68) {
        goto yy144;
      } else {

      }
    }
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy144;
  } else {
    if ((int )yych <= 101) {
      goto yy1526;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy144;
    } else {

    }
    goto yy4;
  }
  yy1526: 
  cursor ++;
  yych = *cursor;
  if ((int )yych <= 82) {
    if ((int )yych <= 41) {
      if ((int )yych <= 40) {
        goto yy4;
      } else {

      }
      goto yy140;
    } else {
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 81) {
        goto yy145;
      } else {

      }
    }
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy145;
  } else {
    if ((int )yych <= 114) {
      goto yy1527;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy145;
    } else {

    }
    goto yy4;
  }
  yy1527: 
  yyaccept = 0U;
  cursor ++;
  tmp___515 = cursor;
  s->ptr = tmp___515;
  yych = *tmp___515;
  if ((int )yych <= 67) {
    if ((int )yych == 41) {
      goto yy140;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych <= 68) {
      goto yy1528;
    } else {

    }
    if ((int )yych != 100) {
      goto yy4;
    } else {

    }
  }
  yy1528: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy1529;
  } else {

  }
  if ((int )yych != 97) {
    goto yy57;
  } else {

  }
  yy1529: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy1530;
  } else {

  }
  if ((int )yych != 121) {
    goto yy57;
  } else {

  }
  yy1530: 
  cursor ++;
  yy1531: 
  s->cur = cursor;
  str = timelib_string(s);
  ptr = str;
  (s->time)->have_relative = 1U;
  (s->time)->have_time = 0U;
  (s->time)->h = (timelib_sll )0;
  (s->time)->i = (timelib_sll )0;
  (s->time)->s = (timelib_sll )0;
  (s->time)->f = (double )0;
  (s->time)->relative.d = (timelib_sll )-1;
  free((void *)str);
  return (310);
  yy1532: 
  yyaccept = 0U;
  cursor ++;
  tmp___516 = cursor;
  s->ptr = tmp___516;
  yych = *tmp___516;
  if ((int )yych <= 82) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy142;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 83) {
      goto yy1524;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy142;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 114) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy147;
  } else {
    if ((int )yych <= 115) {
      goto yy1533;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy147;
    } else {

    }
    goto yy4;
  }
  yy1533: 
  yyaccept = 0U;
  cursor ++;
  tmp___517 = cursor;
  s->ptr = tmp___517;
  yych = *tmp___517;
  if ((int )yych <= 83) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy143;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 84) {
      goto yy1525;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy143;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 115) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy151;
  } else {
    if ((int )yych <= 116) {
      goto yy1534;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy151;
    } else {

    }
    goto yy4;
  }
  yy1534: 
  yyaccept = 0U;
  cursor ++;
  tmp___518 = cursor;
  s->ptr = tmp___518;
  yych = *tmp___518;
  if ((int )yych <= 68) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy144;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 69) {
      goto yy1526;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy144;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 100) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy152;
  } else {
    if ((int )yych <= 101) {
      goto yy1535;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy152;
    } else {

    }
    goto yy4;
  }
  yy1535: 
  yyaccept = 0U;
  cursor ++;
  tmp___519 = cursor;
  s->ptr = tmp___519;
  yych = *tmp___519;
  if ((int )yych <= 81) {
    if ((int )yych <= 45) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      if ((int )yych <= 44) {
        goto yy4;
      } else {

      }
      goto yy148;
    } else {
      if ((int )yych == 47) {
        goto yy148;
      } else {

      }
      if ((int )yych <= 64) {
        goto yy4;
      } else {

      }
      goto yy145;
    }
  } else
  if ((int )yych <= 95) {
    if ((int )yych <= 82) {
      goto yy1527;
    } else {

    }
    if ((int )yych <= 90) {
      goto yy145;
    } else {

    }
    if ((int )yych <= 94) {
      goto yy4;
    } else {

    }
    goto yy148;
  } else
  if ((int )yych <= 113) {
    if ((int )yych <= 96) {
      goto yy4;
    } else {

    }
    goto yy153;
  } else {
    if ((int )yych <= 114) {
      goto yy1536;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy153;
    } else {

    }
    goto yy4;
  }
  yy1536: 
  yyaccept = 0U;
  cursor ++;
  tmp___520 = cursor;
  s->ptr = tmp___520;
  yych = *tmp___520;
  if ((int )yych <= 67) {
    if ((int )yych <= 44) {
      if ((int )yych == 41) {
        goto yy140;
      } else {

      }
      goto yy4;
    } else {
      if ((int )yych == 46) {
        goto yy4;
      } else {

      }
      if ((int )yych <= 47) {
        goto yy148;
      } else {

      }
      goto yy4;
    }
  } else
  if ((int )yych <= 96) {
    if ((int )yych <= 68) {
      goto yy1528;
    } else {

    }
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy4;
  } else {
    if ((int )yych == 100) {
      goto yy1537;
    } else {

    }
    if ((int )yych <= 122) {
      goto yy154;
    } else {

    }
    goto yy4;
  }
  yy1537: 
  cursor ++;
  yych = *cursor;
  if ((int )yych == 65) {
    goto yy1529;
  } else {

  }
  if ((int )yych != 97) {
    goto yy155;
  } else {

  }
  cursor ++;
  yych = *cursor;
  if ((int )yych == 89) {
    goto yy1530;
  } else {

  }
  if ((int )yych != 121) {
    goto yy155;
  } else {

  }
  yyaccept = 33U;
  cursor ++;
  tmp___521 = cursor;
  s->ptr = tmp___521;
  yych = *tmp___521;
  if ((int const   )yybm[(int )yych] & 16) {
    goto yy154;
  } else {

  }
  if ((int )yych <= 46) {
    if ((int )yych == 45) {
      goto yy148;
    } else {

    }
    goto yy1531;
  } else {
    if ((int )yych <= 47) {
      goto yy148;
    } else {

    }
    if ((int )yych == 95) {
      goto yy148;
    } else {

    }
    goto yy1531;
  }
}
}
timelib_time *timelib_strtotime(char *s , int len ,
                                struct timelib_error_container **errors ,
                                timelib_tzdb const   *tzdb ) 
{ 
  Scanner in ;
  int t ;
  char *e ;
  void *tmp ;
  unsigned short const   **tmp___0 ;
  unsigned short const   **tmp___1 ;
  timelib_sll tmp___2 ;
  timelib_sll tmp___3 ;
  timelib_sll tmp___4 ;
  timelib_sll tmp___5 ;
  timelib_sll tmp___6 ;
  double tmp___7 ;
  int tmp___8 ;
  int tmp___9 ;
  unsigned int tmp___10 ;
  void *tmp___11 ;
  int tmp___12 ;
  int tmp___13 ;

  {
  e = (s + len) - 1;
  memset((void *)(& in), 0, sizeof(in));
  tmp = malloc(sizeof(struct timelib_error_container ));
  in.errors = (struct timelib_error_container *)tmp;
  (in.errors)->warning_count = 0;
  (in.errors)->warning_messages = (struct timelib_error_message *)((void *)0);
  (in.errors)->error_count = 0;
  (in.errors)->error_messages = (struct timelib_error_message *)((void *)0);
  if (len > 0) {
    while (1) {
      tmp___0 = __ctype_b_loc();
      if ((int const   )*(*tmp___0 + (int )*s) & 8192) {
        if ((unsigned int )s < (unsigned int )e) {

        } else {
          break;
        }
      } else {
        break;
      }
      s ++;
    }
    while (1) {
      tmp___1 = __ctype_b_loc();
      if ((int const   )*(*tmp___1 + (int )*e) & 8192) {
        if ((unsigned int )e > (unsigned int )s) {

        } else {
          break;
        }
      } else {
        break;
      }
      e --;
    }
  } else {

  }
  if (e - s < 0) {
    in.time = timelib_time_ctor();
    add_error(& in, (char *)"Empty string");
    if (errors) {
      *errors = in.errors;
    } else {
      timelib_error_container_dtor(in.errors);
    }
    tmp___9 = -99999;
    (in.time)->z = tmp___9;
    tmp___8 = tmp___9;
    (in.time)->dst = tmp___8;
    tmp___7 = (double )tmp___8;
    (in.time)->f = tmp___7;
    tmp___6 = (timelib_sll )tmp___7;
    (in.time)->s = tmp___6;
    tmp___5 = tmp___6;
    (in.time)->i = tmp___5;
    tmp___4 = tmp___5;
    (in.time)->h = tmp___4;
    tmp___3 = tmp___4;
    (in.time)->m = tmp___3;
    tmp___2 = tmp___3;
    (in.time)->d = tmp___2;
    (in.time)->y = tmp___2;
    tmp___10 = 0U;
    (in.time)->zone_type = tmp___10;
    (in.time)->is_localtime = tmp___10;
    return (in.time);
  } else {

  }
  e ++;
  tmp___11 = malloc((size_t )((e - s) + 31));
  in.str = (uchar *)tmp___11;
  memset((void *)in.str, 0, (size_t )((e - s) + 31));
  memcpy((void */* __restrict  */)in.str, (void const   */* __restrict  */)s,
         (size_t )(e - s));
  in.lim = (in.str + (e - s)) + 31;
  in.cur = in.str;
  in.time = timelib_time_ctor();
  (in.time)->y = (timelib_sll )-99999;
  (in.time)->d = (timelib_sll )-99999;
  (in.time)->m = (timelib_sll )-99999;
  (in.time)->h = (timelib_sll )-99999;
  (in.time)->i = (timelib_sll )-99999;
  (in.time)->s = (timelib_sll )-99999;
  (in.time)->f = (double )-99999;
  (in.time)->z = -99999;
  (in.time)->dst = -99999;
  in.tzdb = tzdb;
  (in.time)->is_localtime = 0U;
  (in.time)->zone_type = 0U;
  while (1) {
    t = scan(& in);
    if (t != 257) {

    } else {
      break;
    }
  }
  if ((in.time)->have_time) {
    tmp___12 = timelib_valid_time((in.time)->h, (in.time)->i, (in.time)->s);
    if (tmp___12) {

    } else {
      add_warning(& in, (char *)"The parsed time was invalid");
    }
  } else {

  }
  if ((in.time)->have_date) {
    tmp___13 = timelib_valid_date((in.time)->y, (in.time)->m, (in.time)->d);
    if (tmp___13) {

    } else {
      add_warning(& in, (char *)"The parsed date was invalid");
    }
  } else {

  }
  free((void *)in.str);
  if (errors) {
    *errors = in.errors;
  } else {
    timelib_error_container_dtor(in.errors);
  }
  return (in.time);
}
}
static void timelib_time_reset_fields(timelib_time *time ) 
{ 
  timelib_sll tmp ;
  timelib_sll tmp___0 ;

  {
  if ((unsigned int )time != (unsigned int )((void *)0)) {

  } else {
    __assert_fail("time != ((void *)0)", "ext/date/lib/parse_date.re", 1847U,
                  "timelib_time_reset_fields");
  }
  time->y = (timelib_sll )1970;
  time->m = (timelib_sll )1;
  time->d = (timelib_sll )1;
  tmp___0 = (timelib_sll )0;
  time->s = tmp___0;
  tmp = tmp___0;
  time->i = tmp;
  time->h = tmp;
  time->f = 0.0;
  time->tz_info = (timelib_tzinfo *)((void *)0);
  return;
}
}
static void timelib_time_reset_unset_fields(timelib_time *time ) 
{ 


  {
  if ((unsigned int )time != (unsigned int )((void *)0)) {

  } else {
    __assert_fail("time != ((void *)0)", "ext/date/lib/parse_date.re", 1859U,
                  "timelib_time_reset_unset_fields");
  }
  if (time->y == -99999LL) {
    time->y = (timelib_sll )1970;
  } else {

  }
  if (time->m == -99999LL) {
    time->m = (timelib_sll )1;
  } else {

  }
  if (time->d == -99999LL) {
    time->d = (timelib_sll )1;
  } else {

  }
  if (time->h == -99999LL) {
    time->h = (timelib_sll )0;
  } else {

  }
  if (time->i == -99999LL) {
    time->i = (timelib_sll )0;
  } else {

  }
  if (time->s == -99999LL) {
    time->s = (timelib_sll )0;
  } else {

  }
  if (time->f == (double )-99999) {
    time->f = 0.0;
  } else {

  }
  return;
}
}
timelib_time *timelib_parse_from_format(char *format , char *string , int len ,
                                        timelib_error_container **errors ,
                                        timelib_tzdb const   *tzdb ) 
{ 
  char *fptr ;
  char *ptr ;
  char *begin ;
  timelib_sll tmp ;
  Scanner in ;
  Scanner *s ;
  int allow_extra ;
  void *tmp___0 ;
  timelib_relunit const   *tmp___1 ;
  void *tmp___2 ;
  char *tmp___3 ;
  timelib_sll tmp___4 ;
  void *tmp___5 ;
  char *tmp___6 ;
  void *tmp___7 ;
  char *tmp___8 ;
  timelib_sll tmp___9 ;
  long tmp___10 ;
  void *tmp___11 ;
  char *tmp___12 ;
  timelib_sll tmp___13 ;
  void *tmp___14 ;
  char *tmp___15 ;
  timelib_sll tmp___16 ;
  void *tmp___17 ;
  char *tmp___18 ;
  timelib_sll tmp___19 ;
  void *tmp___20 ;
  char *tmp___21 ;
  timelib_sll tmp___22 ;
  void *tmp___23 ;
  char *tmp___24 ;
  timelib_sll tmp___25 ;
  void *tmp___26 ;
  char *tmp___27 ;
  timelib_sll tmp___28 ;
  double f ;
  char *tptr ;
  void *tmp___29 ;
  char *tmp___30 ;
  double tmp___31 ;
  timelib_sll tmp___32 ;
  void *tmp___33 ;
  char *tmp___34 ;
  timelib_ull tmp___35 ;
  timelib_sll tmp___36 ;
  timelib_sll tmp___37 ;
  int tz_not_found ;
  long tmp___38 ;
  char *tmp___39 ;
  int done ;
  char *tmp___40 ;
  int tmp___41 ;
  int tmp___42 ;

  {
  fptr = format;
  ptr = string;
  s = & in;
  allow_extra = 0;
  memset((void *)(& in), 0, sizeof(in));
  tmp___0 = malloc(sizeof(struct timelib_error_container ));
  in.errors = (struct timelib_error_container *)tmp___0;
  (in.errors)->warning_count = 0;
  (in.errors)->warning_messages = (struct timelib_error_message *)((void *)0);
  (in.errors)->error_count = 0;
  (in.errors)->error_messages = (struct timelib_error_message *)((void *)0);
  in.time = timelib_time_ctor();
  (in.time)->y = (timelib_sll )-99999;
  (in.time)->d = (timelib_sll )-99999;
  (in.time)->m = (timelib_sll )-99999;
  (in.time)->h = (timelib_sll )-99999;
  (in.time)->i = (timelib_sll )-99999;
  (in.time)->s = (timelib_sll )-99999;
  (in.time)->f = (double )-99999;
  (in.time)->z = -99999;
  (in.time)->dst = -99999;
  in.tzdb = tzdb;
  (in.time)->is_localtime = 0U;
  (in.time)->zone_type = 0U;
  while (1) {
    if (*fptr) {
      if (*ptr) {

      } else {
        break;
      }
    } else {
      break;
    }
    begin = ptr;
    switch ((int )*fptr) {
    case 108: 
    case 68: 
    tmp___1 = timelib_lookup_relunit(& ptr);
    if (tmp___1) {

    } else {
      add_pbf_error(s, (char *)"A textual day could not be found", string, begin);
    }
    break;
    case 106: 
    case 100: 
    tmp___3 = __builtin_strchr((char *)"0123456789", (int )*ptr);
    if ((unsigned int )tmp___3 == (unsigned int )((void *)0)) {
      add_pbf_error(s, (char *)"Unexpected data found.", string, begin);
    } else {

    }
    tmp___4 = timelib_get_nr(& ptr, 2);
    (s->time)->d = tmp___4;
    if (tmp___4 == -99999LL) {
      add_pbf_error(s, (char *)"A two digit day could not be found", string,
                    begin);
    } else {

    }
    break;
    case 83: 
    timelib_skip_day_suffix(& ptr);
    break;
    case 122: 
    tmp___6 = __builtin_strchr((char *)"0123456789", (int )*ptr);
    if ((unsigned int )tmp___6 == (unsigned int )((void *)0)) {
      add_pbf_error(s, (char *)"Unexpected data found.", string, begin);
    } else {

    }
    tmp = timelib_get_nr(& ptr, 3);
    if (tmp == -99999LL) {
      add_pbf_error(s, (char *)"A three digit day-of-year could not be found",
                    string, begin);
    } else {
      (s->time)->m = (timelib_sll )1;
      (s->time)->d = tmp + 1LL;
    }
    break;
    case 110: 
    case 109: 
    tmp___8 = __builtin_strchr((char *)"0123456789", (int )*ptr);
    if ((unsigned int )tmp___8 == (unsigned int )((void *)0)) {
      add_pbf_error(s, (char *)"Unexpected data found.", string, begin);
    } else {

    }
    tmp___9 = timelib_get_nr(& ptr, 2);
    (s->time)->m = tmp___9;
    if (tmp___9 == -99999LL) {
      add_pbf_error(s, (char *)"A two digit month could not be found", string,
                    begin);
    } else {

    }
    break;
    case 70: 
    case 77: 
    tmp___10 = timelib_lookup_month(& ptr);
    tmp = (timelib_sll )tmp___10;
    if (! tmp) {
      add_pbf_error(s, (char *)"A textual month could not be found", string,
                    begin);
    } else {
      (s->time)->m = tmp;
    }
    break;
    case 121: 
    tmp___12 = __builtin_strchr((char *)"0123456789", (int )*ptr);
    if ((unsigned int )tmp___12 == (unsigned int )((void *)0)) {
      add_pbf_error(s, (char *)"Unexpected data found.", string, begin);
    } else {

    }
    tmp___13 = timelib_get_nr(& ptr, 2);
    (s->time)->y = tmp___13;
    if (tmp___13 == -99999LL) {
      add_pbf_error(s, (char *)"A two digit year could not be found", string,
                    begin);
    } else {

    }
    if ((s->time)->y == -99999LL) {

    } else
    if ((s->time)->y < 100LL) {
      if ((s->time)->y < 70LL) {
        (s->time)->y += 2000LL;
      } else {
        (s->time)->y += 1900LL;
      }
    } else {

    }
    break;
    case 89: 
    tmp___15 = __builtin_strchr((char *)"0123456789", (int )*ptr);
    if ((unsigned int )tmp___15 == (unsigned int )((void *)0)) {
      add_pbf_error(s, (char *)"Unexpected data found.", string, begin);
    } else {

    }
    tmp___16 = timelib_get_nr(& ptr, 4);
    (s->time)->y = tmp___16;
    if (tmp___16 == -99999LL) {
      add_pbf_error(s, (char *)"A four digit year could not be found", string,
                    begin);
    } else {

    }
    break;
    case 104: 
    case 103: 
    tmp___18 = __builtin_strchr((char *)"0123456789", (int )*ptr);
    if ((unsigned int )tmp___18 == (unsigned int )((void *)0)) {
      add_pbf_error(s, (char *)"Unexpected data found.", string, begin);
    } else {

    }
    tmp___19 = timelib_get_nr(& ptr, 2);
    (s->time)->h = tmp___19;
    if (tmp___19 == -99999LL) {
      add_pbf_error(s, (char *)"A two digit hour could not be found", string,
                    begin);
    } else {

    }
    if ((s->time)->h > 12LL) {
      add_pbf_error(s, (char *)"Hour can not be higher than 12", string, begin);
    } else {

    }
    break;
    case 72: 
    case 71: 
    tmp___21 = __builtin_strchr((char *)"0123456789", (int )*ptr);
    if ((unsigned int )tmp___21 == (unsigned int )((void *)0)) {
      add_pbf_error(s, (char *)"Unexpected data found.", string, begin);
    } else {

    }
    tmp___22 = timelib_get_nr(& ptr, 2);
    (s->time)->h = tmp___22;
    if (tmp___22 == -99999LL) {
      add_pbf_error(s, (char *)"A two digit hour could not be found", string,
                    begin);
    } else {

    }
    break;
    case 65: 
    case 97: 
    if ((s->time)->h == -99999LL) {
      add_pbf_error(s,
                    (char *)"Meridian can only come after an hour has been found",
                    string, begin);
    } else {
      tmp = timelib_meridian_with_check(& ptr, (s->time)->h);
      if (tmp == -99999LL) {
        add_pbf_error(s, (char *)"A meridian could not be found", string, begin);
      } else {
        (s->time)->h += tmp;
      }
    }
    break;
    case 105: 
    tmp___24 = __builtin_strchr((char *)"0123456789", (int )*ptr);
    if ((unsigned int )tmp___24 == (unsigned int )((void *)0)) {
      add_pbf_error(s, (char *)"Unexpected data found.", string, begin);
    } else {

    }
    tmp___25 = timelib_get_nr(& ptr, 2);
    (s->time)->i = tmp___25;
    if (tmp___25 == -99999LL) {
      add_pbf_error(s, (char *)"A two digit minute could not be found", string,
                    begin);
    } else {

    }
    break;
    case 115: 
    tmp___27 = __builtin_strchr((char *)"0123456789", (int )*ptr);
    if ((unsigned int )tmp___27 == (unsigned int )((void *)0)) {
      add_pbf_error(s, (char *)"Unexpected data found.", string, begin);
    } else {

    }
    tmp___28 = timelib_get_nr(& ptr, 2);
    (s->time)->s = tmp___28;
    if (tmp___28 == -99999LL) {
      add_pbf_error(s, (char *)"A two digit second could not be found", string,
                    begin);
    } else {

    }
    break;
    case 117: 
    tmp___30 = __builtin_strchr((char *)"0123456789", (int )*ptr);
    if ((unsigned int )tmp___30 == (unsigned int )((void *)0)) {
      add_pbf_error(s, (char *)"Unexpected data found.", string, begin);
    } else {

    }
    tptr = ptr;
    tmp___32 = timelib_get_nr(& ptr, 6);
    f = (double )tmp___32;
    if (f == (double )-99999) {
      add_pbf_error(s, (char *)"A six digit millisecond could not be found",
                    string, begin);
    } else
    if (ptr - tptr < 1) {
      add_pbf_error(s, (char *)"A six digit millisecond could not be found",
                    string, begin);
    } else {
      tmp___31 = pow((double )10, (double )(ptr - tptr));
      (s->time)->f = f / tmp___31;
    }
    break;
    case 32: 
    timelib_eat_spaces(& ptr);
    break;
    case 85: 
    tmp___34 = __builtin_strchr((char *)"0123456789", (int )*ptr);
    if ((unsigned int )tmp___34 == (unsigned int )((void *)0)) {
      add_pbf_error(s, (char *)"Unexpected data found.", string, begin);
    } else {

    }
    (s->time)->have_relative = 1U;
    tmp___35 = timelib_get_unsigned_nr(& ptr, 24);
    tmp = (timelib_sll )tmp___35;
    (s->time)->y = (timelib_sll )1970;
    (s->time)->m = (timelib_sll )1;
    (s->time)->d = (timelib_sll )1;
    tmp___37 = (timelib_sll )0;
    (s->time)->s = tmp___37;
    tmp___36 = tmp___37;
    (s->time)->i = tmp___36;
    (s->time)->h = tmp___36;
    (s->time)->f = 0.0;
    (s->time)->relative.s += tmp;
    (s->time)->is_localtime = 1U;
    (s->time)->zone_type = 1U;
    (s->time)->z = 0;
    break;
    case 79: 
    case 84: 
    case 80: 
    case 101: 
    tmp___38 = timelib_get_zone(& ptr, & (s->time)->dst, s->time,
                                & tz_not_found, s->tzdb);
    (s->time)->z = (int )tmp___38;
    if (tz_not_found) {
      add_pbf_error(s,
                    (char *)"The timezone could not be found in the database",
                    string, begin);
    } else {

    }
    break;
    case 35: 
    if ((int )*ptr == 59) {
      ptr ++;
    } else
    if ((int )*ptr == 58) {
      ptr ++;
    } else
    if ((int )*ptr == 47) {
      ptr ++;
    } else
    if ((int )*ptr == 46) {
      ptr ++;
    } else
    if ((int )*ptr == 44) {
      ptr ++;
    } else
    if ((int )*ptr == 45) {
      ptr ++;
    } else
    if ((int )*ptr == 40) {
      ptr ++;
    } else
    if ((int )*ptr == 41) {
      ptr ++;
    } else {
      add_pbf_error(s,
                    (char *)"The separation symbol ([;:/.,-]) could not be found",
                    string, begin);
    }
    break;
    case 41: 
    case 40: 
    case 45: 
    case 44: 
    case 46: 
    case 47: 
    case 58: 
    case 59: 
    if ((int )*ptr == (int )*fptr) {
      ptr ++;
    } else {
      add_pbf_error(s, (char *)"The separation symbol could not be found",
                    string, begin);
    }
    break;
    case 33: 
    timelib_time_reset_fields(s->time);
    break;
    case 124: 
    timelib_time_reset_unset_fields(s->time);
    break;
    case 63: 
    ptr ++;
    break;
    case 92: 
    tmp___39 = fptr;
    fptr ++;
    if ((int )*ptr == (int )*fptr) {
      ptr ++;
    } else {
      add_pbf_error(s, (char *)"The escaped character could not be found",
                    string, begin);
    }
    break;
    case 42: 
    timelib_eat_until_separator(& ptr);
    break;
    case 43: 
    allow_extra = 1;
    break;
    default: 
    if ((int )*fptr != (int )*ptr) {
      add_pbf_error(s, (char *)"The format separator does not match", string,
                    begin);
    } else {

    }
    ptr ++;
    }
    fptr ++;
  }
  if (*ptr) {
    if (allow_extra) {
      add_pbf_warning(s, (char *)"Trailing data", string, ptr);
    } else {
      add_pbf_error(s, (char *)"Trailing data", string, ptr);
    }
  } else {

  }
  while ((int )*fptr == 43) {
    fptr ++;
  }
  if (*fptr) {
    done = 0;
    while (1) {
      if (*fptr) {
        if (! done) {

        } else {
          break;
        }
      } else {
        break;
      }
      tmp___40 = fptr;
      fptr ++;
      switch ((int )*tmp___40) {
      case 33: 
      timelib_time_reset_fields(s->time);
      break;
      case 124: 
      timelib_time_reset_unset_fields(s->time);
      break;
      default: 
      add_pbf_error(s, (char *)"Data missing", string, ptr);
      done = 1;
      }
    }
  } else {

  }
  if ((s->time)->h != -99999LL) {
    goto _L;
  } else
  if ((s->time)->i != -99999LL) {
    goto _L;
  } else
  if ((s->time)->s != -99999LL) {
    _L: /* CIL Label */ 
    if ((s->time)->h == -99999LL) {
      (s->time)->h = (timelib_sll )0;
    } else {

    }
    if ((s->time)->i == -99999LL) {
      (s->time)->i = (timelib_sll )0;
    } else {

    }
    if ((s->time)->s == -99999LL) {
      (s->time)->s = (timelib_sll )0;
    } else {

    }
  } else {

  }
  if ((s->time)->h != -99999LL) {
    if ((s->time)->i != -99999LL) {
      if ((s->time)->s != -99999LL) {
        tmp___41 = timelib_valid_time((s->time)->h, (s->time)->i, (s->time)->s);
        if (tmp___41) {

        } else {
          add_pbf_warning(s, (char *)"The parsed time was invalid", string, ptr);
        }
      } else {

      }
    } else {

    }
  } else {

  }
  if ((s->time)->y != -99999LL) {
    if ((s->time)->m != -99999LL) {
      if ((s->time)->d != -99999LL) {
        tmp___42 = timelib_valid_date((s->time)->y, (s->time)->m, (s->time)->d);
        if (tmp___42) {

        } else {
          add_pbf_warning(s, (char *)"The parsed date was invalid", string, ptr);
        }
      } else {

      }
    } else {

    }
  } else {

  }
  if (errors) {
    *errors = in.errors;
  } else {
    timelib_error_container_dtor(in.errors);
  }
  return (in.time);
}
}
void timelib_fill_holes(timelib_time *parsed , timelib_time *now , int options ) 
{ 
  void *tmp ;
  size_t __len ;
  size_t tmp___0 ;
  char *__retval ;
  void *tmp___1 ;
  void *tmp___2 ;
  char *tmp___3 ;
  char *tmp___4 ;
  timelib_tzinfo *tmp___5 ;
  timelib_tzinfo *tmp___6 ;

  {
  if (! (options & 1)) {
    if (parsed->have_date) {
      if (! parsed->have_time) {
        parsed->h = (timelib_sll )0;
        parsed->i = (timelib_sll )0;
        parsed->s = (timelib_sll )0;
        parsed->f = (double )0;
      } else {

      }
    } else {

    }
  } else {

  }
  if (parsed->y == -99999LL) {
    if (now->y != -99999LL) {
      parsed->y = now->y;
    } else {
      parsed->y = (timelib_sll )0;
    }
  } else {

  }
  if (parsed->d == -99999LL) {
    if (now->d != -99999LL) {
      parsed->d = now->d;
    } else {
      parsed->d = (timelib_sll )0;
    }
  } else {

  }
  if (parsed->m == -99999LL) {
    if (now->m != -99999LL) {
      parsed->m = now->m;
    } else {
      parsed->m = (timelib_sll )0;
    }
  } else {

  }
  if (parsed->h == -99999LL) {
    if (now->h != -99999LL) {
      parsed->h = now->h;
    } else {
      parsed->h = (timelib_sll )0;
    }
  } else {

  }
  if (parsed->i == -99999LL) {
    if (now->i != -99999LL) {
      parsed->i = now->i;
    } else {
      parsed->i = (timelib_sll )0;
    }
  } else {

  }
  if (parsed->s == -99999LL) {
    if (now->s != -99999LL) {
      parsed->s = now->s;
    } else {
      parsed->s = (timelib_sll )0;
    }
  } else {

  }
  if (parsed->f == (double )-99999) {
    if (now->f != (double )-99999) {
      parsed->f = now->f;
    } else {
      parsed->f = (double )0;
    }
  } else {

  }
  if (parsed->z == -99999) {
    if (now->z != -99999) {
      parsed->z = now->z;
    } else {
      parsed->z = 0;
    }
  } else {

  }
  if (parsed->dst == -99999) {
    if (now->dst != -99999) {
      parsed->dst = now->dst;
    } else {
      parsed->dst = 0;
    }
  } else {

  }
  if (! parsed->tz_abbr) {
    if (now->tz_abbr) {
      tmp___4 = __strdup((char const   *)now->tz_abbr);
      parsed->tz_abbr = tmp___4;
    } else {
      parsed->tz_abbr = (char *)((void *)0);
    }
  } else {

  }
  if (! parsed->tz_info) {
    if (now->tz_info) {
      if (! (options & 2)) {
        tmp___5 = timelib_tzinfo_clone(now->tz_info);
        tmp___6 = tmp___5;
      } else {
        tmp___6 = now->tz_info;
      }
      parsed->tz_info = tmp___6;
    } else {
      parsed->tz_info = (timelib_tzinfo *)((void *)0);
    }
  } else {

  }
  if (parsed->zone_type == 0U) {
    if (now->zone_type != 0U) {
      parsed->zone_type = now->zone_type;
      parsed->is_localtime = 1U;
    } else {

    }
  } else {

  }
  return;
}
}
char *timelib_timezone_id_from_abbr(char const   *abbr , long gmtoffset ,
                                    int isdst ) 
{ 
  timelib_tz_lookup_table const   *tp ;

  {
  tp = zone_search(abbr, gmtoffset, isdst);
  if (tp) {
    return ((char *)tp->full_tz_name);
  } else {
    return ((char *)((void *)0));
  }
}
}
timelib_tz_lookup_table const   *timelib_timezone_abbreviations_list(void) 
{ 


  {
  return (timelib_timezone_lookup);
}
}
